// (C)2019-23 Lisp2D Atomic Mutex
# include "define.h"
# include "atomic_mutex.h"

void  Lisp2D_atomic_mutex_init  ( Lisp2D_atomic_mutex * const m ) {
  atomic_flag_clear ( & m -> atomflag ) ;
# ifdef  LISP2D_BIMSTAT  
  m -> statfast = 0 ;
  m -> statwait = 0 ;
# endif
# ifdef LISP2D_DEBUG
  m -> s = Lisp2DObjectStateConstructed ;
# endif
}

bool  Lisp2D_atomic_mutex_trylockOK  ( Lisp2D_atomic_mutex * const m ) {
# ifdef LISP2D_DEBUG  
  if ( m -> s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2D_atomic_mutex_trylockOK:mutex is not live" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
# ifdef  LISP2D_BIMSTAT
  if ( atomic_flag_test_and_set  ( & m -> atomflag ) ) {
    ++ ( m -> statwait ) ;
    return  false ;
  }
  ++ ( m -> statfast ) ;
  return  true ;
# else
  return  not atomic_flag_test_and_set  ( & m -> atomflag ) ;
# endif  
}
  
void  Lisp2D_atomic_mutex_lock  ( Lisp2D_atomic_mutex * const m ) {
# ifdef LISP2D_DEBUG  
  if ( m -> s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2D_atomic_mutex_lock:mutex is not live" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
# ifdef  LISP2D_BIMSTAT
  if  ( atomic_flag_test_and_set  ( & m -> atomflag ) ) {
    do {
      ++ ( m -> statwait ) ;
    } while ( atomic_flag_test_and_set  ( & m -> atomflag ) ) ; 
  } else
    ++  ( m -> statfast ) ;
# else
  do {
  } while ( atomic_flag_test_and_set  ( & m -> atomflag ) ) ;
# endif
}
  
void  Lisp2D_atomic_mutex_unlock  ( Lisp2D_atomic_mutex * const m ) {
# ifdef LISP2D_DEBUG  
  if ( m -> s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2D_atomic_mutex_unlock:mutex is not live" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
  atomic_flag_clear ( & m -> atomflag ) ;
}

# ifdef LISP2D_DEBUG  
void  Lisp2D_atomic_mutex_destruct  ( Lisp2D_atomic_mutex * const m ) {
  if  ( m -> s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2D_atomic_mutex_destruct:mutex is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  m -> s = Lisp2DObjectStateDestructed ;
}
# endif

# ifdef  LISP2D_BIMSTAT
size_t  atomic_mutex_StatFast  ( Lisp2D_atomic_mutex const * const me ) {
# ifdef LISP2D_DEBUG
  if ( me -> s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& 
      L"Lisp2D_atomic_mutex_StatFast:mutex is not live" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif // LISP2D_DEBUG
  return me -> statfast ;
}

size_t  atomic_mutex_StatWait  ( Lisp2D_atomic_mutex const * const me ) {
# ifdef LISP2D_DEBUG
  if ( me -> s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2D_atomic_mutex_StatWait:mutex is not live" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif // LISP2D_DEBUG
  return me -> statwait ;
}

# endif // LISP2D_BIMSTAT
