// Lisp2D - Boolean type
// ©2012-23 by Alex Glebe

# include "boolean.h"

# ifdef LISP2D_CONSTRSTAT

_Atomic  ( size_t  ) PLOBooleanconstrstat = 0 ;

size_t  PLOBooleanconstrstatCValue  ( void  ) {
  return  PLOBooleanconstrstat  ;
}

void  PLOBooleanconstrstat_Inc  ( void  ) {
  ++  PLOBooleanconstrstat  ;
}

# endif

# include "plo/public.h"

void  PLOBoolean_Init ( PLOBoolean * const me ) {
  ParaLispObject_InitCVTP ( & me -> plo , true ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOBoolean } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOBooleanconstrstat_Inc  ( ) ;
# endif
}

void PLOBooleanDestr  ( ParaLispObject * const me ) {
  ParaLispObject_staticDestr  ( me ) ;
}

PLOBoolean  plotrue ;
PLOBoolean  plofalse  ;

# include "nilc.h"

ParaLispObject  *   ploclassdefboolean  = & plonil . plo ;
ParaLispObject  *   ploclassparentsboolean  = & plonil . plo  ;

ParaLispObject  * PLOBooleanClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassdefboolean  ;
}

ParaLispObject  * PLOBooleanClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassparentsboolean  ;
}

# include "symbolc.h"

ParaLispObject  * PLOBooleanTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return & plosboolean . plo ;
}
  
void  PLOBooleanCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool flag2pprinted  , Vector ( StackPLOP ) * const h ) {
  PLOBooleanCoutToInterpret (  me ,     s , flag2pprinted  ,  h ) ;
}

# include "plostream/public.h"
  
void  PLOBooleanCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool  _ __attribute__ ((unused)) ,
  Vector(StackPLOP)  * _2 __attribute__ ((unused)) )  {
  if  ( me ==  ( &  plofalse . plo ) ) {
    PLOStreamPushPLO ( s , & ( plosfalse . plo ) ) ;
    return  ;
  }
  if  ( me ==  ( &  plotrue . plo ) ) {
     // ⊨ or true ?
    PLOStreamPushPLO ( s , & ( plostrue1 . plo ) ) ;
    return  ;
  }
  PLOStreamPushWChar(s,L'?');
}

# define  BooleanP  Lisp2DBooleanPaEgY
inline  bool  BooleanP ( ParaLispObject const * xp  ) ;
  
# include "stcl/classc.h"
# include "cons/public.h"
# include "stcl/public.h"
# include "stcl/inline.h"

// nil booleanp
TaskForThread bimnilbooleanp ( StackClusterPrivate * const stclpp ) {
  StackClusterPrivate_SetResult ( stclpp ,
    ( BooleanP  ( CCAR ( StackClusterPrivate_ArgTop ( stclpp ) ) ) ?
      & plotrue . plo : & plofalse . plo ) ) ;
  return  TaskForThread_Analyse ;
}

//  nil not
/*
 nil →  nil
 false  → true
 true → false
*/
TaskForThread bimnilnot  ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject  * const ap = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) )  ;
  StackClusterPrivate_SetResult ( stclpp ,
    ( ( ap ==  ( & plonil . plo ) ) ? 
      ( & plonil . plo ) :
      ( ( ap ==  ( & plofalse . plo ) ) ? & plotrue . plo : & plofalse . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// nil -> a b  
/*
 t t t 
 n n t
 f n t
*/
TaskForThread bimnilimply ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject  * const ap = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) )  ;
  ParaLispObject  * const bp =  CCAR ( 
    CCDR ( StackClusterPrivate_ArgTop ( stclpp ) ) )  ;
  ParaLispObject  * res ;
  if  ( ap ==  ( & plofalse . plo ) ) {
    if  ( ( bp ==  ( & plofalse . plo ) ) or 
      ( bp ==  ( & plonil . plo ) ) )
      res = ( & plotrue . plo ) ;
    else 
      res = bp ;
  } else {
    if  ( ap ==  ( &plonil . plo ) )
      if  ( ( bp ==  ( & plofalse . plo ) ) or 
        ( bp ==  ( & plonil .plo ) ) )
        res = ( & plonil . plo ) ;
      else 
        res = bp ;
    else  
      res = bp ;
  }
  StackClusterPrivate_SetResult ( stclpp ,  res ) ;
  return  TaskForThread_Analyse ;
}

# include "string/public.h"

//  boolean string
TaskForThread bimbooleanstring ( StackClusterPrivate * const stclpp  ) {
  ParaLispObject  const * const me = StackClusterPrivate_ThisTop  ( stclpp ) ;
  if  ( me ==   & plofalse . plo  ) {
    PLOStringPub * const  resp  = PLOString_NewCopy ( StringCStr  ( WideChar  ) ( PLOSymbolCValue  ( & plosfalse ) ) ) ;
    StackClusterPrivate_SetResult (stclpp , & resp  -> plo ) ;
    goto  Exit  ;
  }
  if  ( me ==  & plotrue . plo ) {
    PLOStringPub * const  resp  = PLOString_NewCopy ( StringCStr  ( WideChar  ) ( PLOSymbolCValue  ( & plostrue1 ) ) ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & resp -> plo ) ;
    goto  Exit  ;
  }
  PLOStringPub * const rep = PLOString_NewWChar ( L'?' ) ;
  StackClusterPrivate_SetResult ( stclpp , & rep -> plo ) ;
Exit  :
  return  TaskForThread_Analyse ;
}

# include "sharedc.h"
# include "shared/inline.h"
# include "cons/inline.h"
  
//  nil or
/*
 →  false
 nil  → nil
 false  → false
 true → true
 nil  nil → nil
 nil  false → nil
 nil  true  → true
 false  nil → nil
 false  false → false
 false  true  → true
 true nil → true
 true false → true
 true true  → true
*/
TaskForThread bimnilor ( StackClusterPrivate * const stclpp  ) {
  ParaLispObject  * j ;
  unsigned  int tasks = maxListItemProcessing ;
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) ) {
    StackClusterPrivate_Pop ( stclpp ,&j ) ;
    goto  Next  ;
  }
  j = StackClusterPrivate_ArgTop(stclpp ) ;
  StackClusterPrivate_SetResult (stclpp ,& plofalse . plo ) ;
  while ( CConsp ( j ) ) {
Next  :
    { ParaLispObject  * const xp = ClassCons_First(&ParaLispObjectToPLOCons ( j )->cc ) ;
      if  ( xp ==  ( &  plonil . plo ) )
        StackClusterPrivate_SetResult (stclpp ,& plonil . plo ) ;
        // and wait for another news
      else if  ( xp not_eq  ( &  plofalse . plo ) ) {
        // not nil and not false brings good news
        StackClusterPrivate_SetResult (stclpp , xp ) ;
        goto  Exit  ;
      }
    } // xp
    //ClassCons_RestTo ( &ParaLispObjectToPLOCons  ( j )->cc , & j ) ;
    j = ClassCons_Rest ( &ParaLispObjectToPLOCons  ( j )->cc );
    if  ( ( --  tasks ) ==  0 ) {
      if  ( CConsp ( j ) ) {
        StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        StackClusterPrivate_Push ( stclpp , j ) ;
        goto  Exit  ;
      }
      break ;
    }
  } // while consp j
Exit  :
  return  TaskForThread_Analyse ;
}

//  nil nor
/*
 t n f
 n n f
 f f f
*/
TaskForThread bimnilnor ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject  * j ;
  unsigned  int tasks = maxListItemProcessing ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) {
    StackClusterPrivate_Pop ( stclpp ,&j ) ;
    goto  Next  ;
  }
  j = StackClusterPrivate_ArgTop(stclpp ) ;
  StackClusterPrivate_SetResult (stclpp ,& plotrue . plo ) ;
  while ( CConsp ( j ) ) {
Next  :
    { ParaLispObject  * const xp = ClassCons_First(&ParaLispObjectToPLOCons ( j )->cc ) ;
    if  (  xp  ==  ( &  plonil . plo ) )
      StackClusterPrivate_SetResult (stclpp ,& plonil . plo ) ;
      // the float decision
    else
      if  ( xp not_eq  ( & plofalse . plo ) ) {
        // the answer is NO
        StackClusterPrivate_SetResult (stclpp ,& plofalse . plo ) ;
        goto  Exit  ;
      }
    }
    //ClassCons_RestTo ( &ParaLispObjectToPLOCons ( j )->cc , & j ) ;
    j = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc );
    if  ( ( --  tasks ) ==  0 ) {
      if  ( CConsp ( j ) ) {
        StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        StackClusterPrivate_Push ( stclpp , j ) ;
        goto  Exit  ;
      }
      break ;
    }
  } // while consp j
Exit  :
  return  TaskForThread_Analyse ;
}

//  nil nand {}=nil not (nil and {}) = nil or {nil not ..}
/*
 t t t 
 t n n 
 t n f
*/
TaskForThread bimnilnand  ( StackClusterPrivate * const stclpp ) {
  ParaLispObject  * j ;
  unsigned  int tasks = maxListItemProcessing ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) {
    StackClusterPrivate_Pop ( stclpp ,&j ) ;
    goto  Next  ;
  }
  j = StackClusterPrivate_ArgTop ( stclpp ) ;
  StackClusterPrivate_SetResult (stclpp ,& plofalse . plo ) ;
  while ( CConsp ( j ) ) {
Next  :
    { ParaLispObject  * const xp = ClassCons_First(&ParaLispObjectToPLOCons ( j )->cc ) ;
    if  (  xp   ==  ( & plonil . plo ) )
      StackClusterPrivate_SetResult (stclpp ,& plonil . plo ) ;
      // and wait for another news
    else
      if  ( xp ==  ( & plofalse . plo ) ) {
        // not nil and false brings good news
        StackClusterPrivate_SetResult (stclpp ,& plotrue . plo ) ;
        goto  Exit  ;
      }
    }
    //ClassCons_RestTo ( &ParaLispObjectToPLOCons ( j )->cc , & j ) ;
    j = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc );
    if  ( ( --  tasks )  ==  0  ) {
      if  ( CConsp ( j ) ) {
        StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        StackClusterPrivate_Push ( stclpp , j ) ;
        goto  Exit  ;
      }
      break ;
    }
  } // while consp j
Exit  :
  return  TaskForThread_Analyse ;
}

//  nil xor
/*
 →  false
 nil  → nil
 false  → false
 true → true
 nil  nil → nil
 nil  false → nil
 nil  true  → nil
 false  nil → nil
 false  false → false
 false  true  → true
 true nil → nil
 true false → true
 true true  → false
*/
TaskForThread bimnilxor  ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject  * j ;
  ParaLispObject  * res ;
  ParaLispObject  * flagwastrue ;
  unsigned  int tasks = maxListItemProcessing ;
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) ) {
    StackClusterPrivate_Pop ( stclpp ,&flagwastrue ) ;
    StackClusterPrivate_Pop ( stclpp ,&res ) ;
    StackClusterPrivate_Pop ( stclpp ,&j ) ;
    goto  Next  ;
  }
  j = StackClusterPrivate_ArgTop(stclpp ) ;
  res = ( & plofalse . plo ) ;
  flagwastrue  = (  & plofalse . plo ) ;
  while ( CConsp ( j ) ) {
Next  :
    { ParaLispObject  * const xp = ClassCons_First(&ParaLispObjectToPLOCons ( j )->cc ) ;
    if  (  xp  ==  ( &  plonil . plo ) )
      res = ( &  plonil . plo ) ;
    else
      if  ( xp not_eq  ( &  plofalse . plo ) ) {
        if  ( flagwastrue ==  (  &  plotrue . plo ) ) {
          // double true
          StackClusterPrivate_SetResult (stclpp ,& plofalse . plo ) ;
          goto  Exit  ;
        }
        flagwastrue = ( & plotrue . plo ) ;  
        if  ( res  not_eq  ( & plonil . plo ) )
          res =  xp  ;
      }
    }
    //ClassCons_RestTo ( &ParaLispObjectToPLOCons ( j )->cc , & j ) ;
    j = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc );
    if  ( ( --  tasks )  ==  0  ) {
      if  ( CConsp ( j ) ) {
        StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        StackClusterPrivate_Push ( stclpp , j ) ;
        StackClusterPrivate_Push ( stclpp , res ) ;
        StackClusterPrivate_Push ( stclpp , flagwastrue ) ;
        goto  Exit  ;
      }
      break ;
    }
  }  // while consp j
  StackClusterPrivate_SetResult (stclpp ,  res ) ;
Exit  :
  return  TaskForThread_Analyse ;
}

//  nil and
/*
 →  true
 nil  → nil
 false  → false
 true → true
 nil  nil → nil
 nil  false → false
 nil  true  → nil
 false  nil → false
 false  false → false
 false  true  → false
 true nil → nil
 true false → false
 true true  → true
*/
TaskForThread bimniland  ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject  * j ;
  unsigned  int tasks = maxListItemProcessing ;
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) ) {
    StackClusterPrivate_Pop ( stclpp ,&j ) ;
    goto  Next  ;
  }
  j = StackClusterPrivate_ArgTop(stclpp ) ;
  StackClusterPrivate_SetResult (stclpp ,& plotrue . plo ) ;
  while ( CConsp ( j ) ) {
Next  :
    { ParaLispObject  * const xp = ClassCons_First(&ParaLispObjectToPLOCons ( j ) ->cc) ;
      if  (  xp  ==  ( & plonil . plo ) )
        StackClusterPrivate_SetResult (stclpp ,& plonil . plo ) ;
        // the float decision
      else if  (  xp  ==  ( & plofalse . plo ) ) {
        // the answer is NO
        StackClusterPrivate_SetResult (stclpp ,& plofalse . plo ) ;
        goto  Exit  ;
      }
    }
    //ClassCons_RestTo ( &ParaLispObjectToPLOCons  ( j )->cc , & j ) ;
    j = ClassCons_Rest ( &ParaLispObjectToPLOCons  ( j )->cc ) ;
    if  ( ( --  tasks ) ==  0 ) {
      if  ( CConsp ( j ) ) {
        StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        StackClusterPrivate_Push ( stclpp , j ) ;
        goto  Exit  ;
      }
      break ;
    }
  }  // while consp j
Exit  :
  return  TaskForThread_Analyse ;
}

# include "widestring/inline.h"

void  bool_xor_eq ( bool  * const a , bool const b ) {
  if  ( b )
    ( * a ) = not ( * a ) ;
}

inline  bool  BooleanP ( ParaLispObject const * const xp  ) {
  return  ParaLispObject_Type(xp)==TypeGlobalListMemberPLOBoolean;
}
