// Lisp2D - character
// ©2006-23 by Alex Glebe
# include "define.h"
# include "char/public.h"
# include "char/private.h"
  
# ifdef  LISP2D_CONSTRSTAT

_Atomic ( size_t ) PLOChar_constrstat = 0 ;

void  PLOChar_constrstat_Inc  ( void  ) {
  ++  PLOChar_constrstat  ;
}

size_t  PLOChar_constrstat_Value  ( void  ) {
  return  PLOChar_constrstat  ;
}

# endif

# include "char/struct.h"
# include "char/type.h"
# include "plo/public.h"

void  PLOChar_Destr ( PLOCharPriv * const me ) {
  ParaLispObject_staticDestr  ( & me -> pub . plo ) ;
  me  ->  value = L'\xffff' ; //  security
}

void  PLOChar_InitValue ( PLOCharPriv * const me , wchar_t volatile const v ) {
  me -> value = v ;
  ParaLispObject_InitVTP  ( & me  -> pub . plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOChar } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOChar_constrstat_Inc  ( ) ;
# endif
}

# include "stcl/classc.h"
# include "boolean.h"
# include "cons/public.h"
# include "stcl/public.h"

//  nil charp
TaskForThread Lisp2Dbimnilcharp2  ( StackClusterPrivate  * const stclpp ) {
  ParaLispObject  * const x = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) ) ;
  StackClusterPrivate_SetResult ( stclpp , ( Charp ( x ) ?
    x :
    & plofalse . plo ) ) ;
  return  TaskForThread_Analyse ;
}

ParaLispObject  * PLOChar_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassparentschar ;
}

ParaLispObject  * PLOChar_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassdefchar ;
}

# include "newobject.h"

bool  IsNotChar ( StackClusterPrivate  * const stclp  , ParaLispObject * const x ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = x ,
    . typetype  = TypeGlobalListMemberPLOChar ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
  //return  UniversalIsNotType  ( stclp , x , TypeGlobalListMemberPLOChar ) ;
}

bool  Charp ( ParaLispObject const * const x ) {
  return  ParaLispObject_Type ( x ) == TypeGlobalListMemberPLOChar ;
}

ParaLispObject  * PLOChar_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   &  ploschar  .  plo ;
}

bool  PLOChar_Less  ( PLOCharPub const * const a , PLOCharPub const  * const  b ) {
  return  ( a  ==  b ) ?
    false :
    ( PLOChar_Value  ( a )  < PLOChar_Value  ( b ) ) ;
}

bool  PLOChar_Eq  ( PLOCharPub  const * const a , PLOCharPub const * const b ) {
  return  ( a  ==  b ) ?
    true  :
    (  PLOChar_Value ( a )  ==  PLOChar_Value  ( b ) ) ;
}

bool  PLOChar_staticisBADCopyPrivate  ( ParaLispObject const * const me , ParaLispObject  * * const resultp  ) {
  PLOCharPriv * const res = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
  if ( not res )
    return  true  ;
  PLOChar_InitCopy  ( res , ParaLispObjectToPLOCharConst  ( me  ) ) ;
  ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ;
  ( * resultp )  = (  & res -> pub . plo ) ;
  return  false ;
}

# include "sharedc.h"
# include "shared/inline.h"

//  char swap char
TaskForThread Lisp2Dbimcharswap2(StackClusterPrivate  * const stclpp){
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ParaLispObject  * const arg = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) ) ;
  if  ( not  IsNotChar  (   stclpp , arg  ) ) {
    if  ( IsConst ( stclpp , arg  ) ) 
      goto Exit ;
    PLOChar_Swap  ( me  , ParaLispObjectToPLOChar ( arg ) ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & me  ->  plo ) ;
  }
Exit :
  return  TaskForThread_Analyse ; 
}

# include "nilc.h"

//  char setf char
TaskForThread Lisp2Dbimcharsetf2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ParaLispObject * const arg = CCAR ( StackClusterPrivate_ArgTop  ( stclpp ) ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( IsNotChar ( stclpp , arg  ) ?
    ( & plonil  . plo ) :
    ( PLOChar_Set ( me , ParaLispObjectToPLOChar  ( arg ) ) , & me -> plo ) )
    ) ;
  return  TaskForThread_Analyse ; 
}

# include <wctype.h>
# include "cast.h"

// char lower
TaskForThread Lisp2Dbimcharlower2 ( StackClusterPrivate * const stclpp  ) {
  PLOCharPub * const me  = ParaLispObjectToPLOChar ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  if  ( iswlower  ( wchar_cast_wint ( PLOChar_Value ( me  ) ) ) )
    StackClusterPrivate_SetResult ( stclpp ,  & me  ->  plo ) ;
  else  {
    wchar_t const volatile v2  = wint_cast_wchar ( towlower ( wchar_cast_wint ( PLOChar_Value ( me  ) ) ) ) ;
    PLOCharPriv * const res = aligned_alloc ( _Alignof  ( PLOCharPriv ) , sizeof  ( PLOCharPriv ) ) ;
    if ( not res )
      OutOfMemory ( stclpp )  ;
    else {
      PLOChar_InitValue ( res , v2  ) ;
      StackClusterPrivate_SetResult ( stclpp , ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ) ;
    }
  }
  return  TaskForThread_Analyse ;
}

// char upper
TaskForThread Lisp2Dbimcharupper2 ( StackClusterPrivate * const stclpp  ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  if  ( iswupper  ( wchar_cast_wint ( PLOChar_Value ( me  ) ) ) )
    StackClusterPrivate_SetResult ( stclpp ,  & me  ->  plo ) ;
  else  {
    wchar_t const volatile v2  = wint_cast_wchar ( towupper  ( wchar_cast_wint ( PLOChar_Value ( me  ) ) ) ) ;
    PLOCharPriv * const res = aligned_alloc ( _Alignof  ( PLOCharPriv ) , sizeof  ( PLOCharPriv ) ) ;
    if ( not res )
      OutOfMemory ( stclpp ) ;
    else {
      PLOChar_InitValue ( res , v2  ) ;
      StackClusterPrivate_SetResult ( stclpp , ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ) ;
    }
  }
  return  TaskForThread_Analyse ;
}

// char printp
TaskForThread Lisp2Dbimcharprintp2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  StackClusterPrivate_SetResult ( stclpp , ( iswprint ( wchar_cast_wint (
    PLOChar_Value ( me ) ) ) ? & me ->plo  :
    ( & plofalse . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// char controlp
TaskForThread Lisp2Dbimcharcontrolp2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( iswcntrl  ( wchar_cast_wint (
    PLOChar_Value ( me  ) ) ) ? & me  ->  plo  : (  & plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// char spacep
TaskForThread Lisp2Dbimcharspacep2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( iswspace  ( wchar_cast_wint (
    PLOChar_Value ( me  ) ) ) ? & me  ->  plo  : & (  plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// char digitp
TaskForThread Lisp2Dbimchardigitp2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp , (  iswdigit  ( wchar_cast_wint (
    PLOChar_Value ( me  ) ) ) ? & me -> plo  : &  ( plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// char lowerp
TaskForThread Lisp2Dbimcharlowerp2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( iswlower  ( wchar_cast_wint (
    PLOChar_Value ( me  ) ) ) ? & me  ->  plo  : &  ( plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// char upperp
TaskForThread Lisp2Dbimcharupperp2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const  me = ParaLispObjectToPLOChar ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( iswupper  ( wchar_cast_wint ( PLOChar_Value ( me  ) ) ) ?
    & me  ->  plo  :
    &  ( plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// char alphap
TaskForThread Lisp2Dbimcharalphap2  ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( iswalpha  ( wchar_cast_wint ( PLOChar_Value ( me  ) ) ) ?
    & me  ->  plo  :
    &  ( plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

# include "string/public.h"

//  char string
TaskForThread Lisp2Dbimcharstring2  ( StackClusterPrivate  * const stclpp ) {
  wchar_t const wstr [ 2 ] = { PLOChar_Value ( ParaLispObjectToPLOChar (
    StackClusterPrivate_ThisTop ( stclpp ) ) ) ,  L'\00'  } ;
  PLOStringPub * const  resp  = PLOString_NewCopy ( wstr ) ;
  StackClusterPrivate_SetResult ( stclpp , & resp -> plo ) ;
  return  TaskForThread_Analyse ;
}

# include "minmax.h"

# define  CharMore  Lisp2D_CharMoreIWJC
static  Lisp2DBoolType CharMore ( ParaLispObject * const xp , ParaLispObject * const yp , StackClusterPrivate  * const stclpp ) {
  if  ( IsNotChar ( stclpp , yp ) )  
    return  Lisp2DBoolUndef ;
  if  ( PLOChar_More  ( ParaLispObjectToPLOCharConst  ( xp  ) , ParaLispObjectToPLOCharConst  ( yp  ) ) )
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ;
}

# include "lessmore.h"

// char > [{char}]
TaskForThread Lisp2Dbimcharmore3  ( StackClusterPrivate * const stclpp  ) { 
  return  bimUniversalLessMore  ( & CharMore  , stclpp  ) ;
}

// char max [{char}]
TaskForThread Lisp2Dbimcharmax2 ( StackClusterPrivate * const stclpp  ) { 
  return  bimUniversalMinMax  ( & CharMore  , stclpp  ) ;
}

# define  CharLess  Lisp2D_CharLessHfX_
static  Lisp2DBoolType CharLess ( ParaLispObject * const xp ,
  ParaLispObject * const  yp  , StackClusterPrivate  * const stclpp ) {
  if  ( IsNotChar (  stclpp , yp  ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOChar_Less  ( ParaLispObjectToPLOCharConst  ( xp  ) ,
    ParaLispObjectToPLOCharConst  ( yp  ) ) )
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ;
}

// char < [{char}]
TaskForThread Lisp2Dbimcharless3  ( StackClusterPrivate * const stclpp  ) {
  return  bimUniversalLessMore  (  & CharLess , stclpp  ) ;
}

// char min [{char}]
TaskForThread Lisp2Dbimcharmin2 ( StackClusterPrivate * const stclpp  ) { 
  return  bimUniversalMinMax  (  & CharLess , stclpp  ) ;
}

# define  CharMoreEq  Lisp2D_CharMoreEqBOyR
static  Lisp2DBoolType CharMoreEq ( ParaLispObject * const xp , 
  ParaLispObject * const yp , StackClusterPrivate  * const stclpp ) {
  if  ( IsNotChar (  stclpp , yp  ) )  
    return  Lisp2DBoolUndef ;
  if  ( PLOChar_MoreEq  ( ParaLispObjectToPLOCharConst  ( xp  ) ,
    ParaLispObjectToPLOCharConst  ( yp  ) ) )
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ;
}

// char >= [{char}]
TaskForThread Lisp2Dbimcharmoreoreq3  ( StackClusterPrivate * const stclpp  ) { 
  return  bimUniversalLessMore  (  & CharMoreEq , stclpp  ) ;
}

# define  CharLessEq  Lisp2D_CharLessEqrb67
static  Lisp2DBoolType CharLessEq ( ParaLispObject * const xp ,
  ParaLispObject * const yp , StackClusterPrivate  * const stclpp ) {
  if  ( IsNotChar (  stclpp , yp  ) ) 
    return  Lisp2DBoolUndef ;
  if  ( PLOChar_LessEq  ( ParaLispObjectToPLOCharConst  ( xp  ) ,
    ParaLispObjectToPLOCharConst  ( yp  ) ) )
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ;
}

// char <= [{char}]
TaskForThread Lisp2Dbimcharlessoreq3  ( StackClusterPrivate * const stclpp  ) {
  return  bimUniversalLessMore  (  & CharLessEq , stclpp  ) ;
}

// char <> [{x}]
# define  CharNotEq Lisp2D_CharNotEqxcPP
static  Lisp2DBoolType CharNotEq  ( ParaLispObject * const xp  , 
  ParaLispObject * const yp  ) {
  if  ( Charp  ( yp ) and
    PLOChar_Eq  ( ParaLispObjectToPLOCharConst  ( xp )  ,
      ParaLispObjectToPLOCharConst ( yp ) ) )
    return  Lisp2DBoolFalse ;
  return  Lisp2DBoolTrue  ;
}

# include "eqnoteq.h"

TaskForThread Lisp2Dbimcharnotequal2 ( StackClusterPrivate * const stclpp  ) {
  return  bimUniversalEqNotEq (  & CharNotEq  , stclpp  ) ;
}

// char = [{x}]
# define  CharEq  Lisp2D_CharEqkbR_
static  Lisp2DBoolType CharEq  ( ParaLispObject * const xp  ,
  ParaLispObject * const yp  ) {
  if  ( Charp  ( yp ) and
    PLOChar_Eq ( ParaLispObjectToPLOCharConst  ( xp )  ,
      ParaLispObjectToPLOCharConst ( yp ) ) )
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ;
}

TaskForThread Lisp2Dbimcharequal2 ( StackClusterPrivate * const stclpp  ) { 
  return  bimUniversalEqNotEq (  & CharEq  ,  stclpp  ) ;
}

# include "plostream/public.h"

void  PLOChar_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool _ __attribute__ ((unused)) , Vector  ( StackPLOP ) * _2 __attribute__ ((unused)) ) {
  PLOCharPriv const * const pr = CharPub_cast_CharPrivConst ( ParaLispObjectToPLOCharConst (  me  ) ) ;
  PLOStreamPushWChar ( s  , pr -> value  ) ;
}

# include "char.h"
  
static  MapPri  ( SetCharValueName  ) mapPriCharValueName  ;
MapPub  ( SetCharValueName  ) * const mapPubCharValueName = &  mapPriCharValueName . pub ;
static  MapPri  ( SetCharNameValue ) mapPriCharNameValue ;
MapPub  ( SetCharNameValue ) * const mapPubCharNameValue = &  mapPriCharNameValue . pub ;
LISP2DMAPSTACKATBODY( SetCharValueName )
LISP2DMAPSTACKATBODY( SetCharNameValue )
LISP2DMAPSTACKCATBODY( SetCharValueName )
LISP2DMAPSTACKCATBODY( SetCharNameValue )
LISP2DMAPSTACKPUSHBYVALUEBODY ( SetCharValueName )
LISP2DMAPSTACKPUSHBYVALUEBODY ( SetCharNameValue )
LISP2DMAPSTACKPUSHATVALUEBODY ( SetCharValueName )
LISP2DMAPSTACKPUSHATVALUEBODY ( SetCharNameValue )
LISP2DMAPPUBINSERTBODY(  SetCharValueName  )
LISP2DMAPPUBINSERTBODY(  SetCharNameValue  )
LISP2DMAPPUBFINDBODY( SetCharValueName , wchar_t volatile )
LISP2DMAPSTACKPUSHFRONTVALUEBODY( SetCharNameValue )
LISP2DMAPPUBFINDBODY  ( SetCharNameValue , wchar_t volatile const * )

static  int ValueNameLess ( wchar_t volatile const a , wchar_t volatile const b ) {
  if ( a == b )
    return 0 ;
  if ( a < b )
    return -1 ;
  return  1 ;
}

static  int NameValueLess ( wchar_t volatile const * const a , wchar_t volatile const * const b ) {
  int const c = wcscmp  ( wcharconstvolatp_cast_wcharconstp ( a ) , wcharconstvolatp_cast_wcharconstp ( b ) ) ;
  if ( c == 0 )
    return 0 ;
  if ( c < 0 )
    return -1 ;
  return  1 ;
}

typedef struct  scharstring tcharstring ;

struct  scharstring {
  wchar_t volatile c ;
  wchar_t const * s ;
} ;
  
void  CreateSetCharNameValue  ( void  ) {
  static  tcharstring const acharstring [ ] = {
    { L'\00' , L"null" } , // 0
    { L'\a' , L"bell" } , // 7
    { L'\b' , L"backspace"  } , // 8
    { L'\t' , L"tab"  } , // 9
    { L'\n' , L"newline"  } , // 10
    { L'\v' , L"vtab" } , // 11
    { L'\f' , L"feed" } , // 12
    { L'\r' , L"return" } , // 13
    { L'\033' , L"escape" } , // 27
    { L' '  , L"space"  } , // 32
    } ;
  MapPubInit ( SetCharValueName  ) ( mapPubCharValueName  , ValueNameLess ) ;
  MapPubInit ( SetCharNameValue  ) ( mapPubCharNameValue  , NameValueLess ) ;
  for ( int i = sizeof  ( acharstring ) / sizeof  ( tcharstring ) ; i ; ) {
    --  i ;
    MapNode(SetCharValueName) mn = { acharstring [ i ].c,acharstring [ i ].s} ;
    MapPubInsert ( SetCharValueName  ) ( mapPubCharValueName, mn ) ;
    MapNode(SetCharNameValue) nv = { acharstring [ i ].s,acharstring [ i ].c} ;
    MapPubInsert ( SetCharNameValue  ) ( mapPubCharNameValue , nv ) ;
  }
}

void  DeleteSetCharNameValue  ( void  ) {
  MapPubDestr  ( SetCharNameValue  ) ( mapPubCharNameValue  ) ;
  MapPubDestr  ( SetCharValueName  ) ( mapPubCharValueName  ) ;
}
  
void  PLOChar_staticCoutToInterpret ( ParaLispObject const * const me , PLOStream  const * const s ,
  bool  _1 __attribute__ ((unused)) , Vector ( StackPLOP ) * _2 __attribute__ ((unused)) ) {
  PLOStreamPushWideCharArray  ( s , ( PArray_wchar_c) { (wchar_t const(*)[]) & L"#\\"  } ) ;
  size_t  const mapsize = MapPubSize ( SetCharValueName  ) ( mapPubCharValueName  ) ;
  PLOCharPriv const * const pr = CharPub_cast_CharPrivConst ( ParaLispObjectToPLOCharConst (  me  ) ) ;
  size_t  const find  = MapPubFind ( SetCharValueName  ) ( mapPubCharValueName  , pr -> value ) ;
  if ( find ==  mapsize )
    PLOStreamPushWChar ( s  , pr -> value  ) ;
  else
    PLOStreamPushWideCharArray  ( s , (PArray_wchar_c) { (wchar_t const(*)[])
      MapPubCAt ( SetCharValueName  ) ( mapPubCharValueName  , find ) -> value } ) ;
}

# include "stack/public.h"

// char symbol
TaskForThread Lisp2Dbimcharsymbol2(StackClusterPrivate  * const stclpp){
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
  wchar_t ws [ 2 ] ;
  ws [ 0 ] = PLOChar_Value  ( ParaLispObjectToPLOChar ( StackClusterPrivate_ThisTop ( stclpp )  ) ) ;
  ws [ 1 ] = L'\00' ;
  StringConstrCStr ( WideChar )  ( wsp , ws , false ) ;
  StackClusterPrivate_SetResult ( stclpp , SymbolAddMoveString  ( wsp ) ) ;
  Lisp2D_StackFree  ( oldstack ) ;
  return  TaskForThread_Analyse ;
}

# define  voidp_cast_dequebytep Lisp2D_voidp_cast_dequebytep
inline  Deque ( Byte  ) * voidp_cast_dequebytep ( void * p ) ;
  
# include "deque/public-byte.h"
# include "bytedeque/struct.h"
# include "bytedeque/public.h"

// char byte-vector
TaskForThread Lisp2Dbimcharbytevector ( StackClusterPrivate  * const stclpp  ) {
  { ThreadStackPointerType const oldstack_myp = Lisp2DThreadSP ( ) ;
    Deque ( Byte  ) * const deqp  = Lisp2D_StackVar ( StackType ( Deque_Byte  ) ) ;
    Deque_Init  ( Byte )  ( deqp ) ;
    { PLOCharPub * const mep  = ParaLispObjectToPLOChar ( StackClusterPrivate_ThisTop( stclpp ) ) ;
      if  ( isBADWCharPushToByteDeque ( PLOChar_Value( mep ) , deqp ) ) {
        StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ;
        goto Exit ;
      }
    }
    PLOByteDeque  * const v2  = aligned_alloc ( _Alignof  ( PLOByteDeque  ) , sizeof  ( PLOByteDeque  ) ) ;
    if ( not v2  ) {
      OutOfMemory ( stclpp ) ;
      goto Exit ;
    }
    PLOByteDeque_Init(v2);
    ParaLispObject_virtualPushAsNew (&v2  ->  plo ) ;
    PLOByteDeque_SwapDeque  ( v2 , deqp ) ;
    StackClusterPrivate_SetResult (stclpp ,&  v2 -> plo  ) ;
Exit :
    Lisp2D_StackFree  ( oldstack_myp ) ;
  } // deqp
  return  TaskForThread_Analyse ;
}

# include "cons/inline.h"

/*
('char  defmethod  char  (&rest  x)  
  ('x const)  
  (nil  if  x
    (this setf  (x  first)) 
    this))
*/
//  char char [char]
TaskForThread Lisp2Dbimcharchar ( StackClusterPrivate  * const stclpp ) {
  PLOCharPub * const me = ParaLispObjectToPLOChar  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  if  ( not  CConsp ( StackClusterPrivate_ArgTop  ( stclpp )  ) )
    goto Result ;
  { ParaLispObject  * const arg = ClassCons_First ( & ParaLispObjectToPLOCons (
      StackClusterPrivate_ArgTop  ( stclpp ) ) -> cc ) ;
    if  ( IsNotChar (  stclpp , arg ) ) {
      StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
      goto Exit ;
    }
    PLOChar_Set ( me  , ParaLispObjectToPLOChar ( arg ) ) ;
  }
Result :
  StackClusterPrivate_SetResult ( stclpp ,  & me  ->  plo ) ;
Exit :
  return  TaskForThread_Analyse ;
}
  
# include "plocomplex/public.h"
  
//  char number
TaskForThread Lisp2Dbimcharnumber(StackClusterPrivate  * const stclpp)  {
  PLOComplex  * const  res = NewComplexBareExc ( ) ;
  Complex_SetWChar  ( PLOComplexToComplex ( res ) ,
    PLOChar_Value ( ParaLispObjectToPLOChar ( StackClusterPrivate_ThisTop ( stclpp ) ) ) ) ;
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  return  TaskForThread_Analyse;
}

void PLOChar_staticDestr  ( ParaLispObject * const me ) {
  PLOCharPriv * const mep = CharPub_cast_CharPriv ( ParaLispObjectToPLOChar ( me ) ) ;
  PLOChar_Destr ( mep ) ;
}

void  PLOChar_Swap  ( PLOCharPub  * const me , PLOCharPub  * const x  ) {
  if  ( me not_eq  x ) {
    PLOCharPriv * const mep = CharPub_cast_CharPriv ( me ) ;
    PLOCharPriv * const xp = CharPub_cast_CharPriv ( x ) ;
    wchar_t volatile const tmp = mep -> value ;
    mep -> value = xp -> value  ;
    xp -> value  = tmp ;
  }
}

# include "define.h"

void  PLOChar_Init ( PLOCharPriv * const me ) {
  VirtualTypePointer  const vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOChar } ;
  ParaLispObject_InitVTP  ( & me -> pub . plo , vtp ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOChar_constrstat_Inc  ( ) ;
# endif
}

void PLOChar_InitInt ( PLOCharPriv * const me , int const i ) {
  me -> value = i ;
  VirtualTypePointer  const vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOChar } ;
  ParaLispObject_InitVTP  ( & me  -> pub . plo , vtp ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOChar_constrstat_Inc  ( ) ;
# endif
}

void  PLOChar_InitCopy ( PLOCharPriv * const me , PLOCharPub const * const x ) {
  PLOCharPriv const * const xp = CharPub_cast_CharPrivConst ( x ) ;
# ifdef  LISP2D_DEBUG
  if  ( me ==  xp ) {
    fputws  ( L"PLOChar_InitCopy : yourself"  , stderr  ) ;
    return  ;
  }
# endif
  me -> value = xp -> value ;
  VirtualTypePointer  const vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOChar } ;
  ParaLispObject_InitVTP  ( & me -> pub . plo , vtp ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOChar_constrstat_Inc  ( ) ;
# endif
}

bool  PLOChar_LessEq ( PLOCharPub  const * const x  , PLOCharPub  const  * const y ) {
  return not PLOChar_Less ( y  , x ) ;
}

bool  PLOChar_MoreEq ( PLOCharPub  const * const x  , PLOCharPub  const  * const y ) {
  return not PLOChar_Less ( x , y ) ;
}

# include "castb.h"
# include "char/inline.h"
LISP2D_PROTO_MAP_STACK_FREE (  SetCharNameValue )
LISP2DMAPSTACKDESTRBODY (  SetCharNameValue )
LISP2D_MAP_STACK_FREE_BODY (  SetCharNameValue )
LISP2DMAPSTACKPUSHFRONTVALUEBODY ( SetCharValueName )
LISP2D_PROTO_MAP_STACK_FREE (  SetCharValueName )
LISP2DMAPSTACKDESTRBODY (  SetCharValueName )
LISP2D_MAP_STACK_FREE_BODY (  SetCharValueName )
inline  Deque ( Byte  ) * voidp_cast_dequebytep ( void * const p ) {
  return  ( Deque ( Byte  ) * ) p ;
}
# include "char/inlinepri.h"

wchar_t PLOChar_Value  ( PLOCharPub const * const me ) {
  PLOCharPriv const * mep = CharPub_cast_CharPrivConst ( me  ) ;
  return mep -> value  ;
}

void  PLOChar_SetValue ( PLOCharPub  * const me , wchar_t volatile const pc ) {
  PLOCharPriv * mep = CharPub_cast_CharPriv ( me  ) ;
  mep -> value = pc ;
}

# include "stcl/inline.h"
