// Complex All
// 2023
# include <complex.h>

// clang
# ifndef CMPLXL
# define CMPLXL(x, y) ((long double complex)((long double)(x) + \
                      _Complex_I * (long double)(y)))
# endif

# include "complex/private.h"
# include "complex/public.h"
# include "define.h"
# ifdef  LISP2D_CONSTRSTAT
_Atomic ( size_t  ) Complex_constrstat = 0 ;

size_t  Complex_constrstat_Value ( void )  {
  return  Complex_constrstat ;
}

void  Complex_constrstat_Inc  ( void  ) {
  ++  Complex_constrstat  ;
}

# endif // LISP2D_CONSTRSTAT

# include "fraction/public.h"

void  Complex_SetLDComplex  ( Complex * const me , long double complex const x ) {
  ComplexPri * const p = Complex_toPrivate  ( me  ) ;
  FractionSetExactLDouble ( FractionToPub ( & p ->  rep2  ) , creall ( x ) ) ;
  FractionSetExactLDouble ( FractionToPub ( & p ->  imp2  ) , cimagl ( x ) ) ;
}

# include "globallistp/struct.h"

GlobalList  GlobalListOfComplex ;
GlobalList  GlobalListOfComplexGarbage  ;
GlobalList  GlobalListOfComplexGC ;

# include "string/public.h"
# include "complex/inline.h"
# include "stack/public.h"
# include "plostream/public.h"

void  Complex_CoutToInterpret ( Complex const * const mep , PLOStream  const * const  osp ) {
  ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ; 
  String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
  String_Init  ( WideChar  ) ( wsp , true ) ;
  Complex_ToStringForInterpret  (  mep , wsp,  0U );
  PLOStreamPushStringWideChar ( osp , wsp);
  Lisp2D_StackFree  ( oldstack_ws ) ;
}

void  Complex_CoutToHuman ( Complex const * const mep  , PLOStream  const * const  osp ) {
  Lisp2DThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ; 
  String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
  String_Init  ( WideChar  ) ( wsp , true ) ;
  Complex_ToStringForHuman  ( mep , wsp,Complex_CoutDefaultDepth  ) ;
  PLOStreamPushWideCharArray  ( osp , StringCArr ( WideChar ) ( wsp ) ) ;
  Lisp2D_StackFree  ( oldstack_ws ) ;
}

# include "fraction/public.h"

void  Complex_InitCopy  ( Complex * const me , Complex const * const xp ) {
  ComplexPri * const p = Complex_toPrivate  ( me  ) ;
  ComplexPri const * const cx = Complex_toCPrivate  ( xp ) ;
  FractionConstrCopy ( FractionToPub(& p ->  rep2) ,
    FractionCToPub(& cx  ->  rep2) ) ;
  FractionConstrCopy ( FractionToPub(& p ->  imp2) ,
    FractionCToPub(& cx  ->  imp2) ) ;
# ifdef  LISP2D_CONSTRSTAT
  Complex_constrstat_Inc ( ) ;
# endif
}

void  Complex_InitMove( Complex * const me , Complex  * const xp ) {
  ComplexPri * const p = Complex_toPrivate  ( me  ) ;
  ComplexPri * const x = Complex_toPrivate  ( xp ) ;
  FractionConstrMove (&p -> rep2.pub,&x->rep2.pub);
  FractionConstrMove (&p->imp2.pub,&x->imp2.pub);
# ifdef  LISP2D_CONSTRSTAT
  Complex_constrstat_Inc ( ) ;
# endif
}

void  Complex_Set ( Complex  * const mep  , Complex  const  * const  xp ) {
  if ( mep not_eq  xp ) {
    ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
    ComplexPri const * const px = Complex_toCPrivate  ( xp ) ;
    FractionSetFraction  ( FractionToPub(& p ->  rep2 ) ,
      FractionCToPub(& px  ->  rep2) ) ;
    FractionSetFraction  ( FractionToPub(& p ->  imp2 ) ,
      FractionCToPub(& px  ->  imp2 ) ) ;
  }
}

void  Complex_Set0  ( Complex  * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSet0  (FractionToPub(&p->rep2));
  FractionSet0  (FractionToPub(&p->imp2)); 
}

void  ComplexToString ( FuncToStringForInterpretHumanPub fp ,
  Complex  const * const mep ,  String  ( WideChar  ) * const sc  ,
  unsigned  int const digc2 ) {
  if(FractionIs0(Complex_CRe(mep)))  {
    if(FractionIs0(Complex_CIm(mep)))  {
      String_AddChar ( WideChar ) ( sc , L'0' );
      return  ; 
    }
    fp(Complex_CIm(mep),sc,digc2,false);  
  } else {
    fp(Complex_CRe(mep),sc,digc2,false);
    if(FractionIs0(Complex_CIm(mep)))
      return;
    fp(Complex_CIm(mep),sc,digc2,true); 
  }
  if  ( fp == FractionToStringForHuman and ULongIs0 ( FractionCDenomp  ( Complex_CIm(mep))))
    String_AddChar ( WideChar ) ( sc , L'*' );
  String_AddChar ( WideChar ) ( sc , L'i' );
}

void  ComplexPri_Destr ( ComplexPri * const p ) {
  FractionDestr  ( FractionToPub  ( & p ->  imp2 ) ) ;
  FractionDestr  ( FractionToPub  ( & p ->  rep2 ) ) ;  
}

# include "fraction/public.h"

// +0/0+0/0i
void  Complex_Init  ( Complex * const me  ) {
  ComplexPri * const p = Complex_toPrivate  ( me  ) ;
  FractionInit ( & p ->  rep2  . pub ) ;
  FractionInit ( & p ->  imp2  . pub ) ;
# ifdef  LISP2D_CONSTRSTAT
  Complex_constrstat_Inc ( ) ;
# endif
}

bool  Numberp ( ParaLispObject const * const xp  ) {
  return  ParaLispObject_Type(xp)==TypeGlobalListMemberPLOComplex;
}

// this:=1/this
void  Complex_OneDivide ( Complex * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  if(FractionIs0(FractionCToPub(&p->imp2)))
    FractionOverTurn  (FractionToPub(&p->rep2));
  else
    if(FractionIs0(FractionCToPub(&p->rep2))) {
      // x*i -> -(1/x)*i
      FractionOverTurn  (FractionToPub(&p->imp2));
      FractionTurnSign  (FractionToPub(&p->imp2)); 
    } else {
      // r^2=re^2+im^2
      ThreadStackPointerType const oldstack_r2p = Lisp2DThreadSP ( ) ;
      FractionPub * const r2 = NewStackFraction ( ) ;
      FractionConstrCopy ( r2 , FractionCToPub(  & p ->  rep2) ) ;
      FractionSquare  ( r2 ) ;
      { ThreadStackPointerType const oldstack_ip = Lisp2DThreadSP ( ) ;
        FractionPub * const i = NewStackFraction ( ) ;
        FractionConstrCopy ( i , FractionCToPub(& p -> imp2) ) ;
        FractionSquare  ( i ) ;
        FractionAddMoveFraction(r2,i);        
        Lisp2D_StackFree  ( oldstack_ip ) ;
      }
      // (re/(r^2),-im/(r^2))
      FractionDivFractionConst  ( FractionToPub(& p->rep2 ) , r2 ) ;
      FractionDivFractionMove  ( FractionToPub(& p->imp2 ) , r2 ) ;
      Lisp2D_StackFree  ( oldstack_r2p ) ;
      FractionTurnSign  (FractionToPub(&p->imp2));
    }
}

# include "newobject.h" // UniversalIsNotType
      
bool  IsNotNumber( StackClusterPrivate  * const stclp , ParaLispObject * const  xp ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp  ,
    . typetype  = TypeGlobalListMemberPLOComplex ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
}
  
# include "sharedc.h"
# include "shared/inline.h" // ListOfPLO
# include "inerrors.h"
# include "passage/public.h"
# include "nilc.h"

bool  IsNotNumberLockGC ( StackClusterPrivate  * const stclp , ParaLispObject * const  xp ) {
  if  ( Numberp ( xp  ) )
    return  false ;
  static  wchar_t const errs  [ ] = L" is not number" ;
  if  ( stclp ) {
    PassagePassIn ( passageGCCPub ) ;
      { jmp_buf jump_buffer ;
        jmp_buf * const oldjb = Lisp2DJBglobal  ;
        Lisp2DJBglobal = &  jump_buffer ;
        int const volatile exctype = setjmp  ( jump_buffer ) ;
        switch ( exctype )  {
        case  0 :
          { PLOStringPub * const mess = PLOString_NewCopy ( errs ) ;
            ParaLispObject  * list [ ] = { xp , & mess  ->  plo , 0 } ;
            StackClusterPrivate_ExceptionSet  ( stclp , ListOfPLO ( ( PArrayPLOP  ) {(ParaLispObject*(*)[])&list}));
          } // list
          Lisp2DJBglobal = oldjb  ;
          break ;
        default :
          Lisp2DJBglobal = oldjb  ;
          StackClusterPrivate_ExceptionSet  ( stclp , & plonil  . plo ) ;
        }
      }
    PassagePassOut ( passageGCCPub ) ;
  } else
    inErrorS  ( PLOStreamPushPLO  ( & Lisp2Dferr ,  xp  ) ;
      PLOStreamPushWideCharArray  ( & Lisp2Dferr , (  PArray_wchar_c  ) { & errs  } ) ; )
  return  true  ;
}

# include "complex/type.h"
# include "statistics.h"

void  Complex_AddConst ( Complex * const mep , Complex const  * const  x2 ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  ComplexPri const  * const xp = Complex_toCPrivate  ( x2 ) ;
# ifdef  LISP2D_BIMSTAT
  struct  timespec  timeBegin ;
  clock_gettime ( CLOCK_REALTIME  , & timeBegin ) ;
# endif
  // ! if Re == x.Re
  // ! if Im == x.Im
  if  ( mep == x2 ) {
    FractionMul2  ( FractionToPub ( & p ->  rep2  ) ) ;
    FractionMul2  ( FractionToPub ( & p ->  imp2  ) ) ;
  } else  {
    FractionAddConstFraction  ( FractionToPub ( & p ->  rep2  ) , FractionCToPub  ( & xp  ->  rep2  ) ) ;
    FractionAddConstFraction  ( FractionToPub ( & p ->  imp2  ) , FractionCToPub  ( & xp  ->  imp2  ) ) ;
  }
# ifdef  LISP2D_BIMSTAT
  struct  timespec  timeEnd;
  clock_gettime ( CLOCK_REALTIME  , & timeEnd ) ;
  IncreaseStat5 ( & timeBegin , & timeEnd , & statComplexOperatorPlusEq ) ;
# endif
}

void  Complex_AddMove ( Complex * const mep , Complex * const  x2 ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  ComplexPri * const xp = Complex_toPrivate  ( x2 ) ;
  // ! if Re == x.Re
  // ! if Im == x.Im
  if  ( mep == x2 ) {
    FractionMul2  ( FractionToPub ( & p ->  rep2  ) ) ;
    FractionMul2  ( FractionToPub ( & p ->  imp2  ) ) ;
  } else  {
    FractionAddMoveFraction  ( FractionToPub ( & p ->  rep2  ) , FractionToPub  ( & xp  ->  rep2  ) ) ;
    FractionAddMoveFraction  ( FractionToPub ( & p ->  imp2  ) , FractionToPub  ( & xp  ->  imp2  ) ) ;
  }
}

void  Complex_Subtract  ( Complex * const mep , Complex const  * const x2  ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  if  ( mep == x2 ) {
    if(ULongIs0(FractionCDenomp(FractionCToPub(&p->rep2))))   
      FractionSetNaN  (FractionToPub(&p->rep2));
    else
      FractionSet0  (FractionToPub(&p->rep2));
    if(ULongIs0(FractionCDenomp(FractionCToPub(&p->imp2))))    
      FractionSetNaN  (FractionToPub(&p->imp2));
    else
      FractionSet0  (FractionToPub(&p->imp2));
  } else  {
    ComplexPri const  * const xp = Complex_toCPrivate  ( x2 ) ;
    FractionSubtractConst  ( FractionToPub(& p->rep2) ,
      FractionCToPub(& xp -> rep2 )) ;
    FractionSubtractConst  ( FractionToPub(& p->imp2 ) ,
      FractionCToPub(& xp -> imp2 )) ;
  }  
}

// res = x.re^2 + x.im^2  
static  void Abs2(Complex const * const x,FractionPub * const resp){
  FractionSetFraction(resp,Complex_CRe(x));
  FractionSquare  ( resp ) ;
  ThreadStackPointerType const oldstack_ip = Lisp2DThreadSP ( ) ;
  FractionPub * const i = NewStackFraction  ( ) ;
  FractionConstrCopy ( i , Complex_CIm ( x ) ) ;
  FractionSquare  ( i ) ;
  FractionAddMoveFraction(resp,i);
  Lisp2D_StackFree  ( oldstack_ip ) ;
}  

void  Complex_Div ( Complex * const mep , Complex const * const xp ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  if ( mep == xp ) {
    // to check of all zeros in chast and delit, imag and reals
    if((ULongIs0(FractionCDenomp(FractionCToPub(&p->rep2)))) or 
      (ULongIs0(FractionCDenomp(FractionCToPub(&p->imp2))))  or
      ((ULongIs0(FractionCNump(FractionCToPub(&p->rep2)))) and 
        (ULongIs0(FractionCNump(FractionCToPub(&p->imp2)))))){
      FractionSetNaN  (FractionToPub(&p->rep2));
      FractionSetNaN  (FractionToPub(&p->imp2));  
    } else{
      // (1,0)      
      FractionSet1  ( FractionToPub(& p->rep2 ) ) ;
      FractionSet0  (FractionToPub(&p->imp2));
    }
    return ;
  }
  if(FractionIs0(FractionCToPub(&p->imp2))){
    if(FractionIs0(Complex_CIm(xp)))
      // protection for 1/0=inf
      FractionDivFractionConst  ( Complex_Re(mep) ,
         Complex_CRe(xp) ) ;      
    else  {
      if(FractionIs0(Complex_CRe(xp))){
        // re / x.im => (0,-re/x.im)
        FractionDivFractionConst  ( FractionToPub(& p->rep2 ) ,
           Complex_CIm(xp) ) ;
        FractionTurnSign  (FractionToPub(&p->rep2));
        FractionSwapFraction ( FractionToPub(& p ->  rep2  ) ,
          FractionToPub(& p ->  imp2 ) ) ;
      } else{
        // 0/(x+iy) -> 0
        if(FractionIs0(FractionCToPub(&p->rep2)))
          return ;
        // r^2=x.re^2+x.im^2
        ThreadStackPointerType const oldstack_r2p = Lisp2DThreadSP ( ) ;
        FractionPub * const r2 = NewStackFraction ( ) ;
        FractionInit ( r2 ) ;
        Abs2(xp,r2);
        // ((re*x.re)/(r^2),(-re*x.im)/(r^2))
        FractionSetFraction(FractionToPub(&p->imp2 ) , FractionCToPub(&p->rep2 ) );
        FractionMultConstFraction ( FractionToPub(& p->imp2 ) , Complex_CIm (xp ) ) ;
        FractionDivFractionConst  ( FractionToPub(& p->imp2 ) , r2 ) ;
        FractionTurnSign  (FractionToPub(&p->imp2));
        FractionMultConstFraction ( FractionToPub(& p->rep2) , Complex_CRe (xp ) ) ;
        FractionDivFractionMove  ( FractionToPub( &p->rep2 ) , r2 ) ;
        Lisp2D_StackFree  ( oldstack_r2p ) ;
      }
    }
    return ; 
  }
  if(FractionIs0(FractionCToPub(&p->rep2))) {
    if(FractionIs0(Complex_CIm(xp)))
      // im / x.re => (0,im/x.re)  
      FractionDivFractionConst  ( FractionToPub(& p->imp2 ) , Complex_CRe(xp) ) ;
    else {
      if(FractionIs0(Complex_CRe(xp))) {
        // im / x.im => (im/x.im,0)
        FractionDivFractionConst  ( FractionToPub(& p->imp2 ) , Complex_CIm(xp)  ) ;
        FractionSwapFraction ( FractionToPub(& p ->  rep2  ) ,
          FractionToPub(& p ->  imp2 ) ) ; 
      } else {
        // r^2=x.re^2+x.im^2
        ThreadStackPointerType const oldstack_r2p = Lisp2DThreadSP ( ) ;
        FractionPub * const r2 = NewStackFraction ( ) ;
        FractionInit (  r2 ) ;    
        Abs2(xp,r2);
        // ((im*x.im)/(r^2),(im*x.re)/(r^2))  
        FractionSetFraction(FractionToPub(&p->rep2 ) ,
          FractionCToPub(&p->imp2 ) );
        FractionMultConstFraction ( FractionToPub(& p->rep2) ,
           Complex_CIm (xp ) ) ;
        FractionDivFractionConst  ( FractionToPub(& p->rep2 ) , 
           r2 ) ;
        FractionMultConstFraction ( FractionToPub(& p->imp2) ,
           Complex_CRe (xp ) ) ;
        FractionDivFractionMove  ( FractionToPub(& p->imp2) , 
          r2 ) ;    
        Lisp2D_StackFree  ( oldstack_r2p ) ;
      }
    }
    return ;
  }
  if(FractionIs0(Complex_CIm(xp))){
    // (re/x.re,im/x.re)
    FractionDivFractionConst  ( FractionToPub(& p->rep2 ) ,
       Complex_CRe(xp) ) ;
    FractionDivFractionConst  ( FractionToPub(& p->imp2 ) ,
       Complex_CRe(xp) ) ;
    return ;
  }
  if(FractionIs0(Complex_CRe(xp))){
    // (im/x.im,-re/x.im)
    FractionDivFractionConst  ( FractionToPub(& p->rep2 ) ,
       Complex_CIm(xp) ) ;
    FractionTurnSign  (FractionToPub(&p->rep2));
    FractionDivFractionConst  ( FractionToPub(& p->imp2 ) ,
      Complex_CIm(xp) ) ;
    FractionSwapFraction ( FractionToPub(& p ->  rep2 ) ,
      FractionToPub(& p ->  imp2 ) ) ;
    return ;
  }
# ifdef LISP2D_BIMSTAT
  struct  timespec  timeBegin ;
  clock_gettime ( CLOCK_REALTIME  , & timeBegin ) ;
# endif    
  // BIM(Complex,operator  /=(Complex const  &  x)&) n=32768 fulltime=196
  // M=0.00597 σ=0.000975 ( n=16703 ( 51% ) M=0.00534 σ=0.000645 , 
  // n=16065 ( 49% ) M=0.00663 σ=0.000817 )
  // r^2=x.re^2+x.im^2
  ThreadStackPointerType const oldstack_r2p = Lisp2DThreadSP ( ) ;
  FractionPub * const r2 = NewStackFraction ( ) ;
  FractionInit ( r2 ) ;    
  Abs2(xp,r2);
  // ((re*x.re+im*x.im)/(r^2),(im*x.re-re*x.im)/(r^2))
  FractionPub * const re2 = NewStackFraction  ( ) ;
  FractionConstrCopy ( re2 , FractionCToPub(& p ->  rep2) ) ;
  FractionMultConstFraction ( re2 , Complex_CRe (xp ) ) ;
  { ThreadStackPointerType const oldstack_re3p = Lisp2DThreadSP ( ) ;
    FractionPub * const re3 = NewStackFraction  ( ) ;
    FractionConstrCopy ( re3 , FractionCToPub(& p ->  imp2) ) ;
    FractionMultConstFraction ( re3 , Complex_CIm (xp ) ) ;
    FractionAddMoveFraction(re2,re3);
    Lisp2D_StackFree  ( oldstack_re3p ) ;
  }
  FractionDivFractionConst  ( re2 ,  r2 ) ;
  FractionMultConstFraction ( FractionToPub(& p->imp2)  ,
     Complex_CRe (xp ) ) ;
  FractionMultConstFraction ( FractionToPub(& p->rep2) ,
     Complex_CIm (xp ) ) ;
  FractionSubtractMove  ( FractionToPub(& p->imp2 ) ,
    FractionToPub(& p->rep2) ) ;
  FractionDivFractionMove  ( FractionToPub(& p->imp2 ) , r2 ) ;
  FractionSwapFraction ( FractionToPub(& p ->  rep2 ) , re2 ) ;
  Lisp2D_StackFree  ( oldstack_r2p ) ;
# ifdef LISP2D_BIMSTAT
  struct  timespec  timeEnd ;
  clock_gettime ( CLOCK_REALTIME  , & timeEnd ) ;
  IncreaseStat5  (& timeBegin ,& timeEnd ,& statComplexOperatorDivEq ) ;
# endif        
}

void  Complex_Mul ( Complex * const mep , Complex const  * const xp ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  if( mep ==  xp ) {
    if(FractionIs0(FractionCToPub(&p->imp2)))
      // (re*re,0)
      FractionSquare  ( FractionToPub(& p->rep2 ) ) ;  
    else if(FractionIs0(FractionCToPub(&p->rep2))) {
        // (-im*im,0)
        FractionSwapFraction ( FractionToPub(& p ->  rep2  ) , FractionToPub(& p ->  imp2  ) ) ;
        FractionSquare  ( FractionToPub(& p->rep2 ) ) ;  
        FractionTurnSign  ( FractionToPub(&p->rep2))  ;
    } else{
        // (re*re-im*im,2*im*re)
        { ThreadStackPointerType const oldstack_im2 = Lisp2DThreadSP ( ) ;
          FractionPub * const im2 = NewStackFraction ( ) ;
          FractionConstrCopy ( im2 , FractionCToPub(& p ->  imp2 ) ) ;
          FractionSquare  ( im2 ) ;        
          FractionMultConstFraction ( FractionToPub(& p->imp2) , FractionCToPub(& p->rep2) ) ;
          FractionSquare  ( FractionToPub(& p->rep2 ) ) ;          
          FractionSubtractMove  ( FractionToPub(& p->rep2 ) , im2 ) ;
          Lisp2D_StackFree  ( oldstack_im2 ) ;
        }
        FractionMul2  ( FractionToPub(&p->imp2));     
    }
    return ;
  }

/*
  !! BUG ??
  +1/0 * +1/0 == +1/0  ( now 1/0+0/0i )
  +1/0 * +1/0i == +1/0i ( now 0/0+1/0i )
  +1/0 * -1/0 == -1/0 ( now -1/0+0/0i )
  +1/0 * -1/0i == -1/0i ( now 0/0-1/0i )
  +1/0i * +1/0 == +1/0i ( now 0/0+1/0i )
  +1/0i * +1/0i == -1/0 ( now -1/0+0/0i )
  +1/0i * -1/0 == -1/0i ( now 0/0-1/0i )
  +1/0i * -1/0i == +1/0 ( now 1/0+0/0i )
*/

  if  (FractionIs0(FractionCToPub(&p->imp2))) {
    // (re*x.re,re*x.im)
    FractionSetFraction(FractionToPub(&p->imp2 ) ,FractionCToPub(&p->rep2 ) );
    FractionMultConstFraction ( FractionToPub(& p->imp2) , Complex_CIm (xp ) ) ;
    FractionMultConstFraction ( FractionToPub(& p->rep2) , Complex_CRe (xp ) ) ;
  } else  if  (FractionIs0(Complex_CIm(xp)))  {
      // (re*x.re,im*x.re)
      FractionMultConstFraction ( FractionToPub(& p->rep2) , Complex_CRe (xp ) ) ;
      FractionMultConstFraction ( FractionToPub(& p->imp2) , Complex_CRe (xp ) ) ;
  } else if (FractionIs0(FractionCToPub(&p->rep2)))  {
/*
  !! BUG ??
  !! 1/0i * 1/0i == -1/0 ??
  !! 1/0i * -1/0i == +1/0 ??
*/
        // (-im*x.im,im*x.re)
        FractionSetFraction(FractionToPub(&p->rep2 ) , FractionCToPub(&p->imp2 ) );
        FractionMultConstFraction ( FractionToPub(& p->rep2) , Complex_CIm (xp ) ) ;
        FractionTurnSign  (FractionToPub(&p->rep2));
        FractionMultConstFraction ( FractionToPub(& p->imp2) , Complex_CRe (xp ) ) ;
  } else if (FractionIs0(Complex_CRe(xp))) {
          // (-im*x.im,re*x.im)
          FractionMultConstFraction ( FractionToPub(& p->rep2) , Complex_CIm (xp ) ) ;
          FractionMultConstFraction ( FractionToPub(& p->imp2) , Complex_CIm (xp ) ) ;
          FractionTurnSign  (FractionToPub(&p->imp2));
          FractionSwapFraction ( FractionToPub(& p ->  rep2  ) , FractionToPub(& p ->  imp2 ) ) ;
        } else  { 
# ifdef LISP2D_BIMSTAT
          struct  timespec  timeBegin ;
          clock_gettime ( CLOCK_REALTIME  , & timeBegin ) ;
# endif
# if  true
  // BIM(Complex,operator  *=(Complex const  &  x)&) n=32768 fulltime=87.8
  // M=0.00268 σ=0.000525 ( n=17139 ( 52.3% ) M=0.00235 σ=0.000282 , 
  // n=15629 ( 47.7% ) M=0.00304 σ=0.000489 )
    // (re*x.re-im*x.im,im*x.re+re*x.im)
          { ThreadStackPointerType const oldstack_re3 = Lisp2DThreadSP ( ) ;
            FractionPub * const re3 = NewStackFraction ( ) ;
            FractionConstrCopy ( re3 , FractionCToPub(& p ->  imp2) ) ;
            FractionMultConstFraction ( re3 , Complex_CIm (xp ) ) ;
            { ThreadStackPointerType const oldstack_re2 = Lisp2DThreadSP ( ) ;
              FractionPub * const re2 = NewStackFraction ( ) ;
              FractionConstrCopy ( re2 , FractionCToPub(& p ->  rep2) ) ;
              FractionMultConstFraction ( re2 , Complex_CRe (xp ) ) ;
              FractionMultConstFraction ( FractionToPub(& p->imp2) , Complex_CRe (xp ) ) ;
              FractionMultConstFraction ( FractionToPub(& p->rep2) , Complex_CIm (xp ) ) ;
              FractionAddMoveFraction(FractionToPub(&p->imp2), FractionToPub(&p->rep2));
              FractionSwapFraction ( FractionToPub(& p ->  rep2  ) , re2 ) ;
              Lisp2D_StackFree  ( oldstack_re2 ) ;
            }
            FractionSubtractMove  ( FractionToPub(& p->rep2 ) , re3 ) ;
            Lisp2D_StackFree  ( oldstack_re3 ) ;
          } // re3p
# endif
# ifdef LISP2D_BIMSTAT
    struct  timespec  timeEnd ;
    clock_gettime ( CLOCK_REALTIME  , & timeEnd ) ;
    IncreaseStat5  ( &timeBegin ,& timeEnd ,& statComplexOperatorMulEq ) ;
# endif        
  }
}

void  Complex_MulFraction ( Complex * const  mep , FractionPub const  * const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  // (re*x,im*x)
  FractionMultConstFraction ( FractionToPub(& p  ->  rep2) , x ) ;
  FractionMultConstFraction ( FractionToPub(& p  ->  imp2) , x ) ;
}

bool  Complex_Eq(Complex  const * const  xp,Complex const  * const  yp){
  return  FractionEq(Complex_CRe(xp), Complex_CRe(yp)) and
    FractionEq(Complex_CIm(xp),Complex_CIm(yp));
}

bool  Complex_NotEq(Complex  const * const  xp,Complex const  * const  yp){
  return  FractionNotEq(Complex_CRe(xp) , Complex_CRe(yp)) or
    FractionNotEq(Complex_CIm(xp) , Complex_CIm(yp));
}
    
bool  Complex_IsInf ( Complex const * const  xp ) {
  return  FractionIsInf(Complex_CRe(xp))  or
    FractionIsInf(Complex_CIm(xp)); 
}

Lisp2DBoolType Complex_Less(Complex  const * const xp,Complex const * const yp)
{
  if(not  (FractionIs0(Complex_CIm(xp))))
    return  Lisp2DBoolUndef;
  if(not  (FractionIs0(Complex_CIm(yp))))
    return  Lisp2DBoolUndef;
  return  FractionLess(Complex_CRe(xp), Complex_CRe(yp));
}

void  Complex_SetULong  ( Complex * const mep , ULongPub  const * const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetULong  ( FractionToPub ( & p ->  rep2 ) , x ) ;
  FractionSet0  (FractionToPub(&p->imp2)); 
}

void  Complex_SwapULong ( Complex * const mep , ULongPub * const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSwapULong ( FractionToPub ( & p ->  rep2 ) , x ) ;
  FractionSet0  ( FractionToPub ( & p ->  imp2  ) ) ;
}

void  Complex_MoveULong ( Complex * const mep , ULongPub * const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionMoveULong ( FractionToPub ( & p ->  rep2 ) , x ) ;
  FractionSet0  ( FractionToPub ( & p ->  imp2  ) ) ;
}

void  Complex_SetULInt  ( Complex * const mep , unsigned long  int const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetULInt  (FractionToPub(&p->rep2),x);
  FractionSet0  (FractionToPub(&p->imp2));
}

void  Complex_SetLInt ( Complex * const mep , long  int const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetLInt (FractionToPub(&p->rep2),x);
  FractionSet0  (FractionToPub(&p->imp2)); 
}
  
void  Complex_SetOff  ( Complex * const mep , off_t const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetLInt (FractionToPub(&p->rep2),x);
  FractionSet0  (FractionToPub(&p->imp2));  
}

void  Complex_SetSSize ( Complex * const mep , ssize_t const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetSSize (FractionToPub(&p->rep2), x ) ;
  FractionSet0  (FractionToPub(&p->imp2));  
}
  
void  Complex_SetExactFromLD  ( Complex * const mep , long  double  const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetExactLDouble  ( & p  ->  rep2  . pub , x ) ;
  FractionSet0  (FractionToPub(&p->imp2));
}

void Complex_SetInt  ( Complex * const mep , int const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetInt  ( FractionToPub(&p->rep2),x);
  FractionSet0  (FractionToPub(&p->imp2)); 
} 

void  Complex_SetUInt ( Complex * const mep , unsigned int const x  ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetUInt ( FractionToPub(& p->rep2) , x ) ;
  FractionSet0  (FractionToPub(&p->imp2)); 
}

bool  Complex_isBADUCharValue ( Complex const * const mep ,
  unsigned  char * const resp ) {
  if(FractionIs0(Complex_CIm(mep)))
    return   FractionIsBADUCharValue ( Complex_CRe(mep),resp) ;
  return  true;
}  

void  Complex_SetFraction ( Complex * const mep ,
  FractionPub const  * const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetFraction(FractionToPub(&p->rep2 ) , x )  ;
  FractionSet0(FractionToPub(&p->imp2));
}

void  Complex_SwapFraction  ( Complex * const mep  , FractionPub * const f ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSwapFraction ( FractionToPub(& p ->  rep2 ) , f ) ;
  FractionSet0 ( FractionToPub(& p ->  imp2 ) ) ;
}

# include "ulong/private.h"

// recursive
void  Complex_Power ( Complex * const mep , ULongPub  const  * const  n ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  if(ULongIs0( n )){
    if(Complex_Is0(mep)) {
      FractionSetNaN  (FractionToPub(&p-> rep2));
      FractionSetNaN  (FractionToPub(&p-> imp2));  
    } else{
      FractionSet1  ( FractionToPub(& p-> rep2 ) ) ;
      FractionSet0(FractionToPub(&p-> imp2));
    }
    return  ;
  }
  if  ( ULongIs1  ( n ) )
    return  ;
  /*
 !! BUG ??
   ( 1/0+0/1i ) ^ 2 => 1/0
 !! TO DO ??
   ( 1/0+0/1i ) ^ 2 => 1/0+0/0i
  */
  { ThreadStackPointerType const oldstack_n2 = Lisp2DThreadSP ( ) ;
    ULongPub * const n2 = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( n2 , n ) ;
    ULongDiv2 ( n2 ) ;
  // n2=[n/2]
    if  ( ULongOddP ( n ) ) {
      // n%2=1
      Complex * const xp = Complex_FromStack ( ) ;
      Complex_InitCopy  ( xp , mep  ) ;
      Complex_Power ( mep , n2 ) ;
      // x^([n]/2)
      Complex_Mul( mep , mep );
      // x^(n-1)
      Complex_Mul( mep , xp );
      Lisp2D_StackFree  ( oldstack_n2 ) ;
      return  ;  
    }
    // n%2=0
    Complex_Power ( mep , n2 ) ;
    Lisp2D_StackFree  ( oldstack_n2 ) ;
  }
  // x^(n/2)
  Complex_Mul( mep , mep ); 
}

bool  Complex_Is0(Complex const * const xp ) { 
  return  FractionIs0(Complex_CRe(xp)) and  FractionIs0(Complex_CIm(xp));
}

# include <math.h>

void  Complex_Ln  ( Complex  const * const xp  , Complex * const resultp  ) {
  if  (   xp  ==    resultp   ) {
    ThreadStackPointerType const oldstack_x2 = Lisp2DThreadSP ( ) ;
    Complex * const x2p = Complex_FromStack ( ) ;
    Complex_InitCopy  ( x2p ,  xp ) ;
    Complex_Ln  ( x2p , resultp  ) ;
    Lisp2D_StackFree  ( oldstack_x2 ) ;
    return  ;
  }
  if  ( FractionIs0  (  Complex_CIm (xp ) ) ) {
    if  ( FractionIsNaN  ( Complex_CRe (xp ) ) ) {
      FractionSetNaN ( Complex_Re  (resultp ) ) ;
      FractionSet0 ( Complex_Im  ( resultp ) ) ;  
      return  ; 
    }
    if  ( FractionMoreEqUInt (  Complex_CRe (xp ) , 0U  ) ) {
      FractionLn (  Complex_CRe (xp ) ,
        Complex_Re  (resultp ) ) ;
      FractionSet0 ( Complex_Im  (resultp ) ) ;
      return  ;
    }
    { ThreadStackPointerType const oldstack_x2p = Lisp2DThreadSP ( ) ;
      FractionPub * const x2 = NewStackFraction ( ) ;
      FractionConstrCopy ( x2 , Complex_CRe ( xp )  ) ;
      FractionAbs  ( x2 ) ;
      // ln(-r)=ln(r)+I*(PI +- 2PI)
      FractionLn ( x2 , Complex_Re  (resultp ) ) ;
      Lisp2D_StackFree  ( oldstack_x2p ) ;
    }
    FractionSetFraction ( Complex_Im(resultp) , FractionploConstPiPub ) ;
    return  ;
  }
    // z=r*exp(I*phi)
    // ln(z)=ln(r) + I*phi = ln(re*re+im*im)/2 + I*phi
    { ThreadStackPointerType const oldstack_x2p = Lisp2DThreadSP ( ) ;
      FractionPub * const x2 = NewStackFraction ( ) ;
      FractionConstrCopy (  x2 , Complex_CRe ( xp  ) ) ;
      FractionMultConstFraction ( x2 , Complex_CRe(xp) ) ;
      { ThreadStackPointerType const oldstack_y2p = Lisp2DThreadSP ( ) ;
        FractionPub * const y2 = NewStackFraction ( ) ;
        FractionConstrCopy ( y2 , Complex_CIm ( xp  ) ) ;
        FractionMultConstFraction ( y2 , Complex_CIm(xp) ) ;
        FractionAddMoveFraction(x2,y2);
        Lisp2D_StackFree  ( oldstack_y2p ) ;
      }
      FractionLn(x2,Complex_Re(resultp));
      Lisp2D_StackFree  ( oldstack_x2p ) ;
    }
    FractionDiv2  (Complex_Re(resultp));
    long  double const lx = FractionLDValueConst( Complex_CIm(xp));
    long  double const ly = FractionLDValueConst( Complex_CRe(xp));
    FractionSetExactLDouble ( Complex_Im  (resultp ) , atan2l ( lx , ly ) ) ;
}

bool  Complex_Integerp(Complex  const * const xp){
  return  FractionIs0 ( Complex_CIm ( xp  ) ) and
    ULongIs1  ( FractionCDenomp  ( Complex_CRe ( xp  ) ) ) ;
}

void  Complex_Factorial ( Complex * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionFactorial(FractionToPub(&p-> rep2));
  FractionSet0(FractionToPub(&p-> imp2)); 
}

void  Complex_DivFraction ( Complex * const mep , FractionPub  const  * const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionDivFractionConst  ( FractionToPub(& p->rep2 ) , x ) ;
  FractionDivFractionConst  ( FractionToPub(& p->imp2 ) , x ) ;
}

void  Complex_MulUInt ( Complex * const mep , unsigned  int const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionMulUInt (FractionToPub(&p->rep2),x);
  FractionMulUInt (FractionToPub(&p->imp2),x);  
}

void  Complex_DivUInt ( Complex * const mep , unsigned  int const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionDivUInt(FractionToPub(&p->rep2),x);
  FractionDivUInt(FractionToPub(&p->imp2),x); 
}
  
bool  Complex_isReal  ( Complex  const * const  mep ) {
  return  FractionIs0(Complex_CIm( mep));
}

int Complex_IValue(Complex  const * const xp ) { 
  int res ;
  if(FractionIsBADIntValue(Complex_CRe(xp),&res))
    return 0 ;
  return res ;
}

bool  Complex_IsBADIntValue ( Complex const * const xp ,  int * const resp ) {
  if(not  Complex_isReal(xp)) 
    return  true ;
  return  FractionIsBADIntValue ( Complex_CRe(xp),resp);
}

bool  Complex_IsBADUI16Value  ( Complex  const * const xp , uint16_t  * const resp ) {
  if(not  Complex_isReal(xp))  
    return  true;
  return  FractionIsBADUI16Value  ( Complex_CRe(xp),resp); 
}

void  Complex_SetRealFractionUInt ( Complex * const mep ,
  unsigned  int const num  , unsigned int const den ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetUIntUInt ( FractionToPub(& p->rep2 ) , num  , den ) ;
  FractionSet0(FractionToPub(&p->imp2));
}

void  Complex_SetUSize  ( Complex * const mep , size_t  const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetUSize  (FractionToPub(&p->rep2),x);
  FractionSet0(FractionToPub(&p->imp2)); 
}

void  Complex_SetWChar ( Complex * const mep , wchar_t const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetWChar  (FractionToPub(&p->rep2),x);
  FractionSet0(FractionToPub(&p->imp2));  
}

void  Complex_Ceil  ( Complex * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionCeil  (FractionToPub(&p->rep2));
  FractionCeil  (FractionToPub(&p->imp2));  
}

void  Complex_CeilDiv  ( Complex * const mep , Complex const * const dp ) {
  Complex_Div( mep , dp ) ;
  Complex_Ceil ( mep ) ; 
}

void  Complex_Floor ( Complex * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionFloor (FractionToPub(&p->rep2));
  FractionFloor (FractionToPub(&p->imp2)); 
}

void  Complex_FloorDiv ( Complex * const mep , Complex const * const dp ) {
  Complex_Div ( mep ,  dp ) ;
  Complex_Floor ( mep  ) ; 
}

void  Complex_Trunc ( Complex * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionTrunc (FractionToPub(&p->rep2));
  FractionTrunc (FractionToPub(&p->imp2)); 
}

void  Complex_TruncDiv ( Complex * const mep , Complex const * const dp ) {
  Complex_Div ( mep ,  dp ) ;
  Complex_Trunc (  mep  ) ; 
}

void  Complex_ModEq  ( Complex * const mep , Complex const * const dp ) {
  if  ( mep ==  dp ) {
    Complex_Set0  ( mep  ) ;
    goto  Exit  ; 
  }
  { ThreadStackPointerType const oldstack_z = Lisp2DThreadSP ( ) ;
    Complex * const zp = Complex_FromStack ( ) ;
    Complex_InitCopy  (  zp , mep  ) ;
    Complex_FloorDiv ( zp , dp ) ;
    Complex_Mul ( zp , dp ) ;
    Complex_Subtract  ( mep  , zp ) ;
    Lisp2D_StackFree  ( oldstack_z ) ;
  } // zp
Exit  :
  ; 
}

void  Complex_Swap  ( Complex * const mep , Complex  * const x2  ) {
  if  ( mep  not_eq  x2 ) {
    ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
    ComplexPri * const xp = Complex_toPrivate  ( x2 ) ;
    FractionSwapFraction ( FractionToPub(& p ->  rep2  ) ,
      FractionToPub(& xp  ->  rep2  ) ) ;
    FractionSwapFraction ( FractionToPub(& p ->  imp2 ) ,
      FractionToPub(& xp  ->  imp2  ) ) ;
  }
}  

void  Complex_Move  ( Complex * const mep , Complex  * const x2  ) {
  if  ( mep  not_eq  x2 ) {
    ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
    ComplexPri * const xp = Complex_toPrivate  ( x2 ) ;
    FractionMoveFraction ( FractionToPub  ( & p ->  rep2  ) , FractionToPub ( & xp  ->  rep2  ) ) ;
    FractionMoveFraction ( FractionToPub  ( & p ->  imp2  ) , FractionToPub ( & xp  ->  imp2  ) ) ;
  }
}  

void  Complex_Cbrt  ( Complex * const mep , FractionPub  const * const epsp ) {
  if  ( FractionIs0  ( Complex_CIm (mep ) ) ) {
    // real
    if  ( FractionIs0  ( Complex_CRe (mep ) ) )
      return  ;
    // +2 ^ 1/3 => +1.26
    // -2 ^ 1/3 => -1.26
    Fraction_Cbrt_Rel ( Complex_Re(mep) , epsp ) ;
    return  ; 
  }
  // im<>0
  if(FractionIs0(Complex_CRe(mep))){
    // +2i ^ 1/3 => -1.26i
    // -2i ^ 1/3 => +1.26i
    Fraction_Cbrt_Rel ( Complex_Im(mep) , epsp ) ;
    FractionTurnSign(Complex_Im(mep));
    return  ; 
  }
/*
 ! Abs(re)==Abs(im)
 cbrt(±a±ai)=∓b±bi
 b:=cbrt(a/2)
 -2+2i ->  1+1i
 -2-2i ->  1-1i
  2+2i -> -1+1i
  2-2i -> -1-1i
  sign real -> reverse
  sign imag -> the same
 */
  { bool flageq = false ;
    { ULongPub const * const  xrech = FractionCNump( Complex_CRe(mep) );
      ULongPub const * const  xrede = FractionCDenomp(Complex_CRe(mep) );
      ULongPub const * const  ximch = FractionCNump( Complex_CIm (mep ) );
      ULongPub const * const  ximde = FractionCDenomp( Complex_CIm (mep ) );
      if  ( ULongisEqULong  ( xrech , ximch ) )
        if  ( ULongisEqULong  ( xrede , ximde ) )
          flageq = true ;
    }
    if  ( flageq  ) {
      FractionDiv2  (Complex_Im(mep));
      Fraction_Cbrt_Rel ( Complex_Im(mep) , epsp ) ;
      bool const wasRenegativ = FractionIsNegative(Complex_CRe(mep) ) ;
      FractionSetFraction( Complex_Re  (mep ) , Complex_CIm(mep) ) ;
      FractionSetFlagNegative(Complex_Re(mep) , not wasRenegativ ) ;
      return  ;
    }
  }
  // re<>0 and im<>0 and abs(re)<>abs(im)
  Complex_SetLDComplex  ( mep , cpowl (
      CMPLXL ( FractionLDValueTemp ( Complex_Re (
            mep ) ) , FractionLDValueTemp ( Complex_Im (mep ) ) ) ,
      1.0L  / 3.0L  ) ) ;
  Complex_RoundRel ( mep , epsp ) ;
}

// re*(1-eps)..re*(1+eps) , im*(1-eps)..im*(1+eps)
void  Complex_RoundRel ( Complex * const mep  ,
  FractionPub const * const epsp ) {
  ThreadStackPointerType const oldstack_qp = Lisp2DThreadSP ( ) ;
  FractionPub * const reepsp = NewStackFraction ( ) ;
  FractionConstrCopy ( reepsp , epsp ) ;
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionMultConstFraction ( reepsp , FractionCToPub(& p-> rep2) ) ;
  FractionRoundStaticMove ( FractionToPub (& p-> rep2) , reepsp ) ;
  FractionSetFraction ( reepsp , epsp ) ;
  FractionMultConstFraction ( reepsp , FractionCToPub(& p-> imp2) ) ;
  FractionRoundStaticMove ( FractionToPub(& p-> imp2) , reepsp ) ;
  Lisp2D_StackFree  ( oldstack_qp ) ;
}

// returns public pointer to raw complex not constructed
Complex * Complex_FromStack ( void  ) {
  return  & ( ( ComplexPri  * ) Lisp2D_StackVar  ( StackType ( ComplexPri  ) ) ) ->  pub ;
}
# include "widestring/inline.h"
void  Complex_Destr ( Complex * const me  ) {
  ComplexPri_Destr  ( Complex_toPrivate  ( me  ) ) ;
}
void  Complex_ToStringForInterpret  ( Complex  const * const mep ,
  String  ( WideChar  ) * const sc  , unsigned  int const digc2 ) {
  ComplexToString  ( FractionToStringForInterpret ,  mep  , sc  , digc2 ) ;
}
void  Complex_ToStringForHuman  ( Complex  const * const mep ,
  String  ( WideChar  ) * const sc  , unsigned  int const digc2 ) {
  ComplexToString  ( FractionToStringForHuman , mep  , sc  , digc2 ) ;
}
void  Complex_TurnSign  ( Complex * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionTurnSign  ( FractionToPub(& p -> rep2 ) ) ;
  FractionTurnSign  ( FractionToPub(& p -> imp2 ) ) ;
}
// re-=[re] , im-=[im]
void  Complex_Mod ( Complex * const mep ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionMod ( FractionToPub(& p -> rep2 )) ;
  FractionMod  (FractionToPub(& p -> imp2 )) ;
}
bool  Complex_IsBadSizeValue  ( Complex const * const x , size_t * const resultp  ) {
  if  ( not ( FractionIs0 (Complex_CIm (x )) ) )
    return   true   ;
  return   FractionIsBadSizeValue ( Complex_CRe (  x ) , resultp  )  ;
}
// ?? how to control capacity for GC (only for PLOComplex)
void  Complex_SetPointer  ( Complex * const mep , void * const x ) {
  ComplexPri * const p = Complex_toPrivate  ( mep  ) ;
  FractionSetPointer  ( FractionToPub(& p -> rep2 ) , x ) ;
  FractionSet0 (FractionToPub(& p -> imp2));
}
