// Lisp2D - complex number Random
// ©2018-23 by Alex Glebe
# include "complexbim.h"
# include "stcl/classc.h"
# include "random/next.h"
# include "complex/inline.h"
# include "plocomplex/public.h"
# include <math.h> // sqrtl
# include "stcl/public.h"
# include "stcl/inline.h"

# define  Lisp2D_mypi  3.14159265358979323846264338327950288L
// alpha=[-pi..pi]
// r=[0..1]->sqrt  
TaskForThread bimcomplexrandomcomplex ( StackClusterPrivate  * const stclpp ) {
  long  double  ldr ;
  { PLOComplex const * const mep  = ParaLispObjectToPLOComplexConst ( StackClusterPrivate_ThisTop ( stclpp  ) ) ;
    { long  double const maxx = FractionLDValueConst ( Complex_CRe ( PLOComplexToComplexConst  ( mep ) ) ) ;
      long  double const maxy = FractionLDValueConst ( Complex_CIm ( PLOComplexToComplexConst  ( mep ) ) ) ;
      // sqrt((x^2+y^2)*[0..1])
      ldr = sqrtl ( ( ( maxx  * maxx  ) + (  maxy  * maxy ) ) * Lisp2DRandomNext2 ( ) ) ;
    }
  }
  long  double  resx  ;
  long  double  resy  ;
  sincosl ( Lisp2D_mypi * ( Lisp2DRandomNext2 ( )  * 2.0L  - 1.0L )  , & resx  , & resy ) ;
  PLOComplex  * const  res  = NewComplexBareExc (  )  ;
  FractionSetExactLDouble ( Complex_Re  ( PLOComplexToComplex ( res ) ) , resx  *  ldr ) ;
  FractionSetExactLDouble ( Complex_Im  ( PLOComplexToComplex ( res ) ) , resy  *  ldr ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & res ->  plo ) ;
  return  TaskForThread_Analyse ;
}

# include "stack/public.h"

TaskForThread bimcomplexrandom  ( StackClusterPrivate * const stclpp  ) {
// ? zero  
  ThreadStackPointerType const oldstack_frp = Lisp2DThreadSP ( ) ;
  FractionPub * const fr  = NewStackFraction  ( ) ;
  FractionInit ( fr ) ;
  FractionSetExactLDouble ( fr , Lisp2DRandomNext2 ( ) ) ;
  PLOComplex  * const  res2  = NewComplexBareExc ( )  ;
  Complex_Set ( PLOComplexToComplex ( res2  ) , PLOComplexToComplexConst  ( ParaLispObjectToPLOComplex  (
      StackClusterPrivate_ThisTop ( stclpp )  ) ) ) ;
  Complex_MulFraction ( PLOComplexToComplex ( res2  ) , fr ) ;
  Lisp2D_StackFree  ( oldstack_frp ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & res2  ->  plo ) ;
  return  TaskForThread_Analyse ;
}
  
# include "cons/inline.h"

// (medium random-normal sigma)
// No(x,y) = Mtarget+σtarget*Sin[2πy]*√(-2*Ln[x])
TaskForThread bimcomplexrandomnormal ( StackClusterPrivate * const stclpp  ) {
  ParaLispObject * const sigmap = CCAR  ( StackClusterPrivate_ArgTop  ( stclpp )  ) ;
  if  ( IsNotNumber ( stclpp , sigmap ) )
    return   TaskForThread_Analyse ;
  { PLOComplex  * res2 ;
    { ThreadStackPointerType const oldstack_frp = Lisp2DThreadSP ( ) ;
      FractionPub * const fr  = NewStackFraction  ( ) ;
      FractionInit ( fr ) ;
      FractionSetExactLDouble ( fr , sinl ( 2.0L  * Lisp2D_mypi  *
        Lisp2DRandomNext2 ( ) ) * sqrtl ( -2.0L * logl  ( Lisp2DRandomNext2 ( ) ) ) ) ;
      res2 = NewComplexBareExc ( )  ;
      Complex_SwapFraction  ( PLOComplexToComplex ( res2  ) , fr ) ; 
      Lisp2D_StackFree  ( oldstack_frp ) ;
    } // fr
    Complex_Mul ( PLOComplexToComplex ( res2  ) , 
      PLOComplexToComplexConst  ( ParaLispObjectToPLOComplexConst ( sigmap ) ) ) ;
    Complex_AddConst ( PLOComplexToComplex ( res2  ) , PLOComplexToComplexConst  (
      ParaLispObjectToPLOComplexConst ( StackClusterPrivate_ThisTop ( stclpp )  ) ) ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & res2  ->  plo ) ;
  } // res2
  return  TaskForThread_Analyse ;
}
