// complex sqrt [eps]
// (C)2023
# include "complexbim.h"
# include "plocomplex/public.h"
# include "stcl/public.h"
# include "stcl/inline.h"
# include "complex/inline.h"
# include "cons/inline.h"

// 31219-1.pdf
// !! sqrt(x) result will be (a,b,(c,d))=a+1/(b+1/(c+1/(d+1/(c+1/(d+...)))))
// !! to make function with <eps limit
// sqrt(5)=(2,(4))
// a1=2 sqrt(5)=2+1/a2
// a2=1/(sqrt(5)-2)=(sqrt(5)+2)/1=4+1/a3
// ...
// sqrt(32)=(5,(1,1,1,10))
// a1=5 sqrt(32)=5+1/a2
// a2=1/(sqrt(32)-5)=(sqrt(32)+5)/7=1+1/a3
// a3=...
// sqrt(n)=q0,(q1,q2,...,q2,q1,2*q0) - symmetry !
// real sqrt [eps]

// >113.75 sqrt 0.07
// 11 ошибка 0.33 => относительно 0.031

// >113.75 sqrt 0.06
// 10/2/3

/*
 !! to square line

  a^2 -> a ; (a^2)'=1/(2a) ; (a^2)''=-1/(4a^3)
  sqrt(c)=a+f'(a^2)*(c-a)+f''(a^2)*((c-a^2)^2)/2
  sqrt(c) = (3*a^4+6*a^2*c-c^2)/(8*a^3)
          = (3/8)*a+(3/4)*(c/a)-((c/a)^2)/8/a
*/
/*
?
sqrt[y]=x ; x0^2 = y0 ; k = (y - y0) / 2 / (x0^2)
x1 = x0 * (1 + k * (1 - k))
..
*/

struct  Lisp2D_s_complex_sqrt ;
typedef struct  Lisp2D_s_complex_sqrt s_complex_sqrt ;

# define  complex_sqrt_eps  Lisp2D_complex_sqrt_eps
static  TaskForThread complex_sqrt_eps  ( s_complex_sqrt * ) ;

struct  Lisp2D_s_complex_sqrt {
  StackClusterPrivate  * const stclpp ;
  PLOComplex * const xp ;
  Complex const * xpc ;
  bool  flageps ;
  ParaLispObject  const * epsp  ;
} ;

# include "stack/public.h"

//  complex sqrt [eps]
TaskForThread bimcomplexsqrt  ( StackClusterPrivate  * const stclpp ) {
  s_complex_sqrt  s = {
    . stclpp  = stclpp  ,
    . xp  = ParaLispObjectToPLOComplex  ( StackClusterPrivate_ThisTop ( stclpp ) ) ,
  } ;
  s . xpc = PLOComplexToComplexConst ( s . xp ) ;
  if ( ULongIs0 ( FractionCDenomp (  Complex_CIm ( s . xpc ) ) ) ) {
    // im = 0/0 or ±1/0
    if ( ULongIs0 ( FractionCDenomp ( Complex_CRe ( s . xpc ) ) ) ) {
      // re = 0/0 or ±1/0 im = 0/0 or ±1/0
      if ( ULongIs0 ( FractionCNump ( Complex_CIm ( s . xpc ) ) ) ) {
        // re = 0/0 or ±1/0 im = 0/0
        PLOComplex  * const  res  = NewComplexBareExc (  )  ;
        ThreadStackPointerType const oldstack_infp = Lisp2DThreadSP ( ) ;
        FractionPub * const zero = NewStackFraction ( ) ;
        FractionConstrZero ( zero ) ;
        if ( ULongIs0 ( FractionCNump ( Complex_CRe ( s . xpc ) ) ) )
          // re = 0/0 im = 0/0 => 0/0+0i
          FractionSwapFraction ( Complex_Im ( PLOComplexToComplex ( res ) ) , zero ) ;
        else {
          // re = ±1/0 im = 0/0
          FractionPub * const inf = NewStackFraction ( ) ;
          FractionInit ( inf ) ;
          FractionSetUIntUInt ( inf , 1U  , 0U  ) ;
          if ( FractionIsNegative ( Complex_CRe ( s . xpc ) ) ) {
            // re = -1/0 im = 0/0 => +0+∞i
            FractionSwapFraction ( Complex_Re ( PLOComplexToComplex ( res ) ) , zero ) ;
            FractionSwapFraction ( Complex_Im ( PLOComplexToComplex ( res ) ) , inf ) ;
          } else {
            // re = +1/0 im = 0/0 => +∞+0i
            FractionSwapFraction ( Complex_Im ( PLOComplexToComplex ( res ) ) , zero ) ;
            FractionSwapFraction ( Complex_Re ( PLOComplexToComplex ( res ) ) , inf ) ;
          }
        }
        Lisp2D_StackFree  ( oldstack_infp ) ;
        StackClusterPrivate_SetResult ( stclpp , & res ->  plo ) ;
        return  TaskForThread_Analyse ;
      }
      // re = 0/0 or ±1/0 im = ±1/0
      PLOComplex  * const  res= NewComplexBareExc (  );
      ThreadStackPointerType const oldstack_infp = Lisp2DThreadSP ( ) ;
      FractionPub * const inf = NewStackFraction ( ) ;
      FractionInit ( inf ) ;
      FractionSetUIntUInt ( inf , 1U  , 0U  ) ;
        if ( ULongIs0 ( FractionCNump ( Complex_CRe ( s . xpc ) ) ) ) {
          // re = 0/0 im = ±1/0
          if ( FractionIsNegative ( Complex_CIm ( s . xpc ) ) ) {
            // re = 0/0 im = -1/0 => +∞-∞i
            FractionPub * const minusinf = NewStackFraction ( ) ;
            FractionConstrFNUIntUInt ( minusinf ,true,1U,0U);
            FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( res ) ) , inf ) ;
            FractionSwapFraction ( Complex_Im (  PLOComplexToComplex ( res ) ) , minusinf ) ;
          } else { // re = 0/0 im = +1/0 => +∞+∞i
            FractionSetFraction  ( Complex_Re  ( PLOComplexToComplex ( res ) ) , inf ) ;
            FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( res ) ) , inf ) ;
          }
        } else {
          // re = ±1/0 im = ±1/0
          if ( FractionIsNegative ( Complex_CRe ( s . xpc ) ) ) {
            // re = -1/0 im = ±1/0
            if ( FractionIsNegative ( Complex_CIm ( s . xpc ) ) ) {
              // re = -1/0 im = -1/0  => +1-∞i
              FractionPub * const one = NewStackFraction ( ) ;
              FractionInit ( one ) ;
              FractionSetUInt ( one , 1U ) ;
              FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( res ) ) , one ) ;
              FractionPub * const minusinf = NewStackFraction ( ) ;
              FractionConstrFNUIntUInt ( minusinf ,true,1U,0U);
              FractionSwapFraction ( Complex_Im ( PLOComplexToComplex ( res ) ) , minusinf ) ;
            } else { // re = -1/0 im = +1/0 => +1+∞i
              FractionPub * const one = NewStackFraction ( ) ;
              FractionInit ( one ) ;
              FractionSetUInt ( one , 1U ) ;
              FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( res ) ) , one ) ;
              FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( res ) ) , inf ) ;
            }
          } else {
            // re = +1/0 im = ±1/0
            if ( FractionIsNegative ( Complex_CIm ( s . xpc ) ) ) {
              // re = +1/0 im = -1/0 => +∞-1i
              FractionPub * const minusone = NewStackFraction ( ) ;
              FractionSetInt ( minusone , -1 ) ;
              FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( res ) ) , inf ) ;
              FractionSwapFraction ( Complex_Im ( PLOComplexToComplex ( res ) ) , minusone ) ;
            } else { // re = +1/0 im = +1/0 => +∞+1i
              FractionSwapFraction  ( Complex_Re  ( PLOComplexToComplex ( res ) ) , inf ) ;
              FractionPub * const one = NewStackFraction ( ) ;
              FractionInit ( one ) ;
              FractionSetUInt ( one , 1U ) ;
              FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( res ) ) , one ) ;
            }
          }
        }
      Lisp2D_StackFree  ( oldstack_infp ) ;
      StackClusterPrivate_SetResult(stclpp ,&  res->plo);
      return  TaskForThread_Analyse;
    }
    // re = ±1 im = 0/0 or ±1/0
    PLOComplex  * const  res  = NewComplexBareExc (  )  ;
      if ( ULongIs0 ( FractionCNump ( Complex_CIm ( s . xpc ) ) ) ) {
        // re = ±1 im = 0/0 => 0/0+0/0i
      } else { // re = ±1 im = ±1/0
        ThreadStackPointerType const oldstack_infp = Lisp2DThreadSP ( ) ;
        FractionPub * const inf = NewStackFraction ( ) ;
        FractionInit ( inf ) ;
        FractionSetUIntUInt ( inf , 1U  , 0U  ) ;
        if  ( FractionIsNegative ( Complex_CIm ( s . xpc ) ) ) {
          // re = ±1 im = -1/0 => +∞-∞i
          FractionPub * const minusinf = NewStackFraction ( ) ;
          FractionConstrFNUIntUInt  ( minusinf ,true,1U,0U);
          FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( res ) ) , inf ) ;
          FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( res ) ) , minusinf  ) ;
        } else { // re = ±1 im = +1/0 => +∞+∞i
          FractionSetFraction  ( Complex_Re  ( PLOComplexToComplex ( res ) ) , inf ) ;
          FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( res ) ) , inf ) ;
        }
        Lisp2D_StackFree  ( oldstack_infp ) ;
      }
    StackClusterPrivate_SetResult(stclpp ,&  res->plo);
    return  TaskForThread_Analyse;
  } else { // im = ±1
    if  ( ULongIs0  ( FractionCDenomp ( Complex_CRe ( s . xpc ) ) ) ) {
      // re = 0/0 or ±1/0 im = ±1
      PLOComplex  * const  res= NewComplexBareExc (  );
      if  ( ULongIs0  ( FractionCNump ( Complex_CRe ( s . xpc ) ) ) ) {
        // re = 0/0 im = ±1 => 0/0+0/0i
      } else { // re = ±1/0 im = ±1
        ThreadStackPointerType const oldstack_all = Lisp2DThreadSP ( ) ;
        FractionPub * const zero = NewStackFraction ( ) ;
        FractionConstrZero ( zero ) ;
        FractionPub * const inf = NewStackFraction ( ) ;
        FractionInit ( inf ) ;
        FractionSetUIntUInt ( inf , 1U  , 0U  ) ;
        if  ( FractionIsNegative ( Complex_CRe ( s . xpc ) ) ) {
          // re = -1/0 im = ±1 => 0+∞i
          FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( res ) ) , zero ) ;
          FractionSwapFraction ( Complex_Im ( PLOComplexToComplex ( res ) ) , inf ) ;
        } else { // re = +1/0 im = ±1 => +∞+0i
          FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( res ) ) , inf ) ;
          FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( res ) ) , zero  ) ;
        }
        Lisp2D_StackFree  ( oldstack_all ) ;
      }
      StackClusterPrivate_SetResult(stclpp ,&  res->plo);
      return  TaskForThread_Analyse;
    }
  }
  // re = ±1  im = ±1
  s . epsp = CCAR  ( StackClusterPrivate_ArgTop  ( stclpp  ) ) ;
  if  ( Numberp ( s . epsp  ) ) {
    if  ( ULongIs0  ( FractionCDenomp  ( Complex_CRe ( PLOComplexToComplexConst  (
              ParaLispObjectToPLOComplexConst ( s . epsp  ) ) ) ) ) ) {
      PLOComplex  * const  res  = NewComplexBareExc ( ) ;
      Complex_Set0  ( PLOComplexToComplex ( res ) ) ;
      StackClusterPrivate_SetResult ( stclpp ,  & res ->  plo ) ;
      return  TaskForThread_Analyse ;
    }
    if  ( ULongIs0  (  FractionCNump  ( Complex_CRe  ( PLOComplexToComplexConst  (
         ParaLispObjectToPLOComplexConst ( s . epsp ) ) ) ) ) )
      s . flageps = false ;
    else
      s . flageps = true  ;
  } else
    s . flageps = false ;
  // eps is finite and not 0
  return  complex_sqrt_eps  ( & s ) ;
}

static  TaskForThread complex_sqrt_eps  ( s_complex_sqrt * const s ) {
  if  ( FractionIs0  ( Complex_CIm ( s -> xpc ) ) ) {
    // x is real
    if  ( ULongIs0  ( FractionCNump ( Complex_CRe ( s -> xpc ) ) ) ) {
      // 0
      StackClusterPrivate_SetResult ( s -> stclpp , &  s -> xp  ->  plo ) ;
      return  TaskForThread_Analyse ;
    }
    ThreadStackPointerType const oldstack_xrp = Lisp2DThreadSP ( ) ;
    FractionPub * const xr = NewStackFraction ( ) ;
    FractionConstrCopy ( xr , Complex_CRe ( s -> xpc ) ) ;
    if  ( FractionIsNegative  ( xr  ) ) {
      FractionAbs ( xr  ) ;
      if  ( s -> flageps )
        Fraction_Sqrt_Rel ( xr  ,
           Complex_CRe  ( PLOComplexToComplexConst (
             ParaLispObjectToPLOComplexConst  ( s -> epsp  ) ) ) ) ;
      else
        // ; (2 ^ -64) = 5.421e-20
        FractionSqrt  ( xr ) ;
// !? to calc your self around   <- 1.0 ->

      PLOComplex  * const  res= NewComplexBareExc (  )  ;
  { ThreadStackPointerType const oldstack_tmp = Lisp2DThreadSP ( ) ;
    FractionPub * const tmpp = NewStackFraction ( ) ;
    FractionConstrZero ( tmpp ) ;
    FractionSwapFraction  ( Complex_Re  ( PLOComplexToComplex ( res ) ) , tmpp ) ;
    Lisp2D_StackFree  ( oldstack_tmp ) ;
    FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( res ) ) , xr ) ;
    StackClusterPrivate_SetResult(s -> stclpp ,&  res->plo);
  }
    } else
      if  ( s -> flageps ) {
        PLOComplex  * const  pres = NewComplexBareExc ( ) ;
        Fraction_Sqrt_Rel ( xr , Complex_CRe ( PLOComplexToComplexConst  (
          ParaLispObjectToPLOComplexConst ( s -> epsp  ) ) ) ) ;
        Complex_SwapFraction  ( PLOComplexToComplex( pres), xr ) ;
        StackClusterPrivate_SetResult(s -> stclpp ,&  pres->plo);
      } else {
        // ; (2 ^ -64) = 5.421e-20
        FractionSqrt ( xr ) ;
// !? to calc your self around   <- 1.0 ->
        { PLOComplex  * const pres =  NewComplexBareExc (  )  ;
          Complex_SwapFraction  ( PLOComplexToComplex ( pres  ) , xr ) ;
          StackClusterPrivate_SetResult ( s -> stclpp ,  & pres  ->  plo ) ;
        }
      }
    Lisp2D_StackFree  ( oldstack_xrp ) ;
  } else {
// ! to calc complex with high quality - inet formulas
    // x is complex
/*
; a^2 -> a ; (a^2)'=1/(2a) ; (a^2)''=-1/(4a^3)
; f(c)=a+f'(a^2)*(c-a^2)+f''(a^2)*((c-a^2)^2)/2
; f(c) = (3*a^4+6*a^2*c-c^2)/(8*a^3)
;      = (3/8)*a+(3/4)*(c/a)-((c/a)^2)/8/a
(nil let ((c -0.001-1000i) a b a2 ca (eps 1e-100+1e-100i))
  (nil  if  ((c real)  < 0)
    ('a set 1i)
    ('a set 1)
  )
  (12 for*  i
    ('ca set (c / a))
    ('b set ((((a / 2) + ca) * 3/4) - ((ca * ca) / a 8)))
    ('b set (b round eps))
; a^2 -> a ; (a^2)'=1/(2a)
; f(c)=a+f'(a^2)*(c-a^2)
; f(c) = (a + c/a) / 2
    ('b2 set ((a + ca) / 2))
    ('b2 set (b2 round eps))
    (nil  when  ((((b * b) - c) abs) > (((b2 * b2) - c) abs))
      (cout write " ! ")
      (b swap b2)
    )
    (cout write "b = " b " ")
    (cout print b)
    ('a set b)
  )
)
*/

    Complex const * ceps ;
    ThreadStackPointerType const oldstack_ceps = Lisp2DThreadSP ( ) ;
    if  ( s -> flageps ) {
      ceps  = PLOComplexToComplexConst  ( ParaLispObjectToPLOComplexConst ( s -> epsp  ) ) ;
      if ( FractionIs0 ( Complex_CIm ( ceps ) ) ) {
        Complex * const ceps2 = Complex_FromStack ( ) ;
        Complex_Init  ( ceps2 ) ;
        FractionSetFraction ( Complex_Re  ( ceps2 ) , Complex_CRe ( ceps ) ) ;
        FractionSetFraction ( Complex_Im  ( ceps2 ) , Complex_CRe ( ceps ) ) ;
        ceps  = ceps2 ;
      }
    } else {
      FractionPub * const f64p = NewStackFraction ( ) ;
      FractionPub * const d64p = NewStackFraction ( ) ;
      FractionConstrUInt ( f64p , 2U )  ;
      FractionConstrUInt ( d64p , 64U )  ;
      FractionPower ( f64p , d64p ) ;
      FractionOverTurn  ( f64p  ) ;
      Complex * const ceps2 = Complex_FromStack ( ) ;
      Complex_Init  ( ceps2 ) ;
      FractionSetFraction ( Complex_Re  ( ceps2 ) , f64p  ) ;
      FractionMoveFraction  ( Complex_Im  ( ceps2 ) , f64p  ) ;
      ceps  = ceps2 ;
    }
    Complex * const a = Complex_FromStack ( ) ;
    Complex_Init  ( a ) ;
    if (  FractionLessUInt ( Complex_CRe ( s -> xpc ) , 0 ) ) {
      FractionSet0  ( Complex_Re  ( a ) ) ;
      FractionSet1  ( Complex_Im  ( a ) ) ;
    } else  {
      FractionSet1  ( Complex_Re  ( a ) ) ;
      FractionSet0  ( Complex_Im  ( a ) ) ;
    }
    Complex * const ca  = Complex_FromStack ( ) ;
    Complex_Init  ( ca  ) ;
    Complex * const b = Complex_FromStack ( ) ;
    Complex_Init  ( b ) ;
    Complex * const ca2 = Complex_FromStack ( ) ;
    Complex_Init  ( ca2 ) ;
    Complex * const b2  = Complex_FromStack ( ) ;
    Complex_Init  ( b2  ) ;
    Complex * const d = Complex_FromStack ( ) ;
    Complex_Init  ( d ) ;
    Complex * const d2  = Complex_FromStack ( ) ;
    Complex_Init  ( d2  ) ;
    FractionPub * const dd2r  = NewStackFraction ( ) ;
    FractionInit  ( dd2r  ) ;
    FractionPub * const dd2rp = NewStackFraction ( ) ;
    FractionInit  ( dd2rp ) ;
    FractionPub * const dd2i  = NewStackFraction ( ) ;
    FractionInit  ( dd2i  ) ;
    FractionPub * const dd2ip = NewStackFraction ( ) ;
    FractionInit  ( dd2ip ) ;
    do  {
      Complex_Set ( ca  , s -> xpc ) ;
      // ('ca set (c / a))
      Complex_Div ( ca  , a ) ;
      // ('b set ((((a / 2) + ca) * 3/4) - ((ca * ca) / a 8)))
      Complex_Set ( b , a ) ;
      Complex_DivUInt ( b , 2 ) ;
      Complex_AddConst ( b , ca  ) ;
      Complex_MulUInt ( b , 3 ) ;
      Complex_DivUInt ( b , 4 ) ;
      Complex_Set ( ca2 , ca  ) ;
      Complex_Mul ( ca2 , ca  ) ;
      Complex_Div ( ca2 , a ) ;
      Complex_DivUInt ( ca2 , 8 ) ;
      Complex_Subtract  ( b , ca2 ) ;
      // ('b set (b round eps))
      FractionRoundStaticMethod ( Complex_Re  ( b ) , Complex_CRe ( ceps  ) ) ;
      FractionRoundStaticMethod ( Complex_Im  ( b ) , Complex_CIm ( ceps  ) ) ;
      // ('b2 set ((a + ca) / 2))
      Complex_Set ( b2  , a ) ;
      Complex_AddMove ( b2  , ca  ) ;
      Complex_DivUInt ( b2  , 2 ) ;
      // ('b2 set (b2 round eps))
      FractionRoundStaticMethod ( Complex_Re  ( b2  ) , Complex_CRe ( ceps  ) ) ;
      FractionRoundStaticMethod ( Complex_Im  ( b2  ) , Complex_CIm ( ceps  ) ) ;
      // when  ((((b * b) - c) abs) > (((b2 * b2) - c) abs))
      // d := ((b * b) - c) ; d2 := ((b2 * b2) - c)
      // d abs2 > d2 abs2
      // d.r^2 + d.i^2 > d2.r^2 + d2.i^2
      // d.r^2 - d2.r^2 > d2.i^2 - d.i^2
      // (d.r-d2.r)*(d.r+d2.r) > (d2.i-d.i)*(d2.i+d.i)
      Complex_Set ( d , b ) ;
      Complex_Mul ( d , b ) ;
      Complex_Subtract  ( d , s -> xpc ) ;
      Complex_Set ( d2  , b2  ) ;
      Complex_Mul ( d2  , b2  ) ;
      Complex_Subtract  ( d2  , s -> xpc ) ;
      FractionSetFraction ( dd2r  , Complex_CRe ( d ) ) ;
      FractionSubtractConst ( dd2r  , Complex_CRe ( d2  ) ) ;
      FractionSetFraction ( dd2rp , Complex_CRe ( d ) ) ;
      FractionAddConstFraction  ( dd2rp , Complex_CRe ( d2  ) ) ;
      FractionMultMoveFraction  ( dd2r  , dd2rp ) ;
      FractionSetFraction ( dd2i  , Complex_CIm ( d2  ) ) ;
      FractionSubtractConst ( dd2i  , Complex_CIm ( d ) ) ;
      FractionSetFraction ( dd2ip , Complex_CIm ( d ) ) ;
      FractionAddConstFraction  ( dd2ip , Complex_CIm ( d2  ) ) ;
      FractionMultMoveFraction  ( dd2i  , dd2ip ) ;
      BoolType  const more  = FractionMore  ( dd2r  , dd2i  ) ;
      if  ( more  ==  Lisp2DBoolTrue  )
        Complex_Move  ( b , b2  ) ;
      if  ( Complex_Eq ( a , b ) )
        break ;
      Complex_Move  ( a , b ) ;
    } while ( true  ) ;
    PLOComplex  * const res2  = NewComplexBareExc ( ) ;
    Complex_Move  ( PLOComplexToComplex ( res2  ) , a ) ;
    Lisp2D_StackFree  ( oldstack_ceps ) ;
    StackClusterPrivate_SetResult ( s -> stclpp  , & res2  ->  plo ) ;
  }
  return   TaskForThread_Analyse ;

}
