// Lisp2D - PLO complex number
// ©2010-23 by Alex Glebe

# include "errorstra.h"
# include "plocomplex/public.h"

void  PLOComplex_staticDeclareForGC(ParaLispObject * const me) {
  if(not  me -> flagGCPassed){
    me -> flagGCPassed = true ;
    { jmp_buf jb2 ;
      jmp_buf * const oldjb = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn2 = setjmp  ( jb2  ) ;
      if  ( jbn2  ) {
        static  wchar_t  const  compldeclare [ ] =
          L"PLOComplex_staticDeclareForGC:GlobalListPopNotLock("
          L"& GlobalListOfComplexGC , & me -> base );"  ;  
        Lisp2DJBglobal  = oldjb ;
        ErrorStra((PArray_wchar_c){(wchar_t const (*)[])&compldeclare});
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & compldeclare ;
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
      }
      GlobalListPopNotLock(& GlobalListOfComplexGC , & me -> base );
      Lisp2DJBglobal  = oldjb ;
    } // jb2
    GlobalListPushNotLock(&GlobalListOfComplex, & me -> base );
  }
}

# include "plo/public.h"

ParaLispObject  * PLOComplex_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassparentscomplex ; 
}

ParaLispObject  * PLOComplex_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassdefcomplex ; 
}

# include "symbolc.h"

ParaLispObject  * PLOComplex_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return &  ploscomplex . plo ;
}

bool  PLOComplex_staticisBADCopyPrivate  ( ParaLispObject const * const me , ParaLispObject  * * const resultp ) {
  ( * resultp )  = & NewComplexBareExc()->plo;
  Complex_Set ( PLOComplexToComplex( ParaLispObjectToPLOComplex(*resultp)),
    PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst( me)));
  return  false;
}

# include "complex/inline.h"

void  PLOComplex_Destr  ( PLOComplex * const me ) {
  Complex_Destr ( PLOComplexToComplex ( me  ) ) ;
  ParaLispObject_staticDestr  ( & me  ->  plo ) ;
}

// +0/0+0/0i
void  PLOComplex_InitUnDef ( PLOComplex * const me ) {
  Complex_Init  ( PLOComplexToComplex( me ) ) ;
  ParaLispObject_InitVTP  ( & me -> plo , (VirtualTypePointer){ . addr = { . p = me } , . type = TypeGlobalListMemberPLOComplex});
}

# include "sharedc.h"
# include "shared/inline.h"
# include "string/public.h"
# include "inerrors.h"
# include "passage/public.h"
# include "nilc.h"

bool  IsNotRealLockGC ( StackClusterPrivate  * const stclp , PLOComplex * const xp ) {
  if(Complex_isReal(PLOComplexToComplexConst(xp))) 
    return  false;
  { static  wchar_t const errs  [ ] = L" is not real number, it is imagine complex";
    if(stclp ){            
      PassagePassIn ( passageGCCPub ) ;
      { jmp_buf jb2 ;
        jmp_buf * const oldjb = Lisp2DJBglobal  ;
        Lisp2DJBglobal  = & jb2 ;
        int const volatile jbn2 = setjmp  ( jb2  ) ;
        if  ( jbn2 ) {
          Lisp2DJBglobal  = oldjb ;
          StackClusterPrivate_ExceptionSet(stclp ,&plonil.plo); 
        } else  {
          ParaLispObject  * list [ ] = { & xp->plo,& PLOString_NewCopy  ( errs  ) ->  plo, 0 } ;
          StackClusterPrivate_ExceptionSet(stclp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          Lisp2DJBglobal  = oldjb ;
        } 
      } // jb2
      PassagePassOut ( passageGCCPub ) ;    
    } else   
      inErrorS(        PLOStreamPushPLO(& Lisp2Dferr ,&xp->plo);
                       PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) {    (wchar_t const(*)[])&errs });)
  } // errs
  return  true  ;
}  

// returns  pointer to new number 0/0+0/0i or nullptr
// new constructor() from new memory or from garbage
// pushed to globalList of numbers  

PLOComplex  * NewComplexBareExc ( void ) {
  PLOComplex  * const resp = NewComplexBareNotInGlobalListExc  ( ) ;
  if ( resp )
    GlobalListPush  ( & GlobalListOfComplex , & resp  ->  plo . base ) ;
  return  resp ; 
}

# include "inner-history/inline.h"
# include "define.h"

// returns  pointer to new number 0/0+0/0i or nullptr
// new constructor() from new memory or from garbage
// not in global list, to do : manual add GlobalListOfComplex.Push2(x);
// or :    delete x; (drawing to garbage or deletes)
PLOComplex  * NewComplexBareNotInGlobalListExc  ( void  ) {
# ifdef  LISP2D_INNERHISTORY
  { jmp_buf jb2 ;
    jmp_buf * const oldjb = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jb2 ;
    int const volatile jbn2 = setjmp  ( jb2  ) ;
    if  ( jbn2 ) {
      Lisp2DJBglobal  = oldjb ;
      static  wchar_t const newcomplexbare0 [] =
        L"NewComplexBareNotInGlobalListExc :"
        L"NewComplexOnlyMemoryNotInGlobalListExc:bad_alloc"   ;
      Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , newcomplexbare0 ) ;
      longjmp ( * Lisp2DJBglobal  , jbn2  ) ; 
    }
    PLOComplex  * const res = NewComplexOnlyMemoryNotInGlobalListExc  ( ) ;
    Lisp2DJBglobal  = oldjb ;
    // from raw memory, call constructor
    PLOComplex_InitUnDef  ( res ) ;
    return  res ; 
  } // jb2
# else // LISP2D_INNERHISTORY
  PLOComplex  * const res = NewComplexOnlyMemoryNotInGlobalListExc  ( ) ;
  // from raw memory, call constructor
  PLOComplex_InitUnDef  ( res ) ;
  return  res ;
# endif // LISP2D_INNERHISTORY
}

// returns  pointer to memory for number (from garbage or new)
// to do manual call constructor() : new (pointer) PLOComplex(..arguments..)
// manual add GlobalListOfComplex.Push2(x);
// or :    delete x; (calls destructor , drawing to garbage or deletes)  

PLOComplex  * NewComplexOnlyMemoryNotInGlobalListExc  ( void ) {
  GlobalListMember * numberNew = 0 ;
  if  ( not ( GlobalListisOKTryPopFirst ( & GlobalListOfComplexGarbage  , & numberNew ) ) ) {
    PLOComplexPri * const cpp = aligned_alloc ( _Alignof ( PLOComplexPri ) , sizeof  ( PLOComplexPri ) ) ;
    if ( not cpp ) {
# ifdef  LISP2D_INNERHISTORY
      static  wchar_t const  newcomplexmemfree [ ] = L"NewComplexOnlyMemoryNotInGlobalListExc : nomem"   ;
      Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , newcomplexmemfree ) ;
# endif
      Lisp2DExcOutOfMemoryGlobal . value = L"NewComplexOnlyMemoryNotInGlobalListExc:aligned_alloc=null" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
# ifdef LISP2D_DEBUG
    ++  Lisp2D_statMallocPLOComplexPri  ;
# endif
    PLOComplex * const numberNew2 = & cpp -> pub ;
    return   numberNew2 ;
  }
  return  PLOComplex_FromMember ( numberNew ) ;
}

// returns  pointer to memory for number (from garbage or new)
// to do manual call constructor() : new (pointer) PLOComplex(..arguments..)  
// already pushed in global list
// new complex, may be from garbage. 
// NOT inserted to GlobalListOfComplex.
// PLOComplex * && res(new (std::nothrow, & stcl) PLOComplex);
// if(res){ ok }else{ no mem }
PLOComplex * PLOComplex_newraw ( void ) {
  PLOComplex * const resp = NewComplexBareNotInGlobalListExc  ( ) ;
# ifdef  LISP2D_INNERHISTORY  
  static  wchar_t  const  newcomplexsize_t [ ] = 
    L"PLOComplex_newraw:NewComplexBareNotInGlobalListExc-ok." ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory ,
    newcomplexsize_t ) ;  
# endif
  return  resp ;
}

# include "statistics.h"

// call destructor and, pushing memory to garbage  
// OR deletes  
void  PLOComplex_delete ( PLOComplex * const me ) {
# ifdef LISP2D_BIMSTAT
  struct  timespec  timeBegin ;
  clock_gettime ( CLOCK_REALTIME  , & timeBegin ) ;
# endif
  PLOComplex_Destr  ( me  ) ;
  // ~destructor worked
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t  const  operatordelete [ ] = L"void  PLOComplex_delete:Start." ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , operatordelete ) ;
# endif  
  // pushing raw memory without constructor, only links: left right
  if  ( GlobalListisOKTryPush ( & GlobalListOfComplexGarbage  , &  me -> plo . base ) ) {
    // memory passed
# ifdef  LISP2D_INNERHISTORY
    static  wchar_t  const  operatordelete3 [ ] = L"void  PLOComplex_delete:clearing" ;
    Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , operatordelete3  ) ;
# endif    
  } else {
# ifdef  LISP2D_INNERHISTORY
    static  wchar_t  const  operatordelete4 [ ] = L"void  PLOComplex_delete:delete" ;
    Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , operatordelete4  ) ;
# endif 
    // remove memory
    free ( me ) ;
  }
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t  const  operatordelete2  [ ] = L"void  PLOComplex_delete:Quit." ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , operatordelete2  ) ;
# endif  
# ifdef LISP2D_BIMSTAT
  struct  timespec  timeEnd ;
  clock_gettime ( CLOCK_REALTIME  , & timeEnd ) ;
  IncreaseStat5  ( &  timeBegin , & timeEnd , & statComplexDelete ) ;
# endif
}

# include "stack/public.h"

// sign will set by flagNegative
void  LoadComplexFromString ( t_LoadComplexFromString const ar ) {
  ThreadStackPointerType const oldstack_rf0p = Lisp2DThreadSP ( ) ;
  FractionPub * const rf0 = NewStackFraction ( ) ;
  FractionConstrZero  (rf0);
  FractionPub * const rfd = NewStackFraction ( ) ;
  FractionConstrZero  (rfd);
  FractionPub * const rfe = NewStackFraction ( ) ;
  FractionConstrZero  (rfe);
  * ar . flagErrp = true ;

  struct  s_FractionLoadChastDelitFromString  const arg = {
    . wsp = ar . wsp ,
    . jp =  ar . jp  ,
    . rf0 = rf0 ,
    . lerr =  ar . flagErrp  ,
    . i = ( * ar . partsizep ) + 1 ,
    . flagImp = ar . flagImp ,
  } ;
  bool  flagdelit = FractionLoadChastDelitFromString  ( & arg ) ;

  if  ( not  (  * ar . flagErrp  ) ) {

    if  ( not flagdelit ) {
      FractionSetFlagNegative (rf0,ar . flagNegative);
      if  ( *ar . flagImp  )
        FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( ar . complxp ) ) , rf0 ) ;
      else
        FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( ar . complxp ) ) , rf0 ) ;
      goto Exit ;
    }

    ( * ar . flagErrp  ) = true  ;

    struct  s_FractionLoadChastDelitFromString  const arg2 = {
        . wsp = ar . wsp ,
        . jp =  ar . jp  ,
        . rf0 = rfd ,
        . lerr =  ar . flagErrp  ,
        . i = ( * ar . partsizep ) + 1 ,
        . flagImp = ar . flagImp ,
    } ;
    flagdelit = FractionLoadChastDelitFromString  ( & arg2 ) ;

      // flagIm code not writen yet
      if(not  (*ar . flagErrp)){

        if  ( not flagdelit ) {
          FractionDivFractionMove  ( rf0 , rfd ) ;
          FractionSetFlagNegative (rf0,ar . flagNegative);
          if(*ar . flagImp)
            FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( ar . complxp ) ) , rf0 ) ;
          else
            FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( ar . complxp ) ) , rf0 ) ;
          goto Exit ;
        }

        // now loading 1/2/3
        ( * ar . flagErrp  ) = true  ;

          struct  s_FractionLoadChastDelitFromString  const arg3 = {
            . wsp = ar . wsp ,
            . jp =  ar . jp  ,
            . rf0 = rfe ,
            . lerr =  ar . flagErrp  ,
            . i = ( * ar . partsizep ) + 1 ,
            . flagImp = ar . flagImp ,
          } ;
          flagdelit = FractionLoadChastDelitFromString  ( & arg3 ) ;

          // flagIm code not writen yet
          if(not  (*ar . flagErrp)){ // >=0
            if(flagdelit){
              (*ar . flagErrp) = true;
              goto Exit ;              
            } 
            // not  flagdelit              
            FractionDivFractionMove  ( rfd , rfe ) ;
            FractionAddMoveFraction(rf0,rfd);
            FractionSetFlagNegative (rf0,ar . flagNegative);
            if(*ar . flagImp)
              FractionSwapFraction ( Complex_Im  ( PLOComplexToComplex ( ar . complxp ) ) , rf0 ) ;
            else
              FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex ( ar . complxp ) ) , rf0 ) ;
            goto Exit ;
          }
      }
  } // not lerr
Exit :    
  Lisp2D_StackFree  ( oldstack_rf0p ) ;
} // LoadComplexFromString

ParaLispObject  * PLOComplex_staticPushAsNew(ParaLispObject  * const me ) {
  GlobalListPush(&GlobalListOfComplex, & me -> base );
  return  me ; 
}

void  PLOComplex_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool _1 __attribute__ ((unused)) ,
  Vector(StackPLOP)  * _2 __attribute__ ((unused)) )  {
  Complex_CoutToInterpret ( PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(me)),  s ) ; 
}

void  PLOComplex_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool _1 __attribute__ ((unused)) ,
  Vector(StackPLOP)  * _2 __attribute__ ((unused)) ) {
  Complex_CoutToHuman ( PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(me)) , s );  
}

PLOComplex * PLOComplex_FromMember ( GlobalListMember * const m ) {
  if(m){
    if(GlobalListMemberVirt(m).type == TypeGlobalListMemberParaLispObject) {
      ParaLispObject * const plo = GlobalListMemberVirt ( m ) . addr  . p ;
      if  ( plo ->  vaddr . type == TypeGlobalListMemberPLOComplex  ) 
        return  plo ->  vaddr . addr . p ;
      fputws(L"PLOComplex_FromMember ( GlobalListMember * m ):"
        L"plo ->  vaddr . type <> TypeGlobalListMemberPLOComplex\n" ,stderr);
      return 0 ; 
    }
    if(GlobalListMemberVirt(m).type == TypeGlobalListMemberPLOComplex)
      return  GlobalListMemberVirt  ( m ) . addr . p ;
    fputws(L"PLOComplex_FromMember ( GlobalListMember * m ):"
      L"GlobalListMemberVirt(m).type=",stderr);
    fwprintf(stderr,L"%d\n",GlobalListMemberVirt(m).type); 
  }
  return  0 ; 
}

void PLOComplex_staticDestr  ( ParaLispObject * const me ) {
  PLOComplex_Destr  ( ParaLispObjectToPLOComplex  ( me  ) ) ;
}
