// Complex Pow
// (C)2023
# include <complex.h> // cpowl

// clang
# ifndef CMPLXL
# define CMPLXL(x, y) ((long double complex)((long double)(x) + \
                      _Complex_I * (long double)(y)))
# endif

# include "complexbim.h"
# include "plocomplex/public.h"
# include "stcl/public.h"
# include "stcl/inline.h"
# include "cons/inline.h"
# include "complex/inline.h"

# define  bool_xor  Lisp2D_bool_xor5HwR
inline  bool  bool_xor  ( bool const a , bool const b ) {
  return  b ?
    ( not a ) :
    a ;
}

typedef struct  s_pow_arg t_pow_arg ;

enum { complex_pow_continue , complex_pow_indet , complex_pow_out } ;

struct  s_pow_arg {
  PLOComplex * const xp ;
  Complex const * yp ;
  StackClusterPrivate  * const stclpp ;
  int taskgo ;
} ;

# define  complex_pow_y_real  Lisp2D_complex_pow_y_real
static  void  complex_pow_y_real ( t_pow_arg * ) ;

# include "define.h"

//  complex ^ complex
// !! (x^3 == -1) => x=-1 OR x=(-1)^(1/3) OR x=-(-1)^(2/3)=(-1)^(-1/3)
// !! (-1)^(-1/3) = std::pow -> 0.5-0.86i
// ?? how to get answer -1 ?
// I. if Re<0 , Im=0   =>   result = - (-x)^y
// II. if Re=0,Im>0    =>   result = -(im)^y
// III. if Re=0,Im<0    =>   result = (-im)^y
// ?? new function with M answers  for x^(n/m) ?
// std::pow = x = Abs*exp(i*Arg) -> Abs Arg , Abs Arg+(2pi)/3 , Abs Arg-(2pi)/3

TaskForThread bimcomplexpow ( StackClusterPrivate  * const stclpp ) {
  t_pow_arg arg  = {
    . xp  = ParaLispObjectToPLOComplex(
      StackClusterPrivate_ThisTop(stclpp )) ,
    . stclpp  = stclpp  ,
  } ;
  ParaLispObject  const * const  ypp = CCAR(StackClusterPrivate_ArgTop(stclpp ));
  if(not  Numberp(ypp)) {
    StackClusterPrivate_SetResult(stclpp ,&arg.xp->plo);
    return   TaskForThread_Analyse ;
  }
  arg . yp = PLOComplexToComplexConst ( ParaLispObjectToPLOComplexConst ( ypp ) ) ;
  // to check for REAL result
  if(FractionIs0( Complex_CIm(PLOComplexToComplexConst(arg.xp)))) {
    if  ( FractionIs0( Complex_CIm ( arg . yp ) ) ) {
      PLOComplex  * const  pres = NewComplexBareExc  (   )  ;
      StackClusterPrivate_SetResult (stclpp ,&  pres ->plo ) ;
      { FractionPub  const * const xrp  = Complex_CRe ( PLOComplexToComplexConst( arg.xp ) ) ;
        Complex_SetFraction ( PLOComplexToComplex( pres ), xrp ) ;
        if  (  FractionIsNegative ( xrp ) ) {
          ULongPub const * const yrd =FractionCDenomp( Complex_CRe ( arg.yp ) ) ;
          if  ( not ( ULongIs0 ( yrd ) ) )
            if  ( ULongEvenP ( yrd ) ) {
              if  ( not ULongIs0 (  FractionCDenomp( xrp ) ) ) {
                // (-4/9)^(1/2) -> 0+2/3i
                FractionSetFlagNegative ( Complex_Re  ( PLOComplexToComplex( pres) ) ,false );
                FractionPower ( Complex_Re  (PLOComplexToComplex ( pres) ), Complex_CRe ( arg.yp ) ) ;
                goto  Swap ;
              }
              if  ( not ( FractionIsNegative ( Complex_CRe ( arg.yp )) ) )
                if  ( not ( ULongIs0 (  FractionCNump( xrp ))) ) {
                  // (-1/0)^(+x/(2n)) -> 0+1/0i
                  FractionSetFlagNegative ( Complex_Re  ( PLOComplexToComplex( pres) ),false );
                  goto  Swap ;
                }
            }
        }
      } // xr
      FractionPower ( Complex_Re  (PLOComplexToComplex(pres) ),  Complex_CRe (arg.yp ) ) ;
      goto  Out ;
Swap  :
      FractionSwapFraction ( Complex_Re  ( PLOComplexToComplex (
        pres  ) ) , Complex_Im  ( PLOComplexToComplex ( pres ) ) ) ;
      goto  Out ;
    }
    if(FractionIs1( Complex_CRe(PLOComplexToComplexConst( arg.xp)))){
      // 1^y -> 1
      StackClusterPrivate_SetResult(stclpp , &arg.xp->plo);
      return   TaskForThread_Analyse ;
    }
  }
  // x is complex
  if(FractionIs0( Complex_CIm( arg . yp ))) {
    // y is real
    arg . taskgo = complex_pow_continue ;
    complex_pow_y_real ( & arg ) ;
    switch ( arg . taskgo ) {
    case  complex_pow_continue  :
      break ;
    case  complex_pow_indet  :
      goto  Indet ;
    case  complex_pow_out :
      goto  Out ;
    default : ;
# ifdef LISP2D_DEBUG
      Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & L"bimcomplexpow:unknown arg . taskgo" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
# endif
    }
  } // if <PLOComplex const & >(y).CIm().Is0()
  // be care: result will be complex
  { PLOComplex  * const  res= NewComplexBareExc (  );
    Complex_SetLDComplex  ( PLOComplexToComplex ( res ) , cpowl ( CMPLXL  (
          FractionLDValueConst ( Complex_CRe( PLOComplexToComplexConst  (
                arg.xp  ) ) ) , FractionLDValueConst ( Complex_CIm (
              PLOComplexToComplexConst  ( arg.xp  ) ) ) ) , CMPLXL  (
          FractionLDValueConst ( Complex_CRe ( arg.yp ) ) ,
          FractionLDValueConst ( Complex_CIm ( arg.yp ) ) ) ) ) ;
    StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  }
Out :
  return  TaskForThread_Analyse ;
Indet :
  { PLOComplex  * const  pres = NewComplexBareExc  (   ) ;
    StackClusterPrivate_SetResult (stclpp ,&  pres ->plo ) ;
    FractionSet0( Complex_Im  (PLOComplexToComplex(pres) ));
  }
  goto  Out ;
}

static  void  complex_pow_y_real ( t_pow_arg * const arg ) {
  if  ( FractionIs0( Complex_CRe (PLOComplexToComplexConst( arg->xp) ) ) ) {
      // x is pure complex
      // (+-a/bi)^(+-1/d) = ((+-a/b)^(+-1/d))*(i^(+-1/d))
      FractionPub  const * const yre = Complex_CRe ( arg->yp ) ;
      Vector(StackUnsInt) const * const yredevp  = ULongCValue ( FractionCDenomp (  yre ) ) ;
      if  ( VectorEmpty ( StackUnsInt ) (  yredevp ) ) {
        // (±*ⅈ)^(0/0) -> 0/0
        if  ( ULongIs0 ( FractionCNump(yre ))) {
          arg -> taskgo = complex_pow_indet ;
          return ;
        }
        FractionPub  const * const xim = Complex_CIm ( PLOComplexToComplexConst(arg->xp) ) ;
        ULongPub  const * const ximch =FractionCNump( xim )  ;
        ULongPub  const * const ximde =FractionCDenomp(xim )  ;
        // (0)^(±1/0) -> 0/0
        // (0/0)^(±1/0) -> 0/0
        if  ( ULongIs0 ( ximch )) {
          arg -> taskgo = complex_pow_indet ;
          return ;
        }
        // (±ⅈ)^(±1/0) -> 0/0
        if  ( ULongIs1 ( ximde ) and ULongIs1 ( ximch ) ) {
          arg -> taskgo = complex_pow_indet ;
          return ;
        }
        PLOComplex  * const  pres = NewComplexBareExc  (   )  ;
        StackClusterPrivate_SetResult (arg->stclpp ,&  pres ->plo ) ;
        FractionSet0(Complex_Im  (PLOComplexToComplex(pres)));
        // (±0.5ⅈ)^(+1/0) -> 0
        // (±2ⅈ)^(-1/0) -> 0
        if  ( bool_xor  ( ULongLess ( ximch , ximde ) , FractionIsNegative (  yre ) ) )
          FractionSet0( Complex_Re  ( PLOComplexToComplex ( pres ) ) ) ;
        // (±0.5ⅈ)^(-1/0) -> 1/0
        // (±2ⅈ)^(+1/0) -> 1/0
        else
          FractionSetInf  ( Complex_Re  ( PLOComplexToComplex(pres) ) ) ;
        arg -> taskgo = complex_pow_out ;
        return  ;
      }
      unsigned int defr = Lisp2DStackUnsIntFrontCopy(  yredevp )  ;
      if  ( defr bitand 1U  ) {
        Vector(StackUnsInt) const * const yrechvp  = ULongCValue ( FractionCNump (  yre ) ) ;
        if  ( not ( VectorEmpty ( StackUnsInt ) (  yrechvp  ) ) ) {
          PLOComplex  * const  pres =NewComplexBareExc  (   ) ;
          StackClusterPrivate_SetResult (arg->stclpp ,&  pres ->plo ) ;
          unsigned int const  chfr = Lisp2DStackUnsIntFrontCopy( yrechvp )  ;
          if  ( ( chfr bitand 3U )  ==  0 ) {
            // (±ⅈ)^(±4k/(2n+1)) -> +1
            FractionSetFraction  ( Complex_Re  ( PLOComplexToComplex ( pres  ) ) ,  Complex_CIm (
              PLOComplexToComplexConst  ( arg->xp  ) ) ) ;
            FractionPower ( Complex_Re  ( PLOComplexToComplex(pres) ), yre ) ;
            FractionSetFlagNegative ( Complex_Re  ( PLOComplexToComplex( pres) ) ,false );
            FractionSet0( Complex_Im  (PLOComplexToComplex( pres) ));
          } else  if  ( ( chfr bitand 1U )  ==  0 ) {
            // (±ⅈ)^(±2k/(2n+1)) -> -1
            FractionSetFraction ( Complex_Re  ( PLOComplexToComplex ( pres  ) ) , Complex_CIm (
              PLOComplexToComplexConst  ( arg->xp  ) ) ) ;
            FractionPower ( Complex_Re  ( PLOComplexToComplex(pres) ), yre ) ;
            FractionSetFlagNegative ( Complex_Re  ( PLOComplexToComplex( pres) ) ,true );
            FractionSet0( Complex_Im  ( PLOComplexToComplex(pres) ));
          } else {
            // (+ⅈ)^(+(4k+1)/(4n+1)) -> +ⅈ  (+ⅈ)^(-(4k+1)/(4n+1)) -> -ⅈ
            // (+ⅈ)^(+(4k+3)/(4n+3)) -> +ⅈ  (+ⅈ)^(-(4k+3)/(4n+3)) -> -ⅈ
            // (+ⅈ)^(+(4k+1)/(4n+3)) -> -ⅈ  (+ⅈ)^(-(4k+1)/(4n+3)) -> +ⅈ
            // (+ⅈ)^(+(4k+3)/(4n+1)) -> -ⅈ  (+ⅈ)^(-(4k+3)/(4n+1)) -> +ⅈ
            FractionSetFraction ( Complex_Im  ( PLOComplexToComplex ( pres  ) ) ,
              Complex_CIm ( PLOComplexToComplexConst  ( arg->xp  ) ) ) ;
            FractionPower ( Complex_Im  ( PLOComplexToComplex(pres) ) , yre ) ;
            if  ( bool_xor  ( bool_xor  ( ( chfr  bitand  3U  ) ==  1 , ( defr  bitand  3U  ) ==  1 ) ,
              FractionIsNegative (   yre ) ) )
              FractionTurnSign  ( Complex_Im  ( PLOComplexToComplex( pres) ) ) ;
            FractionSet0( Complex_Re  ( PLOComplexToComplex(pres) ));
          }
          arg -> taskgo = complex_pow_out ;
          return  ;
        }
      }
    } // if  x . CRe ( ) . Is0
    if  ( ULongIs1  ( FractionCDenomp  ( Complex_CRe  ( arg->yp ) ) ) ) {
      // y is real finite integer
      ULongPub const * const ul=FractionCNump( Complex_CRe(arg->yp) );
      if( ULongIs0(ul)){
        PLOComplex  * const  res= NewComplexBareExc (  );
        // 0^0=0/0
        if(not Complex_Is0(PLOComplexToComplexConst(arg->xp)))
          Complex_SetUInt ( PLOComplexToComplex( res),1U);
        StackClusterPrivate_SetResult(arg->stclpp ,&  res->plo);
        arg -> taskgo = complex_pow_out ;
        return  ;
      }
      if(FractionIs1( Complex_CRe(arg->yp))){
        StackClusterPrivate_SetResult(arg->stclpp ,& arg->xp->plo);
        arg -> taskgo = complex_pow_out ;
        return  ;
      }
      PLOComplex  * const  res2  = NewComplexBareExc ( );
      Complex_Set ( PLOComplexToComplex( res2), PLOComplexToComplexConst( arg->xp));
      // res:=x^ul

  /*
 !! BUG ??
   ( 1/0+0/1i ) ^ 2 => 1/0
 !! TO DO ??
   ( 1/0+0/1i ) ^ 2 => 1/0+0/0i
  */

      Complex_Power ( PLOComplexToComplex( res2),  ul);
      if(FractionLessUInt( Complex_CRe(arg->yp),  0U))
        // y<0
        // res:=1/res
        Complex_OneDivide ( PLOComplexToComplex( res2));
      StackClusterPrivate_SetResult(arg->stclpp ,&  res2->plo);
      arg -> taskgo = complex_pow_out ;
      return  ;
    } // if <PLOComplex const & >(y).CRe().Delit().Is1()
}
