// Condition
// (C)2019-23 Lisp2D

# include <errno.h>
# include <iso646.h>
# include "condit.h"
# include "main.h"

void  ConditionConstructor  ( Condition * const cp , pthread_condattr_t const * const  attr ) {
  if  ( cp ->  s == Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionInit:double construction"  ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
  int const e = pthread_cond_init  ( & cp -> c , attr ) ;
  switch  ( e ) {
  case  0 :
    cp -> s = Lisp2DObjectStateConstructed ;
    return  ;
  case  EAGAIN  :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionInit:The system lacked the necessary resources (other "
      L"than memory) to initialize another condition variable."  ;
    break ;
  case  ENOMEM  :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionInit:Insufficient memory exists to initialize the "
      L"condition variable." ;
    break ;
  default :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionInit:?"  ;
  }
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
}
  
# include <stdio.h>
  
void  ConditionDestructor  ( Condition * const cp ) {
  if  ( cp ->  s not_eq Lisp2DObjectStateConstructed ) {
    fputws  ( L"ConditionDestroy:is not live" , stderr  ) ; 
    return  ;
  }
  if  ( pthread_cond_destroy  ( & cp -> c ) )
    fputws  ( L"ConditionDestroy:?" , stderr ) ;
  cp -> s =  Lisp2DObjectStateDestructed ;
}
  
void  ConditionSignal ( Condition * const cp ) {
  if  ( cp ->  s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionSignal:is not live"  ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  int const e = pthread_cond_signal  ( & cp -> c ) ;
  switch  ( e ) {
  case 0 :
    return ;
  case EINVAL :  
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionSignal:does not refer to an initialized condition variable." ;
    break ;
  default :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionSignal:?"  ; 
  }
  cp -> s =  Lisp2DObjectStateRaw ;
  longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
}

# include "mutex2/struct.h"

void  ConditionWait ( Condition * const cp , Mutex2 * const m ) {
  if  ( cp ->  s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionWait:is not live" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  if  ( m ->  s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionWait:mutex is not live"  ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  int const e = pthread_cond_wait ( & cp -> c , & m -> m ) ;
  switch  ( e ) {
  case 0 : 
    return ;
  case EINVAL :  
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionWait:does not refer to an initialized condition variable, "
      L"or detects that the  value  specified  by  the mutex  argument does "
      L"not refer to an initialized mutex object"  ;
    break ;
  case  ENOTRECOVERABLE :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionWait:The state protected by the mutex is not recoverable." ;
    break ;
  case  EOWNERDEAD :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionWait:The  mutex  is a robust mutex and the process containing "
      L"the previous owning thread terminated while holding the mutex lock. "
      L"The mutex lock shall be acquired by the calling thread and it is up to "
      L"the new  owner  to  make  the state consistent."  ;
    break ;
  case  EPERM :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionWait:The  mutex  type is PTHREAD_MUTEX_ERRORCHECK or the "
      L"mutex is a robust mutex, and the current thread does not own the mutex."
      ;
    break ;
  default :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ConditionWait:?"  ;
  }
  cp -> s =  Lisp2DObjectStateRaw ;
  longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
}
