// Lisp2D - Cons
// ©2006-23 by Alex Glebe
# include "define.h"
# include "cons/public.h"

# ifdef  LISP2D_CONSTRSTAT

_Atomic ( size_t  ) PLOCons_constrstat = 0 ;

size_t  PLOCons_constrstatValue ( void  ) {
  return  PLOCons_constrstat  ;
}

void  PLOCons_constrstatInc ( void  ) {
  ++  PLOCons_constrstat  ;
}

# endif // LISP2D_CONSTRSTAT

# include "cons/struct.h"
# include "plo/public.h"

void  PLOCons_Destr ( PLOCons * const me  ) {
  ParaLispObject_staticDestr  ( & me  ->  plo ) ;
}

# include "cons/private.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "cons/inline.h"
# include "plostream/public.h"

# define  plo_constcast Lisp2D_plo_constcast
inline  ParaLispObject *  plo_constcast ( ParaLispObject const * p ) ;

void  PLOCons_Cout  ( t_PLOCons_Cout  const * const arg ) {
/*PLOCons const * const mep , FuncCoutToInterpretHuman  funccout  ,
  PLOStream  const * const fop , bool const flag2pprinted , Vector  ( StackPLOP )  * const lookingp  ) {*/
  bool  flagpprinted =  arg -> flag2pprinted ;
  if  ( Printed3Points  ( & arg -> mep -> plo , arg -> fop , & flagpprinted  , arg -> lookingp ) )
    return  ;
  PLOCons const * me  = arg -> mep  ;
  PLOStreamPushWChar  ( arg -> fop , L'('  ) ;
  size_t  const lookingsize = VectorSizeCopy  ( StackPLOP ) ( arg -> lookingp ) ;
  Vector_PushByValue ( StackPLOP ) ( arg -> lookingp , plo_constcast ( & arg -> mep -> plo ) ) ;
Next2:
  arg -> funccout  ( ClassCons_First ( &  me  ->  cc ) , arg -> fop , & flagpprinted   , arg -> lookingp ) ;
  { ParaLispObject const * const merestp =  ClassCons_Rest  ( &me->cc  ) ;
    if(CConsp(merestp))   {         
      PLOStreamPushWChar  ( arg -> fop  , L' '  ) ;
      me  = ParaLispObjectToPLOConsConst(merestp);
      if  ( Printed3Points  ( & me  ->  plo , arg -> fop  , & flagpprinted  , arg -> lookingp  ) )
        goto End ;
      Vector_PushByValue ( StackPLOP )  ( arg -> lookingp , plo_constcast ( & me -> plo ) ) ;
      goto  Next2;
    }
    if(( merestp)  not_eq  (& plonil.plo)){
      PLOStreamPushWideCharArray  ( arg -> fop , ( PArray_wchar_c ) { (wchar_t const(*)[])&L" . " });
      arg -> funccout  ( merestp  , arg -> fop  , & flagpprinted   , arg -> lookingp ) ;
    }
  }
End :
  Vector_PopToSize  ( StackPLOP ) ( arg -> lookingp , lookingsize ) ;
  PLOStreamPushWChar  ( arg -> fop  , L')'  ) ;
}
  
void  PLOCons_InitC ( PLOCons * const cop , ParaLispObject  * const ap , ParaLispObject  * const bp ) {
  ParaLispObject_InitVTP  ( & cop ->  plo , ( VirtualTypePointer ) { . addr = { . p = cop } , . type = TypeGlobalListMemberPLOCons } ) ;
  ClassCons_InitPair  ( & cop ->  cc  , ap  , bp  ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOCons_constrstatInc ( ) ;
# endif
}

bool  PLOCons_staticisBADCopyPrivate  ( ParaLispObject const * const  me ,
  ParaLispObject  * * const resultp  ) {
  ClassCons const * const c = & ParaLispObjectToPLOConsConst  ( me  ) -> cc ;
  PLOCons * const res = NewCons ( ClassCons_First ( c ) ,
     ClassCons_Rest  ( c ) ) ;
  ( * resultp ) = ( & res ->  plo ) ;
  return  false ;
} 

// returns pointer (PLOCons *) or exception
PLOCons * NewCons ( ParaLispObject  * const ap  , ParaLispObject  * const bp  ) {
  GlobalListMember * consNew ;
  PLOCons * consNew2 ;
  if  ( not  GlobalListisOKTryPopFirst  ( & GlobalListOfConsPairsGarbage  , & consNew ) ) {

// ? to optimize speed take memory for 2 members !
// ! and how to free memory ?

    consNew2 = aligned_alloc ( _Alignof ( PLOCons ) , sizeof ( PLOCons ) ) ;
    if ( not consNew2 ) {
      Lisp2DExcOutOfMemoryGlobal . value = L"NewCons:aligned_alloc(PLOCons)" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; 
    }
# ifdef LISP2D_DEBUG
    ++  Lisp2D_statMallocPLOCons  ;
# endif
    PLOCons_InitC ( consNew2  , ap  , bp  ) ;
    consNew = & consNew2 -> plo.base ; 
  } else
    ClassCons_SetFirstRest  ( & PLOCons_FromMember  ( consNew )->cc , ap , bp ) ;
  GlobalListPush ( & GlobalListOfConsPairs , consNew ) ;
  return  PLOCons_FromMember  ( consNew ) ;
}

PLOCons * PLOCons_FromMember ( GlobalListMember * const m ) {
  if  ( m ) {
    if  ( GlobalListMemberVirt(m)  . type == TypeGlobalListMemberParaLispObject ) {
      ParaLispObject * const  p = GlobalListMemberVirt  ( m ) . addr . p ;
      if  ( p  ->  vaddr . type == TypeGlobalListMemberPLOCons )
        return p  ->  vaddr . addr . p ;
    }
  }
  return  0 ;
}      

# include "globallistp/struct.h"

GlobalList  GlobalListOfConsPairs ;
GlobalList  GlobalListOfConsPairsGarbage  ;
GlobalList  GlobalListOfConsPairsGC ;

void  PLOCons_staticGCDeclareContent ( ParaLispObject * const me ) {
  PLOCons * const c = ParaLispObjectToPLOCons( me) ;
  ParaLispObject_virtualDeclareForGC  ( ClassCons_First ( &c->cc ) ) ;
  ParaLispObject_virtualDeclareForGC  ( ClassCons_Rest  ( &c->cc ) ) ; 
} 

# include "symbolc.h"

ParaLispObject  * PLOCons_staticTypeOf ( ParaLispObject const * const _ __attribute__ ((unused)) ) {
  return  & ploscons  . plo ;
}

void PLOCons_staticDestr  ( ParaLispObject * const me) {
  PLOCons_Destr ( ParaLispObjectToPLOCons ( me  ) ) ;
}

ParaLispObject  * PLOCons_staticClassDef  (
  ParaLispObject const * _ __attribute__ ((unused)) )  {
  return  ploclassdefcons ;
}

ParaLispObject  * PLOCons_staticClassParents  (
  ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassparentscons ;
}

void  PLOCons_staticDeclareForGC(ParaLispObject * const me ) {
  if(not  me -> flagGCPassed) {
    me -> flagGCPassed =true;
    GlobalListPopNotLock  ( & GlobalListOfConsPairsGC  , & me -> base )  ; 
    GlobalListPushNotLock ( & GlobalListOfConsPairs , & me -> base ) ;
  }
}

void  PLOCons_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  , Vector  ( StackPLOP ) * const h ) {
  t_PLOCons_Cout  const arg = {
    . mep = ParaLispObjectToPLOConsConst  ( me  ) ,
    . funccout  = ParaLispObject_CoutToInterpret  ,
    . fop = s ,
    . flag2pprinted = flag2pprinted ,
    . lookingp  = h ,
  }  ;
  PLOCons_Cout  ( & arg ) ;
}

void  PLOCons_staticCoutToHuman ( ParaLispObject  const * const me  ,
  PLOStream const * const s , bool const flag2pprinted , Vector  ( StackPLOP ) * const h ) {
  t_PLOCons_Cout  const arg = {
    . mep = ParaLispObjectToPLOConsConst  ( me  ) ,
    . funccout  = ParaLispObject_CoutToHuman  ,
    . fop = s ,
    . flag2pprinted = flag2pprinted ,
    . lookingp  = h ,
  }  ;
  PLOCons_Cout  ( & arg ) ;
}

void  PLOCons_SetPair(PLOCons  * const cp,ParaLispObject  * const ap , ParaLispObject  * const bp ) {
  ClassCons_SetFirst(&cp->cc,ap);
  ClassCons_SetRest(&cp->cc,bp);
}

PLOCons *  PLOCons_LastFast ( PLOCons * const lp ) {
  ParaLispObject  * leftrestslow ;
  ParaLispObject  * leftrest2 ;
  PLOCons * res ;
  bool  tocheck = false ;
  res = lp ;
  leftrestslow  =  ClassCons_Rest ( & lp -> cc )  ;
  leftrest2 = leftrestslow ;
  while ( CConsp ( leftrest2 ) ) {
    res = ParaLispObjectToPLOCons ( leftrest2 ) ;
    leftrest2  = ClassCons_Rest ( & res -> cc ) ;
    if  ( tocheck ) {
      leftrestslow  = ClassCons_Rest  ( & ParaLispObjectToPLOCons ( leftrestslow  ) -> cc ) ;
      if ( leftrestslow == leftrest2 )
        // list cycled
        break ;
      tocheck = false ;
    } else
      tocheck = true  ;
  }
  // list ends
  return  res ;
}

PLOCons * ListCopyNotProtected ( PLOCons  const  * const  mep , PLOCons * * const  reslastp  ) {
  ParaLispObject  *   melast  = ClassCons_Rest ( & mep -> cc ) ;
  PLOCons * const res = NewCons ( ClassCons_First ( & mep -> cc  ) , melast  ) ;
  * reslastp = res ;
  bool  tocheck = false ;
  ParaLispObject  * melastslow = melast ;
  while ( CConsp ( melast ) ) {
    ParaLispObject  * const melast2p = ClassCons_Rest  ( & ParaLispObjectToPLOCons ( melast  )->cc ) ;
    PLOCons * const res2 = NewCons ( ClassCons_First ( & ParaLispObjectToPLOCons ( melast  ) -> cc ) , melast2p ) ;
    ClassCons_SetRest ( & ( * reslastp ) -> cc , & res2 -> plo ) ;
    * reslastp = res2 ;
    melast  = melast2p ;
    if  ( tocheck ) {
      melastslow  = ClassCons_Rest  ( & ParaLispObjectToPLOCons ( melastslow ) -> cc ) ;
      if ( melastslow == melast )
        // list cycled
        break ;
      tocheck = false ;
    } else
      tocheck = true  ;
  }
  return  res ;
}

ParaLispObject  * ListCopyProtected ( PLOCons const * const mep ) {
  ParaLispObject  *   melast  = ClassCons_Rest  ( & mep -> cc  )  ;
  PLOCons * const res = NewCons ( ClassCons_First ( & mep -> cc  ) , melast  ) ;
  PLOCons * reslast = res ;
  while(CConsp( melast)){
    // cycle check
    { PLOCons const * jme = mep ;
      PLOCons * jres = res ;
      for( ;(&jres->plo)  not_eq  melast;
          jres = ParaLispObjectToPLOCons  ( ClassCons_Rest  ( &jres->cc  ) ) ) {
        if((&jme->plo)  ==  melast){
          // cycle found
          ClassCons_SetRest(&reslast->cc,& jres->plo);
          return  & res->plo;
        }
        jme = ParaLispObjectToPLOCons ( ClassCons_Rest  ( &jme->cc ) ) ;
      }
    }
    { ParaLispObject  * const melast2p = ClassCons_Rest  ( &ParaLispObjectToPLOCons (
        melast  )->cc ) ;
      PLOCons * const res2 = NewCons  ( ClassCons_First (
        &ParaLispObjectToPLOCons ( melast  )->cc ) , melast2p ) ; 
      ClassCons_SetRest(&reslast->cc,& res2->plo);
      reslast = res2;
      melast  = melast2p;
    }
  }
  return  & res->plo;
}

// O(n)
ParaLispObject  * ListCopyNotProtected0 ( PLOCons  const * const mep){
  ParaLispObject  *   melast  =   ClassCons_Rest ( & mep ->cc )  ;
  PLOCons * const res = NewCons ( ClassCons_First ( & mep->cc  ) , melast  ) ;
  PLOCons * reslast = res ;
  bool  tocheck = false ;
  ParaLispObject  * melastslow = melast ;
  while(CConsp( melast)){
    ParaLispObject  * const melast2p = ClassCons_Rest  ( &ParaLispObjectToPLOCons (
      melast  )->cc ) ;
    PLOCons * const res2 = NewCons  ( ClassCons_First (
      &ParaLispObjectToPLOCons ( melast  )->cc ) ,
      melast2p ) ; 
    ClassCons_SetRest(&reslast->cc,& res2->plo);
    reslast = res2;
    melast  = melast2p ;
    if  ( tocheck ) {
      melastslow  = ClassCons_Rest  ( & ParaLispObjectToPLOCons ( melastslow ) -> cc ) ;
      if ( melastslow == melast )
        // list cycled
        break ;
      tocheck = false ;
    } else
      tocheck = true  ;
  } // while(CConsp( melast))
  return  & res->plo;
}

# include "newobject.h"

bool  IsNotCons(StackClusterPrivate  * const stclp ,ParaLispObject * const  xp){  
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp ,
    . typetype  = TypeGlobalListMemberPLOCons ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
  //return  UniversalIsNotType(stclp ,xp, TypeGlobalListMemberPLOCons);
}

bool  CConsp ( ParaLispObject const * const xp  ) {
  return  ParaLispObject_Type ( xp  ) ==  TypeGlobalListMemberPLOCons ;
}

ParaLispObject  * CCAR  ( ParaLispObject  const * const xp ) {
  return  ( CConsp ( xp  ) ?
    ClassCons_First ( &ParaLispObjectToPLOConsConst  ( xp  )->cc ) : 
    & plonil  . plo ) ;
}

ParaLispObject  * CCDR  ( ParaLispObject  const * const xp ) {
  return  ( CConsp (  xp  ) ?
    ClassCons_Rest  ( &ParaLispObjectToPLOConsConst  ( xp  )->cc ) : 
    & plonil  . plo ) ;
}

# include "stcl/public.h"
# include "stcl/inline.h"

// cons first
TaskForThread Lisp2Dbimconsfirst2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult ( stclpp  , ClassCons_First ( 
    &ParaLispObjectToPLOCons ( StackClusterPrivate_ThisTop ( stclpp  ) )->cc ) ) ;
  return  TaskForThread_Analyse ;
}

// cons second
TaskForThread Lisp2Dbimconssecond2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult ( stclpp  , CCAR  ( ClassCons_Rest  (
    &ParaLispObjectToPLOCons ( StackClusterPrivate_ThisTop ( stclpp  ) )->cc ) ) ) ;
  return  TaskForThread_Analyse ;
}

// cons third
TaskForThread Lisp2Dbimconsthird2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult ( stclpp  , CCAR  ( CCDR  (
    ClassCons_Rest  ( &ParaLispObjectToPLOCons (    StackClusterPrivate_ThisTop  (
      stclpp  ) )->cc ) ) ) ) ;
  return  TaskForThread_Analyse ;
}

// cons rest
TaskForThread Lisp2Dbimconsrest2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult ( stclpp  , ClassCons_Rest  (
    &ParaLispObjectToPLOCons ( StackClusterPrivate_ThisTop ( stclpp  ) )->cc ) ) ;
  return  TaskForThread_Analyse ;
}

// cons last
TaskForThread Lisp2Dbimconslast2(StackClusterPrivate  * const stclpp) {
  PLOCons * const last = PLOCons_LastFast ( ParaLispObjectToPLOCons (
    StackClusterPrivate_ThisTop ( stclpp  ) ) ) ;
  StackClusterPrivate_SetResult(stclpp,&  last->plo);
  return  TaskForThread_Analyse ;
}

//  list reverse
TaskForThread Lisp2Dbimnilreverse2(StackClusterPrivate  * const stclpp){
  ParaLispObject  *   res = & plonil.plo ;
  { ParaLispObject  *   j =  StackClusterPrivate_ThisTop(stclpp) ;
    ParaLispObject  *   jslow = j ;
    bool  tocheck = false ;
    while( CConsp( j ) ) {
      res = & NewCons ( ClassCons_First (
        &ParaLispObjectToPLOCons ( j )->cc ) , res ) ->  plo ;
      j  = ClassCons_Rest (& ParaLispObjectToPLOCons ( j )->cc ) ;
      if  ( tocheck ) {
        jslow = ClassCons_Rest (& ParaLispObjectToPLOCons ( jslow )->cc ) ;
        if ( j == jslow )
          break ;
        tocheck = false ;
      } else
        tocheck = true  ;
    }
  }
  StackClusterPrivate_SetResult(stclpp ,  res);
  return  TaskForThread_Analyse ;
}

# include "boolean.h"

// O(n)
TaskForThread Lisp2Dbimnilcycledp2  ( StackClusterPrivate  * const stclpp ) {
  ParaLispObject * const  me =   StackClusterPrivate_ThisTop ( stclpp  ) ;
  if  ( not  CConsp ( me  ) )
    goto Normal ;
  { PLOCons const * last = ParaLispObjectToPLOConsConst ( me ) ;
    PLOCons const * last2 = ParaLispObjectToPLOConsConst ( me ) ;
  do  {
    { ParaLispObject const  * const lastrest = ClassCons_CRest  ( &last2->cc ) ;
      if ( not CConsp ( lastrest ) )
        break ;
      last2 = ParaLispObjectToPLOConsConst ( lastrest ) ;
    }
    if ( last2 == last )
      goto  Cycled  ;
    last = ParaLispObjectToPLOConsConst ( ClassCons_CRest ( & last -> cc ) ) ; 
    { ParaLispObject const  * const lastrest = ClassCons_CRest  ( & last2 -> cc )  ;
      if ( not CConsp ( lastrest ) )
        break ;
      last2 = ParaLispObjectToPLOConsConst ( lastrest ) ;
    }
    if ( last2 == last )
      goto  Cycled  ;
  } while ( true ) ;
  }
Normal :
  StackClusterPrivate_SetResult(stclpp ,&plofalse.plo);
  goto Ret ;
Cycled :
  StackClusterPrivate_SetResult(stclpp , me);
Ret :
  return  TaskForThread_Analyse ;
}

// cons setfirst
TaskForThread Lisp2Dbimconssetfirst2(StackClusterPrivate  * const stclpp){
  PLOCons * const m = ParaLispObjectToPLOCons (
    StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ClassCons_SetFirst(&m ->cc,CCAR(StackClusterPrivate_ArgTop(stclpp)));
  StackClusterPrivate_SetResult(stclpp ,&m->plo);
  return  TaskForThread_Analyse ;
}

// cons setrest
TaskForThread Lisp2Dbimconssetrest2(StackClusterPrivate * const stclpp){
  PLOCons * const m = ParaLispObjectToPLOCons (
    StackClusterPrivate_ThisTop ( stclpp));
  ClassCons_SetRest(&m->cc,CCAR(StackClusterPrivate_ArgTop(stclpp)));
  StackClusterPrivate_SetResult(stclpp ,&m->plo);
  return  TaskForThread_Analyse ;
}

// cons += [{cons}]
TaskForThread Lisp2Dbimconsnconc2(StackClusterPrivate * const stclpp){
  PLOCons * const mep = ParaLispObjectToPLOCons(StackClusterPrivate_ThisTop(
    stclpp));
  PLOCons * left = mep ;
  ParaLispObject  * j;
  ParaLispObject  * jslow ;
  { ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop(stclpp);
    if(not  CConsp(arglistp)){
      StackClusterPrivate_SetResult(stclpp,&  left->plo);
      return  TaskForThread_Analyse;
    }
    j =   arglistp ;
    jslow = j ;
  }
  bool  tocheck = false ;
  while ( CConsp ( j ) ) {
    left  =   PLOCons_LastFast(  left);
    // Last(left)
    * ClassCons_PRest ( &left->cc  ) = ClassCons_First ( &ParaLispObjectToPLOCons ( j )->cc ) ;
    j  = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc ) ; 
    if ( tocheck ) {
      jslow = ClassCons_Rest ( & ParaLispObjectToPLOCons ( jslow ) -> cc ) ;
      if ( j == jslow )
        break ;
      tocheck = false ;
    } else
      tocheck = true ;
  }
  StackClusterPrivate_SetResult ( stclpp ,  & mep ->  plo ) ;
  return  TaskForThread_Analyse ;
}

// list + [{list}]
TaskForThread Lisp2Dbimnilplus2(StackClusterPrivate * const stclpp){
  ParaLispObject  * j;
  ParaLispObject  * jslow;
  ParaLispObject  * reslast;
  ParaLispObject  * res;
  { ParaLispObject  * const mep = StackClusterPrivate_ThisTop(stclpp);
    if(CConsp(mep)){
      PLOCons *  reslast2;
      res = &  ListCopyNotProtected (
        ParaLispObjectToPLOCons(mep),&reslast2)->plo;
      reslast = (&reslast2->plo);
    } else{
      res = (&  plonil.plo);
      reslast = (&  plonil.plo);
    }
  }
  { ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp);
    if(not  CConsp(arglistp)){
      StackClusterPrivate_SetResult(stclpp,  res);
      return  TaskForThread_Analyse;
    }
    j =   arglistp;
  }
  jslow = j ;
  bool  tocheck = false ;
  while (  CConsp (  j ) ) {
    ParaLispObject  * const xp =
      ClassCons_First ( &ParaLispObjectToPLOCons ( j )->cc ) ;
    if(CConsp(xp)){
      PLOCons *  xlast;
      PLOCons * const  xcopyp  = ListCopyNotProtected  (
        ParaLispObjectToPLOCons (  xp ),&xlast);
      if(reslast  not_eq  (&  plonil.plo))
        ClassCons_SetRest(&ParaLispObjectToPLOCons(reslast)->cc,&xcopyp->plo);  
      else   
        res = &  xcopyp->plo;
      reslast = &xlast->plo;
    }
    j  = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc )  ;
    if ( tocheck ) {
      jslow = ClassCons_Rest ( & ParaLispObjectToPLOCons ( jslow ) -> cc ) ;
      if ( j == jslow )
        break ;
      tocheck = false ;
    } else
      tocheck = true ;
  }
  StackClusterPrivate_SetResult(stclpp,  res);
  return  TaskForThread_Analyse;
}

// cons copy-tree
TaskForThread Lisp2Dbimconscopytree2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp, CopyTreem1 (
    ParaLispObjectToPLOCons(StackClusterPrivate_ThisTop(
    stclpp))));
  return  TaskForThread_Analyse;
}

// O(n)
// cons copy-list
TaskForThread Lisp2Dbimconscopylist2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp,ListCopyNotProtected0(
    ParaLispObjectToPLOCons(
    StackClusterPrivate_ThisTop(stclpp))));
  return  TaskForThread_Analyse;
}

// O(n²)
// cons copy-list-protected
TaskForThread Lisp2Dbimconscopylistprotected2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp,ListCopyProtected(
    ParaLispObjectToPLOCons(
    StackClusterPrivate_ThisTop(stclpp))));
  return  TaskForThread_Analyse;
}

//  cons setf cons
TaskForThread Lisp2Dbimconssetf2(StackClusterPrivate  * const stclpp){
  PLOCons * const mep = ParaLispObjectToPLOCons(StackClusterPrivate_ThisTop(
    stclpp));
  ParaLispObject  * const argp = CCAR(StackClusterPrivate_ArgTop(stclpp));
  if(not  IsNotCons( stclpp,argp)) {
    PLOCons_Set(mep , ParaLispObjectToPLOCons(argp));
    StackClusterPrivate_SetResult(stclpp, &mep->plo);
  }
  return  TaskForThread_Analyse;
}

//  cons swap cons
TaskForThread Lisp2Dbimconsswap2(StackClusterPrivate  * const stclpp){
  PLOCons * const mep = ParaLispObjectToPLOCons(StackClusterPrivate_ThisTop( stclpp));
  ParaLispObject  * const argp = CCAR(StackClusterPrivate_ArgTop(stclpp));
  if(not  IsNotCons( stclpp,argp)){
    if(IsConst(  stclpp,argp))   
      return  TaskForThread_Analyse;
    ClassCons_Swap  ( & mep->cc  , &ParaLispObjectToPLOCons (  argp )->cc ) ;
    StackClusterPrivate_SetResult(stclpp,&mep->plo);
  }
  return  TaskForThread_Analyse;
}

# include "env/public.h"

// cons for* symbol [{body}]
// symbol have const link
TaskForThread Lisp2Dbimconsforcons2(StackClusterPrivate * const stclpp){
  ParaLispObject  * varC;
  ParaLispObject  * varA;
  ParaLispObject  * arglist;
  ParaLispObject  * indexsymbol;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )
    goto  Next ;
  arglist = StackClusterPrivate_ArgTop(stclpp);
  indexsymbol = CCAR(  arglist);
  if(IsNotSymbol(  stclpp,  indexsymbol))  
    return  TaskForThread_Analyse;
  StackClusterPrivate_SetResult(stclpp,&plonil.plo);
  arglist = CCDR( arglist);
  if(not  CConsp( arglist))
    // have no body, protection from cycled list
    return  TaskForThread_Analyse;
  StackClusterPrivate_EnvNewLevel2(stclpp);
  varA  =   StackClusterPrivate_ThisTop(stclpp);
  
  do{
    if(IsNotEnvironmentLayer(  stclpp,CCAR(StackClusterPrivate_Env(stclpp)))){
      StackClusterPrivate_EnvCDR(stclpp);
      return  TaskForThread_Analyse;
    }
    if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
      CCAR ( StackClusterPrivate_Env(stclpp) ) ) -> envlay , indexsymbol  ,
      ClassCons_First ( &ParaLispObjectToPLOCons ( varA  )->cc ) , false ) ) {
      HasConstLink(  stclpp,  indexsymbol);
      return  TaskForThread_Analyse;
    }
    StackClusterPrivate_Push(stclpp, varA);
    StackClusterPrivate_Push(stclpp, indexsymbol);
    StackClusterPrivate_Push(stclpp, arglist);
    varC  = arglist;
    for ( ; CConsp  ( varC  ) ; varC  = ClassCons_Rest  ( &ParaLispObjectToPLOCons (
      varC  )->cc ) ) {
      StackClusterPrivate_Push(stclpp, varC);
      StackClusterPrivate_Push  ( stclpp  ,
        ClassCons_First ( &ParaLispObjectToPLOCons ( varC  ) -> cc ) ) ;
      StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
      return  TaskForThread_Analyse;
Next:
      varC=StackClusterPrivate_PopValue(stclpp );
      if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
        //arglist
        StackClusterPrivate_Popn(stclpp );
        //indexsymbol
        StackClusterPrivate_Popn(stclpp );
        // varA
        StackClusterPrivate_Popn(stclpp );
        StackClusterPrivate_EnvCDR(stclpp);  
        return  TaskForThread_Analyse;
      }
    }
    arglist=StackClusterPrivate_PopValue(stclpp );
    indexsymbol=StackClusterPrivate_PopValue(stclpp );
    varA  = ClassCons_Rest  ( & ParaLispObjectToPLOCons ( 
      StackClusterPrivate_PopValue  ( stclpp  ) ) -> cc ) ;
  } while(CConsp(  varA));
  //old environment restoring
  StackClusterPrivate_EnvCDR(stclpp);
  return  TaskForThread_Analyse;
}

# include "bimc.h"

// list for i [{body}]
// i has const link
TaskForThread Lisp2Dbimnilfor2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * s2;
  ParaLispObject  * i2;
  ParaLispObject  * jslow ;
  ParaLispObject  * body;
  EvalAddrCont2 const  c = StackClusterPrivate_ClearflagCont (
    stclpp ) ;
  if  ( c ) {
    switch(c){
    case  1:
      StackClusterPrivate_SetResult(stclpp,&plonil.plo);
      return  TaskForThread_Analyse;
    default:;
    }
  }
  i2  = StackClusterPrivate_ThisTop(stclpp);
  if(not  CConsp( i2)){
    StackClusterPrivate_SetResult(stclpp,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  jslow = i2 ;
  body  = StackClusterPrivate_ArgTop(stclpp);
  s2  = CCAR( body);
  if(IsNotSymbol(  stclpp,  s2)) 
    return  TaskForThread_Analyse;
  body  = CCDR( body);
  if(CConsp(CCDR(  body))){
    PLOCons *  body3;
    body3 =   NewCons(  &  plobimnilprogn->base.plo,  body); 
    body  = &  NewCons(&plonil.plo,& body3->plo)->plo;
  } else
    body  = CCAR( body);
  bool  tocheck = false ;
  do {
    StackClusterPrivate * const dopstclp = StackClusterPrivate_NewStCl4(
      stclpp);
    StackClusterPrivate_EnvNewLevel2(dopstclp);
    if(IsNotEnvironmentLayer(  stclpp,
      CCAR(StackClusterPrivate_Env(dopstclp))))
      return  TaskForThread_Analyse;
    if  ( EnvironmentLayerisBADPush (
        & ParaLispObjectToPLOEnvironmentLayer (
          CCAR ( StackClusterPrivate_Env(dopstclp) ) )
        -> envlay ,
        s2  , ClassCons_First ( & ParaLispObjectToPLOCons ( i2  ) -> cc ) ,
        true  ) ) {
      HasConstLink(  stclpp,  s2);
      return  TaskForThread_Analyse;
    }
    StackClusterPrivate_CreateTask(dopstclp ,  body);

    i2  = ClassCons_Rest  ( & ParaLispObjectToPLOCons ( i2  ) -> cc ) ;

    if ( tocheck ) {
      jslow = ClassCons_Rest ( & ParaLispObjectToPLOCons ( jslow ) -> cc ) ;
      if ( i2 == jslow )
        break ;
      tocheck = false ;
    } else
      tocheck = true ;

  } while (  CConsp (  i2 ) ) ;
  StackClusterPrivate_setTaskForManager ( stclpp , 
    TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ; 
  return  TaskForThread_Analyse;
}

/*('cons  defmethod cons(&rest  x)
  (nil  if  x
    (nil  if  (x  rest)
      (nil  parallel
        (this setfirst  (x  first))
        (this setrest (x  second)))
      (this setf  (x  first))))
  this)*/
TaskForThread Lisp2Dbimconscons2(StackClusterPrivate  * const stclpp){
  PLOCons * const  mep = ParaLispObjectToPLOCons(StackClusterPrivate_ThisTop(
    stclpp));
  ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop(stclpp);
  if(CConsp(arglistp)){
    ParaLispObject  * const restp  = ClassCons_Rest  ( & ParaLispObjectToPLOCons (
      arglistp ) -> cc ) ;
    if(CConsp(restp)){
      * ClassCons_PRest ( & mep->cc  ) = ClassCons_First ( &ParaLispObjectToPLOCons (  arglistp )->cc ) ;
      * ClassCons_PRest ( & mep->cc  ) = ClassCons_First ( &ParaLispObjectToPLOCons (  restp  )->cc ) ;
    } else{
      ParaLispObject  * const xp = ClassCons_First ( &ParaLispObjectToPLOCons ( arglistp )->cc )  ;
      if(IsNotCons(  stclpp,xp))       
        return  TaskForThread_Analyse;
      PLOCons_Set(mep , ParaLispObjectToPLOCons(xp));
    }
  }
  StackClusterPrivate_SetResult(stclpp,&mep->plo);
  return  TaskForThread_Analyse;
}

/*(nil  defmethod map (objseq func  &rest arglist &env  env)  \
  (nil  cond  \
    ((nil vectorp objseq) \
      (nil  let ((n (objseq size))) \
        (n  for depth \
          (nil  apply (objseq elt depth)  func  \
            (arglist  for vectori \
              (nil  if  (depth  < (vectori  size))  \
                (vectori  elt depth)))  \
            env)))) \
    ((nil consp objseq) \
      (nil  cons  \
        (nil  apply (objseq first)  func  \
          (arglist  for i \
            (i  first)) \
          env)  \
        (nil  apply nil 'map  \
          (nil  list* (objseq rest) func  \
            (arglist  for i \
              (i  rest))) \
          env)))))*/

// not protected
// cons map func [{cons}]
TaskForThread Lisp2Dbimconsmap2(StackClusterPrivate * const stclpp){
  ParaLispObject  * listthis;
  ParaLispObject  * listthisslow ;
  ParaLispObject  * listoflistargs;
  ParaLispObject  * listoflistargs2;
  ParaLispObject  * listoflistargs2slow;
  ParaLispObject  * reslast;
  ParaLispObject  * res;
  PLOCons *  lastcons2;
  PLOCons *  taski2;
  ParaLispObject  * func;
  { EvalAddrCont2 const  c  = StackClusterPrivate_ClearflagCont (
        stclpp )  ;
    if  ( c ) {
      StackClusterPrivate_SetResult ( stclpp, StackClusterPrivate_PopValue(stclpp));
      return  TaskForThread_Analyse;
    }
  }
  { ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop(stclpp);
    func  = CCAR(arglistp);
    listoflistargs  = CCDR(arglistp);
  }
  if(CConsp(  listoflistargs))   
    listoflistargs  =   ListCopyNotProtected0(
        ParaLispObjectToPLOCons(listoflistargs));
  else
    listoflistargs  = (&  plonil.plo);
  res = &  plonil.plo;
  reslast = &  plonil.plo;
  listthis  =  StackClusterPrivate_ThisTop(stclpp);
  listthisslow = listthis ;
  bool  tocheck2 = false ;
  while ( CConsp ( listthis ) ) {
    lastcons2 =   NewCons(  func,&plonil.plo); 
    taski2 =  NewCons ( ClassCons_First ( & ParaLispObjectToPLOCons ( listthis  )->cc ) , &lastcons2 ->  plo )  ;
    listoflistargs2 = listoflistargs;
    listoflistargs2slow = listoflistargs2 ;
    { bool  tocheck = false ;
    while ( CConsp ( listoflistargs2 ) ) {
      ParaLispObject  * xj;
      { ParaLispObject * const lp = ClassCons_First ( &ParaLispObjectToPLOCons ( listoflistargs2 )->cc ) ;
        if(CConsp(lp)){
          xj = ClassCons_First ( &ParaLispObjectToPLOCons (  lp )->cc ) ;
          * ClassCons_PFirst(&ParaLispObjectToPLOCons(listoflistargs2)->cc) = ClassCons_Rest(&ParaLispObjectToPLOCons(lp)->cc);
        } else{
          xj  = (&  plonil.plo);
          ClassCons_SetFirst(&ParaLispObjectToPLOCons(  listoflistargs2) ->cc,&plonil.plo);
        }
      }
      PLOCons * const next2 =   NewCons(  xj,&plonil.plo);
      ClassCons_SetRest(&lastcons2->cc,&  next2->plo);
      lastcons2 = next2;
      listoflistargs2 = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( listoflistargs2 )->cc ) ;
        
      if ( tocheck ) {
        listoflistargs2slow = ClassCons_Rest ( & ParaLispObjectToPLOCons ( listoflistargs2slow ) -> cc ) ;
        if ( listoflistargs2 == listoflistargs2slow )
          break ;
        tocheck = false ;
      } else
        tocheck = true ;
    
      } // while is list
    } // to check
    PLOCons * const next2 =   NewCons(&plonil.plo,&plonil.plo);
    if(res  ==  &  plonil.plo)
      res = (reslast  = (&next2->plo));
    else{
      ClassCons_SetRest(&ParaLispObjectToPLOCons(reslast)->cc,&  next2->plo);
      reslast = &next2->plo;
    }
    StackClusterPrivate  * const dopstclp = 
      StackClusterPrivate_NewStCl4(stclpp);
    SetResultPlace  (  dopstclp ,
      ClassCons_PFirst ( &ParaLispObjectToPLOCons  ( reslast )->cc ) , reslast ) ;
    StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
    StackClusterPrivate_CreateTask(dopstclp ,&  taski2->plo);
        
    listthis  = ClassCons_Rest  (& ParaLispObjectToPLOCons ( listthis  )->cc ) ;
    
    if ( tocheck2 ) {
        listthisslow = ClassCons_Rest ( & ParaLispObjectToPLOCons (
          listthisslow ) -> cc ) ;
        if ( listthis == listthisslow )
          break ;
        tocheck2 = false ;
    } else
        tocheck2 = true ;
    
  } // while is list
  StackClusterPrivate_Push(stclpp, res);
  StackClusterPrivate_setTaskForManager ( stclpp , 
    TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
}

// cons mapc func [{cons}]
TaskForThread Lisp2Dbimconsmapc2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * listthis;
  ParaLispObject  * listthisslow ;
  ParaLispObject  * listoflistargs;
  ParaLispObject  * listoflistargs2;
  ParaLispObject  * listoflistargs2slow ;
  PLOCons *  lastcons2;
  PLOCons *  taski2;
  ParaLispObject  * func;
  { EvalAddrCont2 const  c  = StackClusterPrivate_ClearflagCont (
      stclpp )  ;
    if  ( c ) {
      StackClusterPrivate_SetResult(stclpp,&plonil.plo);
      return  TaskForThread_Analyse;
    }
  }
  { ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop(stclpp);
    func  = CCAR(arglistp);
    listoflistargs  = CCDR(arglistp);
  }
  if(CConsp(  listoflistargs))   
    listoflistargs  =    ListCopyNotProtected0(
        ParaLispObjectToPLOCons(listoflistargs));
  else
    listoflistargs  = (&  plonil.plo);
  listthis  =   StackClusterPrivate_ThisTop(stclpp);
  listthisslow = listthis ;
  bool tocheck2 = false ;
  while( CConsp( listthis) ) {
    lastcons2 =   NewCons(  func,&plonil.plo);
    taski2 =  NewCons (  ClassCons_First ( &ParaLispObjectToPLOCons ( listthis  )->cc ) , &lastcons2 ->  plo )  ;
    listoflistargs2 = listoflistargs;
    { bool  tocheck = false ;
      listoflistargs2slow = listoflistargs2 ;
      while (  CConsp (  listoflistargs2 ) ) {
        ParaLispObject  * xj;
        { ParaLispObject  * const  lp = ClassCons_First ( &ParaLispObjectToPLOCons ( listoflistargs2 )->cc ) ;
          if(CConsp(lp)){
            xj = ClassCons_First ( &ParaLispObjectToPLOCons (  lp )->cc ) ;
            * ClassCons_PFirst(&ParaLispObjectToPLOCons(listoflistargs2)->cc) = ClassCons_Rest(&ParaLispObjectToPLOCons(lp)->cc);
          } else{
            xj  = (&  plonil.plo);
            ClassCons_SetFirst(&ParaLispObjectToPLOCons(  listoflistargs2) ->cc,&plonil.plo);
          }
        }
        PLOCons * const next2 =   NewCons(  xj,&plonil.plo);
        ClassCons_SetRest(&lastcons2->cc,&  next2->plo);
        lastcons2 = next2;
            
        listoflistargs2 = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( listoflistargs2 )->cc ) ;
      
        if ( tocheck ) {
          listoflistargs2slow = ClassCons_Rest ( & ParaLispObjectToPLOCons ( listoflistargs2slow ) -> cc ) ;
          if ( listoflistargs2 == listoflistargs2slow )
            break ;
          tocheck = false ;
        } else
          tocheck = true ;
      
      } // while is list
    } // tocheck
    StackClusterPrivate  * const dopstclp = StackClusterPrivate_NewStCl4(stclpp);
    StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
    StackClusterPrivate_CreateTask(dopstclp ,&  taski2->plo);
        
    listthis  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( listthis  )->cc ) ;
    
    if ( tocheck2 ) {
        listthisslow = ClassCons_Rest ( & ParaLispObjectToPLOCons (
          listthisslow ) -> cc ) ;
        if ( listthis == listthisslow )
          break ;
        tocheck2 = false ;
    } else
        tocheck2 = true ;
    
  } // while is list
  StackClusterPrivate_setTaskForManager ( stclpp , 
    TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
}

// nil consp
TaskForThread Lisp2Dbimnilconsp2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const xp =CCAR(StackClusterPrivate_ArgTop(stclpp));
  StackClusterPrivate_SetResult(stclpp,(CConsp(xp) ? xp : &plofalse.plo));
  return  TaskForThread_Analyse;
}

//  nil acons x y alist
TaskForThread Lisp2Dbimnilacons23(StackClusterPrivate * const stclpp){
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp);
  ParaLispObject  * const arglistrestp =CCDR(arglistp);
  StackClusterPrivate_SetResult(stclpp,&  NewCons( & NewCons(CCAR(arglistp),
    CCAR(arglistrestp))->plo,CCAR(CCDR(arglistrestp)))->plo);
  return  TaskForThread_Analyse;
}

//  nil copy-tree
TaskForThread Lisp2Dbimnilcopytree2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp,StackClusterPrivate_ThisTop(stclpp));
  return  TaskForThread_Analyse;
}

//  nil copy-list
TaskForThread Lisp2Dbimnilcopylist2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp,&plonil.plo);
  return  TaskForThread_Analyse;
}

// ! 70% of time with ConsIterator
// returns nullptr | val
ParaLispObject  * Assoc ( ParaLispObject  * const sp ,
  ParaLispObject  const * const linksp , StackClusterPrivate  * const stclp  ) {
  ParaLispObject  const *   ja =   linksp ;

// ! ? to check cycle ? !

  while(CConsp(  ja)){
    ParaLispObject  * const envlp  = ClassCons_First ( &
      ParaLispObjectToPLOConsConst  ( ja  )->cc )  ;
    if(IsNotEnvironmentLayer(stclp,envlp))
      break ;
    { ParaLispObject * const res = EnvironmentLayerAssocPassage  (
          & ParaLispObjectToPLOEnvironmentLayer (  envlp  ) -> envlay ,  sp ) ;
      if  ( res ) 
        return  res;
    }
    ja  = ClassCons_Rest  ( &ParaLispObjectToPLOConsConst  ( ja  )->cc ) ;
  }
  return  (ParaLispObject *)0 ;
}

//  nil cons
TaskForThread Lisp2Dbimnilcons2(StackClusterPrivate * const stclpp){
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp);
  StackClusterPrivate_SetResult(stclpp,&  NewCons(CCAR(arglistp),CCAR(CCDR(arglistp)))->plo);
  return  TaskForThread_Analyse;
}

// nil first
TaskForThread Lisp2Dbimnilfirst2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp,&plonil.plo);
  return  TaskForThread_Analyse;
}

//  nil unquote-splicing
TaskForThread Lisp2Dbimnilcommaatsign2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp,CCAR(StackClusterPrivate_ArgTop(stclpp)));
  return  TaskForThread_Analyse;
}

// nil rest
TaskForThread Lisp2Dbimnilrest2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp,&plonil.plo);
  return  TaskForThread_Analyse;
}

//  nil atom
TaskForThread Lisp2Dbimnilatom2(StackClusterPrivate * const stclpp){
  ParaLispObject  * const ap =CCAR(StackClusterPrivate_ArgTop(stclpp));
  StackClusterPrivate_SetResult(stclpp,((( ap  ==  (&  plonil.plo)) or
    (  ap  ==  (&  plofalse.plo))) ?
    &plotrue.plo :
    (CConsp(ap) ? &plofalse.plo  : ap)));
  return  TaskForThread_Analyse;
}

//  nil list
TaskForThread Lisp2Dbimnillist2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp,StackClusterPrivate_ArgTop(stclpp));
  return  TaskForThread_Analyse;
}

//  nil list*
TaskForThread Lisp2Dbimnillistc2(StackClusterPrivate  * const stclpp){
  PLOCons *  lastres;
  ParaLispObject  * next;
  ParaLispObject  * nextslow ;
  ParaLispObject  * j = StackClusterPrivate_ArgTop(stclpp);
  if(not  CConsp( j)){
    // (nil list*) -> nil
    StackClusterPrivate_SetResult(stclpp,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  next  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) ;
  if(not  CConsp( next)){
    // (nil list* list) -> list
    StackClusterPrivate_SetResult ( stclpp  , ClassCons_First (
      &ParaLispObjectToPLOCons ( j )->cc ) ) ;
    return  TaskForThread_Analyse;
  }
  // (nil list* x y ...) -> (x . y) or (x y ...)
  lastres =    NewCons (  ClassCons_First ( 
    &ParaLispObjectToPLOCons ( j )->cc ) , &plonil  . plo )  ;
  StackClusterPrivate_SetResult(stclpp,&  lastres->plo);
  j = next;
  next  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( next  )->cc ) ;
  nextslow = next ;
  bool  tocheck = false ;
  while(CConsp( next)){
    { PLOCons * const lastres3 =   NewCons (
         ClassCons_First ( &ParaLispObjectToPLOCons ( j )->cc ) , &plonil  . plo )  ;
      ClassCons_SetRest(&lastres->cc,&  lastres3->plo);
      lastres = lastres3;
    }
    j = next;
    next  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( next  )->cc ) ;
    if ( tocheck ) {
      nextslow = ClassCons_Rest ( & ParaLispObjectToPLOCons ( nextslow ) -> cc ) ;
      if ( next == nextslow )
        break ;
      tocheck = false ;
    } else
      tocheck = true ;
  }
  // j is Cons
  // First(j) - will be tail
  * ClassCons_PRest ( &lastres->cc ) = ClassCons_First ( &ParaLispObjectToPLOCons ( j )->cc ) ;
  return  TaskForThread_Analyse;
}

//  nil last
TaskForThread Lisp2Dbimnillast2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp,&plonil.plo);
  return  TaskForThread_Analyse;
}

// nil listp
TaskForThread Lisp2Dbimnillistp2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const xp =CCAR(StackClusterPrivate_ArgTop(stclpp));
  switch(ParaLispObject_Type(xp)){
  case  TypeGlobalListMemberPLONil:
    StackClusterPrivate_SetResult(stclpp,&plotrue.plo);
    break;
  case  TypeGlobalListMemberPLOCons:
    StackClusterPrivate_SetResult(stclpp,xp);
    break;
  default:
    StackClusterPrivate_SetResult(stclpp,&plofalse.plo);
  }
  return  TaskForThread_Analyse;
}

//  nil for*
TaskForThread Lisp2Dbimnilforcons2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp,&plonil.plo);
  return  TaskForThread_Analyse;
}

// list without [iteratorend] => list - end
TaskForThread Lisp2Dbimconswithout2(StackClusterPrivate * const stclpp){
  ParaLispObject  * mejslow ;
  PLOCons * resultj;
  ParaLispObject  * indexfirst  = CCAR(StackClusterPrivate_ArgTop(stclpp));
  // travel to the beginning
  ParaLispObject  * mej =   StackClusterPrivate_ThisTop(stclpp);
  if(indexfirst ==  mej){
    StackClusterPrivate_SetResult(stclpp,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  resultj  =  NewCons(ClassCons_First(&ParaLispObjectToPLOCons(mej)->cc),
    &plonil.plo);
  StackClusterPrivate_SetResult(stclpp,&  resultj->plo);
  mej = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( mej )->cc ) ;
  { mejslow = mej ;
    bool  tocheck = false ;
    while ( (mej not_eq  indexfirst) and CConsp( mej)){
      { PLOCons * const newresultj2  = NewCons(
          ClassCons_First(&ParaLispObjectToPLOCons(mej)->cc),&plonil.plo);
        ClassCons_SetRest(&resultj->cc,&  newresultj2->plo);
        resultj = newresultj2;
      }
      
      mej = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( mej )->cc ) ;
      
      if ( tocheck ) {
        mejslow = ClassCons_Rest ( & ParaLispObjectToPLOCons (
          mejslow ) -> cc ) ;
        if ( mej == mejslow )
          break ;
        tocheck = false ;
      } else
        tocheck = true ;
      
    }
  }  // tasks
  if(not  CConsp( mej))
    // end-index placed to the end OR very short list
    return  TaskForThread_Analyse;
  return  TaskForThread_Analyse;
}

// !! to write Template List
  
# define  sListEPLOConscp Lisp2D_sListEPLOConscp
# define  ListEPLOConscp  Lisp2D_ListEPLOConscp
typedef struct  sListEPLOConscp ListEPLOConscp  ;
  
struct  sListEPLOConscp {
  PLOCons const * c ;
  ListEPLOConscp  * p ;
  ListEPLOConscp  * n ;
} ;

# define  sListPLOConscp Lisp2D_sListPLOConscp
# define  ListPLOConscp  Lisp2D_ListPLOConscp
typedef struct  sListPLOConscp ListPLOConscp  ;
  
struct  sListPLOConscp {
  ListEPLOConscp  * f ;
  ListEPLOConscp  * l ;
} ;

# define  ListPLOConscpInit Lisp2D_ListPLOConscpInit
inline  void  ListPLOConscpInit ( ListPLOConscp * me ) ;
  
# define  ListPLOConscpPushBack Lisp2D_ListPLOConscpPushBack
void  ListPLOConscpPushBack ( ListPLOConscp * me , PLOCons const * x ) ;

# define  ListPLOConscpBackValue Lisp2D_ListPLOConscpBackValue
inline  PLOCons const * ListPLOConscpBackValue ( ListPLOConscp * me ) ;

# define  ListPLOConscpPopBack Lisp2D_ListPLOConscpPopBack
static  void ListPLOConscpPopBack ( ListPLOConscp * const  me ) {
  if ( me -> l ) {
    ListEPLOConscp  * const lap  = me  ->  l ->  p ;
    if ( lap )
      lap -> n = (ListEPLOConscp  *)0 ;
    else
      me -> f = (ListEPLOConscp  *)0 ;
    free  ( me -> l ) ;
    me -> l = lap ;
    return  ;
  }
  Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& 
    L"ListPLOConscpPopBack:list is empty" ; 
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
}

# define  ListPLOConscpDestr  Lisp2D_ListPLOConscpDestr
void  ListPLOConscpDestr  ( ListPLOConscp * me ) ;
  
# define  CopyTreem2  Lisp2D_CopyTreem2
static  ParaLispObject  * CopyTreem2 ( PLOCons * const cp , ListPLOConscp * const listhistoryp ) {
  ParaLispObject  * arg = & plonil.plo ;
  PLOCons *  lastarg = (PLOCons *)0 ;
  ParaLispObject  * ja;
  ParaLispObject  * jaslow ;
  bool  flagfirstelementpushed = false ;
  ja  = &  cp->plo;
  jaslow = ja ;
  bool  tocheck = false ;
  while ( CConsp  ( ja  ) ) {
    ListPLOConscpPushBack ( listhistoryp  , ParaLispObjectToPLOConsConst  ( ja  ) ) ;
    if(not  flagfirstelementpushed)
      flagfirstelementpushed  = true;
    ParaLispObject  * const jacarp = ClassCons_First ( & ParaLispObjectToPLOConsConst  ( ja  ) ->  cc ) ;
    PLOCons * const next2 =   NewCons((CConsp(jacarp)  ?
      CopyTreem2(ParaLispObjectToPLOCons(jacarp),listhistoryp)  :
      jacarp),&plonil.plo); 
    
    if(lastarg) 
      ClassCons_SetRest(&lastarg->cc,&  next2->plo);
    else 
      arg = (&next2->plo);
    lastarg = next2;
    
    ja  = ClassCons_Rest  ( &ParaLispObjectToPLOConsConst  ( ja  )->cc ) ;
     
    if ( tocheck ) {
      jaslow = ClassCons_Rest ( & ParaLispObjectToPLOCons ( jaslow ) -> cc ) ;
      if ( ja == jaslow )
        break ;
      tocheck = false ;
    } else
      tocheck = true ;
  }
  if  ( lastarg )
    ClassCons_SetRest ( & lastarg ->  cc  , ja  ) ;
  // clear steps
  { PLOCons const * j = ListPLOConscpBackValue ( listhistoryp ) ;
    while ( j not_eq   cp ) {
      ListPLOConscpPopBack  ( listhistoryp  ) ;
      j = ListPLOConscpBackValue ( listhistoryp ) ;
    }
  }
  if  ( flagfirstelementpushed  )
    ListPLOConscpPopBack  ( listhistoryp  ) ;
  return  arg ;
}
  
ParaLispObject  * CopyTreem1 ( PLOCons * const c ) {
  ListPLOConscp listhistory ;
  ListPLOConscpInit ( & listhistory ) ;
  ParaLispObject  * const res = CopyTreem2  ( c , & listhistory ) ;
  ListPLOConscpDestr  ( & listhistory ) ;
  return  res ;
}

# include "plocomplex/type.h"
# include "plocomplex/public.h"
# include "complex/type.h"
# include "plocomplex/public.h"
# include "complex/inline.h"

//  cons size
TaskForThread Lisp2Dbimconssize2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * j ;
  ParaLispObject  * jslow ;
  PLOComplex  * const res = NewComplexBareExc ( ) ;
  if ( not res )
    return  TaskForThread_Analyse ;
  Complex * const cres = PLOComplexToComplex ( res ) ;
  Complex_Set0  ( cres ) ; 
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  j = StackClusterPrivate_ThisTop(stclpp);
  jslow = j ;
  bool  tocheck = false ;
  while ( CConsp ( j ) ) {
    FractionAdd1 ( Complex_Re  ( cres ) );
    j = ClassCons_Rest ( & ParaLispObjectToPLOCons ( j ) -> cc ) ;
    if ( tocheck ) {
      jslow = ClassCons_Rest ( & ParaLispObjectToPLOCons (
        jslow ) -> cc ) ;
      if ( j == jslow )
        break ;
      tocheck = false ; 
    } else
      tocheck = true ; 
  }
  return  TaskForThread_Analyse ; 
}

# include <vector/type.h>
# include <vector/struct.h>
# include <vector/public.h> // PLOVector_SetElt

//  cons vector
TaskForThread Lisp2Dbimconsvector2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * j;
  ParaLispObject  * jslow ;
  PLOComplex  * const listsize = NewComplexBareExc ( ) ;
  if  ( not listsize  )
    return  TaskForThread_Analyse;
  Complex * const clistsize = PLOComplexToComplex (  listsize  ) ;
  Complex_Set0  ( clistsize ) ;
  j = StackClusterPrivate_ThisTop(stclpp);
  jslow = j ;
  { bool  tocheck = false ;
    // getting size
    while (  CConsp ( j ) ) {
      FractionAdd1  (Complex_Re(clistsize));
      j  = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc ) ;
      if ( tocheck ) {
        jslow = ClassCons_Rest ( & ParaLispObjectToPLOCons (
          jslow ) -> cc ) ;
        if ( j == jslow )
          break ;
        tocheck = false ; 
      } else
        tocheck = true ; 
    }
  } // to check
  size_t  ls;
  if( FractionIsBadSizeValue  (Complex_CRe(clistsize),& ls))
    return  TaskForThread_Analyse;
  // listsize is finite
  PLOVector  * const res = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if ( not  res ) {
    OutOfMemory(stclpp);
    return  TaskForThread_Analyse;
  }
    
  // ! to catch exception
  PLOVector_InitSize(res,ls);
  
  StackClusterPrivate_SetResult ( stclpp  , ( ParaLispObject_virtualPushAsNew (
          & res ->  plo ) ) ) ;
  j =   StackClusterPrivate_ThisTop(stclpp);
  for(size_t  i=0;i  < ls;++ i) {
    PLOVector_SetElt(res,i,CCAR(   j));
    j = CCDR(   j); 
  }
  return  TaskForThread_Analyse; 
}

// nil size
TaskForThread Lisp2Dbimnilsize2(StackClusterPrivate * const stclpp){
  { PLOComplex  * const  result =  NewComplexBareExc() ;
    if  ( result  ) 
      Complex_Set0  ( PLOComplexToComplex(result));    
    else
      return  TaskForThread_Analyse;
    StackClusterPrivate_SetResult(stclpp ,&  result->plo);}
  return  TaskForThread_Analyse;
}

# include "string/public.h"
# include "string/struct.h"

// list subseq [indexfirst [indexend]] => [first..end-1]
TaskForThread Lisp2Dbimconssubseq2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * mejslow ;
  PLOComplex  * indexj;
  ParaLispObject  * indexfirst;
  PLOCons * resultj;
  ParaLispObject  * j =   StackClusterPrivate_ArgTop(stclpp);
  ParaLispObject  * mej =   StackClusterPrivate_ThisTop(stclpp);
  if(not  CConsp( j)){
    // mode to the end
FindEndWithoutIndex0:  
    resultj  =   NewCons (  ClassCons_First ( 
      &ParaLispObjectToPLOCons ( mej )->cc ) , &plonil  . plo ) ;
    StackClusterPrivate_SetResult(stclpp,&  resultj->plo);
    mej = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( mej )->cc ) ;
    { mejslow = mej ;
      bool  tocheck = false ;
      while( CConsp( mej)){
        { PLOCons * const newresultj2  = NewCons(
             ClassCons_First ( &ParaLispObjectToPLOCons ( mej )->cc ) ,
            &plonil  . plo ) ;
          ClassCons_SetRest(&resultj->cc,&  newresultj2->plo);
          resultj = newresultj2;
        }
        
        mej = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( mej )->cc ) ;
        
        if ( tocheck ) {
          mejslow = ClassCons_Rest ( & ParaLispObjectToPLOCons (
            mejslow ) -> cc ) ;
          if ( mej == mejslow )
            break ;
          tocheck = false ; 
        } else
          tocheck = true ; 
      }
    }
    return  TaskForThread_Analyse;
  }  // if(not  Consp(* j))
  indexfirst  = ClassCons_First(&ParaLispObjectToPLOCons(j)->cc);
  if(IsNotNumber(  stclpp,  indexfirst))   
    return  TaskForThread_Analyse;
  if(not  FractionIntegerP(Complex_CRe(PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(
        indexfirst))))){
    ParaLispObject  * list [ ] = { indexfirst,  & PLOString_NewCopy(
      L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  if(FractionLessUInt(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          indexfirst))),0U)){
      ParaLispObject  * list [ ] = { indexfirst,
        & PLOString_NewCopy(L" < 0")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
  }
  // travel to the beginning
  indexj  = NewComplexBareExc();
  Complex_Set0  ( PLOComplexToComplex(indexj));    
  if(not  (FractionIs0(Complex_CRe(
    PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(   indexfirst)))
    ))){
    do  {
      mej = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( mej )->cc ) ;
      FractionAdd1  (Complex_Re(PLOComplexToComplex(indexj)));
    } while (
      ( Complex_Less( PLOComplexToComplexConst(indexj),
          PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
            indexfirst)) )  ==
        Lisp2DBoolTrue ) and  CConsp(  mej) ) ;
    if ( not  CConsp( mej) ) {
      // first element is the end OR very short list
      StackClusterPrivate_SetResult(stclpp,  mej);
      return  TaskForThread_Analyse;
    }
  } // indexfirst not_eq 0
  j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) ;
  if(not  CConsp( j))   
    goto  FindEndWithoutIndex0;
  indexfirst  = ClassCons_First(&ParaLispObjectToPLOCons(j)->cc);
  if(IsNotNumber(  stclpp, indexfirst))   
    return  TaskForThread_Analyse;
  if(not  FractionIntegerP(Complex_CRe(PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(
        indexfirst))))){
    ParaLispObject  * list [ ] = { indexfirst,& PLOString_NewCopy(
      L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  if(FractionLessUInt(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indexfirst))), 0U)){
    ParaLispObject  * list [ ] = { indexfirst,
      & PLOString_NewCopy(L" < 0")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  } // zero  
  if(Complex_Less(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
    indexfirst)),
      PLOComplexToComplexConst(indexj)) == Lisp2DBoolTrue ) {
    ParaLispObject  * list [ ] = { indexfirst,& PLOString_NewCopy(L" < ")
      ->plo,&indexj->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));  
    return  TaskForThread_Analyse;
  }
  if(FractionEq(Complex_CRe(
        PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          indexfirst))),
      Complex_CRe(PLOComplexToComplexConst(indexj)))){
    StackClusterPrivate_SetResult(stclpp,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  resultj  =  NewCons(ClassCons_First(
    &ParaLispObjectToPLOCons(mej)->cc),&plonil.plo);
  StackClusterPrivate_SetResult(stclpp,&  resultj->plo);
  mej = ClassCons_Rest  (& ParaLispObjectToPLOCons ( mej )->cc ) ;
  FractionAdd1  (Complex_Re(PLOComplexToComplex(indexj)));
  mejslow = mej ;
  { bool  tocheck = false ;
    while ( (Complex_Less( PLOComplexToComplexConst(indexj),
      PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(   indexfirst)))
      ==  Lisp2DBoolTrue) and CConsp( mej)) {
      { PLOCons * const newresultj2  = NewCons(
          ClassCons_First(&ParaLispObjectToPLOCons(mej)->cc),&plonil.plo);
        ClassCons_SetRest(&resultj->cc,&  newresultj2->plo);
        resultj = newresultj2 ; 
      } // newresultj2
      FractionAdd1  (Complex_Re(PLOComplexToComplex(indexj)));
      
      mej = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( mej ) ->cc) ; 
        
      if ( tocheck ) {
        mejslow = ClassCons_Rest ( & ParaLispObjectToPLOCons (
          mejslow ) -> cc ) ;
        if ( mej == mejslow )
          break ;
        tocheck = false ;
      } else
        tocheck = true ;
    }
  }  // tasks
  if(not  CConsp( mej))
    // end-index placed to the end OR very short list
    return  TaskForThread_Analyse;
  return  TaskForThread_Analyse;
}

void  ClassCons_Swap  ( ClassCons * const me , ClassCons * const x ) {
  if  ( me not_eq  x ) {
    { ParaLispObject  * const tmp = me -> first ;
      me -> first = x -> first  ;
      x -> first  = tmp ;
    }
    { ParaLispObject  * const tmp = me -> rest ;
      me -> rest = x -> rest ;
      x -> rest = tmp ;
    }
  }
}

void  ClassCons_Set ( ClassCons * const me , ClassCons const * const x ) {
  if  ( me not_eq  x ) {
    me -> first = ( x -> first ) ;
    me -> rest  = ( x -> rest  ) ;
  }
}

void  PLOCons_Init ( PLOCons * const mep ) {
  VirtualTypePointer  const vtp = { . addr = { . p = mep } , . type = TypeGlobalListMemberPLOCons } ;
  ParaLispObject_InitVTP(&mep->plo ,vtp);
  ClassCons_Init(&mep->cc);
# ifdef  LISP2D_CONSTRSTAT
  PLOCons_constrstatInc ( ) ;
# endif
}

inline  ParaLispObject *  plo_constcast ( ParaLispObject const * const p ) {
  return  ( ParaLispObject * ) p ;
}

inline  void  ListPLOConscpInit ( ListPLOConscp * const  me ) {
  me -> f = 0 ;
  me -> l = 0 ;
}

void  ListPLOConscpPushBack ( ListPLOConscp * const  me , PLOCons const * const x ) {
  ListEPLOConscp * const ne = aligned_alloc ( _Alignof ( ListEPLOConscp ) , sizeof ( ListEPLOConscp ) ) ;
  if ( not ne ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"ListPLOConscpPushBack:aligned alloc:not ne" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  ne -> c = x ;
  ne -> p = me  ->  l ;
  ne -> n = 0 ;
  if  ( me  ->  l )
    me  ->  l ->  n = ne  ;
  else
    me  ->  f = ne  ;
  me  ->  l = ne  ;
}

inline  PLOCons const * ListPLOConscpBackValue ( ListPLOConscp * const  me ) {
  if ( me -> l )
    return  me -> l -> c ;
  Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
    L"ListPLOConscpBackValue:list is empty" ;
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
}

void  ListPLOConscpDestr  ( ListPLOConscp * const  me ) {
  while ( me  ->  f )
    ListPLOConscpPopBack  ( me  ) ;
}
