// Lisp2D - deque init
// ©2020-3 by Alex Glebe
# include <stdlib.h>
# include <iso646.h>
# include <string.h>
# include "deque/public-wchar.h"
# include "deque/public-byte.h"
# include "deque/struct.h"
# include "excc.h"

// 840 = 7 × 6 × 5 × 4
enum { DequeBrachDefaultSizeBytes = 840 } ;

# include "cast.h"

// 1 .. 840
inline  unsigned  int DequeBrachDefaultSize ( size_t  s ) ;

void  DequeVoidInit ( DequeVoid * const restrict  dq  ) {
  dq -> map = 0 ;
  dq -> mapsize = 0 ;
  dq -> indexfirst  = 0 ;
  dq -> indexlast = 0 ;
}

# define  constcastvoidpp  Lisp2D_constcastvoidpp
inline  void  const * const * constcastvoidpp ( void  * * p ) ;

# include <stdio.h>

void  Deque_Void_InitCopy ( DequeVoid * const restrict  dq ,
  DequeVoid const * const restrict  dq2 , size_t  const elt_size , unsigned int const ali ) {
  if ( dq == dq2 )
    return  ;
  if ( dq2 -> mapsize == 0 )  {
    DequeVoidInit ( dq ) ;
    return ;
  }
  dq -> mapsize = dq2 -> mapsize ;
  dq -> map = aligned_alloc ( _Alignof ( void * [ dq -> mapsize ] ) , sizeof  ( void * [ dq -> mapsize ] ) ) ;
  if ( not ( dq -> map ) ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Void_InitCopy" ; 
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  dq  ->  indexfirst  = dq2  -> indexfirst ;
  dq  ->  indexlast = dq2  -> indexlast ;
  void  * * jme = dq -> map ;
  void  const * const * jot = constcastvoidpp ( dq2 -> map ) ;
  void  const * const * const jen = constcastvoidpp ( dq -> map + dq -> mapsize ) ;
  size_t  const branchsize  = elt_size * DequeBrachDefaultSize ( elt_size ) ;
  while ( constcastvoidpp ( jme ) < jen ) {
    * jme = aligned_alloc ( ali , branchsize ) ;
    if ( not ( * jme ) ) {
      while ( jme not_eq dq -> map )  {
        --  jme ;
        free  ( * jme ) ;
      }
      free  ( dq -> map ) ;
      Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Void_InitCopy" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    memcpy  ( * jme , * jot , branchsize ) ;
    ++  jme ;
    ++  jot ;
  }
}

void  Deque_Void_InitMove ( DequeVoid * const restrict  dq , DequeVoid * const restrict  dq2 ) {
  if ( dq == dq2 )
    return ;
  dq -> map = dq2 -> map ;
  dq -> mapsize = dq2 -> mapsize ;
  dq  ->  indexfirst = dq2  ->  indexfirst ;
  dq  -> indexlast = dq2  -> indexlast ;
  DequeVoidInit ( dq2 ) ;
}

void  Deque_Void_Set ( DequeVoid * const restrict  dq ,
  DequeVoid const * const restrict  dq2 , size_t  const elt_size , unsigned int const ali ) {
  if ( dq == dq2 )
    return  ;
  DequeVoidDestr  ( dq  ) ;
  Deque_Void_InitCopy ( dq  , dq2 , elt_size , ali ) ;
}

void  Deque_Void_Move ( DequeVoid * const restrict  dq , DequeVoid * const restrict  dq2 ) {
  if ( dq == dq2 )
    return  ;
  DequeVoidDestr  ( dq  ) ;
  Deque_Void_InitMove ( dq  , dq2 ) ;
}

void  DequeVoidDestr ( DequeVoid * const dq  ) {
  if  ( dq -> mapsize ) {
    void  * * const m = dq -> map ;
    { void  * * j = m + dq -> mapsize ;
      do  {
        --  j ;
        free  ( * j ) ;
      } while ( j not_eq  m ) ;
    }
    free  ( m ) ;
  }
}

void  DequeVoidClear  ( DequeVoid * const restrict  dq  ) {
  DequeVoidDestr ( dq ) ;
  DequeVoidInit ( dq ) ;  
}

void  Deque_Void_Swap ( DequeVoid * const restrict  dq , DequeVoid * const restrict  dq2 ) {
  if ( dq == dq2 )
    return  ;
  { void  * * const tmp = dq -> map ;
    dq -> map = dq2 -> map ;
    dq2 -> map =  tmp ;
  }
  { size_t  tmp = dq -> mapsize ;
    dq -> mapsize = dq2 -> mapsize ;
    dq2 -> mapsize =  tmp ;
  }
  unsigned  int tmp = dq  -> indexfirst ;
  dq  -> indexfirst = dq2 -> indexfirst ;
  dq2 -> indexfirst =  tmp ;
  tmp = dq  -> indexlast ;
  dq  -> indexlast = dq2 -> indexlast ;
  dq2 -> indexlast =  tmp ;
}

// second arg is size of one element in bytes
// third arg is count of this elements
void  DequeVoidResize ( DequeVoid * const restrict  dq , size_t const elemsize ,
  size_t const count , unsigned int const ali ) {
  if  ( count ==  0 ) {
    DequeVoidClear  ( dq  ) ;
    return  ;
  }
  if ( dq -> mapsize == 0 )
    dq  -> indexfirst = 0 ;
  unsigned int const branchsize =  DequeBrachDefaultSize ( elemsize ) ;
  // branchsize = 1 .. 840
  ldiv_t divbranches = ldiv  ( sizet_cast_longint  ( count ) + dq  -> indexfirst , uint_cast_longint ( branchsize ) ) ;
  if  ( divbranches . rem )
    ++  divbranches . quot  ;
  if  ( divbranches . quot not_eq sizet_cast_longint ( dq -> mapsize ) ) {
    // increase / decrease map

    // ! to aligned_alloc and memcpy

    void  * * const newmap  = realloc  ( dq -> map , sizeof  ( void  * ) * longint_cast_size_t ( divbranches . quot ) ) ;
    if ( not newmap ) {
      Lisp2DExcOutOfMemoryGlobal . value = L"DequeVoidResize" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    dq -> map = newmap  ;
    size_t  const old_mapsize = dq -> mapsize ;
    dq -> mapsize = longint_cast_size_t ( divbranches . quot ) ;
    if  ( divbranches . quot > sizet_cast_longint ( old_mapsize ) ) {
      unsigned  long  int const branchsizebyte = branchsize * elemsize ;
      void  * * restrict j = & ( dq -> map [ old_mapsize ] ) ;
      do  {
        ( * j ) = aligned_alloc ( ali , branchsizebyte ) ;
        if  ( not ( * j ) ) {
          ++  j ;
          // all pointers must be NULL, not garbage
          while ( j not_eq & ( dq -> map [ divbranches . quot ] ) ) {
            ( * j ) = 0 ;
            ++  j ;
          }
          Lisp2DExcOutOfMemoryGlobal . value = L"DequeVoidResize" ;
          longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
        }
        ++  j ;
      } while ( j not_eq & ( dq -> map [ divbranches . quot ] ) ) ;
    }
  }
  dq  -> indexlast = longint_cast_uint ( ( divbranches . rem + branchsize - 1 ) % branchsize ) ;
}

# define  Deque_ResizeValTempl( NAME , TYPE ) \
void  Deque_ResizeVal ( NAME ) ( Deque ( NAME ) * const restrict me , size_t const count  , TYPE const x ) { \
  size_t  j = Deque_Size  ( NAME  ) ( me  ) ; \
  DequeVoidResize ( me , sizeof ( TYPE ) , count , _Alignof ( TYPE ) ) ; \
  while ( j < count ) { \
    ( * Deque_At ( NAME ) ( me  , j ) ) = x ; \
    ++  j ; \
  } \
}

Deque_ResizeValTempl ( WChar , wchar_t )
Deque_ResizeValTempl ( Byte , uint8_t )

# define  Deque_AtTempl( NAME , TYPE , CAST ) \
TYPE * Deque_At ( NAME ) ( Deque ( NAME ) * const restrict me , size_t const ind ) {  \
  unsigned int const branchsize =  DequeBrachDefaultSize ( sizeof ( TYPE ) ) ;  \
  ldiv_t const  divbranches = ldiv  ( uint_cast_longint ( me  -> indexfirst ) + sizet_cast_longint ( ind ) ,  \
    uint_cast_longint ( branchsize ) ) ;  \
  if ( me -> mapsize == 0 ) { \
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & L"Deque_At : deque is empty" ;  \
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ; \
  } \
  if  ( divbranches . quot + 1 < sizet_cast_longint ( me -> mapsize ) or  \
    ( divbranches . quot + 1 == sizet_cast_longint ( me -> mapsize ) and  \
      divbranches . rem <= me -> indexlast ) ) \
    return  & ( CAST ( me -> map [ divbranches . quot ] ) [ divbranches . rem ] ) ; \
  Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & L"Deque_At : out of range" ;  \
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ; \
}

Deque_AtTempl ( WChar , wchar_t , voidp_cast_wcharp )
Deque_AtTempl ( Byte , uint8_t , voidp_cast_bytep )

# define  Deque_FrontTempl( NAME , TYPE , CAST ) \
TYPE * Deque_Front ( NAME ) ( Deque ( NAME ) * const restrict me ) {  \
  if ( me -> mapsize ) \
    return  & ( CAST ( me -> map [ 0 ] ) [ me -> indexfirst ] ) ; \
  return  NULL ; \
}

Deque_FrontTempl ( WChar , wchar_t , voidp_cast_wcharp )
Deque_FrontTempl ( Byte , uint8_t , voidp_cast_bytep )

# define  Deque_BackTempl( NAME , TYPE , CAST ) \
TYPE * Deque_Back ( NAME ) ( Deque ( NAME ) * const restrict me ) {  \
  if ( me -> mapsize ) \
    return  & ( CAST ( me -> map [ me -> mapsize - 1 ] ) [ me -> indexlast ] ) ; \
  return  NULL ; \
}

Deque_BackTempl ( WChar , wchar_t , voidp_cast_wcharp )
Deque_BackTempl ( Byte , uint8_t , voidp_cast_bytep )

# define  Deque_AtFastTempl( NAME , TYPE , CAST ) \
TYPE * Deque_AtFast ( NAME ) ( Deque ( NAME ) * const restrict me , size_t const ind ) {  \
  unsigned int const branchsize =  DequeBrachDefaultSize ( sizeof ( TYPE ) ) ;  \
  ldiv_t const  divbranches = ldiv  ( uint_cast_longint ( me -> indexfirst ) + sizet_cast_longint ( ind ) ,  \
    uint_cast_longint ( branchsize ) ) ;  \
  return  & ( CAST ( me -> map [ divbranches . quot ] ) [ divbranches . rem ] ) ; \
}

Deque_AtFastTempl ( WChar , wchar_t , voidp_cast_wcharp )
Deque_AtFastTempl ( Byte , uint8_t , voidp_cast_bytep )

# define  Deque_EndTempl( NAME , TYPE ) \
DequeIterator  ( NAME ) Deque_End ( NAME ) ( Deque ( NAME ) * const restrict me ) {  \
  if ( me -> mapsize ) { \
    unsigned int const branchsize =  DequeBrachDefaultSize ( sizeof ( TYPE ) ) ;  \
    if ( me -> indexlast + 1 < branchsize ) \
      return  ( DequeIterator  ( NAME ) ) { \
        . line = & ( me -> map [ me -> mapsize - 1 ] ) , \
        . index = me -> indexlast + 1 } ; \
    return  ( DequeIterator  ( NAME ) ) { \
      . line = & ( me -> map [ me -> mapsize ] ) , \
      . index = 0 } ; \
  } \
  return  ( DequeIterator  ( NAME ) ) { \
    . line = NULL , \
    . index = 0 } ; \
}

Deque_EndTempl ( WChar , wchar_t )
Deque_EndTempl ( Byte , uint8_t )

# define  Deque_CEndTempl( NAME , TYPE ) \
DequeConstIterator  ( NAME ) Deque_CEnd ( NAME ) (  \
  Deque ( NAME ) const * const restrict me ) { \
  if ( me -> mapsize ) { \
    unsigned int const branchsize =  DequeBrachDefaultSize ( sizeof ( TYPE ) ) ;  \
    if ( me -> indexlast + 1 < branchsize ) \
      return  ( DequeConstIterator  ( NAME ) ) { \
        { . line = & ( me -> map [ me -> mapsize - 1 ] ) , \
          . index = me -> indexlast + 1 } } ; \
    return  ( DequeConstIterator  ( NAME ) ) { \
      { . line = & ( me -> map [ me -> mapsize ] ) , \
        . index = 0 } } ; \
  } \
  return  ( DequeConstIterator  ( NAME ) ) { \
    { . line = NULL , \
      . index = 0 } } ; \
}

Deque_CEndTempl ( WChar , wchar_t )
Deque_CEndTempl ( Byte , uint8_t )

# define  Deque_REndTempl( NAME , TYPE ) \
DequeReverseIterator  ( NAME ) Deque_REnd ( NAME ) ( \
  Deque ( NAME ) * const restrict me  ) { \
  if ( me -> mapsize ) { \
    if ( me -> indexfirst ) \
      return  ( DequeReverseIterator  ( NAME ) ) { \
        { . line = & ( me -> map [ 0 ] ) , \
          . index = me -> indexfirst - 1 } } ; \
    unsigned int const branchsize =  DequeBrachDefaultSize ( sizeof ( TYPE ) ) ;  \
    return  ( DequeReverseIterator  ( NAME ) ) { \
      { . line = & ( me -> map [ -1 ] ) , \
        . index = branchsize - 1 } } ; \
  } \
  return  ( DequeReverseIterator  ( NAME ) ) { \
    { . line = NULL , \
      . index = 0 } } ; \
}

Deque_REndTempl ( WChar , wchar_t )
Deque_REndTempl ( Byte , uint8_t )

# define  Deque_CREndTempl( NAME , TYPE ) \
DequeConstReverseIterator  ( NAME ) Deque_CREnd ( NAME ) ( \
  Deque ( NAME ) const * const restrict me  ) { \
  if ( me -> mapsize ) { \
    if ( me -> indexfirst ) \
      return  ( DequeConstReverseIterator  ( NAME ) ) { \
        { . line = & ( me -> map [ 0 ] ) , \
          . index = me -> indexfirst - 1 } } ; \
    unsigned int const branchsize =  DequeBrachDefaultSize ( sizeof ( TYPE ) ) ;  \
    return  ( DequeConstReverseIterator  ( NAME ) ) { \
      { . line = & ( me -> map [ -1 ] ) , \
        . index = branchsize - 1 } } ; \
  } \
  return  ( DequeConstReverseIterator  ( NAME ) ) { \
    { . line = NULL , \
      . index = 0 } } ; \
}

Deque_CREndTempl ( WChar , wchar_t )
Deque_CREndTempl ( Byte , uint8_t )

# define  Deque_BeginTempl( NAME ) \
DequeIterator  ( NAME ) Deque_Begin ( NAME ) ( Deque ( NAME ) const  * const restrict me  ) { \
  return  ( DequeIterator  ( NAME ) ) { \
    . line = me -> map , \
    . index = me -> indexfirst } ; \
}

Deque_BeginTempl ( WChar )
Deque_BeginTempl ( Byte )

# define  Deque_CBeginTempl( NAME ) \
DequeConstIterator  ( NAME ) Deque_CBegin ( NAME ) ( Deque ( NAME ) const  * const restrict me  ) { \
  return  ( DequeConstIterator  ( NAME ) ) { \
    { . line = me -> map , \
      . index = me -> indexfirst } } ; \
}

Deque_CBeginTempl ( WChar )
Deque_CBeginTempl ( Byte )

# define  Deque_RBeginTempl( NAME ) \
DequeReverseIterator  ( NAME ) Deque_RBegin ( NAME ) ( Deque ( NAME ) * const restrict me ) {  \
  if ( me -> mapsize ) \
    return  ( DequeReverseIterator  ( NAME ) ) { \
      { . line = & ( me -> map [ me -> mapsize - 1 ] ) , \
        . index = me -> indexlast } } ; \
  return  ( DequeReverseIterator  ( NAME ) ) { \
    { . line = NULL , \
      . index = 0 } } ; \
}

Deque_RBeginTempl ( WChar )
Deque_RBeginTempl ( Byte )

# define  Deque_CRBeginTempl( NAME ) \
DequeConstReverseIterator  ( NAME ) Deque_CRBegin ( NAME ) ( Deque ( NAME ) const * const restrict me ) { \
  if ( me -> mapsize ) \
    return  ( DequeConstReverseIterator  ( NAME ) ) { \
      { . line = & ( me -> map [ me -> mapsize - 1 ] ) , \
        . index = me -> indexlast } } ; \
  return  ( DequeConstReverseIterator  ( NAME ) ) { \
    { . line = NULL , \
      . index = 0 } } ; \
}

Deque_CRBeginTempl ( WChar )
Deque_CRBeginTempl ( Byte )

# define  Deque_ConstIterator_Inc_Templ( NAME , TYPE ) \
void  Deque_ConstIterator_Inc ( NAME ) ( DequeConstIterator ( NAME ) * const i ) { \
  if ( i -> base . index == DequeBrachDefaultSize ( sizeof  ( TYPE ) ) - 1 ) { \
    i -> base . index = 0 ; \
    ++  ( i -> base . line ) ; \
  } else  \
    ++  ( i -> base . index ) ; \
}

Deque_ConstIterator_Inc_Templ ( WChar , wchar_t )
Deque_ConstIterator_Inc_Templ ( Byte , uint8_t )

# define  Deque_ConstIterator_Dec_Templ( NAME , TYPE ) \
void  Deque_ConstIterator_Dec ( NAME ) ( DequeConstIterator ( NAME ) * const i ) { \
  if ( i -> base . index == 0 ) { \
    i -> base . index = DequeBrachDefaultSize ( sizeof  ( TYPE ) ) - 1 ; \
    --  ( i -> base . line ) ; \
  } else  \
    --  ( i -> base . index ) ; \
}

Deque_ConstIterator_Dec_Templ ( WChar , wchar_t )
Deque_ConstIterator_Dec_Templ ( Byte , uint8_t )

# define  Deque_Iterator_Inc_Templ( NAME , TYPE ) \
void  Deque_Iterator_Inc ( NAME ) ( DequeIterator ( NAME ) * const i ) { \
  if ( i -> index == DequeBrachDefaultSize ( sizeof  ( TYPE ) ) - 1 ) { \
    i -> index = 0 ; \
    ++  ( i -> line ) ; \
  } else  \
    ++  ( i -> index) ; \
}

Deque_Iterator_Inc_Templ ( WChar , wchar_t )
Deque_Iterator_Inc_Templ ( Byte , uint8_t )

# define  Deque_Iterator_Dec_Templ( NAME , TYPE ) \
void  Deque_Iterator_Dec ( NAME ) ( DequeIterator ( NAME ) * const i ) { \
  if ( i -> index == 0 ) { \
    i -> index = DequeBrachDefaultSize ( sizeof  ( TYPE ) ) - 1 ; \
    --  ( i -> line) ; \
  } else  \
    --  ( i -> index) ; \
}

Deque_Iterator_Dec_Templ ( WChar , wchar_t )
Deque_Iterator_Dec_Templ ( Byte , uint8_t )

# define  Deque_ConstIterator_Add_Templ( NAME , TYPE ) \
DequeConstIterator ( NAME ) Deque_ConstIterator_Add ( NAME ) ( DequeConstIterator ( NAME ) const i , ptrdiff_t const d ) { \
  ptrdiff_t const brsi = size_cast_ptrdiff ( DequeBrachDefaultSize ( sizeof  ( TYPE ) ) ) ; \
  ldiv_t  di = ldiv ( d + i . base . index , brsi ) ; \
  if ( di . rem < 0 ) { \
    di .  rem += brsi ; \
    --  di . quot ; \
  } \
  return  ( DequeConstIterator ( NAME ) ) { \
    { . line  = i . base . line + di . quot , \
      . index = longint_cast_uint ( di . rem ) } } ; \
}

Deque_ConstIterator_Add_Templ ( WChar , wchar_t )
Deque_ConstIterator_Add_Templ ( Byte , uint8_t )

# define  Deque_Iterator_Add_Templ( NAME , TYPE ) \
DequeIterator ( NAME ) Deque_Iterator_Add ( NAME ) ( DequeIterator ( NAME ) const i , ptrdiff_t const d ) { \
  ptrdiff_t const brsi = size_cast_ptrdiff ( DequeBrachDefaultSize ( sizeof  ( TYPE ) ) ) ; \
  ldiv_t  di = ldiv ( d + i . index , brsi ) ; \
  if ( di .rem < 0 ) { \
    di .  rem += brsi ; \
    --  di . quot ; \
  } \
  return  ( DequeIterator ( NAME ) ) { \
    . line = i . line + di . quot , \
    . index = longint_cast_uint ( di . rem ) } ; \
}

Deque_Iterator_Add_Templ ( WChar , wchar_t )
Deque_Iterator_Add_Templ ( Byte , uint8_t )

# define  Deque_Erase_Templ(  NAME , TYPE ) \
DequeIterator ( NAME  ) Deque_Erase ( NAME  ) ( Deque ( NAME  ) * const me  , \
  DequeConstIterator  ( NAME  ) const begin , DequeConstIterator  ( NAME  ) const end ) { \
  if  ( ( ( void  * * ) ( begin . base . line ) ==  me -> map ) and \
    ( begin . base . index ==  me -> indexfirst ) ) { \
    if (  ( begin . base . line ) < ( end . base . line ) ) { \
      DequeConstIterator  ( NAME  ) j = begin ; \
      do { \
        free  ( ( void * ) * ( j . base . line ) ) ; \
        ++  ( j . base . line ) ; \
      } while ( j . base . line < end . base . line ) ; \
      memmove ( me -> map , end . base . line ,  \
        longint_cast_size_t ( ( ( me -> map + me -> mapsize ) ) - \
          ( void * * ) ( end . base . line ) ) * sizeof  ( void  * ) ) ; \
      size_t  const newsize = me -> mapsize - \
        longint_cast_size_t ( ( void * * ) ( end . base . line ) - me -> map ) ; \
      if ( newsize ) { \
        void * const ret = realloc ( me -> map , newsize * sizeof ( void * ) ) ; \
        if ( not ret ) { \
          Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Erase : realloc" ;  \
          longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
        } \
        me -> mapsize = newsize ; \
        me -> map = ret ; \
      } else { \
        free  ( me -> map ) ; \
        me -> map = 0 ; \
        me -> mapsize = 0 ; \
      } \
    } \
    me -> indexfirst = end . base . index ; \
    return  ( DequeIterator ( NAME  ) ) { ( void * const * ) ( end . base . line ) , end . base . index } ; \
  } \
  Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & L"Deque_Erase : begin not_eq me . begin" ; \
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ; \
}

Deque_Erase_Templ ( WChar , wchar_t )

# define  DequeVoid_Push_backTempl( NAME , TYPE , CAST ) \
void Deque_Push_back ( NAME ) ( Deque ( NAME ) * const restrict me , TYPE  const wc ) {  \
  size_t  const elt_size  = sizeof  ( TYPE ) ;  \
  if ( me -> mapsize == 0 ) { \
    { void  * * const newmap  = aligned_alloc ( _Alignof ( void  * ) , sizeof  ( void  * ) ) ; \
      if ( not newmap ) { \
        Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_back" ;  \
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
      } \
      me -> map = newmap  ; \
    } \
    me -> map [ 0 ]  = aligned_alloc ( _Alignof ( TYPE ) , DequeBrachDefaultSize ( elt_size  ) * elt_size ) ; \
    if  ( not ( me -> map [ 0 ] ) ) { \
      Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_back" ;  \
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
    } \
    unsigned  int const ind = DequeBrachDefaultSize ( elt_size  ) >> 1  ; \
    CAST ( me -> map [ 0 ] ) [ ind ] = wc ; \
    me -> indexfirst = ind ; \
    me -> indexlast = ind ; \
    me -> mapsize = 1 ; \
  } else  if ( me -> indexlast == DequeBrachDefaultSize ( elt_size ) - 1 ) { \
    { void  * * const newmap  = realloc  ( me -> map , sizeof  ( void  * ) * ( me -> mapsize + 1 ) ) ; \
      if ( not newmap ) { \
        Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_back" ; \
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
      } \
      me -> map = newmap  ; \
    } \
    me -> indexlast = 0 ;  \
    me -> map [ me -> mapsize ]  = aligned_alloc ( _Alignof ( TYPE ) , DequeBrachDefaultSize ( elt_size  ) * elt_size ) ; \
    if  ( not ( me -> map [ me -> mapsize ] ) ) { \
      Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_back" ; \
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
    } \
    CAST ( me -> map [ me -> mapsize ] ) [ me -> indexlast ] = wc ; \
    ++ ( me -> mapsize ) ; \
  } else { \
    ++ ( me -> indexlast )  ; \
    CAST ( me -> map [ me -> mapsize - 1 ] ) [ me -> indexlast ] = wc ; \
  } \
}

DequeVoid_Push_backTempl( WChar , wchar_t , voidp_cast_wcharp )
DequeVoid_Push_backTempl( Byte , uint8_t , voidp_cast_bytep )

# define  DequeVoid_Push_frontTempl( NAME , TYPE , CAST ) \
void Deque_Push_front ( NAME ) ( Deque ( NAME ) * const restrict me , TYPE const wc ) {  \
  if ( me -> mapsize == 0 ) { \
    { void  * * const newmap  = aligned_alloc ( _Alignof ( void  * ) , sizeof  ( void  * ) ) ; \
      if ( not newmap ) { \
        Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_front" ;  \
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
      } \
      me -> map = newmap  ; \
    } \
    size_t  const elt_size  = sizeof  ( TYPE ) ; \
    me -> map [ 0 ]  = aligned_alloc ( _Alignof ( TYPE ) , DequeBrachDefaultSize ( elt_size  ) * elt_size ) ; \
    if  ( not ( me -> map [ 0 ] ) ) { \
      Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_front" ;  \
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
    } \
    unsigned  int const ind = DequeBrachDefaultSize ( elt_size  ) >> 1  ; \
    CAST ( me -> map [ 0 ] ) [ ind ] = wc ; \
    me -> indexfirst = ind ; \
    me -> indexlast = ind ; \
    me -> mapsize = 1 ; \
  } else  if ( me -> indexfirst == 0 ) { \
    { void  * * const newmap  = realloc  ( me -> map , sizeof  ( void  * ) * ( me -> mapsize + 1 ) ) ; \
      if ( not newmap ) { \
        Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_front" ;  \
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
      } \
      memmove ( & ( me -> map [ 1 ] ) , & ( me -> map [ 0 ] ) , sizeof  ( void  * ) * me -> mapsize ) ; \
      me -> map = newmap  ; \
    } \
    size_t  const elt_size  = sizeof  ( TYPE ) ; \
    me -> indexfirst = DequeBrachDefaultSize ( elt_size  ) - 1 ; \
    me -> map [ 0 ]  = aligned_alloc ( _Alignof ( TYPE ) , DequeBrachDefaultSize ( elt_size  ) * elt_size ) ; \
    if  ( not ( me -> map [ 0 ] ) ) { \
      Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Push_front" ;  \
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
    } \
    CAST ( me -> map [ 0 ] ) [ me -> indexfirst ] = wc ; \
    ++ ( me -> mapsize ) ; \
  } else {  \
    -- ( me -> indexfirst )  ;  \
    CAST ( me -> map [ 0 ] ) [ me -> indexfirst ] = wc ; \
  } \
}

DequeVoid_Push_frontTempl( WChar , wchar_t , voidp_cast_wcharp )
DequeVoid_Push_frontTempl( Byte , uint8_t , voidp_cast_bytep )

size_t  DequeVoidSize ( DequeVoid const * const dq , size_t const elemsize ) {
  if ( dq -> mapsize ==  0 )  
    return  0 ;
  return  dq -> indexlast + 1UL - dq -> indexfirst + ( dq -> mapsize - 1UL ) * DequeBrachDefaultSize ( elemsize ) ;
}

bool  DequeVoidEmpty  ( DequeVoid const * const dq  ) {
  return  dq -> mapsize ==  0 ;
}

# define  Deque_InitSizeTempl(  T ) \
void  Deque_InitSize ( T ) ( Deque ( T ) * const me , size_t const s ) { \
  Deque_Init ( T ) ( me ) ; \
  Deque_Resize ( T ) ( me , s ) ; \
}

Deque_InitSizeTempl ( WChar )
Deque_InitSizeTempl ( Byte )

# define  DequeVoid_Pop_backTempl( NAME , TYPE , CAST ) \
TYPE  Deque_Pop_back ( NAME ) ( Deque ( NAME ) * const me ) { \
  if ( me -> mapsize == 0 ) { \
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[]) & L"Deque_Pop_back: deque is empty" ;  \
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ; \
  } \
  TYPE  const res = CAST ( me -> map [ me -> mapsize - 1 ] ) [ me -> indexlast ] ;  \
  if ( me -> mapsize == 1 ) { \
    if ( me -> indexlast == me -> indexfirst ) { \
      free ( me -> map [ 0 ] ) ; \
      Deque_Init ( NAME ) ( me ) ; \
    } else  \
      -- ( me -> indexlast )  ; \
  } else  if ( me -> indexlast == 0 ) { \
    me -> indexlast = DequeBrachDefaultSize ( sizeof  ( TYPE ) ) - 1 ; \
    free ( me -> map [ me -> mapsize - 1 ] ) ; \
    -- ( me -> mapsize ) ; \
    void  * * const newmap  = realloc  ( me -> map , sizeof  ( void  * ) * me -> mapsize ) ; \
    if ( not newmap ) { \
      Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Pop_back" ; \
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
    } \
    me -> map = newmap  ; \
  } else \
    -- ( me -> indexlast )  ; \
  return  res ; \
}

DequeVoid_Pop_backTempl( WChar , wchar_t , voidp_cast_wcharp )
DequeVoid_Pop_backTempl( Byte , uint8_t , voidp_cast_bytep )

# define  DequeVoid_Pop_frontTempl( NAME , TYPE , CAST ) \
TYPE  Deque_Pop_front ( NAME ) ( Deque ( NAME ) * const me ) { \
  if ( me -> mapsize == 0 ) { \
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[]) & L"Deque_Pop_front: deque is empty" ;  \
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ; \
  } \
  TYPE  const res = CAST ( me -> map [ 0 ] ) [ me -> indexfirst ] ; \
  if ( me -> mapsize == 1 ) { \
    if ( me -> indexlast == me -> indexfirst ) { \
      free ( me -> map [ 0 ] ) ; \
      Deque_Init ( NAME ) ( me ) ; \
    } else  \
      ++ ( me -> indexfirst ) ; \
  } else  if ( me -> indexfirst == DequeBrachDefaultSize ( sizeof  ( TYPE ) ) - 1 ) { \
    me -> indexfirst = 0 ; \
    free ( me -> map [ 0 ] ) ; \
    -- ( me -> mapsize ) ; \
    memmove ( & ( me -> map [ 0 ] ) , & ( me -> map [ 1 ] ) , sizeof  ( void  * ) * me -> mapsize ) ; \
    void  * * const newmap  = realloc  ( me -> map , sizeof  ( void  * ) * me -> mapsize ) ; \
    if ( not newmap ) { \
      Lisp2DExcOutOfMemoryGlobal . value = L"Deque_Pop_front" ;  \
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ; \
    } \
    me -> map = newmap  ; \
  } else \
    ++  ( me -> indexfirst ) ; \
  return  res ; \
}

DequeVoid_Pop_frontTempl( WChar , wchar_t , voidp_cast_wcharp )
DequeVoid_Pop_frontTempl( Byte , uint8_t , voidp_cast_bytep )

# define  Deque_ConstIterator_NotEqTempl(  T ) \
bool  Deque_ConstIterator_NotEq ( T ) ( DequeConstIterator ( T ) const i , DequeConstIterator ( T ) const j ) { \
  return  ( i . base . index not_eq j . base . index )  or ( ( i . base . line ) not_eq ( j . base . line ) ) ; \
}

Deque_ConstIterator_NotEqTempl ( WChar )
Deque_ConstIterator_NotEqTempl ( Byte )

# define  Deque_ConstIterator_NotEqIteratorTempl(  T ) \
bool  Deque_ConstIterator_NotEqIterator ( T ) ( DequeConstIterator ( T ) const i , DequeIterator ( T ) const j ) { \
  return  ( i . base . index not_eq j . index )  or ( ( i . base . line ) not_eq j . line ) ; \
}

Deque_ConstIterator_NotEqIteratorTempl ( WChar )
Deque_ConstIterator_NotEqIteratorTempl ( Byte )

Deque_SizeTempl ( WChar , wchar_t )
Deque_SizeTempl ( Byte , uint8_t )

void  Deque_InitCopy ( Byte ) ( Deque ( Byte ) * const me , Deque ( Byte ) const * const from ) {
  Deque_Void_InitCopy ( me , from , sizeof ( uint8_t ) , _Alignof ( uint8_t ) ) ;
}

void  Deque_Destr ( Byte ) ( Deque ( Byte ) * const me ) {
  DequeVoidDestr  ( me ) ;
}

void  Deque_Resize  ( Byte ) ( Deque ( Byte ) * const dq , size_t const count ) {
  DequeVoidResize ( dq , sizeof ( uint8_t ) , count , _Alignof ( uint8_t ) ) ;
}

void  Deque_Set ( Byte ) ( Deque ( Byte ) * const me , Deque ( Byte ) const * const from ) {
  Deque_Void_Set ( me , from , sizeof ( uint8_t ) , _Alignof ( uint8_t ) ) ;
}

void  Deque_Destr ( WChar ) ( Deque ( WChar ) * const me ) {
  DequeVoidDestr  ( me ) ;
}

void  Deque_Resize  ( WChar ) ( Deque ( WChar ) * const dq , size_t const count ) {
  DequeVoidResize ( dq , sizeof ( wchar_t ) , count , _Alignof ( wchar_t ) ) ;
}

# include "castb.h"
// 1 .. 840
inline  unsigned  int DequeBrachDefaultSize ( size_t  const s ) {
  if  ( s < DequeBrachDefaultSizeBytes  )
    return  DequeBrachDefaultSizeBytes  / size_cast_uint ( s ) ;
  return  1 ;
}

inline  void  const * const * constcastvoidpp ( void  * * const p ) {
  return  ( void  const * const * ) p ;
}
