// Lisp2D - Dir type
// ©2008-23 by Alex Glebe
# include "define.h"
# include "dir/private.h"
# include "dir/inline.h"

# ifdef  LISP2D_CONSTRSTAT

_Atomic ( size_t ) PLODir_constrstat  ;

size_t  PLODir_constrstatValue ( void  ) {
  return PLODir_constrstat  ;
}

void  PLODir_constrstatInc ( void  ) {
  ++ PLODir_constrstat  ;
}

# endif // LISP2D_CONSTRSTAT

# include "boolean.h"
# include <errno.h>
# include "errorstra.h"
# include "plostream/public.h"

void  PLODir_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool  _ __attribute__ ((unused)) , Vector  ( StackPLOP )  * _2 __attribute__ ((unused)) ) {
  PLOStreamPushWideCharArray  ( s , ( PArray_wchar_c ) {
      ( ( PLODirPub_cast_PLODirPriConst ( ParaLispObjectToPLODirConst ( me  ) ) -> dir ) ?
        ( wchar_t const  ( * ) [ ] ) & L"Dir" :
        ( wchar_t const ( * ) [ ] ) & L"Dir-null" ) } ) ;
}

ParaLispObject  * PLODir_Openp  ( PLODirPub * const mep ) {
  PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
  if ( me -> dir )  
    return  & mep->plo;
  return  &plofalse.plo;
}

# include "fraction/public.h"

void PLODir_Seek  ( PLODirPub * const mep , FractionPub const * const ip )  {
  PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
  if  ( me -> dir )
    seekdir ( me -> dir , FractionLValue ( ip ) ) ;
}

# include "nilc.h"

ParaLispObject  * PLODir_Rewind ( PLODirPub * const mep ) {
  PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
  if  (  me -> dir  ) {
    rewinddir ( me -> dir ) ;
    return  & mep  ->  plo ;
  }
  return  & plonil  . plo ;
}

int isBADPLODir_Close  ( PLODirPub * const mep ){
  PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
  if(me -> dir){
    int const  res = closedir(me -> dir) ;
    me -> dir = 0 ;
    return  res ;
  }
  return  1 ;
}

void  PLODir_Open(PLODirPub * const mep){
  PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
  me -> dir = opendir(".");
}

void  PLODir_Destr(PLODirPri * const me ){
  if(me -> dir){
    if(closedir(me -> dir))
      switch(errno){
      case  EBADF:
        ErrorStra ( ( PArray_wchar_c ) { (wchar_t const (*)[])&
              L"~PLODir():closedir:Invalid directory stream descriptor"});
        break;
      default:  
        ErrorStra ( ( PArray_wchar_c ) {(wchar_t const (*)[])&L"~PLODir():closedir:?"});
      }
  }
  ParaLispObject_staticDestr(&me->pub.plo);
}

# include "symbolc.h"

ParaLispObject  * PLODir_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  & plosdir . plo ;
}

ParaLispObject  * PLODir_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassdefdir  ;
}

ParaLispObject  * PLODir_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassparentsdir ;
}

# include "sharedc.h"
# include "shared/inline.h"
# include "cons/inline.h"
# include "stcl/public.h"
# include "stcl/inline.h"

TaskForThread Lisp2Dbimdirswap2 ( StackClusterPrivate * const stclpp  ) {
  { ParaLispObject  * const xp =  CCAR ( StackClusterPrivate_ArgTop(stclpp ));
    if(IsNotDir( stclpp ,xp))
      goto  Exit  ;
    if(IsConst(  stclpp ,xp))
      goto  Exit  ;
    PLODirPub * const mep = ParaLispObjectToPLODir(StackClusterPrivate_ThisTop(stclpp ));
    PLODir_Swap (  mep  , ParaLispObjectToPLODir  (  xp ) ) ;
    StackClusterPrivate_SetResult(stclpp ,&mep->plo);
  }
Exit  :
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimnildirp2(StackClusterPrivate * const stclpp){
  ParaLispObject  * const xp =  CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  StackClusterPrivate_SetResult(stclpp ,(Dirp(xp)  ? xp : &plofalse.plo));
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimdirclose2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const mep = StackClusterPrivate_ThisTop(stclpp );
  if(isBADPLODir_Close(ParaLispObjectToPLODir(mep))){
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    goto Exit  ;
  }
  StackClusterPrivate_SetResult(stclpp ,mep);
Exit  :
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimdirread2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult ( stclpp ,
    PLODir_Read(ParaLispObjectToPLODir(StackClusterPrivate_ThisTop(stclpp ))));
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimdiropenp2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult ( stclpp ,
    PLODir_Openp(ParaLispObjectToPLODir(StackClusterPrivate_ThisTop(stclpp ))));
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimdirrewind2(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp , PLODir_Rewind(ParaLispObjectToPLODir(StackClusterPrivate_ThisTop(stclpp ))));
  return  TaskForThread_Analyse;
}

# include "newobject.h"

bool  IsNotDir ( StackClusterPrivate  * const stclp , ParaLispObject * const xp ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp ,
    . typetype  = TypeGlobalListMemberPLODirPri ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
}

void  PLODir_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  , Vector ( StackPLOP )  * const h ) {
  PLODir_staticCoutToInterpret ( me , s , flag2pprinted  , h ) ;
}

# include "stringbim.h"
# include "string/public.h"

TaskForThread Lisp2Dbimdirdir2  ( StackClusterPrivate  * const stclpp ) {
  PLODirPub  * const mep = ParaLispObjectToPLODir ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop ( stclpp )  ;
  if  ( CConsp  ( arglistp  ) ) {
    ParaLispObject  * const xp = ClassCons_First(&ParaLispObjectToPLOCons  ( arglistp  )->cc  ) ;
    
// ! to add get from stream
// and call : DIR *fdopendir(int fd);
// ! (dir dir stream) - eat the stream
// stream.fd=f -> stream.fd=0
    
    if  ( IsNotString (  stclpp , xp  ) ) 
      goto  Exit  ;
    PLODir_OpenPLOString  ( mep  , ParaLispObjectToPLOString ( xp ) , stclpp  ) ;
  } else
    PLODir_Open ( mep ) ;
  StackClusterPrivate_SetResult ( stclpp , & mep ->  plo ) ;
Exit  :
  return  TaskForThread_Analyse ;
}

TaskForThread Lisp2Dbimdiropen2(StackClusterPrivate * const stclpp){
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotString(  stclpp ,xp))
    goto Exit  ;
  { PLODirPub * const  di = ParaLispObjectToPLODir  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
    PLODir_OpenPLOString  ( di  , ParaLispObjectToPLOString ( xp ) , stclpp  ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & di -> plo ) ;
  }
Exit  :
  return  TaskForThread_Analyse;
}
  
// !! to return symbol
void  PLODir_OpenCharS(PLODirPub * const mep , char const * const s , StackClusterPrivate  * const stclp ) {
  PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
  wchar_t const * ws  ;
  if(me -> dir){
    if(closedir(me -> dir)){
      switch(errno){
      case  EBADF:
        ws = L"PLODir::Open(char const *):closedir:"
          L"Invalid directory stream descriptor"  ;
        goto  Exit  ;
      default:  
        ws = L"PLODir::Open(char const *):closedir:?";
      }
      goto Exit;
    }
  }
  me -> dir  = opendir(s) ;
  if(me -> dir) 
    return;
  switch(errno){
  case  EACCES:
    ws  = L"PLODir::Open(char const *):opendir:Permission denied";
    break;
  case  EBADF:
    ws = L"PLODir::Open(char const *):opendir:"
      L"is not a valid file descriptor opened for reading"  ;
    break;
  case  EMFILE:
    ws = L"PLODir::Open(char const *):opendir:"
      L"Too many file descriptors in use by process"  ;
    break;
  case  ENFILE:
    ws = L"PLODir::Open(char const *):opendir:"
      L"Too many files are currently open in the system"  ;
    break;
  case  ENOENT:
    ws = L"PLODir::Open(char const *):opendir:"
      L"Directory does not exist, or name is an empty string" ;
    break;
  case  ENOMEM:
    ws = L"PLODir::Open(char const *):opendir:"
      L"Insufficient memory to complete the operation"  ;
    break;
  case  ENOTDIR:
    ws = L"PLODir::Open(char const *):opendir:is not a directory" ;
    break;
  default:
    ws = L"PLODir::Open(char const *):opendir:?"  ;
  }
Exit  :
  StackClusterPrivate_ExceptionSet  (stclp ,& PLOString_NewCopy ( ws  )->plo ) ;
}
  
# include "plocomplex/public.h"
# include "complex/inline.h"

TaskForThread Lisp2Dbimdirseek2(StackClusterPrivate * const stclpp){
  ParaLispObject  * const ip = CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,ip)) 
    goto  Exit  ;
  if(not  FractionIntegerP ( Complex_CRe(PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst(ip))))){
    ParaLispObject  * list [ ] = { ip,& PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet( stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    goto  Exit  ; 
  }
  { PLODirPub * const di = ParaLispObjectToPLODir  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
    PLODir_Seek ( di  , Complex_CRe  (PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst  ( ip )) ) ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & ( di  ->  plo ) ) ; 
  }
Exit  :
  return  TaskForThread_Analyse ; 
}

TaskForThread Lisp2Dbimdirtell2(StackClusterPrivate * const stclpp) {
  ParaLispObject  * res ;
  { DIR * const mydirp2  = PLODir_DirPointerValue ( ParaLispObjectToPLODir  ( StackClusterPrivate_ThisTop ( stclpp  ) ) ) ;
    if  ( mydirp2 ) {
      res = & NewComplexBareExc ( ) ->  plo ;
      Complex_SetLInt ( PLOComplexToComplex ( ParaLispObjectToPLOComplex ( res )  ) , telldir ( mydirp2 ) ) ;
    } else 
      res = & plonil  . plo ;
  }
  StackClusterPrivate_SetResult ( stclpp , res ) ;
  return  TaskForThread_Analyse ;
}

# include "stack/public.h"

void  PLODir_OpenPLOString  ( PLODirPub * const mep , PLOStringPub  * const sp , StackClusterPrivate  * const stclp )  {
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if ( isBADReadWCharToCharString  ( PLOString_CStr ( sp ) , strtmpc ) ) {
    BadString ( stclp , sp ) ;
    return ;
  }
  PLODir_OpenCharS ( mep , StringCStr  ( Char  ) ( strtmpc ) , stclp ) ;
  Lisp2D_StackFree  ( oldstack_a ) ;
}

ParaLispObject  * PLODir_Read ( PLODirPub * const mep){
  PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
  if(me -> dir){
    struct  dirent  * const result = readdir(me -> dir);
    if  ( result  ) {
      ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
      String  ( WideChar ) * const wan2c = StackNew_StringWideChar ( ) ;
      String_Init ( WideChar  ) ( wan2c , true ) ;
      { String  ( Char ) * const strcres2c = StackNew_StringChar ( ) ;
        StringConstrCStr ( Char  ) ( strcres2c , result->d_name , true ) ;
        if ( isBADReadCStringToWString2 ( strcres2c , wan2c ) ) {
          Lisp2D_StackFree  ( oldstack ) ;
          ErrorStra((PArray_wchar_c){(wchar_t const (*)[])& L"PLODir::Read:BadString"});
          goto  Exit  ;
        }
      } // strcres2          
      PLOStringPub * const  resp = PLOString_NewMoveString ( wan2c ) ;
      Lisp2D_StackFree  ( oldstack ) ;
      return  & resp->plo;
    } // result
  } // dir
Exit  :
  return  & plonil . plo ;
}

void PLODir_staticDestr  ( ParaLispObject * const me ) {
  PLODir_Destr ( PLODirPub_cast_PLODirPri ( ParaLispObjectToPLODir ( me  ) ) ) ;
}

void  PLODir_Swap  ( PLODirPub * const mep , PLODirPub * const xpp ) {
  if  ( mep not_eq xpp ) {
    PLODirPri * const me = PLODirPub_cast_PLODirPri ( mep ) ;
    PLODirPri * const xp = PLODirPub_cast_PLODirPri ( xpp ) ;
    DIR * const tmp = me -> dir ;
    me -> dir = xp -> dir ;
    xp -> dir = tmp ;
  }
}

void  PLODir_Init ( PLODirPri * const me ) {
  me -> dir = 0 ;
  { VirtualTypePointer const vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLODirPri } ;
    ParaLispObject_InitVTP  ( & me  -> pub . plo , vtp ) ;
  }
# ifdef  LISP2D_CONSTRSTAT
  PLODir_constrstatInc (  ) ;
# endif
}
# include "widestring/inline.h"
