// Lisp2D window: draw-line
// ©2007-23 by Alex Glebe
# ifndef  LISP2D_DONTUSEX11
# include <math.h>
# include "window.h"
# include "drawline.h"
# include "minmaxint.h"

// (x1*(x-leftx)+y1*(y-lefty))/dist^2

// (l*x1+leftx-x)^2+(l*y1+lefty-y)^2

// !! to find time of slow members
static  long  double  const Lisp2DPI  = 
# ifdef M_PIl
  M_PIl
# else
  3.1415926535897932384626433832795029L
# endif
  ;

// Sqrt[Pi]
static  long  double  const pisquareroot  = 1.7724538509055160272981674833411452L ;
  
// 1 / Sqrt[Pi]
static  long  double  const pixelradius = 0.56418958354775628694807945156077259L  ;

// 1 / Pi
static  long  double  const pireverse = 0.31830988618379067153776752674502872L  ;
  
static  long  double  const maxcolor  = 255.0L  ;

# include "sharedc.h"
# include "shared/inline.h"
# include "string/public.h"
# include "cons/inline.h"
# include "complex/inline.h"
# include "plocomplex/public.h"
# include "nilc.h"
# include "stcl/public.h"
# include "stcl/inline.h"
# include "lightmem/public.h"
# include "window/public.h"
# include "cast.h"
# include "castb.h"

// (win draw-line col|(r g b)  radius x y x y ... x y)
TaskForThread bimwindowdrawline ( StackClusterPrivate  * const stclpp ) {
  // S=pi*r^2==1
  // sqrt(pi)
  // r=1/sqrt(pi)
  // 1/pi
  PLOWindow * const plowp=ParaLispObjectToPLOWindow( StackClusterPrivate_ThisTop(stclpp));
  ParaLispObject  const * const arglistp=StackClusterPrivate_ArgTop(stclpp);
  ParaLispObject  * const radiusp= CCAR ( CCDR ( arglistp));
  if(IsNotNumber(  stclpp,radiusp))
    return  TaskForThread_Analyse;
  if  ( ULongIs0  ( FractionCNump  ( Complex_CRe(PLOComplexToComplexConst( ParaLispObjectToPLOComplex(radiusp))))) ) {
    StackClusterPrivate_SetResult(stclpp,PLOWindow_plo(plowp));
    return  TaskForThread_Analyse;
  }
  // radius<>0
  long  double  const lra = FractionLDValueConst  ( Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex(radiusp))) );
  long  double  const  radius0=fabsl(lra);
  // radius>0
  ParaLispObject  * const colororlistp= CCAR ( arglistp);
  long  double  drawcolorr;
  long  double  drawcolorg;
  long  double  drawcolorb;
  if(CConsp(colororlistp)) {
    if(IsNotNumber(  stclpp, CCAR ( colororlistp))) 
      return  TaskForThread_Analyse;
    if(IsNotNumber(  stclpp, CCAR ( CCDR ( colororlistp))))
      return  TaskForThread_Analyse;
    if(IsNotNumber(  stclpp,
      CCAR  ( CCDR  ( CCDR  ( colororlistp))))) 
      return  TaskForThread_Analyse;
    drawcolorr = FractionLDValueConst(Complex_CRe( PLOComplexToComplexConst( ParaLispObjectToPLOComplex( CCAR  ( colororlistp)))));
    if(drawcolorr <=  0.0L) 
      drawcolorr  = 0.0L;
    else if(drawcolorr >=  1.0L)
      drawcolorr  = maxcolor;
    else
      drawcolorr  *= maxcolor;
    drawcolorg = FractionLDValueConst (Complex_CRe( PLOComplexToComplexConst(
      ParaLispObjectToPLOComplex( CCAR  ( CCDR  ( colororlistp))))));
    if(drawcolorg <=  0.0L) 
      drawcolorg  = 0.0L;
    else if(drawcolorg >=  1.0L)
      drawcolorg  = maxcolor;
    else
      drawcolorg  *= maxcolor;
    drawcolorb = FractionLDValueConst(Complex_CRe( PLOComplexToComplexConst(
      ParaLispObjectToPLOComplex( CCAR  ( CCDR  ( CCDR  ( colororlistp)))))) );
    if(drawcolorb <=  0.0L)
      drawcolorb  = 0.0L;
    else if(drawcolorb >=  1.0L)
      drawcolorb  = maxcolor;
    else
      drawcolorb  *= maxcolor;
  } else if(Numberp(colororlistp)){
    long  double const lx = FractionLDValueConst(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex( colororlistp))));
    if(lx <=  0.0L) {
      drawcolorr  = 0.0L  ;
      drawcolorg  = 0.0L  ;
      drawcolorb  = 0.0L  ;
    } else  if(lx >=  1.0L) {
      drawcolorr  = maxcolor  ;
      drawcolorg  = maxcolor  ;
      drawcolorb  = maxcolor  ;
    } else  {
      long  double  const tmp = lx * maxcolor ;
      drawcolorr  = tmp ;
      drawcolorg  = tmp ;
      drawcolorb  = tmp ;
    }
  } else{
    ParaLispObject  * list [ ] = { colororlistp,& PLOString_NewCopy(
      L" is not number nor list")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp,ListOfPLO((PArrayPLOP){ (ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse ;
  }
  ParaLispObject  * j;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) 
    StackClusterPrivate_Pop(stclpp ,&j);
  else 
    j = CCDR  ( CCDR  ( arglistp));
  { unsigned  int tasks = maxListItemProcessing ;
    for ( ; ( tasks  > 0  ) and CConsp ( CCDR ( CCDR ( CCDR ( j ) ) ) ) ;
      j = CCDR ( CCDR (  j ) ) , --  tasks){
      bool  flage=false;
      long  double    leftx = 0;
      long  double    lefty = 0;
      long  double    rightx  = 0;
      long  double    righty  = 0;
      { ParaLispObject  const * j2=j;
        ParaLispObject  *  j2CAR2=  CCAR  ( j2);
        if(IsNotNumber(  stclpp , j2CAR2))
          flage = true;
        else {
          leftx = FractionLDValueConst(
            Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex( j2CAR2))));
          j2  = CCDR (  j2);
          j2CAR2  = CCAR ( j2);
          if(IsNotNumber(  stclpp , j2CAR2))
            flage = true;
          else {
            lefty = FractionLDValueConst(
              Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex( j2CAR2))));
            j2  = CCDR (  j2);
            j2CAR2  = CCAR ( j2);
            if(IsNotNumber(  stclpp , j2CAR2))
              flage = true;
            else {
              rightx = FractionLDValueConst(
                Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex( j2CAR2))));
              j2  = CCDR (  j2);
              j2CAR2  = CCAR ( j2);
              if(IsNotNumber(  stclpp , j2CAR2))
                flage = true;
              else {
                righty = FractionLDValueConst(
                  Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex( j2CAR2))));
                if(rightx < leftx){
                  { long  double  const tmp = rightx  ;
                    rightx  = leftx ;
                    leftx = tmp ;
                  }
                  { long  double  const tmp = righty  ;
                    righty  = lefty ;
                    lefty = tmp ;
                  }
                }
              }
            }
          }
        }
      }
      if ( flage )  {
        PLOWindow_LockFlush ( plowp ) ;
        StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
        return  TaskForThread_Analyse ;
      }
// to make: special methods for INF && 0/0
      // leftx <= rightx
      // unsafe
      if((fabsl(rightx - leftx)  > (1.0L / 256.0L))  or (fabsl(righty  - lefty)  > (1.0L / 256.0L))){
        bool  flagxyswap;
        // for start point calculation: protection from division by 0
        if((rightx  - leftx)  < fabsl(righty - lefty)){
            { long  double  const tmp = leftx ;
              leftx  = lefty ;
              lefty = tmp ;
            }
            { long  double  const tmp = rightx  ;
              rightx  = righty ;
              righty = tmp ;
            }
            flagxyswap  = true;
            if(rightx < leftx){
              { long  double  const tmp = rightx ;
                rightx  = leftx ;
                leftx = tmp ;
              }
              { long  double  const tmp = righty  ;
                righty  = lefty ;
                lefty = tmp ;
              }
            }
        } else
          flagxyswap  = false;
        // leftx may be > rightx
          long  double  const   x1=rightx - leftx;
          long  double  const   y1=righty - lefty;
          // all coordinates around (leftx,lefty)
          long  double  const   length2=(x1 * x1) + (y1 * y1);
          // length2>0
          long  double  const   D=sqrtl(length2);
          //D=sqrtl(D);
          // d/D
          long  double  const   dD=radius0  / D;
          //dD/=D;
          long  double  const   radiusx=fabsl(dD * y1);
          // radiusx>=0
          // xl>=0
          long  double  const   xr=ceill(rightx + radiusx);
          long  double  const   radiusy=fabsl(dD * x1);
          // radiusy>=0
          long  double  const   yu=floorl(((y1  > 0)  ? lefty : righty) - radiusy);
          // yu>=0
          long  double  const   yd=ceill(((y1 > 0)  ? righty  : lefty)  + radiusy);
          long  double  const ldzero = 0.0L ;
          long  double  const tmp_  = floorl ( * Lisp2DMaxLDoubleCP  ( & xr  , & ldzero  ) ) ;
          size_t  uintxr  = ldouble_cast_size ( tmp_ ) ;
          if  ( flagxyswap  )
            uintxr  = Lisp2DMinSize ( uintxr  , PLOWindow_VectorHeight  ( plowp ) ) ;
          else
            uintxr  = Lisp2DMinSize  ( uintxr  , PLOWindow_VectorWidth ( plowp ) ) ;
          long  double  const _xl1 = floorl ( leftx  - radiusx  )  ;
          size_t  const   xl2 = ldouble_cast_size ( * Lisp2DMaxLDoubleCP  ( & _xl1 , &  ldzero ) ) ;
        for ( size_t  ux  = xl2 ; ux < uintxr ; ++ ux ) {
            // lambda==0 -> uy=lefty-(ux+0.5-leftx)*x1/y1-0.5
            // lambda==1 -> uy=(length2-(ux+0.5-leftx)*x1)/y1+lefty-0.5
// !! to calc starting uy and ending - VERY BIG TIME
            // uy=((-rD+ux*y1)/x1 .. (+rD+ux*y1)/x1)
            long  double  const   tmpyux  = y1  * ( ux - leftx  ) ;
            long  double  const   tmpDr=D  * radius0;
            long  double  const   uyup=((tmpyux - tmpDr) / x1) + lefty;
            long  double  const   uydw=((tmpyux + tmpDr) / x1) + lefty;
            size_t  const   yu2=((yu  < 0)  ? 0 : ldouble_cast_size(yu));
            size_t  uy  = yu2 ;
            if  ( uyup > uy ) {
              long  double  const tmp = floorl(uyup)  ;
              uy = ldouble_cast_size(tmp) ;
            }
            unsigned  int uylimit;
            { long  double  const tmp_2 = ceill  ( yd  )  ;
              long  double  const tmp_3 = ceill(uydw) ;
              int  const   uylimit0 = Lisp2DMinInt  ( ldouble_cast_int ( tmp_2 ) , ldouble_cast_int(tmp_3));
              if(uylimit0 < 0)
                uylimit = 0;
              else
                uylimit = int_cast_uint(uylimit0);
            }
            if  ( flagxyswap  )
              uylimit = Lisp2DMinUInt ( uylimit , size_cast_uint ( PLOWindow_VectorWidth ( plowp ) ) ) ;
            else 
              uylimit = Lisp2DMinUInt  ( uylimit , size_cast_uint ( PLOWindow_VectorHeight  ( plowp ) ) ) ;
          for ( ; uy <=  uylimit  ; ++  uy  ) {
              long  double  const  fx = size_cast_longdouble ( ux  ) + 0.5L  ;
              long  double  const  fy = size_cast_longdouble (  uy  ) + 0.5L  ;
              // temporary center of pixel
              // (x1*(x-leftx)+y1*(y-lefty))/dist^2
              long  double  const   lambda00=(((fx - leftx)  * x1)  + ((fy  - lefty)  * y1))  / length2;
              //lambda10
              //lambda01
              //lambda11
            if((lambda00  > 0)  and (lambda00 < 1)){
                long  double  distance200;
                //(x,y,leftx,lefty,x1,y1,lambda,res)
                // (l*x1+leftx-x)^2+(l*y1+lefty-y)^2
                { long  double    x2=((x1 * lambda00) + leftx)  - fx;
                  distance200 = (x2 * x2);
                  x2  = (((y1 * lambda00) + lefty)  - fy);
                  distance200 +=  (x2 * x2);
                }
                // rpixel=1/sqrt(pi) S=1
                // distance may be radius-rpixel .. radius+rpixel
                long  double  const  distance=sqrtl(distance200);
              if(distance >=  (radius0  - pixelradius)){
                if(distance <=  (radius0  + pixelradius)){
                    long  double  const  x=radius0 - distance;
                    // S=2*Integral(sqrt(rpixel^2-y^2),y=-rpixel..x)
                    // S=x*sqrt(1/pi-x^2)+arcsin(x*sqrt(pi))/pi+1/2
                    long  double  const   s=((x * sqrtl(pireverse - (x  * x)))  + (asinl(x  * pisquareroot) / Lisp2DPI))  + 0.5L;
                    // S(x) ≈ 1/2 + 2*x/sqrt(Pi)-sqrt(Pi)/3*(x^3) =
                    //      = 1/2 + x * (6 - Pi * (x^2)) / 3 / sqrt(Pi)
                    long  double  const   s2=1.0L - s;
                    // s=0..1
                    tUCharRGB uc  ;
                    UIntXY  ab = {
                      size_cast_uint ( flagxyswap ? uy  : ux ) ,
                      size_cast_uint ( flagxyswap ? ux  : uy ) } ;
                    if  ( PLOWindow_isBADColorsWithSizeCheck  ( plowp  , ab , & uc  ) )
                      return  TaskForThread_Analyse;
                    long  double  oldcolr = Lisp2DmemorytolightLDouble ( uchar_cast_ldouble ( uc . r ) ) ;
                    long  double  oldcolg = Lisp2DmemorytolightLDouble ( uchar_cast_ldouble ( uc . g ) ) ;
                    long  double  oldcolb = Lisp2DmemorytolightLDouble ( uchar_cast_ldouble ( uc . b ) ) ;
                    long  double  newcolr ;
                    long  double  newcolg ;
                    long  double  newcolb ;
                    { long  double  tmpr  = Lisp2DmemorytolightLDouble (  drawcolorr  ) ;
                      newcolr = ((oldcolr  * s2) + (tmpr * s));
                      tmpr = Lisp2DmemorytolightLDouble (  drawcolorg  ) ;
                      newcolg = ((oldcolg  * s2) + (tmpr * s));
                      tmpr = Lisp2DmemorytolightLDouble ( drawcolorb  ) ;
                      newcolb = ((oldcolb  * s2) + (tmpr * s))  ;
                    } // tmpr
                    int   const  ix2  = size_cast_int ( flagxyswap ? uy  : ux ) ;
                    int   const  iy2  = size_cast_int ( flagxyswap  ? ux  : uy  ) ;
                    uint8_t const ltmr  = Lisp2DlighttomemoryLDouble (  newcolr )  ;
                    uint8_t const ltmg  = Lisp2DlighttomemoryLDouble (  newcolg )  ;
                    uint8_t const ltmb  = Lisp2DlighttomemoryLDouble (  newcolb )  ;
                    PLOWindow_DrawPoint ( plowp  , (tUCharRGB) { ltmr  , ltmg  , ltmb  } , (IntXY) { ix2 , iy2 } ) ;
                } // if
              } else {
                  int const ix2 = size_cast_int ( flagxyswap ? uy  : ux ) ;
                  int const iy2 = size_cast_int ( flagxyswap  ? ux  : uy  ) ;
                  // inside
                  // !!! to write for thin lines
                  // ! 82.3% of time
                  long  double  const tmp_r = floorl(drawcolorr + 0.5L) ;
                  unsigned  char  const   ltmr=ldouble_cast_uchar( tmp_r);
                  long  double  const tmp_g = floorl(drawcolorg + 0.5L) ;
                  unsigned  char  const   ltmg=ldouble_cast_uchar( tmp_g);
                  long  double  const tmp_b = floorl(drawcolorb + 0.5L) ;
                  unsigned  char  const   ltmb=ldouble_cast_uchar( tmp_b);
                  PLOWindow_DrawPoint ( plowp  , (tUCharRGB) { ltmr  , ltmg  , ltmb  } , (IntXY) { ix2 , iy2 } ) ;
              } // else
              // !! to make soft edges
            } // if
          } // for
        } // for
      } //  length2>0
      //  not flage
    } // for
  } // tasks
  PLOWindow_LockFlush(plowp);
  if(CConsp(  CCDR  ( CCDR  ( CCDR  (  j))))){
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    StackClusterPrivate_Push(stclpp , j);
    return  TaskForThread_Analyse;
  }
  StackClusterPrivate_SetResult(stclpp ,PLOWindow_plo(plowp));
  return  TaskForThread_Analyse ;
}

# include "window/inline.h"

# endif
