// Lisp2D window: draw-square
// ©2007-23 by Alex Glebe
# ifndef  LISP2D_DONTUSEX11
# include <math.h>
# include "window.h"
# include "drawsquare.h"
# include "minmaxint.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "cast.h"
# include "window/public.h"

# define  s_PLOWindow_DrawSquareC Lisp2D_s_PLOWindow_DrawSquareC
# define  tPLOWindow_DrawSquareC  Lisp2D_tPLOWindow_DrawSquareC
typedef struct  s_PLOWindow_DrawSquareC t_PLOWindow_DrawSquareC  ;

struct  s_PLOWindow_DrawSquareC {
  PLOWindow_pri * const me ;
  ParaLispObject  * const colsp  ;
  IntXY const xy  ;
  IntXY const wh  ;
  StackClusterPrivate  * const stclpp ;
} ;

// !! to find time of slow members
// ! Int -> unsigned int
# define  PLOWindow_DrawSquareC  Lisp2D_PLOWindow_DrawSquareC2yXO
// IntXY wh -> %rcx , %ecx one int , %ecx>>32 second int
// IntXY xy -> %rdx 
static  void  PLOWindow_DrawSquareC  ( t_PLOWindow_DrawSquareC const arg ) {
  if  ( ( arg . wh  . x <=  0 ) or  ( arg . wh  . y <=  0 ) )
    return  ;
  tUCharRGB ulcols ;
  if(isBADColorsCharsFromObjectOrList(arg . colsp,&ulcols,arg . stclpp))
    return;
  unsigned long int ulc;
  if(isBADCreateColor(ulcols,PLOWindow_ColorDefaultDepth( arg . me),&ulc))
    return;  
  ptrdiff_t const   x1  = Lisp2DMinPtrDiff ( size_cast_ptrdiff ( PLOWindow_VectorWidth  ( &arg . me ->pub ) ) ,
    ( arg . xy  . x  + arg . wh . x ) ) ;
  ptrdiff_t const   y1  = Lisp2DMinPtrDiff ( size_cast_ptrdiff (  PLOWindow_VectorHeight  ( &arg . me->pub ) ) ,
    ( arg . xy  . y + arg . wh  . y ) ) ;
  int const   ix0 = Lisp2DMaxInt  ( 0 , arg . xy  . x ) ;
  int const   yi0 = Lisp2DMaxInt  ( 0 , arg . xy  . y ) ;
  { int xi  = ix0 ;
    for(;xi < x1;++ xi){
      unsigned  long  int * j = (VectorBegin(UnsLongInt)(&arg . me->cmatrixpixel ) + (int_cast_size(xi) * arg . me->matrixsizeh)) + yi0;
      int yi=yi0;
      while(yi  < y1){
        (*  j)  = ulc ;
        ++  j;
        ++  yi;
      }
    }
  }
  if(arg . me->opened){
    unsigned  int const tmpx  = ptrdiff_cast_uint (x1 - ix0);
    unsigned  int const tmpdy = ptrdiff_cast_uint (y1 - yi0);
    XLockDisplay(arg . me->dpy);
    XSetForeground(arg . me->dpy,arg . me->gc,ulc );
    XFillRectangle(arg . me->dpy,arg . me->win,arg . me->gc,ix0,yi0,tmpx,tmpdy);
    XUnlockDisplay(arg . me->dpy ) ;
  }
}
  
# include "cons/inline.h"

# define  PLOWindow_DrawSquare  Lisp2d_PLOWindow_DrawSquarevavT
static  void  PLOWindow_DrawSquare  ( PLOWindow_pri * , aLisp2DDrawSquare2 ) ;

# include "complex/inline.h"
# include "plocomplex/public.h"
# include "stcl/public.h"
# include "stcl/inline.h"
# include "window/private.h"

// (win draw-square col|(r g b) x y w h ... x y w h)
TaskForThread bimdrawsquare(StackClusterPrivate  * const stclpp){
  PLOWindow * const plowp=ParaLispObjectToPLOWindow( StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  const * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * j;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )
    StackClusterPrivate_Pop(stclpp ,&j); 
  else 
    j = CCDR ( arglistp);
  aLisp2DDrawSquare2 local = { 0 , 0.0L , 0.0L , 0.0L , 0.0L , CCAR ( arglistp) , stclpp } ;
  { unsigned  int tasks = maxListItemProcessing ;
    PLOWindow_pri * const mep = PLOWindow_to_pri ( plowp ) ;
    for ( ; ( tasks  > 0  ) and CConsp (  j ) ; --  tasks ) {
      ParaLispObject  *   x= CCAR(   j);
      if(IsNotNumber(  stclpp ,  x)){
        PLOWindow_LockFlush(plowp);
        return  TaskForThread_Analyse;
      }
      local.left = FractionLDValueConst( Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst( x))));
      local.right = local.left;
      j = CCDR ( j );
      x = CCAR ( j );
    if(IsNotNumber(  stclpp ,  x)){
      PLOWindow_LockFlush(plowp);
      return  TaskForThread_Analyse;
    }
    local.up = FractionLDValueConst( Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst( x))));
    local.down  = local.up;
    j = CCDR ( j );
    x = CCAR ( j );
    if(IsNotNumber(  stclpp ,  x)){
      PLOWindow_LockFlush(plowp);
      return  TaskForThread_Analyse;
    }
    { long  double const lwidth = FractionLDValueConst( Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst( x))));
// ! double variable    
      //( ( lwidth < 0 ) ? local . left  : local . right ) += lwidth  ;
      if(lwidth  < 0)
        local.left  +=  lwidth;
      else
        local.right +=  lwidth; 
    }
    j = CCDR ( j );
    x = CCAR (  j );
    if(IsNotNumber(  stclpp ,  x)){
      PLOWindow_LockFlush(plowp);
      return  TaskForThread_Analyse;
    }
    { long  double const lheight = FractionLDValueConst( Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst( x))));
// ! double variable      
      //( ( lheight < 0 ) ? local . up  : local . down  ) += lheight  ;
    if(lheight < 0)
      local.up  +=  lheight;
    else
      local.down  +=  lheight ;
    }
    j = CCDR ( j );
    local  . ilx2input = ldouble_cast_int ( floorl ( local.left  ) ) ;
      PLOWindow_DrawSquare  ( mep , local)  ;
    } //for
  }
  PLOWindow_LockFlush(plowp);
  if(CConsp(  j)){
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    StackClusterPrivate_Push(stclpp , j);
    return  TaskForThread_Analyse; 
  }
  StackClusterPrivate_SetResult(stclpp ,PLOWindow_plo(plowp));
  return  TaskForThread_Analyse; 
}

# define  PLOWindow_DrawPointWithoutSizeCheck3 \
  Lisp2D_PLOWindow_DrawPointWithoutSizeCheck3GuIr
static  void  PLOWindow_DrawPointWithoutSizeCheck3 ( PLOWindow_pri * const me  , tUCharRGB const rgb , IntXY  const ixy ) {
  unsigned long int ulcols;
  if(isBADCreateColor(rgb,me->colordefaultdepth,&ulcols))
    fwprintf ( stderr ,
      L"PLOWindow::DrawPointWithoutSizeCheck3(UChar):isBADCreateColor:"
      L"colordefaultdepth=%d\n" , me->colordefaultdepth ) ;
  else {
    *VectorAt(UnsLongInt)(&me -> cmatrixpixel ,
      ( int_cast_size ( ixy . x ) * me -> matrixsizeh ) +
      int_cast_size ( ixy . y )) = ulcols ;
    if(me->opened and me->dpy) {
      XLockDisplay(me->dpy);
      XSetForeground(me->dpy,me->gc,ulcols );
      XDrawPoint(me->dpy,me->win,me->gc,ixy.x,ixy.y);
      XFlush(me->dpy);
      XUnlockDisplay(me->dpy);
    }
  }
}
      
# define  PLOWindow_isBADColorsWOSizeCheck  Lisp2d_PLOWindow_isBADColorsWOSizeCheck
static  BadUCharRGB  PLOWindow_isBADColorsWOSizeCheck ( PLOWindow_pri const * const me , UIntXY const  xy ) {
  BadUCharRGB result  ;
  result.isBAD = isBADLoadColor( VectorAtCopy(UnsLongInt)(&me -> cmatrixpixel ,
      (xy.x * me -> matrixsizeh) + xy.y), &result.rgb,PLOWindow_ColorDefaultDepth(me));
  return result ;
}

# include "lightmem/public.h"
# include "lightmem/struct.h"

// %edi = UCharXY xy

// UCharRGB  ulcolsrgb -> input by %rcx -> transfer to stack -> load in memory by bytes
// UCharRGB  const & ulcolsrgb -> input by pointer %rcx -> then (%rcx) ,
//  1(%rcx) , 2(%rcx)
# define  PLOWindow_isBADDrawPointSmooth  Lisp2D_PLOWindow_isBADDrawPointSmooth
static  bool  PLOWindow_isBADDrawPointSmooth  ( PLOWindow_pri * const me ,
  IntXY const ixy , long  double  const * const dopmul , tUCharRGB const * const ulcolsrgbp ) {
  BadUCharRGB const result  = PLOWindow_isBADColorsWOSizeCheck  (
    me , ( UIntXY ) { int_cast_uint(ixy.x),int_cast_uint(ixy . y ) } ) ;
  if(result.isBAD)
    return true ;
  unsigned  char  col3r=result.rgb.r;
  unsigned  char  col3g=result.rgb.g;
  unsigned  char  col3b=result.rgb.b;
  col3r=Lisp2DColorMerge((Lisp2DByteXY){col3r,ulcolsrgbp->r},*dopmul);
  col3g=Lisp2DColorMerge((Lisp2DByteXY){col3g,ulcolsrgbp->g},*dopmul);
  col3b=Lisp2DColorMerge((Lisp2DByteXY){col3b,ulcolsrgbp->b},*dopmul);
  PLOWindow_DrawPointWithoutSizeCheck3  ( me , (tUCharRGB) { col3r , col3g , col3b } , (IntXY) { ixy . x , ixy . y } ) ;
  return  false;
}

static  void  PLOWindow_DrawSquare  ( PLOWindow_pri * const me  , aLisp2DDrawSquare2 const arg ) {
  int lx3 = arg.ilx2input   ;
  long  double  floatleft=arg.left  - floorl(arg.left);
  if(fabsl(floatleft)  > 0){
    ++  lx3 ;
    floatleft = (1.0L - floatleft);
  }
  long  double  const ldtmp = floorl(arg.up) ;
  int   ly=ldouble_cast_int(ldtmp);
  long  double   floatup=(arg.up)  - floorl(arg.up);
  if(fabsl(floatup)  > 0){
    ++  ly;
    floatup = (1.0L - floatup);
  }
  long  double  const ldtmp2  = floorl(arg.right) ;
  long  double  const   floatright=arg.right  - ldtmp2;
  int const   l2x=ldouble_cast_int(ldtmp2);
  long  double  const ldtmp3  = floorl(arg.down)  ;
  long  double  const   floatdown=arg.down  - ldtmp3;
  int const   l2y=ldouble_cast_int(ldtmp3);
  PLOWindow * const mepub = & me -> pub ;
  if((l2x >=  0)  and (l2y  >=  0)  and (lx3 < size_cast_ptrdiff(PLOWindow_VectorWidth(mepub))) and
    (ly < size_cast_ptrdiff(PLOWindow_VectorHeight(mepub)))){
    IntXY xy={lx3,ly};
    IntXY wh={l2x - lx3,l2y  - ly};
    PLOWindow_DrawSquareC ( ( t_PLOWindow_DrawSquareC ) { . me  = me  , . colsp = arg . cols  , . xy  = xy  , . wh  = wh  ,
      . stclpp  = arg . pstclp } ) ;
  }
  tUCharRGB ulcolsrgb ;
  if(isBADColorsCharsFromObjectOrList((arg.cols),&ulcolsrgb,(arg.pstclp)))
    return;
  // now floating sides ....
  if(lx3 <=  l2x){
    if(fabsl(floatleft)  > 0){
      // the same
      IntXY ixy ={ lx3 - 1 , 0  } ;
      if((ixy.x >=  0) and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
        for ( ixy . y = Lisp2DMaxInt  ( ly  , 0 ) ; ixy . y < l2y ; ++  ixy . y )
          if(int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub)){
            if(PLOWindow_isBADDrawPointSmooth(me,ixy,&floatleft,&ulcolsrgb))
              return;
          }
    }
    if(fabsl(floatright) > 0){
      IntXY ixy ={ l2x , 0  } ;
      if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
        for ( ixy . y = Lisp2DMaxInt  ( ly  , 0 ) ; ixy . y < l2y ; ++  ixy . y )
          if(int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub)){
            if(PLOWindow_isBADDrawPointSmooth(me,ixy,&floatright,&ulcolsrgb))
              return ;
          }
    }
    // (lx<=l2x)
  } else{ // not lx<=l2x
    IntXY ixy ={ lx3  - 1 , 0  } ;
    if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
      for ( ixy . y = Lisp2DMaxInt  ( ly  , 0 ) ; ixy . y < l2y ; ++  ixy . y )
        if(int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub)){
          long  double  const dopmul=(floatleft + floatright) - 1.0L;
          if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
            return;
        }
  }  // not lx<=l2x
  if(ly <=  l2y){
    if(fabsl(floatup)  > 0){
      IntXY ixy ={ 0 , ly - 1  } ;
      if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub)))
        for ( ixy . x = Lisp2DMaxInt  ( lx3 , 0 ) ; ixy . x  < l2x  ; ++  ixy . x )
          if(int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)) {
            if(PLOWindow_isBADDrawPointSmooth(me,ixy,&floatup,&ulcolsrgb)) 
              return;
          }
    }
    if(fabsl(floatdown)  > 0){
      IntXY ixy ={ 0 , l2y  } ;
      if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub)))
        for ( ixy . x = Lisp2DMaxInt  ( lx3 , 0 ) ; ixy . x  < l2x  ; ++  ixy . x )
          if(int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)){
            if(PLOWindow_isBADDrawPointSmooth(me,ixy,&floatdown,&ulcolsrgb)) 
              return;
          }
    }
    // ly<=l2y
  } else{ // ly>l2y
    IntXY ixy ={ 0 , ly - 1  } ;        
    if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub)))
      for ( ixy . x = Lisp2DMaxInt  ( lx3 , 0 ) ; ixy . x  < l2x  ; ++  ixy . x )
        if(int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)){
          long  double  const   dopmul =(floatup  + floatdown) - 1.0L;
          if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb)) 
            return;
        }
  } // ly>l2y
  // angles
  if((lx3  > l2x) and (ly > l2y)){
    IntXY ixy ={ lx3  - 1 , ly - 1  } ;
    if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
      if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))){
        long  double  const   dopmul=((floatup + floatdown) - 1.0L) * ((floatleft  + floatright)  - 1.0L);
        if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
          return;
      }
  } else
    if(lx3  > l2x){
      IntXY ixy ={ lx3  - 1 , ly - 1  } ;      
      if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
        if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))){
          long  double  const   dopmul=floatup  * ((floatleft  + floatright)  - 1.0L);
          if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
            return;
        }
      ixy.x  = lx3 - 1;
      ixy.y  = l2y;
      if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
        if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))){
          long  double  const   dopmul=floatdown  * ((floatleft + floatright) - 1.0L);
          if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
            return;
        }
      // lx2.i  > l2x
    } else
      if(ly > l2y){
        IntXY ixy ={ lx3  - 1 , ly - 1  } ;        
        if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
          if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))) {
            long  double  const   dopmul=((floatup  + floatdown)  - 1.0L) * floatleft;
            if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
              return;
          }
        ixy.x  = l2x;
        ixy.y  = l2y;
        if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
          if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))) {
            long  double  const   dopmul=((floatup  + floatdown)  - 1.0L) * floatright;
            if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
              return;
          }
        // ly > l2y
      } else{
        if(fabsl(floatleft)  > 0){
          if(fabsl(floatup)  > 0){
            IntXY ixy ={ lx3  - 1 , ly - 1  } ;
            if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
              if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))){
                long  double  const   dopmul=floatleft  * floatup;
                if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
                  return;
              }
          }
          if(fabsl(floatdown)  > 0){
            IntXY ixy ={ lx3  - 1 , l2y  } ;
            if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
              if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))){
                long  double  const   dopmul=floatleft  * floatdown;
                if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb)) 
                  return;
              }
          }
        }
        if(fabsl(floatright) > 0){
          if(fabsl(floatup)  > 0){
            IntXY ixy ={ l2x , ly - 1  } ;
            if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
              if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))){
                long  double  const   dopmul=floatright * floatup;
                if(PLOWindow_isBADDrawPointSmooth(me,ixy,&dopmul,&ulcolsrgb))
                  return;
              }
          }
          if(fabsl(floatdown)  > 0){
            IntXY ixy ={ l2x , l2y  } ;
            if((ixy.x  >=  0)  and (int_cast_uint(ixy.x)  < PLOWindow_VectorWidth(mepub)))
              if((ixy.y  >=  0)  and (int_cast_uint(ixy.y)  < PLOWindow_VectorHeight(mepub))){
                long  double  const   dopmul=floatright * floatdown;
                if  ( PLOWindow_isBADDrawPointSmooth  ( me  , ixy , & dopmul  , & ulcolsrgb ) )
                  return  ;
              }
          }
        }
      }  // lx<=l2x
} //drawsquare

# include "window/inline.h"
# include "castb.h"
# include "window/inlinepri.h"

# else

static  char  const Lisp2D_ch1933  __attribute__ ((unused)) = ' ' ;

# endif // LISP2D_DONTUSEX11
