// Lisp2D environment
// ©2009-23 by Alex Glebe

# include "symbolc.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "define.h"

# ifdef LISP2D_CONSTRSTAT
_Atomic ( size_t  ) PLOEnvironmentLayer_constrstat = 0 ;
# endif

# include "env/public.h"
# include "globallistp/struct.h"

GlobalList  GlobalListOfLayer ;
GlobalList  GlobalListOfLayerGarbage  ;
GlobalList  GlobalListOfLayerGC ;  

LISP2DMAPSTACKPUSHBYVALUEBODY ( PLOValueFlagConst )
LISP2DMAPSTACKPUSHATVALUEBODY ( PLOValueFlagConst )
LISP2DMAPSTACKCATBODY ( PLOValueFlagConst )
LISP2DMAPSTACKATBODY  ( PLOValueFlagConst )
LISP2DMAPINSERTBODY ( PLOValueFlagConst )
LISP2DMAPFINDBODY ( PLOValueFlagConst , ParaLispObject * )

void  EnvironmentLayerGCDeclare  ( EnvironmentLayer  * const ep ) {
  Map ( PLOValueFlagConst ) * const elmap = & ep -> maplinks ;
  size_t  i = MapSize ( PLOValueFlagConst )  ( elmap )  ;
  while ( i ) {
    -- i ;
    MapNode ( PLOValueFlagConst ) * const no = MapAt ( PLOValueFlagConst )  ( elmap , i )  ;
    ParaLispObject_virtualDeclareForGC  ( no  ->  key ) ;
    ParaLispObject_virtualDeclareForGC  ( no  ->  value . value ) ; 
  }
}

// returns error when OutOfmemory OR ConstLink modifying  
bool  EnvironmentLayerisBADPush ( EnvironmentLayer * const me ,
  ParaLispObject  * const sp , ParaLispObject  * const vp ,
  bool  const flagconst ) {
  PassagePassLock ( PassageToPub ( & me -> passagec ) ) ;
  bool  result = false ;
  Map ( PLOValueFlagConst ) * mymaplinks = & me  ->  maplinks ;
  size_t  const i = MapFind ( PLOValueFlagConst ) ( mymaplinks , sp ) ;
  if  ( i ==  MapSize ( PLOValueFlagConst ) ( mymaplinks ) ) {
    MapNode ( PLOValueFlagConst ) mn = { sp  , { vp , flagconst } } ;
    MapInsert ( PLOValueFlagConst ) ( mymaplinks , mn ) ; 
  } else {
    MapNode ( PLOValueFlagConst ) * const mn = MapAt  ( PLOValueFlagConst )
      ( mymaplinks ,  i ) ;
    if  ( mn  ->  value . flagconst == true ) {
      result  = true  ;
      goto  Exit  ; 
    }
    mn -> value = ( EnvironmentLayerValueAndFlagConst ) { vp , flagconst } ;
  }
Exit  :
  PassagePassUnLock ( PassageToPub ( & me -> passagec ) ) ;
  return  result ; 
}
  
ParaLispObject  * EnvironmentLayerAssocPassage  ( EnvironmentLayer  * const ep , ParaLispObject * const sp  ) {
  PassagePassIn ( PassageToPub ( & ep -> passagec ) ) ;
  { Map ( PLOValueFlagConst ) * const  mymaplinks = & ep  ->  maplinks ;
    size_t  const i = MapFind ( PLOValueFlagConst ) ( mymaplinks , sp ) ;
    if  ( i ==  MapSize ( PLOValueFlagConst ) ( mymaplinks ) ) {
      PassagePassOut ( PassageToPub ( & ep -> passagec ) ) ;
      return  0 ;
    }
    MapNode ( PLOValueFlagConst ) * const mn  = MapAt ( PLOValueFlagConst ) ( mymaplinks  , i ) ;
    ParaLispObject  * const v = mn  ->  value . value ;
    PassagePassOut ( PassageToPub ( & ep -> passagec ) ) ;
    return  v ;
  }
}
  
// returns false when value not present OR have Const Links
bool  EnvironmentLayerisOKTryChangeValue  ( t_EnvironmentLayerisOKTryChangeValue const arg ) {
  PassagePassIn ( PassageToPub ( & arg . me -> passagec ) ) ;
  { Map ( PLOValueFlagConst ) * const mymaplinks = & arg . me  ->  maplinks ;
    size_t  const i = MapFind ( PLOValueFlagConst ) ( mymaplinks , arg . sp ) ;
    if  ( i ==  MapSize ( PLOValueFlagConst ) ( mymaplinks ) ) {
      PassagePassOut ( PassageToPub ( & arg . me -> passagec ) ) ;
      ( * arg . flagwasconstp ) = false ;
      return  false ;
    }
    MapNode ( PLOValueFlagConst ) * const mn = MapAt  ( PLOValueFlagConst ) ( mymaplinks ,  i ) ;
    if  ( mn  ->  value . flagconst == true ) {
      PassagePassOut ( PassageToPub ( & arg . me -> passagec ) ) ;
      ( * arg . flagwasconstp ) = true  ;
      return  false ;
    }
    mn  ->  value = ( EnvironmentLayerValueAndFlagConst ) { arg . vp , arg . flagconst } ;
  }
  PassagePassOut ( PassageToPub ( & arg . me -> passagec ) ) ;
  ( * arg . flagwasconstp ) = false ;
  return  true  ;
}
  
// returns false when value not present
bool  EnvironmentLayerisOKTryChangeValueIgnoreWasConst  ( EnvironmentLayer * const me , ParaLispObject * const sp  ,
  ParaLispObject  * const vp ,  bool  const flagconst )  {
  PassagePassIn ( PassageToPub ( & me -> passagec ) ) ;
  { Map ( PLOValueFlagConst ) * mymaplinks = & me  ->  maplinks ;
    size_t  const i = MapFind ( PLOValueFlagConst ) ( mymaplinks , sp ) ;
    if  ( i ==  MapSize ( PLOValueFlagConst ) ( mymaplinks ) ) {
      PassagePassOut ( PassageToPub ( & me -> passagec ) ) ;
      return  false ;
    }
    MapNode ( PLOValueFlagConst ) * const mn = MapAt  ( PLOValueFlagConst ) ( mymaplinks ,  i ) ;
    mn  ->  value = ( EnvironmentLayerValueAndFlagConst ) { vp , flagconst } ;
  }
  PassagePassOut ( PassageToPub ( & me -> passagec ) ) ;
  return  true  ;
}
  
bool  EnvironmentLayerisOKSetConstFlag  ( EnvironmentLayer  * const me , ParaLispObject * const sp  , bool  const f ) {
  PassagePassIn ( PassageToPub ( & me -> passagec ) ) ;
  { Map ( PLOValueFlagConst ) * mymaplinks = & me  ->  maplinks ;
    size_t  const i = MapFind ( PLOValueFlagConst ) ( mymaplinks , sp ) ;
    if  ( i ==  MapSize ( PLOValueFlagConst ) ( mymaplinks ) ) {
      PassagePassOut ( PassageToPub ( & me -> passagec ) ) ;
      return  false ;
    }
    MapNode ( PLOValueFlagConst ) * const mn = MapAt  ( PLOValueFlagConst ) ( mymaplinks ,  i ) ;
    mn  ->  value . flagconst = f ;
  }
  PassagePassOut ( PassageToPub( & me -> passagec ) ) ;
  return  true  ;
}
  
bool  EnvironmentLayerisOKGetConstFlag  ( EnvironmentLayer  * const ep , ParaLispObject * const sp  , bool * const fp ) {
  PassagePassIn ( PassageToPub ( & ep -> passagec ) ) ;
  { Map ( PLOValueFlagConst ) * const  mymaplinks = & ep  ->  maplinks ;
    size_t  const i = MapFind ( PLOValueFlagConst ) ( mymaplinks , sp ) ;
    if  ( i ==  MapSize ( PLOValueFlagConst ) ( mymaplinks ) ) {
      PassagePassOut ( PassageToPub ( & ep -> passagec ) ) ;
      return  false ;
    }
    MapNode ( PLOValueFlagConst ) * const mn = MapAt  ( PLOValueFlagConst ) ( mymaplinks ,  i ) ;
    ( * fp  ) = mn  ->  value . flagconst ;
  }
  PassagePassOut ( PassageToPub ( & ep -> passagec ) ) ;
  return  true  ;
}
  
void  EnvironmentLayerPushAll ( EnvironmentLayer  * const mep  , EnvironmentLayer  const * const el  ) {
  PassagePassLock ( PassageToPub ( & mep  ->  passagec ) ) ;
// !?  how to protect EL from destruction by another tasks
  Map ( PLOValueFlagConst ) const * const elmap = EnvironmentLayerMap ( el  ) ;
  for ( size_t  i = MapSize ( PLOValueFlagConst )  ( elmap )  ; i ; ) {
    -- i ;
    MapNode ( PLOValueFlagConst ) const * const no = MapCAt ( PLOValueFlagConst )  ( elmap , i )  ;
    MapInsert ( PLOValueFlagConst ) ( & mep ->  maplinks  , * no  ) ;    
  }
  PassagePassUnLock ( PassageToPub ( & mep -> passagec ) ) ; 
}

# ifdef  LISP2D_CONSTRSTAT  
size_t  PLOEnvironmentLayer_ConstrstatValue  ( void ) {
  return  PLOEnvironmentLayer_constrstat  ;
}
  
void  PLOEnvironmentLayer_ConstrstatAdd ( void ) {
  ++  PLOEnvironmentLayer_constrstat  ;
}
# endif
  
void  PLOEnvironmentLayer_Init ( PLOEnvironmentLayer  * const me ) {
  EnvironmentLayerInit  ( & me -> envlay  ) ;
  ParaLispObject_InitVTP  ( & me -> plo , ( VirtualTypePointer  ) { . addr = { . p = me } ,
    . type = TypeGlobalListMemberPLOEnvironmentLayer } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOEnvironmentLayer_ConstrstatAdd ( ) ;
# endif
}

void  PLOEnvironmentLayer_InitCopy_Exc ( PLOEnvironmentLayer  * const me ,
  PLOEnvironmentLayer const * const x ) {
  ParaLispObject_InitVTP  ( & me -> plo , ( VirtualTypePointer  ) { . addr = { . p = me } ,
    . type = TypeGlobalListMemberPLOEnvironmentLayer } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOEnvironmentLayer_ConstrstatAdd ( ) ;
# endif
  EnvironmentLayer_InitCopy ( & me -> envlay , & x -> envlay ) ;
}
  
void  PLOEnvironmentLayer_Destr ( PLOEnvironmentLayer  * const me ) {
  EnvironmentLayerDestr ( & me  ->  envlay  ) ;
  ParaLispObject_staticDestr  ( & me  ->  plo ) ;
}

void PLOEnvironmentLayer_staticDestr  ( ParaLispObject * const me ) {
  PLOEnvironmentLayer_Destr ( ParaLispObjectToPLOEnvironmentLayer ( me  ) ) ;
}
  
ParaLispObject  * PLOEnvironmentLayer_staticClassDef  ( ParaLispObject const * _ __attribute__  ( ( unused  ) ) ) {
  return  ploclassdefenvlayer  ; 
}

ParaLispObject  * PLOEnvironmentLayer_staticClassParents  ( ParaLispObject const * _ __attribute__  ( ( unused  ) ) ) {
  return  ploclassparentsenvlayer  ; 
}

ParaLispObject  * PLOEnvironmentLayer_staticTypeOf ( ParaLispObject const * _ __attribute__  ( ( unused  ) ) ) {
  return &  plosenvironmentlayer  . plo  ;
}  

void  PLOEnvironmentLayer_staticGCDeclareContent ( ParaLispObject * const me ) {
  EnvironmentLayerGCDeclare  ( & ParaLispObjectToPLOEnvironmentLayer(me) -> envlay  ) ;
} 

ParaLispObject  * PLOEnvironmentLayer_staticPushAsNew ( ParaLispObject  * const me ) {
  GlobalListPush ( & GlobalListOfLayer , & me -> base ) ;
  return   me  ; 
}  
  
void  PLOEnvironmentLayer_staticDeclareForGC (ParaLispObject * const me ) {
  if  ( not  me -> flagGCPassed ) {
    me -> flagGCPassed =  true  ;
    GlobalListPopNotLock  ( & GlobalListOfLayerGC , & me -> base ) ;
    GlobalListPushNotLock ( & GlobalListOfLayer , & me -> base ) ;
  }
}
    
bool  PLOEnvironmentLayer_staticisBADCopyPrivate ( ParaLispObject const * const me , ParaLispObject  * * const resultp  ) {
  ( * resultp ) = & NewEnvironmentLayerCopy ( ParaLispObjectToPLOEnvironmentLayerConst ( me ) ) -> plo ;
  return  false ; 
}

PLOEnvironmentLayer * NewEnvironmentLayerCopy ( PLOEnvironmentLayer const * const xp ) {
  GlobalListMember * layerNew  ;
  if  ( not  ( GlobalListisOKTryPopFirst ( & GlobalListOfLayerGarbage , & layerNew  ) ) ) {
    PLOEnvironmentLayer * const layerNew2 = aligned_alloc ( _Alignof ( PLOEnvironmentLayer ) , sizeof ( PLOEnvironmentLayer ) ) ;
    if ( not  layerNew2 ) {
      Lisp2DExcOutOfMemoryGlobal . value =
        L"NewEnvironmentLayer ( PLOEnvironmentLayer const * const xp ):"
        L"aligned_alloc  ( sizeof  ( PLOEnvironmentLayer ) )" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    PLOEnvironmentLayer_InitCopy_Exc  ( layerNew2 ,  xp ) ;
    // ? first : to push OR init
    GlobalListPush ( & GlobalListOfLayer , & layerNew2 -> plo . base  ) ;
    return  layerNew2 ;
  }
  // in garbage layer cleaned
  EnvironmentLayerSet ( & PLOEnvironmentLayer_FromMember ( layerNew  ) -> envlay , & xp -> envlay ) ;
  GlobalListPush ( & GlobalListOfLayer , layerNew  ) ;
  return  PLOEnvironmentLayer_FromMember ( layerNew  ) ;
}
  
PLOEnvironmentLayer * PLOEnvironmentLayer_FromMember (
  GlobalListMember * const m ) {
  if  ( m ) {
    if  ( GlobalListMemberVirt  ( m ) . type ==
      TypeGlobalListMemberParaLispObject ) {
      ParaLispObject * const p = GlobalListMemberVirt  ( m ) . addr . p ;
      if  ( p  ->  vaddr . type == TypeGlobalListMemberPLOEnvironmentLayer )
        return (  PLOEnvironmentLayer * ) ( p  ->  vaddr . addr . p ) ;
    } else {
      if  ( m ->  vaddr . type == TypeGlobalListMemberPLOEnvironmentLayer )
        return  ( PLOEnvironmentLayer * ) ( m ->  vaddr . addr . p ) ;
    }
  }
  return  (void*)0 ; 
}

# include "plostream/public.h"  
  
# define  CPLOcastPLO Lisp2D_CPLOcastPLO
inline  ParaLispObject  * CPLOcastPLO ( ParaLispObject  const * p ) ;
  
# define  PLOEnvironmentLayerCoutPrivate Lisp2DPLOEnvironmentLayerCoutPrivatex5qd
  
static  void  PLOEnvironmentLayerCoutPrivate  (
  PLOEnvironmentLayer const * const me , FuncCoutToInterpretHuman  funccout  ,
  PLOStream  const * const osp , bool  flag2pprinted ,
  Vector ( StackPLOP ) * const historyp ) {
  if  ( Printed3Points  ( & me -> plo , osp  , & flag2pprinted , historyp ) )
    return  ;
  Vector_PushByValue  ( StackPLOP ) ( historyp  ,
    CPLOcastPLO ( & me -> plo ) ) ;  
  // passage protects tree from destruction  
  PassagePassIn ( PassageToPub ( EnvironmentLayerRWThePassageC (
    & me -> envlay ) ) ) ;
  PLOStreamPushWideCharArray  ( osp , (PArray_wchar_c)
    { (wchar_t const(*)[])&L"EnvironmentLayer("} ) ;
  wchar_t const skc = L')'   ;
  { size_t  i = 0 ;
    if  ( i not_eq  MapSize( PLOValueFlagConst ) (
      EnvironmentLayerMap ( & me -> envlay ) ) ) {
      // first element without space
      PLOStreamPushWChar(osp,L'(');
      funccout  ( MapCAt(PLOValueFlagConst)(EnvironmentLayerMap (
        & me -> envlay  ),i) ->  key ,  osp  , & flag2pprinted ,  historyp ) ;
      if  ( MapCAt(PLOValueFlagConst)(EnvironmentLayerMap (
        & me -> envlay ) , i ) -> value . flagconst ) 
        PLOStreamPushWideCharArray  ( osp ,
          (PArray_wchar_c) { (wchar_t const(*)[])&L" .const. "});
      else  
        PLOStreamPushWideCharArray  ( osp ,
          (PArray_wchar_c) { (wchar_t const(*)[])&L" . "});
      funccout ( MapCAt(PLOValueFlagConst)(EnvironmentLayerMap (
        & me -> envlay ) , i ) -> value . value ,  osp  , & flag2pprinted ,
        historyp ) ;
      PLOStreamPushWChar(osp,skc);
      // next element
      ++  i ;
      // next elements with space in front
      while ( i  not_eq  (  MapSize( PLOValueFlagConst ) (
        EnvironmentLayerMap ( & me -> envlay ) ) ) ) {
        // write space and element
        PLOStreamPushWideCharArray  ( osp ,
          (PArray_wchar_c) { (wchar_t const(*)[])&L" ("});
        funccout ( MapCAt(PLOValueFlagConst)(EnvironmentLayerMap (
          & me -> envlay ) , i ) ->  key ,  osp  ,  & flag2pprinted ,
          historyp ) ;  
        if  ( MapCAt(PLOValueFlagConst)(EnvironmentLayerMap ( & me -> envlay ) ,
          i ) -> value . flagconst )
          PLOStreamPushWideCharArray  ( osp ,
            (PArray_wchar_c) { (wchar_t const(*)[])&L" .const. "});
        else  
          PLOStreamPushWideCharArray  ( osp ,
            (PArray_wchar_c) { (wchar_t const(*)[])&L" . "});
        funccout ( MapCAt(PLOValueFlagConst)(EnvironmentLayerMap (
          & me -> envlay ) , i ) -> value . value ,  osp  , & flag2pprinted ,
          historyp ) ;  
        PLOStreamPushWChar(osp,skc);
        ++  i ;
      } // while if != mapsize
    } // if != mapsize
  }  // i
  PLOStreamPushWChar(osp,skc);
  PassagePassOut ( PassageToPub ( EnvironmentLayerRWThePassageC ( & me -> envlay ) ) ) ;
  VectorPop(StackPLOP) (historyp);
}
  
void  PLOEnvironmentLayer_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool flag2pprinted  , Vector(StackPLOP)  * const h ) {
  PLOEnvironmentLayerCoutPrivate  ( ParaLispObjectToPLOEnvironmentLayerConst (
    me  ) , ParaLispObject_CoutToHuman ,   s , flag2pprinted ,  h ) ;
} 

void  PLOEnvironmentLayer_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool flag2pprinted  , Vector  ( StackPLOP ) * const h )  {
  PLOEnvironmentLayerCoutPrivate  ( ParaLispObjectToPLOEnvironmentLayerConst  (
    me ) , ParaLispObject_CoutToInterpret ,  s , flag2pprinted ,  h ) ;
}

# ifdef LISP2D_CONSTRSTAT
# define  PLOEnv_constrstat Lisp2D_PLOEnv_constrstat
_Atomic ( size_t  ) PLOEnv_constrstat = 0 ;

size_t  PLOEnv_constrstatValue  ( void  ) {
  return  PLOEnv_constrstat ;
}

void  PLOEnv_constrstatIncr  ( void  ) {
  ++  PLOEnv_constrstat ;
}

# endif

void  PLOEnv_Init  ( PLOEnvC * const me ) {
  VirtualTypePointer  vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOEnv }  ;
  ParaLispObject_InitVTP  ( & me -> plo , vtp ) ; 
  Environment_Init  ( & me -> env ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOEnv_constrstatIncr ( ) ;
# endif
}

ParaLispObject  * PLOEnv_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  & plosenv . plo ;
}
  
bool  PLOEnv_staticisBADCopyPrivate ( ParaLispObject const * const me , ParaLispObject * * const resultp  ) {
  PLOEnvC * const res = aligned_alloc ( _Alignof ( PLOEnvC ) , sizeof ( PLOEnvC ) ) ;
  if  ( res ) {
    PLOEnv_InitCopy(res, ParaLispObjectToPLOEnvConst ( me));
    ParaLispObject_virtualPushAsNew ( & res ->  plo ) ;
    ( * resultp )  = & res  ->  plo ;
    return  false ;
  }
  return  true  ;
}

void  PLOEnv_Destr  ( PLOEnvC  * const e ) { 
  ParaLispObject_staticDestr  ( & e ->  plo ) ;
}

void PLOEnv_staticDestr  ( ParaLispObject * const me ) {
  PLOEnv_Destr  ( ParaLispObjectToPLOEnv  ( me  ) ) ;
}

void  PLOEnv_staticGCDeclareContent ( ParaLispObject * const  me )  {
  PLOEnvC * const  e = ParaLispObjectToPLOEnv ( me  ) ;
  ParaLispObject_virtualDeclareForGC ( Environment_Env ( & e -> env ) ) ;
  ParaLispObject_virtualDeclareForGC ( Environment_NamespacePropList (
    & e -> env )  ) ;
} 

ParaLispObject  *   ploclassdefenv  = & plonil  . plo ;

ParaLispObject  * PLOEnv_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) )  {
  return   ploclassdefenv  ;
}

ParaLispObject  *   ploclassparentsenv  = & plonil  . plo ;

ParaLispObject  * PLOEnv_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassparentsenv  ;
}

PLOEnvironmentLayer * NewEnvironmentLayer ( void ) {
  GlobalListMember * layerNew  ;
  if  ( not  ( GlobalListisOKTryPopFirst  ( & GlobalListOfLayerGarbage  ,
        & layerNew  ) ) ) {
    PLOEnvironmentLayer * const layerNew2 = aligned_alloc ( _Alignof ( PLOEnvironmentLayer ) , sizeof ( PLOEnvironmentLayer ) ) ;
    if  ( layerNew2 )
      PLOEnvironmentLayer_Init  ( layerNew2 ) ;
    else {
      Lisp2DExcOutOfMemoryGlobal . value = L"NewEnvironmentLayer" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    GlobalListPush ( & GlobalListOfLayer , & layerNew2 -> plo . base  ) ;
    return layerNew2 ;
  }
  // in garbage layer cleaned
  GlobalListPush ( & GlobalListOfLayer , layerNew ) ;
  return PLOEnvironmentLayer_FromMember ( layerNew  ) ;
}

# include "newobject.h"
  
bool  IsNotEnv  ( StackClusterPrivate  * const stclp  , ParaLispObject * const xp ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp ,
    . typetype  = TypeGlobalListMemberPLOEnv ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
  //return  UniversalIsNotType  ( stclp , xp , TypeGlobalListMemberPLOEnv ) ;
}

# include "stcl/public.h"
# include "stcl/inline.h"

// nil environment-external
TaskForThread Lisp2Dbimnilenvironmentexternal2  ( StackClusterPrivate  * const stclpp  ) {
  PLOEnvC * const pe = aligned_alloc ( _Alignof ( PLOEnvC ) , sizeof ( PLOEnvC ) ) ;
  if  ( pe ) {
    PLOEnv_Init(pe);
    ParaLispObject_virtualPushAsNew (&pe  ->  plo ) ;
    Environment_SetPair ( & pe -> env , StackClusterPrivate_ExtEnv( stclpp ) ,
      StackClusterPrivate_NamespaceProplist ( stclpp ) ) ;
    StackClusterPrivate_SetResult (stclpp ,&  pe->plo  ) ;
  } else 
    OutOfMemory (  stclpp ) ;
  return  TaskForThread_Analyse ;
}

// nil environment
TaskForThread Lisp2Dbimnilenvironment2  ( StackClusterPrivate  * const stclpp  ) {
  PLOEnvC * const pe = aligned_alloc ( _Alignof ( PLOEnvC ) , sizeof ( PLOEnvC ) ) ;
  if  ( pe ) {
    PLOEnv_Init(pe);
    ParaLispObject_virtualPushAsNew ( & pe  ->  plo) ;
    Environment_SetPair ( & pe -> env , StackClusterPrivate_Env( stclpp ) ,
      StackClusterPrivate_NamespaceProplist ( stclpp ) ) ;
    StackClusterPrivate_SetResult (stclpp ,&  pe->plo  ) ;
  } else 
    OutOfMemory (  stclpp ) ;
  return  TaskForThread_Analyse ;
}

bool  IsNotEnvironmentLayer ( StackClusterPrivate  * const stclp  , ParaLispObject * const  x ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = x ,
    . typetype  = TypeGlobalListMemberPLOEnvironmentLayer ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
  //return  UniversalIsNotType  ( stclp , x , TypeGlobalListMemberPLOEnvironmentLayer  ) ;
}

# include "cons/struct.h"
# include "cons/public.h"

void  Environment_NewLevel ( Environment  * const e ,  ParaLispObject * const level  ) {
  PLOCons * const newe  =   NewCons (  level ,  e -> env ) ;
  e -> env = & newe -> plo  ;
}

void  Environment_ReleaseLevel  ( Environment * const e ) {
  e -> env = CCDR ( e -> env ) ;
}

# include "cons/inline.h"

// env setq [{symbol task}]
TaskForThread Lisp2Dbimenvsetq2 ( StackClusterPrivate * const stclpp ) {
  ParaLispObject  * varA  ;
  ParaLispObject  * j ;
  ParaLispObject  * varB  ;
  Lisp2DEvalAddrCont2 addr  = StackClusterPrivate_ClearflagCont  ( stclpp ) ;
  if  ( addr ) {
    switch  ( addr  ) {
    case  1 :
      goto  Next  ;
    case  2 :
      return  TaskForThread_Analyse ;
    case  3 :
      varB = StackClusterPrivate_PopValue ( stclpp ) ;
      varA = StackClusterPrivate_PopValue ( stclpp ) ;
      j = StackClusterPrivate_PopValue ( stclpp )  ;
      if  ( StackClusterPrivate_exceptionnotprocessed  ( stclpp ) ) {
        StackClusterPrivate_setTaskForManager (
          stclpp , TaskWaitSonsAlmost ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
        return  TaskForThread_Analyse ;
      }
      goto  NextVars  ;
    default :
# ifdef LISP2D_DEBUG      
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& 
        L"env setq:addr is bad" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;      
# endif
      ;
      } } // if addr
  varA  =  &  plonil.plo  ;
  varB  =  &  plonil.plo  ;
  j = StackClusterPrivate_ArgTop(stclpp ) ;
NextVars  :
  { unsigned int tasks  = maxTasksCreate   ;
  for ( ; ( tasks  > 0  )  and CConsp  (  j ) ;
      j  = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc ) , j = CCDR ( j ) ,
    --  tasks ) {
    { PLOCons * next2  = NewCons ( &plonil.plo  ,& plonil.plo  )  ; 
      // if Null
      if  ( varA ==  (  &  plonil.plo ) )
        varA  = & next2->plo ;
      else 
        ClassCons_SetRest(&ParaLispObjectToPLOCons ( varB  )->cc ,&  next2->plo ) ;
      varB  = &next2->plo ; } // next2
    if  ( IsNotSymbol ( stclpp ,
        ClassCons_First(&ParaLispObjectToPLOCons ( j )->cc ) ) ) {
      StackClusterPrivate_setTaskForManager (
        stclpp , TaskWaitSonsAlmost ) ;
      StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
      return  TaskForThread_Analyse ; }
    StackClusterPrivate  * const dopstclp =
      StackClusterPrivate_NewStCl4  (stclpp ) ;
    SetResultPlace  (  dopstclp ,
      ClassCons_PFirst ( &ParaLispObjectToPLOCons ( varB  )->cc ) , varB  ) ;
    StackClusterPrivate_CreateTask (dopstclp ,
      CCAR ( ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc ) ) ) ; } // for tasks
    } // tasks
  if  ( CConsp (  j ) ) {
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 3U  ) ;
    StackClusterPrivate_Push ( stclpp , j ) ;
    StackClusterPrivate_Push (stclpp ,  varA  ) ;
    StackClusterPrivate_Push (stclpp , varB  ) ;
    return  TaskForThread_Analyse ; } // if consp j
// !!!! didnot make new list, if need make new variable,
//  Assoc and place result in there presl=PLOPRest
  StackClusterPrivate_Push (stclpp ,  varA  ) ;
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse ;
Next  :
  varA = StackClusterPrivate_PopValue ( stclpp ) ;
  if  ( StackClusterPrivate_exceptionnotprocessed  ( stclpp ) )
    return  TaskForThread_Analyse ;
// ! to use TaskPause (cycle protection)
  PLOEnvC  * const mep  =  ParaLispObjectToPLOEnv (  StackClusterPrivate_ThisTop (
    stclpp ) ) ;
  { ParaLispObject  * j3  = StackClusterPrivate_ArgTop(stclpp )  ;
    for ( ; CConsp (  j3  ) ;
      varA  = CCDR ( varA  ) ,
      j3  = ClassCons_Rest ( &ParaLispObjectToPLOCons ( j3  )->cc ) ,
      j3  = CCDR ( j3  ) ) {
      ParaLispObject  * const varDp = ClassCons_First(&ParaLispObjectToPLOCons ( j3  )->cc ) ;
      { ParaLispObject  * const valp = CCAR ( varA  ) ;
        bool  flagsetisok = false ;
        { ParaLispObject  * j2  =  Environment_Env ( & mep -> env )   ;
          for ( ; CConsp (  j2  ) ;
            j2  = ClassCons_Rest( &ParaLispObjectToPLOCons ( j2  )->cc ) ) {
            ParaLispObject  * const jlp  = ClassCons_First(&ParaLispObjectToPLOCons ( j2  )->cc ) ;
            if  ( IsNotEnvironmentLayer (  stclpp  , jlp  ) )
              return  TaskForThread_Analyse ;
            bool  wasconst  ;
            if  ( EnvironmentLayerisOKTryChangeValue  ( ( t_EnvironmentLayerisOKTryChangeValue ) {
              . me = & ParaLispObjectToPLOEnvironmentLayer  ( jlp  ) -> envlay ,
              . sp = varDp  , . vp = valp , . flagwasconstp = & wasconst , . flagconst = false } ) ) {
              flagsetisok = true  ;
              break ;
            }
            if  ( wasconst  ) {
              HasConstLink  (  stclpp , varDp  ) ;
              return  TaskForThread_Analyse ;
            }
          } //  for j2
        } // j2
        if  ( not  flagsetisok  ) {
          ParaLispObject  * setenv2 = Environment_Env ( & mep -> env )  ;
          if  ( IsNotCons (  stclpp ,  setenv2 ) )
            return  TaskForThread_Analyse ;
          setenv2 = ClassCons_First ( &ParaLispObjectToPLOCons ( setenv2 )->cc ) ;
          if  ( IsNotEnvironmentLayer (  stclpp  ,  setenv2 ) )
            return  TaskForThread_Analyse ;
          if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
              setenv2 ) -> envlay , varDp  , valp , false ) ) {
            HasConstLink  (  stclpp , varDp  ) ;
            return  TaskForThread_Analyse ;
          }
        }
      }
    } // for j3
  } // j3
  StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ;
  return  TaskForThread_Analyse ;
}

// env funcall obj meth args .. args
TaskForThread Lisp2Dbimenvfuncall2  ( StackClusterPrivate  * const stclpp  ) {
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) )
    goto  Next  ;
  StackClusterPrivate_Push (stclpp ,StackClusterPrivate_Env( stclpp ) )  ;
  StackClusterPrivate_Push (stclpp ,StackClusterPrivate_ExtEnv( stclpp ) )  ;
  StackClusterPrivate_Push (stclpp , StackClusterPrivate_NamespaceProplist ( 
    stclpp ) ) ;
  { PLOEnvC  * const mep  =
      ParaLispObjectToPLOEnv (StackClusterPrivate_ThisTop(stclpp )) ;
    StackClusterPrivate_SetEnv(stclpp , Environment_Env ( & mep ->env ) ) ;
    // external field have GLOBAL variables
    // another namespace is not accepted
    StackClusterPrivate_SetExtEnv ( stclpp ,
      Environment_Env ( & mep -> env ) ) ;
    SetNamespaceProplist  ( stclpp , Environment_NamespacePropList (
        & mep -> env ) ) ; } // me
  StackClusterPrivate_setflagApply  ( stclpp , true  ) ;
  { ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp ) ;
    ParaLispObject  * const argrestp = CCDR ( arglistp ) ;
    PLOCons * const l2  =  NewCons (  CCAR ( argrestp ) ,  CCDR ( argrestp ) )  ;
    PLOCons * const l = NewCons (  CCAR ( arglistp ) , & l2 -> plo )  ;
    StackClusterPrivate_Push (stclpp ,&  l->plo )  ; } // arglist , argrest , l2 , l
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse ;
Next  :
  SetNamespaceProplist  (  stclpp ,  StackClusterPrivate_PopValue ( stclpp ) ) ;
  StackClusterPrivate_SetExtEnv ( stclpp ,
    StackClusterPrivate_PopValue ( stclpp ) ) ;
  StackClusterPrivate_SetEnv  ( stclpp ,
    StackClusterPrivate_PopValue ( stclpp ) ) ;
  return  TaskForThread_Analyse ; }

// env apply obj meth arglist
TaskForThread Lisp2Dbimenvapply2  ( StackClusterPrivate  * const stclpp  ) {
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) ) 
    goto  Next  ;
  StackClusterPrivate_setflagApply  ( stclpp , true  ) ;
  { ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp ) ;
    PLOEnvC  * const mep  =ParaLispObjectToPLOEnv (
      StackClusterPrivate_ThisTop(stclpp ))  ;
    StackClusterPrivate_Push (stclpp ,StackClusterPrivate_Env( stclpp ) ) ;
    StackClusterPrivate_Push (stclpp ,StackClusterPrivate_ExtEnv( stclpp ) ) ;
    StackClusterPrivate_Push (stclpp , StackClusterPrivate_NamespaceProplist (
      stclpp ) ) ;
    StackClusterPrivate_SetEnv(stclpp , Environment_Env ( & mep -> env ) ) ;
    // external field have GLOBAL variables
    // another namespace is not accepted
    StackClusterPrivate_SetExtEnv ( stclpp , Environment_Env ( & mep -> env ) ) ;
    SetNamespaceProplist  ( stclpp , Environment_NamespacePropList (
      & mep -> env ) ) ;
    ParaLispObject  * const argrestp = CCDR ( arglistp ) ;
    PLOCons * const l2 = NewCons  (  CCAR ( argrestp ) , 
      CCAR ( CCDR ( argrestp ) ) ) ;
    PLOCons * const l = NewCons (  CCAR ( arglistp ) , & l2->plo  )  ;
    StackClusterPrivate_Push (stclpp ,&  l->plo ) ; }  // arglist , me
  StackClusterPrivate_SetflagCont  ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse ;
Next  :
  SetNamespaceProplist  (  stclpp , StackClusterPrivate_PopValue (
    stclpp ) ) ;
  StackClusterPrivate_SetExtEnv ( stclpp ,
    StackClusterPrivate_PopValue ( stclpp ) ) ;
  StackClusterPrivate_SetEnv  ( stclpp , StackClusterPrivate_PopValue (
    stclpp ) ) ;
  return  TaskForThread_Analyse ; }

// env eval obj
TaskForThread Lisp2Dbimenveval2 ( StackClusterPrivate * const stclpp ) {
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) )
    goto  Next  ;
  { ParaLispObject  * const objp = CCAR ( StackClusterPrivate_ArgTop(stclpp )) ;
    StackClusterPrivate_Push (stclpp ,StackClusterPrivate_Env( stclpp ) ) ;
    StackClusterPrivate_Push (stclpp ,StackClusterPrivate_ExtEnv( stclpp ) ) ;
    StackClusterPrivate_Push (stclpp , StackClusterPrivate_NamespaceProplist (
      stclpp ) ) ;
    PLOEnvC  * const mep  = ParaLispObjectToPLOEnv (
      StackClusterPrivate_ThisTop(stclpp )) ;
    StackClusterPrivate_SetEnv(stclpp , Environment_Env ( & mep -> env  ) ) ;
    // external field have GLOBAL variables
    // another namespace is not accepted
    StackClusterPrivate_SetExtEnv ( stclpp , Environment_Env ( & mep -> env  ) ) ;
    SetNamespaceProplist  ( stclpp , Environment_NamespacePropList (
      & mep -> env ) ) ;
    StackClusterPrivate_Push (stclpp , objp )  ; } // obj
  StackClusterPrivate_SetflagCont  ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse ;
Next  :
  SetNamespaceProplist  ( stclpp , StackClusterPrivate_PopValue (
    stclpp ) ) ;
  StackClusterPrivate_SetExtEnv ( stclpp ,
    StackClusterPrivate_PopValue ( stclpp ) ) ;
  StackClusterPrivate_SetEnv  ( stclpp , StackClusterPrivate_PopValue (
    stclpp ) ) ;
  return  TaskForThread_Analyse ;
}

# include "boolean.h"
  
// nil envp obj
TaskForThread Lisp2Dbimnilenvp2 ( StackClusterPrivate * const stclpp ) {
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop(stclpp)) ;
  if  ( Envp  ( xp ) ) {
    StackClusterPrivate_SetResult (stclpp , xp ) ;
    return  TaskForThread_Analyse ; 
  }
  StackClusterPrivate_SetResult (stclpp ,& plofalse.plo  ) ;
  return  TaskForThread_Analyse ; 
}

// private
# define  PLOEnv_Cout PLOEnv_Cout_pub
static  void  PLOEnv_Cout  ( PLOEnvC const * const me  ,
  FuncCoutToInterpretHuman  funccout  , PLOStream  const * const osp ,
  bool  flag2pprinted , Vector(StackPLOP)  * const historyp  ) {
  bool  flagpprinted  = flag2pprinted ;
  if  ( Printed3Points  ( & me -> plo  , osp  , &flagpprinted  , historyp ) )
    return  ;
  Vector_PushByValue(StackPLOP) ( historyp ,
    & ( ( PLOEnvC * ) me  ) -> plo ) ;
  PLOStreamPushWideCharArray  ( osp ,
    ( PArray_wchar_c ) { (wchar_t const(*)[])&L"Environment"});
  funccout  ( Environment_Env ( & me -> env )  , osp  , & flagpprinted ,
    historyp ) ;  
  VectorPop(StackPLOP)(historyp);    
}
  
void  PLOEnv_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  ,
  Vector  ( StackPLOP ) * const h )  {
  PLOEnv_Cout  ( ParaLispObjectToPLOEnvConst ( me ) ,
    ParaLispObject_CoutToInterpret , s , flag2pprinted , h ) ; 
}
  
void  PLOEnv_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  ,
  Vector  ( StackPLOP ) * const h ) {
  PLOEnv_Cout  ( ParaLispObjectToPLOEnvConst ( me ) ,
    ParaLispObject_CoutToHuman , s , flag2pprinted , h ) ;
}

int EnvironmentPLOLess  ( ParaLispObject * const a , ParaLispObject * const b  ) {
  if  ( a < b )
    return -1 ;
  if  ( a > b )
    return 1  ;
  return 0  ;
}

void  EnvironmentLayerClear ( EnvironmentLayer  * const ep ) {
  PassageClear  ( PassageToPub ( & ep -> passagec ) ) ;
  MapClear(PLOValueFlagConst)  ( & ep  ->  maplinks  ) ;
}

void  EnvironmentLayerInit  ( EnvironmentLayer  * const me  ) {
  PassageConstructor  ( PassageToPub ( & me -> passagec ) ) ;
  Lisp2DMapInit ( PLOValueFlagConst ) ( & me -> maplinks , EnvironmentPLOLess ) ;
}

void  EnvironmentLayerDestr ( EnvironmentLayer  * const me ) {
  Lisp2DMapDestr  ( PLOValueFlagConst ) ( & me -> maplinks  ) ;
  PassageDestructor ( PassageToPub ( & me -> passagec ) ) ;
}

EnvironmentLayer * EnvironmentLayerSet ( EnvironmentLayer  * const mep  ,
  EnvironmentLayer  const * const x ) {
  if  ( mep  not_eq  x )
    MapSet(PLOValueFlagConst)  ( & mep  ->  maplinks , & x ->  maplinks ) ;
  return  mep ;
}

void  EnvironmentLayer_InitCopy ( EnvironmentLayer  * const me ,
  EnvironmentLayer const * const x ) {
  PassageConstructor  ( PassageToPub ( & me -> passagec ) ) ;
  MapInitCopy ( PLOValueFlagConst ) ( & me -> maplinks , & x -> maplinks ) ;
}

void  PLOEnv_InitCopy  ( PLOEnvC * const me , PLOEnvC const * const ploenv ) {
  VirtualTypePointer  vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOEnv }  ;
  ParaLispObject_InitVTP  ( & me -> plo , vtp ) ;
  Environment_InitCopy  ( & me -> env , & ploenv -> env ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOEnv_constrstatIncr ( ) ;
# endif
}

LISP2D_PROTO_MAP_STACK_FREE (  PLOValueFlagConst )
LISP2DMAPSTACKDESTRBODY (  PLOValueFlagConst )
LISP2D_MAP_STACK_FREE_BODY (  PLOValueFlagConst )
LISP2DMAPSTACKSETBODY ( PLOValueFlagConst )
LISP2DMAPSTACKPUSHFRONTVALUEBODY  ( PLOValueFlagConst )
LISP2DMAPSTACKCLEARBODY ( PLOValueFlagConst )
LISP2DMAPINITCOPYBODY (  PLOValueFlagConst )
LISP2DMAPSTACKINITCOPYBODY (  PLOValueFlagConst )

inline  ParaLispObject  * CPLOcastPLO ( ParaLispObject  const * const p ) {
  return  ( ParaLispObject  * ) p ;
}
