// Lisp2D - Fraction number
// ©2011-23 by Alex Glebe

# include <math.h> // logl
# include <float.h> // LDBL_MIN
# include "minmaxint.h"
# include "define.h"
# include "fraction/private.h"
# ifdef  LISP2D_CONSTRSTAT
_Atomic ( size_t  ) Lisp2DFraction2Constrstat = 0 ;
# endif
# include "fraction/inlinepri.h"
# include "ulong/inline.h"
# include "fraction/public.h"

// +0/0 = +nan
void  FractionInit ( FractionPub * const f ) {
  Fraction2 * const fp = Fraction_toPrivate  ( f ) ;
  ULong_Init  ( ULongToPub  ( & fp  ->  numer ) ) ;
  ULong_Init  ( ULongToPub  ( & fp  ->  denom ) ) ;
  fp ->  flagNegative = false  ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat ;
# endif        
}
  
void  FractionConstrUInt ( FractionPub * const fp ,
  unsigned int const x ) {
  Fraction2 * const f = Fraction_toPrivate  ( fp ) ;
  f->flagNegative = false ;
  ULongConstrUInt ( ULongToPub  ( & f ->  numer ) , x ) ;
  ULongConstrUInt ( ULongToPub  ( & f ->  denom ) , 1U  ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif  
}

void  FractionConstrZero ( FractionPub * const fp ) {
  Fraction2 * const f = Fraction_toPrivate  ( fp ) ;
  f->flagNegative = false ;
  ULong_Init  ( ULongToPub  ( & f ->  numer ) ) ;
  ULongConstrUInt ( ULongToPub  ( & f ->  denom ) , 1U  ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif  
}
  
void  FractionConstrFNUIntUInt ( FractionPub * const mp ,
  bool const f  , unsigned int const n , unsigned int const d ) {
  Fraction2 * const m = Fraction_toPrivate  ( mp ) ;
  m->flagNegative = f ;
  ULongConstrUInt ( ULongToPub  ( & m ->  numer ) , n ) ; 
  ULongConstrUInt ( ULongToPub  ( & m ->  denom ) , d ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif      
  FractionSimplify ( m ) ;
}

void  FractionConstrCopy ( FractionPub * const mp ,
  FractionPub const * const xp ) {
  Fraction2 * const m = Fraction_toPrivate  ( mp ) ;
  Fraction2 const * const x = Fraction_toCPrivate  ( xp ) ;
# ifdef  LISP2D_DEBUG    
  if  ( m == x  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DFraction2ConstrCopy:m==x" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif            
  m ->  flagNegative  = x ->  flagNegative  ;
  ULong_Init  ( ULongToPub  ( & m ->  numer ) ) ;
  ULong_Init  ( ULongToPub  ( & m ->  denom ) ) ;
// ! to use Nump
  ULongSet ( ULongToPub(& m ->  numer) , ULongCToPub(& x ->  numer )) ;
  ULongSet ( ULongToPub(& m ->  denom) , ULongCToPub(& x ->  denom) ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif
}  
  
void  FractionConstrMove ( FractionPub * const mp ,
  FractionPub * const xp ) {
# ifdef  LISP2D_DEBUG
  if  ( mp == xp  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DFraction2ConstrMove:mp==xp" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  Fraction2 * const m = Fraction_toPrivate  ( mp ) ;
  Fraction2 * const x = Fraction_toPrivate  ( xp ) ;
  m->flagNegative = (x->flagNegative) ;
  ULongConstrMove ( ULongToPub  ( & m ->  numer ) ,
    ULongToPub  ( & x ->  numer ) ) ;
  ULongConstrMove ( ULongToPub  ( & m ->  denom ) ,
    ULongToPub  ( & x ->  denom ) ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat ;
# endif      
}

void  FractionConstrFNULongULong ( FractionPub * const mp ,
  bool const flagneg , ULongPub const * const cp , ULongPub const * const dp ) {
  Fraction2 * const m = Fraction_toPrivate  ( mp ) ;
  m ->  flagNegative  = flagneg ;
  ULong_Init  ( ULongToPub  ( & m ->  numer ) ) ;
  ULong_Init  ( ULongToPub  ( & m ->  denom ) ) ;
  ULongSet  ( ULongToPub  ( & m ->  numer ) , cp ) ;
  ULongSet  ( ULongToPub  ( & m ->  denom ) , dp ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif
  FractionSimplify ( m ) ;
}

void  FractionConstrMoveFNULongULong ( FractionPub * const mp ,
  bool const flagneg , ULongPub * const cp , ULongPub * const dp ) {
  Fraction2 * const m = Fraction_toPrivate  ( mp ) ;
  m->flagNegative = flagneg ;
  ULongConstrMove ( ULongToPub  ( & m ->  numer ) , cp ) ;
  ULongConstrMove ( ULongToPub  ( & m ->  denom ) , dp ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif      
  FractionSimplify ( m ) ;
}
  
void  FractionConstrCopyULong ( FractionPub * const mp , ULongPub const * const xp ) {
  Fraction2 * const m = Fraction_toPrivate  ( mp ) ;
  m->flagNegative = false ;
  ULong_Init  ( ULongToPub  ( & m ->  denom ) ) ;
  ULong_ConstrCopy ( ULongToPub  ( & m ->  numer ) , xp ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif      
  ULongSetUInt  ( ULongToPub ( &  m ->  denom ) , 1U  ) ;
}

void  FractionConstrULInt ( FractionPub * const fp , unsigned long int const x ) {
  Fraction2 * const f = Fraction_toPrivate  ( fp ) ;
  f->flagNegative = false ;
  ULong_Init  ( ULongToPub  ( & f ->  numer ) ) ;
  ULong_Init  ( ULongToPub  ( & f ->  denom ) ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif  
  ULongSetUInt  ( ULongToPub  ( & f ->  denom ) , 1U  ) ;
  ULongSetULInt ( ULongToPub ( & f  ->  numer ) , x ) ;
}

# include "cast.h"

void  FractionConstrLInt ( FractionPub * const fp , long int const li ) {
  Fraction2 * const f = Fraction_toPrivate  ( fp ) ;
  f->flagNegative = (li < 0) ;
  ULong_Init  ( ULongToPub  ( & f ->  numer ) ) ;
  ULongConstrUInt ( ULongToPub  ( & f ->  denom ) , 1U  ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif
  ULongSetULInt ( ULongToPub ( & f->numer ) , (f->flagNegative ?
    lint_cast_ulint(- li) :
    lint_cast_ulint(li) )) ;
}
  
void  FractionConstrUSize ( FractionPub * const fp , size_t const s ) {
  Fraction2 * const f = Fraction_toPrivate  ( fp ) ;
  f->flagNegative = false ;
  ULong_Init  ( ULongToPub  ( & f ->  numer ) ) ;
  ULongConstrUInt ( ULongToPub  ( & f ->  denom ) , 1U  ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DFraction2Constrstat;
# endif  
  ULongSetUSize ( ULongToPub  ( & f ->  numer ) , s ) ;
}    

# include "stack/public.h"

void  FractionSimplify ( Fraction2  * const me ) {
  if  ( ULongIs0 (ULongCToPub( & me -> denom ) ) ){
    if  ( ULongIs0 ( ULongCToPub(& me -> numer ) ))
      return ;
    if  ( not ( ULongIs1 ( ULongCToPub ( & me -> numer ) ) ) )
      ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
    return ;
  }
  if  ( ULongIs0 ( ULongCToPub(& me -> numer ) )) {
    if  ( not ( ULongIs1 ( ULongCToPub ( & me -> denom ) ) ) )
      ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
    return ;
  }
  if  ( ULongIs1 ( ULongCToPub ( & me -> denom ) ) )
    return ;
  if  ( ULongIs1 ( ULongCToPub ( & me -> numer ) ) )
    return ;
  ThreadStackPointerType const oldstack_mynod = Lisp2DThreadSP ( ) ;
  ULongPub * const mynod = StackNew_ULong ( ) ;
  ULong_ConstrCopy ( mynod , ULongCToPub ( & me -> numer ) ) ;
  ULongGCD ( mynod , ULongCToPub ( & me -> denom ) ) ;
  if  ( not ULongIs1 ( mynod ) ) {
    ULongDivConst ( ULongToPub ( & me -> numer ) , mynod ) ;
    ULongDivMove ( ULongToPub ( & me -> denom ) , mynod ) ;
  }
  Lisp2D_StackFree  ( oldstack_mynod ) ;
}

# include "boolean.h"

void FractionMultConstFraction ( FractionPub * const mep , FractionPub const * const pxp ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  Fraction2 const * const px = Fraction_toCPrivate  ( pxp ) ;
// ! to make vars nump , denomp  
  if  ( me ==  px ) {
    FractionSquare  ( FractionToPub( me) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 (ULongCToPub( & me -> denom ) ) ){
    if  ( ULongIs0 (ULongCToPub( & me -> numer ) ))
      goto  Exit  ;
    if  ( ULongIs0 ( ULongCToPub(&(px -> numer )) ) ){
      ULongSet0  ( ULongToPub(&me -> numer) ) ;
      goto  Exit  ;
    }
    bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
    ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 ( ULongCToPub(&(px -> denom )) ) ){
    ULongSet0  ( ULongToPub(&me -> denom ) );
    if  ( ULongIs0 ( ULongCToPub(& me -> numer ) ))
      goto  Exit  ;
    if  ( ULongIs0 ( ULongCToPub(&(px -> numer )) ) ){
      ULongSet0  (ULongToPub( &me -> numer ) );
      goto  Exit  ;
    }
    ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
    bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
    goto  Exit  ;
  }
  // all finite
  if  ( ULongIs0 ( ULongCToPub ( & me -> numer ) ) ) {
    ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 ( ULongCToPub(&(px -> numer )) ) ){
    ULongSet0  ( ULongToPub(&me -> numer ) );
    ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
    goto  Exit  ;
  }
  // all not 0
  { ThreadStackPointerType const oldstack_xchde = Lisp2DThreadSP ( ) ;
    ULongPub * const xchde = StackNew_ULong ( ) ;
    ULong_Init  ( xchde ) ;
    { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
      ULongPub * const gcd = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( gcd , ULongCToPub ( & me -> numer ) ) ;
      ULongGCD ( gcd , ULongCToPub ( & px -> denom ) ) ;
      { ThreadStackPointerType const oldstack_gcd2 = Lisp2DThreadSP ( ) ;
        ULongPub * const gcd2 = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( gcd2 , ULongCToPub ( & me -> denom ) ) ;
        ULongGCD ( gcd2 , ULongCToPub ( & px -> numer ) ) ;
        ULongDivConst ( ULongToPub ( & me -> numer ) , gcd ) ;
// while x is && -> x is not needed 
        ULongSet  ( xchde , ULongCToPub(& px ->  numer ) ) ;
        ULongDivConst ( xchde , gcd2 ) ;
        ULongMultiplyMove ( ULongToPub ( & me -> numer ) , xchde ) ;
        ULongDivMove ( ULongToPub ( & me -> denom ) , gcd2 ) ;
        Lisp2D_StackFree  ( oldstack_gcd2 ) ;
      } // gcd2
      ULongSet ( xchde , ULongCToPub(&px ->  denom));
      ULongDivMove ( xchde , gcd ) ; 
      Lisp2D_StackFree  ( oldstack_gcd ) ;
    } // gcd    
    ULongMultiplyMove ( ULongToPub ( & me -> denom ) , xchde ) ;
    Lisp2D_StackFree  ( oldstack_xchde ) ;
  } // xchde
  bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
Exit : ;
}

void FractionSquare  ( FractionPub * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongSquare ( ULongToPub ( & me -> numer ) ) ;
  ULongSquare ( ULongToPub ( & me -> denom ) ) ;
  me -> flagNegative  = false ;
  // simplifying is not needed
}

void FractionMultMoveFraction ( FractionPub * const  mep ,
  FractionPub * const pxp ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  Fraction2 * const px = Fraction_toPrivate  ( pxp ) ;
  if  ( me ==  px ) {
    FractionSquare  ( FractionToPub( me) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 (ULongCToPub( & me -> denom ) ) ){
    if  ( ULongIs0 (ULongCToPub( & me -> numer ) ) )
      goto  Exit  ;
    if  ( ULongIs0 (ULongCToPub( &(px -> numer )) ) ){
      ULongSet0  ( ULongToPub(&me -> numer ) );
      goto  Exit  ; }
    bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
    ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
    goto  Exit  ; 
  }
  if  ( ULongIs0 (ULongCToPub( &(px -> denom )) ) ){
    ULongSet0  ( ULongToPub(&me -> denom ) );
    if  ( ULongIs0 (ULongCToPub( & me -> numer ) ))
      goto  Exit  ;
    if  ( ULongIs0 (ULongCToPub( &(px -> numer )) ) ){
      ULongSet0  (ULongToPub( &me -> numer ) );
      goto  Exit  ; 
    }
    ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
    bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
    goto  Exit  ; 
  }
  // all finite
  if  ( ULongIs0 (ULongCToPub( & me -> numer ) ) ){
    ULongSet1 ( ULongToPub( & me -> denom ) ) ;
    goto  Exit  ; 
  }
  if  ( ULongIs0 (ULongCToPub( &(px -> numer )) ) ){
    ULongSet0  ( ULongToPub(&me -> numer ) );
    ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
    goto  Exit  ; 
  }
  // all not 0
  { ThreadStackPointerType const oldstack_xchde = Lisp2DThreadSP ( ) ;
    ULongPub * const xchde = StackNew_ULong ( ) ;
    ULong_Init ( xchde ) ;
    { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
      ULongPub * const gcd = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( gcd , ULongCToPub ( & me -> numer ) ) ;
      ULongGCD ( gcd , ULongCToPub ( & px -> denom ) ) ;
      { ThreadStackPointerType const oldstack_gcd2 = Lisp2DThreadSP ( ) ;
        ULongPub * const gcd2 = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( gcd2 , ULongCToPub ( & me -> denom ) ) ;
        ULongGCD ( gcd2 , ULongCToPub ( & px -> numer ) ) ;
        ULongDivConst ( ULongToPub ( & me -> numer ) , gcd ) ;
        ULongSwap ( xchde  , ULongToPub(& px ->  numer ) ) ;
        ULongDivConst ( xchde , gcd2 ) ;
        ULongMultiplyMove ( ULongToPub ( & me -> numer ) ,  xchde ) ;
        ULongDivMove ( ULongToPub ( & me -> denom ) , gcd2 ) ;
        Lisp2D_StackFree  ( oldstack_gcd2 ) ;
      } // gcd2
      ULongSwap(xchde,ULongToPub(&px -> denom));
      ULongDivMove ( xchde , gcd ) ;
      Lisp2D_StackFree  ( oldstack_gcd ) ;
    } // gcd    
    ULongMultiplyMove ( ULongToPub ( & me -> denom ) ,  xchde ) ;
    Lisp2D_StackFree  ( oldstack_xchde ) ;
  } // xchde
  bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
Exit  : ;
} 

void  FractionMultULInt ( FractionPub * const mep , unsigned long int const x ) {
  ThreadStackPointerType const oldstack_tmp = Lisp2DThreadSP ( ) ;
  FractionPub * const tmp = NewStackFraction ( ) ;
  FractionConstrULInt ( tmp , x ) ;
  FractionMultMoveFraction  ( mep , tmp ) ;
  Lisp2D_StackFree  ( oldstack_tmp ) ;
}
  
void  FractionDivULInt  ( FractionPub  * const mep , unsigned long int const x ) {
  ThreadStackPointerType const oldstack_tmp = Lisp2DThreadSP ( ) ;
  FractionPub * const tmp = NewStackFraction ( ) ;
  FractionConstrULInt ( tmp , x ) ;
  FractionDivFractionMove  ( mep , tmp) ;
  Lisp2D_StackFree  ( oldstack_tmp ) ;
}  
  
void  FractionMultULongConst  ( FractionPub  * const mep , ULongPub  const * const px ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  if  ( ULongIs0  (ULongCToPub( & me  ->  numer ) ))
    // (0/0)*0 = 0/0
    // (0/0)*1 = 0/0
    // (0/0)*2 = 0/0
    // (0/1)*0 = 0/1
    // (0/1)*1 = 0/1
    // (0/1)*2 = 0/1
    goto  Exit;
  if(ULongIs0(ULongCToPub(& me->denom ))){
    // (1/0)*0 = 0/0
    // (1/0)*1 = 1/0
    // (1/0)*2 = 1/0
    if(ULongIs0(px))
      ULongSet0(ULongToPub( &me->numer ));
    
    goto  Exit ;  
  }
  if(ULongIs0(px)){
    // (1/1)*0 = 0/1
    // (1/2)*0 = 0/1
    ULongSet0( ULongToPub(&me->numer ));
    ULongSet1 ( ULongToPub( & me  ->  denom ) ) ;
    goto  Exit ;   
  }
  if  ( ULongIs1  ( px ) )
    goto Exit ; 
  if  ( ULongIs1  ( ULongCToPub ( & me  -> denom )  ) ) {
    // (a/1)*x=((a*x)/1)
    ULongMultiplyConst ( ULongToPub ( & me  -> numer  ) , px ) ;
    goto Exit ;
  }
  // (a/b)*x=(a/(b/gcd))*(x/gcd)=((a*x/gcd)/(b/gcd))
  { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
    ULongPub * const gcd = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( gcd , ULongCToPub ( & me  -> denom  ) ) ;
    ULongGCD ( gcd , px ) ;
    { ThreadStackPointerType const oldstack_x2 = Lisp2DThreadSP ( ) ;
      ULongPub * const x2 = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( x2 , px ) ;
      ULongDivConst ( x2 , gcd ) ;
      ULongMultiplyMove ( ULongToPub ( & me -> numer ) , x2 ) ;
      Lisp2D_StackFree  ( oldstack_x2 ) ;
    }
    ULongDivMove ( ULongToPub ( & me -> denom ) , gcd ) ;
    Lisp2D_StackFree  ( oldstack_gcd ) ;
  } // gcd
Exit  : ; 
}

void  FractionDivFractionConst  ( FractionPub  * const mep ,
  FractionPub const * const pxp  ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  Fraction2 const * const px = Fraction_toCPrivate  ( pxp ) ;
  if  ( me ==  px ) {
    if  ( ULongIs0 (ULongCToPub( & me -> numer ) ) ){
      ULongSet0  (ULongToPub( &me -> denom ) );
      goto  Exit  ;
    }
    if  ( ULongIs0 (ULongCToPub( & me -> denom ) ) ){
      ULongSet0  (ULongToPub( &me -> numer ) );
      goto  Exit  ;
    }  
    ULongSet1 ( ULongToPub  ( & me ->  numer  ) ) ;
    ULongSet1 ( ULongToPub  ( & me ->  denom  ) ) ;
    me ->  flagNegative  = false ;       
    goto  Exit  ;
  }
  if  ( ULongIs0 (ULongCToPub( & me ->  denom ) ) ){
    if  ( ULongIs0 (ULongCToPub( & me ->  numer ) ))
      goto  Exit  ;
    if  ( ULongIs0 (ULongCToPub( &(px -> denom )) ) ){
      ULongSet0  (ULongToPub( &me ->  numer ) );
      goto  Exit  ;
    }
    if  ( not ( ULongIs0 (ULongCToPub( &(px -> numer )) ) ))
      bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
    ULongSet1 ( ULongToPub  ( & me ->  numer  ) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 (ULongCToPub( &(px -> numer )) ) ){
    ULongSet0  (ULongToPub( &me ->  denom ) );
    if  ( ULongIs0 (ULongCToPub( & me ->  numer ) ))
      goto  Exit  ;
    if  ( ULongIs0 (ULongCToPub( &(px -> denom )) ) ){
      ULongSet0  (ULongToPub( &me ->  numer ) );
      goto  Exit  ;
    }
    ULongSet1 ( ULongToPub  ( & me ->  numer  ) ) ;
    goto  Exit  ;
  }
  // all finite
  if  ( ULongIs0 (ULongCToPub( & me ->  numer ) ) ){
    ULongSet1 ( ULongToPub  ( & me ->  denom  ) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 (ULongCToPub( &(px -> denom )) ) ){
    ULongSet0  (ULongToPub( &me ->  numer ) );
    ULongSet1 ( ULongToPub  ( & me ->  denom  ) ) ;
    goto  Exit  ;
  }
  // all not 0
  { ThreadStackPointerType const oldstack_xchde = Lisp2DThreadSP ( ) ;
    ULongPub * const xchde = StackNew_ULong ( ) ;
    ULong_Init ( xchde ) ;
    { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
      ULongPub * const gcd = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( gcd , ULongCToPub ( & me -> numer ) ) ;
      ULongGCD ( gcd , ULongCToPub ( & px -> numer ) ) ;
      { ThreadStackPointerType const oldstack_gcd2 = Lisp2DThreadSP ( ) ;
        ULongPub * const gcd2 = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( gcd2 , ULongCToPub ( & me -> denom ) ) ;
        ULongGCD ( gcd2 , ULongCToPub ( & px -> denom ) ) ;
        ULongDivConst ( ULongToPub ( & me ->  numer ) , gcd ) ;
// while x is && -> x is not needed    
        ULongSet  ( xchde , ULongCToPub(& px -> denom ) );
        ULongDivConst ( xchde , gcd2 ) ;
        ULongMultiplyMove ( ULongToPub ( & me ->  numer ) , xchde ) ;
        ULongDivMove ( ULongToPub ( & me ->  denom ) , gcd2 ) ;
        Lisp2D_StackFree  ( oldstack_gcd2 ) ;
      } // gcd2
      ULongSet ( xchde ,ULongCToPub(&px ->  numer));
      ULongDivMove ( xchde , gcd ) ;
      Lisp2D_StackFree  ( oldstack_gcd ) ;
    } // gcd
    ULongMultiplyMove ( ULongToPub ( & me ->  denom ) , xchde ) ;
    Lisp2D_StackFree  ( oldstack_xchde ) ;
  }  // xchde
  bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
Exit  :  ;
}
  
void FractionDivFractionMove  ( FractionPub  * const mep ,
  FractionPub * const pxp  ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  Fraction2 * const px = Fraction_toPrivate  ( pxp ) ;
  if  ( me ==  px ) {
    if  ( ULongIs0 (ULongCToPub( & me -> numer ) ) ){
      ULongSet0  (ULongToPub( &me -> denom ) );
      goto  Exit  ;
    }
    if  ( ULongIs0 (ULongCToPub( & me -> denom ) ) ){
      ULongSet0  (ULongToPub( &me -> numer ) );
      goto  Exit  ;
    }  
    ULongSet1 ( ULongToPub  ( & me ->  numer  ) ) ;
    ULongSet1 ( ULongToPub  ( & me ->  denom  ) ) ;
    me ->  flagNegative  = false ;       
    goto  Exit  ; 
  }
  if  ( ULongIs0 (ULongCToPub( & me ->  denom ) ) ){
    if  ( ULongIs0 (ULongCToPub( & me ->  numer ) ))
      goto  Exit  ;
    if  ( ULongIs0 (ULongCToPub( &(px -> denom )) ) ){
      ULongSwap(ULongToPub(&me ->  numer),ULongToPub(&px -> denom));
      goto  Exit  ;
    }
    if  ( not ( ULongIs0 (ULongCToPub( &(px -> numer )) ) ))
      bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
    ULongSet1 ( ULongToPub  ( & me ->  numer  ) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 (ULongCToPub( &(px -> numer )) ) ){
    ULongSwap(ULongToPub(&me ->  denom),ULongToPub(&px -> numer));
    if  ( ULongIs0 (ULongCToPub( & me ->  numer) ))
      goto  Exit  ;
    if  ( ULongIs0 ( ULongCToPub(&(px -> denom )) )) {
      ULongSwap(ULongToPub(&me ->  numer),ULongToPub(&px -> denom));
      goto  Exit  ;
    }
    ULongSet1 ( ULongToPub  ( & me ->  numer  ) ) ;
    goto  Exit  ;
  }
  // all finite
  if  ( ULongIs0 ( ULongCToPub(& me ->  numer ) ) ){
    ULongSet1 ( ULongToPub  ( & me ->  denom  ) ) ;
    goto  Exit  ;
  }
  if  ( ULongIs0 ( ULongCToPub(&(px -> denom )) ) ){
    ULongSwap(ULongToPub(&me ->  numer),ULongToPub(&px -> denom));
    ULongSet1 ( ULongToPub  ( & me ->  denom  ) ) ;
    goto  Exit  ;
  }
  // all not 0
  { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
    ULongPub * const gcd = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( gcd , ULongCToPub ( & me -> numer ) ) ;
    ULongGCD ( gcd , ULongCToPub ( & px -> numer ) ) ;
    { ThreadStackPointerType const oldstack_gcd2 = Lisp2DThreadSP ( ) ;
      ULongPub * const gcd2 = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( gcd2 , ULongCToPub ( & me -> denom ) ) ;
      ULongGCD ( gcd2 , ULongCToPub ( & px -> denom ) ) ;
      ULongDivConst ( ULongToPub ( & me ->  numer ) , gcd ) ;
// while x is && -> x is not needed    
      ULongDivConst ( ULongToPub ( & px -> denom ) , gcd2 ) ;
      ULongMultiplyMove ( ULongToPub ( & me ->  numer ) , ULongToPub ( & px -> denom ) ) ;
      ULongDivMove ( ULongToPub ( & me ->  denom ) , gcd2 ) ;
      Lisp2D_StackFree  ( oldstack_gcd2 ) ;
    } // gcd2
    ULongDivMove ( ULongToPub ( & px -> numer ) , gcd ) ; 
    Lisp2D_StackFree  ( oldstack_gcd ) ;
  } // gcd
  ULongMultiplyMove ( ULongToPub ( & me ->  denom ) , ULongToPub ( &  px -> numer ) ) ;
  bool_xor_eq ( & me  ->  flagNegative  , px  ->  flagNegative  ) ;
Exit  : ;
}  
  
void FractionSetFraction ( FractionPub  * const mep ,
  FractionPub  const * const xp ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  Fraction2 const * const x = Fraction_toCPrivate  ( xp ) ;
  if  ( me not_eq  x  ) {
    me -> flagNegative  = x -> flagNegative ;
    ULongSet  ( ULongToPub(& me -> numer ) , ULongCToPub(& x -> numer  ) );
    ULongSet  ( ULongToPub(& me -> denom ) , ULongCToPub(& x -> denom  ) ); 
  }
}

// ! to use FractionMoveFraction to optimize
    
void  FractionSwapFraction  ( FractionPub * const mep ,
  FractionPub * const xp ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  Fraction2 * const x = Fraction_toPrivate  ( xp ) ;
  ULongSwap  ( ULongToPub(&me-> numer) ,ULongToPub(&x -> numer )) ;
  ULongSwap  ( ULongToPub(&me-> denom ),ULongToPub(&x -> denom ) );
  { bool const  tmp=me-> flagNegative;
    me-> flagNegative=x -> flagNegative;
    x -> flagNegative=tmp;
  }
}    

void  FractionMoveFraction  ( FractionPub * const mep ,
  FractionPub * const xp ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  Fraction2 * const x = Fraction_toPrivate  ( xp ) ;
  ULongMove ( ULongToPub(&  me  -> numer) , ULongToPub( & x -> numer )) ;
  ULongMove ( ULongToPub(&  me  -> denom) , ULongToPub( & x -> denom ) );
  me  -> flagNegative = x -> flagNegative ;
}

void  FractionSetULong ( FractionPub * const mep , ULongPub const * const c ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = false ;
  ULongSet (  ULongToPub  ( & me -> numer ) , c ) ;
  ULongSet1  ( ULongToPub ( & me -> denom ) ) ;
}

void  FractionSwapULong  ( FractionPub * const mep , ULongPub  * const c ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = false ;
  ULongSwap ( ULongToPub  ( & me -> numer ) , c ) ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
}

void  FractionMoveULong  ( FractionPub * const mep , ULongPub  * const c ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = false ;
  ULongMove ( ULongToPub  ( & me -> numer ) , c ) ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
}

void  FractionSetUInt ( FractionPub * const mep ,
  unsigned  int const x ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = false ;
  ULongSetUInt (  ULongToPub  ( & me -> numer ) , x ) ;
  ULongSet1  (  ULongToPub  ( & me -> denom ) ) ;
}  
  
// +n/d  
void  FractionSetUIntUInt ( FractionPub * const mep ,
  unsigned int const num , unsigned int const denom ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative = false ;
  ULongSetUInt  ( ULongToPub  ( & me -> numer ) , num  ) ;
  ULongSetUInt  ( ULongToPub  ( & me -> denom ) , denom  ) ;
  FractionSimplify ( me ) ;
}
  
// ±n/d  
void  FractionSetSignUIntUInt( FractionPub * const mep ,
  bool const flagneg, unsigned int const  num , unsigned int const  denom) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = flagneg;
  ULongSetUInt (  ULongToPub  ( & me -> numer ) , num ) ;
  ULongSetUInt (  ULongToPub  ( & me -> denom ) , denom ) ;
  FractionSimplify ( me ) ; 
}
  
// for LoadChastDelit2
void  FractionClearSwapSignULongULong  ( FractionPub * const mep ,
  bool const  flagneg , ULongPub * const nump , ULongPub * const denomp  ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = flagneg ;
  ULongClearSwap  ( ULongToPub ( & me -> numer ) , nump ) ;
  ULongClearSwap  ( ULongToPub ( & me -> denom ) , denomp ) ;
  FractionSimplify ( me  ) ; 
}

// for LoadChastDelit2
void  FractionClearMoveSignULongULong  ( FractionPub * const mep ,
  bool const  flagneg , ULongPub * const nump , ULongPub * const denomp  ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = flagneg ;
  ULongClearMove  ( ULongToPub ( & me -> numer ) , nump ) ;
  ULongClearMove  ( ULongToPub ( & me -> denom ) , denomp ) ;
  FractionSimplify ( me  ) ;
}

// for LoadChastDelit2
void  FractionSwapSignULongULong ( FractionPub * const mep ,
  bool const  flagneg , ULongPub * const nump , ULongPub * const denomp ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = flagneg ;
  ULongSwap ( ULongToPub  ( & me -> numer) , nump ) ;
  ULongSwap ( ULongToPub  ( & me -> denom ) , denomp ) ;
  FractionSimplify  ( me  ) ; 
}

// for LoadChastDelit2
void  FractionMoveSignULongULong ( FractionPub * const mep ,
  bool const  flagneg , ULongPub * const nump , ULongPub * const denomp ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = flagneg ;
  ULongMove ( ULongToPub  ( & me -> numer) , nump ) ;
  ULongMove ( ULongToPub  ( & me -> denom ) , denomp ) ;
  FractionSimplify  ( me  ) ;
}

void FractionMoveNumPositiveULongULong  ( FractionPub * const mep ,
  ULongPub * const c , ULongPub const * const  d  ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = false;
  ULongMove ( ULongToPub(&me -> numer) , c  ) ;
  ULongSet (  ULongToPub(&me -> denom), d ) ;
  FractionSimplify  ( me ) ;
}
  
void FractionMoveDenomPositiveULongULong  ( FractionPub * const mep ,
  ULongPub const * const c , ULongPub * const d ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = false;
  ULongSet (ULongToPub(&me -> numer), c ) ;
  ULongMove(ULongToPub(&me -> denom), d ) ;
  FractionSimplify  ( me ) ;
}
  
void FractionMulUInt  ( FractionPub * const mep ,unsigned int const  x) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  switch  ( x ) {
  case  0 :
    // 0/0 -> 0/0 , 1/0 -> 0/0 , 0 -> 0 , 1/1 -> 0/1 , 1/2 -> 0/1 , 2/1 -> 0/1
    ULongSet0  (ULongToPub( &me -> numer ) );
    if  ( not ( ULongIs0 (ULongCToPub( & me -> denom ) ) ))
      ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
    break ;
  case  1 : 
    break;
  case  2 :
    if  ( ULongIs0 ( ULongCToPub(& me-> denom ) ) )
      break ;
    // 0/0 -> 0/0 , 1/0 -> 1/0 , 0 -> 0 , 1/1 -> 2/1 , 1/2 -> 1/1 , 2/1 -> 4/1
    if  ( not ( ULongIs0 ( ULongCToPub(& me -> numer ) ) ) ){
      if  ( ULongEvenP ( ULongCToPub ( & me -> denom ) ) )
        ULongDiv2 ( ULongToPub  ( & me -> denom ) ) ;
      else
        ULongMul2  ( ULongToPub ( & me -> numer ) ) ; 
    }
    break ;
  default :
    if  ( ULongIs0 ( ULongCToPub(& me-> denom ) ) )
      break ;
    if  ( ULongIs0 ( ULongCToPub(& me-> numer ) ) )
      break ;
    unsigned  int  x2 = x  ;
    { unsigned  int const del  =
        ULongGCD3 ( ULongToPub ( & me -> denom ) , x2  )  ;
      x2  /=  del ;
      ULongDividingAndReturnsRemainder ( ULongToPub ( & me -> denom ) , del ) ; 
    }
    ULongMultiplyUInt ( ULongToPub ( & me -> numer ) , x2 ) ; 
  } 
}
    
void  FractionMul2  ( FractionPub * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  // 0/0 -> 0/0 , 1/0 -> 1/0 , 0 -> 0 , 1/1 -> 2/1 , 1/2 -> 1/1 , 2/1 -> 4/1
  if(not  (ULongIs0(ULongCToPub(& me-> numer )))){
    if  ( ULongEvenP  ( ULongCToPub ( & me -> denom ) ) )
      ULongDiv2 ( ULongToPub  ( & me  -> denom  ) ) ;
    else
      ULongMul2 ( ULongToPub ( & me -> numer ) ) ;
  }
}
    
void  FractionDiv2  ( FractionPub * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  // 0/0 -> 0/0 , 1/0 -> 1/0 , 0 -> 0 , 1/1 -> 1/2 , 1/2 -> 1/4 , 2/1 -> 1/1
  if  ( ULongEvenP (  ULongCToPub ( & me  -> numer ) ) )
    ULongDiv2 ( ULongToPub  ( & me  -> numer  ) ) ;
  else
    ULongMul2  ( ULongToPub ( & me -> denom ) ) ;
}    
    
void FractionDivUInt ( FractionPub * const mep , unsigned int x ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  if  ( x ==  0U ) {
    if  ( ULongIs0 ( ULongCToPub(& me -> denom ) ))
      goto  Out ;
    ULongSet0  (ULongToPub( &me-> denom ) );
    if  ( ULongIs0 ( ULongCToPub(& me-> numer ) ) )
      goto  Out ;
    ULongSet1 ( ULongToPub  ( & me  -> numer  ) ) ;
    goto  Out ;
  }
  if  ( x ==  1U )
    goto  Out ;
  if(x == 2U) {
    FractionDiv2  (FractionToPub(me));
    goto Out;  
  }
  // x>1
  if  ( ULongIs0 ( ULongCToPub(& me-> numer ) ))
    goto  Out ;  
  { unsigned  int const gc  = ULongGCD3 ( ULongCToPub ( & me -> numer ) , x ) ;
    if  ( gc > 1U  ) {
      ULongDividingAndReturnsRemainder ( ULongToPub ( & me -> numer ) , gc  ) ;
      x  /=  gc  ;
    }
  }
  ULongMultiplyUInt ( ULongToPub ( & me -> denom ) , x ) ;
Out : ;
}

void  FractionSetInt  ( FractionPub * const mep ,int const x) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  { unsigned  int    uitmp  ;
    if(x  < 0)  {
      me -> flagNegative  = true  ;
      uitmp = int_cast_uint (-  x);
    } else {
      me -> flagNegative  = false ;
      uitmp = int_cast_uint (x); 
    }
    ULongSetUInt  ( ULongToPub  ( & me -> numer ) , uitmp ) ;  
  }
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ; 
}
  
void  FractionSet0  ( FractionPub * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongSet0(ULongToPub( &me-> numer ));
  ULongSet1 ( ULongToPub  ( & me  -> denom  ) ) ;
}
  
void  FractionSet1  ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
  me -> flagNegative  = false; 
}
  
# ifdef LISP2D_CONSTRSTAT  
size_t  FractionconstrstatGetValue ( void )  {
  return  Lisp2DFraction2Constrstat ; 
}
# endif

bool  FractionIsNaN ( FractionPub const * const mep ) {
  Fraction2 const * const me = Fraction_toCPrivate  ( mep ) ;
  return  (ULongIs0(ULongCToPub(& me -> numer ) ) ? 
    ULongIs0(ULongCToPub(& me -> denom ) ):
    false);
}  
  
bool  FractionIs0 ( FractionPub const * const mep ) {
  Fraction2 const * const me = Fraction_toCPrivate  ( mep ) ;
  return  ( ULongIs0 ( ULongCToPub(& me -> numer )) and 
    ( not ULongIs0 ( ULongCToPub(& me -> denom ) ) ) ); 
}      

// 0/0    
void  FractionSetNaN  ( FractionPub * const mep) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongSet0(ULongToPub( &me -> numer ));
  ULongSet0(ULongToPub( &me -> denom )); 
}
  
// +1/0
void  FractionSetInf  ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  me -> flagNegative  = false;
  ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
  ULongSet0(ULongToPub( &me -> denom ));
}
  
// -2/3 -> -3/2  ==  this=1/this
void  FractionOverTurn  ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongSwap( ULongToPub(&me-> numer) ,ULongToPub( &me-> denom ));
}
  
void  FractionFloor ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongPub * const mydelit = ULongToPub ( & me -> denom ) ;
  if  ( ULongIs0 ( mydelit ) )
    goto  Exit ;
  { ULongPub * const mychast = ULongToPub ( & me -> numer ) ;
    if  ( ULongIs0 ( mychast ) )
      goto  Exit  ;
    if  ( ULongIs1 ( mydelit ) )
      goto  Exit  ;
    // this is not integer
    if  ( ULongIs1 ( mychast ) ) {
      if  ( not ( me -> flagNegative ) )
        ULongSet0  ( mychast ) ;
      ULongSet1 ( mydelit ) ;
      goto  Exit  ; 
    }
    ULongDivConst ( mychast , mydelit ) ;
    ULongSet1 ( mydelit ) ;
    if  ( me -> flagNegative  )
      ULongAdd1 ( mychast ) ;
  }
Exit : ;
}

void  FractionTrunc ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongPub * const mydelit = ULongToPub ( & me -> denom ) ;
  if  ( ULongIs0 ( mydelit ) )
    goto  Exit ;
  { ULongPub * const mychast = ULongToPub ( & me -> numer ) ;
    if  ( ULongIs0 ( mychast ) )
      goto  Exit  ;
    if  ( ULongIs1 ( mydelit ) )
      goto  Exit  ;
    // this is not integer
    if  ( ULongIs1 ( mychast ) ) {
      FractionSet0  ( mep ) ;
      goto  Exit  ; 
    }  
    ULongDivConst ( mychast , mydelit ) ;
  }
  ULongSet1 ( mydelit ) ;
Exit  : ; 
}

// sign didn't touch  
void  FractionAbsFloor  ( FractionPub  * const  mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongPub * const mydelit = ULongToPub ( & me -> denom ) ;
  if  ( ULongIs0 ( mydelit ) )
    goto  Exit ;
  { ULongPub * const mychast = ULongToPub ( & me -> numer ) ;
    if  ( ULongIs0 ( mychast ) )
      goto  Exit  ;
    if  ( ULongIs1 ( mydelit ) )
      goto  Exit  ;
    ULongDivConst ( mychast , mydelit ) ;
  }
  ULongSet1 ( mydelit )  ;
Exit  : ;
}

void  FractionCeil  ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongPub * const mydelit = ULongToPub ( & me -> denom ) ;
  if  ( ULongIs0 ( mydelit ) )
    goto  Exit  ;
  { ULongPub * const mychast = ULongToPub ( & me -> numer ) ;
    if  ( ULongIs0 ( mychast ) )
      goto  Exit  ;
    if  ( ULongIs1 ( mydelit ) )
      goto  Exit  ;
    // this is not integer
    if  ( ULongIs1 ( mychast ) ) {
      if  ( me -> flagNegative  ) 
        ULongSet0  ( mychast ) ;
      ULongSet1 ( mydelit ) ;
      goto  Exit  ; 
    }
    ULongDivConst ( mychast , mydelit ) ;
    ULongSet1 ( mydelit ) ;
    if  ( not ( me -> flagNegative ) ) 
      ULongAdd1 ( mychast ) ;
  }
Exit  : ; 
}

void  FractionAdd1  ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongPub * const mydelit = ULongToPub(& me -> denom );
  if  ( ULongIs0 ( mydelit ) )
    goto  Exit  ;
  { ULongPub * const mychast = ULongToPub ( & me -> numer ) ;
    // delit>0
    if  ( ULongIs0 ( mychast ) ) {
      ULongSet1  ( mychast ) ;
      return ; 
    }
    if  ( ULongIs1 ( mydelit ) ) {
      if  ( not ( me -> flagNegative ) )
        ULongAdd1 ( mychast ) ;
      else  
        ULongSub1 ( mychast ) ;
      goto  Exit  ;
    }
    // **
    if  ( not ( me -> flagNegative ) ) 
      ULongAddConst ( mychast , mydelit ) ;
    else if  ( ULongLess ( mychast , mydelit ) ) {
      { ThreadStackPointerType const oldstack_c = Lisp2DThreadSP ( ) ;
        ULongPub * const c = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( c , mydelit ) ;
        ULongDifferenceEq ( c , mychast ) ;
        ULongMove ( mychast , c ) ;
        Lisp2D_StackFree  ( oldstack_c ) ;
      }
      me -> flagNegative  = false ; 
    } else
      ULongDifferenceEq ( mychast , mydelit ) ;
  } // mychastp
Exit  : ; 
}

void  FractionSub1  ( FractionPub  * const mep ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongPub * const mydelit = ULongToPub(& me -> denom );
  if  ( ULongIs0 ( mydelit  ))
    goto  Exit  ;
  { ULongPub * const mychast = ULongToPub ( & me -> numer ) ;
    // delit>0
    if  ( ULongIs0 ( mychast ) ) {
      FractionSetInt  ( FractionToPub(me) ,-1);
      goto  Exit  ; 
    }
    if  ( ULongIs1 ( mydelit ) ) {
      if  ( not ( me -> flagNegative ) )
        ULongSub1 ( mychast ) ;
      else 
        ULongAdd1 ( mychast ) ;
      goto  Exit  ; 
    }
    if  ( me -> flagNegative  )   
      ULongAddConst ( mychast , mydelit ) ;
    else if  ( ULongLess ( mychast ,  mydelit ) ) {
      { ThreadStackPointerType const oldstack_c = Lisp2DThreadSP ( ) ;
        ULongPub * const c = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( c , mydelit ) ;
        ULongDifferenceEq ( c , mychast ) ;
        ULongMove ( mychast , c ) ;
        Lisp2D_StackFree  ( oldstack_c ) ;
      }
      me -> flagNegative  = true  ; 
    } else
      ULongDifferenceEq ( mychast , mydelit ) ;
  }
Exit  : ; 
}

void  FractionSetLInt ( FractionPub * const mep , long  int const l ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
  if  ( l < 0 ) {
    ULongSetULInt ( ULongToPub ( & me -> numer ) , lint_cast_ulint ( - l ) )  ;
    me -> flagNegative  = true  ; 
    return  ;
  }
  me -> flagNegative  = false ;  
  if  ( l ==  0 ) {
    ULongSet0  (ULongToPub( &me -> numer ) );
    return  ;
  }
  ULongSetULInt ( ULongToPub ( & me -> numer ) , lint_cast_ulint ( l ) ) ;
} 
  
// this-=[this]
void  FractionMod ( FractionPub * const  mep ) {
  ThreadStackPointerType const oldstack_z = Lisp2DThreadSP ( ) ;
  FractionPub * const z = NewStackFraction ( ) ;
  FractionConstrCopy ( z , mep ) ;
  FractionFloor (z);
  FractionSubtractMove  ( mep ,  z ) ;
  Lisp2D_StackFree  ( oldstack_z ) ;
}
  
bool  FractionIsBadSizeValue  ( FractionPub  const * const x ,
  size_t  * const resultp) {
  if  ( not  ULongIs1 ( FractionCDenomp  ( x ) ) )
    return  true;
  if(ULongIs0(FractionCNump(x))){
    (*resultp)  = 0;
    return  false;
  }
  if(FractionIsNegative(x))
    return  true;
  return  ULongIsBADSizeValue  ( FractionCNump  ( x ) ,
    resultp  ) ;
}

bool  FractionLessUInt ( FractionPub  const * const  mep  ,
  unsigned int const  ux ) {
  if  ( ULongIs0 ( FractionCDenomp(mep ) ) ){
    if  ( ULongIs0 ( FractionCNump(mep )) )
      return  false ;
    goto  Negative  ; 
  }
  // this is finite
  if  ( ULongIs0 ( FractionCNump(mep)) )
    return  0 < ux  ;
  // me is not 0
  if  ( ux  ==  0 ) 
    goto  Negative  ;

  if  ( FractionIsNegative (  mep )  )
    return  true  ;
  ThreadStackPointerType const oldstack_d = Lisp2DThreadSP ( ) ;
  ULongPub * const d = StackNew_ULong ( ) ;
  ULong_ConstrCopy ( d , FractionCDenomp  ( mep ) ) ;
  ULongMultiplyUInt ( d , ux ) ;
  bool const res = ULongLess ( FractionCNump ( mep ) , d ) ;
  Lisp2D_StackFree  ( oldstack_d ) ;
  return  res ;
Negative  :
  return  FractionIsNegative (mep) ; 
}
  
bool  FractionMoreEqUSize ( FractionPub const * const mep ,
  size_t const x )  {
  ThreadStackPointerType const oldstack_tmp = Lisp2DThreadSP ( ) ;
  FractionPub * const tmp = NewStackFraction  ( ) ;
  FractionConstrUSize ( tmp , x ) ;
  bool const  res = (FractionMoreEq (mep,tmp) ==  Lisp2DBoolTrue);
  Lisp2D_StackFree  ( oldstack_tmp ) ;
  return res ;
}
  
bool  FractionMoreUSize ( FractionPub  const * const mep ,
  size_t const x){
  ThreadStackPointerType const oldstack_tmp = Lisp2DThreadSP ( ) ;
  FractionPub * const tmp = NewStackFraction ( ) ;
  FractionConstrUSize ( tmp , x ) ;
  bool const res = ( FractionMore ( mep , tmp ) ==  Lisp2DBoolTrue ) ;
  Lisp2D_StackFree  ( oldstack_tmp ) ;
  return res ;
}  
  
bool  FractionLessUSize ( FractionPub  const * const mep,size_t const x){
  ThreadStackPointerType const oldstack_tmp = Lisp2DThreadSP ( ) ;
  FractionPub * const tmp = NewStackFraction ( ) ;
  FractionConstrUSize ( tmp , x ) ;
  bool const res = ( FractionLess ( mep , tmp ) ==  Lisp2DBoolTrue ) ;
  Lisp2D_StackFree  ( oldstack_tmp ) ;
  return res ;
}  
  
BoolType  FractionLess  ( FractionPub const * const mep ,
  FractionPub  const * const xp){
  if (   mep  ==      xp )
    return  FractionIsNaN ( mep ) ?
      Lisp2DBoolUndef :
      Lisp2DBoolFalse ;
  ULongPub const * const mydelit = FractionCDenomp(mep) ;
  ULongPub const * const xdelit = FractionCDenomp(xp) ;
  ULongPub const * const mychast0 = FractionCNump(mep);
  ULongPub const * const xchast = FractionCNump(xp) ;
  if( ULongIs0( mydelit )){
    if( ULongIs0(mychast0))
      return  Lisp2DBoolUndef;
    if(not   FractionIsNegative(mep))
      // +inf <
      return  Lisp2DBoolFalse;
    // -inf <
    if( ULongIs0(xdelit)) {
      if( ULongIs0(xchast))
        return  Lisp2DBoolUndef;
      return  (not   FractionIsNegative(xp)) ?
        Lisp2DBoolTrue :
        Lisp2DBoolFalse;
    }
    return  Lisp2DBoolTrue;
  }
  // me is finite
  if( ULongIs0(xdelit)){
    if( ULongIs0(xchast))
      return  Lisp2DBoolUndef;
    return  (not   FractionIsNegative(xp)) ?
      Lisp2DBoolTrue :
      Lisp2DBoolFalse;
  }
  // finite numbers
  if( ULongIs0(mychast0))
    return   ULongIs0(xchast)  ?
      Lisp2DBoolFalse :
      ((not   FractionIsNegative(xp)) ?
        Lisp2DBoolTrue :
        Lisp2DBoolFalse);
  // me is not 0
  if( ULongIs0(xchast))
    return   FractionIsNegative(mep) ?
      Lisp2DBoolTrue :
      Lisp2DBoolFalse;
  // all not 0
  if(not   FractionIsNegative(mep)){
    if( FractionIsNegative(xp)) 
      return  Lisp2DBoolFalse;
  } else if  (not   FractionIsNegative(xp))
    return  Lisp2DBoolTrue;
  // sign == x.sign  
  /*
 !!
 xc < yc && xd >= yd  =>  xc/xd < yc/yd => OK
 xc > yc && xd <= yd  =>  xc/xd > yc/yd => False
 xc == yc && xd > yd  =>  xc/xd < yc/yd => OK
 xc == yc && xd <= yd  =>  xc/xd < yc/yd => False
 */  
  if  ( ULongLess ( mychast0 , xchast ) ) {
    if(ULongMoreOrEq( mydelit , xdelit)){
      if( FractionIsNegative(mep))  
        return  Lisp2DBoolFalse;
      return  Lisp2DBoolTrue;
    }
  } else if(ULongMore(mychast0 , xchast)){
      if(ULongLessOrEq( mydelit , xdelit)){
        if( FractionIsNegative(mep))     
          return  Lisp2DBoolTrue;
        return  Lisp2DBoolFalse;
      }
  } else if(ULongMore( mydelit , xdelit)){
        if( FractionIsNegative(mep))   
          return  Lisp2DBoolFalse;
        return  Lisp2DBoolTrue;
  }
  ThreadStackPointerType const oldstack_mychast = Lisp2DThreadSP ( ) ;
  ULongPub * const mychast = StackNew_ULong ( ) ;
  ULong_ConstrCopy ( mychast , mychast0 ) ;
  ULongMultiplyConst (  mychast , xdelit ) ;
  ULongPub * const mychast2 = StackNew_ULong ( ) ;
  ULong_ConstrCopy ( mychast2 , xchast ) ;
  ULongMultiplyConst  ( mychast2 , mydelit ) ;
  BoolType const res = (not  FractionIsNegative(mep)) ?
    ( ULongLess ( mychast , mychast2 ) ?
      Lisp2DBoolTrue :
      Lisp2DBoolFalse) :
    ( ULongLess ( mychast2 , mychast ) ? 
      Lisp2DBoolTrue :
      Lisp2DBoolFalse);
  Lisp2D_StackFree  ( oldstack_mychast ) ;
  return res ; 
}
  
bool  FractionIntegerP  ( FractionPub const * const xp  ) {
  return  ULongIs1  ( FractionCDenomp  ( xp  ) ) ;
}

bool  FractionEq  ( FractionPub const * const  mep ,
  FractionPub const * const xp  ) {
  if( mep ==  xp ) 
    return  true;
  if  ( not ULongisEqULong  ( FractionCDenomp  ( mep ) ,
      FractionCDenomp  ( xp ) ) )
    return  false;
  if  ( not ULongisEqULong  ( FractionCNump  ( mep ) ,
    FractionCNump  ( xp ) ) )
    return  false;
  // check for chast==0 before isNegative
  if(ULongIs0(FractionCNump(mep)))
    return  true;
  return  not ( bool_xor  ( FractionIsNegative ( mep ) ,
    FractionIsNegative ( xp ) ) ) ; 
}
  
unsigned  int FractionUIValue ( FractionPub  const * const x ) {
  if(ULongIs0(FractionCDenomp(x)))
    return  0;
  if(ULongIs0(FractionCNump(x)))
    return  0;
  if  ( ULongIs1  ( FractionCDenomp  ( x ) ) ) {
    unsigned  int result;
    if  ( ULongIsBADUIValue  ( FractionCNump  ( x ) , & result  ) )
      return  0 ;
    return  result;
  }
  unsigned  int result;  
  { ThreadStackPointerType const oldstack_ch = Lisp2DThreadSP ( ) ;
    ULongPub * const ch = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( ch , FractionCNump  ( x ) ) ;
    ULongDivConst ( ch , FractionCDenomp ( x ) ) ;
    if  ( ULongIsBADUIValue  ( ch , & result  ) ) {
      Lisp2D_StackFree  ( oldstack_ch ) ;
      return  0 ; 
    }
    Lisp2D_StackFree  ( oldstack_ch ) ;
  }
  return  result;
}
  
long  double  FractionLDValueConst ( FractionPub  const * const xp ) {
  ThreadStackPointerType const oldstack_xcopy = Lisp2DThreadSP ( ) ;
  FractionPub * const xcopy = NewStackFraction ( ) ;
  FractionConstrCopy ( xcopy , xp ) ;
  long  double const  res = FractionLDValueTemp  (xcopy ) ;
  Lisp2D_StackFree  ( oldstack_xcopy ) ;
  return res ;
}
  
bool  FractionMoreUInt ( FractionPub const * const mep , unsigned  int const ux ) {
  if  ( ULongIs0 ( FractionCDenomp(mep ) ) ){
    if  ( ULongIs0 ( FractionCNump(mep) ) )
      goto  False ;
    goto  Positive  ; 
  }
  // this is finite
  if  ( ULongIs0 ( FractionCNump(mep) ))
    goto  False ;
  // me is not 0
  if  ( ux  ==  0 ) 
    goto  Positive  ;
  if  ( not (  FractionIsNegative (mep) ) ) {
    ThreadStackPointerType const oldstack_d = Lisp2DThreadSP ( ) ;
    ULongPub * const d = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( d , FractionCDenomp  ( mep ) ) ;
    ULongMultiplyUInt ( d ,  ux  );
    bool const resb = ULongMore ( FractionCNump(mep) , d ) ;
    Lisp2D_StackFree  ( oldstack_d ) ;
    return resb ;
  }
False :    
  return  false ;
Positive  :
  return  not (  FractionIsNegative (mep) ) ; 
}

bool  FractionIsBADIntValue ( FractionPub const * const x ,int  * const resp){
  if(ULongIs0(FractionCDenomp(x)))
    return  true;
  if(ULongIs0(FractionCNump(x))){
    (*resp) = 0;
    return  false;
  }
  if  ( ULongIs1  ( FractionCDenomp  ( x ) ) ) {
    unsigned  int ures;
    if  ( ULongIsBADUIValue  ( FractionCNump  ( x ) , & ures  ) )
      return  true  ;
    int const tmp = uint_cast_int(ures);
    if(tmp  <=  0)    
      return  true;
    if(FractionIsNegative(x)) 
      (*resp) = (-  tmp);
    else
      (*resp) = tmp;
    return  false;
  }
  return  true;
}

long  int  FractionLValue ( FractionPub  const * const  x ) {
  if(ULongIs0(FractionCDenomp(x)))
    goto Zero ;
  if(ULongIs0(FractionCNump(x)) )
    goto Zero ;
  { ThreadStackPointerType const oldstack_c = Lisp2DThreadSP ( ) ;
    ULongPub * const c = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( c , FractionCNump  ( x ) ) ;
    ULongDivConst ( c , FractionCDenomp ( x ) ) ;
    unsigned  long  int ul;
    if  ( ULongisOKULValue ( c , & ul  ) ) {
      Lisp2D_StackFree  ( oldstack_c ) ;
      // if value size > 31 bits then BAD
      if(ul >> ((sizeof(unsigned long int)*8)-1))
        return 0 ;
      if(FractionIsNegative(x)) 
        return -  (ulint_cast_lint(ul)); 
      return ulint_cast_lint(ul); 
    }
    Lisp2D_StackFree  ( oldstack_c ) ;
  }
Zero  :
  return 0 ;
}

# define  FractionploConstPi  Lisp2D_FractionploConstPi
Fraction2 FractionploConstPi ;
FractionPub const * const FractionploConstPiPub =
  &  FractionploConstPi . pub ;
FractionPub * const FractionploPiPub =
  &  FractionploConstPi . pub ;
# define  FractionploConst2Pi Lisp2D_FractionploConst2Pi
Fraction2 FractionploConst2Pi ;
FractionPub * const Fractionplo2PiPub  = & FractionploConst2Pi . pub ;
FractionPub const * const FractionploConst2PiPub  = 
  & FractionploConst2Pi . pub ;
# define  FractionploConstHalfPi  Lisp2D_FractionploConstHalfPi
Fraction2 FractionploConstHalfPi  ;
FractionPub * const FractionploHalfPiPub  =
  & FractionploConstHalfPi . pub ;
FractionPub const * const FractionploConstHalfPiPub  =
  & FractionploConstHalfPi . pub ;
# define  FractionploConstE Lisp2D_FractionploConstE
Fraction2 FractionploConstE ;
FractionPub * const FractionploEPub  = & FractionploConstE . pub ;
FractionPub const * const FractionploConstEPub  = 
  & FractionploConstE . pub ;

// ! to speed up  
void  FractionAddUInt ( FractionPub * const me , unsigned  int const x ) {
  if ( x ) { 
    ThreadStackPointerType const oldstack_tmp = Lisp2DThreadSP ( ) ;
    FractionPub * const tmp = NewStackFraction  ( ) ;
    FractionConstrUInt  ( tmp ,x);
    FractionAddMoveFraction ( me , tmp ) ;
    Lisp2D_StackFree  ( oldstack_tmp ) ;
  }
}

// 0..255
unsigned  int FractionForColor  ( FractionPub const * const x ) {
  if  ( ULongIs0 ( FractionCNump(x) )  or FractionIsNegative ( x)  )
    goto  Zero  ;
  { enum { bytemax = 0xFFU } ;
    if  ( not ( ULongLess ( FractionCNump  ( x ) , FractionCDenomp  ( x ) ) ) )
      return  bytemax ;
    unsigned  int result  ;
    { ThreadStackPointerType const oldstack_cc = Lisp2DThreadSP ( ) ;
      ULongPub * const cc = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( cc , FractionCNump  ( x ) ) ;
      ULongMultiplyUInt ( cc ,  bytemax );
      { ThreadStackPointerType const oldstack_dd = Lisp2DThreadSP ( ) ;
        ULongPub * const dd = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( dd , FractionCDenomp  ( x ) ) ;
        ULongDiv2 ( dd ) ;
        ULongAddTemp  ( cc , dd ) ;
        Lisp2D_StackFree  ( oldstack_dd ) ;
      } // dd
      ULongDivConst ( cc , FractionCDenomp ( x ) ) ;
      if  ( ULongIsBADUIValue ( cc , & result  ) ) {
        Lisp2D_StackFree  ( oldstack_cc ) ;
        goto  Zero  ; 
      }
      Lisp2D_StackFree  ( oldstack_cc ) ;
    } // cc
    return  result  ; 
  } //  bytemax
Zero  :
  return  0 ; 
}

bool  FractionIsBADByteValue( FractionPub const * const me , uint8_t * const resp)  {
  if(ULongIs0(FractionCDenomp(me)) )
    return  true;
  if(ULongIs0(FractionCNump(me))){
    (*resp) = 0;
    return  false;
  }
  if(FractionIsNegative(me))    
    return  true;
  if  ( ULongIs1  ( FractionCDenomp  ( me  ) ) )
    return  ULongisBADByteValue ( FractionCNump  ( me  ) ,
      resp  ) ;
  return  true;
}
  
void  FractionAddMoveFraction  ( FractionPub * const me , FractionPub * const x ) {
  if  ( me ==  x ) {
    FractionMul2  (me);
    goto  Exit  ;
  }
  if  ( FractionIs0(x)  or FractionIsNaN  ( me) )
    goto  Exit  ;
  if  ( FractionIsNaN ( x) or  FractionIs0(me ) ) {
    FractionSwapFraction ( me , x ) ;
    goto  Exit ;
  }
  Fraction2 * const mep = Fraction_toPrivate(me);
  Fraction2 * const xp = Fraction_toPrivate(x);
  if  ( ULongIs0 ( ULongCToPub(&(mep-> denom )) ) ){
    if  ( ULongIs0 ( ULongCToPub(&(xp-> denom )) ) ){
      if  ( bool_xor  ( mep ->  flagNegative  , xp  ->  flagNegative  ) )
        ULongSet0  (ULongToPub(&mep-> numer ) ); 
    }
    goto  Exit  ;
  }
  if  ( ULongIs0 ( ULongCToPub(&(xp-> denom )) ) ){
    FractionSwapFraction ( me , x ) ;
    goto  Exit  ;
  }
  { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
    ULongPub * const gcd = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( gcd , ULongCToPub ( & mep -> denom  ) ) ;
    ULongGCD ( gcd , ULongCToPub ( & xp -> denom ) ) ;
    ULongDivConst ( ULongToPub ( & mep -> denom ) , gcd ) ;
    { ULongPub * const xdelit = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( xdelit , ULongCToPub ( & xp  -> denom ) ) ;
      ULongDivMove ( xdelit , gcd ) ;
      ULongMultiplyMove ( ULongToPub ( & mep -> numer ) , xdelit ) ;
    }
    Lisp2D_StackFree  ( oldstack_gcd ) ;
  } // gcd
  { ULongPub  * const c  = ULongToPub(&xp-> numer ) ;
    ULongMultiplyConst ( c , ULongCToPub ( & ( mep -> denom )  ) ) ;
    ULongMultiplyMove ( ULongToPub ( & mep -> denom ) , ULongToPub ( & xp -> denom ) ) ;
    // ** 
    if  ( not ( bool_xor  ( mep ->  flagNegative  , xp  ->  flagNegative  ) ) )
      ULongAddTemp  ( ULongToPub ( & mep -> numer ) , c ) ;
    else if  ( ULongLess ( ULongCToPub ( & mep -> numer  ) , c ) ) {
      ULongDifferenceEq ( c , ULongCToPub ( & mep -> numer ) ) ;
      ULongSwap  ( ULongToPub(&mep-> numer) ,c ) ;
      mep-> flagNegative  =  not  (mep-> flagNegative ) ;
    } else
      ULongDifferenceEq ( ULongToPub ( & mep -> numer ) , c ) ;
  } // ULong & c
  FractionSimplify  ( mep ) ;
Exit  :  ;
}

void  FractionSubtractMove  ( FractionPub  * const mep , FractionPub * const pxp  ) {
  Fraction2 * const me = Fraction_toPrivate(mep);
  if  ( mep ==  pxp  ) {
    ULongSet0  (ULongToPub( &me ->  numer ) );
    FractionSimplify  (  me  ) ;
    goto  Exit  ;
  }
  if  ( FractionIs0(pxp )  or  FractionIsNaN  ( mep ) )
    goto  Exit  ;
  if  ( FractionIsNaN ( pxp) ) {
    FractionSwapFraction ( mep , pxp ) ;
    goto  Exit  ;
  }
  if  ( FractionIs0(mep ) ) {
    FractionSwapFraction ( mep , pxp ) ;
    me ->  flagNegative  = ( not  (me ->  flagNegative )) ;
    goto  Exit  ;
  }
  Fraction2 * const px = Fraction_toPrivate(pxp);
  if  ( ULongIs0 ( ULongCToPub(& me ->  denom ) ) ){
    if  ( ULongIs0 ( ULongCToPub(&(px ->  denom )) ) ){
      if  ( not ( bool_xor  ( me  ->  flagNegative  , px  ->  flagNegative  ) 
        ) )
        ULongSet0  ( ULongToPub(&me ->  numer ) );
    }
    goto  Exit  ;
  }
  if  ( ULongIs0 ( ULongCToPub(&(px ->  denom )) )) {
    FractionSwapFraction ( mep , pxp ) ;
    me ->  flagNegative  = ( not  (me ->  flagNegative ) );
    goto  Exit  ;
  }
  { ThreadStackPointerType const oldstack_xdelit = Lisp2DThreadSP ( ) ;
    ULongPub * const xdelitplp = StackNew_ULong ( ) ;
    ULong_Init  ( xdelitplp ) ;
    { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
      ULongPub * const gcd = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( gcd , ULongCToPub ( & me ->  denom  ) ) ;
      ULongGCD ( gcd , ULongCToPub ( & px ->  denom ) ) ;
      ULongDivConst ( ULongToPub ( & me ->  denom ) , gcd ) ;
      ULongSet (xdelitplp ,ULongCToPub(&px ->  denom));
      ULongDivMove ( xdelitplp , gcd ) ;
      Lisp2D_StackFree  ( oldstack_gcd ) ;
    } // gcd
    ULongMultiplyMove ( ULongToPub ( & me ->  numer ) , xdelitplp ) ;
    Lisp2D_StackFree  ( oldstack_xdelit ) ;
  } // xdelit
  { ULongPub * const c = ULongToPub ( & px ->  numer ) ;
    ULongMultiplyConst ( c , ULongCToPub ( & me ->  denom )  ) ;
    ULongMultiplyMove ( ULongToPub ( & me ->  denom ) , ULongToPub ( & px ->  denom ) ) ;
    // ** 
    if  ( bool_xor  ( me  ->  flagNegative  , px  ->  flagNegative  ) )
      ULongAddTemp ( ULongToPub ( & me ->  numer ) , c ) ;
    else if  ( ULongLess ( ULongCToPub ( & me ->  numer  ) , c ) ) {
        ULongDifferenceEq ( c , ULongCToPub ( & me ->  numer ) ) ;
        ULongSwap  ( ULongToPub(&me ->  numer) , c ) ;
        me ->  flagNegative  = ( not  (me ->  flagNegative )) ; 
    } else
        ULongDifferenceEq ( ULongToPub ( & me ->  numer ) , c ) ;
  } // c 
  FractionSimplify  (  me  ) ;
Exit  : ;
}

void  FractionAddConstFraction  ( FractionPub * const mep , FractionPub const * const pxp  ) {
  if  ( mep == pxp ) {
    FractionMul2  (mep);
    goto  Exit  ;
  }
  if  ( FractionIs0(pxp)  or FractionIsNaN  ( mep ) )
    goto  Exit  ;
  if  ( FractionIsNaN ( pxp) or  FractionIs0(mep) ) {
    FractionSetFraction  ( mep , pxp ) ;
    goto  Exit ;
  }
  Fraction2 * const me = Fraction_toPrivate(mep);
  Fraction2 const * const px = Fraction_toCPrivate(pxp);
  if  ( ULongIs0 ( ULongCToPub(& me-> denom ) ) ) {
    if  ( ULongIs0 ( ULongCToPub ( &(px ->  denom)) ) ) {
      if  ( bool_xor  ( me  ->  flagNegative  , px  ->  flagNegative  ) )
        ULongSet0  ( ULongToPub(&me-> numer) ); 
    }
    goto  Exit  ; 
  }
  if  ( ULongIs0 ( ULongCToPub ( &(px -> denom)) ) ) {
    FractionSetFraction  ( mep , pxp ) ;
    goto  Exit  ; 
  }
  { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
    ULongPub * const gcd  = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( gcd , ULongCToPub ( & me  -> denom  ) ) ;
    ULongGCD ( gcd , ULongCToPub ( & px ->  denom ) ) ;
    ULongDivConst ( ULongToPub ( & me -> denom ) , gcd ) ;
    { ULongPub * const xdelit = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( xdelit , ULongCToPub ( & px ->  denom  ) ) ;
      ULongDivMove ( xdelit , gcd ) ;
      ULongClear(gcd); // or destruct ?
      ULongMultiplyMove ( ULongToPub ( & me -> numer ) ,  xdelit ) ;
    } // xdelit
    Lisp2D_StackFree  ( oldstack_gcd ) ;
  } // gcd
  { ThreadStackPointerType const oldstack_c = Lisp2DThreadSP ( ) ;
    ULongPub * const c = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( c , ULongCToPub ( & px ->  numer  ) ) ;
    ULongMultiplyConst ( c , ULongCToPub ( & me  -> denom )  ) ;
    ULongMultiplyConst (  ULongToPub  ( & me  -> denom  ) , ULongCToPub ( & ( px ->  denom )  ) ) ;
    // ** 
    if  ( not ( bool_xor  ( me  ->  flagNegative  , px  ->  flagNegative  ) ) )
      ULongAddTemp  ( ULongToPub ( & me -> numer ) , c ) ;    
    else
      if  ( ULongLess ( ULongCToPub ( & me  -> numer  ) , c ) ) {
        ULongDifferenceEq ( c , ULongCToPub ( & me  -> numer ) ) ;
        ULongMove ( ULongToPub(&me-> numer) ,c ) ;
        me-> flagNegative  = ( not  me-> flagNegative ) ;
      } else
        ULongDifferenceEq ( ULongToPub ( & me -> numer ) ,  c ) ;
    Lisp2D_StackFree  ( oldstack_c ) ;
  } // ULong c
  FractionSimplify  ( me ) ;
Exit  :  ;
}

void  FractionSubtractConst  ( FractionPub * const  mep , FractionPub const * const pxp  ) {
  Fraction2 * const me = Fraction_toPrivate ( mep ) ;
  if  ( mep ==  pxp  ) {
    ULongSet0  ( ULongToPub(&  me ->  numer )) ;
    FractionSimplify (  me  ) ;
    goto  Exit  ; 
  }
  if  ( FractionIs0 ( pxp )  or FractionIsNaN  ( mep ) )
    goto  Exit  ;
  if  ( FractionIsNaN ( pxp ) ) {
    FractionSetFraction ( mep , pxp ) ;
    goto  Exit  ;
  }
  if  ( FractionIs0 ( mep ) ) {
    FractionSetFraction ( mep , pxp ) ;
    me ->  flagNegative  = not ( me ->  flagNegative ) ;
    goto  Exit  ;
  }
  Fraction2 const * const px = Fraction_toCPrivate  ( pxp ) ;
  if  ( ULongIs0 ( ULongCToPub ( & me ->  denom ) ) ) {
    if  ( ULongIs0 ( ULongCToPub ( &  px ->  denom ) ) ) {
      if  ( not ( bool_xor  ( me  ->  flagNegative  , px  ->  flagNegative  ) ) )
        ULongSet0  (ULongToPub( &  me ->  numer ) ); 
    }
    goto  Exit  ;
  }
  if  ( ULongIs0 ( ULongCToPub ( &  px ->  denom ) ) ) {
    FractionSetFraction ( mep , pxp ) ;
    me ->  flagNegative  = not  ( me ->  flagNegative ) ;
    goto  Exit  ;
  }
  { ThreadStackPointerType const oldstack_xdelit = Lisp2DThreadSP ( ) ;
    ULongPub * const xdelitplp = StackNew_ULong ( ) ;
    ULong_Init  ( xdelitplp ) ;
    { ThreadStackPointerType const oldstack_gcd = Lisp2DThreadSP ( ) ;
      ULongPub * const gcd = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( gcd , ULongCToPub ( & me ->  denom  ) ) ;
      ULongGCD ( gcd , ULongCToPub ( & px ->  denom ) ) ;
      ULongDivConst ( ULongToPub ( & me ->  denom ) , gcd ) ;
      ULongSet ( xdelitplp , ULongCToPub( & px ->  denom  ) );
      ULongDivMove ( xdelitplp , gcd ) ;
      Lisp2D_StackFree  ( oldstack_gcd ) ;
    } // gcd
    ULongMultiplyMove ( ULongToPub ( & me ->  numer ) , xdelitplp ) ;
    Lisp2D_StackFree  ( oldstack_xdelit ) ;
  } // xdelit
  { ThreadStackPointerType const oldstack_c = Lisp2DThreadSP ( ) ;
    ULongPub * const c = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( c , ULongCToPub ( & px ->  numer ) ) ;
    ULongMultiplyConst ( c , ULongCToPub ( & me ->  denom ) ) ;
    ULongMultiplyConst ( ULongToPub ( & me ->  denom ) , ULongCToPub  ( & px ->  denom ) ) ;
    // ** 
    if  ( bool_xor  ( me  ->  flagNegative  , px  ->  flagNegative  ) )
      ULongAddTemp  ( ULongToPub ( & me ->  numer ) , c ) ;    
    else
      if  ( ULongLess ( ULongCToPub ( & me ->  numer ) ,  c ) ) {
        ULongDifferenceEq ( c , ULongCToPub ( & me ->  numer ) ) ;
        ULongMove ( ULongToPub(&  me ->  numer) , c ) ;
        me ->  flagNegative  = not  ( me ->  flagNegative ) ; 
      } else
        ULongDifferenceEq ( ULongToPub ( & me ->  numer ) , c ) ; 
    Lisp2D_StackFree  ( oldstack_c ) ;
  } // c 
  FractionSimplify  (  me  ) ;
Exit  : ;
}

bool  FractionIsInf ( FractionPub  const * const x ) {
  return  (ULongIs0(FractionCDenomp(x)) ) ?
    (not (ULongIs0(FractionCNump(x))) ):
    false;
}
  
void  FractionSetULInt  ( FractionPub * const mep ,
  unsigned long  int const  x  ) {
  Fraction2 * const me = Fraction_toPrivate(mep);
  ULongSet1 ( ULongToPub  ( & me  -> denom  ) ) ;  
  me-> flagNegative  = false ;
  ULongSetULInt  ( ULongToPub ( & me -> numer ) , x ) ; 
}

void  FractionSetSSize ( FractionPub * const mep , ssize_t const ss ) {
  Fraction2 * const me = Fraction_toPrivate(mep);
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
  if  (   ss < 0 ) {
    ULongSetUSize  ( ULongToPub ( & me -> numer ) ,ssize_cast_size(- ss)  );    
    me-> flagNegative  = true;
    return  ; 
  }
  me-> flagNegative  = false ;  
  if  (   ss  ==  0 ) {
    ULongSet0  (ULongToPub( &me-> numer ) );
    return  ; 
  }
  ULongSetUSize ( ULongToPub (&me-> numer) ,ssize_cast_size(ss) ) ; 
}

void  FractionSetExactLDouble ( FractionPub * const mep ,
  long double  const x ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  if  ( FractionIsSimpleSetExactPrivate  ( me  , x ) )
    return ;
  FractionSetExactLDoublePrivate ( me , x ) ;
}

bool  FractionIsBADUCharValue( FractionPub const * const me ,
  unsigned  char * const resp) {
  if(ULongIs0(FractionCDenomp(me)))
    return  true;
  if(ULongIs0(FractionCNump(me))){
    (*resp) = 0 ;
    return  false;
  }
  if  ( ULongIs1  ( FractionCDenomp  ( me  ) ) ) {
    if  ( FractionIsNegative (me) ) 
      return true;
    return  ULongisBADUCharValue  ( FractionCNump  (
      me ) , resp  )  ;
  }
  return true; 
}
  
void  FractionLn ( FractionPub  const * const x ,
  FractionPub * const  result ) {
  if  ( x ==  result ) {
    ThreadStackPointerType const oldstack_x2p = Lisp2DThreadSP ( ) ;
    FractionPub * const x2 = NewStackFraction ( ) ;
    FractionConstrCopy ( x2 , x ) ;
    FractionLn ( x2 , result ) ;
    Lisp2D_StackFree  ( oldstack_x2p ) ;
    return  ;
  }
  if  ( ULongIs0  ( FractionCDenomp ( x ) ) ) {
    if  ( not  (  ULongIs0  ( FractionCNump ( x ) ) ) ) {
      if  ( FractionIsNegative ( x ) ) 
        // -inf -> 0/0
        FractionSetNaN ( result) ;
      else
        FractionSetFraction ( result , x ) ;
    }  // inf -> inf
    return  ;
  }
  if  ( ULongIs0  ( FractionCNump ( x ) ) ) {
    // 0 -> -inf
    FractionSetSignUIntUInt ( result , true , 1U  , 0U  ) ;
    return  ; 
  }
  if  ( FractionIsNegative ( x ) ) {
    // -1 -> 0/0
    FractionSetNaN ( result) ;
    return  ; 
  }
  if  ( FractionEqUInt ( x , 1U  ) ) {
    // 1 -> 0
    FractionSetUInt ( result , 0 ) ;
    return  ; 
  }
  // 0 < x < 1 , 1 < x < inf
  // ln(x)=x-1 Error=((x-1)^2)/2
  /*Lisp2DFraction2SetFraction ( resultp , xp ) ;
  if  ( Lisp2DFraction2LessUInt ( xp  , 1U  ) )
    Lisp2DFraction2OverTurn  ( resultp ) ;
  // x < 1 -> 1/x > 1
  Lisp2DFraction2MinusUInt(resultp,1U);*/
  // x-1>0 or (1/x)-1>0
// ! Only while need quality EPS
  FractionSetExactLDouble ( result , logl (
    FractionLDValueConst  ( x ) ) ) ; 
}

void  FractionFactorial ( FractionPub * const mep ) {
  Fraction2 * const me  = Fraction_toPrivate(mep);
  // 0!=1 1!=1
  if  ( FractionIntegerP ( mep) ) {
    ULongFactorial ( ULongToPub ( & me ->  numer ) ) ;
    if  ( ULongIs0 ( ULongCToPub ( & me ->  numer ) ) )
      // infinity
      FractionSetUIntUInt ( mep , 1U , 0U ) ;
  } else  {
    // x!=sqrt(2 * Pi * x) * ((x / E)^x)
    ThreadStackPointerType const oldstack_np = Lisp2DThreadSP ( ) ;
    FractionPub * const n = NewStackFraction  ( ) ;
    FractionConstrCopy ( n , mep ) ;
    // n*=2
    FractionMul2  (n);
    FractionMultConstFraction ( n , FractionploConstPiPub ) ;
    FractionSqrt  ( n ) ;
    { ThreadStackPointerType const oldstack_fpowp = Lisp2DThreadSP ( ) ;
      FractionPub * const fpow = NewStackFraction ( ) ;
      FractionConstrCopy ( fpow , mep ) ;
      FractionDivFractionConst  ( mep , FractionploConstEPub ) ;
      FractionPower ( FractionToPub(me) , fpow ) ; 
      Lisp2D_StackFree  ( oldstack_fpowp ) ;
    }
    FractionMultMoveFraction  ( mep , n ) ;
    Lisp2D_StackFree  ( oldstack_np ) ;
  }
}
    
bool  FractionEqUInt  ( FractionPub const * const mep ,
  unsigned int const x ) {
  if  ( ULongIs0 ( FractionCDenomp(mep) ) )
    goto False  ;
  if  ( ULongIs0 ( FractionCNump ( mep ) ) )
    return  x  ==  0  ;
  if  ( x ==  0 )
    goto False  ;
  if  ( ULongIs1 ( FractionCDenomp ( mep ) ) )
    if  ( not ( FractionIsNegative ( mep ) ) )
      return  Lisp2DULongisEqUInt  ( FractionCNump ( mep ) , x ) ;
False :    
  return  false ; 
}

bool  FractionIsBADUI16Value  ( FractionPub const * const x ,
  uint16_t  * const resp  ) {
  if(ULongIs0(FractionCDenomp(x)))   
    return  true;
  if(ULongIs0(FractionCNump(x))){
    (*resp) = 0;
    return  false;
  }
  if  ( ULongIs1  ( FractionCDenomp  ( x ) ) )
    return ULongIsBADUI16Value  ( FractionCNump  ( x ) , resp ) ;
  return  true;
}
  
void  FractionSetUSize ( FractionPub * const mep , 
  size_t  const  s  )  {
  Fraction2 * const me  = Fraction_toPrivate  ( mep ) ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
  me -> flagNegative  = false ;
  ULongSetUSize ( ULongToPub ( &me-> numer) ,s ) ; 
}
  
void  FractionSetWChar  ( FractionPub * const mep ,
  wchar_t const s  ) {
  Fraction2 * const me  = Fraction_toPrivate  ( mep ) ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
  me -> flagNegative  = false ;
  ULongSetWChar  ( ULongToPub ( & me  -> numer ) ,  s ) ; 
}
  
BoolType  FractionMoreEq  ( FractionPub const * const xp  ,
  FractionPub  const * const yp ) {
  return  Lisp2DBoolNot(FractionLess(xp,yp));
}
  
// ! to speed up    
void FractionMinusUInt  ( FractionPub * const mep , unsigned  int const x ) {
  if  ( x ) {
    ThreadStackPointerType const oldstack_tmpp = Lisp2DThreadSP ( ) ;
    FractionPub * const tmp = NewStackFraction ( ) ;
    FractionConstrUInt  ( tmp ,x);
    FractionSubtractMove  ( mep ,  tmp ) ; 
    Lisp2D_StackFree  ( oldstack_tmpp ) ;
  }
}
  
void  FractionSqrt  ( FractionPub * const x ) {
  if  ( ULongIs0  ( FractionCDenomp  ( x ) ) ) {
    if  ( not  ULongIs0 ( FractionCNump  ( x ) ) )
      if  ( FractionIsNegative  ( x ) )
        // -1/0 -> 0/0
        FractionSetNaN  ( x ) ;  
        // +1/0 -> +1/0
    // 0/0 -> 0/0
  } else 
    if  ( not  ULongIs0 ( FractionCNump  ( x ) ) ) {
      if  ( FractionIsNegative  ( x ) )
        // -x/y -> 0/0
        FractionSetNaN  ( x ) ;
      else {
        ThreadStackPointerType const oldstack_tmpp = Lisp2DThreadSP ( ) ;
        // ; (2 ^ -64) = 5.421e-20
        FractionPub * const f64p = NewStackFraction ( ) ;
        FractionPub * const d64p = NewStackFraction ( ) ;
        FractionConstrUInt ( f64p , 2U )  ;  
        FractionConstrUInt ( d64p , 64U )  ;
        FractionPower ( f64p , d64p ) ;
        FractionOverTurn  ( f64p) ;
        Fraction_Sqrt_Rel ( x , f64p ) ;
        Lisp2D_StackFree  ( oldstack_tmpp ) ;
      }
    }
  // 0/1 -> 0/1
}
    
void  FractionSetFromBits ( FractionPub * const  mep ,
  void  const * const x  , size_t const  bits ) {
# ifdef  LISP2D_DEBUG
  if  ( bits ==  0  ) {
    fputws(L"Fraction::SetFromBits: bits=0\n",stderr);
    return  ; 
  }
# endif
  Fraction2 * const me  = Fraction_toPrivate  ( mep ) ;
  me  -> flagNegative  = false  ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
  ULongSetFromBits ( ULongToPub ( & me -> numer ) , x , bits  )  ; 
}
  
// [this+0.5]
void FractionRound ( FractionPub  * const mep  ) {
  { ThreadStackPointerType const oldstack_onehalfp = Lisp2DThreadSP ( ) ;
    FractionPub * const onehalf = NewStackFraction ( ) ;
    FractionInit ( onehalf ) ;
    FractionSetUIntUInt ( onehalf , 1U , 2U ) ;
    FractionAddMoveFraction(mep,onehalf); 
    Lisp2D_StackFree  ( oldstack_onehalfp ) ;
  }
  FractionFloor (mep);
}

/*

; f(x) = f(x0) + f'(x0)*(x-x0) + f''(x0)*((x-x0)^2)/2
; f=x^2 f'=2*x f''=2
; x^2 == A ? x
; x0^2 - A == dB
; x1 = x0 - dB / f'(x0) = x0 - (x0^2 - A)/(2*x0) = 
;  = x0-x0/2+A/(2*x0)  = (x0+A/x0) / 2
; x1r = round ( x1 , x1 * eps / 2 )
; dx = x1-x0
; R = f''(x0)*(dx^2)/2/f'(x0) = 2*(dx^2)/2/(2*x0) =
;   = (dx^2)/(2*x0)
; пока R * 2 > eps * x1r
; половина ошибки из за параболы, половина из за округления

('number defmethod square-rootrel ( startpoint eps )
  (nil  let ((a this) dx (x0 startpoint) x02
    r real realdiff x1r x1 realdiff2 realdiff3 
    realdiff4)
    (cout write "a = " a " , ")
    (cout print a)
    ('real set (a sqrt))
    (cout write "real = " real " , ")
    (cout print real)
    ('realdiff3 set (a - (real ^ 2)))
    (cout write "a-sqrt(a)^2 = " realdiff3 " , ")
    (cout print realdiff3)
    
    (nil  do-while
    
    (cout write "x0 = " x0 " , ")
    (cout print x0)
    ('x1 set (((a / x0) + x0) / 2))
    (cout write "x1 = " x1 " , ")
    (cout print x1)
    ('x1r set (x1 round (x1 * (eps / 2))))
    (cout write "x1r = " x1r " , ")
    (cout print x1r)
    ('dx set (x1 - x0))
    (cout write "dx = " dx " , ")
    (cout print dx)
    ('r set ((dx * dx) / (2 * x0)))
    (cout write "r = " r " , ")
    (cout print r)
    ('realdiff set (real - x1))
    (cout write "realdiff = " realdiff " , ")
    (cout print realdiff)
    ('realdiff2 set (a - (x1 ^ 2)))
    (cout write "a-x1^2 = " realdiff2 " , ")
    (cout print realdiff2)
    ('realdiff4 set (a - (x1r ^ 2)))
    (cout write "a-x1r^2 = " realdiff4 " , ")
    (cout print realdiff4)
    
    (x0 swap x1r)
    ; ошибка r + округление r = 2*r
    ((r * 2) > (eps * x1)))
  x0))

*/

void  Fraction_Sqrt_Rel ( FractionPub * const mep ,
  FractionPub  const * const epsp ) {
  ThreadStackPointerType const oldstack_epsp = Lisp2DThreadSP ( ) ;
  FractionPub * const x0p = NewStackFraction ( ) ;
  FractionConstrCopy ( x0p , mep ) ;
  { Fraction2 * const x0  = Fraction_toPrivate  ( x0p ) ;
    ULongISqrt ( ULongToPub ( & x0  -> numer ) ) ;
    if  ( ULongIs0  (ULongCToPub( & x0 -> numer ) ) )
      ULongSetUInt  ( ULongToPub ( & x0 -> numer ) , 1U ) ;
    ULongISqrt ( ULongToPub ( & x0  -> denom ) ) ;
    if  ( ULongIs0  ( ULongCToPub(& x0 -> denom ) ) )
      ULongSetUInt  ( ULongToPub ( & x0 -> denom ) , 1U ) ;
    FractionSimplify ( x0 ) ;
  } // x0
  FractionPub * const dxp = NewStackFraction ( ) ;
  FractionInit ( dxp ) ;
  FractionPub * const r2p = NewStackFraction ( ) ;
  FractionInit ( r2p ) ;
  FractionPub * const x1p = NewStackFraction ( ) ;
  FractionInit ( x1p ) ;
  FractionPub * const x1rp = NewStackFraction ( ) ;
  FractionInit ( x1rp ) ;
  FractionPub * const x1rep = NewStackFraction ( ) ;
  FractionInit ( x1rep ) ;
  do {
    // ('x1 set (((a / x0) + x0) / 2))
    FractionSetFraction ( x1p , mep ) ;
    FractionDivFractionConst ( x1p , x0p ) ;
    FractionAddConstFraction ( x1p , x0p ) ;
    FractionDiv2 ( x1p ) ;
    // ('x1r set (x1 round (x1 * (eps / 2))))
    FractionSetFraction ( x1rp , x1p ) ;
    FractionSetFraction ( x1rep , x1p ) ;
    FractionMultConstFraction ( x1rep , epsp ) ;
    FractionDiv2 ( x1rep ) ;
    FractionRoundStaticMove ( x1rp , x1rep ) ;
    // ('dx set (x1 - x0))
    FractionSetFraction ( dxp , x1p ) ;
    FractionSubtractConst  ( dxp , x0p ) ;
    // ('r set ((dx * dx) / (2 * x0)))
    // ('r2 set ((dx * dx) /  x0))
    FractionSetFraction ( r2p , dxp ) ;
    FractionMultConstFraction ( r2p , dxp ) ;
    FractionDivFractionConst ( r2p , x0p ) ;
    // (x0 swap x1r)
    FractionSwapFraction ( x0p , x1rp ) ;
    // ((r * 2) > (eps * x1))
    // (r2 > (eps * x1))
    FractionMultConstFraction ( x1p , epsp ) ;
  } while ( FractionMore ( r2p , x1p ) == Lisp2DBoolTrue ) ;
  FractionSwapFraction ( mep , x0p ) ;
  Lisp2D_StackFree  ( oldstack_epsp ) ;
}

/*

; f(x) = f(x0) + f'(x0)*(x-x0) + f''(x0)*((x-x0)^2)/2
; f=x^3 f'=3(x^2) f''=6x
; x^3 == A ? x
; x0^3-A == dB
; x1 = x0 - dB / f'(x0) = x0 - (x0^3-A)/(3(x0^2)) = 
;  = x0-x0/3+A/(3(x0^2))  = (2*x0+A/(x0^2)) / 3
; x1r = round ( x1 , x1 * eps / 2 )
; dx = x1-x0 = (A/(x0^2)-x0) / 3
; R = f''(x0)*(dx^2)/2/f'(x0) = 6*x0*(dx^2)/2/(3*(x0^2)) =
;   = x0*(dx^2)/(x0^2) = (dx^2)/x0
; пока R * 2 > eps * x1
; половина ошибки из за параболы, половина из за округления

('number defmethod cube-root ( eps )
  (nil  let ((a this) dx (x0 1)
    x02 r real realdiff x1r x1 realdiff2 realdiff3 
    realdiff4)
    (cout write "a = " a " , ")
    (cout print a)
    ('real set (a cbrt))
    (cout write "real = " real " , ")
    (cout print real)
    ('realdiff3 set (a - (real ^ 3)))
    (cout write "a-cbrt(a)^3 = " realdiff3 " , ")
    (cout print realdiff3)
    
    (nil  do-while
    
    (cout write "x0 = " x0 " , ")
    (cout print x0)
    ('x02 set (x0 * x0))
    (cout write "x0^2 = " x02 " , ")
    (cout print x02)
    ('dx set (((a / x02) - x0) / 3))
    (cout write "dx = " dx " , ")
    (cout print dx)
    ('x1 set (x0 + dx))
    (cout write "x1 = " x1 " , ")
    (cout print x1)
    ('r set ((dx ^ 2) / x0))
    (cout write "r = " r " , ")
    (cout print r)
    ('realdiff set (real - x1))
    (cout write "realdiff = " realdiff " , ")
    (cout print realdiff)
    ('realdiff2 set (a - (x1 ^ 3)))
    (cout write "a-x1^3 = " realdiff2 " , ")
    (cout print realdiff2)
    ('x1r set (x1 round (x1 * (eps / 2))))
    (cout write "x1r = " x1r " , ")
    (cout print x1r)
    ('realdiff4 set (a - (x1r ^ 3)))
    (cout write "a-x1r^3 = " realdiff4 " , ")
    (cout print realdiff4)
    
    (x0 swap x1r)
    ; ошибка r + округление r = 2*r
    ((r * 2) >= (eps * x1)))
  x0))

*/

// eps > 0
void  Fraction_Cbrt_Rel ( FractionPub * const mep ,
  FractionPub  const * const epsp ) {
  Fraction2 * const me  = Fraction_toPrivate  ( mep ) ;
  bool  const mewasneg = me -> flagNegative ;
  me -> flagNegative = false ;
  ThreadStackPointerType const oldstack_epsp = Lisp2DThreadSP ( ) ;
  FractionPub * const x0p = NewStackFraction  ( ) ;
  FractionConstrUInt ( x0p , 1U ) ;
  FractionPub * const x02p = NewStackFraction  ( ) ;
  FractionInit ( x02p ) ;
  FractionPub * const dxp = NewStackFraction  ( ) ;
  FractionInit ( dxp ) ;
  FractionPub * const rp = NewStackFraction  ( ) ;
  FractionInit ( rp ) ;
  FractionPub * const x1p = NewStackFraction  ( ) ;
  FractionInit ( x1p ) ;
  FractionPub * const x1rp = NewStackFraction  ( ) ;
  FractionInit ( x1rp ) ;
  FractionPub * const x1rep = NewStackFraction  ( ) ;
  FractionInit ( x1rep ) ;
  do {
    // ('x02 set (x0 * x0))
    FractionSetFraction ( x02p , x0p ) ;
    FractionMultConstFraction ( x02p , x0p ) ;
    // ('dx set (((a / x02) - x0) / 3))
    FractionSetFraction ( dxp , mep ) ;
    FractionDivFractionConst ( dxp , x02p ) ;
    FractionSubtractConst ( dxp ,  x0p ) ;
    FractionDivUInt  ( dxp , 3U ) ;
    // ('x1 set (x0 + dx))
    FractionSetFraction ( x1p , x0p ) ;
    FractionAddConstFraction  ( x1p , dxp ) ;
    // ('r set ((dx ^ 2) / x0))
    FractionSetFraction ( rp , dxp ) ;
    FractionMultConstFraction ( rp , dxp ) ;
    FractionDivFractionConst ( rp , x0p ) ;
    // ('x1r set (x1 round (x1 * (eps / 2))))
    FractionSetFraction ( x1rp , x1p ) ;
    FractionSetFraction ( x1rep , x1p ) ;
    FractionMultConstFraction ( x1rep , epsp ) ;
    FractionDiv2  ( x1rep ) ;
    FractionRoundStaticMove ( x1rp , x1rep ) ;
    // (x0 swap x1r)
    FractionSwapFraction ( x0p , x1rp ) ;
    // ((r * 2) >= (eps * x1))
    FractionMultConstFraction ( x1p , epsp ) ;
    FractionMul2 ( rp ) ;
  } while ( FractionMoreEq ( rp , x1p ) == Lisp2DBoolTrue ) ;
  FractionSwapFraction ( mep , x0p ) ;
  Lisp2D_StackFree  ( oldstack_epsp ) ;
  me -> flagNegative = mewasneg ;
}

/*

; cos( x , e ) =
;  = round(1 - (x ^ 2) / 2,e-(x^4)/4!) + O ( ( x ^ 4 ) / 24 ) ,
;      e/2 >= ( x ^ 4 ) / 24
;  = round(2*cos( x / 2 , e / 5)^2-1,e/5) , e/2 < ( x ^ 4 ) / 24

('number defmethod coseps (eps)
  (nil if (eps < ((this ^ 4) / 12)) ; eps/2 < (x^4)/4!
    ((((((this / 2) coseps (eps / 5)) ^ 2) * 2) - 1) round (eps / 5))
    ((1 - ((this ^ 2) / 2)) round (eps - ((this ^ 4) / 24)))))

*/

// eps > 0
void  Fraction_Cos_Eps ( FractionPub * const mep ,
  FractionPub  const * const epsp ) {
  if  ( ULongIs0  ( FractionCDenomp ( mep ) ) ) {
    FractionSetNaN ( mep ) ;
    return  ;
  }
  if ( FractionIs0 ( mep) ) {
    FractionSet1 ( mep ) ;
    return ; 
  }
  ThreadStackPointerType const oldstack_epsp = Lisp2DThreadSP ( ) ;
  FractionPub * const eps2p = NewStackFraction ( ) ;
  FractionConstrCopy ( eps2p , epsp ) ;
  // abs ( eps2 )
  Fraction2 * const eps2 = Fraction_toPrivate ( eps2p ) ;
  eps2 -> flagNegative = false ;
  FractionPub * const me4p = NewStackFraction ( ) ;
  { ThreadStackPointerType const oldstack_me2 = Lisp2DThreadSP ( ) ;
    FractionPub * const me2p = NewStackFraction ( ) ;
    FractionConstrCopy ( me2p , mep ) ;
    // (this ^ 2)
    FractionMultConstFraction ( me2p , mep ) ;
    FractionConstrCopy ( me4p , me2p ) ;
    // (this ^ 4)
    FractionMultMoveFraction ( me4p , me2p ) ;
    Lisp2D_StackFree  ( oldstack_me2 ) ;
  }
  // ((this ^ 4) / 12)
  FractionDivUInt ( me4p , 12U ) ;
  unsigned  int depth = 0 ;
  // (eps < ((this ^ 4) / 12))
  while ( FractionLess ( eps2p, me4p )  == Lisp2DBoolTrue ) {
    ++  depth ;
    // (this / 2)
    FractionDiv2 ( mep ) ;
    // (eps / 5)
    FractionDivUInt ( eps2p , 5U ) ;
    // (this / 2) ((this ^ 4) / 12)
    FractionDivUInt ( me4p , 16U ) ;
  }
  // (this ^ 2)
  FractionSquare  ( mep ) ;
  // ((this ^ 2) / 2)
  FractionDiv2 ( mep ) ;
  // (1 - ((this ^ 2) / 2))
  FractionSub1 ( mep ) ;
  Fraction2 * const me = Fraction_toPrivate ( mep ) ;
  me -> flagNegative = not (  me -> flagNegative  ) ;
  // ((1 - ((this ^ 2) / 2)) round (eps - ((this ^ 4) / 24)))
  { ThreadStackPointerType const oldstack_eps3 = Lisp2DThreadSP ( ) ;
    FractionPub * const eps3p = NewStackFraction ( ) ;
    // (eps - ((this ^ 4) / 24))
    FractionConstrCopy ( eps3p , eps2p ) ;
    FractionDiv2 ( me4p ) ;
    FractionSubtractMove ( eps3p , me4p ) ;
    FractionRoundStaticMove ( mep , eps3p ) ;
    Lisp2D_StackFree  ( oldstack_eps3 ) ;
  }
  while ( depth ) {
    --  depth ;
    // cos( x / 2 , e / 5)^2
    FractionSquare  ( mep ) ;
    // 2*cos( x / 2 , e / 5)^2
    FractionMul2 ( mep ) ;
    // 2*cos( x / 2 , e / 5)^2-1
    FractionSub1 ( mep ) ;
    // ((((((this / 2) coseps (eps / 5)) ^ 2) * 2) - 1) round (eps / 5))
    FractionRoundStaticConst ( mep , eps2p ) ;
    // from , e / 5)
    FractionMulUInt  ( eps2p , 5U  ) ; 
  }
  Lisp2D_StackFree  ( oldstack_epsp ) ;
}

/*

; ln(a,e) = round(2 * ln(sqrt(a,e/5),e/5),e/5)
; ln(1+x,e) = x - x^2/2 + (x^3)/3 - O((x^4)/4)
;           = round ( ( ( x / 3 - 1 / 2 ) * x + 1 ) * x , e / 2 ) 
; e/2 >= (x^4)/4

('number  defmethod lne (e)
  (nil  let ((a (this - 1)))
    (nil  if  ((e abs)  >=  ((a ^ 4) / 2))
      ((((((a / 3) - 1/2) * a) + 1) * a)  round (e / 2))
      (nil  let ((s (this sqrt  (e / 5))))
        (((s  lne (e / 5)) * 2) round (e / 5))))))

if x < 0 
  ln ( - x ) := ln ( x ) + ⅈ * ( ℼ ± 2ℼ )
        
*/

void  Fraction_Ln_Eps ( FractionPub * const mep ,
  FractionPub  const * const epsp ) {
  Fraction2 * const me = Fraction_toPrivate ( mep ) ;
  if  ( ULongIs0  ( FractionCDenomp ( mep ) ) ) {
    if ( ULongIs0  ( FractionCNump ( mep ) ) )
      // 0/0 -> 0/0
      return  ;
    if ( me -> flagNegative )
      // -1/0 -> + ∞ + ⅈ * ℼ
      me -> flagNegative  = false ;
    //  1/0 -> 1/0
    return  ;
  }
  if ( ULongIs0  ( FractionCNump ( mep ) ) ) {
    // +-0/1 -> -1/0
    ULongSet1 ( ULongToPub  ( & me -> numer ) ) ;
    ULongSet0 (ULongToPub( & me -> denom ) );
    me -> flagNegative  = true  ;
    return ; 
  }
  if (  me -> flagNegative )  {
    // ln(-x) => ln(+x)
    me -> flagNegative  = false ;
  }
  ThreadStackPointerType const oldstack_epsp = Lisp2DThreadSP ( ) ;
  FractionPub * const ap = NewStackFraction  ( ) ;
  FractionInit ( ap ) ;
  FractionPub * const a4p = NewStackFraction  ( ) ;
  FractionInit ( a4p ) ;
  FractionPub * const eps2p = NewStackFraction  ( ) ;
  Fraction2 * const eps2  = Fraction_toPrivate  ( eps2p ) ;
  if ( ULongIs0  ( FractionCNump ( epsp) ) ) {
    // if eps == 0 => eps := 2 ^ -64
    // ; (2 ^ -64) = 5.421e-20
    FractionPub * const d64p = NewStackFraction ( ) ;
    FractionConstrUInt ( eps2p , 2U )  ;  
    FractionConstrUInt ( d64p , 64U )  ;
    FractionPower ( eps2p , d64p ) ;
    FractionOverTurn  ( eps2p) ;
  } else  {
    FractionConstrCopy ( eps2p , epsp ) ;
    // abs ( eps2 )
    eps2 -> flagNegative = false ;
  }
  unsigned  int depth = 0 ;
  do {
    // (nil  let ((a (this - 1)))
    FractionSetFraction ( ap , mep ) ;
    FractionSub1 ( ap ) ;
    // (nil  if  ((e abs)  >=  ((a ^ 4) / 2))
    FractionSetFraction ( a4p , ap ) ;
    FractionMultConstFraction ( a4p , ap ) ;
    FractionSquare ( a4p ) ;
    FractionDiv2 ( a4p ) ;
    if  ( FractionMoreEq ( eps2p  , a4p ) ==  Lisp2DBoolTrue  )
      break ;
    ++  depth ;
    // (nil  let ((s (this sqrt  (e / 5))))
    //    (((s  lne (e / 5)) * 2) round (e / 5)))
    FractionDivUInt ( eps2p , 5U ) ;
    Fraction_Sqrt_Rel ( mep  , eps2p  ) ; 
  } while ( true ) ;
  // ((((((a / 3) - 1/2) * a) + 1) * a)  round (e / 2))
  FractionDiv2 ( eps2p ) ;
  FractionSetFraction ( mep , ap ) ;
  FractionDivUInt ( mep , 3U ) ;
  FractionPub * const onehalfp = NewStackFraction ( ) ;
  FractionConstrFNUIntUInt ( onehalfp , false , 1U  , 2U  ) ;
  FractionSubtractMove ( mep  , onehalfp ) ;
  FractionMultConstFraction  ( mep , ap ) ;
  FractionAdd1 ( mep ) ;
  FractionMultConstFraction  ( mep , ap ) ;
  FractionRoundStaticConst ( mep , eps2p ) ;
  if  ( depth ) {
    FractionMul2 ( eps2p ) ;
    do {
      // (((s  lne (e / 5)) * 2) round (e / 5))
      // * 2 round (e/5) , e *= 5 ^ depth
      FractionMul2 ( mep ) ;
      FractionRoundStaticConst ( mep , eps2p ) ;
      --  depth ;
      if ( depth == 0 )
        break ;
      FractionMulUInt  ( eps2p , 5U  ) ;
    } while ( true ) ; 
  }
  Lisp2D_StackFree  ( oldstack_epsp ) ;
}

// !! exp(1e5) -> inf REWRITE !!
void  FractionExp  ( FractionPub  const * const  x ,
  FractionPub  * const  result ) {
  if( x  ==    result ){
    ThreadStackPointerType const oldstack_x2p = Lisp2DThreadSP ( ) ;
    FractionPub * const x2 = NewStackFraction ( ) ;
    FractionConstrCopy (x2,x);
    FractionExp(x2,result);
    Lisp2D_StackFree  ( oldstack_x2p ) ;
    return;
  }
  if(ULongIs0(FractionCDenomp(x))){
    if(ULongIs0(FractionCNump(x)))
      // 0/0->0/0
      FractionSetNaN  (result);
    else if(FractionIsNegative(x))       
      // -inf -> 0
      FractionSet0(result);
    else     
      // +inf -> +inf
      FractionSetInf  (result);
    return;
  }
  if(ULongIs0(FractionCNump(x))){
    // 0 -> 1
    FractionSet1  ( result ) ;
    return;
  }
  if  ( ULongIs1  ( FractionCNump  ( x ) ) and
    ULongIs1  ( FractionCDenomp  ( x ) ) ) {
    // E^1=E E^(-1)=1/E
    FractionSetFraction ( result , FractionploConstEPub ) ;
    if(FractionIsNegative(x))
      FractionOverTurn  ( result) ;
    return;
  }
  // exp(x)=1+x+(x^2)/(2!)+...+(x^k)/(k!)+Error((x^(k+1))/((k+1)!)) , x<0
  // x<0
  // exp(x)=1+x Error=(x^2)/2
  // x>0
  // exp(x)=1/exp(-x)
// ! Only while need quality EPS  
  FractionSetExactLDouble ( result , expl ( FractionLDValueConst(x) ) ) ;
}
    
bool  FractionIs1 ( FractionPub  const * const x  ) {
  return  (not (FractionIsNegative(x))) and
    ULongIs1  ( FractionCDenomp  ( x ) ) and
    ULongIs1  ( FractionCNump  ( x ) ) ;
}

void  FractionRoundStaticMethod ( FractionPub * const  mep ,
  FractionPub  const * const  q )  {
  Fraction2 * const me  = Fraction_toPrivate (  mep ) ;
  Fraction2 const * const qp  = Fraction_toCPrivate (  q ) ;
  if(ULongIs0(ULongCToPub(& me-> denom ))){
    if(not  (ULongIs0(ULongCToPub(& me-> numer )))){
      // (+-inf)+-(inf|0/0) -> 0/0
      if(ULongIs0(ULongCToPub(&(qp-> denom ))))
        ULongSet0(ULongToPub( &me-> numer ));
    }
    return;
  }
  if(ULongIs0(ULongCToPub(& me-> numer ))){
    // 0+-0/0 -> 0/0
    if(FractionIsNaN(q))    
      ULongSet0(ULongToPub( &me-> denom ));
    return;
  }
  if(ULongIs0(ULongCToPub(&(qp-> denom )))){
    if(ULongIs0(ULongCToPub(&(qp-> numer )))){
       // x+-0/0 -> 0/0
      ULongSet0(ULongToPub( &me-> numer ));
      ULongSet0(ULongToPub( &me-> denom ));
      return;
    }
    // x+-inf -> 0
    ULongSet0(ULongToPub( &me-> numer ));
    ULongSet1 ( ULongToPub  ( & me  -> denom  ) ) ;
    return;    
  }
  // x+-0 -> x    
  if(ULongIs0(ULongCToPub(&(qp-> numer ))))  
    return;
  // 0<abs(q)<0.5
  // test for [this-q]<>[this+q]
  FractionRoundStaticConst(mep,q);
}

bool  FractionIsBADUShortValue  ( FractionPub const * const x ,
  unsigned short int  * const resp ) {
  if(ULongIs0(FractionCDenomp(x)))  
    return  true;
  if(ULongIs0(FractionCNump(x))){
    (*resp) = 0;
    return  false;
  }
  if  ( ULongIs1  ( FractionCDenomp  ( x ) ) ) 
    return ULongisBADUShortValue  ( FractionCNump  ( 
      x ) ,  resp ) ;
  return  true;
}

// this -> [this-q..this+q]
void  FractionRoundStaticMove ( FractionPub * const mep ,
  FractionPub * const qp ) {
  if(ULongIs0(FractionCNump(qp)))  
    return;
  // this is integer
  if  ( ULongIs1  ( ULongCToPub ( & Fraction_toPrivate  ( mep ) -> denom )  ) )
    return;
  // q<>0
  Fraction_toPrivate  ( qp ) ->  flagNegative  = false ;
  FractionRoundStaticMovePriv ( mep , qp  ) ;
}
  
// this -> [this-q..this+q]
void  FractionRoundStaticConst  ( FractionPub * const mep , FractionPub  const * const pqinp ) {
  if  ( ULongIs0  ( FractionCNump ( pqinp ) ) )
    return  ;
  // this is integer
  if  ( ULongIs1  ( ULongCToPub ( & Fraction_toPrivate  ( mep ) -> denom ) ) )
    return ;
  // q<>0
  ThreadStackPointerType const oldstack_q = Lisp2DThreadSP ( ) ;
  FractionPub * const q = NewStackFraction ( ) ;
  FractionConstrFNULongULong ( q , false , FractionCNump ( pqinp ) , FractionCDenomp ( pqinp ) ) ;
  FractionRoundStaticMovePriv ( mep , q ) ;
  Lisp2D_StackFree  ( oldstack_q ) ;
}

void  FractionSetRoundEps ( FractionPub * const mep ,
  long  double x2 , FractionPub const  * const epsp  ) {
  Fraction2 * const me  = Fraction_toPrivate  ( mep ) ;
  if(isnan(x2)){
    ULongSet0(ULongToPub( &me-> numer ));
    ULongSet0(ULongToPub( &me-> denom ));
    return;
  }
  if(isinf(x2)){
    me-> flagNegative  = (x2 < 0);
    ULongSet1 ( ULongToPub  ( & me  -> numer  ) ) ;
    ULongSet0(ULongToPub( &me-> denom ));
    return;
  }
  // ! is unsafe
  //if(x2==0)
  if(fabsl(x2) <=  LDBL_MIN ){
    FractionSet0(mep);
    return;
  }
  if(x2  < 0.0L) {
    x2 = (-  x2);
    me->flagNegative  = true;  
  }
  else 
    me-> flagNegative  = false;
  // x>0
  int pow;
  x2 = frexpl(x2,&  pow);
  // x=[1/2 .. 1)
  x2 = ldexpl(x2,1);
  --  pow;
  // number of bits
  // ld = [ 1.0 ... 2.0 ) , number = ld * 2^pow
  unsigned  int floorx;
  { long  double  ldfloorx;
    modfl(x2,& ldfloorx);
    floorx  = ldouble_cast_uint(floorl(ldfloorx)); 
  }
  ULongSetUInt ( ULongToPub ( & me  -> numer  ) , floorx ) ;
  ULongSet1 ( ULongToPub  ( & me  -> denom  ) ) ;
// ? to write more faster: by ints ?
  int poweps;
  { // by bits
    long  double  ldeps = FractionLDValueConst(epsp);
    frexpl(ldeps,&  poweps);  // ldeps=[0.5..1)*2^poweps
    // half eps for sqrt & half eps for simplify
    --  poweps;
    // eps=0.001 poweps = -8
    // pow was step up -=1
    // +1 step - practically needed
    int const   steps = (((pow + 1) - poweps)  + 1);  // by bits
    // unsafe
    for(unsigned  int   j = 0 ;(((j < LDBL_MANT_DIG) and (x2 > 0))  and
        (uint_cast_int(j) < steps));++  j){
      x2 -=  floorx;
      // by bits
      x2 = ldexpl(x2,1);
      ULongMul2Power ( ULongToPub ( & me  -> numer ) , 1U  ) ;
      ULongMul2Power ( ULongToPub ( & me  -> denom ) , 1U  ) ;      
      { long  double  ldfloorx;
        modfl(x2,& ldfloorx);
        floorx  = ldouble_cast_uint(ldfloorx);
      }
      ULongAddUInt ( ULongToPub( & me -> numer ) , floorx ) ;          
    }
  }
  if(pow<0)
    ULongMul2Power (  ULongToPub  ( & me  -> denom  ) ,
      int_cast_size ( - pow ) ) ;  
  else
    ULongMul2Power (  ULongToPub  ( & me  -> numer  ) ,
      int_cast_size ( pow ) ) ;  
  { int const  pow3  = ( ( Lisp2DMinInt  ( 
      LDBL_MANT_DIG ,  -  poweps ) + 1 )  - pow  )  +  2 ;
    if  ( pow3  ) {
      ThreadStackPointerType const oldstack_limit = Lisp2DThreadSP ( ) ;
      ULongPub * const limit = StackNew_ULong ( ) ;
      ULongConstrUInt ( limit , 1U  ) ;
      ULongPub * const longone = StackNew_ULong ( ) ;
      ULongConstrUInt ( longone , 1U  ) ;
      FractionPub * const tmp = NewStackFraction  ( ) ;
      if  ( pow3 < 0  ) {
        ULongMul2Power ( limit , int_cast_size ( - pow3  ) ) ;
        // (2*(2^(64-pow+1)))
        FractionConstrMoveFNULongULong  ( tmp , false , limit , longone ) ;
      } else  {
        ULongMul2Power ( limit , int_cast_size ( pow3  ) ) ;
        // 1/(2*(2^(64-pow+1)))
        FractionConstrMoveFNULongULong  ( tmp , false , longone , limit ) ;
      }
      FractionRoundStaticMove ( mep ,  tmp);      
      Lisp2D_StackFree  ( oldstack_limit ) ;
    } else  {
      // pow3 =0
      // 2 ^ 0 = 1
      ThreadStackPointerType const oldstack_tmpp = Lisp2DThreadSP ( ) ;
      FractionPub * const tmp = NewStackFraction  ( ) ;
      FractionConstrUInt  ( tmp ,1U);
      FractionRoundStaticMove(mep,tmp);
      Lisp2D_StackFree  ( oldstack_tmpp ) ;
    }
  }
}
    
# define  FractionToStringWhereMeChastIs0ForUniversal Lisp2DFractionToStringWhereMeChastIs0ForUniversal2674
static  void  FractionToStringWhereMeChastIs0ForUniversal ( FractionPub const * const pointerToMe ,
  bool  pointertoflagshowsign , String(WideChar) * const stringtarget ,
  wchar_t const * const nans , wchar_t const * const infs ) {
  if  ( ULongIs0 ( FractionCNump ( pointerToMe ) ) ) {
    if ( pointertoflagshowsign  ) 
      String_AddChar ( WideChar ) ( stringtarget  , L'+' ) ;
    StringAddCharArray( WideChar )(stringtarget,(ULongIs0 ( 
      FractionCDenomp ( pointerToMe ) ) ? 
      nans :
      L"0") ); 
  } else  {
    if  ( FractionIsNegative ( pointerToMe ) )
      String_AddChar ( WideChar ) (stringtarget,  L'-' );
    else {
      if  ( pointertoflagshowsign  ) 
        String_AddChar ( WideChar ) (stringtarget,  L'+' ) ;
    }
    StringAddCharArray( WideChar )(stringtarget,infs); 
  } 
}
    
void  FractionToStringWhereMeChastIs0ForHuman ( FractionPub const * const pointerToMe  ,
  bool  pointertoflagshowsign , String(WideChar) * const stringtarget ) {
  FractionToStringWhereMeChastIs0ForUniversal ( pointerToMe  ,
   pointertoflagshowsign ,  stringtarget , L"NaN" , L"Inf" ) ; 
}
   
void  FractionToStringWhereMeChastIs0ForInterpret ( 
  FractionPub const * const pointerToMe  ,
  bool  pointertoflagshowsign , String(WideChar) * const stringtarget ) {
  FractionToStringWhereMeChastIs0ForUniversal ( pointerToMe  ,
   pointertoflagshowsign ,  stringtarget , L"0/0" , L"1/0" ) ; 
}
  
void  FractionToString ( FractionPubAddrToStringWhereMeChastIs0ForInterpretHuman f  ,
  FractionPub const * const me , String(WideChar) * const sc  , unsigned int digc0  , bool flagshowsign ) {
  if  ( ULongIs0 ( FractionCNump(me)) ) {
    f  ( me  , flagshowsign  , sc  ) ;
    return ; 
  }
  if  ( ULongIs0 ( FractionCDenomp(me)) ) {
    f  ( me , flagshowsign  , sc ) ;
    return  ;
  }
  if  ( digc0 )
    FractionToStringWithDigitsCountLocal ( ( t_FractionToStringWithDigitsCountLocal ) { . pointerToMe2 = Fraction_toCPrivate(me)  ,
      . stringtarget = sc , . digc = digc0 , . toflagshowsign = flagshowsign } ) ;
  else
    FractionToStringForInterpretLocal ( Fraction_toCPrivate(me) , flagshowsign , sc ) ;
}

void  FractionToStringForInterpret  ( FractionPub const * const me ,
  String  ( WideChar  ) * const sc  , unsigned int  const  digc0  , bool flagshowsign ) {
  FractionToString ( FractionToStringWhereMeChastIs0ForInterpret , me , sc  , digc0 , flagshowsign  ) ;
}
    
// digc=0 -> full exact
// digc=12 -> simplified, not by fraction
  
void  FractionToStringForHuman  ( FractionPub const * const me ,
  String(WideChar) * const sc  ,
  unsigned int  digc0  , bool flagshowsign ) {
  FractionToString ( FractionToStringWhereMeChastIs0ForHuman ,
    me , sc  , digc0 , flagshowsign  ) ; 
}  

void  FractionSetExactLDoublePrivate ( Fraction2 * const me , 
  long double x ) {
  me -> flagNegative  = false ;
  if(x  < 0.0L) {
    x = -  x ;
    me -> flagNegative  = true; 
  }
  // x>0
  int pow ;
  x = frexpl(x,&  pow);
  // x=[1/2 .. 1)
  x = ldexpl(x,1);
  --  pow;
  // number of bits
  // ld = [ 1.0 ... 2.0 ) , number = ld * 2^pow
  unsigned  int floorx;
  { long  double  ldfloorx;
    modfl(x,& ldfloorx);
// ? floorx ==is always== 1        
    floorx  = ldouble_cast_uint(ldfloorx); 
  }
  ULongSetUInt (  ULongToPub  ( & me -> numer ) , floorx ) ;
  ULongSet1 ( ULongToPub  ( & me -> denom ) ) ;
  x -=  floorx ;
  { enum { digmaxn = (LDBL_MANT_DIG >>  3) } ;
    for(unsigned  int  j = 0;(j < digmaxn)  and (x  > 0);++ j){
      { enum { eight  = 8 } ;
        x = ldexpl(x,eight);
        ULongMul2Power (  ULongToPub  ( & me -> numer ) , eight ) ;
        pow -=    eight  ; 
      } // eight
      { long  double  ldfloorx;
        modfl(x,& ldfloorx);
        floorx  = ldouble_cast_uint ( ldfloorx  ) ; 
      }
      ULongAddUInt ( ULongToPub ( & me -> numer ) , floorx ) ;
      x -=  floorx ;
    } // for j
  } // digmaxn 
  if(pow  < 0){
    size_t   pow2 = int_cast_size ( - pow ) ;
    size_t dpow  = 0 ;
    while ( ( floorx  bitand  1U  )  ==  0 ) {
      floorx  >>= 1U  ;
      ++  dpow  ; 
    }
    if  ( dpow < pow2 ) {
      ULongDiv2Power  ( ULongToPub  ( & me -> numer ) , dpow  ) ;
      pow2 -= dpow ;        
      ULongMul2Power (  ULongToPub  ( & me -> denom ) , pow2 )  ;
    } else 
      ULongDiv2Power  ( ULongToPub  ( & me -> numer ) , pow2  ) ;    
  } else 
    ULongMul2Power (  ULongToPub  ( & me -> numer ) ,
      int_cast_size ( pow ) ) ;
}

bool  FractionIsSimpleSetExactPrivate  ( Fraction2  * const me ,
  long  double const x2 )  {
  if(isnan(x2)){
    FractionSetUIntUInt ( FractionToPub(  me ) , 0U  , 0U  ) ;
    return  true;
  }
  if(isinf(x2)){
    FractionSetSignUIntUInt ( FractionToPub( me ) , x2 < 0 , 1U  , 0U  ) ;
    return  true;
  }
  // ! (x2==0) is unsafe  
  if(fabsl(x2) <=  LDBL_MIN ){  
    FractionSet0(FractionToPub(me));
    return  true;
  }
  return  false;
}

# define  vectorboolsettrueat Lisp2Dvectorboolsettrueat8320
static void vectorboolsettrueat(uint8_t * const arr , size_t const ind){
  uint8_t * const place = &(arr[ind >> 3] );
  (*place) = uint_cast_uint8((*place) bitor (1U << (ind bitand 7))); 
}

# define  vectorboolsetfalseat  Lisp2Dvectorboolsetfalseat2625
void vectorboolsetfalseat ( uint8_t * arr , size_t ind ) ;

# define  vectorboolat  Lisp2Dvectorboolat3557
bool vectorboolat ( uint8_t const * arr , size_t ind);
  
long  double  FractionLDValueTemp  ( FractionPub * const mep ) {
  ULongPub * mydelit  ;
  ULongPub * mychast  ;
  { Fraction2 * const me  = Fraction_toPrivate (  mep ) ;
    mydelit = ULongToPub ( & me -> denom ) ;
    mychast = ULongToPub ( & me -> numer ) ;
  }
  bool const fn = FractionIsNegative ( mep) ;
  if(ULongIs0( mydelit )) {
    if(ULongIs0( mychast ))
      return (0.0L / 0.0L);
    if(fn)
      return ((- 1.0L) / 0.0L);
    return (1.0L / 0.0L); 
  }
  if(ULongIs0(mychast ) )
    return  0 ;
  int   pow = 0 ;
  if( ULongLess ( mychast , mydelit ) ) {
    do {
      ULongMul2 ( mychast ) ;
      --  pow;
    } while ( ULongLess ( mychast , mydelit ) ) ;
  } else {
    do {
      ULongMul2 ( mydelit ) ;
      ++  pow;
    }while(ULongMoreOrEq( mychast , mydelit ));
    ULongDiv2 ( mydelit ) ;
    --  pow; 
  }
  // ch >= de x=(ch/de)*(2^pow)
  enum  { arrresN = ( ( LDBL_MANT_DIG ) >>  3 ) + 1 } ;
  uint8_t arrres  [ arrresN ] ;
  memset  ( arrres  , 0 , arrresN ) ;
  size_t  iterarrres = 0 ;
  for(unsigned  int   step = LDBL_MANT_DIG ;
      ( not ULongIs0 ( mychast ) ) and (step  > 0);
        ++  iterarrres ) {
    --  step;
    if( ULongMoreOrEq( mychast , mydelit )) {
      vectorboolsettrueat(arrres , iterarrres) ;
      ULongDifferenceEq ( mychast , mydelit ) ;
    } else
      vectorboolsetfalseat(arrres , iterarrres) ;
    ULongMul2 ( mychast ) ;
  } // for step
  if( ULongMoreOrEq( mychast , mydelit ))
    // += 0.00...001b
    do{
      --  iterarrres  ;
      if(vectorboolat(arrres ,iterarrres)) 
        vectorboolsetfalseat(arrres , iterarrres) ;
      else {
        vectorboolsettrueat(arrres , iterarrres) ;
        break; 
      }
      if(iterarrres  ==  0){
        // 1.11...111b -> 10b -> 1*(2^1)
        vectorboolsettrueat(arrres , iterarrres) ;
        ++  pow;
        break; 
      }
    }while(true);
  long  double  result  = 0  ;
  iterarrres = LDBL_MANT_DIG;
  do{
    --  iterarrres  ;
    if(vectorboolat(arrres ,iterarrres))
      result  +=  1.0L;
    result  = ldexpl(result,- 1);
  }while(iterarrres);
  result  = ldexpl(result,pow + 1);
  if(fn)
    result  = (- result);
  return  result  ; 
}

// returns dead fraction
FractionPub * NewStackFraction (  void  ) {
  Fraction2 * const frp = Lisp2D_StackVar ( Lisp2DStackTypeFraction ) ;
  return  & frp ->  pub ;
}

bool  FractionMoreEqUInt  ( FractionPub const * const xp , unsigned  int const y ) {
  return not FractionLessUInt  (xp,y);
}

bool  FractionNotEq ( FractionPub const * const xp , FractionPub  const * const yp ) {
  return not  FractionEq( xp, yp ) ;
}

bool  FractionLessEqUInt  ( FractionPub const * const xp  , unsigned  int const y  ) {
  return not  FractionMoreUInt  ( xp  , y  ) ;
}

void  FractionSetPointer  ( FractionPub * const mep , void * const  x ) {
  Fraction2 * const me = Fraction_toPrivate  ( mep ) ;
  ULongSetPointer ( ULongToPub ( & me  -> numer ) , x ) ;
  ULongSet1  ( ULongToPub ( & me  -> denom ) ) ;
  me-> flagNegative  = false ;
}

void  FractionDestr ( FractionPub * const m ) {
  FractionDestrPri ( Fraction_toPrivate  ( m ) ) ;
}

# include "castb.h"

void  FractionDestrPri  ( Fraction2 * const f ) {
  ULongDestr  ( & f ->  numer ) ;
  ULongDestr  ( & f ->  denom ) ;
}

void  vectorboolsetfalseat ( uint8_t * const arr , size_t const ind  ) {
  uint8_t * const place = &(arr[ind >> 3] );
  (*place) = uint_cast_uint8((*place) bitand (compl(1U << (ind bitand 7))));
}

bool  vectorboolat  ( uint8_t const * const arr , size_t const ind  ) {
  return ((arr[ind >> 3]) bitand (1U << (ind bitand 7)));
}
