// Lisp2D - Fraction number power
// ©2011-23 by Alex Glebe

# include "fraction/public.h"
# include <math.h> // powl

typedef struct  Lisp2D_s_FractionPower s_FractionPower  ;

# define  FractionPowerNotNegative  Lisp2D_FractionPowerNotNegative
static  void  FractionPowerNotNegative  ( s_FractionPower * ) ;

struct  Lisp2D_s_FractionPower {
  Fraction2 const * const np  ;
  Fraction2 * const me  ;
  FractionPub * const mep ;
  FractionPub  const * const n  ;
} ;

# include "stack/public.h"

void  FractionPower ( FractionPub * const mep ,  FractionPub  const * const n ) {
  if  ( mep ==  n ) {
    ThreadStackPointerType const oldstack_n2p = Lisp2DThreadSP ( ) ;
    FractionPub * const n2 = NewStackFraction  ( ) ;
    FractionConstrCopy ( n2 , mep ) ;
    FractionPower ( mep , n2 ) ;
    Lisp2D_StackFree  ( oldstack_n2p ) ;
    return ;
  }
  s_FractionPower st = {
    . np  = Fraction_toCPrivate ( n ) ,
    . me  = Fraction_toPrivate  ( mep ) ,
    . mep = mep ,
    . n = n ,
  } ;
// ! to use FractionCDenomp ( np )
  if  ( ULongIs0 (ULongCToPub( &( st . np -> denom )) ) ){
    if  ( ULongIs0 (ULongCToPub( &(st . np-> numer )) ) ){
      // 1^(0/0)=1
      if  ( FractionEqUInt  (mep,1U) )
        return  ;
      // this^(0/0)=(0/0)
      ULongSet0  (ULongToPub( &st . me ->  numer ) );
      ULongSet0  (ULongToPub( &st . me-> denom ) );
      return  ;
    }
    if  ( not ( st . np -> flagNegative  ) ) {
      // this^inf
      if  ( ULongIs0 (ULongCToPub( & st . me ->  denom ) ) ){
        // (0/0)^inf=(0/0)
        if  ( ULongIs0 ( ULongCToPub(& st . me ->  numer ) ) )
          return  ;
        // inf^inf=inf;
        if  ( not (st . me ->  flagNegative  ))
          return  ;
        // -inf^inf=(0/0)
        ULongSet0  (ULongToPub( &st . me ->  numer )) ;
        return  ;
      }
      // 0^inf=0
      if  ( ULongIs0 (ULongCToPub( & st . me-> numer ) ) )
        return  ;
      if  ( ULongLess ( ULongCToPub ( & st . me -> numer ) ,
        ULongCToPub ( & st . me -> denom ) ) ) {
        // +-0.5^inf=0
        ULongSet0  (ULongToPub( &st . me-> numer ) );
        ULongSet1 ( ULongToPub  ( & st . me  -> denom  ) ) ;
        return  ;
      }
      if  ( ULongisEqULong  ( ULongCToPub ( & st . me  -> numer ) , ULongCToPub ( & st . me  -> denom ) ) ) {
        // ±1^+inf=(0/0)
        ULongSet0  (ULongToPub( &st . me-> numer ) );
        ULongSet0  (ULongToPub( &st . me-> denom ) );
        return  ;
      }
      // +-2^inf
      // 2^inf=inf
      if  ( not (st . me-> flagNegative ) )
        ULongSet1 ( ULongToPub  ( & st . me  -> numer  ) ) ;
      // -2^inf=(0/0)
      else
        ULongSet0  (  ULongToPub  ( & st . me -> numer ) )  ;
      ULongSet0  (ULongToPub( &st . me-> denom ) )  ;
      return  ;
    }
    // this^-inf
    if  ( ULongIs0 (ULongCToPub( & st . me-> denom ) ) )  {
      // (0/0)^-inf=(0/0)
      if  ( ULongIs0 (ULongCToPub( & st . me-> numer ) ) )
        return  ;
      // +-inf^-inf=0
      ULongSwap  ( ULongToPub(&st . me-> numer) , ULongToPub(&st . me-> denom ) );
      return  ;
    }
    if  ( ULongIs0 (ULongCToPub( & st . me-> numer ) ) ){
      // 0^-inf=+inf
      ULongSwap  ( ULongToPub(&st . me-> numer) , ULongToPub(&st . me-> denom ) );
      st . me-> flagNegative  = false  ;
      return  ;
    }
    if  ( ULongMore(ULongCToPub(&st . me-> numer ) , ULongCToPub(&st . me-> denom) ) ) {
      // +-2^-inf=0
      FractionSet0(FractionToPub(st . me));
      return  ;
    }
    if  ( ULongisEqULong  ( ULongCToPub ( & st . me  -> numer ) ,
      ULongCToPub ( & st . me  -> denom ) ) ) {
      // ±1^-inf=(0/0)
      ULongSet0  (ULongToPub( &st . me-> numer ) );
      ULongSet0  (ULongToPub( &st . me-> denom ) );
      return  ;
    }
    // +-0.5^-inf
    // 0.5^-inf=inf
    if  ( not (st . me-> flagNegative  ))
      ULongSet1 ( ULongToPub  ( & st . me  -> numer  ) ) ;
    // -0.5^-inf=(0/0)
    else
      ULongSet0  (ULongToPub( &st . me-> numer ) );
    ULongSet0  (ULongToPub( &st . me-> denom ) );
    return  ;
  }
  // n=finite
  if  ( ULongIs0 (ULongCToPub( & ( st . np -> numer ) ) ) ){
    // (±∞)^0 -> 0/0 , (0/0)^0 -> 0/0
    if  ( ULongIs0 (ULongCToPub( & st . me-> denom ) ))
      ULongSet0  (ULongToPub( &st . me-> numer ) );
      // 0^0 -> 0/0
    else if  ( ULongIs0 (ULongCToPub( & st . me-> numer ) ))
      ULongSet0  (ULongToPub( &st . me-> denom ) );
      // this^0=1
    else
      FractionSet1  ( mep) ;
    return  ;
  }
  // n<>0

  if  ( not ( st . np -> flagNegative  ) ) {
    FractionPowerNotNegative  ( & st ) ;
    return  ;
  }
  // n<0

  if  ( ULongIs1 ( ULongCToPub (  & st . np  -> denom  ) ) ) {
    // n=-1,-2,-3,...
    if  ( ULongIs1 ( ULongCToPub  ( & st . np  -> numer  ) ) ) {
      // n=-1
      if  ( ULongIs0 (ULongCToPub( & st . me-> numer ) ) ){
        // 0/0^-1=0/0
        if  ( ULongIs0 (ULongCToPub( & st . me-> denom ) ))
          return  ;
        // (0/1)^-1=+(1/0)
        st . me-> flagNegative  = false ;
      }
      ULongSwap  ( ULongToPub(&st . me-> denom) , ULongToPub(&st . me-> numer ) );
      return  ;
    }
    // n=-2,-3,-4,...
    if  ( ULongIs0 (ULongCToPub( & st . me-> denom ) ) ){
      // 0/0^-2 -> 0/0 , +-1/0^-2 -> 0/1
      if  ( not ( ULongIs0 (ULongCToPub( & st . me-> numer ) ) ))
        ULongSwap  ( ULongToPub(&st . me-> numer) , ULongToPub(&st . me-> denom ) );
      return  ;
    }
    if  (  ULongIs0 (ULongCToPub( &  st . me-> numer ) ) ){
      // 0^-x/y=+inf
      ULongSwap  (ULongToPub( &st . me-> numer) ,ULongToPub( &st . me-> denom )) ;
      st . me-> flagNegative  = false ;
      return  ;
    }
    // n=-2,-4,-6,... -> this=abs(this)
    if  ( Lisp2DULongEvenP ( ULongCToPub  ( & st . np -> numer ) ) )
      st . me-> flagNegative  = false ;
    // -1^3=-1 , 1^3=1 , 1^2=1
// !? .Is1()
    if  ( ULongisEqULong ( ULongCToPub ( & st . me -> numer ) , ULongCToPub ( & ( st . me  -> denom  ) ) ) )
      return  ;
    if  ( ULongValueSize ( ULongCToPub ( & st . np -> numer) )  > 1  ) {
      // n=big
      if  ( ULongLess ( ULongCToPub ( & st . me  -> numer  ) , ULongCToPub ( & st . me  -> denom ) ) ) {
        // +-0.5^-333333=+-inf
        ULongSet1 ( ULongToPub  ( & st . me  -> numer  ) ) ;
        ULongSet0  (ULongToPub( & st . me-> denom ) );
      } else
        // +-2^-3333333=0
        FractionSet0(mep);
      return  ;
    }
    // n=small
    // (+-2/3)^-3=+-27/8
    unsigned  long  int pow ;
    if  ( ULongisOKULValue ( ULongCToPub ( & st . np  -> numer ) , & pow ) ) {
      FractionOverTurn  ( mep) ;
      ULongPowerSize  ( ULongToPub ( & st . me -> numer ) , pow ) ;
      ULongPowerSize  ( ULongToPub ( & st . me -> denom ) , pow ) ;
    }
    return  ;
  }
  //  n.delit>=2 , n<0
  if  ( ULongIs0 (ULongCToPub( & st . me-> denom ) ) ){
    // +-inf^-2/3=0
    if  ( not ( ULongIs0 (ULongCToPub( & st . me-> numer ) ) ))
      ULongSwap  ( ULongToPub(&st . me-> numer) ,ULongToPub( &st . me-> denom ) );
    // 0/0^-2/3=0/0
    return  ;
  }
  if  ( ULongIs0 (ULongCToPub( & st . me-> numer ) ) ){
    ULongSwap  ( ULongToPub(&st . me-> numer ), ULongToPub(&st . me-> denom) ) ;
    st . me-> flagNegative  = false ;
    return  ;
  }
  // (-x)^(3/2) = +-i
  if  ( ( ULongEvenP (  ULongCToPub ( & st . np  -> denom ) ) ) and ( st . me  -> flagNegative ) ) {
    ULongSet0  (ULongToPub( &st . me-> numer ) );
    ULongSet0  (ULongToPub( &st . me-> denom ) );
    return  ;
  }
  // n=2,4,6,... -> this=abs(this)
  if  ( ULongEvenP (  ULongCToPub ( & st . np  -> numer ) ) )
    st . me-> flagNegative  = false ;
  // +-1^2/3=+-1
// !? .Is1()
  if  ( ULongisEqULong  ( ULongCToPub ( & st . me  -> numer ) , ULongCToPub ( & ( st . me  -> denom  ) ) ) )
    return  ;
  bool  const me_was_negative = st . me -> flagNegative ;
  st . me -> flagNegative  = false ;
  FractionSetExactLDouble ( mep , powl ( FractionLDValueTemp  ( mep ) , FractionLDValueConst  ( n ) ) ) ;
  st . me ->  flagNegative  = me_was_negative  ;
}

static  void  FractionPowerNotNegative  ( s_FractionPower * const st ) {
  // n>0
  if  ( ULongIs1 ( ULongCToPub  ( & st -> np  -> denom  ) ) ) {
      // n=1,2,3,...
      // n=1
      // this^1=this
      if  ( ULongIs1 ( ULongCToPub  ( & st -> np  -> numer  ) ) )
        return  ;
      // n=2,3,4,...
      // n=2,4,6,... -> this=abs(this)
      if  ( ULongEvenP ( ULongCToPub  ( & st -> np -> numer ) ) )
        st -> me-> flagNegative  = false ;
      // 0/0^3=0/0
      // +-inf^3=+-inf
      if  ( ULongIs0 (ULongCToPub( & st -> me-> denom ) ) )
        return  ;
      // 0^3=0
      if  ( ULongIs0 (ULongCToPub( & st -> me-> numer ) ) )
        return  ;
      // +-1^3=+-1
      if  ( ULongisEqULong  ( ULongCToPub ( & st -> me  -> numer ) ,
        ULongCToPub ( & ( st -> me  -> denom  ) ) ) )
        return  ;
      if  ( ULongValueSize ( ULongCToPub ( & st -> np  -> numer ) )  > 1  ) {
        // n=big
        if  ( ULongLess ( ULongCToPub ( & st -> me  -> numer  ) , ULongCToPub ( & st -> me  -> denom ) ) ) {
          // +-0.5^333333=0
          FractionSet0(st -> mep);
          return  ;
        }
        // +-2^3333333=+-inf
        ULongSet1 ( ULongToPub  ( & st -> me  -> numer  ) ) ;
        ULongSet0  (ULongToPub( &st -> me-> denom ) );
        return  ;
      }
      // n=small
      // (+-2/3)^3=+-8/27
      unsigned  long  int pow ;
      if  ( ULongisOKULValue ( ULongCToPub ( & st -> np  -> numer ) , & pow ) ) {
        ULongPowerSize ( ULongToPub ( & st -> me -> numer ) , pow ) ;
        ULongPowerSize ( ULongToPub ( & st -> me -> denom ) , pow ) ;
      }
      return  ;
  }
  // 0/0^2/3=0/0 0^2/3=0
  if  ( ULongIs0 (ULongCToPub( & st -> me-> numer ) ) )
    return  ;
  //  n.delit>=2 , n>0
  if  ( ULongIs0 (ULongCToPub( & st -> me-> denom ) ) ) {
      // +inf^2/3=+inf
      if  ( not (st -> me-> flagNegative ) )
        return  ;

      if  ( not ULongOddP ( ULongCToPub( & st -> np -> denom ) ) ) {
        // -inf^3/4=±∞ⅈ
        ULongSet0  (ULongToPub( &st -> me-> numer ) );
        return  ;
      }
      // -inf^2/3=+inf
      if  ( ULongEvenP ( ULongCToPub  ( & st -> np -> numer ) ) )
        st -> me-> flagNegative  = false ;
      // -inf^1/3=-inf
      return  ;
  }
  // n=2,4,6,... -> this=abs(this)
  if  ( ULongEvenP ( ULongCToPub  ( & st -> np -> numer ) ) )
    st -> me-> flagNegative  = false ;
  // (-1)^(3/2)=±ⅈ
  if  ( ( Lisp2DULongEvenP ( ULongCToPub  ( & st -> np -> denom ) ) ) and ( st -> me  -> flagNegative ) ) {
      ULongSet0  (ULongToPub( &st -> me-> numer ) );
      ULongSet0  (ULongToPub( &st -> me-> denom ) );
      return  ;
  }
  // +-1^2/3=+-1
// !? to do : .Is1() and .Is1()
  if  ( ULongisEqULong  ( ULongCToPub ( & st -> me  -> numer ) , ULongCToPub ( & ( st -> me  -> denom  ) ) ) )
    return  ;
  bool  const me_was_negative = st -> me  -> flagNegative ;
  st -> me  -> flagNegative  = false ;
  FractionSetExactLDouble ( st -> mep , powl ( FractionLDValueTemp ( st -> mep ) , FractionLDValueConst  ( st -> n ) ) ) ;
  st -> me  -> flagNegative  = me_was_negative ;
}
