// Lisp2D - Fraction number
// ©2011-23 by Alex Glebe

# include "fraction/private.h"
# include "slong/public.h"
# include "slong/inline.h"
# include "fraction/public.h"

typedef struct s_FractionRoundMove t_FractionRoundMove ;

struct s_FractionRoundMove {
  Lisp2DSLongPub * const realc3pp ;
  ULongPub * d1 ;
  ULongPub * d2 ;
  Lisp2DSLongPub  * c1pp  ;
  FractionPub * const qp  ;
  Lisp2DSLongPub  * c2pp  ;
  ULongPub * const reald3plp  ;
  Fraction2 * const me  ;
  FractionPub * const mep ;
  bool  flagreturn  ;
} ;

static  void  statFractionRoundMove ( t_FractionRoundMove * )  ;

# include "stack/public.h"

// this -> [this-q..this+q]
void  FractionRoundStaticMovePriv ( FractionPub * const mep , FractionPub * const qp ) {
  // q>0
  ThreadStackPointerType const oldstack_real = Lisp2DThreadSP ( ) ;
  t_FractionRoundMove fr = {
    . realc3pp = StackNew_SLong ( ) ,
    . qp = qp ,
    . reald3plp = StackNew_ULong ( )  ,
    . me  = Fraction_toPrivate  ( mep ) ,
    . mep = mep ,
    . flagreturn = false ,
  } ;
  Lisp2DSLongPub * const realc1pp = StackNew_SLong ( ) ;
  Lisp2DSLongConstrUInt ( realc1pp , 1U ) ;
  ULongPub * const reald1plp = StackNew_ULong ( ) ;
  ULong_Init  ( reald1plp ) ; // 0
  Lisp2DSLongPub * const realc2pp = StackNew_SLong ( ) ;
  SLongConstrSignULong ( ( t_SLongConstrSignULong ) {
    . my = realc2pp , . xp = & fr . me -> numer , . fn = fr . me  -> flagNegative  } ) ;
  ULongPub * const reald2 = StackNew_ULong ( ) ;
  ULongConstrUInt ( reald2 , 1U ) ;
  Lisp2DSLongInit ( fr . realc3pp ) ;
  ULong_Init  ( fr . reald3plp ) ;
  // c1=1   d1=0
  // c2=[x] d2=1
  // x:= x-[x]
  // x2 := 1/x
  // c3=c2*[x2]+c1
  // d3=d2*[x2]+d1
  fr . c1pp = realc1pp ;
  fr . d1 = reald1plp  ;
  fr . c2pp =   realc2pp ;
  Lisp2DSLongDivULong  ( fr . c2pp , & fr . me-> denom ) ;
  // c2 is integer part
  fr . d2 = reald2  ;
  // x:= x-[x]
  { Lisp2DSLong * const intcp = Lisp2D_StackVar ( Lisp2DStackTypeSLong ) ;
    Lisp2DSLongConstrCopy ( SLongToPub  ( intcp ) , fr . c2pp ) ;
    Lisp2DSLongMulULong ( SLongToPub  (intcp),&fr . me-> denom);
    // double control (if was not simplifyed)
    if  ( ULongisEqULong  ( ULongCToPub ( Lisp2DSLongCabs ( SLongCToPub ( intcp ) ) ) , ULongCToPub ( & fr . me  -> numer ) ) ) {
      // was integer
      ULongSet (ULongToPub(&fr . me -> numer), ULongCToPub(Lisp2DSLongCabs( fr . c2pp )) );
      ULongSet1 ( ULongToPub  ( & fr . me  -> denom  ) ) ;
      Lisp2D_StackFree  ( oldstack_real ) ;
      return;
    }
    // this is not integer
    // 1.7 -> 1, 0.7
    ULongDifferenceEq ( ULongToPub ( &  fr . me  -> numer ) , ULongCToPub ( Lisp2DSLongCabs ( SLongCToPub ( intcp ) ) ) ) ;
  }
  if( fr . me-> flagNegative ){
    Lisp2DSLongDec1 ( fr . c2pp);
    // [-1.7] -> -1 -> -2
    // -1.7 -> -2, 0.3
    ThreadStackPointerType const oldstack_delit2 = Lisp2DThreadSP ( ) ;
    ULongPub * const delit2 = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( delit2 , ULongCToPub ( & fr . me  -> denom  ) ) ;
    ULongDifferenceEq ( delit2 , ULongCToPub ( & fr . me -> numer ) ) ;
    ULongMove ( ULongToPub(&fr . me-> numer) ,delit2);
    Lisp2D_StackFree  ( oldstack_delit2 ) ;
  }
  // x2 := 1/x
  // 3/10 -> 10/3
  FractionOverTurn  ( fr . mep ) ;
  // make [x2]
  { ThreadStackPointerType const oldstack_intc = Lisp2DThreadSP ( ) ;
    ULongPub * const intc = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( intc , ULongCToPub ( & fr . me  -> numer  ) ) ;
    ULongDivConst ( intc , ULongCToPub ( & fr . me -> denom ) ) ;
    Lisp2DSLongSetULong  ( fr . realc3pp , ULong_toPrivate(intc) );
    ULongSet (fr . reald3plp ,intc);
    ULongMultiplyConst ( intc , ULongCToPub ( & fr . me  -> denom )  ) ;
    ULongDifferenceEq ( ULongToPub ( & fr . me -> numer ) ,  intc ) ;
    Lisp2D_StackFree  ( oldstack_intc ) ;
  } // intc 10-=(3*3)
  statFractionRoundMove ( & fr ) ;
  if ( fr . flagreturn ) {
    Lisp2D_StackFree  ( oldstack_real ) ;
    return  ;
  }
  // take №2
  ULongSwap(ULongToPub( &fr . me-> numer) ,ULongToPub(Lisp2DSLongabs( fr . c2pp)));
  ULongSwap(ULongToPub( &fr . me-> denom) ,  fr . d2 );
  fr . me-> flagNegative  = Lisp2DSLongFlagNegative(fr . c2pp);
  Lisp2D_StackFree  ( oldstack_real ) ;
}

void  statFractionRoundMove ( t_FractionRoundMove * const fr ) {
  Lisp2DSLongPub  *  c3pp =   fr -> realc3pp ;
  Lisp2DSLongMulSLong ( c3pp,fr -> c2pp);
  Lisp2DSLongAddSLong ( c3pp,fr -> c1pp);

  // d2=1 d1=0 -> d3=[x2]
  ULongPub * d3 =  fr -> reald3plp ;
  // err=1/(d2*d3)
  ThreadStackPointerType const oldstack_errp = Lisp2DThreadSP ( ) ;
  FractionPub * const err = NewStackFraction ( ) ;
  FractionConstrCopyULong ( err , d3 ) ;

  // d2=1
  FractionOverTurn  ( err) ;
  while(FractionMore(err,fr -> qp)  ==  Lisp2DBoolTrue){
    if(ULongIs0(ULongCToPub(& fr -> me-> numer ))){
      Lisp2D_StackFree  ( oldstack_errp ) ;
      ULongSet (ULongToPub(&fr -> me-> numer), ULongCToPub(Lisp2DSLongCabs( c3pp )) ) ;
      ULongSet (ULongToPub(&fr -> me-> denom),d3);
      fr -> me-> flagNegative  = Lisp2DSLongFlagNegative( c3pp ) ;
      fr -> flagreturn = true ;
      return;
    }

// !! to Swaps

    // remove №1
    { Lisp2DSLongPub  * const tmpp = fr -> c1pp ;
      fr -> c1pp = fr -> c2pp ;
      fr -> c2pp = c3pp ;
      c3pp = tmpp ;
    }

    { ULongPub * const tmp = fr -> d1 ;
      fr -> d1 = fr -> d2 ;
      fr -> d2 = d3 ;
      d3 = tmp ;
    }
    // x := 1/x
    FractionOverTurn  ( fr -> mep ) ;
    // make [x2]
    { ThreadStackPointerType const oldstack_intc = Lisp2DThreadSP ( ) ;
      ULongPub * const intc = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( intc , ULongCToPub ( & fr -> me  -> numer  ) ) ;
      ULongDivConst ( intc , ULongCToPub ( & fr -> me -> denom ) ) ;
      Lisp2DSLongSetULong  ( c3pp , ULong_toPrivate (intc) ) ;
      ULongSet ( d3 , intc ) ;
      // x:= x-[x]
      ULongMultiplyConst ( intc , ULongCToPub ( & fr -> me  -> denom )  ) ;
      ULongDifferenceEq ( ULongToPub ( & fr -> me -> numer ) , intc ) ;
      Lisp2D_StackFree  ( oldstack_intc ) ;
    } // intc
    Lisp2DSLongMulSLong (c3pp,fr -> c2pp);
    Lisp2DSLongAddSLong (c3pp,fr -> c1pp);
    ULongMultiplyConst (  d3 , fr -> d2 ) ;
    ULongAddTemp ( d3 , fr -> d1 ) ;
    FractionSetULong  ( err , fr -> d2 ) ;
    FractionMultULongConst  ( err , d3 ) ;
    FractionOverTurn  ( err) ;
  }  // while
  Lisp2D_StackFree  ( oldstack_errp ) ;
}
