// Lisp2D - Fraction number private to string
// ©2011-23 by Alex Glebe

# include "fraction/private.h"
# include "cast.h"
# include "fraction/public.h"

enum { Lisp2D_fraction_tostring_bufs = 0x100 } ;
static  _Thread_local wchar_t Lisp2D_fraction_tostring_buf [ Lisp2D_fraction_tostring_bufs  ] ;

typedef struct  s_fract_string_digit t_fract_string_digit ;

struct  s_fract_string_digit {
  Vector  ( StackUnsInt ) const * const vectorDigits  ;
  String  ( WideChar  ) * const stringtarget  ;
  long  int const lpow2 ;
} ;

# define  fract_digc_eq_one Lisp2D_fract_digc_eq_one
static  void  fract_digc_eq_one ( t_fract_string_digit * )  ;

# include "stack/public.h"

void  FractionToStringWithDigitsCountLocal ( t_FractionToStringWithDigitsCountLocal arg ) {
  if  (   FractionIsNegative (FractionCToPub ( arg . pointerToMe2 ) ) )
    String_AddChar ( WideChar )(arg . stringtarget,  L'-' );
  else if ( arg . toflagshowsign )
    String_AddChar ( WideChar )(arg . stringtarget,  L'+' );
    // simplified, not by fraction
    // not 0,0/0,1/0,-1/0
// ! remake longint -> int OR ssize_t ?
  long  int lpow2 = 0 ;
  ThreadStackPointerType const oldstack_myp = Lisp2DThreadSP ( ) ;
  FractionPub * const my = NewStackFraction ( ) ;
  FractionInit ( my ) ;
  enum  { uiten = 10U } ;
  { ULongPub const * const originchast = FractionCNump( FractionCToPub(arg . pointerToMe2));
    ULongPub const * const origindelit = FractionCDenomp( FractionCToPub(arg . pointerToMe2));
    if  ( ULongLess ( originchast , origindelit ) ) {
      ThreadStackPointerType const oldstack_mychast = Lisp2DThreadSP ( ) ;
      ULongPub * const mychast = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( mychast , originchast ) ;
      do  {
        ULongMultiplyUInt ( mychast , uiten ) ;
        --  lpow2 ;
      } while ( ULongLess ( mychast , origindelit ) ) ;
      FractionMoveNumPositiveULongULong ( my , mychast ,  origindelit ) ;
      Lisp2D_StackFree  ( oldstack_mychast ) ;
    } else  {
      ThreadStackPointerType const oldstack_mydelitbackup = Lisp2DThreadSP ( ) ;
      ULongPub * const mydelitbackupplp = StackNew_ULong ( ) ;
      ULong_Init ( mydelitbackupplp ) ;
      { ThreadStackPointerType const oldstack_mydelit = Lisp2DThreadSP ( ) ;
        ULongPub * const mydelit = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( mydelit , origindelit ) ;
        // chast>delit
        do  {
          ULongSet (mydelitbackupplp , mydelit);
          ULongMultiplyUInt ( mydelit , uiten ) ;
          ++  lpow2 ;
        } while ( ULongLessOrEq(mydelit, originchast ) ) ;
        Lisp2D_StackFree  ( oldstack_mydelit ) ;
      } // mydelit
      --  lpow2 ;
      FractionMoveDenomPositiveULongULong  ( my , originchast , mydelitbackupplp ) ;
      Lisp2D_StackFree  ( oldstack_mydelitbackup ) ;
    }
  } // originchastp
  // chast>=delit
  // digc protection
  if  ( arg . digc < 2  )
    arg . digc  = 1 ;
  // digc>0
  // rounding (very slow)
  if  ( arg . digc ==  1  ) {
    ThreadStackPointerType const oldstack_fpowp = Lisp2DThreadSP ( ) ;
    FractionPub * const fpow = NewStackFraction ( ) ;
    FractionConstrFNUIntUInt ( fpow , false , 1U , 2U );
    FractionAddMoveFraction(my,fpow);
    Lisp2D_StackFree  ( oldstack_fpowp ) ;
  } else  {
    ThreadStackPointerType const oldstack_fdelit = Lisp2DThreadSP ( ) ;
    ULongPub * const fdelitplp = StackNew_ULong ( ) ;
    ULong_Init ( fdelitplp ) ;
    ULongSetUInt ( fdelitplp , uiten ) ;
    ULongPowerSize  ( fdelitplp , arg . digc  - 1 ) ;
    ULongMul2Power ( fdelitplp , 1U )  ;
    ULongPub * const oneplp = StackNew_ULong ( ) ;
    ULong_Init ( oneplp ) ;
    ULongSetUInt ( oneplp , 1U  ) ;
    FractionPub * const fpow = NewStackFraction ( ) ;
    FractionInit   ( fpow ) ;
    FractionMoveSignULongULong ( fpow , false , oneplp , fdelitplp )  ;
    FractionAddMoveFraction ( my  , fpow  ) ;
    Lisp2D_StackFree  ( oldstack_fdelit ) ;
  }
  { ThreadStackPointerType const oldstack_udelit = Lisp2DThreadSP ( ) ;
    ULongPub * const udelit = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( udelit , FractionCDenomp  ( my  ) ) ;
    ULongMultiplyUInt ( udelit , uiten ) ;
    if  ( ULongMoreOrEq(FractionCNump(my) , udelit) ) {
      // step back
      FractionDivUInt (my,uiten);
      ++  lpow2 ;
    }
    Lisp2D_StackFree  ( oldstack_udelit ) ;
  } // udelit
  // chast>=udelit
// !! to use DivWithOst ??
  // making digits
  Vector  ( StackUnsInt ) * const vectorDigits = Lisp2D_StackVar ( Lisp2DStackTypeStackUnsInt ) ;
  VectorInit  ( StackUnsInt ) ( vectorDigits  , false ) ;
  Lisp2DStackUnsIntResize_ExcMem(vectorDigits,arg . digc);
  for ( unsigned  int i = 0 ; i  < arg . digc ; ++ i  ) {
    { unsigned  int const uldig = FractionUIValue ( my ) ;
      ( * Lisp2DStackUnsIntAt ( vectorDigits  , i ) ) = uldig ;
      FractionMinusUInt ( my  , uldig ) ;
    }
    FractionMulUInt  ( my , uiten ) ;
  }
  // kill zeros
  { unsigned int const * j = Vector_CRBegin ( StackUnsInt ) ( vectorDigits  ) ;
    while ( ( * j ) ==  0 ) {
      --  arg . digc  ;
      --  j ;
    }
  }
  if  ( arg . digc ==  1  ) {
    t_fract_string_digit  b = {
      . vectorDigits = vectorDigits ,
      . stringtarget = arg . stringtarget ,
      . lpow2 = lpow2 ,
    } ;
    fract_digc_eq_one ( & b ) ;
    goto  Exit  ;
  }
  // digc>=2
  if  ( ( lpow2 >=  0 ) and ( ( longint_cast_size_t( lpow2 ) + 1UL )  < arg . digc ) ) {
    // 1.234 12.34 123.4
    size_t  li  ;
    unsigned int const * j = VectorCBegin(StackUnsInt)(vectorDigits);
    for ( li  = 0 ; li  <=  longint_cast_size_t( lpow2 ) ; ++ li , ++  j )
      String_AddChar ( WideChar )(arg . stringtarget, uint_cast_wchar(L'0' + ( * j )) );
    String_AddChar ( WideChar )(arg . stringtarget,  L'.' );
    for ( ; li < arg . digc ; ++ li , ++  j )
      String_AddChar ( WideChar )(arg . stringtarget, uint_cast_wchar(L'0' + ( * j ) ));
    goto Exit ;
  }
  // 0.0001234 0.1234 1234 1234000
  if  ( lpow2  > 0  )
    goto  LPowMore0 ;
  // 1.234e-4 0.001234 0.01234 0.1234
  if  ( ( - lpow2 ) < 4 ) {
    // 0.001234 0.01234 0.1234
    StringAddCharArray( WideChar )(arg . stringtarget, L"0." );
    for ( long  int li  = ( ( - lpow2 ) - 1 ) ; li > 0  ; --  li  )
      String_AddChar ( WideChar )(arg . stringtarget,  L'0' );
// unsinged int -> size_t
    unsigned int const * j = VectorCBegin(StackUnsInt)(vectorDigits) ;
    for ( unsigned  int i = 0 ; i  < arg . digc ; ++ i  , ++  j )
      String_AddChar ( WideChar )(arg . stringtarget, uint_cast_wchar ( L'0' + ( * j ) ) ) ;
    goto Exit ;
  }
  // 1.234e-5 1.234e-4
  String_AddChar ( WideChar )(arg . stringtarget, uint_cast_wchar ( L'0' + Lisp2DStackUnsIntAtCopy ( vectorDigits  , 0 ) ) ) ;
  String_AddChar ( WideChar )(arg . stringtarget,  L'.' );
// unsinged int -> size_t
  { unsigned int const * j = VectorCBegin ( StackUnsInt ) ( vectorDigits  ) + 1 ;
    for ( unsigned  int i = 1 ; i < arg . digc  ; ++  i , ++  j )
      String_AddChar ( WideChar )  ( arg . stringtarget, uint_cast_wchar ( L'0' + ( * j ) ) ) ;
  }
  StringAddCharArray( WideChar )(arg . stringtarget, L"e-" );
  if ( swprintf ( Lisp2D_fraction_tostring_buf , Lisp2D_fraction_tostring_bufs  , L"%zu"  , longint_cast_size_t ( - lpow2 ) ) > 0 )
    StringAddCharArray  ( WideChar )  ( arg . stringtarget  , Lisp2D_fraction_tostring_buf ) ;
  goto  Exit  ;
LPowMore0 :
  // 1234 12340 123400 1234000 1.234e7
    if  ( ( ( lpow2 + 1 ) - ( uint_cast_int ( arg . digc  ) ) ) < 4 ) {
      // 1234 12340 123400 1234000
// !? unsigned int -> size_t
      { unsigned int const * j = VectorCBegin ( StackUnsInt ) ( vectorDigits  ) ;
        for ( unsigned  int   i = 0 ; i < arg . digc  ; ++  i , ++  j )
          String_AddChar ( WideChar ) ( arg . stringtarget  , uint_cast_wchar ( L'0' + ( * j )  ) ) ;
      }
      for ( long  int li  = ( ( lpow2 + 1 ) - ( uint_cast_int ( arg . digc ) ) ) ; li  > 0 ; --  li  )
        String_AddChar ( WideChar ) ( arg . stringtarget  ,  L'0' ) ;
      goto Exit ;
    }
    // 1.234e7 1.234e8
    String_AddChar ( WideChar ) ( arg . stringtarget  , uint_cast_wchar ( L'0' + Lisp2DStackUnsIntAtCopy ( vectorDigits  , 0 ) ) ) ;
    String_AddChar ( WideChar ) ( arg . stringtarget  ,  L'.' ) ;
    { unsigned int const * j = VectorCBegin ( StackUnsInt ) ( vectorDigits  ) + 1 ;
      for ( unsigned  int i = 1 ; i < arg . digc  ; ++  i , ++  j )
        String_AddChar ( WideChar ) ( arg . stringtarget  , uint_cast_wchar ( L'0' + ( * j ) ) ) ;
    }
    String_AddChar ( WideChar ) ( arg . stringtarget  ,  L'e' ) ;
    if ( swprintf ( Lisp2D_fraction_tostring_buf , Lisp2D_fraction_tostring_bufs  , L"%zu"  , longint_cast_size_t ( lpow2 ) ) > 0 )
      StringAddCharArray  ( WideChar )  ( arg . stringtarget  , Lisp2D_fraction_tostring_buf )  ;
    goto Exit ;
Exit :
  Lisp2D_StackFree  ( oldstack_myp ) ;
}

static  void  fract_digc_eq_one ( t_fract_string_digit * const arg ) {
  wchar_t  const charnew = uint_cast_wchar ( L'0' +
    Lisp2DStackUnsIntAtCopy ( arg -> vectorDigits  , 0 ) ) ;
  // 2e-3 0.02 0.2 2 20 200 2e3
  switch  ( arg -> lpow2 ) {
  case  0 :
    String_AddChar ( WideChar )(arg -> stringtarget,  charnew );
    break ;
  case  1 :
    String_AddChar ( WideChar )(arg -> stringtarget,  charnew );
    String_AddChar ( WideChar )(arg -> stringtarget,  L'0' );
    break ;
  case  -1  :
    StringAddCharArray( WideChar )(arg -> stringtarget, L"0." );
    String_AddChar ( WideChar )(arg -> stringtarget,  charnew );
    break ;
  case  2 :
    String_AddChar ( WideChar )(arg -> stringtarget,  charnew );
    StringAddCharArray( WideChar )(arg -> stringtarget, L"00" );
    break ;
  case  -2  :
    StringAddCharArray( WideChar )(arg -> stringtarget, L"0.0" );
    String_AddChar ( WideChar )(arg -> stringtarget,  charnew );
    break ;
  default :
    String_AddChar ( WideChar )(arg -> stringtarget,  charnew );
    String_AddChar ( WideChar )(arg -> stringtarget,  L'e' );
    if  ( arg -> lpow2  > 0  ) {
      if ( swprintf(Lisp2D_fraction_tostring_buf,Lisp2D_fraction_tostring_bufs,L"%zu",
        longint_cast_size_t(arg -> lpow2)) > 0 )
        StringAddCharArray( WideChar )(arg -> stringtarget, Lisp2D_fraction_tostring_buf );
    } else  {
      String_AddChar ( WideChar )(arg -> stringtarget,  L'-' );
      if ( swprintf(Lisp2D_fraction_tostring_buf,Lisp2D_fraction_tostring_bufs,L"%zu",
        longint_cast_size_t(-(arg -> lpow2))) > 0 )
        StringAddCharArray( WideChar )(arg -> stringtarget, Lisp2D_fraction_tostring_buf );
    }
  }
}

typedef struct  s_fract_local t_fract_local ;

struct  s_fract_local {
  Fraction2 const * const pointerToMe2  ;
  ULongPub const * const mydelit  ;
  String ( WideChar  ) * const stringtarget ;
  bool  flagreturn  ;
} ;

# define  fract_local Lisp2D_fract_local
static  void  fract_local ( t_fract_local * ) ;

void  FractionToStringForInterpretLocal ( Fraction2 const * const pointerToMe2 ,
  bool  const flagshowsign , String ( WideChar  ) * const stringtarget ) {
  // full exact
  // simplify for delit 3/6 -> 1/2 -> 0.5 , for integer 21/7  -> 3/1
  // this is simplified
  if  ( ULongIs1 ( FractionCDenomp ( FractionCToPub  ( pointerToMe2  ) ) ) ) {
    if  (   FractionIsNegative ( FractionCToPub(pointerToMe2)) )
      String_AddChar ( WideChar )(stringtarget,  L'-' );
    else
      if  ( flagshowsign )
        String_AddChar ( WideChar )(stringtarget,  L'+' );
    // integer
    // chast>0
    { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
      String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
      String_Init ( WideChar ) ( wsp , true ) ;
// ! to catch except
      Lisp2DULongAddToStrWideChar ( FractionCNump  (
        FractionCToPub(pointerToMe2 ) ) , wsp ) ;
      StringAddString( WideChar ) ( stringtarget , wsp ) ;
      Lisp2D_StackFree  ( oldstack_ws ) ;
    }
    return  ;
  }
  if  ( FractionIsNegative ( FractionCToPub(pointerToMe2)) )
    String_AddChar ( WideChar )(stringtarget,  L'-' );
  else
    if  ( flagshowsign )
      String_AddChar ( WideChar )(stringtarget,  L'+' );
  t_fract_local fl  = {
    . pointerToMe2 = pointerToMe2 ,
    . mydelit = FractionCDenomp ( FractionCToPub ( pointerToMe2 ) ) ,
    . stringtarget = stringtarget ,
    . flagreturn  = false ,
  } ;
  fract_local ( & fl ) ;
  if ( fl . flagreturn )
    return  ;
  String_AddChar ( WideChar )(stringtarget,  L'/' );
  { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
    String  ( WideChar  ) * const wsp = StackNew_StringWideChar ( ) ;
    String_Init ( WideChar ) ( wsp , true ) ;
    Lisp2DULongAddToStrWideChar ( fl . mydelit , wsp ) ;
    StringAddString( WideChar ) ( stringtarget , wsp ) ;
    Lisp2D_StackFree  ( oldstack_ws ) ;
  }
}

static  void  fract_local ( t_fract_local * const fl ) {
  ThreadStackPointerType const oldstack_myost = Lisp2DThreadSP ( ) ;
  ULongPub  * const myost = StackNew_ULong ( ) ;
  ULong_Init ( myost ) ;
// ? how to drink out Chast & Delit ?
    { ThreadStackPointerType const oldstack_mychast = Lisp2DThreadSP ( ) ;
      ULongPub * const mychast = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( mychast , FractionCNump  ( FractionCToPub  ( fl -> pointerToMe2  ) ) ) ;
      ULongDivWithOstSet ( mychast , fl -> mydelit , myost ) ;
      enum { uiten = 10 } ;
      if  ( ULongIsx2x5 ( fl -> mydelit ) ) {
        ULongPub  * const myost2 = StackNew_ULong ( ) ;
        ULong_Init ( myost2 ) ;
        if  ( ULongIs0 ( mychast ) ) {
          // to write 1.2345e-100
// ? size_t -> int ?
          volatile size_t lexpn = 0 ;
          ULongPub  * const myost3 = StackNew_ULong ( ) ;
          ULong_ConstrCopy ( myost3 ,  myost ) ;
          do  {
            ULongMultiplyUInt ( myost3 , uiten ) ;
            ++  ( lexpn  ) ;
            ULongDivWithOstSet ( myost3 , fl -> mydelit , myost2 ) ;

            if  ( not ULongIs0 ( myost3 ) )
              break ;
            ULongSwap(myost3,myost2);

          } while ( true  ) ;
          if  ( lexpn < ( ULongIs0 ( myost2 ) ? 3 : 4 ) )
            goto  Usual ;
          { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
            String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
            String_Init ( WideChar ) ( wsp , true ) ;
            Lisp2DULongAddToStrWideChar ( myost3 , wsp ) ;
            StringAddString( WideChar ) ( fl -> stringtarget , wsp ) ;
            Lisp2D_StackFree  ( oldstack_ws ) ;
          }
          ULongSwap  ( myost3, myost2 ) ;
          if  ( not  ( ULongIs0 ( myost3 ) ) )
            String_AddChar ( WideChar ) ( fl -> stringtarget,  L'.' );
          else  {
            StringAddCharArray( WideChar )( fl -> stringtarget, L"e-" );
            { wchar_t  buf  [ 256 ] ;
              if ( swprintf ( buf , 256 , L"%zu"  , lexpn ) > 0 )
                StringAddCharArray  ( WideChar )  ( fl -> stringtarget  , buf ) ;
            }
            Lisp2D_StackFree  ( oldstack_myost ) ;
            fl -> flagreturn = true ;
            return  ;
          }
          bool  flag0 = true ;
          do  {
            ULongMultiplyUInt ( myost3 , uiten ) ;
            ULongDivWithOstSet ( myost3 , fl -> mydelit , myost2 ) ;
            { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
              String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
              String_Init ( WideChar ) (  wsp , true ) ;
              Lisp2DULongAddToStrWideChar ( myost3 , wsp ) ;
              StringAddString ( WideChar ) ( fl -> stringtarget , wsp ) ;
              Lisp2D_StackFree  ( oldstack_ws ) ;
            }
            if ( flag0 and  ( not  ( ULongIs0 ( myost3 ) ) ) )
              flag0 = false ;
            ULongSwap  ( myost3, myost2 ) ;
          } while ( not  ( ULongIs0 ( myost3 ) ) ) ;
          StringAddCharArray( WideChar )( fl -> stringtarget, L"e-" );
          { wchar_t  buf[256];
            if ( swprintf(buf,256,L"%zu",lexpn) > 0 )
              StringAddCharArray( WideChar )( fl -> stringtarget, buf );
          }
          Lisp2D_StackFree  ( oldstack_myost ) ;
          fl -> flagreturn = true ;
          return  ;
        } // if  mychast . Is0 , myost3
Usual :
    if  ( not ( ULongIs0 ( myost ) ) ) {
      { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
        String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
        String_Init ( WideChar ) (  wsp , true  ) ;
        Lisp2DULongAddToStrWideCharWithoutExp ( mychast , wsp ) ;
        StringAddString( WideChar ) ( fl -> stringtarget , wsp ) ;
        Lisp2D_StackFree  ( oldstack_ws ) ;
      } // ws
      String_AddChar ( WideChar ) ( fl -> stringtarget,  L'.' );
    } else  {
      { String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
        String_Init ( WideChar ) (  wsp , true ) ;
        Lisp2DULongAddToStrWideChar ( mychast , wsp ) ;
        StringAddString( WideChar ) ( fl -> stringtarget , wsp ) ;
      }
      Lisp2D_StackFree  ( oldstack_myost ) ;
      fl -> flagreturn = true ;
      return  ;
    }
    bool  flag0 = true ;
    do  {
      ULongMultiplyUInt ( myost , uiten ) ;
      ULongDivWithOstSet ( myost , fl -> mydelit , myost2 ) ;
      { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
        String ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
        String_Init ( WideChar ) (  wsp , true ) ;
        Lisp2DULongAddToStrWideChar ( myost , wsp ) ;
        StringAddString( WideChar ) ( fl -> stringtarget , wsp ) ;
        Lisp2D_StackFree  ( oldstack_ws ) ;
      }
      if  ( flag0 )
        if  ( not ( ULongIs0 ( myost ) ) )
          flag0 = false ;
      ULongSwap  ( myost,myost2 ) ;
    } while ( not ( ULongIs0 ( myost ) ) ) ;
    Lisp2D_StackFree  ( oldstack_myost ) ;
    fl -> flagreturn = true ;
    return  ;
  } // if  ( mydelit . Isx2x5
  if  ( not ( ULongIs0 ( mychast ) ) ) {
    { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
      String  ( WideChar  ) * const wsp = StackNew_StringWideChar ( ) ;
      String_Init ( WideChar ) ( wsp , true ) ;
      Lisp2DULongAddToStrWideChar ( mychast , wsp ) ;
      StringAddString( WideChar ) ( fl -> stringtarget , wsp ) ;
      Lisp2D_StackFree  ( oldstack_ws ) ;
    }
        if  ( not ( ULongIs0 ( myost ) ) )
          String_AddChar ( WideChar )( fl -> stringtarget,  L'/' );
        else {
          Lisp2D_StackFree  ( oldstack_myost ) ;
          fl -> flagreturn = true ;
          return  ;
        }
      }
      Lisp2D_StackFree  ( oldstack_mychast ) ;
    } // mychast
    { ThreadStackPointerType const oldstack_ws = Lisp2DThreadSP ( ) ;
      String  ( WideChar  ) * const wsp = StackNew_StringWideChar ( ) ;
      String_Init ( WideChar ) ( wsp , true ) ;
      Lisp2DULongAddToStrWideChar ( myost , wsp ) ;
      StringAddString( WideChar ) ( fl -> stringtarget , wsp ) ;
      Lisp2D_StackFree  ( oldstack_ws ) ;
  }
  Lisp2D_StackFree  ( oldstack_myost ) ;
}

# include "castb.h"
# include "widestring/inline.h"
