// Lisp2D - Garbage collector
// ©2007-23 by Alex Glebe

# include <stdbool.h>
# include <stddef.h>
# include <iso646.h>
# include "gc.h"
# include "sharedc.h"

bool  TimeToGC  ( void ) {
  if  ( not allowgc )
    return  false ;
  size_t  const gcobjectsnow2 = GCCount3  ( ) ;
  return  ( gcobjects2  < ( gcobjectsnow2  >>  1  ) )  or ( gcobjectsnow2 < ( gcobjects2 >>  1  ) ) ;
}

void  UniversalDeclareForGC  ( ParaLispObject  * const o ) {
  if  ( o )
    ParaLispObject_virtualDeclareForGC ( o  ) ; 
}

# include "env/public.h"
# include "stcl/public.h"

void  GCDeclareContent ( StackClusterPrivate * const stclpp ) {
  { ParaLispObject  * * d = VectorBegin ( StackPLOP ) ( StackClusterPrivate_stackcThisp ( stclpp  ) ) ;
    size_t i  = VectorSizeCopy  ( StackPLOP ) ( StackClusterPrivate_stackcThisp ( stclpp  ) ) ;
    for ( ; i > 0 ; -- i  , ++  d )
      ParaLispObject_virtualDeclareForGC ( * d  ) ;
  }
  { ParaLispObject  * * d = VectorBegin ( StackPLOP ) ( StackClusterPrivate_stackcMethodp ( stclpp  ) ) ;
    size_t  i = VectorSizeCopy  ( StackPLOP ) ( StackClusterPrivate_stackcMethodp ( stclpp  ) ) ;
    for ( ; i > 0 ; -- i  , ++  d )
      ParaLispObject_virtualDeclareForGC ( * d) ; 
  }
  { ParaLispObject  * *  d  = VectorBegin ( StackPLOP ) ( StackClusterPrivate_stackcproArgp  ( stclpp  ) ) ;
    size_t  i = VectorSizeCopy  ( StackPLOP ) ( StackClusterPrivate_stackcproArgp ( stclpp  ) ) ;
    for ( ; i > 0 ; -- i  , ++  d ) 
      ParaLispObject_virtualDeclareForGC ( * d) ; 
  }
  { ParaLispObject  * * d = VectorBegin ( StackPLOP ) ( StackClusterPrivate_stackcp ( stclpp  ) ) ;
    size_t  i = VectorSizeCopy(StackPLOP)  ( StackClusterPrivate_stackcp  ( stclpp  ) ) ;
    for ( ; i > 0 ; -- i  , ++  d )
      ParaLispObject_virtualDeclareForGC ( * d  ) ;
  }
  ParaLispObject_virtualDeclareForGC ( Environment_Env ( StackClusterPrivate_envLocal ( stclpp  ) ) ) ;
  ParaLispObject_virtualDeclareForGC ( Environment_Env ( StackClusterPrivate_envExternal  ( stclpp ) ) ) ;
  ParaLispObject_virtualDeclareForGC ( Environment_NamespacePropList ( StackClusterPrivate_envExternal  ( stclpp) ) ) ;
  ParaLispObject_virtualDeclareForGC ( Environment_NamespacePropList ( StackClusterPrivate_envLocal ( stclpp ) ) ) ;
  ParaLispObject_virtualDeclareForGC ( StackClusterPrivate_result ( stclpp  ) ) ;
  if  ( StackClusterPrivate_resultplace ( stclpp  ) )
    UniversalDeclareForGC  ( * StackClusterPrivate_resultplace  ( stclpp  ) ) ;
  if  ( StackClusterPrivate_resultplaceforgc  ( stclpp  ) )
    ParaLispObject_virtualDeclareForGC ( StackClusterPrivate_resultplaceforgc ( stclpp  ) ) ;
  UniversalDeclareForGC  ( StackClusterPrivate_whole  ( stclpp  ) ) ;
  UniversalDeclareForGC  ( StackClusterPrivate_exceptionvalue ( stclpp  ) ) ;
  UniversalDeclareForGC  ( StackClusterPrivate_exceptionhistory ( stclpp  ) ) ;
  StackClusterHistoryGCDeclareContent ( StackClusterPrivate_myHistory ( stclpp  ) ) ;
}
    
# include "complex/public.h"
# include "cons/private.h"
# include "vector/public.h"
# include "bytedeque/vectorlength.h"
# include "globallista/public.h"

size_t  GCCount3  ( void ) {
  return  ( GlobalListSizePublic  ( & GlobalListOfObjects ) +
// ! may be 0 but not empty 
    GlobalListAtomicSize ( & Lisp2DGlobalListOfStackClusters2 ) +
    GlobalListSizePublic(&GlobalListOfComplex ) + GlobalListSizePublic(&GlobalListOfLayer  ) +
# ifndef LISP2D_DONTUSEX11
    GlobalListSizePublic(&GlobalListOfWindows  ) +
# endif  // LISP2D_DONTUSEX11  
    GlobalListSizePublic(&GlobalListOfConsPairs  ) +
    PLOVector_gcvectorslength_CValue  ( ) + bytevector_length_value ( ) +
    MapPubSize  ( treeSymbolsLessType ) ( treeSymbols_pub ) ) ;
}

# ifndef LISP2D_DONTUSEX11

# include "window/public.h"

# define  PLOWindow_HaveThread  Lisp2D_PLOWindow_HaveThread
inline  bool  PLOWindow_HaveThread  ( PLOWindow const * me ) ;
# endif // LISP2D_DONTUSEX11

# include "inerrors.h"
  
// !! dismiss        UniversalDeclareForGC2(ParaLispObject  *)  => 
//  code will use itself       if(..)..DeclareForGC2();
  
// nullptr -> nothing
// object -> to GlobalListOfObjects
// window -> to GlobalListOfWindows
// Lock's not used
    
# include "statistics.h"
# include "cons/inline.h"
# include "plocomplex/public.h"

typedef struct  sLisp2D_lambdaGC  Lisp2D_lambdaGC ;

struct  sLisp2D_lambdaGC  {
  GlobalList  * liga ;
  GlobalList  * ligl ;
} ;

# define  lambdaGC_Init2Lists Lisp2D_lambdaGC_Init2Lists
inline  void  lambdaGC_Init2Lists  ( Lisp2D_lambdaGC * me  , GlobalList  * a , GlobalList  * l ) ;
  
# include "define.h"

static  void  Lisp2D_lambdaGC_Fun ( Lisp2D_lambdaGC * const me  ) {
  GlobalListClearSwap (  me ->  liga ,  me  ->  ligl ) ;
  ParaLispObject  * j = ParaLispObjectFromMember ( GlobalListFirst ( me -> liga ) ) ;
  for ( ; j ;
    j = ParaLispObjectFromMember ( GlobalListMemberNext ( & j -> base ) ) ) {
# ifdef LISP2D_DEBUG
    if ( j -> objstate not_eq Lisp2DObjectStateConstructed ) {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"GC:j->objstate not_eq  Lisp2DObjectStateConstructed\n" ;
      fputws  ( &((*Lisp2DExceptionStringGlobal . str)[0]) , stderr ) ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
    }
# endif              
    j -> flagGCPassed =false ;
  }
}
  
# define  threadLambdaGC2  Lisp2D_threadLambdaGC2
static  void  * threadLambdaGC2 (  void  * const p ) {
  Lisp2D_lambdaGC  * const lp  = ( Lisp2D_lambdaGC  * ) p ;
  Lisp2D_lambdaGC_Fun ( lp ) ;
  return  ( void  * ) 0 ;
}
  
# include "listplo/public.h"
# include <errno.h>
# include "globallista/public.h"
# include "globallistp/struct.h"
# include "inner-history/inline.h"
# include "window/private.h"

void  GarbageCollector  ( void ) {
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t const  Lisp2Dwsih_gc0 [ ] = L"GC:Start" ;
  static  wchar_t const  Lisp2Dwsih_gc12  [ ] = L"GC:Lisp2DPassagePassLock ( & passageGCC)=OK" ;
  static  wchar_t const  Lisp2Dwsih_gc7 [ ]  = L"GC:rolling by width0" ;
  static  wchar_t const  Lisp2Dwsih_gc2 [ ]  = L"GC:rolling by width" ;
  static  wchar_t const  Lisp2Dwsih_gc14 [ ] = L"GC:GCDeclareContent2 ( * iclust )" ;
  static  wchar_t const  Lisp2Dwsih_gc8 [ ]  = L"GC:alltasks" ;
  static  wchar_t const  Lisp2Dwsih_gc9 [ ] = L"GC:for ( Lisp2D::NameSpaceDef  & ns  : Lisp2D::namespacedef  )" ;
  static  wchar_t const  Lisp2Dwsih_gc10 [] = L"GC:Lisp2D::envtop2 ->  DeclareForGC2" ;
  static  wchar_t const  Lisp2Dwsih_gc11[]  = L"GC:create initial elements" ;
  static  wchar_t const  Lisp2Dwsih_gc1[]  = L"GC:End" ;
# endif
// ! to put innerhistory
# ifdef LISP2D_DEBUG
  debugGCRunning  = true  ;
# endif
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc0  ) ;
# endif    
  PassagePassLock ( passageGCCPub ) ;
  { 
# ifdef LISP2D_BIMSTAT
  struct  timespec  timeBegin ;
  clock_gettime ( CLOCK_REALTIME  , & timeBegin ) ;
# endif
  PLOVector_gcvectorslength_Clear ( ) ;
  bytevector_length_clear ( ) ;
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc12 ) ;
# endif          
  { pthread_t ptar1 ;
# ifndef  LISP2D_DONTUSEX11    
    pthread_t ptar2 ;
# endif  
    pthread_t ptar3 ;
    pthread_t ptar4 ;
    pthread_t ptar5 ;
  
  struct  threadlists {
    wchar_t const * const name ;
    pthread_t * const ptp ;
    GlobalList  * const gap  ;
    GlobalList  * const glp  ;
  } ;
    
  struct threadlists  const  vector_pair_global_list [ ] = {
    { L"Objects"  , & ptar1, & GlobalListOfObjectsGarbage  , & GlobalListOfObjects } ,
    { L"Complex"  , & ptar3, & GlobalListOfComplexGC , & GlobalListOfComplex } ,
    { L"Layer"  , & ptar4, & GlobalListOfLayerGC , & GlobalListOfLayer } ,
    { L"ConsPairs"  , & ptar5, & GlobalListOfConsPairsGC , & GlobalListOfConsPairs } ,
# ifndef  LISP2D_DONTUSEX11
    { L"Windows"  , & ptar2, & GlobalListOfWindowsGarbage  , & GlobalListOfWindows } ,
# endif
  }  ;
  enum  { arr_size = sizeof  ( vector_pair_global_list ) / sizeof  ( vector_pair_global_list [ 0 ] ) } ;
  Lisp2D_lambdaGC arr_lgc2 [ arr_size ] ;
  for ( int i = 0 ; i < arr_size ; ++ i ) {
    Lisp2D_lambdaGC * const j2  = & arr_lgc2  [ i ] ;
    struct threadlists  const * const  tl  = & vector_pair_global_list [ i ] ;
    Lisp2D_lambdaGC_Init2Lists  ( j2  , tl -> gap , tl -> glp ) ;
    int const er  = pthread_create  ( tl  ->  ptp , ( pthread_attr_t const * ) 0 , threadLambdaGC2  , j2 ) ;
    if ( er ) {
      fwprintf  ( stderr  , L"GC:%ls\n" , strerror  ( errno ) ) ;
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"GC:pthread_create" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    }
  }
  
# ifdef LISP2D_BIMSTAT
  struct  timespec  timeBeginSymbols  ;
  clock_gettime ( CLOCK_REALTIME  , & timeBeginSymbols  ) ;
# endif  

  GlobalListClear ( & GlobalListOfSymbolsGarbage  ) ;
  // symbols will be always
  { size_t  const symbols = MapPubSize  ( treeSymbolsLessType ) ( treeSymbols_pub ) ;
    for ( size_t  j = 0 ; j < symbols ; ++  j ) {
      PLOSymbol * const i = MapPubCAt ( treeSymbolsLessType ) ( treeSymbols_pub , j ) -> key ;
      i ->  plo . flagGCPassed = false ;
      GlobalListPushNotLock ( & GlobalListOfSymbolsGarbage  , & i -> plo  . base ) ;
    }
  }
  MapPubClear ( treeSymbolsLessType ) ( treeSymbols_pub ) ;
# ifdef LISP2D_BIMSTAT
  struct  timespec  timeEndSymbols  ;
  clock_gettime ( CLOCK_REALTIME  , & timeEndSymbols  ) ;
  IncreaseStat5  (& timeBeginSymbols  ,& timeEndSymbols  ,& GCstatSymbols ) ;
# endif
  { struct threadlists const * i2  = & vector_pair_global_list [ arr_size ] ;
    do  {
      --  i2  ;
      pthread_join ( * ( i2 -> ptp ) , (  void  * * ) 0 ) ;
    } while ( i2 > & vector_pair_global_list [ 0 ] ) ;
  } // i2
  } // threads
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc11 ) ;
# endif          
  // create initial elements
  { Lisp2D_ListNode ( PLO ) * j = Lisp2D_ListFirst  ( PLO ) ( list_global_atomsc  ) ;
    while ( j ) {
# ifdef LISP2D_DEBUG  
      if ( Lisp2D_ListNodeValue  ( PLO ) ( j ) -> objstate not_eq Lisp2DObjectStateConstructed ) {
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
          L"GC:j -> ObjState ( ) not_eq  Lisp2DObjectStateConstructed\n" ;
        fputws  ( * Lisp2DExceptionStringGlobal . str , stderr ) ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
      }
# endif
      ParaLispObject_virtualDeclareForGC ( Lisp2D_ListNodeValue  ( PLO ) ( j ) ) ;
      j = Lisp2D_ListNodeNext ( PLO ) ( j ) ;
    }
  }
// ! ? envtop2 : Eval2 - modifies variable at exit  
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc10 ) ;
# endif          
  ParaLispObject_virtualDeclareForGC ( envtop2) ;
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc9 ) ;
# endif        
  { NameSpaceDef * nsj = VectorBegin ( NameSpaceDefC )  ( & namespacedefc ) ;
    NameSpaceDef * const nsend = VectorEnd ( NameSpaceDefC )  ( & namespacedefc ) ;
    while ( nsj not_eq nsend ) {
      UniversalDeclareForGC  ( &nsj  -> name ->plo ) ;
      UniversalDeclareForGC  ( nsj  -> var ) ;
      UniversalDeclareForGC  ( nsj  -> prop  ) ;
      ++  nsj ;
    }
  }
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc8 ) ;
# endif      
  // alltasks

  { StackClusterPrivate * iclustnext  ;
    StackClusterPrivate * clustbase = GlobalListMemberToStackClusterPrivate (
        GlobalListAtomicFirst ( & Lisp2DGlobalListOfStackClusters2 )  ) ;
    StackClusterPrivate * iclust = clustbase  ;
    for ( ; iclust  ; iclust  = iclustnext  ) {
      if  ( GlobalListMemberNext  ( StackClusterPrivate_globallistmember  ( iclust ) ) )  {
        iclustnext  = GlobalListMemberToStackClusterPrivate (
            GlobalListMemberNext  ( StackClusterPrivate_globallistmember  ( iclust ) ) ) ;
      } else
        iclustnext  = 0 ;
      if  ( StackClusterPrivate_wanttobeingarbage  ( iclust ) ) {
        GlobalListAtomicPopNotLock ( & Lisp2DGlobalListOfStackClusters2 , StackClusterPrivate_globallistmember ( iclust ) ) ;
# ifdef LISP2D_DEBUG        
        if  ( countStackClusterWantToBeInGarbage_Value  ( ) ==  0  )
          fputws(L"GC:countStackClusterWantToBeInGarbage ==  0\n",stderr);
        else
# endif          
          countStackClusterWantToBeInGarbage_Dec  ( ) ;
        if  ( ( GlobalListSizePublic(&GlobalListOfStackClustersGarbage2   ) + 1 ) <
          GlobalListAtomicSize ( & Lisp2DGlobalListOfStackClusters2 ) ) {
          StackClusterPrivate_setwanttobeingarbage ( iclust , false ) ;
          Lisp2DStackPLOPClear  ( StackClusterPrivate_stackcp ( iclust ) ) ;
          Lisp2DStackPLOPClear  ( StackClusterPrivate_stackcThisp ( iclust ) ) ;
// !? to Clear ?!
          GlobalListPushNotLock ( & GlobalListOfStackClustersGarbage2  , StackClusterPrivate_globallistmember  ( iclust ) ) ;
        } else {
          StackClusterPrivate_Destructor  ( iclust ) ;
          free  ( iclust ) ;
        }
        // if  iclust  ->  WantToBeInGarbage
      } else {
# ifdef  LISP2D_INNERHISTORY
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc14 ) ;
# endif                    
        GCDeclareContent ( iclust ) ;
      }
    }
  } // iclustnext
# ifdef LISP2D_DEBUG        
  { size_t const wo = countStackClusterWantToBeInGarbage_Value  ( ) ;
    if  ( wo > 0  )
      fwprintf ( stderr , L"GC:(countStackClusterWantToBeInGarbage > 0)=%zu\n"  , wo ) ;
  }
# endif  
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc7 ) ;
# endif  
  // rolling by width
  { ParaLispObject  *   oldFirstObjects = 0 ;
    ParaLispObject  *   oldFirstObjects2  = 0 ;
    size_t  countNewGarbage ;
    size_t volatile countNewGarbage2  = GlobalListSizePublic(&GlobalListOfObjectsGarbage   )  ;

    PLOCons * oldFirstCons  = 0 ;
    PLOCons * oldFirstCons2 = 0 ;
    size_t  countNewConsGC  ;
    size_t volatile countNewConsGC2 =  GlobalListSizePublic(& GlobalListOfConsPairsGC  )  ;
    
    ParaLispObject  * oldFirstLayers  = 0 ;
    ParaLispObject  * oldFirstLayers2 = 0 ;
    size_t  countNewLayerGarbage  ;
    size_t volatile countNewLayerGarbage2 = GlobalListSizePublic(&GlobalListOfLayerGarbage   )  ;
# ifndef  LISP2D_DONTUSEX11    
    PLOWindow * oldFirstWindows = 0 ;
    PLOWindow * oldFirstWindows2  = 0 ;
    size_t countNewWindowsGarbage  ;
    size_t volatile countNewWindowsGarbage2 = GlobalListSizePublic(&GlobalListOfWindowsGarbage   )  ;
# endif    
// !? parallelize cycle by all processors ?!
    
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc2 ) ;
# endif
    
    do  {
      countNewGarbage = countNewGarbage2  ;
      countNewConsGC  = countNewConsGC2 ;
      countNewLayerGarbage  = countNewLayerGarbage2 ;
# ifndef  LISP2D_DONTUSEX11          
      countNewWindowsGarbage  = countNewWindowsGarbage2 ;
# endif      
      oldFirstObjects2  = ParaLispObjectFromMember  ( GlobalListFirst(&GlobalListOfObjects  ) ) ;
      oldFirstCons2 = PLOCons_FromMember ( GlobalListFirst ( & GlobalListOfConsPairs  ) ) ;
      
      oldFirstLayers2  = ParaLispObjectFromMember ( GlobalListFirst(&GlobalListOfLayer  ) ) ;
# ifndef  LISP2D_DONTUSEX11          
      oldFirstWindows2  = PLOWindow_FromMember ( GlobalListFirst(&GlobalListOfWindows  ) ) ;
# endif      
    // oldFirstObjects2 .. objectsi .. oldFirstObjects
      for ( ParaLispObject  * objectsi  = oldFirstObjects2 ; 
        objectsi not_eq  oldFirstObjects  ;
        objectsi  = ParaLispObjectFromMember ( GlobalListMemberNext(&objectsi  -> base  ) ) )
        ParaLispObject_virtualGCDeclareContent ( objectsi ) ;

      { PLOCons * objectsi  = oldFirstCons2  ;
        for ( ; objectsi  not_eq  oldFirstCons  ;
          objectsi  = PLOCons_FromMember ( GlobalListMemberNext( &objectsi  -> plo.base  ) ) )
          PLOCons_staticGCDeclareContent (&objectsi->plo ) ;
      }

      for ( ParaLispObject  * objectsi  = oldFirstLayers2  ;
        objectsi not_eq  oldFirstLayers ;
        objectsi  = ParaLispObjectFromMember ( GlobalListMemberNext ( & objectsi  -> base  ) ) )
        ParaLispObject_virtualGCDeclareContent ( objectsi ) ;

# ifndef  LISP2D_DONTUSEX11            
      for ( PLOWindow * objectsi  = oldFirstWindows2   ;
           objectsi not_eq  oldFirstWindows  ;
        objectsi  = PLOWindow_FromMember ( GlobalListMemberNext  ( & PLOWindow_plo(objectsi)  -> base ) ) )
        PLOWindow_staticGCDeclareContent (  PLOWindow_plo(objectsi) ) ;  
# endif        

      oldFirstObjects = oldFirstObjects2  ;
      oldFirstCons  = oldFirstCons2 ;
      oldFirstLayers = oldFirstLayers2  ;
# ifndef  LISP2D_DONTUSEX11          
      oldFirstWindows = oldFirstWindows2  ;
# endif      
      
      countNewGarbage2  =  GlobalListSizePublic(&GlobalListOfObjectsGarbage   )  ;
      countNewConsGC2 =  GlobalListSizePublic(&GlobalListOfConsPairsGC  )  ;
      countNewLayerGarbage2 =  GlobalListSizePublic(&GlobalListOfLayerGarbage   )  ;
# ifndef  LISP2D_DONTUSEX11          
      countNewWindowsGarbage2 =  GlobalListSizePublic(&GlobalListOfWindowsGarbage   )  ;
# endif      
    } while ( ( countNewGarbage not_eq  countNewGarbage2  ) or
      ( countNewConsGC not_eq  countNewConsGC2  )  or
      ( countNewLayerGarbage not_eq  countNewLayerGarbage2  )
# ifndef  LISP2D_DONTUSEX11          
      or   (  countNewWindowsGarbage not_eq  countNewWindowsGarbage2  )
# endif      
      ) ;
  } // countNewGarbage, oldFirstObjects
  // kill objects
  { ParaLispObject * arrj = ParaLispObjectFromMember ( GlobalListFirst ( & GlobalListOfObjectsGarbage ) ) ;
    if  ( arrj ) {
      do  {
        ParaLispObject  * const  arrjNext = ParaLispObjectFromMember ( GlobalListMemberNext ( & arrj  -> base  ) )  ;
        // !? to use virtual StoreToDeletedObjects
        ParaLispObject_virtualDestr  ( arrj ) ;

        // removing NOT ParaLispObject,
        // BUT full object with private members and .plo field
        free  ( arrj -> vaddr . addr . p ) ;
        arrj  = arrjNext  ;
      } while ( arrj  ) ;
    
      GlobalListClear ( & GlobalListOfObjectsGarbage ) ;
    } // if arrj
  } // arrj
  // remove unnecessary complex
  { size_t  complexs = GlobalListSizePublic ( & GlobalListOfComplex  ) ;
    size_t  complexgarbages = GlobalListSizePublic  ( & GlobalListOfComplexGarbage ) ;
//fwprintf(stdout,L"s=%zu g=%zu ",complexs,complexgarbages);fflush(stdout);
    while ( complexgarbages > complexs  ) {
      GlobalListMember * x ;
# ifdef LISP2D_DEBUG      
      if  ( not  GlobalListisOKPopFirstNotLock(&GlobalListOfComplexGarbage , & x ) )
        fputws  ( L"GC:remove unnecessary complex1:x  ==  nullptr\n"  , stderr  ) ;
      else  
# else
      Lisp2DGlobalListisOKPopFirstNotLock(&GlobalListOfComplexGarbage , & x ) ;
#endif
      --  complexgarbages ;
# ifdef LISP2D_DEBUG
    ++  Lisp2D_statFreePLOComplexPri  ;
# endif      
      // with destructor  
      // only memory no destructor
      free  ( PLOComplex_FromMember  ( x ) ) ;
    } // while complexgarbages > complexs
    size_t  complexgcs = GlobalListSizePublic(&GlobalListOfComplexGC  ) ;
    while ( ( complexgarbages  < complexs ) and ( complexgcs > 0  ) ) {
      GlobalListMember * x ;
# ifdef LISP2D_DEBUG      
      if  ( not ( GlobalListisOKPopFirstNotLock(&GlobalListOfComplexGC , & x ) ) )
        fputws  ( L"GC:remove unnecessary complex2:x  ==  nullptr\n"  , stderr  ) ;
      else
# else
      Lisp2DGlobalListisOKPopFirstNotLock(&GlobalListOfComplexGC , & x ) ;
# endif
      --  complexgcs  ;
      // valid object
      // destructed object ,memory ok
      PLOComplex_Destr  ( PLOComplex_FromMember ( x ) ) ;
      GlobalListPushNotLock ( & GlobalListOfComplexGarbage  , x ) ;
      ++  complexgarbages ;
    } // while
    while ( complexgcs  > 0 ) {
      GlobalListMember * x ;
# ifdef LISP2D_DEBUG      
      if  ( not  ( GlobalListisOKPopFirstNotLock(& GlobalListOfComplexGC , & x ) ) )
        fputws  ( L"GC:remove unnecessary complex3:x  ==  nullptr\n"  , stderr  ) ;
      else  
# else
      Lisp2DGlobalListisOKPopFirstNotLock(& GlobalListOfComplexGC , & x ) ;
# endif      
        --  complexgcs  ;
      // with destructor  , checks garbage place OR free memory 
      // big unnecessary works          
      // destructor, free memory place, NO PLOComplex::delete
      { PLOComplex * const cx = PLOComplex_FromMember ( x ) ;
        PLOComplex_staticDestr  ( &cx->plo ) ;
# ifdef LISP2D_DEBUG
        ++  Lisp2D_statFreePLOComplexPri  ;
# endif        
        free  ( cx ) ;
      } // cx
    } // while complexgcs  > 0
  }  // complexs
  // remove unnecessary cons pairs
  { size_t  conssize = GlobalListSizePublic(&GlobalListOfConsPairs);
    size_t  conspairgarbages = GlobalListSizePublic(&GlobalListOfConsPairsGarbage);
    while(conspairgarbages  > conssize) {
      GlobalListMember * x;
# ifdef  LISP2D_DEBUG      
      if(not  GlobalListisOKPopFirstNotLock(&GlobalListOfConsPairsGarbage,&x))
        fputws  ( L"GC:remove unnecessary conspairs1:x  ==  nullptr\n"  , stderr  ) ;
      else
# else
      Lisp2DGlobalListisOKPopFirstNotLock(&GlobalListOfConsPairsGarbage,&x);
# endif
      --  conspairgarbages;
      { PLOCons * const cx = PLOCons_FromMember ( x ) ;
        PLOCons_staticDestr  ( &cx->plo ) ;
# ifdef LISP2D_DEBUG
        ++  Lisp2D_statFreePLOCons  ;
# endif        
        free  ( cx ) ;
      } // cx
    } // while garbagesize > conssize
    size_t  conspairsgcs = GlobalListSizePublic(&GlobalListOfConsPairsGC);
    while((conspairgarbages  < conssize)  and (conspairsgcs > 0)){
      GlobalListMember * x;
# ifdef  LISP2D_DEBUG      
      if(not  GlobalListisOKPopFirstNotLock(&GlobalListOfConsPairsGC,&x))
        fputws  ( L"GC:remove unnecessary conspairs2:x  ==  nullptr\n"  , stderr  ) ;
      else
# else
      GlobalListisOKPopFirstNotLock(&GlobalListOfConsPairsGC,&x);
# endif
      --  conspairsgcs;
      PLOCons_Clear ( PLOCons_FromMember ( x ) ) ;
      GlobalListPushNotLock(&GlobalListOfConsPairsGarbage,  x);
      ++  conspairgarbages;
    } // while
    while(conspairsgcs  > 0){
      GlobalListMember * x;
# ifdef  LISP2D_DEBUG      
      if(not  GlobalListisOKPopFirstNotLock(&GlobalListOfConsPairsGC,&x))
        fputws  ( L"GC:remove unnecessary conspairs3:x  ==  nullptr\n"  , stderr  ) ;
      else
# else
      GlobalListisOKPopFirstNotLock(&GlobalListOfConsPairsGC,&x);
# endif
      --  conspairsgcs;
      { PLOCons * const cx = PLOCons_FromMember ( x ) ;
        PLOCons_staticDestr  ( &cx->plo ) ;
# ifdef LISP2D_DEBUG
        ++  Lisp2D_statFreePLOCons  ;
# endif        
        free  ( cx ) ;
      }
    } // while conspairsgcs  > 0
  }  // cons pairs
  // remove unnecessary layers
  { size_t  layers = GlobalListSizePublic(&GlobalListOfLayer);
    size_t  layergarbages = GlobalListSizePublic(&GlobalListOfLayerGarbage);
    while(layergarbages > layers){
      
      GlobalListMember * x;
# ifdef  LISP2D_DEBUG      
      if(not  GlobalListisOKPopFirstNotLock(&GlobalListOfLayerGarbage,&x))
        fputws  ( L"GC:remove unnecessary layer\n" , stderr ) ;
      else
# else
      GlobalListisOKPopFirstNotLock(&GlobalListOfLayerGarbage,&x);
# endif
      --  layergarbages;
      { PLOEnvironmentLayer * const cx = PLOEnvironmentLayer_FromMember ( x ) ;
        PLOEnvironmentLayer_staticDestr  ( & cx  ->  plo ) ;
        free  ( cx ) ;
      }
    } // while
    size_t  layergcs = GlobalListSizePublic(&GlobalListOfLayerGC);
    while((layergarbages  < layers) and (layergcs > 0)){
      
      GlobalListMember * x ;
# ifdef  LISP2D_DEBUG
      if  ( not  GlobalListisOKPopFirstNotLock  ( & GlobalListOfLayerGC , & x ) )
        fputws  ( L"GC:remove unnecessary layer2\n" , stderr  ) ;
      else
# else
      GlobalListisOKPopFirstNotLock ( & GlobalListOfLayerGC , & x ) ;
# endif
      { --  layergcs;
        EnvironmentLayerClear ( & PLOEnvironmentLayer_FromMember ( x ) -> envlay ) ;
        ParaLispObject_virtualisOKSetFlagConstFalse ( & PLOEnvironmentLayer_FromMember ( x ) ->plo) ;
        GlobalListPushNotLock(&GlobalListOfLayerGarbage,  x);
        ++  layergarbages;
      }
    } // while
    while(layergcs  > 0){
      
      GlobalListMember * x;
# ifdef  LISP2D_DEBUG      
      if  ( not  GlobalListisOKPopFirstNotLock  ( & GlobalListOfLayerGC , & x ) )
        fputws  ( L"GC:remove unnecessary layer3\n" , stderr  ) ;
      else
# else
      GlobalListisOKPopFirstNotLock(&GlobalListOfLayerGC,&x);
# endif
      --  layergcs;
      { PLOEnvironmentLayer * const cx = PLOEnvironmentLayer_FromMember ( x ) ;
        PLOEnvironmentLayer_staticDestr  ( & cx -> plo ) ;
        free  ( cx ) ;
      }
    } // while layergcs  > 0
  }  // layers

# ifndef  LISP2D_DONTUSEX11
  // kill windows
  { PLOWindow * winj = PLOWindow_FromMember  ( GlobalListFirst ( & GlobalListOfWindowsGarbage ) ) ;
    if  ( winj ) {
# ifdef  LISP2D_INNERHISTORY
      static  wchar_t const  Lisp2Dwsih_gc3 [ ]  = L"GC:kill windows.start" ;
      Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc3  ) ;
# endif      
    do  {
      PLOWindow * const  winjNext  = PLOWindow_FromMember ( GlobalListMemberNext ( & PLOWindow_plo  ( winj  )  -> base  ) )  ;
      if  ( PLOWindow_HaveThread  ( winj ) ) {
//fwprintf(stderr,L" WinHaveThread %p ",winj);
# ifdef  LISP2D_INNERHISTORY
        static  wchar_t const  Lisp2Dwsih_gc5 [ ]  = L"GC:kill windows.HaveThread" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc5  ) ;
# endif                
        GlobalListPush  ( & GlobalListOfWindows , & PLOWindow_plo ( winj  ) -> base ) ;
        // still there
        PLOWindow_Close ( winj ) ;
      } else  {    
//fwprintf(stderr,L" FreeWin %p ",winj);
# ifdef  LISP2D_INNERHISTORY
        static  wchar_t const  Lisp2Dwsih_gc6 [ ]  = L"GC:kill windows.delete winj" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc6  ) ;
# endif     
        PLOWindow_staticDestr  ( PLOWindow_plo  ( winj  ) ) ;
        free  ( PLOWindow_to_pri  ( winj  ) ) ;
      }
      winj  = winjNext  ;
    } while ( winj  ) ;
      GlobalListClear(&GlobalListOfWindowsGarbage   ) ;
# ifdef  LISP2D_INNERHISTORY
      static  wchar_t const  Lisp2Dwsih_gc4 [ ]  = L"GC:kill windows.end" ;
      Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc4  ) ;
# endif                
    } // if winj
  }
# endif // LISP2D_DONTUSEX11
  // kill symbols
  { GlobalListMember * i = GlobalListOfSymbolsGarbage  .  first  ;
    while ( i ) {
      GlobalListMember * next = GlobalListMemberNext ( i ) ;
    
      { ParaLispObject * ip = ParaLispObjectFromMember ( i ) ;
        ParaLispObject_virtualDestr  ( ip ) ;
        // removing NOT ParaLispObject, BUT Symbols with .plo field
        free  ( ip -> vaddr . addr . p ) ;
      }

      i = next  ;
    }
  } // GlobalListMember * i
  GlobalListClear ( & GlobalListOfSymbolsGarbage ) ;
  
    //freeing space
      
    // globalsize
   
    while(GlobalListSizePublic(&GlobalListOfStackClustersGarbage2) > GlobalListAtomicSize ( & Lisp2DGlobalListOfStackClusters2 ) ) {
      GlobalListMember * x;
# ifdef  LISP2D_DEBUG      
      if(not  GlobalListisOKPopFirstNotLock(&GlobalListOfStackClustersGarbage2,&x))
        fputws  ( L"GC:remove unnecessary StackCluster\n" , stderr  ) ;
      else
# else
      GlobalListisOKPopFirstNotLock(&GlobalListOfStackClustersGarbage2,&x);
# endif
      { StackClusterPrivate * const y = GlobalListMemberToStackClusterPrivate ( x )  ;
        StackClusterPrivate_Destructor  ( y ) ;
        free  ( y ) ;
      }
    }

    //---------------------------
  
    gcobjects2  = GCCount3  ( ) ;
# ifdef  LISP2D_BIMSTAT
    struct  timespec  timeEnd;
    clock_gettime(CLOCK_REALTIME,&  timeEnd);
    IncreaseStat5(&timeBegin,&timeEnd,&GCstat);
# endif
    PassagePassUnLock ( passageGCCPub ) ;
  } // unquiepassagelock
  
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_gc1  ) ;
# endif    
  
# ifdef  LISP2D_DEBUG
  debugGCRunning  = false;
# endif  
  //fputws(L"GC:end.",stdout);fflush(stdout);
} // GarbageCollector

# include "window/inline.h"
# include "listplo/inlinepri.h"
# ifndef LISP2D_DONTUSEX11
inline  bool  PLOWindow_HaveThread  ( PLOWindow const * const me ) {
  return  PLOWindow_to_cpri(me) -> boolHaveThread  ;
}
# endif // LISP2D_DONTUSEX11
inline  void  lambdaGC_Init2Lists  ( Lisp2D_lambdaGC * const me  , GlobalList  * const a , GlobalList  * const l ) {
  me -> liga = a ;
  me -> ligl = l ;
}
# include "window/inlinepri.h"
# include "stcl/inline.h"
# include "shared/inline.h"
