// Lisp2D from GlobalListMember to ParaLispObject
// ©2003-23 by Alex Glebe

# include "define.h"
# include "plocomplex/structpub.h"
# include "plostream/struct.h"
# include "object/struct.h"
# include "cons/struct.h"
# include "backquote/struct.h"
# include "string/struct.h"
# include "char/type.h"
# include "char/structpri.h"
# include "vector/struct.h"
# include "bytedeque/struct.h"
# include "window.h"
# include "comma/struct.h"
# include "dir/struct.h"
# include "socket/struct.h"
# include "unquotesplicing/struct.h"
# include "stat/structpri.h"
# include "quote/struct.h"
# include "lock/type.h"
# include "lock/struct.h"
# include "signall.h"
# include "bimc.h"
# include "boolean.h"
# include "nilc.h"
# include "time/struct.h"
# include "window/public.h"
# include "plocomplex/type.h"

ParaLispObject * ParaLispObjectFromMember ( GlobalListMember * const m ) {
  // for cycles : if null then null
  if  ( not m )
    return  0 ;
  void  * const a = m ->  vaddr  . addr . p ;
  switch  ( m ->  vaddr  . type  ) {
  case  TypeGlobalListMemberBase  :
# ifdef LISP2D_DEBUG
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& 
      L"ParaLispObjectFromMember:type=TypeGlobalListMemberBase" ; 
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
# endif      
    return  0 ;
  case  TypeGlobalListMemberStClPriv  :
# ifdef LISP2D_DEBUG            
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& 
        L"ParaLispObjectFromMember:type=TypeGlobalListMemberStClPriv" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
# endif      
      return  0 ;
    case  TypeGlobalListMemberPLOComplex  :
      return  &(( PLOComplex *  )  a  )->plo ;
    case  TypeGlobalListMemberParaLispObject  :
      return  ( ParaLispObject *  ) ( a  ) ;
    case  TypeGlobalListMemberPLOStream :
      return  &(( PLOStream *  )  a  )->plo ;
    case  TypeGlobalListMemberPLOEnv  :
      return  & ( ( PLOEnvC  * )  a  ) ->  plo ;
    case  TypeGlobalListMemberPLOObject :
      return  & ( ( PLOObject  * )  a  )  ->  plo ;
    case  TypeGlobalListMemberPLOCons :
      return  &(( PLOCons  * )  a )->plo ;
    case  TypeGlobalListMemberBackQuotePrivate  :
      return  &(( PLOBackQuote * )  a )->plo ;
    case  TypeGlobalListMemberPLOString :
      return  &(( PLOStringPri  * )  a )-> pub . plo ;
    case  TypeGlobalListMemberPLOChar :
      return  & ( ( PLOCharPriv * )  a )  ->  pub . plo ;
    case  TypeGlobalListMemberPLOVector :
      return  &(( PLOVector * )  a )->plo ;
    case  TypeGlobalListMemberPLOByteDeque  :
      return  &(( PLOByteDeque * )  a )->plo ;
# ifndef  LISP2D_DONTUSEX11  
    case  TypeGlobalListMemberPLOWindowPri :
      return  PLOWindow_plo(&(( PLOWindow_pri * )  a )->pub) ;
# endif  
    case  TypeGlobalListMemberPLOTime :
      return  &(( PLOTime * )  a )-> pub . plo ;
    case  TypeGlobalListMemberPLOComma  :
      return  &(( PLOComma * )  a )->plo ;
    case  TypeGlobalListMemberPLODirPri  :
      return  & ( ( PLODirPri * )  a )  -> pub . plo ;
    case  TypeGlobalListMemberPLOSocket :
      return  &(( PLOSocket * )  a )->plo ;
    case  TypeGlobalListMemberPLOUnquoteSplicing  :
      return  &(( PLOUnQuoteSplicing * )  a )->plo ;
    case  TypeGlobalListMemberPLOStatPri :
      return  &(( PLOStatPri * )  a )->pub.plo ;
    case  TypeGlobalListMemberPLOQuote  :
      return  &(( PLOQuote * )  a )->plo ;
    case  TypeGlobalListMemberPLOLock :
      return  &(( PLOLock * )  a )->plo ;
    case  TypeGlobalListMemberPLOSignal :
      return  & ( ( PLOSignal * )  a )  ->  plo ;
    case  TypeGlobalListMemberPLOEnvironmentLayer :
      return  & ( ( PLOEnvironmentLayer * )  a ) -> plo ;
    case  TypeGlobalListMemberPLONil  :
      return  &(( PLONil * )  a )->plo ;
    case  TypeGlobalListMemberPLOSymbol :
      return  &(( PLOSymbol * )  a )->plo ;
    case  TypeGlobalListMemberBIFPrivate  :
      return  & ( ( BIFPrivate * )  a ) ->  plo ;
    case  TypeGlobalListMemberPLOBoolean  :
      return  &(( PLOBoolean * )  a )->plo ;
    default :
# ifdef LISP2D_DEBUG      
      wprintf(L"ParaLispObjectFromMember:unkonwn m ->  vaddr . type=%d",
              m ->  vaddr . type);
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& 
        L"ParaLispObjectFromMember:unkonwn m ->  vaddr . type" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
# endif      
       return 0 ;
  }
}

# include "window/inline.h"
