// Lisp2D - GlobalList
// ©2010-23 by Alex Glebe

# include "globallist.h"
# include "define.h"

_Atomic ( size_t ) GlobalListconstrstat = 0  ;

GlobalList  GlobalListOfStackClustersGarbage2 ;

# include "globallistp/struct.h"

void  GlobalListPush ( GlobalList * const me , GlobalListMember * const x )  {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListPush2:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif    
  Lisp2D_atomic_mutex_lock  ( & me -> amutex ) ;
  GlobalListPushNotLock ( me , x ) ;
  Lisp2D_atomic_mutex_unlock  ( & me -> amutex ) ;
}

# include "globallistm/struct.h"

void  GlobalListPushNotLock ( GlobalList * const me , GlobalListMember * const x ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListPushNotLock:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif    
  ++  ( me -> count_private ) ;
  // x will be first
  x -> prev  = 0 ;
  x -> next  = me -> first ;
  if  ( me -> first )
    me -> first ->  prev  = x ;
  me -> first = x ;
}

void  GlobalListPop  ( GlobalList * const me , GlobalListMember * const x ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListPop2:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }    
# endif    
  Lisp2D_atomic_mutex_lock  ( & me -> amutex ) ;
  GlobalListPopNotLock  ( me  , x ) ;
  Lisp2D_atomic_mutex_unlock  ( & me -> amutex ) ;      
}

bool  GlobalListisBADPopFirst  ( GlobalList * const me , GlobalListMember * * const result ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListPopFirst:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }    
# endif    
  Lisp2D_atomic_mutex_lock  ( & me -> amutex ) ;
  bool  bad = not GlobalListisOKPopFirstNotLock ( me , result  ) ;
  Lisp2D_atomic_mutex_unlock  ( & me -> amutex ) ;  
  return  bad ;  
}
  
bool  GlobalListisOKTryPopFirst ( GlobalList * const me , GlobalListMember * * const result  ) {
# ifdef LISP2D_DEBUG
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListisOKTryPopFirst:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
  if  ( not Lisp2D_atomic_mutex_trylockOK ( & me -> amutex ) )
    return  false ;
  bool  const res = GlobalListisOKPopFirstNotLock ( me , result  ) ;
  Lisp2D_atomic_mutex_unlock  ( & me -> amutex ) ;
  return  res ;
}
  
// without freeze  
bool  GlobalListisOKTryPush ( GlobalList * const me , GlobalListMember * const item  ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListisOKTryPush:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
  if  ( not Lisp2D_atomic_mutex_trylockOK ( & me -> amutex ) )
    return  false ;
  GlobalListPushNotLock ( me , item  ) ;
  Lisp2D_atomic_mutex_unlock  ( & me -> amutex ) ;
  return  true  ;
}

bool  GlobalListisOKPopFirstNotLock ( GlobalList * const me , GlobalListMember * * const result  ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListisOKPopFirstNotLock:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif    
  if  ( me -> count_private  ==  0  ) 
    return  false ;
  --  me -> count_private ;
  ( * result  )  = me -> first ;
  me -> first = ( me -> first ->  next  ) ;
  if  ( me -> first )
    me -> first ->  prev = 0 ;
  return  true  ; 
}

// without freeze  
bool  GlobalListisOKTryPop  ( GlobalList * const me , GlobalListMember * const item  ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListisOKTryPop:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
  if  ( not Lisp2D_atomic_mutex_trylockOK ( & me -> amutex ) )
    return  false ;
  GlobalListPopNotLock ( me , item );
  Lisp2D_atomic_mutex_unlock  ( & me -> amutex ) ;
  return  true  ;
}

# include "inner-history/inline.h"

void  GlobalListPopNotLock ( GlobalList * const me , GlobalListMember * const x ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListPopNotLock:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif    
  if  ( me -> count_private ==  0 ) {
    # define  wsih_popnlock0  Lisp2Dwsih_popnlock08234
    static  wchar_t  const wsih_popnlock0 [ ] =
      L"GlobalList  ::  isBADPopNotLock ( GlobalListMember2 & x ):count_private ==  0"  ;
# ifdef  LISP2D_INNERHISTORY
    Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_popnlock0  ) ;
# endif          
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&wsih_popnlock0  ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
  --  (me -> count_private );
  if  ( x == me -> first  ) {
    // it's was first
    me -> first = ( x -> next  ) ;
    if  ( me -> first )
      me -> first ->  prev = 0 ;
    // else it was one
  } else  {
    if  ( x -> prev ==  0 ) {
      # define  wsih_popnlock1  Lisp2Dwsih_popnlock11319
      static  wchar_t  const wsih_popnlock1 []=
        L"GlobalList  ::  isBADPopNotLock ( GlobalListMember2 & x ):x . prev ==  nullptr"  ;
# ifdef  LISP2D_INNERHISTORY
      Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_popnlock1 ) ;
# endif                  
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&wsih_popnlock1  ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
    }
    x -> prev  ->  next  = ( x -> next  ) ;
    if  ( x -> next  )
      x -> next  ->  prev  = ( x -> prev  ) ;
    // else it was last
  } 
}

# include "globallistp/public.h"

GlobalList  * GlobalListSet ( GlobalList  * const me  , GlobalList  const * const  father  ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListSet:my list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
  if  ( father -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListSet:father's list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif    
  me  -> first = ( father  ->  first ) ;
  me  -> count_private = ( father  ->  count_private ) ;
  return  me  ; 
}
  
void  GlobalListClearSwap ( GlobalList  * const me  , GlobalList  * const x )  {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListClearSwap:my list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
  if  ( x -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListClearSwap:x's list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif    
  me  -> first = ( x -> first ) ;
  x -> first = 0 ;
  me  -> count_private = ( x -> count_private ) ;
  x -> count_private = 0 ; 
}

void  GlobalListSwapM  ( GlobalListMember * const a  , GlobalListMember * const b ) {
  GlobalListMember  * tmp = a -> prev ;
  a -> prev = b -> prev ;
  b -> prev = tmp ;
  tmp = a -> next ;
  a -> next = b -> next ;
  b -> next = tmp ; 
}  

void GlobalListConstructor  ( GlobalList * const me )  {
# ifdef LISP2D_DEBUG 
  me -> s = Lisp2DObjectStateRaw ; 
# endif
  me -> first = 0 ;
  me -> count_private = 0 ;  
  Lisp2D_atomic_mutex_init  ( & me -> amutex );
# ifdef  LISP2D_CONSTRSTAT
  ++ GlobalListconstrstat ;
# endif    
# ifdef LISP2D_DEBUG  
  me -> s = Lisp2DObjectStateConstructed ; 
# endif  
}
  
# ifdef LISP2D_BIMSTAT  
size_t  GlobalListStatFast  ( GlobalList const * const me ) { 
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListStatFast:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  } 
# endif    
  return  atomic_mutex_StatFast ( & me -> amutex  ) ;
}  

size_t  GlobalListStatWait ( GlobalList const * const me ) { 
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListStatWait2:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  } 
# endif    
  return  atomic_mutex_StatWait(& me -> amutex) ; 
}
# endif

# ifdef LISP2D_DEBUG  
void GlobalListDestructor  ( GlobalList * const me  ) { 
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListDestructor:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  } 
  Lisp2D_atomic_mutex_destruct(& me -> amutex);
  me -> s = Lisp2DObjectStateDestructed ;  
}   
# endif    
  
GlobalListMember * GlobalListFirst ( GlobalList const * const me ) { 
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListFirst:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  } 
# endif    
  return me -> first  ; 
} 
  
size_t  GlobalListSizePublic  ( GlobalList const * const me ) { 
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) { 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListSizePublic:list is not live" ; 
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  } 
# endif    
  return  me -> count_private ; 
} 

void  GlobalListClear ( GlobalList * const me ) {
# ifdef LISP2D_DEBUG  
  if  ( me -> s not_eq Lisp2DObjectStateConstructed ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DGlobalListClear:list is not live" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif    
  me -> first = 0 ;
  me -> count_private = 0 ; 
}

size_t  GlobalListconstrstatValue ( void  ) {
  return  GlobalListconstrstat  ; 
}
