// Lisp2D - GlobalListAtomic
// ©2019-23 by Alex Glebe

# include <stdatomic.h>
# include <iso646.h> // not
# include "globallista/private.h"

_Atomic ( size_t  ) GlobalListAtomicconstrstat = 0 ;

# include "globallista/public.h"

GlobalListAtomic  GlobalListOfStackClusters2 ;

void  GlobalListAtomicConstrStatic  ( GlobalListAtomic  * const me  ) {
  GlobalListAtomicConstr  ( me  ) ; 
}

size_t  GlobalListAtomicSizeStatic  ( GlobalListAtomic const * const me ) {
  return  GlobalListAtomicSize  ( me  ) ;
}
  
bool  GlobalListAtomicEmptyStatic ( GlobalListAtomic const * const me ) {
  return  GlobalListAtomicEmpty ( me  ) ;
}  

// may be nullptr
GlobalListMember * GlobalListAtomicFirstStatic ( GlobalListAtomic const * const me ) {
  return  GlobalListAtomicFirst ( me  ) ;
}

# include "excc.h"
# include "globallistm/struct.h"
# include "globallista/struct.h"

void  GlobalListAtomicPopNotLock  ( GlobalListAtomic  * const me , GlobalListMember * const xp ) {
  if  ( not me  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"GlobalListAtomicPopNotLock:not me" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  if  ( not xp  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"GlobalListAtomicPopNotLock:not xp" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  if ( atomic_load ( & me -> count ) == 0 ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"GlobalListAtomicPopNotLock:me -> count == 0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  --  ( me -> count ) ;
  GlobalListMember * oldfirst  = atomic_load  ( & me -> first ) ;
  if  ( xp == oldfirst ) {
    // it's was first
    atomic_store  ( & me -> first , xp -> next  ) ;
    oldfirst  = atomic_load ( & me -> first ) ;
    if  ( oldfirst  )
      oldfirst  ->  prev  = 0 ;
    // else it was one
  } else  {
    if ( not  ( xp -> prev ) ) {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"GlobalListAtomicPopNotLock:not  ( xp -> prev )" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
    }
    xp -> prev  ->  next  = xp -> next ;
    if  ( xp -> next  )
      xp -> next  ->  prev  = xp -> prev ;
    // else it was last
  }
}

void  GlobalListAtomicPushNotLock ( GlobalListAtomic  * const me  , GlobalListMember * const xp ) {
  ++ ( me -> count ) ;
  // x will be first
  xp -> prev  = 0 ;
  { GlobalListMember * const oldfirst = atomic_load ( & me -> first ) ;
    xp -> next  = oldfirst  ;
    if  ( oldfirst  )
      oldfirst  ->  prev  =   xp  ;
  }
  atomic_store  ( & me -> first , xp )  ;
}
  
void  GlobalListAtomicPushLockNoFreeze  ( GlobalListAtomic  * const me  , GlobalListMember  * const xp ) {
  xp -> prev  = 0 ;
  { GlobalListMember * oldfirst = atomic_load ( & me -> first ) ;
    do  {
      xp -> next  = oldfirst  ;
    } while ( not ( atomic_compare_exchange_weak ( & me -> first  , & oldfirst  , xp ) ) ) ;
    if  ( oldfirst  )
      oldfirst  ->  prev  =   xp  ;
  }
  ++ ( me -> count ) ;
}

size_t  GlobalListAtomicconstrstatValue ( void  ) {
  return  GlobalListAtomicconstrstat  ;
}

# include "define.h"

void  GlobalListAtomicConstr  ( GlobalListAtomic  * const me  ) {
  me  ->  first = 0 ;
  me  ->  count = 0 ;
# ifdef  LISP2D_CONSTRSTAT
  ++  GlobalListAtomicconstrstat  ;
# endif
}

size_t  GlobalListAtomicSize  ( GlobalListAtomic const * const me ) {
  return  me -> count ;
}

bool  GlobalListAtomicEmpty ( GlobalListAtomic const * const me ) {
  return me -> first  ==  0 ;
}

// may be nullptr
GlobalListMember * GlobalListAtomicFirst ( GlobalListAtomic const * const me ) {
  return me -> first ;
}
