// Net: Host Name Addr
// ©2012-23 by Alex Glebe
# include "host.h"
# include "cons/public.h"
# include <netdb.h> // sethostent
# include "boolean.h"
# include "nilc.h"
# include "stcl/public.h"
# include "stcl/inline.h"

// nil sethostent bool-stayopen
// returns: nil
TaskForThread bimnilsethostent ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject  const * const xp = CCAR ( StackClusterPrivate_ArgTop  ( stclpp ) ) ;
  sethostent  ( ( ( xp  not_eq  ( & plonil . plo ) ) and ( xp  not_eq  ( & plofalse . plo ) ) ) ) ;
  StackClusterPrivate_SetResult ( stclpp , & plonil . plo ) ;
  return  TaskForThread_Analyse ; 
}

// nil endhostent
// returns: nil
TaskForThread bimnilendhostent ( StackClusterPrivate  * const stclpp  ) {
  endhostent  ( ) ;
  StackClusterPrivate_SetResult ( stclpp , & plonil . plo ) ;
  return  TaskForThread_Analyse ; 
}

# include "string/public.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "cons/inline.h"
# include "plocomplex/public.h"
# include "cast.h"
# include "stack/public.h"

// nil gethostent
// returns: nil OR (string-officialname (list of string-names) type:
//   symbol(inet) or symbol(inet6) or integer ..
// .. 
TaskForThread bimnilgethostent ( StackClusterPrivate  * const stclpp  ) {
  struct hostent const  *  const e  = gethostent  ( ) ;
  if  ( not e ) {
    StackClusterPrivate_SetResult (stclpp ,& plonil . plo ) ;
    goto  Exit  ;
  }
  { PLOCons * r2  ;
    { PLOStringPub * newstr  ;
      { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
        String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ;
        String_Init ( WideChar  ) ( wanc , true ) ;
        { bool  bad ;
          { ThreadStackPointerType const oldstack_r = Lisp2DThreadSP ( ) ;
            String  ( Char ) * const strcresp = StackNew_StringChar ( ) ;
            StringConstrCStr ( Char  ) ( strcresp , e ->  h_name , true ) ;
            bad = isBADReadCStringToWString2 ( strcresp , wanc ) ; 
            Lisp2D_StackFree  ( oldstack_r ) ;
          }
          // first : destruct strcres then goto
          if  ( bad ) {
            Lisp2D_StackFree  ( oldstack ) ;
            goto  BadString ; 
          } 
        } // bad
        newstr  = PLOString_NewMoveString ( wanc  ) ;
        Lisp2D_StackFree  ( oldstack ) ;
      } // wanc
      if  ( not newstr  )
        goto  OutOfMemory ;
      r2  =  NewCons ( &newstr -> plo , &plonil . plo ) ; 
    } // newstr
    PLOCons * rlast = 0 ;
    // aliases
    { PLOCons * aliaseslist = 0 ;
      { PLOCons * aliaseslast = 0 ;
        for ( char * const * a = e ->  h_aliases ; * a ; ++  a ) {
          PLOCons * newlast2  ;
          { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
            String  ( WideChar ) * const wan2c = StackNew_StringWideChar ( ) ;
            String_Init ( WideChar  ) ( wan2c , true ) ;
            { bool  bad ;
              { ThreadStackPointerType const oldstack_r = Lisp2DThreadSP ( ) ;
                String  ( Char ) * const strcres2p = StackNew_StringChar ( ) ;
                StringConstrCStr ( Char  ) ( strcres2p , * a , true ) ;
                bad = isBADReadCStringToWString2 ( strcres2p , wan2c ) ; 
                Lisp2D_StackFree  ( oldstack_r ) ;
              }
              if  ( bad ) {
                Lisp2D_StackFree  ( oldstack ) ;
                goto  BadString ; 
              } 
            }
            PLOStringPub * const newstr = PLOString_NewMoveString ( wan2c ) ;
            Lisp2D_StackFree  ( oldstack ) ;
            if  ( newstr )
              newlast2  =  NewCons (& newstr -> plo , &plonil . plo ) ;
            else 
              goto  OutOfMemory ; 
          } // wan2c
          if  ( aliaseslast ) {
            ClassCons_SetRest(&aliaseslast->cc ,& newlast2 -> plo ) ;
            aliaseslast = newlast2  ; 
          }
          else
            aliaseslist = ( aliaseslast = newlast2  ) ; 
        } 
      } // aliaseslast
      rlast = (  NewCons ( ( aliaseslist ? &( aliaseslist -> plo ) :
        &( plonil . plo ) ) , &plonil . plo  ) ) ;
    } //  aliaseslist
    ClassCons_SetRest(&r2 ->cc ,& rlast -> plo ) ;
    // addrtype
    { PLOCons * const constype2 =  NewCons ( &plonil . plo , &plonil . plo ) ;
      { wchar_t const * wstr  ;
        { int const t = e ->  h_addrtype  ;
        switch  ( t ) {
        case  AF_INET :
          wstr  = L"inet" ;
          goto  WStr  ;
        case  AF_INET6  :
          wstr  = L"inet6"  ;
          goto  WStr  ;
        default : {
          PLOComplex  * const res = NewComplexBareExc  (   )  ;
          Complex_SetInt  ( PLOComplexToComplex( res) , t ) ;
          ClassCons_SetFirst(&constype2 ->  cc,& res -> plo ) ;
          goto  Next  ; 
          } 
        } // switch t
      } // t
WStr  :
        { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
          String  ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
          StringConstrCStr ( WideChar )  ( wsp , wstr , true ) ;
          ClassCons_SetFirst(&constype2 ->  cc, SymbolAddMoveString ( wsp ) ) ;
          Lisp2D_StackFree  ( oldstack ) ;
        } 
      } // wstr
Next  :
      ClassCons_SetRest(&rlast->cc ,& constype2 -> plo ) ;
      rlast = constype2 ; 
    } // constype2
    // addrlength
    { PLOCons * consaddrlength2 ;
      { PLOComplex  * const l = NewComplexBareExc  (   )  ;
        Complex_SetInt  ( PLOComplexToComplex( l) , e ->  h_length  ) ;
        consaddrlength2 =  NewCons ( &l -> plo , &plonil . plo ) ; 
      } // l
      ClassCons_SetRest(&rlast->cc ,& consaddrlength2 -> plo ) ;
      rlast = consaddrlength2 ; 
    } // consaddrlength2
    // addrslist
    { PLOCons * addrslist = 0 ;
      PLOCons * addrslast = 0 ;
      for ( char  * const * a = e ->  h_addr_list ; * a ; ++  a ) {
        PLOCons * result  = 0 ;
        { char  const * b = * a ;
          PLOCons * last  = 0 ;
          for ( int i = 0 ; i < ( e ->  h_length  ) ; ++  i , ++  b ) {
            PLOComplex  * const cb  = NewComplexBareExc  (   )  ;
            Complex_SetUInt  ( PLOComplexToComplex( cb) , char_cast_uchar(* b)
              ) ;
            PLOCons * const newitem2  =  NewCons ( &cb -> plo ,
              & plonil . plo )  ;
            if  ( last  ) {
              ClassCons_SetRest(&last->cc  ,& newitem2 -> plo ) ;
              last  = newitem2  ; 
            } else
              result  = ( last  = newitem2  ) ; 
          }
        } // last ib
        PLOCons * const newlast2  =
          NewCons ( ( result  ?
            &( result -> plo ) :
            &( plonil . plo ) ) ,
            &plonil . plo ) ;
        if  ( addrslast ) {
          ClassCons_SetRest(&addrslast->cc ,& newlast2 -> plo ) ;
          addrslast = newlast2  ; 
        } else
          addrslist = ( addrslast = newlast2  ) ; 
      } // for a
      PLOCons * const newrest =
        NewCons (
          ( addrslist ?
          ( &addrslist -> plo ) :
          & ( plonil .plo ) ) , 
          &plonil .plo ) ;
      ClassCons_SetRest(&rlast->cc ,& newrest -> plo ) ; 
    } // addrslist addrslast
    StackClusterPrivate_SetResult (stclpp ,& r2 -> plo ) ;
  }
Exit  :  
  return  TaskForThread_Analyse ;
BadString :
  BadStringN (  stclpp ) ;
  goto  Exit  ;
OutOfMemory :
  OutOfMemory ( stclpp ) ;
  goto  Exit  ; 
}

# include "castb.h"
# include "widestring/inline.h"
