// Lisp2D inner history
// ©2018-23 by Alex Glebe
# include "define.h"

# ifdef  LISP2D_INNERHISTORY

# include <wchar.h> // fputws
# include <iso646.h> // not_eq
# include <stdio.h> // stderr
# include <string.h> // memcpy
# include "inner-history/struct.h"
# include "inner-history/inline.h"
struct Lisp2DInnerHistory Lisp2DinnerHistory
# ifdef LISP2D_DEBUG
  = { . status  = Lisp2DInnerHistoryNotConstructed
  }
# endif
  ;

static  pthread_mutex_t Lisp2DInnerHistoryMutexClog = PTHREAD_MUTEX_INITIALIZER ;

//# warning remove jump buffer argument, use global JBlink

void  InnerHistoryConstructorDefault  ( struct Lisp2DInnerHistory * const mep  ) {
# ifdef LISP2D_DEBUG
  if (  mep ->  status == Lisp2DInnerHistoryLive ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryConstructorDefault:me . p ->  status == "
      L"Lisp2DInnerHistoryLive" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
# endif
  Lisp2D_atomic_mutex_init  ( & ( mep -> amutex ) ) ;
  mep ->  arrayqueue  = NULL  ;
  mep ->  arraysize = 0 ;
  mep ->  indexto = 0 ;
  mep ->  flagfull  = false ;
# ifdef LISP2D_DEBUG
  mep ->  status =  Lisp2DInnerHistoryLive  ;
# endif
}

# include <stdlib.h> // free

void  Lisp2DInnerHistoryDestructor  ( struct  Lisp2DInnerHistory  * const me  ) {
# ifdef LISP2D_DEBUG
  if  ( me  ->  status  not_eq  Lisp2DInnerHistoryLive  ) {
    fputws  ( L"Lisp2DInnerHistoryDestructor:me  ->  status  not_eq  Lisp2DInnerHistoryLive\n"  , stderr  ) ;
    return  ;
  }
# endif
  free  ( me  ->  arrayqueue  ) ;
# ifdef LISP2D_DEBUG
  Lisp2D_atomic_mutex_destruct  ( & ( me -> amutex ) ) ;
  me  ->  status  = Lisp2DInnerHistoryDestructed  ;
# endif
}

void  Lisp2DInnerHistoryResize ( struct Lisp2DInnerHistory * const  me  , size_t const s ) {
# ifdef LISP2D_DEBUG
  if  ( me  ->  status  not_eq  Lisp2DInnerHistoryLive  ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryResize:me  ->  status  not_eq  Lisp2DInnerHistoryLive" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  if  ( s ==  0 ) {
    free  ( me  ->  arrayqueue  ) ;
    me  ->  arrayqueue  = NULL  ;
    me  ->  arraysize  = 0 ;
    me  ->  indexto = 0 ;
    me  ->  flagfull  = false ;
    return  ;
  }
  struct Lisp2DInnerHistoryStep  (* const arrayQueueNew)[] = realloc ( me  ->  arrayqueue  ,
    s * sizeof  ( struct Lisp2DInnerHistoryStep ) ) ;
  if  ( arrayQueueNew ==  NULL  ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryResize:arrayQueueNew ==  NULL" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  me  ->  arrayqueue  = arrayQueueNew ;
  me  ->  arraysize  = s ;
  if  ( me  ->  indexto >= s ) {
    me  ->  indexto = 0 ;
    me  ->  flagfull  = true  ;
  }
}
  
# define  InnerHistoryShowSteps Lisp2DInnerHistoryShowStepscUgA
static  void  InnerHistoryShowSteps ( struct Lisp2DInnerHistoryStep const * i ,
  struct Lisp2DInnerHistoryStep const * const e ) {
  while ( i not_eq e  ) {
    fwprintf ( stdout  , L"%p : %ls\n"  , i  -> th_id , *(i  -> place) ) ;
    ++  i ;
  }
}
  
void  Lisp2DInnerHistoryShow  ( struct Lisp2DInnerHistory * const me  ) {
# ifdef LISP2D_DEBUG
  if (  me  ->  status not_eq Lisp2DInnerHistoryLive ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryShow:me  ->  status not_eq Lisp2DInnerHistoryLive" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  Lisp2D_atomic_mutex_lock  ( & ( me -> amutex ) ) ;
  struct Lisp2DInnerHistory localcopy
# ifdef LISP2D_DEBUG
    = { . status  = Lisp2DInnerHistoryNotConstructed
    }
# endif
    ;
  jmp_buf jb2 ;
  jmp_buf * const oldjmpbuf = Lisp2DJBglobal ;
  Lisp2DJBglobal = & jb2 ;
  int const volatile  ext = setjmp  ( jb2 ) ;
  if  ( ext ) {
    Lisp2DJBglobal = oldjmpbuf ;
    Lisp2D_atomic_mutex_unlock  ( & ( me -> amutex ) ) ;
    longjmp ( * Lisp2DJBglobal  , ext ) ;
  }
  Lisp2DInnerHistoryConstructorCopy  ( & localcopy , me ) ;
  Lisp2DJBglobal = oldjmpbuf ;
  Lisp2D_atomic_mutex_unlock  ( & ( me -> amutex ) ) ;
  if  ( pthread_mutex_lock  ( & Lisp2DInnerHistoryMutexClog ) ) {
    Lisp2DInnerHistoryDestructor  ( & localcopy ) ;
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryShow:pthread_mutex_lock  ( & Lisp2DInnerHistoryMutexClog )" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  if  ( fwprintf  ( stdout  , L"---Inner History---\n"  ) == -1 )
    goto Exit ;
  if (  localcopy .  arraysize ) {
    struct Lisp2DInnerHistoryStep const  * const indexold =
      ( (*localcopy . arrayqueue)  + ( localcopy . indexto ) ) ;
    if  ( localcopy . flagfull  ) {
      struct Lisp2DInnerHistoryStep const * const arrayend =
        ( (*localcopy  . arrayqueue)  + ( localcopy . arraysize ) ) ;
      InnerHistoryShowSteps ( indexold  , arrayend  ) ;
    }
    struct Lisp2DInnerHistoryStep const * arraybegin = *(localcopy  . arrayqueue ) ;
    InnerHistoryShowSteps ( arraybegin  , indexold  ) ;
  }
  fwprintf  ( stdout  , L"---\n"  ) ;
Exit :
  if  ( pthread_mutex_unlock  ( & Lisp2DInnerHistoryMutexClog ) ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryShow:pthread_mutex_unlock  ( & Lisp2DInnerHistoryMutexClog )" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  Lisp2DInnerHistoryDestructor  ( & localcopy ) ;
}

void  Lisp2DInnerHistoryPush  ( struct Lisp2DInnerHistory * const me , void  const  * const p ) {
# ifdef LISP2D_DEBUG
  if (  me  ->  status not_eq Lisp2DInnerHistoryLive ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryPush:me  ->  status not_eq Lisp2DInnerHistoryLive" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  Lisp2D_atomic_mutex_lock  ( & ( me -> amutex ) ) ;
  if ( me ->  arraysize ) {
    { struct Lisp2DInnerHistoryStep * const to = (*(me -> arrayqueue))  +
          ( me -> indexto ) ;
      to -> th_id =  pthread_self ( ) ;
      to -> place =  (wchar_t const ( * ) [ ])p ;
    }
    ++  ( me  ->  indexto ) ;
    if  ( me  ->  indexto ==  ( me ->  arraysize  ) ) {
      me  ->  indexto = 0 ;
      me  ->  flagfull  = true  ;
    }
  }
  Lisp2D_atomic_mutex_unlock  ( & ( me -> amutex ) ) ;
}

void Lisp2DInnerHistoryConstructorCopy  ( struct Lisp2DInnerHistory * const me , 
  struct Lisp2DInnerHistory const * const ihx ) {
# ifdef LISP2D_DEBUG
  if (  ihx ->  status not_eq Lisp2DInnerHistoryLive ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryConstructorCopy:ihx ->  status not_eq "
      L"Lisp2DInnerHistoryLive" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
# endif
  if  ( ihx ->  arraysize == 0 ) {
    InnerHistoryConstructorDefault ( me ) ;
    return ;
  }
# ifdef LISP2D_DEBUG
  if (  me  ->  status == Lisp2DInnerHistoryLive ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryConstructorCopy:"
      L"me  ->  status == Lisp2DInnerHistoryLive" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
# endif
  size_t const memsize = ihx ->  arraysize * sizeof  ( struct Lisp2DInnerHistoryStep ) ;
  struct Lisp2DInnerHistoryStep ( * const newarrayqueue ) [ ] = aligned_alloc ( _Alignof ( struct Lisp2DInnerHistoryStep ) , memsize ) ;
  if  ( not newarrayqueue ) {
    Lisp2DExcOutOfMemoryGlobal . value =  
      L"Lisp2DInnerHistoryConstructorCopy:newarrayqueue == NULL" ; 
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  memcpy  ( newarrayqueue , ihx -> arrayqueue , memsize ) ;
  Lisp2D_atomic_mutex_init  ( & ( me -> amutex ) ) ;
  me -> arrayqueue  = newarrayqueue ;
  me  ->  arraysize = ( ihx ->  arraysize ) ;
  me  ->  indexto = ( ihx -> indexto ) ;
  me  ->  flagfull  = ( ihx -> flagfull ) ;
# ifdef LISP2D_DEBUG
  me  ->  status =  Lisp2DInnerHistoryLive  ;
# endif
}

void  InnerHistoryConstructorSize ( struct Lisp2DInnerHistory * const mep ,
  enum  Lisp2DtypeSizeStartInnerHistory const  newsize ) {
# ifdef LISP2D_DEBUG
  if (  mep ->  status == Lisp2DInnerHistoryLive ) {
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) &
      L"Lisp2DInnerHistoryConstructorSize:"
      L"me . p ->  status == Lisp2DInnerHistoryLive" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
# endif
  Lisp2D_atomic_mutex_init  ( & ( mep -> amutex ) ) ;
  if  ( newsize ) {
    struct Lisp2DInnerHistoryStep ( * const newarrayqueue ) [ ] =
      aligned_alloc ( _Alignof ( struct Lisp2DInnerHistoryStep ) , newsize * sizeof  ( struct Lisp2DInnerHistoryStep ) ) ;
    if ( not newarrayqueue ) {
# ifdef LISP2D_DEBUG  
      Lisp2D_atomic_mutex_destruct  ( & ( mep -> amutex ) ) ;
# endif      
      Lisp2DExcOutOfMemoryGlobal . value = 
        L"Lisp2DInnerHistoryConstructorSize:" 
        L"not aligned_alloc ( newsize * sizeof  ( struct Lisp2DInnerHistoryStep ) )" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    } else
      mep ->  arrayqueue  = newarrayqueue ;
  } else    
    mep ->  arrayqueue  = NULL ;
  mep ->  arraysize = newsize ;
  mep ->  indexto = 0 ;
  mep ->  flagfull  = false ;
# ifdef LISP2D_DEBUG
  mep ->  status =  Lisp2DInnerHistoryLive  ;
# endif
}  
# else
_Bool LISP2D_INNERHISTORYsomething = 0 ;
# endif // LISP2D_INNERHISTORY
