// Lisp2D interpreter
// ©2003-23 by Alex Glebe
# include <signal.h>
# include "main.h"
# include "sharedc.h"
# include "define.h"
# include "unix.h"
# include "bimstatshow.h"
# include "lock/public.h"
# include "vector/public.h"
# include "thread.h"
# include "threadc.h"
# include "string/public.h"
# include "boolean.h"
# include "backquote/inline.h"
# include "bytedeque/vectorlength.h"
# include "comma/public.h"
# include "dir/inline.h"
# include "object/inline.h"
# include "quote/public.h"
# include "signall.h"
# include "unquotesplicing/public.h"
# include "socket/public.h"
# include "errorstra.h"
# include "virttabl.h"
# include "setbims.h"
# include "deque/public-wchar.h"
# include "startincl.h"
# include "manager.h"
# include "complex/inline.h"
# include "plocomplex/public.h"
# include "string/struct.h"
# include "cons/inline.h"
# include "char/public.h"
# include "listplo/public.h"
# include "cons/private.h"

# ifdef  LISP2D_CONSTRSTAT

# define  classconstrstat2  Lisp2D_classconstrstat2
# define  structclassconstrstat2  Lisp2D_structclassconstrstat2
typedef struct  structclassconstrstat2  classconstrstat2  ;

struct  structclassconstrstat2 {
  wchar_t const * classname  ;
  size_t n  ; 
} ;
  
# define  comparestat Lisp2D_comparestat
static  int comparestat  ( void const * const a , void const * const b ) {
  if (  ( ( ( classconstrstat2 const *  ) a ) ->  n ) ==
    ( ( ( classconstrstat2 const *  ) b ) ->  n ) )
    return  0 ;
  if (  ( ( ( classconstrstat2 const *  ) a ) ->  n ) <
    ( ( ( classconstrstat2 const *  ) b ) ->  n ) )
    return  -1  ;
  return  1 ; 
}

# include "consiter/public.h"
# include "env/public.h"
# include "slong/inline.h"
# include "time/public.h"
# include "globallista/public.h"

# define  StatConstrShow  Lisp2DStatConstrShowhbJz
static  void  StatConstrShow  ( void  ) {
  fputws  ( L"\n---Constr Stat---\n"  , stderr  ) ;
  classconstrstat2  allconstrstat [ ] = { {L"GlobalList",GlobalListconstrstatValue()},
    {L"Lisp2DGlobalListAtomic" , GlobalListAtomicconstrstatValue ( ) } , {L"PLONil",PLONilconstrstatValue()},
    { L"PLOStream"  , StreamConstrstatValue ( ) } ,    { L"PLOSymbol" , PLOSymbolConstrStatValue ( )  } ,
    { L"PLOEnvironmentLayer"  , PLOEnvironmentLayer_ConstrstatValue ( ) } ,    { L"PLOEnv" , PLOEnv_constrstatValue  ( ) } ,
    { L"StackCluster" , StackCluster_constrstat_Value ( ) } ,    { L"PLOLock"  , PLOLockConstrStatValue  ( ) } ,
    { L"ULong"  , Lisp2DULongconstrstatGetValue ( ) } ,    { L"SLong"  , Lisp2DSLongconstrstatGetValue ( ) } ,
    { L"Fraction2",FractionconstrstatGetValue ( )},    { L"PLOBIF" , NSBIF_constrstat_Value  ( ) } ,
    { L"PLOCons"  , PLOCons_constrstatValue ( ) } ,    { L"PLOVector"  , PLOVector_constrstatCValue  ( ) } ,
# ifndef  LISP2D_DONTUSEX11
    { L"PLOWindow"  , PLOWindow_constrstatValue ( ) } ,
# endif
    { L"RunningPthArgs" , RunningPthArgsconstrstatValue ( ) } ,    {L"RunningPthread",RunningPthreadStat()},
    { L"Complex"  , Complex_constrstat_Value (  ) } ,    { L"PLOString"  , PLOString_constrstatValue ( ) } ,
    {L"PLOBoolean",PLOBooleanconstrstatCValue()},    { L"PLOBackQuote" , PLOBackQuote_constrstat_Value ( ) } ,
    { L"PLOByteDeque" , PLOByteDeque_constrstat_value ( ) } ,    { L"PLOChar"  , PLOChar_constrstat_Value ( ) } ,
    {L"PLOComma",PLOComma_constrstat_Value()},    { L"ConsIterator" , ConsIteratorconstrstatValue ( ) } ,
    { L"PLODir" , PLODir_constrstatValue  ( ) } ,    { L"PLOObject"  , PLOObject_constrstat_Value  ( ) } ,
    { L"PLOQuote" , PLOQuote_constrstat_Value ( ) } ,    { L"PLOSignal"  , PLOSignal_constrstat_Value  ( ) } ,
    { L"PLOTime"  , PLOTime_constrstat_Val  ( ) } ,    {L"PLOUnquoteSplicing",UnQuoteSplicing_constrstat_Value()},
    { L"PLOSocket"  , PLOSocket_constrstatValue ( ) }
  };
  int const arraysize = sizeof  ( allconstrstat ) / sizeof  ( classconstrstat2  ) ;
  qsort ( allconstrstat , (size_t)arraysize , sizeof  ( classconstrstat2  ) , comparestat ) ;
  { int j = arraysize ;
    do {
      --  j ;
      if ( allconstrstat [ j ] . n )
        fwprintf  ( stderr  , L"%ls = %d\n" , allconstrstat [ j ] . classname , allconstrstat [ j ] . n ) ;
    } while ( j ) ;
  }
# ifdef LISP2D_DEBUG
  fputws  ( L"\n---alloc Stat---\n"  , stderr  ) ;
  fwprintf  ( stderr  , L"Lisp2D_statMallocPLOComplexPri = %zu\n" ,    Lisp2D_statMallocPLOComplexPri  ) ;
  fwprintf  ( stderr  , L"Lisp2D_statFreePLOComplexPri = %zu\n" ,    Lisp2D_statFreePLOComplexPri  ) ;
  fwprintf  ( stderr  , L"Lisp2D_statMallocPLOCons = %zu\n" ,    Lisp2D_statMallocPLOCons  ) ;
  fwprintf  ( stderr  , L"Lisp2D_statFreePLOCons = %zu\n" ,    Lisp2D_statFreePLOCons  ) ;
    
# endif // LISP2D_DEBUG
  
}  // StatConstrShow

# endif // LISP2D_CONSTRSTAT

# include "mutex2/public.h"

# define  SignalThreadToStop  Lisp2D_SignalThreadToStop
static  void  SignalThreadToStop  ( void ) {
  Mutex2Lock  ( & Lisp2Dmutex2ManagerTasks  ) ;
  flagThreadsShutDown = true;
  if  ( runpthargs  ) {
    RunningPthArgs  * j=runpthargs;
    for(unsigned  int i=maxCountThreads;i  > 0;++  j){
      --  i;
      if(j->flagWaitSignal){
        j->flagWaitSignal = false;
        if(pthread_cond_signal(&j -> condrunning2)){
          Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
          Lisp2DExcMutexGlobal . value = L"SignalThreadToStop:pthread_cond_signal" ;
          longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeMutex ) ; 
        }
      } 
    } 
  }
  Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
  
  { RunningPthread  * j = * RunningPthreadArray ( ) ;
    RunningPthArgs  * jargs=runpthargs;
    for(unsigned  int i=maxCountThreads;i  >  0;++ j , ++ jargs ){
      --  i;
      
      pthread_mutex_lock(&jargs ->ready_mutex);
      while ( not jargs ->  promise_state ) {
        pthread_cond_wait(&jargs ->ready_cond, &jargs ->ready_mutex); 
      }
      pthread_mutex_unlock(&jargs ->ready_mutex);
      if (jargs ->promise_exc)
        fputws  ( L"SignalThreadToStop\n" , stderr  ) ;
      
      RunningPthreadDestr ( j ) ;
      
      RunningPthArgsDestr ( jargs ) ; 
    }
  }
  
  free ( RunningPthreadArray ( ) ) ;
  RunningPthreadSetArray  ( 0 ) ;
  free (runpthargs);
# ifndef  LISP2D_DONTUSEX11
  Mutex2Lock  ( & Lisp2DmutexWindowsExit ) ;
  if  ( CountWindowsThreads2() > 0  ) {
        Lisp2DflagMainWaitWindows = true  ;
        jmp_buf jb2  ;
        jmp_buf * const jbold = Lisp2DJBglobal  ;
        Lisp2DJBglobal  = & jb2 ;
        int const volatile jbn = setjmp  ( jb2 ) ;
        if  ( jbn  ) {
          Lisp2DJBglobal  = jbold ;
          Mutex2Unlock  ( & Lisp2DmutexWindowsExit ) ;
          longjmp ( * Lisp2DJBglobal  , jbn ) ;   
        }
        ConditionWait ( & Lisp2DsignalWindowsExit , & Lisp2DmutexWindowsExit  ) ;
        Lisp2DJBglobal  = jbold ;
  }
  Mutex2Unlock  ( & Lisp2DmutexWindowsExit ) ;
# endif   
  Mutex2Lock ( & Lisp2Dmutex2ManagerTasks ) ;
  { size_t const  ce = countExtraThreads2Value ( ) ;
    while ( ce ) {
# ifdef  LISP2D_DEBUG    
      fwprintf  ( stderr  , L"main:waiting for ExtraThreads stops. countExtraThreads2 = %zu\n" , ce  ) ;
# endif    
      flagManagerWaitSignal = true ;
      jmp_buf jb2 ;
      jmp_buf * const oldjb = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2  ) ;
      if  ( jbn ) {
        Lisp2DJBglobal  = oldjb ;
        Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
        longjmp ( * Lisp2DJBglobal  , jbn ) ;
      }
      ConditionWait ( & Lisp2DsignalManager , & Lisp2Dmutex2ManagerTasks ) ;
      Lisp2DJBglobal  = oldjb ;
    } // while ce
  } // ce
  Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
}

# include "globallista/public.h"
# include "inner-history/inline.h"

# define  sigSegv Lisp2D_sigSegv
static  void  sigSegv ( int const s ) {
  if  ( Lisp2DflagTTY ) {
    switch  ( s ) {
    case  SIGHUP  :
    case  SIGINT  :
    case  SIGQUIT :
    case  SIGPIPE :
# ifdef  LISP2D_DEBUG
      switch  ( s ) {
        case SIGHUP :
          Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGHUP (Hangup)" } ) ;
          break ;
        case SIGINT :
          Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGINT (Terminal interrupt signal)" } ) ;
          break ;
        case SIGQUIT  :
          Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGQUIT (Terminal quit signal)"} ) ;
          break ;
        case SIGPIPE  :
          Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGPIPE (Write on a pipe with no one to read it)" } ) ;
          break ;
        default : ; 
      }
# endif
# ifdef  LISP2D_INNERHISTORY
      Lisp2DInnerHistoryShow  ( & Lisp2DinnerHistory  ) ;
# endif
# ifdef  LISP2D_BIMSTAT
      BIMStatShow ( ) ;
# endif
# ifdef  LISP2D_CONSTRSTAT
      StatConstrShow  ( ) ;
# endif
      fputwc  ( L'\n' , stderr  ) ;
      exit  ( 0 ) ;
// ? doesnt returns to thread      
      return  ;
    case  SIGTERM :
      Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGTERM (Termination signal)" }) ;
      break ;
    case  SIGBUS  : 
      Lisp2D_writelog  ( (Lisp2Dt_writelog) {
        (wchar_t const (*)[])&L"SIGBUS (Access to an undefined portion of a memory object)" }) ;
      break ;
    case  SIGFPE  :
      Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGFPE (Erroneous arithmetic operation)" } ) ;
      break ;
    case  SIGILL  : 
      Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGILL (Illegal instruction)" }) ;
      break ;
    case  SIGSEGV :  
      Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGSEGV (Invalid memory reference)" }) ;
      break ;
    case  SIGSYS  : 
      Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGSYS (Bad system call)" }) ;
      break ;
    case  SIGXCPU :  
      Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGXCPU (CPU time limit exceeded)" } ) ;
      break ;
    case  SIGXFSZ :  
      Lisp2D_writelog  ( (Lisp2Dt_writelog) { (wchar_t const (*)[])&L"SIGXFSZ (File size limit exceeded)" }) ;
      break ;
    default : ;
  }
# ifdef  LISP2D_INNERHISTORY
    Lisp2DInnerHistoryShow  ( & Lisp2DinnerHistory  ) ;
# endif
    // passport of all threads
    fwprintf  ( stderr  , L"\nmaxCountThreads = %u\n" , maxCountThreads ) ;
    fwprintf  ( stderr  , L"runpthcount = %d\n" , runpthcount_Value ( ) ) ;
    fwprintf  ( stderr  , L"flagManagerWaitSignal = %ls\n" ,
      ( flagManagerWaitSignal ? L"true" : L"false" ) ) ;
    fwprintf  ( stderr  , L"flagThreadsShutDown = %ls\n" ,
      ( flagThreadsShutDown ? L"true" : L"false"  ) ) ;
    fwprintf  ( stderr  , L"flagStopToGC = %ls\n" ,
      ( flagStopToGC ? L"true" : L"false"  ) ) ;
# ifdef  LISP2D_DEBUG      
    fwprintf  ( stderr  , L"debugGCRunning = %ls\n" ,
      ( debugGCRunning ? L"true" : L"false"  ) ) ;
# endif
    fwprintf  ( stderr  , L"GlobalListOfStackClusters.Size() = %zu\n" ,
      GlobalListAtomicSize ( & GlobalListOfStackClusters2  ) ) ;
    fwprintf  ( stderr  , L"countStackClusterWantToBeInGarbage = %zu\n" ,
      countStackClusterWantToBeInGarbage_Value  ( ) ) ;  
    fwprintf  ( stderr  , L"countExtraThreads = %zu\n" ,
      countExtraThreads2Value ( ) ) ;    
    fputws  ( L"mutexManagerTasks = " , stderr ) ;
    fflush  ( stderr ) ;
    if  ( not Mutex2IsBadTryLock  ( & Lisp2Dmutex2ManagerTasks ) ) {
      fputws  ( L"unlocked\n" , stderr ) ;
      Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
    } else   
      fputws  ( L"locked\n" , stderr ) ;
    { RunningPthArgs  * rpi = runpthargs  ;
      if  ( rpi ) {
      for ( unsigned  int i = maxCountThreads ; i  >  0 ; ++ rpi  ) {
        --  i;
        fwprintf(stderr,L"%u : ",i);
        fflush(stderr);
        if  ( ! ( rpi ->  taskp ) )
          fputws  ( L"nullptr\n"  , stderr  ) ;
        else
          fputws  ( L"ok\n"  , stderr  ) ;
          // ... analyse them          
        fwprintf  ( stderr  , L"flagWaitSignal = %ls\n" ,
          ( ( rpi ->  flagWaitSignal  ) ? L"true" : L"false"  ) ) ;
# ifdef  LISP2D_DEBUG        
        fwprintf  ( stderr  , L"debugStartEval = %ls\n" ,
          ( ( rpi ->  debugStartEval  ) ? L"true" : L"false"  ) ) ;  
# endif
      }  // for i
      }else       
        fputws(L"runpthargs=nullptr\n",stderr);
      
    } // rpi
          
          
# ifdef  LISP2D_BIMSTAT
      BIMStatShow ( ) ;
# endif
# ifdef  LISP2D_CONSTRSTAT
      StatConstrShow();
# endif        
  } //  if(Lisp2D::flagTTY)
    
  SignalThreadToStop();
  
  exit(EXIT_FAILURE);
// ? doesnt returns to thread
}

# ifndef  LISP2D_DONTUSEX11
# define  xlib_error_handler  Lisp2Dxlib_error_handlervpUR
static  int xlib_error_handler(Display  * _1 __attribute__ ((unused)) ,
  XErrorEvent * _2 __attribute__ ((unused)) ){
  ErrorStra((PArray_wchar_c){(wchar_t const (*)[])&L"xlib_error_handler\n"});
# ifdef  LISP2D_BIMSTAT
  BIMStatShow ( ) ;
# endif
# ifdef  LISP2D_CONSTRSTAT
  StatConstrShow();
# endif  

  return  1 ; 
}
  
# define  xlib_io_error_handler Lisp2Dxlib_io_error_handleryR0y
static  int xlib_io_error_handler(Display * _1 __attribute__ ((unused)) ){
  ErrorStra((PArray_wchar_c){(wchar_t const (*)[])&L"IO Error on display\n"});
  sigSegv(0);
  return  2 ; 
}
  
# endif // LISP2D_DONTUSEX11

# include "fraction/public.h"
# include "env/public.h"
# include "stcl/public.h"
# include "stack/public.h"
# include "plostream/public.h"

void  mainCPP  ( int const * const argc2 , char * * const argvp , int volatile * const result ) {
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t const  wsih_main1[]  = 
    L"void  main(Int  const & argc2,char  * * argv,"
    L"Int  & result):init-Manager()" ;
# endif
  maxTasksCreate  = maxCountThreads ;
  PassageConstructor ( passageGCCPub ) ;
  queueAtomicStClPrivFirst  = NewConsAtomicPointer ( ) ;
  queueAtomicStClPrivLast = queueAtomicStClPrivFirst  ;
  Lisp2DMutexConstructor  ( & Lisp2DMutexQueueAtomicStClFirstC ) ;
  Lisp2DMutexConstructor  ( & Lisp2DMutexQueueAtomicStClLastC ) ;
# ifdef  LISP2D_INNERHISTORY
  InnerHistoryConstructorSize  ( & Lisp2DinnerHistory , Lisp2DsizeStartInnerHistory ) ;
 // debug
# endif // innerhistory
  bool  flagerror=false;
  int  resmain=0;
  
# ifdef  LISP2D_BIMSTAT 
  typedef struct  Lisp2D_stStatString Lisp2D_StatString ;
  typedef Lisp2D_StatString StatString  ;
  struct  Lisp2D_stStatString  {
    struct  Lisp2DStatistics  * const stat  ;
    wchar_t const * const name  ;
  } ;

  StatString  arrstat [ ] = {
    { & GCstat  , L"GCstat" } ,
    { & GCstatSymbols , L"GCstatSymbols"  } ,
    { & statCreateLinks2  , L"statCreateLinks2"}  ,
    { & statULongSquare , L"statULongSquare"  } ,
    { & statComplexOperatorPlusEq , L"statComplexOperatorPlusEq"  } ,
    { & statComplexOperatorMulEq , L"statComplexOperatorMulEq"  } ,
    { & statComplexOperatorDivEq , L"statComplexOperatorDivEq"  } ,
    { & statComplexDelete , L"statComplexDelete"  } ,
    { & statULongDivWithOstBig  , L"statULongDivWithOstBig" } ,
    { & statULongDivWithOstBigFast  , L"statULongDivWithOstBigFast" } ,
    { & statULongDivWithOstBig2 , L"statULongDivWithOstBig2"  } ,
  } ;
# endif  
  {
// !! write  read-eval-print  is window mode (font is need)
  Lisp2DcheckTerminal ( ) ;

  if  ( ( * argc2 ) <=  1 )
    if  ( not Lisp2DflagTTY ) {
      (*result) = -1 ;
      return  ;
    }
  { sigset_t  set ;
    struct  sigaction sigactionnew;
    if  ( sigfillset  ( & set ) ) // all signals setting
      fputws ( L"main: sigfillset returns error\n" , stderr ) ;
    if(pthread_sigmask(SIG_SETMASK  , & set , 0 ) ) // change blocked signals
      fputws  ( L"main: pthread_sigmask(fill) returns error\n" , stderr ) ;
    memset(&  sigactionnew  , 0 , sizeof(sigactionnew));
    // fill memory by zeros
    if(sigfillset(& (sigactionnew.sa_mask))) // all signals setting
      fputws  ( L"main: sigfillset(sigactionnew.sa_mask) returns error\n" ,
        stderr ) ;
    sigactionnew.sa_handler = SIG_IGN; // ignore users signals
    
    // users signals

    typedef struct  Lisp2D_structSignalCodeName  Lisp2D_typeSignalCodeName  ;
typedef Lisp2D_typeSignalCodeName typeSignalCodeName  ;

struct  Lisp2D_structSignalCodeName  {
  int const code  ;
  wchar_t const * const name  ; 
} ;

    { typeSignalCodeName const arrPair [ ] = {
      { SIGHUP  , L"SIGHUP" } , { SIGINT  , L"SIGINT" } ,
      { SIGQUIT , L"SIGQUIT"  } ,
      { SIGPIPE , L"SIGPIPE"  } } ;

    unsigned int const s = sizeof (arrPair)/sizeof(arrPair [ 0 ]);
    typeSignalCodeName const * const en = & ( arrPair [ s ] ) ;
    typeSignalCodeName const * i = & ( arrPair [ 0 ] ) ;
    do {
      int const er = sigaction(i->code , & sigactionnew  , 0);
      if  ( er )
        fwprintf  ( stderr  , L"main: sigaction(%ls) returns error %d\n"  ,
          i ->  name  , er ) ;
      ++  i ;
    } while ( i < en ) ; 
    } // arrPair
    
    // error signals
    sigactionnew.sa_handler = sigSegv ;
    
    // ? to restart ?
    // sigactionnew.sa_flags = SA_RESTART; // continue execution
    
    { typeSignalCodeName const arrPair [ ] = {
      { SIGTERM , L"SIGTERM"  } , { SIGBUS  , L"SIGBUS" } ,
      { SIGFPE  , L"SIGFPE" } ,
      { SIGILL  , L"SIGILL" } , { SIGSEGV , L"SIGSEGV"  } ,
      { SIGSYS  , L"SIGSYS" } ,
      { SIGXCPU , L"SIGXCPU"  } , { SIGXFSZ , L"SIGXFSZ"  } } ;
      
    unsigned int const s = sizeof (arrPair)/sizeof(arrPair [ 0 ]);
    typeSignalCodeName const * const en = & ( arrPair [ s ] ) ;
    typeSignalCodeName const * i = & ( arrPair [ 0 ] ) ;
    do {
      int const er = sigaction(i->code , & sigactionnew  , 0);
      if  ( er )
        fwprintf  ( stderr  , L"main: sigaction(%ls) returns error %d\n"  ,
          i ->  name  , er ) ;
      ++  i ;
    } while ( i < en ) ; 
    } // arrPair
    
    if(sigemptyset(& set)) // all signals freeing
      fputws(L"main: sigemptyset returns error\n",stderr);
    if(pthread_sigmask(SIG_SETMASK  , & set , 0)) // release signals
      fputws  (L"main: pthread_sigmask(empty) returns error\n",stderr);
  } // set

# ifndef  LISP2D_DONTUSEX11
  if(XInitThreads())
    ;
  else  {
    fputws  ( L"main:XInitThreads() returns bad\n"  , stderr  ) ;
    ( * result ) = -2 ;
    return  ;
  }
# endif
  
# ifndef  LISP2D_DONTUSEX11
  XSetErrorHandler(xlib_error_handler);
  XSetIOErrorHandler(xlib_io_error_handler);
# endif
# ifdef  LISP2D_BIMSTAT
  unsigned int const as = sizeof (arrstat)/sizeof(arrstat [ 0 ]);
    StatString const * const en = & ( arrstat [ as ] ) ;
    {StatString const * i = & ( arrstat [ 0 ] ) ;
    do {
      Lisp2DStatisticsConstructor  ( i  -> stat ) ;
      if(Lisp2DStatisticsIsBADInit( i->stat,2)){
        fwprintf(stderr,L"main:%ls.isBADInit\n",i->name);
        flagerror = true;
        goto  Exit;
      }
      ++  i ;
    } while ( i < en ) ;
  } // i
       // ss liststat
# endif      

  MakeVirtualTables ( ) ;

  // makes new BIF's (needs virtual PushNewObject)
  if(not  Lisp2DisOKSetBIMs()){
    fputws  ( L"main:not  isOKSetBIMs()\n"  , stderr  ) ;
    (*result) = -3 ;
    return;
  }
    
// ! How , where to destruct PLOBIF ?    
  
  flagThreadsShutDown = false;
  
  ParaLispObject  * objfromst;
  
// ? create runpth with     operator new(sizeof(...*..
//fputws(L"mainCPP:3,5.",stdout);fflush(stdout);  
  // creating pthreads
  runpthargs = aligned_alloc ( _Alignof ( RunningPthArgs ) , sizeof ( RunningPthArgs ) * maxCountThreads ) ;
  if  ( not runpthargs )  {
    fputws(L"main:out of memory for runpthargs\n",stderr);
    flagerror = true;
    goto  Exit2;
  }
  { RunningPthread  * const a = aligned_alloc ( _Alignof ( RunningPthread ) , sizeof ( RunningPthread ) * maxCountThreads ) ;
    if ( not a ) {
      free(runpthargs);
      fputws(L"main:out of memory for runpthread\n",stderr);
      flagerror = true;
      goto  Exit2;
    }
    RunningPthreadSetArray  ( a ) ;
  }
  { RunningPthArgs  * j=runpthargs;
    for ( unsigned  int i = maxCountThreads ; i  >  0 ; ++ j  ) {
      --  i;
// ! to make argument (bool & returnerror)      
      RunningPthArgsInit  ( j ) ; 
    } 
  }
         
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t const  wsih_main3[] = 
    L"void  main(Int  const & argc2,char  * * argv,Int  & result):"
    L"BEGIN:Create threads" ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_main3  ) ;
# endif
  { RunningPthread  * j = * RunningPthreadArray ( ) ;
    RunningPthArgs  * ja=runpthargs;
    for ( unsigned  int i = maxCountThreads ; i  >  0 ; ++ j  , ++  ja  ) {
      --  i;
// ! to make argument (..,bool & returnerror)            
      RunningPthreadInit ( j , pthEval , ja ) ; 
    }
  }

# ifdef  LISP2D_INNERHISTORY
  static  wchar_t const  wsih_main4 [ ] = 
    L"void  main(Int  const & argc2,char  * * argv,Int  & result):END:"
    L"Threads created" ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_main4  ) ;
# endif
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    Deque ( WChar ) * const dqstl = Lisp2D_StackVar ( StackType ( Deque_WChar  ) ) ;
    Deque_Init ( WChar ) ( dqstl ) ;
    StartingInclude ( dqstl ) ;
// ! try : to debug defmethods    
    
  objfromst = (&  plonil.plo);
  { StackClusterPrivate  * const  topstcl2p =
       GlobalListMemberToStackClusterPrivate (
         GlobalListAtomicFirst ( & Lisp2DGlobalListOfStackClusters2 ) ) ;
    if(topstcl2p){
      if(not (StackClusterPrivate_LoadObjectS(topstcl2p,&objfromst,0, dqstl ))){
        ErrorStra((PArray_wchar_c){(wchar_t const (*)[])&
          L"main:not(topstcl2p->LoadObjectS(objfromst,0,dqstl))"});
        ( * result  ) = -4 ;
        Lisp2D_StackFree  ( oldstack ) ;
        return; 
      }

      allowgc = true;
        // replace to DebugWithoutGC !! memory will be very small
      
        StackClusterPrivate_Push(topstcl2p ,  objfromst);
        { struct Lisp2DAddrForReturn2  a ;
          t_AddrForReturn2ConstructorFlagInBIM  st  = {
            . me  = & a , . addr = Lisp2DEvalAddrContTopProcessFinish2  , . inbim = false } ;
          AddrForReturn2ConstructorFlagInBIM  ( & st ) ;
          StackClusterAddrPush  ( topstcl2p , a ) ;
        }
        SetResultPlace  ( topstcl2p  ,&  globalresult2 , globalresult2 ) ;
        TaskQueuePush ( topstcl2p ) ;
      } else {
        ErrorStra((PArray_wchar_c){(wchar_t const (*)[])& L"main: GlobalListOfStackClusters.Front()=nullptr"});
        ( * result  ) = 1 ;
        Lisp2D_StackFree  ( oldstack ) ;
        return;
      }
    }  // topstcl
    Lisp2D_StackFree  ( oldstack ) ;
  } // dqst
# ifdef  LISP2D_INNERHISTORY
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_main1  ) ;
# endif  
  Manager ( ) ;
  { StackClusterPrivate  * const topstclp  = NewStClOrphan  ( LISP2DdefaultHistorySize  ) ;  // have no father
    GlobalListAtomicPushNotLock ( & Lisp2DGlobalListOfStackClusters2 ,  StackClusterPrivate_globallistmember  ( topstclp ) ) ;
    { NameSpaceDef  *  const nsstd = NameSpaceAssoc( & plosstd ) ;
# ifdef  LISP2D_DEBUG
      if  ( ! nsstd ) {
        ErrorStra((PArray_wchar_c){(wchar_t const (*)[])& L"STD namespace is not defined (2)"});
        (*result) = -14 ;
        return;
      }
# endif
      ParaLispObject  * const stdvarp=  nsstd->var;
      ParaLispObject  * const stdpropp= nsstd->prop;
      // using std & protect it
      { ParaLispObject  * list [ ] = { &  NewEnvironmentLayer  ( ) -> plo , stdvarp , NULL } ;
        ParaLispObject  * list2 [ ] = { &  plonil.plo, stdpropp, NULL } ;
        StackClusterPrivate_SetEnv3(topstclp , ListOfPLO ( ( PArrayPLOP ) { ( ParaLispObject  * ( * ) [ ] )
          & list  } ) , ListOfPLO ( ( PArrayPLOP ) { ( ParaLispObject  * ( * ) [ ] ) & list2 } ) ) ;
        StackClusterPrivate_SetExtEnv ( topstclp , StackClusterPrivate_Env ( topstclp ) ) ;
      }
    }  // nsstd

  // to get plospi
  { ParaLispObject * const  varA2  = Assoc ( & plospi . plo , StackClusterPrivate_Env ( topstclp ) , topstclp ) ;
    if(varA2) {
      if(not  IsNotNumber( topstclp, varA2)) {
        FractionConstrCopy ( FractionploPiPub ,
          Complex_CRe ( PLOComplexToComplexConst  (
            ParaLispObjectToPLOComplexConst ( varA2 ) ) ) )  ;
        FractionSetFraction ( Fractionplo2PiPub ,
          FractionploConstPiPub ) ;
        FractionMul2  ( Fractionplo2PiPub ) ;
        FractionSetFraction ( FractionploHalfPiPub ,
          FractionploConstPiPub ) ;
        FractionDiv2  (FractionploHalfPiPub);
      }
    } else  
      fputws(L"assoc(pi)not",stderr ) ;
  }
  { ParaLispObject * const varA2  = Assoc ( & plose . plo , StackClusterPrivate_Env ( topstclp ) , topstclp ) ;
    // to get plose
    if(varA2)  {
      if(not  IsNotNumber( topstclp, varA2)){
        FractionConstrCopy ( FractionploEPub ,
          Complex_CRe ( PLOComplexToComplexConst  (
            ParaLispObjectToPLOComplexConst ( varA2 ) ) ) )  ;
      }
    } else  
      fputws  ( L"assoc(e)not\n"  , stderr  ) ;
  }
// ! to always make list arg :

// lisp2d               -> nil 
// lisp2d prog.lisp2d   -> prog.lisp2d
// lisp2d prog.lisp2d 1 -> prog.lisp2d 1

// and always start read-eval-print

  { ParaLispObject * argl = & plonil . plo ;
    PLOCons * arglast = 0 ;
    for(int i = 1 ; i < ( * argc2 ) ; ++  i ) {
      PLOCons * arglast3 ;
      { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
        String  ( WideChar ) * const wstrtmpc = StackNew_StringWideChar ( ) ;
        String_Init ( WideChar  ) ( wstrtmpc , true ) ;
        { String  ( Char ) * const strtmpc = StackNew_StringChar ( ) ;
          StringConstrCStr ( Char  ) ( strtmpc , argvp  [ i ] , true ) ;
          if ( isBADReadCStringToWString2 ( strtmpc , wstrtmpc ) ) {
            Lisp2D_StackFree  ( oldstack ) ;
            fputws  ( L"main:isBADReadStringToWString2\n"  , stderr  ) ;
            (*result) = -6 ;
            return ;
          }
        }
        PLOStringPub * const  newstrp = PLOString_NewMoveString ( wstrtmpc ) ;
        Lisp2D_StackFree  ( oldstack ) ;
        arglast3  = NewCons ( & newstrp  -> plo , & plonil  . plo ) ;
      }
      if  ( arglast )
        ClassCons_SetRest(&arglast->cc,&  arglast3->plo);
      else 
        argl  = &arglast3->plo;
      arglast = arglast3;
    }
    { ParaLispObject  * const envcarp = CCAR ( StackClusterPrivate_Env ( topstclp ) ) ;
      if(IsNotEnvironmentLayer(  topstclp ,envcarp)){
        (*result) = -9 ;
        return;
      }
      if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
            envcarp  ) -> envlay , & ploarg  . plo , argl  , true  ) ) {
        fputws  ( L"main:envcar.isBADPush(ploarg)\n" , stderr  ) ;
        (*result) = -13 ;
        return;
      }
    }  // envcar
    { PLOCons * const main0 = NewCons ( & plosmain  . plo , argl  ) ;
      PLOCons * const main1 = NewCons ( & plonil  . plo , & main0 ->  plo ) ;
      StackClusterPrivate_Push  ( topstclp ,  & main1 ->  plo ) ;
    }
  }  // argl
    { struct Lisp2DAddrForReturn2 a ;
      Lisp2DAddrForReturn2ConstructorDefault(&a);
    
      a.inBIM = false;
      EvalAddrSetCont ( & a . addr . pub , Lisp2DEvalAddrContTopProcessFinish2 )  ;
    
      StackClusterAddrPush  ( topstclp , a ) ;
    }
    SetResultPlace  (  topstclp , &  globalresult2 , globalresult2 ) ;    
    TaskQueuePush ( topstclp ) ;
  }
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t const  wsih_main2[]  = 
    L"void  main(Int  const & argc2,char  * * argv,Int  & result):"
    L"second-Manager()" ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_main2  ) ;
# endif
 
# ifdef LISP2D_DEBUG
  //testFractionInit  ( ) ;
# endif
  
  Manager ( ) ;
  
  // signal please for ManagerX
  if(Numberp( globalresult2)) {
    
    if(FractionIsBADIntValue(Complex_CRe(
      PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(&
          *  globalresult2))),&resmain))
      resmain = -1 ; 
  } else
    if(Charp( globalresult2))     
      resmain = (int)(
        PLOChar_Value(ParaLispObjectToPLOChar(globalresult2)));
  // remove threads
   SignalThreadToStop(); 
// ! to make function   
// ! wait for windows thread
  // to do:
  // number of win-threads (array of threads) & send signal
// ! exits and doesnt wait system-thread to stop
# ifdef  LISP2D_BIMSTAT   
Exit  : ;
# endif
# ifdef  LISP2D_INNERHISTORY
  
  Lisp2DInnerHistoryShow  ( & Lisp2DinnerHistory  ) ;
# endif

# ifdef  LISP2D_BIMSTAT
  BIMStatShow ( ) ;
# endif
# ifdef  LISP2D_CONSTRSTAT
  StatConstrShow();
# endif
  }  
  // release signals
  { sigset_t  set;
    
    // !? keyboard will be dead
    // ?! what to do
    if(sigemptyset(& set)) // all signals freeing
      fputws(L"main: sigemptyset returns error\n",stderr);
    if(pthread_sigmask(SIG_SETMASK  , & set , 0))
      // change blocked signals
      fputws (L"main:exit: pthread_sigmask returns error\n",stderr);
  } // set
  if(flagerror)  
    (*result) = -13 ;
  else  
    (*result) = resmain ;
  
  { int rt  = repareTerminal ( ) ;
    if  ( rt  and ( ( * result  ) ==  0 ) )
      ( * result  ) = rt  ;
  }
  
  if(flagdeletetzname02){
    //delete  []  tzname[0];
    free(tzname[0]);
    tzname[0] = 0;
  }
  if(flagdeletetzname12){
    //delete  []  tzname[1];
    free(tzname[1]);
    tzname[1] = 0;
  }
    
  jmp_buf jb3 ;  
    
# ifdef  LISP2D_INNERHISTORY
  { jmp_buf * oldjb = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jb3  ;
    int const volatile exct = setjmp  ( jb3  ) ;
    if  ( exct  ) {
      if  ( exct == Lisp2DExcTypeString )
        fwprintf  ( stderr  ,
          L"Lisp2DmainCPP:Lisp2DInnerHistoryDestructor ( & Lisp2DinnerHistory ):%ls\n" ,
          &((*Lisp2DExceptionStringGlobal . str)[0]) ) ;
      else 
        fwprintf  ( stderr  ,
          L"Lisp2DmainCPP:Lisp2DInnerHistoryDestructor ( & Lisp2DinnerHistory"
          L"):exception №%i\n" ,
          exct ) ; 
    } else 
      Lisp2DInnerHistoryDestructor ( & Lisp2DinnerHistory ) ;
    Lisp2DJBglobal  = oldjb  ; 
  }
# endif
    
# ifdef LISP2D_BIMSTAT  
  { unsigned int const as = sizeof (arrstat)/sizeof(arrstat [ 0 ]);
    StatString const * const en = & ( arrstat [ as ] ) ;
    { StatString const * i = & ( arrstat [ 0 ] ) ;
      do {
        Lisp2DStatisticsDestructor  ( i -> stat ) ;
        ++  i ;
      }while ( i < en ) ;
    } // i
  }
# endif
  
  { jmp_buf * oldjb = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jb3  ;
    int const volatile exct = setjmp  ( jb3  ) ;
    if  ( exct ) {
      if  ( exct == Lisp2DExcTypeString )
        fwprintf  ( stderr  ,
          L"Lisp2DmainCPP:Lisp2DPassageDestructor ( & passageGCC ) :%ls\n" ,
          &((*Lisp2DExceptionStringGlobal . str)[0]) ) ;
      else
        fwprintf  ( stderr  ,
          L"Lisp2DmainCPP:Lisp2DPassageDestructor ( & passageGCC ) :"
          L"exception №%i\n" , exct ) ;
    } else
      PassageDestructor ( passageGCCPub ) ;
    int const volatile exct2 = setjmp  ( jb3  ) ;
    if  ( exct2 ) {
      if  ( exct2 == Lisp2DExcTypeString )
        fwprintf  ( stderr  ,
          L"Lisp2DmainCPP:Lisp2DMutexDestructor  ( "
          L"& Lisp2DMutexQueueAtomicStClFirstC ) :%ls\n" ,
          &((*Lisp2DExceptionStringGlobal . str)[0]) ) ;
      else
        fwprintf  ( stderr  , 
          L"Lisp2DmainCPP:Lisp2DMutexDestructor  ( "
          L"& Lisp2DMutexQueueAtomicStClFirstC ) :exception №%i\n"
          , exct2 ) ;
    } else 
      Lisp2DMutexDestructor  ( & Lisp2DMutexQueueAtomicStClFirstC ) ;
    int const volatile exct3 = setjmp  ( jb3  ) ;
    if  ( exct3 ) {
      if  ( exct3 == Lisp2DExcTypeString )
        fwprintf  ( stderr  ,
          L"Lisp2DmainCPP:Lisp2DMutexDestructor  ( "
          L"& Lisp2DMutexQueueAtomicStClLastC ) :%ls\n" ,
          &((*Lisp2DExceptionStringGlobal . str)[0]) ) ;
      else
        fwprintf  ( stderr  ,
          L"Lisp2DmainCPP:Lisp2DMutexDestructor  ( "
          L"& Lisp2DMutexQueueAtomicStClLastC ) :exception №%i\n" ,
          exct3 ) ;
    } else 
      Lisp2DMutexDestructor  ( & Lisp2DMutexQueueAtomicStClLastC ) ;
    
    Lisp2DJBglobal  = oldjb  ;
  }
Exit2 :  
# ifdef  LISP2D_DEBUG
  fwprintf  ( stderr  , L"main : result2 = %d\n"  , * result ) ;
# endif
  Lisp2D_ListDestr  ( PLO ) ( list_global_atomsc  ) ;
  MapPubDestr ( errnotosymbol ) ( seterrnotosymbol_pub ) ;
  MapPubDestr ( setTypeSymbolNameType ) ( setTypeSymbolNameType_pub ) ;
  MapPubDestr ( FlessMkdirModes ) ( symboltomkdirmodes_pub  ) ;
  MapPubDestr ( FlessOpenModes  ) ( symboltoopenmodes_pub ) ;

  { size_t  const symbols = MapPubSize  ( treeSymbolsLessType ) ( treeSymbols_pub ) ;
    //fwprintf  ( stderr  , L"symbols = %zu "  , symbols ) ; fflush ( stderr ) ;
    for ( size_t  j = 0 ; j < symbols ; ++  j ) {
      PLOSymbol * const s = MapPubCAt ( treeSymbolsLessType ) ( treeSymbols_pub , j ) -> key ;
      //fwprintf  ( stderr  , L"s = %p "  , s ) ; fflush ( stderr ) ;
      bool  const wasStatic = ParaLispObject_IsStatic ( & s -> plo ) ;
      PLOSymbolDestr  ( s ) ;
      if ( not wasStatic )
        free  ( s ) ;
    }
    //fwprintf  ( stderr  , L"finas "  , symbols ) ; fflush ( stderr ) ;
  }
  MapPubClear ( treeSymbolsLessType ) ( treeSymbols_pub ) ;
  //fwprintf  ( stderr  , L"clear " ) ; fflush ( stderr ) ;
  MapPubDestr ( treeSymbolsLessType ) ( treeSymbols_pub ) ;
  //fwprintf  ( stderr  , L"destr " ) ; fflush ( stderr ) ;

// ! to free ( ) all BIM's

  GlobalListDestructor ( & GlobalListOfStackClustersGarbage2 ) ;
  GlobalListDestructor ( & GlobalListOfComplex ) ;
  GlobalListDestructor ( & GlobalListOfComplexGarbage ) ;
  GlobalListDestructor ( & GlobalListOfComplexGC ) ;
  GlobalListDestructor ( & GlobalListOfLayer ) ;
  GlobalListDestructor ( & GlobalListOfLayerGarbage ) ;
  GlobalListDestructor ( & GlobalListOfLayerGC ) ;  
# ifndef  LISP2D_DONTUSEX11    
  GlobalListDestructor ( & GlobalListOfWindows ) ;
  GlobalListDestructor ( & GlobalListOfWindowsGarbage ) ;
# endif  
  //  PLOCons *
  { PLOCons * objectsi  = PLOCons_FromMember ( GlobalListFirst ( & GlobalListOfConsPairs  ) ) ;
    while ( objectsi ) {
      PLOCons * const next = PLOCons_FromMember ( GlobalListMemberNext ( & objectsi  -> plo  . base  ) ) ;
      PLOCons_Destr ( objectsi  ) ;
      free  ( objectsi ) ;
      //fwprintf  ( stderr  , L"p = %p " , objectsi ) ; fflush ( stderr ) ;
      objectsi  = next  ;
    }
  }
  GlobalListDestructor ( & GlobalListOfConsPairs ) ;
  GlobalListDestructor ( & GlobalListOfConsPairsGarbage ) ;
  GlobalListDestructor ( & GlobalListOfConsPairsGC ) ;
  GlobalListDestructor  ( & GlobalListOfSymbolsGarbage  ) ;
  DeleteSetCharNameValue  ( ) ;
} // Lisp2DmainCPP
# include "widestring/inline.h"
# include "socket/inline.h"
# include "stcl/inline.h"
# include "shared/inline.h"
