// Lisp2D - Mutex auto init/destroy with statistics
// ©2012-23 by Alex Glebe
# include <stdio.h> // stderr
# include <iso646.h> // not_eq
# include <pthread.h>
# include <errno.h> // EBUSY
# include "define.h"
# include "mutex.h"
# include "excc.h"

void  Lisp2DMutexLock ( Lisp2DMutex  * const  me )  {
# ifdef LISP2D_DEBUG
  if (  me  ->  state not_eq Lisp2DStructLive ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"MutexLock:me  ->  state not_eq Lisp2DStructLive" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
# ifdef LISP2D_BIMSTAT
  int err = pthread_mutex_trylock ( & ( me  ->  m ) ) ;
  switch ( err )  {
  case 0 :
    ++  ( me  -> statfast  ) ;
    break ;
  case EBUSY :
    err = pthread_mutex_lock ( & ( me  ->  m ) ) ;
    switch  ( err ) {
    case  0 :
      break ;
    case  EAGAIN  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The mutex could not "
        L"be acquired because the maximum number of recursive locks for mutex "
        L"has been exceeded." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  EINVAL  :
      Lisp2DExcMutexGlobal . value =
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The mutex was "
        L"created with the protocol attribute having the value "
        L"PTHREAD_PRIO_PROTECT and the calling thread's priority is higher "
        L"than the mutex's current priority ceiling." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  ENOTRECOVERABLE  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The state protected "
        L"by the mutex is not recoverable." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  EOWNERDEAD  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The  mutex is a "
        L"robust mutex and the process containing the previous owning thread "
        L"terminated while holding the mutex lock. The mutex lock shall be "
        L"acquired by the calling thread and it is up to the new owner to make "
        L"the state  consistent." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  EDEADLK  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The mutex type is "
        L"PTHREAD_MUTEX_ERRORCHECK and the current thread already owns the "
        L"mutex." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    default :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) )" ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    }
    ++  ( me  -> statwait ) ;
    break ;
  case  EAGAIN  :
    Lisp2DExcMutexGlobal . value = 
      L"MutexLock:pthread_mutex_trylock ( & ( me  ->  m ) ):The mutex could "
      L"not  be acquired because the maximum number of recursive locks for "
      L"mutex has been exceeded." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  case  EINVAL  :
    Lisp2DExcMutexGlobal . value = 
    L"MutexLock:pthread_mutex_trylock ( & ( me ->  m ) ):The mutex was "
    L"created with the protocol attribute having the value "
    L"PTHREAD_PRIO_PROTECT and the calling thread's priority is higher than "
    L"the mutex's current priority ceiling." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  case  ENOTRECOVERABLE  :
    Lisp2DExcMutexGlobal . value = 
      L"MutexLock:pthread_mutex_trylock ( & ( me  ->  m ) ):The state "
      L"protected by the mutex is not recoverable." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  case  EOWNERDEAD  :
    Lisp2DExcMutexGlobal . value = 
      L"MutexLock:pthread_mutex_trylock ( & ( me  ->  m ) ):The  mutex is a "
      L"robust mutex and the process containing the previous owning thread "
      L"terminated while holding the mutex lock. The mutex lock shall be "
      L"acquired by the calling thread and it is up to the new owner to make "
      L"the state  consistent." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  default : 
    Lisp2DExcMutexGlobal . value = 
      L"MutexLock:pthread_mutex_trylock ( & ( me  ->  m ) ):?" ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ; 
  }
  ++  ( me  -> statwait ) ;
# else // ! LISP2D_BIMSTAT
  { int err = pthread_mutex_lock ( & ( me  ->  m ) ) ;
    switch  ( err ) {
    case  0 :
      break ;
    case  EAGAIN  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The mutex could not "
        L"be acquired because the maximum number of recursive locks for mutex "
        L"has been exceeded." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  EINVAL  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The mutex was "
        L"created with the protocol attribute having the value "
        L"PTHREAD_PRIO_PROTECT and the calling thread's priority is higher "
        L"than the mutex's current priority ceiling." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  ENOTRECOVERABLE  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The state protected "
        L"by the mutex is not recoverable." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  EOWNERDEAD  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The  mutex is a "
        L"robust mutex and the process containing the previous owning thread "
        L"terminated while holding the mutex lock. The mutex lock shall be "
        L"acquired by the calling thread and it is up to the new owner to make "
        L"the state  consistent." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    case  EDEADLK  :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):The mutex type is "
        L"PTHREAD_MUTEX_ERRORCHECK and the current thread already owns the "
        L"mutex." ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    default :
      Lisp2DExcMutexGlobal . value = 
        L"MutexLock:pthread_mutex_lock ( & ( me  ->  m ) ):?" ;
      longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
    }
  }
# endif // LISP2D_BIMSTAT
}

void  Lisp2DMutexUnLock ( Lisp2DMutex  * const  me ) {
# ifdef LISP2D_DEBUG
  if (  me  ->  state not_eq Lisp2DStructLive ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"MutexUnLock:me  ->  state not_eq Lisp2DStructLive" ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif  
  int const err = pthread_mutex_unlock ( & ( me  ->  m ) ) ;
  switch  ( err ) {
  case  0 :
    break ;
  case  EPERM :
    Lisp2DExcMutexGlobal . value =
      L"Lisp2DMutexUnLock:pthread_mutex_unlock ( & ( me  ->  m ) ):The mutex type is PTHREAD_MUTEX_ERRORCHECK or"
      L"PTHREAD_MUTEX_RECURSIVE, or the mutex is a robust mutex, and the current thread does not own the mutex." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  default :
    Lisp2DExcMutexGlobal . value =
      L"Lisp2DMutexUnLock:pthread_mutex_unlock ( & ( me  ->  m ) ):?" ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  }
}

void  Lisp2DMutexConstructor ( Lisp2DMutex  * const me ) {
# ifdef LISP2D_DEBUG
  me  ->  state = Lisp2DStructNotConstructed  ;
# endif
# ifdef LISP2D_DEBUG
  if (  me  ->  state == Lisp2DStructLive ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DMutexConstructor:me  ->  state == Lisp2DStructLive" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  int const err = pthread_mutex_init ( & ( me  ->  m ) , 0 ) ;
  switch  ( err ) {
  case  0 :
    break ;
  case  EAGAIN  :
    Lisp2DExcMutexGlobal . value =
      L"Lisp2DMutexConstructor:pthread_mutex_init ( & ( me  ->  m ) , 0 ):The system lacked the"
      L"necessary resources (other than memory) to initialize another mutex." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  case  ENOMEM  :
    Lisp2DExcMutexGlobal . value =
      L"Lisp2DMutexConstructor:pthread_mutex_init ( & ( me  ->  m ) , 0 ):"
      L"Insufficient memory exists to initialize the mutex." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  case  EPERM :
    Lisp2DExcMutexGlobal . value =
      L"Lisp2DMutexConstructor:pthread_mutex_init ( & ( me  ->  m ) , 0 ):"
      L"The caller does not have the privilege to perform the operation." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  case  EINVAL  :
    Lisp2DExcMutexGlobal . value =
      L"Lisp2DMutexConstructor:pthread_mutex_init ( & ( me  ->  m ) , 0 ):The attributes object referenced by"
      L"attr has the robust mutex attribute set without the process-shared attribute being set." ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  default :
    Lisp2DExcMutexGlobal . value =
      L"Lisp2DMutexConstructor:pthread_mutex_init ( & ( me  ->  m ) , 0 ):?" ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  }
# ifdef  LISP2D_BIMSTAT
  me  ->  statfast = 0 ;
  me  ->  statwait = 0 ;
# endif
# ifdef LISP2D_DEBUG
  me  ->  state = Lisp2DStructLive ;
# endif
}

void  Lisp2DMutexDestructor  ( Lisp2DMutex  * const me ) {
# ifdef LISP2D_DEBUG
  if  ( me  ->  state  not_eq  Lisp2DStructLive  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DMutexDestructor:me  ->  state  not_eq  Lisp2DStructLive" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif  
  if  ( pthread_mutex_destroy ( & ( me  ->  m ) ) ) {
    Lisp2DExcMutexGlobal . value =
      L"pthread_mutex_destroy ( & ( me  ->  m ) ):?" ;
    longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeMutex ) ;
  }
# ifdef LISP2D_DEBUG
  me  ->  state = Lisp2DStructDestructed ;
# endif
}

pthread_mutex_t * Lisp2DMutexAddr ( Lisp2DMutex  * const  me ) {
# ifdef LISP2D_DEBUG
  if  ( me  ->  state  not_eq  Lisp2DStructLive  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DMutexAddr:me  ->  state  not_eq  Lisp2DStructLive" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif  
  return  & ( me  ->  m ) ;
}
