// Lisp2D - symbol newobject
// ©2006-23 by Alex Glebe

# include "signall.h"
# include "newobject.h"
# include "boolean.h"
# include "inerrors.h"
# include "object/inline.h"
# include "dir/inline.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "cons/public.h"
# include "consiter/public.h"
# include "consiter/struct.h"
# include "env/public.h"
# include "stcl/public.h"
# include "stcl/inline.h"
# include "stack/public.h"

# define  NewObjectSetFields  Lisp2DNewObjectSetFields2103
static  void   NewObjectSetFields  ( StackClusterPrivate * const stclpp , ParaLispObject * const classdefp  ,
  PLOObject * const dopobjp  ) {
  { ThreadStackPointerType const oldstack_j = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const j = NewStackCConsIterator ( ) ;
    for ( CConsIteratorInit ( j , ClassGetParents (  classdefp  ) ) ;
      CConsIteratorHaveItem  ( j ) ;
      CConsIteratorNext  ( j ) )
      NewObjectSetFields  ( stclpp , CCDR ( GetPropPair ( Environment_NamespacePropList (
            StackClusterPrivate_envLocal ( stclpp ) ) ,
           CConsIteratorItem  ( j ) , & plossystemclass . plo  , stclpp ) ) , dopobjp ) ;
    Lisp2D_StackFree  ( oldstack_j ) ;
  }
  ParaLispObject  * const layerp = ClassGetVars  (  classdefp  ) ;
  if  ( not IsNotEnvironmentLayer  (  stclpp ,  layerp ) ) {
    ParaLispObject  * const envlayerp  = PLOObject_staticContext  ( & dopobjp ->  plo ) ;
    if  ( not  IsNotEnvironmentLayer  (  stclpp , envlayerp  ) )
      EnvironmentLayerPushAll ( & ParaLispObjectToPLOEnvironmentLayer ( envlayerp  ) -> envlay ,
         & ParaLispObjectToPLOEnvironmentLayer ( layerp ) -> envlay ) ; 
  }
}

# include "quote/public.h"
# include "string/public.h"
# include "backquote/inline.h"
# include "comma/public.h"
# include "bytedeque/public.h"
# include "bytedeque/struct.h"
# include "char/public.h"
# include "cons/inline.h"
# include "vector/struct.h"
# include "vector/public.h"
# include "lock/public.h"
# include "lock/struct.h"
# include "plocomplex/public.h"
# include "time/public.h"
# include "stat/public.h"
# include "unquotesplicing/struct.h"
# include "unquotesplicing/public.h"
# include "stat/structpri.h"
# include "socket/public.h"
# include "socket/structpri.h"

// symbol newobject [{args}]
TaskForThread bimsymbolnewobject  ( StackClusterPrivate * const stclpp  ) {
  ParaLispObject  * varA  ;
  PLOSymbol * me2 ;
  
// ! to catch exception ?
  
  PLOCons * const localenvp = NewCons  ( &  plonil  . plo , & plonil  . plo ) ;

  // ? Assoc - need ParaLispObject
  ParaLispObject  * linkconstr ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) 
    goto  Next  ;
  me2 =   ParaLispObjectToPLOSymbol(StackClusterPrivate_ThisTop(stclpp ));
  { ParaLispObject  * const  varC = CCDR ( GetPropPair ( StackClusterPrivate_NamespaceProplist ( stclpp ) ,
         & me2->plo, & plossystemclass  . plo , stclpp ));
    if(varC ==  &  plonil.plo){
      ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"class ")->plo,
        &me2->plo,& PLOString_NewCopy(L" is not defined")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      goto Exit ;
    }
    { ParaLispObject  *   varB = CCDR  ( CCDR  ( CCDR  ( CCDR  ( varC))));
      if(CConsp(  varB)) {
        // its BinClass
        varB = ClassCons_First ( &ParaLispObjectToPLOCons ( varB  )->cc ) ;
        if(IsNotSymbol(  stclpp ,  varB))  
          goto Exit ;
        PLOSymbol * const varBs = ParaLispObjectToPLOSymbol ( varB  ) ;
        if  ( varBs ==  & ploscons  ) {
          PLOCons  * const co = aligned_alloc ( _Alignof ( PLOCons ) , sizeof ( PLOCons ) ) ;
          if (  not co  )
            goto  MemOut  ;
          PLOCons_Init(co);
          varA = & co->plo;
          ParaLispObject_virtualPushAsNew(varA);
          goto  Constr;
        }
        if  ( varBs == & plosstream ) {
          PLOStream  * const plos = aligned_alloc ( _Alignof ( PLOStream ) , sizeof ( PLOStream ) ) ;
          if ( not plos )
            goto  MemOut  ;
          PLOStreamInit(plos);
          varA = &(plos->plo);
          ParaLispObject_virtualPushAsNew(varA);
          goto  Constr;
        }
        if  ( varBs == & plosstring ) {
          PLOStringPub  * const stp = PLOString_New ( ) ;
          varA = & stp -> plo ;
          goto  Constr  ;
        }
        if  ( varBs == & plosvector ) {
          PLOVector  * const tmpv = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
          if(not tmpv )
            goto  MemOut  ;
          PLOVector_Init(tmpv);
          varA = & tmpv->plo;
          PLOVector_staticPushAsNew(varA);
          goto  Constr;
        }
        if  ( varBs == & ploscomplex  ) {
          varA = &NewComplexBareExc()->plo;
          Complex_Set0  ( PLOComplexToComplex( ParaLispObjectToPLOComplex(varA)));    
          goto  Constr;
        }
        if  ( varBs == & ploschar ) {
          PLOCharPriv  * const ca = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
          if ( not ca )
            goto  MemOut  ;
          PLOChar_Init  ( ca  ) ;
          varA  = & ca -> pub . plo ;
          ParaLispObject_virtualPushAsNew ( varA  ) ;
           // temporary space
          goto  Constr  ;
        }
        if ( varBs == & plossymbol ) {
// ! to write new PLOSymbol (local); constructor with argument char,string,symbol;
// push newSymbol in kernel
          ParaLispObject  * list [ ] = { &  PLOString_NewCopy(
              L"constructor for symbols is not defined")->plo, NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          goto Exit ;
        }
# ifndef  LISP2D_DONTUSEX11
        if  ( varBs ==  & ploswindow  ) {
          PLOWindow_pri  * const w = aligned_alloc ( _Alignof ( PLOWindow_pri ) , sizeof ( PLOWindow_pri ) ) ;
          if  ( w ) {
            PLOWindow_Init  ( w ) ;
            varA  = & w -> plo ;
            PLOWindow_staticPushAsNew ( varA  ) ;
          } else
            goto  MemOut  ;
          goto  Constr;
        }
# endif
        if  ( varBs == & plostime  ) {
          PLOTime  * const t = aligned_alloc ( _Alignof ( PLOTime ) , sizeof ( PLOTime ) ) ;
          if  ( not t )
            goto  MemOut  ;
          PLOTime_Init  ( t ) ;
          varA  = & t -> pub . plo ;
          ParaLispObject_virtualPushAsNew ( varA ) ;
          goto  Constr  ;
        }
        if  ( varBs  == & plosdir){
          PLODirPri  * const d = aligned_alloc ( _Alignof ( PLODirPri ) , sizeof ( PLODirPri ) ) ;
          if ( not d )
            goto  MemOut  ;
          PLODir_Init ( d ) ;
          varA  = & d ->  pub . plo ;
          ParaLispObject_virtualPushAsNew(varA);
          goto  Constr;
        }
        if  ( varBs == & ploslock ) {
          PLOLock * const lo = PLOLock_malloc ( ) ;
          if(lo) {
            PLOLock_Init(lo);
            varA  = & lo -> plo ;
            ParaLispObject_virtualPushAsNew(varA);
          } else
            goto  MemOut  ;
          goto  Constr;
        }
        if  ( varBs ==  & plossignal  ) {
          PLOSignal  * const s = aligned_alloc ( _Alignof ( PLOSignal ) , sizeof ( PLOSignal ) ) ;
          if (  not s )
            goto  MemOut  ;
          PLOSignal_Init  ( s ) ;
          varA  = & s -> plo ;
          ParaLispObject_virtualPushAsNew ( varA ) ;
          goto  Constr ;
        }
        if  ( varBs  == & plosenv)  {
          PLOEnvC  * const e = aligned_alloc ( _Alignof ( PLOEnvC ) , sizeof ( PLOEnvC ) ) ;
          if  ( not e )
            goto  MemOut  ;
          PLOEnv_Init ( e ) ;
          varA = & e -> plo ;
          ParaLispObject_virtualPushAsNew(varA);
          Environment_SetPair ( & ParaLispObjectToPLOEnv ( varA  ) -> env ,
            StackClusterPrivate_Env( stclpp ) , StackClusterPrivate_NamespaceProplist ( stclpp )  ) ;
          goto  Constr;
        }
        if  ( varBs == & plosquote2  ) {
          PLOQuote  * const q = aligned_alloc ( _Alignof ( PLOQuote ) , sizeof ( PLOQuote ) ) ;
          if  ( q ) {
            PLOQuote_Init  ( q ) ;
            varA = & q -> plo ;
            ParaLispObject_virtualPushAsNew ( varA  ) ;
          } else
            goto  MemOut  ;
          goto  Constr  ;
        }
        if  ( varBs == & plosbackquote ) {
          PLOBackQuote  * const bq = aligned_alloc ( _Alignof ( PLOBackQuote ) , sizeof ( PLOBackQuote ) ) ;
          if  ( not bq  )
            goto  MemOut  ;
          PLOBackQuote_Init  ( bq ) ;
          varA = & bq -> plo ;
          ParaLispObject_virtualPushAsNew ( varA  ) ;
          goto  Constr  ;
        }
        if  ( varBs == & ploscomma  ) {
          PLOComma  * const co = aligned_alloc ( _Alignof ( PLOComma ) , sizeof ( PLOComma ) ) ;
          if ( not co )
            goto  MemOut  ;
          PLOComma_Init  ( co ) ;
          varA = & co -> plo ;
          ParaLispObject_virtualPushAsNew ( varA  ) ;
          goto  Constr  ;
        }
        if  ( varBs == & plosUnquoteSplicing  ) {
          PLOUnQuoteSplicing * const uq = aligned_alloc ( _Alignof ( PLOUnQuoteSplicing ) , sizeof ( PLOUnQuoteSplicing ) ) ;
          if  ( uq  ) {
            PLOUnQuoteSplicing_Init ( uq  ) ;
            varA = & uq->plo;
            ParaLispObject_virtualPushAsNew(varA);
          } else {
            varA = &plonil.plo;
            goto  MemOut  ;
          }
          goto  Constr;
        }
        if  ( varBs == & plosbytedeque  ) {
          PLOByteDeque * const bdq = aligned_alloc ( _Alignof ( PLOByteDeque ) , sizeof ( PLOByteDeque ) ) ;
          if (  not bdq )
            goto  MemOut  ;
          PLOByteDeque_Init ( bdq ) ;
          varA = & bdq -> plo ;
          ParaLispObject_virtualPushAsNew ( varA  ) ;
          goto  Constr  ;
        }
        if  ( varBs ==  & plossocket  ) {
          PLOSocketPri * const s = aligned_alloc ( _Alignof ( PLOSocketPri ) , sizeof ( PLOSocketPri ) ) ;
          if  ( not s )
            goto  MemOut  ;
          PLOSocket_Init  ( & s ->  pub ) ;
          varA  = & s ->  pub . plo ;
          ParaLispObject_virtualPushAsNew ( varA  ) ;
          goto  Constr  ;
        }
        if  ( varBs  == & plosstat ) {
          PLOStatPri * const s = aligned_alloc ( _Alignof ( PLOStatPri ) , sizeof ( PLOStatPri ) ) ;
          if ( not s )
            goto  MemOut  ;
          PLOStat_Init ( & s -> pub ) ;
          varA = & s -> pub . plo ;
          ParaLispObject_virtualPushAsNew (varA ) ;
          goto  Constr  ;
        }
          
        ParaLispObject  * list [ ] = { &  PLOString_NewCopy( L"class ")->plo,
          varB,&  PLOString_NewCopy(L" is not defined")->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        goto Exit ;
      }  // if Consp(varB)
    } // varB
    { PLOObject * const ob = aligned_alloc ( _Alignof ( PLOObject ) , sizeof ( PLOObject ) ) ;
      if  ( ob )  {
        PLOObject_Init  ( ob ) ;
        varA  = & ob -> plo ;
        ParaLispObject_virtualPushAsNew ( varA  ) ;
      } else
        goto  MemOut  ;
      PLOObject_SetClassName  ( ob , & me2 ->  plo ) ;
      PLOObject_SetNewContext ( ob , NewEnvironmentLayer ( ) ) ;
      PLOObject_SetClassDef ( ob , varC  ) ;
      // recursion
      NewObjectSetFields(stclpp , varC,  ob);
    } // ob
    { ParaLispObject  * contp = ParaLispObject_virtualContext(varA);
      if(IsNotEnvironmentLayer(  stclpp ,contp))  
        goto Exit ;
      if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
            contp ) -> envlay , & plosthis  . plo , varA  , true  ) ) {
        HasConstLink(  stclpp , & plosthis  . plo ) ;
        goto Exit ;
      }
    }
// ! to write constructor for all build-in types
Constr  :
    // constructor using
    ClassCons_SetFirst  ( & localenvp ->  cc  , ClassGetMethods ( varC  ) ) ;
  }
  linkconstr  = Assoc ( & me2 ->  plo , & localenvp ->  plo , stclpp )  ;
  if  ( linkconstr  ) {
    StackClusterPrivate_Push(stclpp , varA);
    // (symbol newobject a b c) -> ('obj symbol a b c)
    { PLOCons  *  task2 = NewCons(  linkconstr,StackClusterPrivate_ArgTop( stclpp ));
      { PLOCons * task;
        PLOQuote  * const varQ = aligned_alloc ( _Alignof ( PLOQuote ) , sizeof ( PLOQuote ) ) ;
        if ( varQ ) {
          PLOQuote_Init(varQ);
          PLOQuote_SetItem ( varQ , varA ) ;
          ParaLispObject_virtualPushAsNew(&varQ->plo);
          varA  = &varQ->plo;
        } else
          goto PopMemOut ;
        task  =  NewCons( varA, &task2->plo);
        StackClusterPrivate_Push(stclpp ,& task->plo);
      }
    }
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    goto Exit ;
Next  :
    varA = StackClusterPrivate_PopValue(stclpp );
  }
  StackClusterPrivate_SetResult(stclpp ,  varA);
  goto Exit ;
MemOut  :
  OutOfMemory (  stclpp ) ;
  goto Exit ;
PopMemOut :
  varA = StackClusterPrivate_PopValue ( stclpp ) ;
  goto  MemOut  ;
Exit :
  return  TaskForThread_Analyse ; 
}

# include "define.h"
# include "symbolc.h"

void  UniversalIsNotTypess  ( tUniversalIsNotType * const arg ) {
  if  (  ParaLispObject_Type  ( arg ->  xp ) ==  arg ->  typetype  )
    goto  OK  ;
  { static  bool  flagarrOk = false ;
    static  PLOSymbol  * arrtypesymb [ TypeGlobalListMemberSIZE ] = { 0 } ;
    if  ( not flagarrOk ) {
      arrtypesymb [ TypeGlobalListMemberBase  ] = & plosnil ;
      arrtypesymb [ TypeGlobalListMemberStClPriv  ] = & plosnil ;
      arrtypesymb [ TypeGlobalListMemberParaLispObject  ] = & plosnil ;
      arrtypesymb [ TypeGlobalListMemberPLONil  ] = & plosnil ;
      arrtypesymb [ TypeGlobalListMemberPLOSymbol ] = & plossymbol  ;
      arrtypesymb [ TypeGlobalListMemberPLOCons ] = & ploscons  ;
      arrtypesymb [ TypeGlobalListMemberPLOString ] = & plosstring  ;
      arrtypesymb [ TypeGlobalListMemberBIFPrivate  ] = & plosbif ;
      arrtypesymb [ TypeGlobalListMemberPLOVector ] = & plosvector  ;
      arrtypesymb [ TypeGlobalListMemberPLOChar ] = & ploschar  ;
      arrtypesymb [ TypeGlobalListMemberPLOStream ] = & plosstream  ;
      arrtypesymb [ TypeGlobalListMemberPLOLock ] = & ploslock  ;
      arrtypesymb [ TypeGlobalListMemberPLOObject ] = & plossystemclass ;
# ifndef LISP2D_DONTUSEX11
      arrtypesymb [ TypeGlobalListMemberPLOWindowPri ] = & ploswindow  ;
# endif
      arrtypesymb [ TypeGlobalListMemberPLOTime ] = & plostime  ;
      arrtypesymb [ TypeGlobalListMemberPLODirPri  ] = & plosdir ;
      arrtypesymb [ TypeGlobalListMemberPLOSignal ] = & plossignal  ;
      arrtypesymb [ TypeGlobalListMemberPLOEnv  ] = & plosenv ;
      arrtypesymb [ TypeGlobalListMemberPLOEnvironmentLayer ] = & plosenvironmentlayer  ;
      arrtypesymb [ TypeGlobalListMemberPLOQuote  ] = & plosquote2  ;
      arrtypesymb [ TypeGlobalListMemberBackQuotePrivate  ] = & plosbackquote ;
      arrtypesymb [ TypeGlobalListMemberPLOComma  ] = & ploscomma ;
      arrtypesymb [ TypeGlobalListMemberPLOUnquoteSplicing  ] = & plosUnquoteSplicing ;
      arrtypesymb [ TypeGlobalListMemberPLOComplex  ] = & ploscomplex ;
      arrtypesymb [ TypeGlobalListMemberPLOBoolean  ] = & plosboolean ;
      arrtypesymb [ TypeGlobalListMemberPLOByteDeque  ] = & plosbytedeque ;
      arrtypesymb [ TypeGlobalListMemberPLOSocket ] = & plossocket  ;
      arrtypesymb [ TypeGlobalListMemberPLOStatPri  ] = & plosstat  ;
# ifdef LISP2D_DEBUG
      { int i = TypeGlobalListMemberSIZE  ;
        do {
          --  i ;
          if  ( not ( arrtypesymb [ i ] ) )
            fwprintf  ( stderr  , L"UniversalIsNotType:No symbol for type %d\n" , i ) ;
        } while ( i ) ;
      }
# endif
      flagarrOk = true  ;
    }
    if  ( arg ->  typetype  < 0 or  arg ->  typetype  >=  TypeGlobalListMemberSIZE  ) {
      fwprintf  ( stderr  , L"UniversalIsNotType:Unknown typetype %d\n" , arg ->  typetype ) ;
      goto  BAD ;
    }
    PLOSymbol * const symboltypename  = arrtypesymb [ arg -> typetype  ] ;
    { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
      String  ( WideChar ) * const wstr = StackNew_StringWideChar ( ) ;
      StringConstrCStr ( WideChar ) ( wstr , L" is not " , true ) ;
      StringAddCharArray ( WideChar ) ( wstr ,
        StringCStr  ( WideChar  ) ( PLOSymbolCValue ( symboltypename  ) ) ) ;
      if  ( arg -> stclp ) {
// ! how to move string memory ?
        PLOStringPub * const e = PLOString_NewCopy ( StringCStr  ( WideChar  ) ( wstr  ) ) ;
        ParaLispObject  * list [ ] = { arg -> xp , & e -> plo , 0 } ;
        StackClusterPrivate_ExceptionSet ( arg -> stclp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      } else
        inErrorS  ( PLOStreamPushPLO ( & Lisp2Dferr , arg -> xp ) ;
          PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
            ( PArray_wchar_c ) { (wchar_t const(*)[]) (
              StringCStr  ( WideChar  ) ( wstr  ))});)
      Lisp2D_StackFree  ( oldstack ) ;
    } // wstr
  } // symboltypename
BAD :
  arg ->  res = true  ;
  return  ;
OK  :
  arg ->  res = false ;
}
# include "widestring/inline.h"
# include "stat/inline.h"
# include "socket/inline.h"
