// Lisp2D - NIL type
// ©2006-23 by Alex Glebe

# include "nilc.h"

PLONil  plonil  ;

_Atomic ( size_t  ) PLONilconstrstat = 0 ;  

# include "define.h"

# ifdef LISP2D_CONSTRSTAT
void  AddPLONilconstrstat ( void  ) {
  ++  PLONilconstrstat  ;
}
  
size_t  PLONilconstrstatValue ( void  ) {
  return  PLONilconstrstat  ;
}
# endif

# include "nil/public.h"

void PLONilDestr  ( ParaLispObject * const me ) {
  PLONilDestrStatic ( ParaLispObjectToPLONil  ( me  ) ) ;
}

void PLONilDestrStatic  ( PLONil * const me ) {
  ParaLispObject_staticDestr(&me->plo);
}

ParaLispObject  *   ploclassdefnil = & plonil.plo ;
  
ParaLispObject  * PLONilClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassdefnil ;
}

ParaLispObject  * PLONilClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return &  plonil.plo ;
}

ParaLispObject  * PLONilTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  & plonil.plo  ;
}
  
# include "stcl/classc.h"
# include "cons/public.h"
# include "boolean.h"
# include "stcl/public.h"
# include "stcl/inline.h"
  
//  nil eq [y]
TaskForThread bimnileq(StackClusterPrivate * const stclpp){
  ParaLispObject  * const ap = StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const xp = CCAR ( ap);
  StackClusterPrivate_SetResult(stclpp ,(
    (  xp  ==  CCAR ( CCDR (  ap))) ? ((( xp  ==  &  plofalse.plo) or
    (  xp  ==  &  plonil.plo)) ?
      &plotrue.plo : xp)  : &plofalse.plo));
  return  TaskForThread_Analyse;
}

//  nil type-of
TaskForThread bimniltypeof ( StackClusterPrivate * const stclpp  ) {
  StackClusterPrivate_SetResult (stclpp , ParaLispObject_virtualTypeOf  ( 
      CCAR ( StackClusterPrivate_ArgTop(stclpp )) ) ) ;
  return  TaskForThread_Analyse ;
}

void  PLONilCoutToHuman ( ParaLispObject const * const me , PLOStream  const * const s , bool flag2pprinted ,
  Vector(StackPLOP)  * const h ) {
  PLONilCoutToInterpret ( me ,    s , flag2pprinted  , h ) ;
}
  
# include "plostream/public.h"

void  PLONilCoutToInterpret ( ParaLispObject const * _1 __attribute__ ((unused)) ,
  PLOStream  const * const s , bool _2 __attribute__ ((unused)) ,
  Vector(StackPLOP)  * _3 __attribute__ ((unused)) ) {
  PLOStreamPushPLO(s ,&(  plosnil . plo ));
}

# include <unistd.h>
# include "plocomplex/public.h"

//  nil geteuid
TaskForThread bimnilgeteuid(StackClusterPrivate  * const stclpp){
  unsigned  int const ur=geteuid();
  PLOComplex  * const res=NewComplexBareExc(  );
  Complex_SetUInt ( PLOComplexToComplex( res),ur);
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  return  TaskForThread_Analyse ;
}
  
//  nil getegid
TaskForThread bimnilgetegid(StackClusterPrivate  * const stclpp){
  unsigned  int const ur=getegid();
  PLOComplex  * const res=NewComplexBareExc(  );
  Complex_SetUInt ( PLOComplexToComplex( res),ur);
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  return  TaskForThread_Analyse ;
}

//  nil getuid
TaskForThread bimnilgetuid(StackClusterPrivate * const stclpp){
  unsigned  int const ur=getuid();
  PLOComplex  * const res=NewComplexBareExc(  );
  Complex_SetUInt ( PLOComplexToComplex( res),ur);
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  return  TaskForThread_Analyse;
}

//  nil getgid
TaskForThread bimnilgetgid(StackClusterPrivate * const stclpp){
  unsigned  int const ur=getgid();
  PLOComplex  * const res=NewComplexBareExc(  );
  Complex_SetUInt ( PLOComplexToComplex( res),ur);
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  return  TaskForThread_Analyse;
}

# include <sys/sysinfo.h>
  
//  nil uptime
TaskForThread bimniluptime(StackClusterPrivate * const stclpp){
  struct  sysinfo si;
  if(sysinfo(&  si)){
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  PLOComplex  * const res=NewComplexBareExc(  );
  Complex_SetLInt ( PLOComplexToComplex( res) , si  . uptime  ) ;
  StackClusterPrivate_SetResult(stclpp ,&   res->plo);
  return  TaskForThread_Analyse;
}

//  nil totalram
TaskForThread bimniltotalram(StackClusterPrivate * const stclpp){
  struct  sysinfo si;
  if(sysinfo(&  si)){
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  PLOComplex  * const pret=NewComplexBareExc( );
  Complex_SetULInt  ( PLOComplexToComplex( pret),si.totalram);
  StackClusterPrivate_SetResult(stclpp ,&  pret->plo);
  return  TaskForThread_Analyse;
}

//  nil freeram
TaskForThread bimnilfreeram(StackClusterPrivate  * const stclpp){
  struct  sysinfo si;
  if(sysinfo(&  si)){
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  PLOComplex  * const pret=NewComplexBareExc( );
  Complex_SetULInt  ( PLOComplexToComplex( pret),si.freeram);
  StackClusterPrivate_SetResult(stclpp ,&  pret->plo);
  return  TaskForThread_Analyse;
}
  
# include "sharedc.h"
# include "shared/inline.h"
# include "cons/inline.h"

// nil,false,true =
TaskForThread bimnilequal(StackClusterPrivate  * const stclpp){
  ParaLispObject   *  j;
  ParaLispObject  const * const  mep=StackClusterPrivate_ThisTop(stclpp );
  unsigned  int tasks = maxListItemProcessing ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) {
    j = StackClusterPrivate_PopValue(stclpp );
    goto  Next;
  }
  j = StackClusterPrivate_ArgTop(stclpp );
  while(CConsp( j)){
Next:;
    if ( ClassCons_First(&ParaLispObjectToPLOCons(j)->cc) not_eq   mep ) {
      StackClusterPrivate_SetResult(stclpp ,&plofalse.plo);
      return  TaskForThread_Analyse;
    }
    j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j ) ->cc) ;
    if((--  tasks)  ==  0){
      if(CConsp(  j)){
        StackClusterPrivate_setTaskForManager ( stclpp , 
          TaskPause ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        StackClusterPrivate_Push(stclpp , j);
        return  TaskForThread_Analyse;
      }
      break;
    }
  }
  StackClusterPrivate_SetResult(stclpp ,&plotrue.plo);
  return  TaskForThread_Analyse;
}

// nil,false,true <>
TaskForThread bimnilnotequal(StackClusterPrivate * const stclpp){
  ParaLispObject  *  j;
  ParaLispObject  const * const  mep=StackClusterPrivate_ThisTop(stclpp );
  unsigned  int tasks = maxListItemProcessing ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) {
    j=StackClusterPrivate_PopValue(stclpp );
    goto  Next;
  }
  j = StackClusterPrivate_ArgTop(stclpp );
  while(CConsp( j)){
Next:;
    if( ClassCons_First(&ParaLispObjectToPLOCons(j)->cc) ==   mep){
      StackClusterPrivate_SetResult(stclpp ,&plofalse.plo);
      return  TaskForThread_Analyse;
    }
    j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) ;
    if((--  tasks)  ==  0){
      if(CConsp(  j)){
        StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        StackClusterPrivate_Push(stclpp , j);
        return  TaskForThread_Analyse;
      }
      break;
    }
  }
  StackClusterPrivate_SetResult(stclpp ,&plotrue.plo);
  return  TaskForThread_Analyse;
}

// nil boundp
TaskForThread bimnilboundp(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,&plotrue.plo);
  return  TaskForThread_Analyse;
}

# include "string/public.h"
# include "string/struct.h"
# include "env/public.h"
# include "stack/public.h"
  
//  nil defmethod symbol
TaskForThread bimnildefmethod(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  { ParaLispObject  * const namep=  CCAR ( arglistp);
    if(IsNotSymbol(  stclpp , namep))
      return  TaskForThread_Analyse;
    ParaLispObject  * const classjp  = CCDR ( GetPropPair ( 
        StackClusterPrivate_NamespaceProplist ( stclpp ) ,
        & plonil  . plo  , & plossystemclass  . plo ,   stclpp ) ) ;
    if(classjp==&plonil.plo ) {
      ParaLispObject  * list [ ] = {
        & PLOString_NewCopy(L"class nil is not defined")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      goto Exit ; 
    }
    ParaLispObject  * const classj2p=ClassGetMethods(classjp);
    if(IsNotEnvironmentLayer(  stclpp , classj2p)) 
      return  TaskForThread_Analyse;
    PLOCons * const res2 = NewCons  ( & plolambdaw  . plo ,  CCDR ( arglistp  ) ) ;
    if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
        classj2p ) -> envlay , namep  , &  res2 ->  plo , false ) ) {
      HasConstLink(  stclpp , namep);
      return  TaskForThread_Analyse;
    }
    StackClusterPrivate_SetResult(stclpp ,&  res2 ->  plo);
    // second method nil::name
    ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    String  ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
    StringConstrCStr ( WideChar )  ( wsp , L"nil::" , true ) ;
    StringAddCharArray ( WideChar )  ( wsp , StringCStr ( WideChar  ) (
      PLOSymbolCValue  ( ParaLispObjectToPLOSymbol ( namep  ) ) ) ) ;
    ParaLispObject  * const symbolmethodnamep  = SymbolAddMoveString ( wsp ) ;
    Lisp2D_StackFree  ( oldstack ) ;
    if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
      classj2p ) -> envlay , symbolmethodnamep  , &  res2 ->  plo , false ) ) {
      HasConstLink  (  stclpp , symbolmethodnamep  ) ;
      goto  Exit  ; 
    }
  } // name
Exit :
  return  TaskForThread_Analyse ; 
}

// nil getmethod
TaskForThread bimnilgetmethod ( StackClusterPrivate  * const stclpp ) {
  PLOCons localenv  ;
  PLOCons_InitC ( & localenv  , ClassGetMethods (  CCDR ( 
        GetPropPair ( StackClusterPrivate_NamespaceProplist (
            stclpp  ) , StackClusterPrivate_ThisTop ( stclpp ) ,
          & plossystemclass . plo , stclpp )  ) ) , & plonil  . plo ) ;
// ! to use EnvLayer:AssocPassage without NEW Cons
  ParaLispObject  * const res = Assoc ( CCAR ( StackClusterPrivate_ArgTop (
        stclpp  ) ) , & localenv  . plo , stclpp ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( res  ? res : & plonil . plo ) ) ;
  PLOCons_Destr ( & localenv ) ;
  return  TaskForThread_Analyse ;
}

//  nil|symbol get symbol
TaskForThread bimnilget ( StackClusterPrivate  * const stclpp ) {
  StackClusterPrivate_SetResult ( stclpp , CCDR ( 
    GetPropPair ( StackClusterPrivate_NamespaceProplist ( stclpp ) ,
    StackClusterPrivate_ThisTop ( stclpp ) ,
      CCAR ( StackClusterPrivate_ArgTop  ( stclpp )  ) , stclpp ) ) ) ;
  return  TaskForThread_Analyse ;
}

//  nil|symbol put symbol value
TaskForThread bimnilput ( StackClusterPrivate  * const stclpp ) {
  ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop ( stclpp )  ;
  ParaLispObject  * const yp = CCAR ( CCDR ( arglistp ) ) ;
  SetProp2  ( StackClusterPrivate_NamespaceProplist (
      stclpp ) , StackClusterPrivate_ThisTop(stclpp ) ,
    CCAR ( arglistp ) , yp ,  stclpp ) ;
  StackClusterPrivate_SetResult ( stclpp , yp ) ;
  return  TaskForThread_Analyse ;
}

# include "vector/public.h"
# include "inerrors.h"

# define  BaseRemProp Lisp2D_BaseRemPropRRhR  
// returns TRUE or FALSE (was or not)
static  ParaLispObject  * BaseRemProp ( ParaLispObject  * const pro ,
  ParaLispObject * const sp , ParaLispObject * const pp ,
  StackClusterPrivate  * const stclp ) {
  if  ( not  Vectorp  ( pro ) )   
    return & plofalse.plo;
  ParaLispObject  *   sl= PLOVector_Assoc3(ParaLispObjectToPLOVector(pro),sp);
  if(not  CConsp( sl))  
    return & plofalse.plo;
  sl  = CCDR ( sl);
  if(not  Vectorp( sl)){
    if(stclp) {     
      ParaLispObject  * list [ ] = { & PLOString_NewCopy(L"RemProp: ")->plo,sl,
        & PLOString_NewCopy(L" is not a vector")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclp,ListOfPLO(
        ( PArrayPLOP  ) { ( ParaLispObject*(*)[])&list}));
    } else
      inErrorS(PLOStreamPushWideCharArray  ( &Lisp2Dferr ,
          ( PArray_wchar_c ) { (  wchar_t const(*)[])& L"RemProp: "});
    PLOStreamPushPLO(& Lisp2Dferr ,  sl);
    PLOStreamPushWideCharArray  ( &Lisp2Dferr ,
          ( PArray_wchar_c )  { ( wchar_t const(*)[])&  L" is not a vector"});)
    return & plonil.plo;
  }
  return  PLOVector_RemProp(ParaLispObjectToPLOVector(sl),pp ) ;
}
  
//  nil|symbol remprop symbol
TaskForThread bimnilremprop(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult ( stclpp , BaseRemProp ( CCAR (
        StackClusterPrivate_NamespaceProplist ( stclpp ) ) ,
      StackClusterPrivate_ThisTop ( stclpp ) ,
      CCAR ( StackClusterPrivate_ArgTop ( stclpp )  ) , stclpp ) ) ;
  return  TaskForThread_Analyse ;
}

//  nil defmacro symbol
// const link
TaskForThread bimnildefmacro(StackClusterPrivate * const stclpp){
  ParaLispObject  const * const arglistp = StackClusterPrivate_ArgTop(stclpp ) ;
  ParaLispObject  * const namep = CCAR  ( arglistp ) ;
  if(IsNotSymbol(  stclpp , namep)) 
    return  TaskForThread_Analyse;
  PLOCons * const res2 =  NewCons ( & plomacronotmod . plo , CCDR ( arglistp ) ) ;
  { ParaLispObject  * const localenvfirstp = ClassGetMethods(  CCDR ( 
      GetPropPair (  StackClusterPrivate_NamespaceProplist (
        stclpp ) , & plonil  . plo , & plossystemclass . plo , stclpp ) ) ) ;
    if(IsNotEnvironmentLayer(  stclpp , localenvfirstp ) ) 
      return  TaskForThread_Analyse ;
    if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
          localenvfirstp ) -> envlay , namep , & res2  ->  plo , false ) ) {
      HasConstLink(  stclpp , namep ) ;
      return  TaskForThread_Analyse;
    }
    // second macro nil::name
    ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    String  ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
    StringConstrCStr ( WideChar )  ( wsp , L"nil::" , true ) ;
    StringAddCharArray ( WideChar )  ( wsp , StringCStr ( WideChar  ) ( PLOSymbolCValue  (
          ParaLispObjectToPLOSymbol ( namep ) ) ) ) ;    
    ParaLispObject  * const symbolmethodnamep = SymbolAddMoveString ( wsp ) ;
    Lisp2D_StackFree  ( oldstack ) ;
    if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
          localenvfirstp ) -> envlay , symbolmethodnamep , &  res2 ->  plo ,
          false ) ) {
      HasConstLink  (  stclpp , symbolmethodnamep ) ;
      goto  Exit  ;
    }
  } // localenvfirst
  StackClusterPrivate_SetResult(stclpp ,&  res2->plo);
Exit :  
  return  TaskForThread_Analyse ;
}

//  nil copy
TaskForThread bimnilcopy(StackClusterPrivate * const stclpp){
  ParaLispObject  * result;
  if  ( ParaLispObject_isBADCopy (
      StackClusterPrivate_ThisTop ( stclpp ) , & result  ) ) 
    OutOfMemory(   stclpp );    
  else   
    StackClusterPrivate_SetResult(stclpp ,  result);
  return  TaskForThread_Analyse;
}

//  nil nprocs
TaskForThread bimnilnprocs  ( StackClusterPrivate * const stclpp  ) {
  int const r = get_nprocs  ( ) ;
  PLOComplex  * const res = NewComplexBareExc ( ) ;
  Complex_SetInt ( PLOComplexToComplex  ( res ) , r ) ;
  StackClusterPrivate_SetResult ( stclpp  , & res ->  plo ) ;
  return  TaskForThread_Analyse ;
}

//  nil getclass
TaskForThread bimnilgetclass  ( StackClusterPrivate * const stclpp  ) {
  StackClusterPrivate_SetResult ( stclpp  , CCDR  ( GetPropPair (
        StackClusterPrivate_NamespaceProplist ( stclpp  ) ,
        StackClusterPrivate_ThisTop ( stclpp  ) , & plossystemclass . plo , stclpp  ) ) ) ;
  return  TaskForThread_Analyse ;
}

/*(defmacro do-while  (body res)
  `(progn (do-while-null  (if ,body nil true))  ,res))*/
// nil do-while
TaskForThread bimnildowhile ( StackClusterPrivate  * const stclpp ) {
  ParaLispObject  * varC  ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )
    goto  Next  ;
  // protection for cycle
  StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
  do  {
    varC  = StackClusterPrivate_ArgTop  ( stclpp ) ;
    for ( ; CConsp  ( varC  ) ;
      varC  = ClassCons_Rest  ( & Lisp2DParaLispObjectToPLOCons ( varC  ) ->  cc ) ) {
      StackClusterPrivate_Push  ( stclpp , varC ) ;
      StackClusterPrivate_Push  ( stclpp ,
        ClassCons_First ( & Lisp2DParaLispObjectToPLOCons ( varC  ) ->  cc  ) ) ;
      StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
      return  TaskForThread_Analyse ;
Next  :
      varC = StackClusterPrivate_PopValue ( stclpp )  ;
      if  ( StackClusterPrivate_exceptionnotprocessed  ( stclpp ) )
        return  TaskForThread_Analyse ;
    }
    varC  = StackClusterPrivate_result  ( stclpp )  ;
  } while ( ( varC  not_eq  ( & plonil  . plo ) ) and 
    ( varC not_eq  (  & plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

/*
(nil  try body
  object-name body
  ...
  object-name body
  NIL body)*/
TaskForThread bimniltry(StackClusterPrivate  * const stclpp){
  ParaLispObject  *  j;
  unsigned  int  tasks  = maxListItemProcessing ;
  ParaLispObject  * ev3;
  ParaLispObject  * eh;
  { EvalAddrCont2 const fl  = StackClusterPrivate_ClearflagCont ( stclpp ) ;
  if  ( fl ) {
    switch  ( fl  ) {
    case  1 :
      goto  Next  ;
    case  2 :
      if  ( not  StackClusterPrivate_exceptionnotprocessed  ( stclpp ) ) {
        StackClusterPrivate_setexceptionvalue ( stclpp , &  plonil . plo  ) ;
        StackClusterPrivate_setexceptionhistory ( stclpp , &  plonil.plo  ) ;
      }
      return  TaskForThread_Analyse;
    case  3 :
      eh = StackClusterPrivate_PopValue(stclpp );
      ev3 = StackClusterPrivate_PopValue(stclpp );
      j=StackClusterPrivate_PopValue(stclpp );
      goto  Next2;
    default : ;
# ifdef  LISP2D_DEBUG
      fwprintf ( stderr  , L"nil::try: fl=%u\n"  , (unsigned int)(fl));
# endif
      StackClusterPrivate_ExceptionSet  ( stclpp , & plonil . plo ) ;
      return  TaskForThread_Analyse ;
    } // switch fl
    return  TaskForThread_Analyse ;
  } // if fl
  } // fl
  { ParaLispObject  const * const  varCp =StackClusterPrivate_ArgTop(stclpp );
    if(not  CConsp(varCp)){
      StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
      return  TaskForThread_Analyse;
    }
    StackClusterPrivate_Push(stclpp ,
      ClassCons_First(&ParaLispObjectToPLOConsConst(varCp)->cc));
  }
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
Next:
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
    eh  = StackClusterPrivate_exceptionhistory ( stclpp );
    ev3 = StackClusterPrivate_exceptionvalue ( stclpp ) ;
    j = CCDR ( StackClusterPrivate_ArgTop(stclpp ));
    while(CConsp(   j)){
Next2:
      { ParaLispObject  const * const xp=
          ClassCons_First(&ParaLispObjectToPLOCons(j)->cc );
        if((  xp  ==  (  &  plonil.plo))  or  ( xp  ==  ev3)){
          StackClusterPrivate_setexceptionnotprocessed  ( stclpp , false ) ;
          StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
          StackClusterPrivate_Push(stclpp ,
            CCAR ( ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc ) ) ) ;
          return  TaskForThread_Analyse;
        }
      }
      // ->Rest->Rest
      j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) ;
      if(CConsp(  j))
        j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) ;
      else    
        return  TaskForThread_Analyse;
      if((--  tasks)  ==  0){
        if(CConsp(  j)){
          StackClusterPrivate_setTaskForManager ( stclpp , 
            TaskPause ) ;
          StackClusterPrivate_SetflagCont ( stclpp , 3U  ) ;
          StackClusterPrivate_Push(stclpp , j);
          StackClusterPrivate_Push(stclpp , ev3);
          StackClusterPrivate_Push(stclpp , eh);
          return  TaskForThread_Analyse;
        }
        break;
      }
    }
  }
  return  TaskForThread_Analyse ;
}

// !? to pass exception by all list, not first argument  
// (nil throw symbol symbol) !? =>    throws listof symbol's
// (nil throw '(symbol symbol)) !? =>   throws listof symbol's
// (nil throw symbol) !? =>   throws one symbol
// (nil throw '(symbol)) !? =>   throws listwith one symbol
TaskForThread bimnilthrow(StackClusterPrivate  * const stclpp){
  ParaLispObject * e;
  { ParaLispObject * const lp=StackClusterPrivate_ArgTop(stclpp );
    if(CConsp(lp)){
      if  ( CConsp  ( ClassCons_Rest  (& ParaLispObjectToPLOCons ( lp )->cc ) ) )
        e =  lp ;
      else 
        e = ClassCons_First(&ParaLispObjectToPLOCons ( lp )->cc );
    } else
      e = (& plonil.plo ) ;
  } // lp
  StackClusterPrivate_ExceptionSet(stclpp ,e);
  return  TaskForThread_Analyse ;
}

TaskForThread bimnilexception(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp , StackClusterPrivate_exceptionvalue ( stclpp ));
  return  TaskForThread_Analyse;
}

TaskForThread bimnilexceptionhistory(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,
    StackClusterPrivate_exceptionhistory ( stclpp ));
  return  TaskForThread_Analyse;
}
  
TaskForThread bimnilloglist(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult ( stclpp ,
    StackClusterPrivate_ListOfEvalHistory(stclpp ));
  return  TaskForThread_Analyse;
}

// nil progn [{body}]
TaskForThread bimnilprogn(StackClusterPrivate  * const stclpp){
  ParaLispObject  * varC  ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )
    goto  Next  ;
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))
    return  TaskForThread_Analyse;
  varC  = StackClusterPrivate_ArgTop(stclpp );
  while(CConsp(  varC)){
    StackClusterPrivate_Push(stclpp , varC);
    StackClusterPrivate_Push(stclpp ,
      ClassCons_First(&Lisp2DParaLispObjectToPLOCons(varC)->cc));
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    return  TaskForThread_Analyse;
Next:
    varC = StackClusterPrivate_PopValue(stclpp );
    if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))
      return  TaskForThread_Analyse;
    varC  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varC  )->cc ) ;
  }
  return  TaskForThread_Analyse ;
}

// nil cond ((test  body)..(test  body))
TaskForThread bimnilcond(StackClusterPrivate * const stclpp){
  ParaLispObject  * varC;
  ParaLispObject  * varB;
  ParaLispObject  * varA;
  unsigned  int tasks = maxListItemProcessing ;
  { EvalAddrCont2  const addr  = StackClusterPrivate_ClearflagCont ( stclpp ) ;
    if  ( addr ) {
      switch(addr){
      case  1:
        goto  Next1;
      case  2:
        goto  Next2;
      case  3:
        varC=StackClusterPrivate_PopValue(stclpp );
        if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
          StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
          StackClusterPrivate_SetflagCont ( stclpp , 4U  ) ;
          return  TaskForThread_Analyse;
        }
        goto  Next3;
      case  4:
        return  TaskForThread_Analyse;
      default :
        fwprintf(stderr,L"nil cond: addr=%u\n",(unsigned int)(addr  ) ) ;
        StackClusterPrivate_ExceptionSet  ( stclpp , & plonil . plo ) ;
        return  TaskForThread_Analyse ;
      } // switch addr
    } // if addr
  } // addr
  varC  = StackClusterPrivate_ArgTop(stclpp );
  for ( ; CConsp  ( varC  ) ;
    varC  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varC  )->cc ) ) {
    varA = ClassCons_First ( &ParaLispObjectToPLOCons ( varC  )->cc ) ;
    if(not  CConsp( varA)){
      if(tasks  > 0){
        --  tasks;
        goto  Next3;
      }
      // cycle protection
      StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
      StackClusterPrivate_SetflagCont ( stclpp , 3U  ) ;
      StackClusterPrivate_Push(stclpp , varC);
      return  TaskForThread_Analyse;
    }
    StackClusterPrivate_Push(stclpp , varA);
    StackClusterPrivate_Push(stclpp , varC);
    StackClusterPrivate_Push(stclpp , ClassCons_First(&Lisp2DParaLispObjectToPLOCons(varA)->cc));
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    return  TaskForThread_Analyse;
Next1:
    varC=StackClusterPrivate_PopValue(stclpp );
    varA=StackClusterPrivate_PopValue(stclpp );
    if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )) 
      return  TaskForThread_Analyse;
    varB  = StackClusterPrivate_result  ( stclpp );
    if((varB  not_eq  (&  plofalse.plo))  and (varB not_eq  (&  plonil.plo))){
      for ( varC  = ClassCons_Rest  ( &Lisp2DParaLispObjectToPLOCons ( varA  )->cc ) ;
        CConsp( varC);
        varC  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varC  )->cc ) ) {
        StackClusterPrivate_Push(stclpp , varC);
        StackClusterPrivate_Push(stclpp , ClassCons_First(&Lisp2DParaLispObjectToPLOCons(varC)->cc  ));
        StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
        return  TaskForThread_Analyse;
Next2:
        varC=StackClusterPrivate_PopValue(stclpp );
        if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))
          return  TaskForThread_Analyse;
      }  // for varC
    return  TaskForThread_Analyse;
    }  // if varB
Next3 : ;
  } // for varC
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse ;
}

# include "inner-history/inline.h"
  
// nil apply obj meth arglist
TaskForThread bimnilapply ( StackClusterPrivate  * const stclpp ) {
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t  const  wsih_nilapply [ ] = L"nil::apply:Start." ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_nilapply ) ;
# endif
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )
    goto Exit ;
  StackClusterPrivate_setflagApply  ( stclpp , true  ) ;
  { ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
    ParaLispObject  * const argrestp= CCDR ( arglistp);
    PLOCons * const argrest2 = NewCons ( CCAR ( argrestp ) ,
      CCAR ( CCDR ( argrestp ) ) ) ;
    PLOCons * const body2 = NewCons ( CCAR ( arglistp ) , & argrest2 -> plo ) ;
    StackClusterPrivate_Push(stclpp ,& body2->plo);
  }
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
Exit : ;
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t  const  wsih_nilapply2[] = L"nil::apply:Quit." ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , wsih_nilapply2  ) ;
# endif  
  return  TaskForThread_Analyse ;
}

// (nil funcall obj meth args .. args)
TaskForThread bimnilfuncall(StackClusterPrivate  * const stclpp){
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )   
    return  TaskForThread_Analyse;
  StackClusterPrivate_setflagApply  ( stclpp , true  ) ;
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const argrestp= CCDR ( arglistp);
  PLOCons * const argrest2 =  NewCons ( CCAR ( argrestp),  CCDR ( argrestp));
  { PLOCons * const body2 =  NewCons(  CCAR ( arglistp), &argrest2->plo);
    StackClusterPrivate_Push(stclpp ,& body2->plo);
  }
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse ;
}

// nil if test then else [oops]
TaskForThread bimnilif(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const arglp=StackClusterPrivate_ArgTop(stclpp );
  {  EvalAddrCont2 const addr  = StackClusterPrivate_ClearflagCont (
    stclpp ) ;
  if  ( addr ) {
    switch(addr){
    case  1:
      goto  Next1;
    case  2:
      return  TaskForThread_Analyse;
    default : ;
# ifdef  LISP2D_DEBUG
      { PLOComplex  * const  res=NewComplexBareExc();
        Complex_SetUInt ( PLOComplexToComplex( res),addr);
        ParaLispObject  * list [ ] = { & PLOString_NewCopy(L"nil::if:addr=")->plo,
          & res->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list  } ) ) ;
      } // res
      fwprintf(stderr,L"nil::if:addr=%u\n",(unsigned  int)(addr));
      return  TaskForThread_Analyse ;
# endif
    } // switch addr
  } // if addr
  } // addr
  if(not  CConsp(arglp)){
    // (nil if)
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  StackClusterPrivate_Push(stclpp ,ClassCons_First(&
    ParaLispObjectToPLOCons(arglp)->cc));
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
Next1:
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )) 
    return  TaskForThread_Analyse;
  ParaLispObject  * const cdrp= CCDR ( arglp);
  { ParaLispObject  * const testp = StackClusterPrivate_result  ( stclpp );
    ParaLispObject  * const cddrp= CCDR ( cdrp);
    if(   testp ==  ( & plonil.plo)){
      ParaLispObject  * const cdddrp= CCDR ( cddrp);
      if(CConsp( cdddrp)){
        //(nil  if  nil a b c) -> c
        StackClusterPrivate_Push(stclpp ,
          ClassCons_First(&ParaLispObjectToPLOCons( cdddrp)->cc));
        StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
      } else{
        //(nil  if  nil a [b]) -> b|nil
        if(CConsp( cddrp)){
          StackClusterPrivate_Push(stclpp ,
            ClassCons_First(&ParaLispObjectToPLOCons( cddrp)->cc));
          StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
        } else
          StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
      }
      return  TaskForThread_Analyse;
    }
    if(  testp ==  ( & plofalse.plo)){
      //(nil  if  nil a [b]) -> b|nil
      if(CConsp( cddrp)){
        StackClusterPrivate_Push(stclpp ,
          ClassCons_First(&ParaLispObjectToPLOCons( cddrp)->cc));
        StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
      } else
        StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
      return  TaskForThread_Analyse;
    }
  } // testp
  //(nil  if true [a]) -> a|nil
  if(CConsp( cdrp)){
    StackClusterPrivate_Push(stclpp ,
      ClassCons_First(&ParaLispObjectToPLOCons( cdrp)->cc));
    StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
  } else
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

/*(nil  defmacro  when(test &rest d)
  `(nil if  ,test (nil  progn ,@d)))*/
TaskForThread bimnilwhen(StackClusterPrivate * const stclpp){
  ParaLispObject  * varA;
  { EvalAddrCont2 const  addr = StackClusterPrivate_ClearflagCont (
      stclpp ) ;
  if  ( addr ) {
    switch(addr){
    case  1:
      goto  Next1;
    case  2:
      goto  Next2;
    default:
      fwprintf(stderr,L"nil when: addr=%u\n",(unsigned int)(addr));
      StackClusterPrivate_ExceptionSet  ( stclpp , & plonil . plo ) ;
      return  TaskForThread_Analyse ;
    } // switch addr
  } // if addr
  } // addr
  StackClusterPrivate_Push(stclpp ,
    CCAR ( StackClusterPrivate_ArgTop(stclpp )));
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
Next1:
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))  
    return  TaskForThread_Analyse;
  { ParaLispObject  * const testp = StackClusterPrivate_result  ( stclpp );
    if((  testp ==  (&  plofalse.plo)) or  (  testp ==  (&  plonil.plo)))
      return  TaskForThread_Analyse ;
  }
  varA  = CCDR ( StackClusterPrivate_ArgTop(stclpp ));
  for(;CConsp(  varA);
    varA  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varA  )->cc ) ) {
    StackClusterPrivate_Push(stclpp , varA);
    StackClusterPrivate_Push(stclpp ,
      ClassCons_First(&Lisp2DParaLispObjectToPLOCons(varA)->cc));
    StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
    return  TaskForThread_Analyse;
Next2:
    varA=StackClusterPrivate_PopValue(stclpp );
    if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))
      return  TaskForThread_Analyse ;
  }
  return  TaskForThread_Analyse ;
}

// nil parallel
TaskForThread bimnilparallel(StackClusterPrivate * const stclpp){
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t  const  Lisp2Dwsih_nilparallel[] = L"nil::parallel" ;  
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_nilparallel  ) ;
# endif  
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

// nil while test [{body}]
TaskForThread bimnilwhile(StackClusterPrivate  * const stclpp){
  ParaLispObject  * varC;
  { EvalAddrCont2 const  addr = StackClusterPrivate_ClearflagCont (
      stclpp ) ;
  if  ( addr ) {
    switch(addr) {
    case  1:
      goto  Next1;
    case  2:
      goto  Next2;
    default:
      fwprintf(stderr,L"nil while: addr=%u\n",(unsigned int)(addr ) ) ;
      StackClusterPrivate_ExceptionSet  ( stclpp , & plonil . plo ) ;
      return  TaskForThread_Analyse ;
    } // switch addr
  } // if addr
  } // addr
BIFWhile :
  StackClusterPrivate_Push(stclpp ,
    CCAR ( StackClusterPrivate_ArgTop(stclpp )));
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
Next1:
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))   
    return  TaskForThread_Analyse;
  { ParaLispObject  * const resp = StackClusterPrivate_result  ( stclpp );
    if((  resp  ==  (&  plonil.plo))  or 
      ( resp  ==  (&  plofalse.plo)))
      return  TaskForThread_Analyse;
  }  // resp
  varC  = CCDR ( StackClusterPrivate_ArgTop(stclpp ));
  for(;CConsp(  varC);
    varC  = ClassCons_Rest  ( &Lisp2DParaLispObjectToPLOCons ( varC  )->cc ) ) {
    StackClusterPrivate_Push(stclpp , varC);
    StackClusterPrivate_Push(stclpp ,
      ClassCons_First(&Lisp2DParaLispObjectToPLOCons(varC)->cc));
    StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
    return  TaskForThread_Analyse;
Next2:
    varC=StackClusterPrivate_PopValue(stclpp );
    if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))  
      return  TaskForThread_Analyse;
  }
  goto  BIFWhile ;
}

// nil eval obj
TaskForThread bimnileval(StackClusterPrivate * const stclpp){
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )    
    return  TaskForThread_Analyse;
  StackClusterPrivate_Push(stclpp ,
    CCAR ( StackClusterPrivate_ArgTop(stclpp )));
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
}

// nil setq [{symbol task}]
TaskForThread bimnilsetq(StackClusterPrivate * const stclpp){
  ParaLispObject  * varA;
  ParaLispObject  * j;
  ParaLispObject  * varB;
  {  EvalAddrCont2  const  addr  = StackClusterPrivate_ClearflagCont (
        stclpp ) ;
  if  ( addr ) {
    switch(addr){
    case  1 :
      if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
# ifdef  LISP2D_INNERHISTORY        
        static  wchar_t  const  Lisp2Dwsih_setq3[] =
          L"setq: switch1 exception catch and wait sons" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_setq3  ) ;
# endif      
        StackClusterPrivate_setTaskForManager ( stclpp , 
          TaskWaitSonsAlmost ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
        varA=StackClusterPrivate_PopValue(stclpp );
        return  TaskForThread_Analyse;
      }
      goto  Next;
    case  2: 
      return  TaskForThread_Analyse;
    case  3:
      varB=StackClusterPrivate_PopValue(stclpp );
      varA=StackClusterPrivate_PopValue(stclpp );
      j=StackClusterPrivate_PopValue(stclpp );
      if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
# ifdef  LISP2D_INNERHISTORY
        static  wchar_t  const  Lisp2Dwsih_setq []=
          L"setq: switch3 exception catch and wait sons" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_setq  ) ;
# endif  
        StackClusterPrivate_setTaskForManager ( stclpp , 
          TaskWaitSonsAlmost ) ;  
        StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
        return  TaskForThread_Analyse;
      }
      goto  NextVars;
    default : ;
# ifdef LISP2D_DEBUG
      PLOComplex  * const  res=NewComplexBareExc(  );
      Complex_SetUInt ( PLOComplexToComplex( res),addr);
      ParaLispObject  * list [ ] = { & PLOString_NewCopy(L"nil setq: addr=")->plo,
        & res->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
# endif
    } // switch addr
  } // if addr
  } // addr
  varA  =  & plonil.plo;
  varB  =  & plonil.plo;
  j = StackClusterPrivate_ArgTop(stclpp );
NextVars:
  for(unsigned  int tasks = maxTasksCreate ;(tasks  > 0)  and CConsp(   j);
      j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) , j  = CCDR ( j ) ,
      --  tasks ) {
    { PLOCons * const next2 =   NewCons(&plonil.plo,&plonil.plo);
      if(varA ==  ( & plonil.plo)) //  Null
        varA  = &next2->plo;
      else {
# ifdef  LISP2D_DEBUG
        if(not  (CConsp(  varB)))
          fwprintf(stderr,L"nil::setq: varB is not cons\n");
        else
# endif        
        ClassCons_SetRest(&ParaLispObjectToPLOCons(varB)->cc,& next2->plo);
      }
      varB  = &next2->plo;
    }
    if(IsNotSymbol(  stclpp ,ClassCons_First(&ParaLispObjectToPLOCons(j)->cc))){
      StackClusterPrivate_setTaskForManager ( stclpp , 
        TaskWaitSonsAlmost ) ;
      StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
      return  TaskForThread_Analyse;
    }
    StackClusterPrivate  * const dopstclp =
      StackClusterPrivate_NewStCl4(stclpp );
    SetResultPlace  (  dopstclp ,
      ClassCons_PFirst ( &ParaLispObjectToPLOCons ( varB  )->cc ) , varB  ) ;
    StackClusterPrivate_CreateTask(dopstclp ,
      CCAR ( ClassCons_Rest ( &ParaLispObjectToPLOCons ( j )->cc ) ) ) ;
  } // for tasks
  if(CConsp(  j)){
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 3U  ) ;
    StackClusterPrivate_Push(stclpp ,   j);
    StackClusterPrivate_Push( stclpp , varA);
    StackClusterPrivate_Push(stclpp ,  varB);
    return  TaskForThread_Analyse;
  }
// !!!! didnot make new list, if need make new variable,
// Assoc and place result in there presl=PLOPRest
  StackClusterPrivate_Push( stclpp ,  varA);
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
Next  :
  varA=StackClusterPrivate_PopValue(stclpp );
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
# ifdef  LISP2D_INNERHISTORY
    static  wchar_t  const  Lisp2Dwsih_setq2[] =
      L"setq: switch1 exception catch and go out" ;
    Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_setq2  ) ;
# endif      
    return  TaskForThread_Analyse;
  }
// ! to use TaskPause (cycle protection)
  for(ParaLispObject  * j3=  StackClusterPrivate_ArgTop(stclpp );
      CConsp(  j3);
    varA  = CCDR ( varA ) ,
    j3 = ClassCons_Rest (& ParaLispObjectToPLOCons ( j3  )->cc ) ,
    j3  = CCDR  ( j3  ) ) {
    ParaLispObject  * const varDp=ClassCons_First(&ParaLispObjectToPLOCons(j3)->cc);
    { ParaLispObject  * const valp= CCAR (   varA);
      bool flagsetisok=false;
      for(ParaLispObject  *   j2= StackClusterPrivate_Env (stclpp );CConsp(   j2);
        j2  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j2  )->cc ) ) {
        ParaLispObject  * const jlp=ClassCons_First(&ParaLispObjectToPLOCons(j2)->cc);
        if(IsNotEnvironmentLayer(  stclpp ,jlp))         
          return  TaskForThread_Analyse;
        bool  wasconst;
        if  ( EnvironmentLayerisOKTryChangeValue  ( ( t_EnvironmentLayerisOKTryChangeValue ) {
            . me = & ParaLispObjectToPLOEnvironmentLayer ( jlp  ) -> envlay , . sp = varDp  ,
            . vp = valp , . flagwasconstp = & wasconst , . flagconst = false } ) ) {
          flagsetisok = true;
          break;
        }
        if(wasconst)  {
          HasConstLink(  stclpp ,varDp ) ;
          return  TaskForThread_Analyse ;
        }
      }
      if(not  flagsetisok ) {
        ParaLispObject  * setenv2= StackClusterPrivate_Env( stclpp ) ;
        if(IsNotCons(  stclpp ,  setenv2 ) )    
          return  TaskForThread_Analyse ;
        setenv2 = ClassCons_First ( &ParaLispObjectToPLOCons ( setenv2 )->cc ) ;
        if(IsNotEnvironmentLayer(  stclpp ,  setenv2 ) )
          return  TaskForThread_Analyse ;
        if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
              setenv2 ) -> envlay , varDp  , valp , false ) ) {
          HasConstLink(  stclpp ,varDp ) ;
          return  TaskForThread_Analyse ;
        }
      }
    } // valp
  } // for j3
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo ) ;
  return  TaskForThread_Analyse ;
}

# include "bimc.h"
  
// nil fork {t} -> nil progn {t}
// nil fork t -> t
// nil fork ->
TaskForThread bimnilfork(StackClusterPrivate * const stclpp){
  StackClusterPrivate  * const newstclp = 
    StackClusterPrivate_NewStCl4(stclpp );
  StackClusterPrivate_setscFatherp ( newstclp , ((StackClusterPrivate*)0) ) ;
  StackClusterPrivate_SetEnv(newstclp,StackClusterPrivate_Env(stclpp ));
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp ) ;
  if  ( CConsp ( arglistp ) ) {
    ParaLispObject *  task  ;
    if  ( CConsp ( ClassCons_Rest ( &ParaLispObjectToPLOCons ( arglistp )->cc ) ) ) {
      PLOCons * const task2 =  NewCons(&(   plobimnilprogn->base.plo ),
        StackClusterPrivate_ArgTop(stclpp ));
      task  = &( NewCons(&plonil.plo,& task2->plo))->plo;
    } else
      task = ClassCons_First(&ParaLispObjectToPLOCons ( arglistp )->cc  ) ;
    StackClusterPrivate_CreateTask(newstclp , task);
  }
      
// !! doesnot returns exception ??
// to check...
      
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

// ! create forkregion

// nil let ([{symbol|([&const] symbol [{task}]}])) [{body}]
TaskForThread bimnillet(StackClusterPrivate  * const stclpp){
  ParaLispObject  * varA;
  ParaLispObject  * varB;
  ParaLispObject  * arglist;
  ParaLispObject  * j;
  enum  { Llet1 = 1U , Llet2 = 2U , Llet3 = 3U , Llet4 = 4U } ;
  { EvalAddrCont2 const  addr = StackClusterPrivate_ClearflagCont ( stclpp ) ;
    if  ( addr ) {
      switch(addr){
      case  Llet1 :
        goto  Next1;
      case  Llet2 :
        goto  Next2;
      case  Llet3 :
        varB=StackClusterPrivate_PopValue(stclpp );
        varA=StackClusterPrivate_PopValue(stclpp );
        j=StackClusterPrivate_PopValue(stclpp );
        if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
          StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
          StackClusterPrivate_SetflagCont ( stclpp , Llet4 ) ;
          return  TaskForThread_Analyse;
        }
        goto  Next;
      case  Llet4 :
        return  TaskForThread_Analyse;
      default:;
        PLOComplex  * const  res=NewComplexBareExc(  );
        Complex_SetUInt ( PLOComplexToComplex( res),addr);
        ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"nil::let: addr=")->plo, &res->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        return  TaskForThread_Analyse;
      } // switch addr
    } // if addr
  } // addr
  varA  = & plonil.plo;
  varB  = & plonil.plo;
  // ArgTop = ( ( v b .. b) .. ) [body] ..
  j = CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  // j = ( ( v b .. b) .. )
Next:

// ! first: create envrinoment
// ! second: run task with PFirst of new env

  for(unsigned  int tasks = maxTasksCreate ;(tasks  > 0) and CConsp(   j);
    j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) , -- tasks  ) {
    ParaLispObject  * lp=ClassCons_First(&ParaLispObjectToPLOCons(j)->cc);
    // l = ([&const] v b .. b) or ([&const] v) or ([&const]) or (v)  or v
    if(CConsp(  lp)){
      // l = ([&const] v b .. b) or ([&const] v) or ([&const])
      if  ( ClassCons_First(&ParaLispObjectToPLOCons ( lp  )->cc ) == & ploandconst . plo )
        lp = ClassCons_Rest ( &ParaLispObjectToPLOCons ( lp  )->cc ) ;
      if(CConsp(  lp)){
        // l = ( v b .. b) or (v)
        PLOCons *  next2 = NewCons(&plonil.plo,&plonil.plo);
        if(varB ==  ( & plonil.plo)) //  Null
          varA  = &next2->plo;
        else
          ClassCons_SetRest(&ParaLispObjectToPLOCons(varB)->cc,&  next2->plo);
        varB  = &next2->plo;
        ParaLispObject  * const taskp  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( lp  )->cc ) ;
        // task = (b b .. b) or ()
        if(CConsp(taskp)){
          // task = (b b .. b) or (b)
          ParaLispObject  * const taskrestp  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( taskp  )->cc ) ;
          // taskrest = (b b .. b) or ()
          StackClusterPrivate  * const dopstclp = StackClusterPrivate_NewStCl4(stclpp );
          SetResultPlace  (  dopstclp , ClassCons_PFirst ( &ParaLispObjectToPLOCons ( varB  )->cc ) , varB  ) ;
          StackClusterPrivate_SetEnv(dopstclp ,StackClusterPrivate_Env(stclpp ));
          if(CConsp(taskrestp)){
            // task = (b b) or (b b b) or ...
            PLOCons *  task2 = NewCons( &(   plobimnilprogn->base.plo ),taskp);
            PLOCons *  task3 = NewCons(&plonil.plo, &  task2->plo);
            StackClusterPrivate_CreateTask(dopstclp ,&   task3->plo);
            // if Consp(taskrest)
          } else{ // if not Consp(taskrest)
            // task = (b)
            ParaLispObject  * const taskfirstp=ClassCons_First(& ParaLispObjectToPLOCons(taskp)->cc);
            StackClusterPrivate_CreateTask(dopstclp ,taskfirstp);
          // task was atom
          }  // if Consp(taskrest)
        } // if Consp(task)
      } // if Consp(lp)
    } // if Consp(lp)
  }  // for
  if(CConsp(   j)){
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclpp , Llet3 ) ;
    StackClusterPrivate_Push(stclpp ,   j);
    StackClusterPrivate_Push(stclpp ,   varA);
    StackClusterPrivate_Push(stclpp ,  varB);
    return  TaskForThread_Analyse;
  }
  StackClusterPrivate_Push(stclpp ,   varA);
  StackClusterPrivate_setTaskForManager ( stclpp , 
    TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , Llet1 ) ;
  return  TaskForThread_Analyse;
Next1:
  varA=StackClusterPrivate_PopValue(stclpp );
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )) 
    return  TaskForThread_Analyse;
  
// ! first: create envrinoment
// ! second: run task with PFirst of new env      
  
  //new level of environment
  StackClusterPrivate_EnvNewLevel2(stclpp );
  arglist = StackClusterPrivate_ArgTop(stclpp );
  // arglist = ( ( v b .. b) .. ) [body] ..
  for(ParaLispObject  *   j2 = CCAR (  arglist);
    CConsp(   j2);
    j2  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j2  )->cc ) ) {
    // j2 = ( ( v b .. b) .. )
    ParaLispObject  * lp=ClassCons_First(&ParaLispObjectToPLOCons(j2)->cc);
    // l = ([&const] v b .. b) or ([&const] v) or v
    if(CConsp(  lp)){
      // l = ([&const] v b .. b) or ([&const] v) or (v)
      bool  flagconst;
      if(ClassCons_First(&ParaLispObjectToPLOCons(lp)->cc)  == & ploandconst . plo ) {
        lp = ClassCons_Rest ( &ParaLispObjectToPLOCons ( lp  )->cc ) ;
        flagconst = true;
      } else
        flagconst = false;
      if(CConsp(  lp)){
        // l = ( v b .. b) or (v)
        ParaLispObject  * const sp=ClassCons_First(&ParaLispObjectToPLOCons(lp)->cc);
        if(not  IsNotSymbol(   stclpp ,sp)){
          ParaLispObject  * const layerp= CCAR ( StackClusterPrivate_Env(stclpp ));
          if(IsNotEnvironmentLayer(   stclpp ,layerp)){
            StackClusterPrivate_EnvCDR(stclpp );
            return  TaskForThread_Analyse;
          }
          if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer ( layerp ) -> envlay ,
            sp , ClassCons_First(&ParaLispObjectToPLOCons ( varA  )->cc ) , flagconst ) ) {
            HasConstLink(  stclpp ,sp);
            return  TaskForThread_Analyse;
          }
          // getpair & run task
        } // if(not  IsNotSymbol(   stclpp,s))
        varA  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varA  )->cc ) ;
      } // if Consp(lp)
      // if Consp(lp)
    } else
      // l = v
      if(not  IsNotSymbol(   stclpp ,  lp)){
        ParaLispObject  * const layerp= CCAR ( StackClusterPrivate_Env(stclpp ));
        if(IsNotEnvironmentLayer(   stclpp ,layerp)){
          StackClusterPrivate_EnvCDR(stclpp );
          return  TaskForThread_Analyse;
        }
        if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
              layerp ) -> envlay , lp  , & plonil  . plo , false ) ) {
          HasConstLink(  stclpp ,  lp);
          return  TaskForThread_Analyse;
        }
        // getpair & run task
      } // if(not  IsNotSymbol(   stclpp,l))
  } // for j2
  varA  = CCDR(   arglist);
  for(;CConsp(    varA);
    varA  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varA  )->cc ) ) {
    StackClusterPrivate_Push(stclpp , varA);
    StackClusterPrivate_Push(stclpp ,ClassCons_First(&ParaLispObjectToPLOCons(varA) ->cc));
    StackClusterPrivate_SetflagCont ( stclpp , Llet2 ) ;
    return  TaskForThread_Analyse;
Next2:
    varA=StackClusterPrivate_PopValue(stclpp );
    if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
      StackClusterPrivate_EnvCDR(stclpp );
      return  TaskForThread_Analyse;
    }
  }
  //old environment restoring
  StackClusterPrivate_EnvCDR(stclpp );
  return  TaskForThread_Analyse;
}

# include "quote/public.h"
# include "consiter/public.h"
# include "consiter/struct.h"
  
# define  UnrollCommaAtsign Lisp2D_UnrollCommaAtsign
static  ParaLispObject  * UnrollCommaAtsign ( ParaLispObject  * const argp ,
  ParaLispObject  * const res2p  , Vector ( StackPLOP ) * const bufp ) {
  switch(ParaLispObject_Type(argp)){
  case  TypeGlobalListMemberPLOCons:
    { size_t const  bufs0=VectorSizeCopy(StackPLOP)(bufp);
      ParaLispObject  *   r= res2p;
      ParaLispObject  *   rold=&  plonil.plo;
      ParaLispObject  * i =  argp ;
    for(;CConsp( i);
      i = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( i )->cc ) , rold = r  ,
      r = CCDR ( r  ) ) {
      { ParaLispObject * const * j2 = VectorCBegin  ( StackPLOP ) ( bufp ) ;
        ParaLispObject * const * const je = VectorCEnd ( StackPLOP ) ( bufp ) ;
        for(;j2 < je;++ j2)
          if(*j2  ==  i){
            Vector_Resize(StackPLOP)(bufp,bufs0);
            return res2p;
          }
      }
      Vector_PushByValue(StackPLOP)(bufp,i);
      ParaLispObject * const xp = ClassCons_First(&
        Lisp2DParaLispObjectToPLOCons(i)->cc);
      switch(ParaLispObject_Type( xp)){
      case  TypeGlobalListMemberPLOCons:
        if(CConsp(  r))
          ClassCons_SetFirst(&Lisp2DParaLispObjectToPLOCons(r)->cc,
            UnrollCommaAtsign(xp,ClassCons_First(&
              Lisp2DParaLispObjectToPLOCons(r)->cc),bufp));
        break;
      case  TypeGlobalListMemberPLOQuote:
# ifdef  LISP2D_DEBUG
        if(not  CConsp( r)){
          fputws(L"UnrollCommaAtsign: 1. not(r->Consp())\n",stderr);
          return res2p ; 
        }
# endif
        UnrollCommaAtsign( xp,ClassCons_First(&
          Lisp2DParaLispObjectToPLOCons(r)->cc),bufp);
        break;
      case  TypeGlobalListMemberPLOUnquoteSplicing:
        { ParaLispObject  * const restp = CCDR (  r);
// !! TaskPause
          ThreadStackPointerType const oldstack_k = Lisp2DThreadSP ( ) ; 
          CConsIteratorPub * const k = NewStackCConsIterator ( ) ;
          CConsIteratorInit ( k , CCAR ( r) ) ;
          if(CConsIteratorHaveItem( k )){
            ClassCons_SetFirst(&Lisp2DParaLispObjectToPLOCons(r)->cc,
              CConsIteratorItem( k ));
            ClassCons_SetRest(&Lisp2DParaLispObjectToPLOCons(r)->cc, restp);
            CConsIteratorNext( k );
            while(CConsIteratorHaveItem( k )){
              { PLOCons  * const next2 =  NewCons( CConsIteratorItem(
                  k ), restp);
                ClassCons_SetRest(&Lisp2DParaLispObjectToPLOCons(r)->cc,
                  &  next2->plo);
                r = &next2->plo;
              }
              CConsIteratorNext( k );
            } 
          } else {
            // kill it
            if(CConsp(  rold))
              ClassCons_SetRest(&Lisp2DParaLispObjectToPLOCons(rold)->cc,
                restp); 
          } 
          Lisp2D_StackFree  ( oldstack_k ) ;
        } // restp
        break ;
        default : ; 
      }  // switch
    } // for i
      if(CConsp(  rold))
        ClassCons_SetRest(&Lisp2DParaLispObjectToPLOCons(rold)->cc,
          UnrollCommaAtsign( i, r,bufp));
      Vector_Resize(StackPLOP)(bufp,bufs0);
    } // bufs0
    break;
  case  TypeGlobalListMemberPLOQuote:
    { ParaLispObject * const * j2 = VectorCBegin  ( StackPLOP ) ( bufp ) ;
      ParaLispObject * const * const je = VectorCEnd ( StackPLOP ) ( bufp ) ;
      for(;j2 < je ;++j2)
        if(*j2  ==  argp )
          return res2p;
    }
# ifdef  LISP2D_DEBUG
    if(not  QuoteP ( res2p ) ) {
      fputws(L"UnrollCommaAtsign: 2. not(res2.QuoteP())\n",stderr);
      return res2p ;
    }
# endif
    Vector_PushByValue(StackPLOP)(bufp,argp);
    PLOQuote_SetItem  ( ParaLispObjectToPLOQuote  ( res2p  ) ,
      UnrollCommaAtsign(PLOQuote_Item(ParaLispObjectToPLOQuote(argp)),
        PLOQuote_Item(ParaLispObjectToPLOQuote(res2p)),bufp));
    VectorPop(StackPLOP)(bufp);
    return res2p;
# ifdef  LISP2D_DEBUG
  case  TypeGlobalListMemberPLOUnquoteSplicing:
    fputws(L"UnrollCommaAtsign: arg is ,@\n",stderr);
    return res2p ;
# endif
  default : ;
  }  // switch
  return res2p ;
}

# define  CopyTreeAndQuote  Lisp2D_CopyTreeAndQuote
static  ParaLispObject  * CopyTreeAndQuote(StackClusterPrivate * const stclpp ,
  ParaLispObject * const cp , Vector ( StackPLOP ) * const historyp ) {
  switch(ParaLispObject_Type(cp)){
  case  TypeGlobalListMemberPLOCons:
    { size_t const  topsize = VectorSizeCopy(StackPLOP)(historyp);
      ParaLispObject  *   arg=& plonil.plo;
      PLOCons *  lastarg=0;
      ParaLispObject  *   ja=  cp;
    do{
      { bool  flagcycled=false;
        { ParaLispObject * const * j2 = VectorCBegin  ( StackPLOP ) ( historyp ) ;
          ParaLispObject * const * const je = VectorCEnd ( StackPLOP ) ( historyp ) ;
          for(;j2 < je ;++ j2)
            if((* j2)  ==  ja){
              flagcycled  = true;
              break ;
            }
        }
        if(flagcycled)
          break ;
      }
      Vector_PushByValue(StackPLOP)(historyp,ja);
      ParaLispObject  * const jacarp=ClassCons_First(&ParaLispObjectToPLOCons(ja)->cc);
      { PLOCons  * const next2 =  NewCons( CopyTreeAndQuote(stclpp,jacarp,historyp),&plonil.plo);
        if(lastarg)
          ClassCons_SetRest(&lastarg->cc,&  next2->plo);
        else
          arg = &next2->plo;
        lastarg = next2;
      } // next2
      ja  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( ja  )->cc ) ;
    }while(CConsp(  ja));
      if(lastarg)
        ClassCons_SetRest(&lastarg->cc,CopyTreeAndQuote(stclpp, ja,historyp));
      Vector_Resize(StackPLOP)(historyp,topsize);
      return  arg;
    } // topsize
  case  TypeGlobalListMemberPLOQuote:
    { ParaLispObject * const * j2 = VectorCBegin  ( StackPLOP ) ( historyp ) ;
      ParaLispObject * const * const je = VectorCEnd ( StackPLOP ) ( historyp ) ;
      for(;j2 < je ;++  j2)
        if((* j2)  ==    cp)    
          return cp;
    }
    Vector_PushByValue(StackPLOP)(historyp, cp);
    { PLOQuote  * const res = aligned_alloc ( _Alignof ( PLOQuote ) , sizeof ( PLOQuote ) ) ;
      if  ( res ) {
        PLOQuote_Init(res);
        PLOQuote_SetItem(res, CopyTreeAndQuote(stclpp,
            PLOQuote_Item(ParaLispObjectToPLOQuote(cp)), historyp));
        VectorPop(StackPLOP)(historyp);
        return  ParaLispObject_virtualPushAsNew(&res->plo ) ;
      }
      VectorPop ( StackPLOP ) ( historyp  ) ;
      OutOfMemory ( stclpp ) ;
      return &  plonil . plo ;
    }
  default :  ;
  }
  return cp ;
}

# include "comma/public.h"
# include "unquotesplicing/public.h"

// `(a + b) backquote(..,(a+b))
// !! this is NOT method, but function for eval()
TaskForThread bimnilbackquote23 ( StackClusterPrivate * const stclpp , ParaLispObject  *  varA2p ) {
  ParaLispObject  *   varA=  varA2p;
  enum  { Lbac1 = 1U , Lbac2 = 2U } ;
  { EvalAddrCont2 const  addr = StackClusterPrivate_ClearflagCont ( stclpp ) ;
    if  ( addr ) {
      switch(addr){
      case  Lbac1:
        return  TaskForThread_Analyse;
      case  Lbac2:
        goto  Next2;
      default:
        inErrorS (
          PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
            (PArray_wchar_c ){ (wchar_t const(*)[])&L"` (backquote): addr=" });
          PLOStreamPushEvalAddrCont( & Lisp2Dferr , addr ) ; )
      } // switch addr
    } // if addr
  } // addr
  switch(ParaLispObject_Type(varA)){
  case  TypeGlobalListMemberPLOCons:
    // varA is Cons
    // (nil comma-at-sign ..)
    break;
  case  TypeGlobalListMemberPLOQuote:
    break;
  case  TypeGlobalListMemberPLOComma:
    // `,x -> eval(x)
    StackClusterPrivate_Push(stclpp ,
      PLOComma_Item(ParaLispObjectToPLOComma(varA ) ) ) ;
    StackClusterPrivate_SetflagCont ( stclpp , Lbac1  ) ;
    return  TaskForThread_Analyse;
  case  TypeGlobalListMemberPLOUnquoteSplicing:
    // `,@x -> eval(x)
    StackClusterPrivate_Push(stclpp ,
      PLOUnQuoteSplicing_Item(ParaLispObjectToPLOUnQuoteSplicing(varA ) ) ) ;
    StackClusterPrivate_SetflagCont ( stclpp , Lbac1  ) ;
    return  TaskForThread_Analyse;
  default:
    StackClusterPrivate_SetResult(stclpp,  varA);
    return  TaskForThread_Analyse;
  } // switch varA type
  { ThreadStackPointerType const oldstack_myp = Lisp2DThreadSP ( ) ;
    Vector  ( StackPLOP ) * const historyp = StackNew_VectorPLOP ( ) ;
    VectorInit  ( StackPLOP ) ( historyp , false ) ;
    varA  = CopyTreeAndQuote(stclpp, varA,historyp) ;    
    VectorClear ( StackPLOP ) ( historyp ) ;
    StackClusterPrivate_CreateTaskForBackquote  ( stclpp ,  varA  , historyp ) ;
    Lisp2D_StackFree  ( oldstack_myp ) ;
  }
  StackClusterPrivate_Push(stclpp ,  varA);
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , Lbac2  ) ;
  return  TaskForThread_Analyse;
Next2:
  varA=StackClusterPrivate_PopValue(stclpp );
  if  ( not  (  StackClusterPrivate_exceptionnotprocessed  ( stclpp ) ) ) {
    ThreadStackPointerType const oldstack_myp = Lisp2DThreadSP ( ) ;
    Vector  ( StackPLOP ) * const bufp = StackNew_VectorPLOP ( ) ;
    VectorInit  ( StackPLOP ) ( bufp , false ) ;
    StackClusterPrivate_SetResult(stclpp,UnrollCommaAtsign( varA2p, varA,bufp));
    Lisp2D_StackFree  ( oldstack_myp ) ;
  }
  return  TaskForThread_Analyse ; 
}

# include "complex/inline.h"
  
// returns error: (true/false)  
//  nil setdebug integer|false
TaskForThread bimnilsetdebug(StackClusterPrivate * const stclpp){
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  size_t  news2 =0;
  if(Numberp(xp)){
    if(FractionFiniteP(Complex_CRe(PLOComplexToComplexConst(
      ParaLispObjectToPLOComplex(xp))))){
      if(FractionMoreUInt(Complex_CRe(PLOComplexToComplexConst(
        ParaLispObjectToPLOComplex(xp))),0U)){
        if( FractionIsBadSizeValue(Complex_CRe(
          PLOComplexToComplexConst(
          ParaLispObjectToPLOComplex(xp))), &news2))
          return  TaskForThread_Analyse;
      } else{
        StackClusterPrivate_SetResult(stclpp ,&plotrue.plo);
        return  TaskForThread_Analyse;
      }
    } else{
      StackClusterPrivate_SetResult(stclpp ,&plotrue.plo);
      return  TaskForThread_Analyse;
    }
  }
  if(isBADResizeHistory(stclpp,news2))
    StackClusterPrivate_SetResult(stclpp ,&plotrue.plo);
  else  
    StackClusterPrivate_SetResult(stclpp ,&plofalse.plo);
  return  TaskForThread_Analyse;
}

//  nil getdebug
TaskForThread bimnilgetdebug(StackClusterPrivate  * const stclpp){
  PLOComplex  * const  res=NewComplexBareExc(  );
  { size_t const   ms  = Lisp2DStackClusterHistoryMemorySize (
      StackClusterPrivate_myHistoryC ( stclpp ) ) ;
    Complex_SetUSize  ( PLOComplexToComplex( res),ms);
    StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  }
  return  TaskForThread_Analyse;
}

# include "stcl/pushhistory.h"
  
//  nil log [{message}]
TaskForThread bimnillog(StackClusterPrivate  * const stclpp){
  PushHistory ( stclpp , StackClusterPrivate_ArgTop( stclpp ) ) ;
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

// nil map fn {lists}
TaskForThread bimnilmap(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

TaskForThread bimnilgetpid(StackClusterPrivate * const stclpp){
  int const i=getpid();
  PLOComplex  * const  res=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( res),i);
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  return  TaskForThread_Analyse;
}

TaskForThread bimnilgetppid(StackClusterPrivate  * const stclpp){
  int const i=getppid();
  PLOComplex  * const  res=NewComplexBareExc(  );
  Complex_SetInt  ( PLOComplexToComplex( res),i);
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  return  TaskForThread_Analyse;
}

# ifdef LISP2D_DEBUG
// nil *crash*
TaskForThread bimnilcrash(StackClusterPrivate  *  _ __attribute__ ((unused)) ){
  (* (volatile  int  *)  0 ) = 0;
  return  TaskForThread_Analyse;
}
# endif

//  nil constp x
TaskForThread bimnilconstp(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,
    (( CCAR ( StackClusterPrivate_ArgTop(stclpp )) -> flagConst) ?
    & plotrue.plo : & plofalse.plo));
  return  TaskForThread_Analyse ;
}

// ! to make all const : nil const [{x}]
//  nil const x
TaskForThread bimnilconst(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  ParaLispObject_SetFlagConstTrue  (  xp ) ;
  StackClusterPrivate_SetResult(stclpp ,xp);
  return  TaskForThread_Analyse ;
}

// ! to make all unconst : nil unconst [{x}]
//  nil unconst x
TaskForThread bimnilunconst(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(ParaLispObject_virtualisOKSetFlagConstFalse(xp) )
    StackClusterPrivate_SetResult(stclpp ,xp);
  else   { 
    ParaLispObject  * list [ ] = { xp ,
      & PLOString_NewCopy(L" can not be modified")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
  }
  return  TaskForThread_Analyse;
}

//  nil subseq [indexfirst [indexend]]
TaskForThread bimnilsubseq(StackClusterPrivate * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

//  nil without [indexend]
TaskForThread bimnilwithout(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

//  nil address-of x
TaskForThread bimniladdressof(StackClusterPrivate * const stclpp) {
  PLOComplex  * const  resp=NewComplexBareExc( );
  Complex_SetPointer  ( PLOComplexToComplex( resp) , CCAR (
      StackClusterPrivate_ArgTop(stclpp )));
  StackClusterPrivate_SetResult(stclpp ,&  resp->plo);
  return  TaskForThread_Analyse;
}

// nil private obj {body}
TaskForThread bimnilprivate(StackClusterPrivate  * const stclpp){
  ParaLispObject  * varC;
  ParaLispObject  * obj;
  ParaLispObject  * objclassdef;
  { EvalAddrCont2 const  addr = StackClusterPrivate_ClearflagCont (
      stclpp ) ;
  if  ( addr ) {
    switch(addr){
    case  1:
      goto  Next1;
    case  2:
      goto  Next2;
    default:
      inErrorS  ( PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        (PArray_wchar_c) { (wchar_t const(*)[])& L"nil::private: addr="}  ) ;
      PLOStreamPushEvalAddrCont(& Lisp2Dferr ,  addr); )
      } // switch addr
    } // if addr
  } // addr
  varC  = StackClusterPrivate_ArgTop(stclpp );
  if(not  CConsp(  varC))
    goto Exit ;
    // nil private -> nil
  obj = ClassCons_First(&Lisp2DParaLispObjectToPLOCons(varC)->cc);
  // eval obj
  StackClusterPrivate_Push(stclpp , varC);
  StackClusterPrivate_Push(stclpp , obj);
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  goto Exit ;
Next1 :
  varC=StackClusterPrivate_PopValue(stclpp );
  if(StackClusterPrivate_exceptionnotprocessed  ( stclpp ))
    goto Exit ;
  obj = StackClusterPrivate_result  ( stclpp );
  objclassdef =   ParaLispObject_virtualClassDef(obj);
  { ParaLispObject * const  symbolassoc = Assoc (
      & plosthis  . plo , StackClusterPrivate_Env ( stclpp )  , stclpp ) ;
    if  ( symbolassoc ) {
      if(( ParaLispObject_virtualClassDef(symbolassoc))  not_eq  objclassdef){
        ParaLispObject  * list [ ] = { obj,& PLOString_NewCopy(L" and this = ")
          ->plo,symbolassoc, & PLOString_NewCopy(L" has different type")->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        goto Exit ;
      }
    } else {
      ParaLispObject  * list [ ] = { & PLOString_NewCopy(
        L"method \"private\" works only inside object's method")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      goto Exit ;
    }
  } // symbolassoc
  // push this environment
  StackClusterPrivate_Push(stclpp ,StackClusterPrivate_Env(stclpp ));
  // trance to obj env
  // make variable this
  StackClusterPrivate_EnvNewLevel2(stclpp );
  { ParaLispObject  * const envLocalEnvCARp  = CCAR ( Environment_Env (
      StackClusterPrivate_envLocal ( stclpp ) ) ) ;
    if(IsNotEnvironmentLayer(  stclpp ,envLocalEnvCARp)){
      StackClusterPrivate_SetEnv  ( stclpp , StackClusterPrivate_PopValue(stclpp ));
      goto Exit ;
    }
    if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
          envLocalEnvCARp  ) -> envlay , & plosthis  . plo , obj , true  ) ) {
      HasConstLink(  stclpp , & plosthis  . plo ) ;
      goto Exit ;
    }
  }
  // static variables not needed
  { ParaLispObject  * const objContextp  = ParaLispObject_virtualContext  ( obj) ;
    if( objContextp not_eq  (& plonil.plo)){
      jmp_buf jump_buffer ;
      jmp_buf * const oldjb = Lisp2DJBglobal  ;
      Lisp2DJBglobal = &  jump_buffer ;
      int const volatile ex = setjmp  ( jump_buffer ) ;
      if  ( ex ) {
        Lisp2DJBglobal = oldjb  ;
        OutOfMemory( stclpp );
        StackClusterPrivate_SetEnv(stclpp , StackClusterPrivate_PopValue(stclpp ));
        goto Exit ;
      }
      Environment_NewLevel ( StackClusterPrivate_envLocal( stclpp ) , objContextp  ) ;
      Lisp2DJBglobal = oldjb  ;
    }
  }
  // eval {body}
  varC  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varC  )->cc ) ;
  for ( ; CConsp  ( varC  ) ; varC  = ClassCons_Rest  (
    &ParaLispObjectToPLOCons ( varC  )->cc ) ) {
    StackClusterPrivate_Push(stclpp , varC);
    StackClusterPrivate_Push(stclpp ,
      ClassCons_First(&Lisp2DParaLispObjectToPLOCons(varC)->cc));
    StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
    goto Exit ;
Next2:
    varC=StackClusterPrivate_PopValue(stclpp );      
    if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
      StackClusterPrivate_SetEnv(stclpp , 
        StackClusterPrivate_PopValue(stclpp ));
      goto Exit ;
    }
  }
  // load old environment
  StackClusterPrivate_SetEnv(stclpp ,
    StackClusterPrivate_PopValue(stclpp ));
Exit  :
  return  TaskForThread_Analyse ;
}

# include <sys/utsname.h>
# include <errno.h>
  
# define  LISP2D_BIM_UNAME( FUN_NAME , MEMBER_NAME ) \
TaskForThread FUN_NAME ( StackClusterPrivate * const stclpp ) { \
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ; \
    String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ; \
    String_Init ( WideChar  ) ( wanc , true ) ; \
    { struct  utsname u ; \
      if ( uname ( & u ) ) { \
        Lisp2D_StackFree  ( oldstack ) ; \
        StackClusterPrivate_SetResult ( stclpp  , \
          & SymbolFromIntErrno  ( errno ) -> plo ) ; \
        return  TaskForThread_Analyse ; \
      } \
      String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ; \
      StringConstrCStr  ( Char  ) ( strtmpc , u . MEMBER_NAME , true ) ; \
      if  ( isBADReadCStringToWString2 ( strtmpc , wanc ) ) { \
        Lisp2D_StackFree  ( oldstack ) ; \
        BadStringN(stclpp ); \
        return  TaskForThread_Analyse ; \
      } \
    } \
    { PLOStringPub * const  res2 = PLOString_NewMoveString ( wanc ) ; \
      StackClusterPrivate_SetResult ( stclpp ,&res2->plo); \
    } \
    Lisp2D_StackFree  ( oldstack ) ; \
  } \
  return  TaskForThread_Analyse;  \
}

//  nil sysname
LISP2D_BIM_UNAME ( bimnilsysname , sysname )

//  nil release
LISP2D_BIM_UNAME ( bimnilrelease , release )

//  nil version
LISP2D_BIM_UNAME ( bimnilversion , version )

//  nil machine
LISP2D_BIM_UNAME ( bimnilmachine , machine )

//  nil nodename
LISP2D_BIM_UNAME ( bimnilnodename , nodename )

//  nil getcwd
TaskForThread bimnilgetcwd(StackClusterPrivate * const stclpp){
  char  * const buf = getcwd  ( 0 , 0 ) ;
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ; 
    String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
    StringConstrCStr  ( Char  ) ( strtmpc , buf , true ) ;
    free(buf);
    String  ( WideChar ) * const wstrtmpc = StackNew_StringWideChar ( ) ;
    String_Init ( WideChar  ) ( wstrtmpc , true ) ;
    if  ( isBADReadCStringToWString2 ( strtmpc , wstrtmpc ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      BadStringN(stclpp );
      return  TaskForThread_Analyse ;
    }
    { PLOStringPub * const  res2 = PLOString_NewMoveString ( wstrtmpc ) ;
      StackClusterPrivate_SetResult( stclpp ,&res2->plo);
    }
    Lisp2D_StackFree  ( oldstack ) ;
  }
  return  TaskForThread_Analyse ;
}

//  nil getlogin
TaskForThread bimnilgetlogin  ( StackClusterPrivate * const stclpp  ) {
  char  * const reslog  = getlogin  ( ) ;
  if ( reslog ) {
    ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ; 
    String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
    StringConstrCStr  ( Char  ) ( strtmpc , reslog , true ) ;
    String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ;
    String_Init ( WideChar  ) ( wanc , true ) ;
    if  ( isBADReadCStringToWString2 ( strtmpc , wanc ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      BadStringN(stclpp );
      return  TaskForThread_Analyse ;
    }
    { PLOStringPub * const  ress2 = PLOString_NewMoveString ( wanc ) ;
      StackClusterPrivate_SetResult( stclpp ,&ress2->plo);
    } // ress
    Lisp2D_StackFree  ( oldstack ) ;
    return  TaskForThread_Analyse ;
  }
  StackClusterPrivate_SetResult ( stclpp , & SymbolFromIntErrno  ( errno ) -> plo ) ;
  return  TaskForThread_Analyse ;
}

//  nil environ-list
TaskForThread bimnilenvironlist  ( StackClusterPrivate  * const stclpp  ) {
  { ParaLispObject  * result  = & plonil.plo ;
    { PLOCons * lastresult  = 0 ;
      { char  * const * ei  = __environ ;
        ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ; 
        String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
        String_Init  ( Char  ) ( strtmpc , true ) ;
        String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ;
        String_Init ( WideChar  ) ( wanc , true ) ;
        for ( ; * ei  ; ++  ei  ) {
          StringSetArray ( Char  ) ( strtmpc , * ei ) ;
          if  ( isBADReadCStringToWString2 ( strtmpc , wanc ) ) {
            Lisp2D_StackFree  ( oldstack ) ;
            BadStringN (  stclpp ) ;
            goto  Exit  ;
          }
          { PLOStringPub * const  res2 = PLOString_NewMoveString ( wanc ) ;
            StringClear ( WideChar  ) ( wanc ) ;
            PLOCons * const constmp = NewCons  ( & res2->plo ,& plonil.plo ) ;
            if ( lastresult )
              ClassCons_SetRest(&lastresult->cc ,&  constmp->plo ) ;
            else
              result  = & constmp->plo ;
            lastresult  = constmp  ;
            // if new PLOString
          }
        }
        Lisp2D_StackFree  ( oldstack ) ;
      }
    } // lastresult
    StackClusterPrivate_SetResult (stclpp ,  result  ) ;
  } // result
Exit  :
  return  TaskForThread_Analyse ;
}

void  PLONilConstr  ( PLONil * const me ) {
  VirtualTypePointer  vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLONil } ;
  ParaLispObject_InitCVTP(&me->plo,true , vtp ) ;
  ADDPLONilconstrstat
}

# include "nil/inline.h"
# include "widestring/inline.h"
# include "stclhistory/inline.h"
