// Lisp2D - cool Passage
// ©2018-23 by Alex Glebe

# include <iso646.h>
# include "excc.h"
# include "passage/struct.h"
# include "passage/private.h"

struct  Lisp2DPassagePrivate passageGCC  ;

# include "passage/public.h"

Lisp2DPassagePub  * const passageGCCPub = & passageGCC . pub ;

void  PassagePassIn  ( Lisp2DPassagePub * const p ) {
  struct  Lisp2DPassagePrivate * const pp = Passage_toPrivate ( p ) ;
  if  ( pthread_mutex_lock  ( & pp -> mutexEntrancec ) )
    goto Bad ;
  if  ( pthread_mutex_lock  ( & pp -> mutexGuests ) )
    goto Bad2 ;
  ++  ( pp -> guests ) ;
  if  ( pthread_mutex_unlock  ( & pp -> mutexGuests ) )
    goto Bad2 ;
  if  ( pthread_mutex_unlock  ( & pp -> mutexEntrancec ) )
    goto Bad ;
  return  ;
Bad2 :
  pthread_mutex_unlock  ( & pp -> mutexEntrancec ) ;
Bad :
  Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DPassagePassIn:" ;
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
}
  
void  PassagePassOut ( Lisp2DPassagePub * const p ) {
  struct  Lisp2DPassagePrivate * const pp = Passage_toPrivate ( p ) ;
  if  ( pthread_mutex_lock  ( & pp -> mutexGuests ) )
    goto Bad ;
  --  ( pp -> guests ) ;
  if  ( pthread_mutex_unlock  ( & pp -> mutexGuests ) )
    goto Bad ;
  if  ( pthread_mutex_lock  ( & pp -> mutexOutletc  ) )
    goto  Bad ;
  if  ( pthread_mutex_lock  ( & pp -> mutexGuests ) )
    goto Bad2 ;
  { unsigned  int const wasguests = pp -> guests ;
    if  ( pthread_mutex_unlock  ( & pp -> mutexGuests ) )
      goto Bad2 ;
    if  ( ( wasguests ==  0 )  and (  pp -> flagToShout ) ) {
      pp -> flagToShout = false ;
      if  ( pthread_mutex_unlock  ( & pp -> mutexOutletc  ) )
        goto  Bad ;
      if  ( pthread_cond_signal ( & pp -> signalShoutc  ) )
        goto  Bad ;
    } else  {
      if  ( pthread_mutex_unlock  ( & pp -> mutexOutletc  ) )
        goto  Bad ;
    }
  } // wasguests
  return  ;
Bad2  :
  pthread_mutex_unlock  ( & pp -> mutexOutletc )  ;
Bad :
  Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DPassagePassOut:" ;
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;  
}
  
void  PassagePassLock  ( Lisp2DPassagePub * const p ) {
  struct  Lisp2DPassagePrivate * const pp = Passage_toPrivate ( p ) ;
  if  ( pthread_mutex_lock  ( & pp -> mutexEntrancec ) )
    goto Bad2 ;
  if  ( pthread_mutex_lock  ( & pp -> mutexGuests ) )
    goto Bad ;
  { unsigned  int wasguests = pp -> guests ;
    if  ( pthread_mutex_unlock  ( & pp -> mutexGuests ) )
      goto Bad ;
    if  ( wasguests ) {
      if  ( pthread_mutex_lock  ( & pp -> mutexOutletc  ) )
        goto  Bad ;
      do  {
        pp -> flagToShout = true  ;
        if  ( pthread_cond_wait ( & pp -> signalShoutc  , & pp -> mutexOutletc  ) )
          goto  Bad3 ;
        if  ( pthread_mutex_lock  ( & pp -> mutexGuests ) )
          goto Bad3 ;
        wasguests = pp -> guests ;
        if  ( pthread_mutex_unlock  ( & pp -> mutexGuests ) )
          goto Bad3 ;
      } while ( wasguests ) ;
      if  ( pthread_mutex_unlock  ( & pp -> mutexOutletc  ) )
        goto  Bad ;
    } // if wasguests
  } // wasguests
  return  ;
Bad3  :
  pthread_mutex_unlock  ( & pp -> mutexOutletc  ) ;
Bad :
  pthread_mutex_unlock  ( & pp -> mutexEntrancec  ) ;
Bad2 :
  Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DPassagePassLock:" ;
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
}

void  PassagePassUnLock  ( Lisp2DPassagePub * const p ) {
  struct  Lisp2DPassagePrivate * const pp = Passage_toPrivate ( p ) ;
  if  ( pthread_mutex_unlock  ( & pp -> mutexEntrancec ) ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DPassagePassUnLock:" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
}
  
void  PassageDestructor ( Lisp2DPassagePub * const p ) {
  struct  Lisp2DPassagePrivate * const pp = Passage_toPrivate ( p );
  pthread_mutex_destroy  ( & pp ->  mutexEntrancec ) ;
  pthread_mutex_destroy  ( & pp ->  mutexOutletc ) ;
  pthread_cond_destroy  ( & pp ->  signalShoutc ) ;
  pthread_mutex_destroy  ( & pp ->  mutexGuests ) ;
}

void  PassageConstructor ( Lisp2DPassagePub * const p ) {
  struct  Lisp2DPassagePrivate * const pp = Passage_toPrivate ( p );
  pp -> guests  = 0 ;
  pp -> flagToShout = false ;
  if  ( pthread_mutex_init  ( & pp ->  mutexEntrancec  , NULL ) )
    goto Bad ;
  if  ( pthread_mutex_init  ( & pp ->  mutexOutletc , NULL ) )
    goto Bad2 ;
  if  ( pthread_cond_init ( & pp ->  signalShoutc  , NULL ) )
    goto Bad3 ;
  if  ( pthread_mutex_init ( & pp ->  mutexGuests  , NULL ) )
    goto Bad4 ;
  return  ;
Bad4 :
  pthread_cond_destroy  ( & pp ->  signalShoutc ) ;
Bad3 :
  pthread_mutex_destroy  ( & pp ->  mutexOutletc ) ;
Bad2 :
  pthread_mutex_destroy  ( & pp ->  mutexEntrancec ) ;
Bad :
  Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DPassageConstructor:" ;
  longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
}
