// Lisp2D - Stream
// ©2011-23 by Alex Glebe

# include "plostream/public.h"

PLOStream Lisp2Dferr ;
  
PLOStream Lisp2Dfout ;
  
PLOStream Lisp2Dfin ;
  
# include "streamc.h"
# include "plostream/struct.h"

void  PLOStreamInitFD  ( PLOStream * const me , bool const inner , int const fd  ,
  int const flags ) {
  StreamInitFD(&me->str,inner , fd  , flags);
  ParaLispObject_InitVTP  ( & me  ->  plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOStream } ) ;
}
    
void  PLOStreamCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  ,
  Vector(StackPLOP)  * const h ) {
  PLOStreamCoutToInterpret (  me ,     s ,  flag2pprinted  ,  h ) ;    
}

void PLOStreamDestr  ( ParaLispObject * const me) {
  PLOStreamStaticDestr(ParaLispObjectToPLOStream(me));
}  

void PLOStreamStaticDestr  ( PLOStream * const me ) {
  StreamDestr(&me->str);
  ParaLispObject_staticDestr  ( & me  ->  plo ) ;
}
  
PLOStream const * PLOStreamPushWChar ( PLOStream  const * const mep , wchar_t const pc ) {
  wchar_t const buf [ ] = { pc , L'\00' } ;
  StreamPushWCharArray  ( & mep ->  str  ,
    ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) & buf } ) ;
  return mep ;
}

# include "stcl/classc.h"
# include "boolean.h"
# include "cons/public.h"
# include "stcl/public.h"
# include "stcl/inline.h"

// nil streamp
TaskForThread bimnilstreamp ( StackClusterPrivate  * const stclpp ) {
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop  ( stclpp ) ) ;
  StackClusterPrivate_SetResult ( stclpp ,
    ( Streamp ( xp  ) ? xp : &  plofalse  . plo  ) ) ;
  return  TaskForThread_Analyse ; 
}

# include <errno.h>
# include "nilc.h"

//  stream close
TaskForThread bimstreamclose ( StackClusterPrivate * const stclpp ) {
  PLOStream * const sp = ParaLispObjectToPLOStream ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  int res = 0 ;
  StreamClose ( & sp -> str , & res ) ;
  if  ( res ) {
    PLOSymbol * sres  = & plosquestion   ;
    switch  ( res ) {
    case EBADF  :
      sres  = & plosbadf  ;
      break ;
    case  EINTR :
      sres  = & plosintr  ;
      break ;
    case  EIO :
      sres  = & plosio  ;
      break ;
    default : ; 
    }
    StackClusterPrivate_SetResult ( stclpp ,  &  sres ->  plo ) ; 
  } else
    StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
  return   TaskForThread_Analyse  ; 
}
   
# include "errorstra.h"
# include "define.h"

void  PLOStreamCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool _ __attribute__ ((unused)) ,
  Vector ( StackPLOP ) * _2 __attribute__ ((unused)) ) {
  PLOStreamPushWideCharArray  ( s ,
    ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) & L"STREAM:"  } ) ;
  switch  ( StreamStreType  ( & ParaLispObjectToPLOStreamConst  ( me  ) ->
    str ) ) {
  case  StreamTypeName  ( No  ) : 
    PLOStreamPushWideCharArray  ( s ,
      ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) & L"No" } ) ;
    break ;
  case   StreamTypeName ( FD  ) :
    PLOStreamPushWideCharArray  ( s ,
      ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) & L"FD" } ) ;
    break ;
  default :
# ifdef  LISP2D_DEBUG      
    ErrorStra ( ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) &
      L"PLOStream::Cout: unknown type of stream"  } )
# endif
    ;
  }
}

//  stream listen
TaskForThread bimstreamlisten ( StackClusterPrivate  * const stclpp ) {
  PLOStream * const mep = ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop ( stclpp ) ) ;
  if  ( not  StreamReadable ( & mep ->  str ) ) {
    StackClusterPrivate_SetResult ( stclpp , & plofalse  . plo  ) ;
    return  TaskForThread_Analyse ; }
  StackClusterPrivate_SetResult ( stclpp ,
    ( ( not  (   StreamEof (  & mep ->  str ) ) ) ?
      & ( mep  ->  plo  )  :
      & ( plofalse . plo  ) ) ) ;
  return  TaskForThread_Analyse ; 
}

# include <unistd.h>
# include "passage/public.h"

// stream flush
TaskForThread bimstreamflush ( StackClusterPrivate * const stclpp ) {
  PLOStream * const mep = ParaLispObjectToPLOStream ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  switch ( StreamStreType ( & mep -> str ) ) {
  case StreamTypeName ( No ) :      
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult ( stclpp , & plonil . plo ) ;
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse ;
  case  StreamTypeName ( FD ) :
    if  ( StreamWritableFD  ( & mep  -> str ) ) {
      if  ( fsync (  StreamFDValue ( & mep -> str ) ) )
        return   SetResultErrnoLockGC  ( stclpp )  ; 
    }     
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult ( stclpp , & plonil . plo ) ; 
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse ;
  default :
# ifdef  LISP2D_DEBUG    
    fputws  ( L"PLOStream::Flush: type is unknown" , stderr  ) ;
    fflush  ( stderr  ) ;
# endif    
    return  TaskForThread_Analyse ; 
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult ( stclpp , & plonil . plo ) ;
  PassagePassOut ( passageGCCPub ) ;  
  return  TaskForThread_Analyse ; 
}

// stream openp
TaskForThread bimstreamopenp ( StackClusterPrivate * const stclpp ) {
  PLOStream * const streamp = ParaLispObjectToPLOStream ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp ,
    ( ( StreamStreType ( &  streamp ->  str ) ==  StreamTypeName  ( No  ) ) ?
      & plofalse  . plo :
      & streamp ->  plo ) ) ;
  return  TaskForThread_Analyse ;
}

ParaLispObject  * PLOStreamTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  & plosstream  . plo ;
}

# include "stack/public.h"

PLOStream const * PLOStreamPushPLO  ( PLOStream  const * const sp , ParaLispObject const * const op ) {
  ThreadStackPointerType const oldstack_myp = Lisp2DThreadSP ( ) ;
  Vector  ( StackPLOP ) * const historyp = Lisp2D_StackVar ( Lisp2DStackTypeStackPLOP ) ;
  VectorInit  ( StackPLOP ) ( historyp , false ) ;
  bool  const ffalse = false ;
  ParaLispObject_CoutToInterpret (  op ,  sp , & ffalse  , historyp  ) ;
  Lisp2D_StackFree  ( oldstack_myp ) ;
  return  sp  ;
}

# include "sharedc.h"
# include "shared/inline.h"
# include "inner-history/inline.h"
  
void  PLOStreamCoutHistoryError ( PLOStream * const  strep ) {
  if  ( not  flagThreadsShutDown  ) {
# ifdef  LISP2D_INNERHISTORY
    Lisp2DInnerHistoryShow  ( & Lisp2DinnerHistory  ) ;
# endif
    PLOStreamPushThreadIdC  (  strep , pthread_self  ( ) ) ;
    PLOStreamPushWideCharArray  (  strep , ( PArray_wchar_c ) { (  wchar_t const ( * ) [ ] ) & L" : Error: " } ) ;
  }
}
      
// S_ISVTX
# define  __USE_MISC
# include <sys/stat.h>

void  MKDirClass_RwxUsr(mode_t * const omp)  {
  (*omp)  or_eq S_IRWXU ;
}
  
void  MKDirClass_RUsr(mode_t * const omp)  {        
  (*omp)  or_eq S_IRUSR ;
}
  
void  MKDirClass_WUsr(mode_t * const omp)  {      
  (*omp)  or_eq S_IWUSR ;
}
  
void  MKDirClass_XUsr(mode_t * const omp)  {      
  (*omp)  or_eq S_IXUSR ;
}
  
void  MKDirClass_RwxGrp(mode_t * const omp) {
  (*omp)  or_eq S_IRWXG ;
}
  
void  MKDirClass_RGrp(mode_t * const omp) {      
  (*omp)  or_eq S_IRGRP ;
}
  
void  MKDirClass_WGrp(mode_t * const omp) {      
  (*omp)  or_eq S_IWGRP ;
}
  
void  MKDirClass_XGrp(mode_t * const omp) {      
  (*omp)  or_eq S_IXGRP ;
}
  
void  MKDirClass_RwxOth(mode_t * const omp) {      
  (*omp)  or_eq S_IRWXO ;
}
  
void  MKDirClass_ROth(mode_t * const omp) {      
  (*omp)  or_eq S_IROTH ;
}
  
void  MKDirClass_WOth(mode_t * const omp) {      
  (*omp)  or_eq S_IWOTH ;
}
  
void  MKDirClass_XOth(mode_t * const omp) {      
  (*omp)  or_eq S_IXOTH ;
}
  
void  MKDirClass_Suid(mode_t * const omp) {      
  (*omp)  or_eq S_ISUID ;
}
  
void  MKDirClass_Sgid(mode_t * const omp) {      
  (*omp)  or_eq S_ISGID ;
}
  
void  MKDirClass_Svtx(mode_t * const omp) {      
  (*omp)  or_eq S_ISVTX ;
}

# define  symboltoopenmodes_pri Lisp2D_symboltoopenmodes_pri
MapPri  ( FlessOpenModes  ) symboltoopenmodes_pri ;
MapPub  ( FlessOpenModes  ) * const symboltoopenmodes_pub =
  & symboltoopenmodes_pri . pub ;

int symboltoopenmodes_fun ( ParaLispObject * const x , ParaLispObject * const y ) {
  return  x < y ;
}

LISP2DSTACKPUSHBYVALUE ( StackMapNodeFlessOpenModes , MapNode ( FlessOpenModes ) , )
LISP2DSTACKPUSHATVALUE ( StackMapNodeFlessOpenModes , MapNode ( FlessOpenModes ) )
LISP2DSTACKCAT ( StackMapNodeFlessOpenModes ,
  MapNode ( FlessOpenModes ) )
LISP2DMAPPUBFINDBODY ( FlessOpenModes  , ParaLispObject *  )

# define  symboltomkdirmodes_pri Lisp2D_symboltomkdirmodes_pri
MapPri  ( FlessMkdirModes ) symboltomkdirmodes_pri ;
MapPub  ( FlessMkdirModes ) * const symboltomkdirmodes_pub = & symboltomkdirmodes_pri . pub ;

int symboltomkdirmodes_fun ( ParaLispObject * const x , ParaLispObject * const y ) {
  return  x < y ;
}

LISP2DSTACKPUSHBYVALUE ( StackMapNodeFlessMkdirModes , MapNode ( FlessMkdirModes ) , )
LISP2DSTACKPUSHATVALUE ( StackMapNodeFlessMkdirModes , MapNode ( FlessMkdirModes ) )
LISP2DSTACKCAT ( StackMapNodeFlessMkdirModes , MapNode ( FlessMkdirModes ) )
LISP2DMAPPUBFINDBODY ( FlessMkdirModes , ParaLispObject  * )

# include "string/public.h"
# include "string/struct.h"
# include "complex/public.h"
# include "plocomplex/public.h"

//  stream read-byte [place]
TaskForThread Lisp2Dbimstreamreadbyte2  ( StackClusterPrivate  * const stclpp ) {
  PLOStream * const mep = ParaLispObjectToPLOStream ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  if  ( not  StreamReadable ( & mep ->  str ) ) {
    PassagePassIn ( passageGCCPub ) ;
    { ParaLispObject  * list [ ] = { &  mep ->  plo , & PLOString_NewCopy (
        L" is not readable")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    }
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse ;
  }
  uint8_t uitmp;
  switch  ( StreamStreType  ( & mep ->  str ) ) {
  case  (StreamTypeName(FD)):
    { ssize_t   readsize  =
        read  ( StreamFDValue  (& mep  -> str ) , & uitmp , 1U  ) ;
      if  ( readsize  ==  -1  )
        return   SetResultErrnoLockGC  ( stclpp )  ;
      if  ( readsize  ==  0 ) {
        StreamSetEofFD  (&mep  -> str ) ;
        PassagePassIn ( passageGCCPub ) ;
        StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ;
        PassagePassOut ( passageGCCPub ) ;
        return   TaskForThread_Analyse  ;
      }
    } //  ssize_t &&  readsize
    goto  Next  ;
  case  (StreamTypeName(No)):
  default:
# ifdef  LISP2D_DEBUG      
    ErrorStra((PArray_wchar_c){(wchar_t const (*)[])&
      L"PLOStream::Read-byte: unknown type of stream"});
# else
    return  TaskForThread_Analyse;
# endif    
  } // switch(me.StreType())
  if(StreamEof(&mep->str)){     
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo); 
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse;
  }
Next  : ;
  ParaLispObject  * const placep=CCAR(StackClusterPrivate_ArgTop(stclpp ));
  if(Numberp(placep)) {
    Complex_SetUInt ( PLOComplexToComplex ( ParaLispObjectToPLOComplex  ( placep ) ) , uitmp ) ;
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult(stclpp ,placep);
    PassagePassOut ( passageGCCPub ) ; 
  } else  {
    PassagePassIn ( passageGCCPub ) ;
    PLOComplex  * const  res=NewComplexBareExc();
    Complex_SetUInt ( PLOComplexToComplex( res),uitmp);
    StackClusterPrivate_SetResult(stclpp ,&  res->plo); 
    PassagePassOut ( passageGCCPub ) ; 
  } // if(not Numberp(place))
  return  TaskForThread_Analyse; 
}

# include "bytedeque/public.h"
# include "cast.h"
  
# define  isBADWrite  Lisp2D_isBADWrite
static  bool  isBADWrite ( StackClusterPrivate * const stclp  ,
  PLOStream * const mep  , PLOByteDeque  * const bp , ssize_t * const ssp ) {
  { size_t const s = PLOByteDeque_Size ( bp ) ;
    * ssp  = size_cast_ssize ( s ) ;
  }
  if  ( * ssp  < 0 )
    return  true  ;
  char  * const buf = aligned_alloc ( _Alignof ( char ) , sizeof ( char ) * ssize_cast_size ( * ssp ) ) ;
  if  ( not buf ) {
    OutOfMemory (  stclp  ) ;
    return   true  ;
  }
  { DequeConstIterator ( Byte  ) bi = Deque_CBegin ( Byte  ) ( PLOByteDeque_CValue ( bp ) ) ;
    char  * bufi  = buf ;
    ssize_t i = 0 ;
    for ( ; i < * ssp  ; ++  i ) {
      ( * bufi  ) = uint8_cast_char ( * Deque_ConstIterator_Place ( Byte ) ( bi ) ) ;
      ++  bufi  ;
      Deque_ConstIterator_Inc ( Byte  ) ( & bi  ) ; 
    }
  }
  switch  ( StreamStreType  ( & mep ->  str ) ) {
    case  (StreamTypeName(FD)):
      *ssp  = write ( StreamFDValue  (&mep  -> str ) , & buf , ssize_cast_size ( *ssp  ) ) ;
      free ( buf ) ;
      if  ( * ssp  < 0 ) {
        SetResultErrnoLockGC  ( stclp ) ;
        return   true  ; 
      }
      break;
    case  (StreamTypeName(No)):
    default:
      free ( buf ) ;
# ifdef  LISP2D_DEBUG             
      PassagePassIn ( passageGCCPub ) ;
      StackClusterPrivate_ExceptionSet(stclp ,
        &PLOString_NewCopy(L"PLOStream::isBADWrite: unknown type of stream")
          ->plo);
      PassagePassOut ( passageGCCPub ) ;      
# endif    
      return true ; 
  } // switch(me.StreType())
  return false ; 
}
  
# include "complex/inline.h"
  
//  stream write-byte number|byte-vector
TaskForThread Lisp2Dbimstreamwritebyte2(StackClusterPrivate * const stclpp){
  PLOStream * const mep  = ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop ( stclpp ) ) ;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {     
    PassagePassIn ( passageGCCPub ) ;
    ParaLispObject  * list [ ] = { &  mep->plo,&  PLOString_NewCopy(
      L" is not writeable")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){(
      ParaLispObject*(*)[])&list}));
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse;
  }
  ParaLispObject  * const bp=CCAR(StackClusterPrivate_ArgTop(stclpp ));
  ssize_t   ss  = 1 ;
  if  ( ParaLispObject_Type  ( bp)  ==   TypeGlobalListMemberPLOByteDeque  ) {
    if  ( isBADWrite ( stclpp  , mep  , ParaLispObjectToPLOByteDeque (
      bp ) , & ss ) ) 
      return  TaskForThread_Analyse;
    goto  Return  ; 
  }
  if  ( IsNotNumberLockGC (   stclpp , bp ) )    
    return  TaskForThread_Analyse;
  switch(StreamStreType(&mep->str)){
    case  (StreamTypeName(FD)):
      { uint8_t buf ;
        if  ( FractionIsBADByteValue  ( 
            Complex_CRe  (  PLOComplexToComplexConst  (
              ParaLispObjectToPLOComplexConst ( bp )) ) , &buf ) ) {      
          PassagePassIn ( passageGCCPub ) ;
          ParaLispObject  * list [ ] = { bp ,
            & PLOString_NewCopy ( L" number is bad" )->plo , NULL } ;
          StackClusterPrivate_ExceptionSet(
            stclpp ,ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          PassagePassOut ( passageGCCPub ) ; 
          return   TaskForThread_Analyse  ; 
        }
        ss  = write ( StreamFDValue  (&mep->str ) , & buf , 1U  ) ;
        if  ( ss  < 0 ) 
          return   SetResultErrnoLockGC  ( stclpp )  ; 
      } // uint8_t buf ;
      break;
    case  (StreamTypeName(No)):
    default:
# ifdef  LISP2D_DEBUG             
      PassagePassIn ( passageGCCPub ) ;
      StackClusterPrivate_ExceptionSet(stclpp ,&PLOString_NewCopy(
        L"PLOStream::Write-byte: unknown type of stream")->plo);
      PassagePassOut ( passageGCCPub ) ;      
# endif    
      return  TaskForThread_Analyse;      
  } // switch(me.StreType())
Return:      
  { PLOComplex  * const  res =NewComplexBareExc  (   )  ;
    Complex_SetSSize ( PLOComplexToComplex(  res) , ss  ) ;
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult (stclpp ,&  res->plo ) ;
    PassagePassOut ( passageGCCPub ) ; 
  }
  return   TaskForThread_Analyse  ; 
}

# include "string/public.h"
  
// string truncate number
TaskForThread Lisp2Dbimstringtruncate2(StackClusterPrivate  * const stclpp)  {
  PLOStringPub * const mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char ) * const strtmp = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmp , true ) ;
  if  ( isBADReadWCharToCharString ( PLOString_CStr  ( mep ) , strtmp  ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString(stclpp ,  mep ) ; 
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse;
  }
  ParaLispObject  * const xp=CCAR(StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumberLockGC(  stclpp,xp))  {
    Lisp2D_StackFree  ( oldstack ) ;
    return  TaskForThread_Analyse;
  }
  if(truncate(StringCStr ( Char  ) ( strtmp ),FractionLValue  ( 
     Complex_CRe  (PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst (
       xp )) ) ))) {
    Lisp2D_StackFree  ( oldstack ) ;
    return   SetResultErrnoLockGC  ( stclpp )  ;
  }
  Lisp2D_StackFree  ( oldstack ) ;
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  PassagePassOut ( passageGCCPub ) ;  
  return  TaskForThread_Analyse; 
} // bimstringtruncate2   

//  stream truncate length
TaskForThread Lisp2Dbimstreamtruncate(StackClusterPrivate * const stclpp){
  PLOStream * const mep=
    ParaLispObjectToPLOStream(StackClusterPrivate_ThisTop(stclpp ));
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    PassagePassIn ( passageGCCPub ) ;
    ParaLispObject  * list [ ] = { &  mep->plo,&  PLOString_NewCopy(
      L" is not writeable")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse; 
  }
  ParaLispObject  * const bp=CCAR(StackClusterPrivate_ArgTop(stclpp ));
  if  ( IsNotNumberLockGC (   stclpp , bp ) )   
    return  TaskForThread_Analyse;
  if(StreamStreType(&mep->str)  ==  StreamTypeName(FD)){
    { int const err =  ftruncate ( StreamFDValue  (&mep  ->str ) ,
          FractionLValue  (
            Complex_CRe  (  PLOComplexToComplexConst  (
              ParaLispObjectToPLOComplexConst ( bp ) ) ) ) ) ;
      if  ( err  < 0 ) 
        return   SetResultErrnoLockGC  ( stclpp )  ;
    } // int err     
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult (stclpp ,& plonil.plo ) ;
    PassagePassOut ( passageGCCPub ) ; 
    return   TaskForThread_Analyse  ;
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_ExceptionSet(stclpp ,&PLOString_NewCopy(
    L"stream::truncate: stream type not supported")->plo); 
  PassagePassOut ( passageGCCPub ) ;    
  return  TaskForThread_Analyse ;
}  // Lisp2Dbimstreamtruncate

//  stream chdir
TaskForThread Lisp2Dbimstreamchdir  ( StackClusterPrivate * const stclpp  ) {
  PLOStream * const mep= ParaLispObjectToPLOStream(StackClusterPrivate_ThisTop(stclpp ));
  if(StreamStreType(&mep->str)==StreamTypeName(FD)){
    { int const  e = fchdir ( StreamFDValue  (&mep  ->str ) ) ;
      if  ( e ) 
        return   SetResultErrnoNotLockGC  ( stclpp )  ;
    }
    StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ;
  } else
    StackClusterPrivate_ExceptionSet( stclpp ,
      &PLOString_NewCopy(L"stream::chdir: stream type not supported")->plo);
  return  TaskForThread_Analyse;
}

# include "deque/public-wchar.h"

//  stream read
TaskForThread Lisp2Dbimstreamread2(StackClusterPrivate  * const stclpp){
  PLOStream * const mep = ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop ( stclpp ));
  if(not  StreamReadable(&mep->str)){     
    PassagePassIn ( passageGCCPub ) ;
    ParaLispObject  * list [ ] = { &  mep->plo,&  PLOString_NewCopy(
      L" is not readable")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse;
  }         
  PassagePassIn ( passageGCCPub ) ;
  {  ParaLispObject  *   dop= & plonil.plo;
    { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
      Deque ( WChar ) * const emptybufl = Lisp2D_StackVar ( StackType ( Deque_WChar  ) ) ;
      Deque_Init ( WChar ) ( emptybufl ) ;
// ! to read in new thread
      StackClusterPrivate_LoadObjectS ( stclpp ,  & dop , & mep ->  str ,
        emptybufl ) ;   
      Lisp2D_StackFree  ( oldstack ) ;
    }
    StackClusterPrivate_SetResult(stclpp ,  dop); 
  }
  PassagePassOut ( passageGCCPub ) ;
  return  TaskForThread_Analyse ; 
}

# define  seekNo  Lisp2D_seekNo
static  TaskForThread seekNo  ( StackClusterPrivate * const stclp  )  {   
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult (stclp ,& plonil.plo  ) ; 
  PassagePassOut ( passageGCCPub ) ;  
  return   TaskForThread_Analyse  ; 
}
  
# define  LockGCSetResultStreamPos  Lisp2D_LockGCSetResultStreamPos
static  void  LockGCSetResultStreamPos ( StackClusterPrivate * const stclp ,
  off_t const sp  ) {
  PassagePassIn ( passageGCCPub ) ;
  PLOComplex  *  const cres  =NewComplexBareExc  (   )  ;
  Complex_SetOff  ( PLOComplexToComplex(  cres)  , sp )  ;
  StackClusterPrivate_SetResult (stclp ,&  cres->plo  ) ; 
  PassagePassOut ( passageGCCPub ) ; 
}
  
# define  seekFD  Lisp2DseekFDorWF
static  TaskForThread seekFD  (PLOStream  * me,PLOComplex const *  numb,
  ParaLispObject const * mode, StackClusterPrivate * const stclp  )  {
  off_t const lv  = FractionLValue  ( Complex_CRe  ( PLOComplexToComplexConst(numb)    ) ) ;
  off_t res = -1  ;
  if  ( (  mode  ) == & ploscur .  plo )
    res = lseek ( StreamFDValue (&me -> str ) , lv , SEEK_CUR ) ;
  else if  ( (  mode  ) == & plosend . plo )
    res = lseek ( StreamFDValue (&me -> str ) , lv , SEEK_END ) ;
  else if  ( (  mode  ) == & plosdata . plo )
# ifdef SEEK_DATA
    res = lseek ( StreamFDValue (&me -> str ) , lv , SEEK_DATA ) ;
# else
    res = lseek ( StreamFDValue (&me -> str ) , lv , SEEK_SET ) ;
# endif          
  else if  ( (  mode  ) == & ploshole . plo )
# ifdef SEEK_HOLE
    res = lseek ( StreamFDValue (&me -> str ) , lv , SEEK_HOLE ) ;
# else           
    res = lseek ( StreamFDValue (&me -> str ) , 0 , SEEK_END ) ;
# endif                      
  else
    res = lseek ( StreamFDValue (&me -> str ) , lv , SEEK_SET ) ;
  if  ( res ==  -1  )    
    return   SetResultErrnoLockGC  ( stclp )  ;
  if( StreamEof(&me -> str) )
    StreamUnSetEofFD(&me -> str);              
  LockGCSetResultStreamPos ( stclp  , res  ) ;  
  return  TaskForThread_Analyse;
}
  
//  stream seek number [beg|cur|end|data|hole] 
TaskForThread Lisp2Dbimstreamseek ( StackClusterPrivate  * const stclpp  ) {
  PLOStream * mep  =ParaLispObjectToPLOStream (StackClusterPrivate_ThisTop(
    stclpp ) ) ;
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp ) ;
  ParaLispObject  * const numbp  =CCAR ( arglistp )  ;
  if  ( IsNotNumberLockGC (  stclpp  , numbp  ) )   
    return   TaskForThread_Analyse  ;
  ParaLispObject const * const modep  =CCAR ( CCDR ( arglistp ) ) ;
  switch  ( StreamStreType(&mep  -> str) ) {
  case  StreamTypeName(FD)  :
    seekFD(mep, ParaLispObjectToPLOComplex (  numbp  ) ,  modep  ,
           stclpp  ) ;
    break ;
  case  StreamTypeName(No)  :
    seekNo( stclpp  ) ;
    break ;
  default : ;
# ifdef  LISP2D_DEBUG  
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_ExceptionSet  (stclpp ,& PLOString_NewCopy(
      L"seek: unknown type of stream")->plo ) ;
    PassagePassOut ( passageGCCPub ) ;
# endif
  } // switch
  return  TaskForThread_Analyse ; 
}

# define  tellFD  Lisp2D_tellFD
static  void  tellFD  (PLOStream const * me, StackClusterPrivate * const stclp  )  {
  off_t sp2 = lseek ( StreamFDValue (&me ->str ) , 0 , SEEK_CUR )  ;
  LockGCSetResultStreamPos ( stclp , sp2  ) ;        
}    

# define  tellNo  Lisp2D_tellNo
static  void  tellNo  ( StackClusterPrivate * const stclp  )  {
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult (stclp ,& plonil.plo  ) ;  
  PassagePassOut ( passageGCCPub ) ; 
}
  
//  stream tell
TaskForThread Lisp2Dbimstreamtell ( StackClusterPrivate  * const stclpp  ) {
  PLOStream * const mep  =ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop( stclpp ) ) ;
  switch  ( StreamStreType(&mep  -> str) ) {
  case  StreamTypeName(FD)  :
    tellFD  (  mep  , stclpp  ) ;
    break ;
  case  StreamTypeName(No)  :
    tellNo  ( stclpp  ) ;
    break ;
  default : ;
# ifdef  LISP2D_DEBUG  
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_ExceptionSet  (stclpp ,& PLOString_NewCopy (
      L"tell: unknown type of stream" )->plo ) ;
    PassagePassOut ( passageGCCPub ) ;
# endif
  }    
  return  TaskForThread_Analyse ; 
}
  
//  stream terpri
TaskForThread Lisp2Dbimstreamterpri2(StackClusterPrivate  * const stclpp){
  PLOStream * const mep=ParaLispObjectToPLOStream(
    StackClusterPrivate_ThisTop(stclpp ));
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {     
    PassagePassIn ( passageGCCPub ) ;
    ParaLispObject  * list [ ] = { &  mep->plo,&  PLOString_NewCopy(
      L" is not writeable")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse;
  }
  PLOStreamEndl ( mep);   
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclpp ,&mep->plo);
  PassagePassOut ( passageGCCPub ) ;  
  return  TaskForThread_Analyse;
}

PLOStream const * PLOStreamEndl ( PLOStream const * const  mep  ) {
  if(StreamStreType(&mep->str)  ==  (StreamTypeName(No))) 
    return  mep;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) 
    goto ErrWr ;
  switch(StreamStreType(&mep->str)){
  case  (StreamTypeName(FD)):
    { char  const c = '\n' ;
      ssize_t   ss = write (  StreamFDValue (&mep ->str ) ,  &  c , 1U  ) ;  
      if  ( ss  < 0 ) {
        PLOSymbol * const errname =  SymbolErrno();
        fwprintf  ( stderr  , L"Endl(PLOStream):write: %ls\n" ,
          StringCStr  ( WideChar  ) (  PLOSymbolCValue  ( errname ) ) ) ;
        break;
      }
    }
    fsync ( StreamFDValue (&mep ->str ) ) ;    
    break;
  case StreamTypeName(No):
  default:
# ifdef  LISP2D_DEBUG    
    fputws(L"Endl(PLOStream): type is unknown\n",stderr)
# endif 
    ;
  }
  return  mep;
ErrWr :
# ifdef  LISP2D_DEBUG    
  fputws(L"Endl(PLOStream): stream is not writable\n",stderr);
# endif    
  return  mep;
}

void  functionisBADReadCharToWChar_Init  ( functionisBADReadCharToWChar  * const me  ,
  StreamType const metype0 , Stream * const s ) {
  functionboolcharp_Init  ( & me -> base ,  type_functionisBADReadCharToWChar ) ;
  me -> metype = metype0 ;
  me -> sp = s ;
}

    
bool  functionisBADReadCharToWChar_Fun ( functionboolcharp  * const p , char  * const x ) {
  functionisBADReadCharToWChar  * const me  =
    fromFunctionboolcharpToFunctionisBADReadCharToWChar ( p ) ;
    switch  ( me  ->  metype  ) {
    case  (StreamTypeName(FD)):        
      { ssize_t const  readsize  =
          read  ( StreamFDValue  ( me  ->  sp ) , x , 1U  ) ;
        if  ( readsize  ==  -1  )  
          return true;           
        if  ( readsize  ==  0 ) {
          StreamSetEofFD  ( me  ->  sp ) ;
          return true;
        }
      } //  ssize_t &&  readsize
      return  false;
      break;
    case  (StreamTypeName(No)):
    default : ;
    }  // switch(metype)
    return  false  ;
}

// ! to make fork: to read      
# define  isBADReadCharToWChar  Lisp2D_isBADReadCharToWChar
static  bool  isBADReadCharToWChar ( char const x , wchar_t  * const yp ,
  StreamType const  metype  , Stream * const sp  ) {
  functionisBADReadCharToWChar  f ;
  functionisBADReadCharToWChar_Init ( & f , metype ,  sp ) ;
  if  ( isBADReadWCharUniversal ( x , & f . base ,  yp ) )
    return   true   ;
  return   false  ;
}

# include "char/public.h"
    
// stream read-char [place]
TaskForThread  Lisp2Dbimstreamreadchar2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const placep=CCAR(StackClusterPrivate_ArgTop(stclpp ));  
  PLOStream * const mep  = ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop( stclpp ) ) ;
  StreamType  const  metype = StreamStreType(&mep ->str)  ;
  if(StreamReadable(&mep->str)){
    if(  metype == (StreamTypeName(FD))){      
      char  st[2];  
      switch(metype){
      case  (StreamTypeName(FD)):
        { ssize_t   readsize  =
            read  ( StreamFDValue  (&mep  -> str ) , st , 1U  ) ;
          if  ( readsize  ==  -1  )
            return   SetResultErrnoLockGC  ( stclpp )  ;
          if  ( readsize  ==  0 ) {
            StreamSetEofFD  (&mep  -> str ) ;
            PassagePassIn ( passageGCCPub ) ;
// ? or ploseof          
            StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
            PassagePassOut ( passageGCCPub ) ;
            return  TaskForThread_Analyse;
          }
        } //  ssize_t &&  readsize
        break;
      case  StreamTypeName(No):
      default:;
      }
      if(StreamEof(&mep->str)  ){         
        PassagePassIn ( passageGCCPub ) ;
// ? or ploseof                  
        StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
        PassagePassOut ( passageGCCPub ) ;        
        return  TaskForThread_Analyse;
      }
      wchar_t y;
      if(isBADReadCharToWChar(st[0],&y,metype,&mep->str)){         
        PassagePassIn ( passageGCCPub ) ;
        StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
        PassagePassOut ( passageGCCPub ) ;        
        return  TaskForThread_Analyse;
      }
      if(Charp(placep)){
        PLOChar_SetValue ( ParaLispObjectToPLOChar ( placep ) , y ) ;         
        PassagePassIn ( passageGCCPub ) ;
        StackClusterPrivate_SetResult(stclpp ,placep);
        PassagePassOut ( passageGCCPub ) ; 
      } else  {
        PLOCharPriv  * const res = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
        if ( not res )
          OutOfMemory (   stclpp ) ;
        else {
          PLOChar_InitValue ( res , y ) ;
          PassagePassIn ( passageGCCPub ) ;
          StackClusterPrivate_SetResult ( stclpp , ParaLispObject_virtualPushAsNew  ( & res ->  pub . plo ) ) ;
          PassagePassOut ( passageGCCPub ) ;      
        }
      }
      return  TaskForThread_Analyse;
    }   
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_ExceptionSet(stclpp ,&PLOString_NewCopy(
      L"stream::read-char: stream type is unknown")->plo);
    PassagePassOut ( passageGCCPub ) ;  
    return  TaskForThread_Analyse;
  }  // readable 
  PassagePassIn ( passageGCCPub ) ;
  ParaLispObject  * list [ ] = { & mep->plo, & PLOString_NewCopy(
    L" is not readable")->plo, NULL } ;
  StackClusterPrivate_ExceptionSet(stclpp ,
    ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
  PassagePassOut ( passageGCCPub ) ;    
  return  TaskForThread_Analyse;
}

# include "inerrors.h"
    
// ! to #define D(stream,int const * l,*l)  
PLOStream const * PLOStreamPushInt  ( PLOStream  const * const mep , int lv ) {

  if(StreamStreType(&mep->str)  ==  (StreamTypeName(No)))  
    return  mep;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    inErrorS(
      
      PLOStreamPushPLO(&Lisp2Dferr ,&mep->plo);
      PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        (PArray_wchar_c) { (wchar_t const(*)[])&  L" is not writeable"});)
    return  mep;
  }
  switch(StreamStreType(&mep->str)){
  case   ( StreamTypeName(FD)  ) :
    { int bufw = dprintf  ( StreamFDValue ( & mep ->  str ) , "%d" , lv ) ;
      
      if  ( bufw < 0 ) {
        inErrorS  (
          PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
            (PArray_wchar_c) { (wchar_t const(*)[])&
              L"PLOStreamPushInt:dprintf<0"}); )
        break ;
      }
      
      
// ! writes 10101 as 10,101        
      
    } // std ::  string  st  ;
    break;
  case   ( StreamTypeName(No)  ) :
  default:
    inErrorS(
      PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        (PArray_wchar_c) { (wchar_t const(*)[])&
          L"<<(int*): unknow stream type: " } ); 
      PLOStreamPushUInt(& Lisp2Dferr ,StreamStreType(&mep->str));)
  }
  // switch me.StreType
  return  mep;
}

ParaLispObject  * PLOStreamClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassdefstream;
}

ParaLispObject  * PLOStreamClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ){
  return  ploclassparentsstream;
} 

# define  bimstringopenbinary20 Lisp2D_bimstringopenbinary20YVCb
static  TaskForThread bimstringopenbinary20  ( 
  StackClusterPrivate  * const stclpp , ParaLispObject  * const kp ) {
  PassagePassIn ( passageGCCPub ) ;
  { ParaLispObject  * list [ ] = { & PLOString_NewCopy (
        L"string::open:unknown flag " )->plo ,  kp , NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
  }
  PassagePassOut ( passageGCCPub ) ;      
  return   TaskForThread_Analyse  ;  
}
  
# include "consiter/public.h"
# include "consiter/struct.h"
  
// string open [{flag}]
// flags by default = in + out
TaskForThread Lisp2Dbimstringopenbinary2  ( StackClusterPrivate  * const stclpp ) {
  OpenModes modes ;
  OpenModes_Init  ( & modes ) ;
// !? TaskPause
  { ThreadStackPointerType const oldstack_j = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const j = NewStackCConsIterator ( ) ;
    CConsIteratorInit ( j , StackClusterPrivate_ArgTop  ( stclpp  ) ) ;
    for ( ; CConsIteratorHaveItem  ( j ) ; CConsIteratorNext  ( j ) ) {
      ParaLispObject  * const kp = CConsIteratorItem  ( j ) ;
    /*
#define O_ACCMODE	   0003
#define O_RDONLY	     00
#define O_WRONLY	     01
#define O_RDWR		     02    
    */
// !? how to do : FAST switch
// ! std::set<PLOSymbol,C}) >  

      size_t  const i = MapPubFind  ( FlessOpenModes  ) ( symboltoopenmodes_pub ,  kp ) ;
      if  ( i  == MapPubSize ( FlessOpenModes  ) ( symboltoopenmodes_pub ) )
        return bimstringopenbinary20  (  stclpp ,  kp ) ;
      MapPubCAt ( FlessOpenModes  ) ( symboltoopenmodes_pub , i ) -> value ( & modes ) ;
    } // for ConsIterator j
    Lisp2D_StackFree  ( oldstack_j ) ;
  } // jp
    
    if  ( modes.flagread  )
      modes . flagsFD = ( modes . flagwrite ?
        ( ( modes.flagsFD bitand (~ O_ACCMODE)) bitor O_RDWR ) :
        ( ( modes.flagsFD bitand (~ O_ACCMODE)) bitor O_RDONLY ) ) ;
    else
      if  ( modes.flagwrite )     
        modes.flagsFD = (modes.flagsFD bitand (~ O_ACCMODE)) bitor O_WRONLY  ;
      else  
        // default
        modes.flagsFD = (modes.flagsFD bitand (~ O_ACCMODE)) bitor O_RDWR  ;
        
    PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) ) ;
    { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
      String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
      String_Init ( Char  ) ( strtmpc , true ) ;
      if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
        Lisp2D_StackFree  ( oldstack ) ;
        PassagePassIn ( passageGCCPub ) ;
        BadString ( stclpp  , mep  ) ; 
        PassagePassOut ( passageGCCPub ) ;        
        return   TaskForThread_Analyse  ; 
      }
      int   newfd = open  ( StringCStr  ( Char  ) ( strtmpc ) , modes.flagsFD , modes.modeFD  ) ;
      Lisp2D_StackFree  ( oldstack ) ;
      if  ( newfd  ==  -1 ) 
        return   SetResultErrnoLockGC  ( stclpp )  ;
      if  ( modes.flagatend ) {
        if  ( lseek ( newfd , 0 , SEEK_END  ) == -1 ) {
          // trouble
          SetResultErrnoLockGC(stclpp );
          // ... close fd   ... to return symbol error
          close ( newfd ) ; // ignore second error          
          return   TaskForThread_Analyse  ; 
        }
      } // lseek ( newfd , 0 , SEEK_END  ) == -1        
      //  create PLOStream ... 
      PLOStream  * const stro2 = aligned_alloc ( _Alignof ( PLOStream ) , sizeof ( PLOStream ) ) ;
      if  ( stro2 ) {
        PLOStreamInit ( stro2 ) ;
        StreamSetFDFromInt  ( & stro2->str , newfd ) ;
        StreamSetFDFlags  ( & stro2->str , modes . flagsFD ) ;
        StreamSetFDMode ( & stro2->str , modes . modeFD  ) ;
        PassagePassIn ( passageGCCPub ) ;
        ParaLispObject_virtualPushAsNew (&stro2 ->  plo ) ;
        StackClusterPrivate_SetResult  (stclpp ,&  stro2->plo  ) ;
        PassagePassOut ( passageGCCPub ) ; 
      } else   
        OutOfMemory (   stclpp ) ;
    } // std ::  string  strtmp  ;
    return   TaskForThread_Analyse  ; 
}

# include "cons/inline.h"
    
# define  isBADLoadUIDGID Lisp2D_isBADLoadUIDGID
static  bool  isBADLoadUIDGID ( StackClusterPrivate * const stclp  ,
  uid_t * const newuidp  , gid_t * const newgidp  ) {
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclp )  ;
  if  ( not CConsp ( arglistp ) ) {     
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult (stclp ,& plonil.plo  ) ; 
    PassagePassOut ( passageGCCPub ) ;    
    return   true   ; 
  }
  ParaLispObject  * x = ClassCons_First(&ParaLispObjectToPLOCons (
    arglistp ) ->cc )  ;
  if  ( ( & plonil.plo ) not_eq  (   x ) ) {
    if  ( IsNotNumberLockGC (   stclp ,  x ) ) 
      return   true   ;
    // ! to UIDValue
    * newuidp  = FractionUIValue (
      Complex_CRe  (PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst (
        x )) ) ) ; 
  }
  x = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( arglistp )->cc ) ;
  if ( CConsp (  x ) )  {
    x = ClassCons_First(&ParaLispObjectToPLOCons ( x )->cc ) ;
    if  ( ( & plonil.plo ) not_eq  (   x ) ) {
      if  ( IsNotNumberLockGC (   stclp ,  x ) ) 
        return   true   ;
      // ! to GIDValue
      * newgidp  = FractionUIValue (
        Complex_CRe  (PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst ( x )) ) ) ; 
    }
  }
  return   false  ; 
}
    
//  string change-owner-group [nil|number [nil|number]]
TaskForThread Lisp2Dbimstringchownusergroup ( StackClusterPrivate  * const stclpp  ) {
  uid_t newuid  = ( uid_t ) ( -1  ) ;
  gid_t newgid  = ( gid_t ) ( -1  ) ;
  if  ( isBADLoadUIDGID ( stclpp  , & newuid  , & newgid  ) ) 
    return  TaskForThread_Analyse  ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  PLOStringPub * const  mep =ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ))  ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString ( stclpp , mep  ) ;
    PassagePassOut ( passageGCCPub ) ;
    return   TaskForThread_Analyse  ; 
  }
  if  ( chown ( StringCStr  ( Char  ) ( strtmpc ) , newuid  , newgid  ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    return   SetResultErrnoLockGC  ( stclpp ) ; 
  }
  Lisp2D_StackFree  ( oldstack_a ) ;
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ; 
  PassagePassOut ( passageGCCPub ) ;  
  return   TaskForThread_Analyse  ; 
}

//  string lchange-owner-group [nil|number [nil|number]]
TaskForThread Lisp2Dbimstringlchownusergroup (
  StackClusterPrivate  * const stclpp  ) {
  uid_t newuid  = ( uid_t ) ( -1  ) ;
  gid_t newgid  = ( gid_t ) ( -1  ) ;
  if  ( isBADLoadUIDGID ( stclpp  , & newuid  , & newgid  ) )    
    return   TaskForThread_Analyse  ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString ( stclpp  , mep  ) ;
    PassagePassOut ( passageGCCPub ) ;
    return   TaskForThread_Analyse  ; 
  }
  if  ( lchown ( StringCStr  ( Char  ) ( strtmpc ) , newuid  , newgid  ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    return   SetResultErrnoLockGC  ( stclpp ) ; 
  }
  Lisp2D_StackFree  ( oldstack ) ;
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ; 
  PassagePassOut ( passageGCCPub ) ;  
  return   TaskForThread_Analyse  ; 
}
  
//  stream change-owner-group [nil|number [nil|number]]
TaskForThread Lisp2Dbimstreamchownusergroup ( 
  StackClusterPrivate  * const stclpp  ) {
  uid_t newuid  = ( uid_t ) ( -1  ) ;
  gid_t newgid  = ( gid_t ) ( -1  ) ;
  if  ( isBADLoadUIDGID ( stclpp  , & newuid  , & newgid  ) )   
    return   TaskForThread_Analyse  ;
  PLOStream * const mep=
    ParaLispObjectToPLOStream(StackClusterPrivate_ThisTop(stclpp ));
  if(StreamStreType(&mep->str) == StreamTypeName(FD)){
    if ( fchown( StreamFDValue  (&mep->str ) , newuid,newgid) ) 
      return   SetResultErrnoLockGC  ( stclpp )  ; 
  } else{
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_ExceptionSet(stclpp ,&PLOString_NewCopy(
      L"stream::change-owner-group: stream type not supported")->plo); 
    PassagePassOut ( passageGCCPub ) ;
    return  TaskForThread_Analyse; 
  }   
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ; 
  PassagePassOut ( passageGCCPub ) ;  
  return   TaskForThread_Analyse  ; 
}   

# include "stat/public.h"
  
TaskForThread bimstringstat ( StackClusterPrivate * const stclp  ,
  PLOStringPub * const  mep , PLOStat * const resultp  ) {
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString ( stclp , mep ) ;
    PassagePassOut ( passageGCCPub ) ;
    return   TaskForThread_Analyse  ; 
  }
  { struct  stat  ss ;
    if  ( stat ( StringCStr  ( Char  ) ( strtmpc ) , &  ss  ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      return   SetResultErrnoLockGC  ( stclp ) ; 
    }
    Lisp2D_StackFree  ( oldstack ) ;
    PLOStat_SetValue ( resultp , & ss ) ;
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult (stclp ,& resultp -> plo ) ; 
    PassagePassOut ( passageGCCPub ) ; 
  } // ss
  return   TaskForThread_Analyse  ; 
}

//  string stat [stat]
TaskForThread Lisp2Dbimstringstat ( StackClusterPrivate  * const stclpp  ) {
  PLOStringPub * const mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) )  ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString ( stclpp  , mep ) ;
    PassagePassOut ( passageGCCPub ) ;
    return   TaskForThread_Analyse  ; 
  }
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp ) ;
  if  ( CConsp ( arglistp  ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    ParaLispObject  * const sp  = ClassCons_First(&ParaLispObjectToPLOCons (
      arglistp )->cc ) ;     
    PassagePassIn ( passageGCCPub ) ;
    if  ( IsNotStat ( sp , stclpp ) ) {
      PassagePassOut ( passageGCCPub ) ;
      return   TaskForThread_Analyse  ; 
    }
    PassagePassOut ( passageGCCPub ) ;    
    return  bimstringstat ( stclpp  , mep ,
      ParaLispObjectToPLOStat ( sp ) ) ; 
  } // consp arglist
  { struct  stat  ss ;
    if  ( stat ( StringCStr  ( Char  ) ( strtmpc ) , &  ss  ) ) { 
      Lisp2D_StackFree  ( oldstack ) ;
      return   SetResultErrnoLockGC  ( stclpp ) ; 
    }
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    { PLOStat * const  res = NewStat ( stclpp , & ss ) ;
      if  ( res )  
        StackClusterPrivate_SetResult (stclpp ,&  res->plo ) ;
    }
    PassagePassOut ( passageGCCPub ) ;
  } // ss
  return   TaskForThread_Analyse  ;
}

# define  bimstringlstat  Lisp2D_bimstringlstat
static  TaskForThread bimstringlstat ( StackClusterPrivate * const stclp  ,
  PLOStringPub * const  mep , PLOStat * const resultp  ) {
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString ( stclp , mep ) ;
    PassagePassOut ( passageGCCPub ) ;
    return   TaskForThread_Analyse  ;
  }
  { struct  stat  ss ;
    if  ( lstat ( StringCStr  ( Char  ) ( strtmpc ) , &  ss  ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      return   SetResultErrnoLockGC  ( stclp ) ; 
    }
    Lisp2D_StackFree  ( oldstack ) ;
    PLOStat_SetValue ( resultp , & ss ) ;    
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult (stclp ,& resultp->plo ) ; 
    PassagePassOut ( passageGCCPub ) ; 
  } // ss
  return   TaskForThread_Analyse  ; 
}
  
//  string lstat [stat]
TaskForThread Lisp2Dbimstringlstat ( StackClusterPrivate  * const stclpp  ) {
  PLOStringPub * const mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) )  ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString ( stclpp  , mep ) ;
    PassagePassOut ( passageGCCPub ) ;
    return   TaskForThread_Analyse  ; 
  }
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp ) ;
  if  ( CConsp ( arglistp  ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    ParaLispObject  * const sp  = ClassCons_First(&
      ParaLispObjectToPLOCons ( arglistp )->cc ) ;     
    PassagePassIn ( passageGCCPub ) ;
    if  ( IsNotStat ( sp , stclpp ) )    { 
      PassagePassOut ( passageGCCPub ) ;
      return   TaskForThread_Analyse  ; 
    }
    PassagePassOut ( passageGCCPub ) ;    
    return  bimstringlstat ( stclpp  , mep ,
      ParaLispObjectToPLOStat( sp ) ) ;
  }
  { struct  stat  ss ;
    if  ( lstat ( StringCStr  ( Char  ) ( strtmpc ) , &  ss  ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      return   SetResultErrnoLockGC  ( stclpp ) ;
    }
    Lisp2D_StackFree  ( oldstack ) ;
    PassagePassIn ( passageGCCPub ) ;
    { PLOStat * const  res = NewStat ( stclpp , & ss ) ;
      if  ( res )  
        StackClusterPrivate_SetResult ( stclpp ,& res->plo ) ;
    } // res
    PassagePassOut ( passageGCCPub ) ; 
  } // ss
  return   TaskForThread_Analyse  ;
}

TaskForThread bimstreamstat ( StackClusterPrivate * const stclp  ,
  PLOStream * const mep , PLOStat * const resultp ) {
  if(StreamStreType(&mep -> str) == StreamTypeName(FD)) {
    struct  stat  buf ;
    if  ( fstat  (  StreamFDValue  (&mep -> str ) , &  buf  ) )       
      return   SetResultErrnoLockGC  ( stclp )  ;
    PLOStat_SetValue ( resultp , & buf ) ;       
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult ( stclp ,&resultp -> plo ) ; 
    PassagePassOut ( passageGCCPub ) ;      
    return   TaskForThread_Analyse  ; 
  } // err , buf
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_ExceptionSet  (stclp ,& PLOString_NewCopy (
    L"stream::stat: stream type not supported"  )->plo ) ; 
  PassagePassOut ( passageGCCPub ) ;    
  return   TaskForThread_Analyse  ;
}
  
//  stream stat [stat]
TaskForThread Lisp2Dbimstreamstat ( StackClusterPrivate  * const stclpp  ) {
  PLOStream * const mep  =ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop( stclpp ) ) ;
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp ) ;
  if  ( CConsp (  arglistp  ) ) {
    ParaLispObject  * const sp  = ClassCons_First(
      &ParaLispObjectToPLOCons ( arglistp )->cc  ) ;     
    PassagePassIn ( passageGCCPub ) ;
    if  ( IsNotStat ( sp , stclpp ) )    { 
      PassagePassOut ( passageGCCPub ) ;
      return   TaskForThread_Analyse  ; 
    }
    PassagePassOut ( passageGCCPub ) ;    
    return  bimstreamstat ( stclpp , mep ,
      ParaLispObjectToPLOStat ( sp ) ) ; 
  }
    if(StreamStreType(&mep ->str) == StreamTypeName(FD)) {
      struct  stat  buf ;
        if  ( fstat  ( StreamFDValue  (&mep ->str ) , &  buf  ) ) 
          return   SetResultErrnoLockGC  ( stclpp )  ;         
      PassagePassIn ( passageGCCPub ) ;
        { PLOStat * const  res = NewStat ( stclpp , & buf  ) ;
          if  ( res )
            StackClusterPrivate_SetResult (stclpp ,&  res->plo ) ; 
        }
      PassagePassOut ( passageGCCPub ) ;        
      return   TaskForThread_Analyse  ; 
    } // err , buf
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_ExceptionSet  (stclpp ,& PLOString_NewCopy (
    L"stream::stat: stream type not supported"  )->plo ) ;
  PassagePassOut ( passageGCCPub ) ;      
  return   TaskForThread_Analyse  ; 
}

void  OpenModes_In  ( OpenModes * const omp ) {
  omp -> flagsFD = ((omp -> flagsFD) bitand (~ O_ACCMODE)) bitor O_RDONLY  ;
  omp -> flagread  = true  ; 
}
    
void  OpenModes_Out ( OpenModes * const omp ) {
  omp->flagsFD = (omp->flagsFD bitand (~ O_ACCMODE)) bitor O_WRONLY  ;
  omp->flagwrite = true  ; 
}    

void  OpenModes_InOut ( OpenModes * const omp ) {
  omp->flagsFD = (omp->flagsFD bitand (~ O_ACCMODE)) bitor O_RDWR  ;
  omp->flagread  = true  ;
  omp->flagwrite = true  ; 
}
      
void OpenModes_Trunc   ( OpenModes * const omp ) {     
  omp->flagsFD  or_eq O_TRUNC ; 
}

void OpenModes_App   ( OpenModes * const omp ) {           
  omp->flagsFD  or_eq O_APPEND ; 
}
      
void OpenModes_Ate ( OpenModes * const omp ) {                 
  omp->flagatend = true  ; 
}    
      
void OpenModes_Async ( OpenModes * const omp ) {      
  omp->flagsFD  or_eq O_ASYNC ; 
}
      
void OpenModes_Cloexec ( OpenModes * const omp ) {      
  omp->flagsFD  or_eq O_CLOEXEC ; 
}
      
void OpenModes_Creat ( OpenModes * const omp ) {      
  omp->flagsFD  or_eq O_CREAT ;
}    

void  OpenModes_RwxUsr(OpenModes * const omp )  {  
  omp ->modeFD  or_eq S_IRWXU ; 
}
      
void  OpenModes_RUsr(OpenModes * const omp )  {        
  omp->modeFD  or_eq S_IRUSR ; 
}
      
void  OpenModes_WUsr(OpenModes * const omp )  {      
  omp->modeFD  or_eq S_IWUSR ; 
}
      
void  OpenModes_XUsr(OpenModes * const omp )  {      
  omp->modeFD  or_eq S_IXUSR ; 
}  
      
void  OpenModes_RwxGrp(OpenModes * const omp ) {
  omp->modeFD  or_eq S_IRWXG ; 
}

void  OpenModes_RGrp(OpenModes * const omp) {      
  omp->modeFD  or_eq S_IRGRP ; 
}
      
void  OpenModes_WGrp(OpenModes * const omp ) {      
  omp->modeFD  or_eq S_IWGRP ; 
}
      
void  OpenModes_XGrp(OpenModes * const omp ) {      
  omp->modeFD  or_eq S_IXGRP ; 
} 
      
void  OpenModes_RwxOth(OpenModes * const omp) {      
  omp->modeFD  or_eq S_IRWXO ;  
}
      
void  OpenModes_ROth(OpenModes * const omp) {      
  omp->modeFD  or_eq S_IROTH ;  
}

void  OpenModes_WOth(OpenModes * const omp) {      
  omp->modeFD  or_eq S_IWOTH ;  
}
      
void  OpenModes_XOth(OpenModes * const omp) {      
  omp->modeFD  or_eq S_IXOTH ; 
}  
      
void  OpenModes_Suid(OpenModes * const omp ) {      
  omp->modeFD  or_eq S_ISUID ; 
}
      
void  OpenModes_Sgid(OpenModes * const omp) {      
  omp->modeFD  or_eq S_ISGID ; 
}
      
void  OpenModes_Svtx(OpenModes * const omp) {      
  omp->modeFD  or_eq S_ISVTX ; 
}  

void  OpenModes_Direct(OpenModes * const omp) {      
  omp->flagsFD  or_eq __O_DIRECT ; 
}
      
void  OpenModes_Directory(OpenModes * const omp ) {      
  omp->flagsFD  or_eq O_DIRECTORY ;
}
      
void  OpenModes_Dsync(OpenModes * const omp) {      
  omp->flagsFD  or_eq O_DSYNC ;
}
      
void  OpenModes_Excl(OpenModes * const omp) {      
  omp->flagsFD  or_eq O_EXCL ; 
}
      
void  OpenModes_Largefile(OpenModes * const omp) {      
  omp->flagsFD  or_eq __O_LARGEFILE ; 
}

void  OpenModes_Noatime(OpenModes * const omp) {       
  omp->flagsFD  or_eq __O_NOATIME ;
}
      
void  OpenModes_Noctty(OpenModes * const omp) {      
  omp->flagsFD  or_eq O_NOCTTY ; 
}
      
void  OpenModes_Nofollow(OpenModes * const omp) {      
  omp->flagsFD  or_eq O_NOFOLLOW ;  
}
      
void  OpenModes_NonblockOrNdelay(OpenModes * const omp) {            
  omp->flagsFD  or_eq O_NONBLOCK ; 
}  
      
void  OpenModes_Path(OpenModes * const omp) {      
  omp->flagsFD  or_eq __O_PATH ; 
}
      
void  OpenModes_Sync(OpenModes * const omp) {      
  omp->flagsFD  or_eq O_SYNC ;  
}

void  OpenModes_Tmpfile(OpenModes * const omp) {      
  omp->flagsFD  or_eq __O_TMPFILE ;
}

# include <sys/ioctl.h>
  
//  stream getwinsize
TaskForThread Lisp2Dbimstreamgetwinsize (
  StackClusterPrivate  * const stclpp  ) {
  PLOStream * const mep  = ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop( stclpp ) ) ;
  if  ( StreamStreType(&mep->str) not_eq StreamTypeName(FD) ) {
    ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    String  ( WideChar ) * const wsp = StackNew_StringWideChar ( ) ;
    StringConstrCStr ( WideChar )  ( wsp , L"notfd" , true ) ;
    StackClusterPrivate_SetResult (stclpp , SymbolAddMoveString ( wsp ) ) ;
    Lisp2D_StackFree  ( oldstack ) ;
    return TaskForThread_Analyse  ;
  }
  struct winsize ws ; 
  int res = ioctl ( StreamFDValue (&mep ->str ) , TIOCGWINSZ , & ws ) ;
  if(res){
    StackClusterPrivate_SetResult (stclpp ,& SymbolFromIntErrno ( res ) ->
      plo ) ;
    return TaskForThread_Analyse  ; 
  }
  PLOComplex  * const reswp  =NewComplexBareExc ( )  ;
  PLOComplex  * const reshp  =NewComplexBareExc ( )  ;
  FractionSet0(Complex_Im(PLOComplexToComplex(reswp)));
  FractionSet0(Complex_Im(PLOComplexToComplex(reshp)));
  FractionSetUInt ( Complex_Re  (PLOComplexToComplex(reswp) )
    , ws . ws_col ) ;
  FractionSetUInt ( Complex_Re  (PLOComplexToComplex(reshp) )
    , ws . ws_row ) ;
  ParaLispObject  * list [ ] = { & reswp->plo , & reshp->plo , NULL } ;
  StackClusterPrivate_SetResult(stclpp ,ListOfPLO(
    (PArrayPLOP){(ParaLispObject*(*)[])&list}));
  
  return TaskForThread_Analyse  ; 
}
      
// stream setwinsize x y !!! not works  
// ioctl ( me . FDValue ( ) , TIOCSWINSZ , & ws ) ;

# define  castStreamTypeToSInt Lisp2D_castStreamTypeToSInt
inline  signed int castStreamTypeToSInt ( StreamType s ) ;

PLOStream const * PLOStreamPushStreamType ( PLOStream  const * const mep ,
  StreamType const st ) {
  return PLOStreamPushSInt ( mep , castStreamTypeToSInt  ( st ) ) ;
}

PLOStream const * PLOStreamPushUInt  ( PLOStream  const * const mep ,
  unsigned int const  ui ) {
  if  ( StreamStreType  ( & mep ->  str ) ==  StreamTypeName  ( No  ) )  
    return  mep ;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    inErrorS  (     
      PLOStreamPushPLO  ( & Lisp2Dferr ,  & mep ->  plo ) ;
      PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (  wchar_t const ( * ) [ ] ) &
          L" is not writeable"  } ) ; )
    return  mep ;
  }
  if ( StreamStreType  ( & mep ->  str ) == StreamTypeName  ( FD  ) ) {
    ThreadStackPointerType const oldstack_str = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const strtmp = StackNew_StringChar ( ) ;
    if ( sizeof ( unsigned int ) not_eq 4 and
      sizeof ( unsigned int ) not_eq 8 ) {
      Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
        L"PLOStreamPushUInt:unknown size of unsigned int" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    }
    // ceil ( log ( 10 , 2 ^ 32 ) ) + 1 = 11
    // ceil ( log ( 10 , 2 ^ 64 ) ) + 1 = 21
    enum  { busize  = ( sizeof ( unsigned int ) == 4 ? 11 : 21 ) } ;
    char  strnum  [ busize  ] ;
    int const l = snprintf  ( & strnum  [ 0 ] , busize  , "%u"  , ui  ) ;
    if  ( l <= 0 or l >= busize  ) {
      Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
        L"PLOStreamPushUInt:l <= 0 or l >= busize" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    }
    StringConstrCStr ( Char ) ( strtmp , strnum , true ) ;
    { PLOSymbol * const symbolerror = isBADPushCStringToStreamFD  ( 
        StreamPointerToConstFD  ( & mep ->  str ) , strtmp ) ;
      if  ( symbolerror ) 
        inErrorS  ( PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
            ( PArray_wchar_c ) {
              ( wchar_t const ( * ) [ ] ) & L"PLOStreamPushUInt:" } ) ;
        PLOStreamPushPLO  ( & Lisp2Dferr ,  & symbolerror ->  plo ) ; )
    }
    Lisp2D_StackFree  ( oldstack_str ) ;
    // string  tmp  ;
  } else {
    inErrorS  ( PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (  wchar_t const ( * ) [ ] ) &
          L"PLOStreamPushUInt: unknow stream type: " } ) ;
      PLOStreamPushStreamType ( & Lisp2Dferr ,
        StreamStreType  ( & mep ->  str ) ) ; )
  }
  return  mep ;
}

PLOStream const * PLOStreamPushSInt  ( PLOStream  const * const mep ,
  signed int const  si ) {
  if  ( StreamStreType  ( & mep ->  str ) ==  StreamTypeName  ( No  ) )  
    return  mep ;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    inErrorS  (     
      PLOStreamPushPLO  ( & Lisp2Dferr ,  & mep ->  plo ) ;
      PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (  wchar_t const ( * ) [ ] ) &
          L" is not writeable"  } ) ; )
    return  mep ;
  }
  if ( StreamStreType  ( & mep ->  str ) == StreamTypeName  ( FD  ) ) {
    ThreadStackPointerType const oldstack_str = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const strtmp = StackNew_StringChar ( ) ;
    if ( sizeof ( signed int ) not_eq 4 and sizeof ( signed int ) not_eq 8 ) {
      Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
        L"PLOStreamPushSInt:unknown size of signed int" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    }
    // ceil ( log ( 10 , 2 ^ 31 ) ) + 2 = 12
    // ceil ( log ( 10 , 2 ^ 63 ) ) + 2 = 21
    enum  { busize  = ( sizeof ( signed int ) == 4 ? 12 : 21 ) } ;
    char  strnum  [ busize  ] ;
    int const l = snprintf  ( & strnum  [ 0 ] , busize  , "%d"  , si  ) ;
    if  ( l <= 0 or l >= busize  ) {
      Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
        L"PLOStreamPushSInt:l <= 0 or l >= busize" ; 
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    }
    StringConstrCStr ( Char ) ( strtmp , strnum , true ) ;
    { PLOSymbol * const symbolerror = isBADPushCStringToStreamFD  ( 
        StreamPointerToConstFD  ( & mep ->  str ) , strtmp ) ;
      if  ( symbolerror ) 
        inErrorS  ( PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
            ( PArray_wchar_c ) {
              ( wchar_t const ( * ) [ ] ) & L"PLOStreamPushSInt:" } ) ;
        PLOStreamPushPLO  ( & Lisp2Dferr ,  & symbolerror ->  plo ) ; )
    }
    Lisp2D_StackFree  ( oldstack_str ) ;
    // string  tmp  ;
  } else {
    inErrorS  ( PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (  wchar_t const ( * ) [ ] ) &
          L"PLOStreamPushSInt: unknow stream type: " } ) ;
      PLOStreamPushStreamType ( & Lisp2Dferr ,
        StreamStreType  ( & mep ->  str ) ) ; )
  }
  return  mep ;
}
  
# define  castEvalAddrCont2ToSInt Lisp2D_castEvalAddrCont2ToSInt
inline  signed int castEvalAddrCont2ToSInt ( EvalAddrCont2 ea ) ;
  
PLOStream const * PLOStreamPushEvalAddrCont  ( PLOStream  const * const mep ,
  EvalAddrCont2 const lv ) {
  return PLOStreamPushSInt ( mep , castEvalAddrCont2ToSInt  ( lv  ) ) ;
}

PLOStream const * PLOStreamPushThreadIdC  ( PLOStream  const * const mep ,
  pthread_t const pxv ) {
  if(StreamStreType(&mep->str)  ==  StreamTypeName(No))
    return  mep;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    inErrorS(
      PLOStreamPushPLO(& Lisp2Dferr ,&mep->plo);
      PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        (PArray_wchar_c) { (wchar_t const(*)[])&  L" is not writeable"});)
    return  mep;
  }
  switch(StreamStreType(&mep->str)){
  case   ( StreamTypeName(FD)  ) :
    { ThreadStackPointerType const oldstack_str = Lisp2DThreadSP ( ) ;
      String(Char) * const strtmp = StackNew_StringChar ( ) ;
      if ( sizeof ( pthread_t ) not_eq 8 and
        sizeof ( pthread_t ) not_eq 4 ) {
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
          L"PLOStreamPushThreadIdC:unknown size of pthread_t" ; 
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ; }
      // 8 * 2 + 3 = 19
      // 4 * 2 + 3 = 11
      enum  { busize  = ( sizeof ( pthread_t ) == 8 ? 19 : 11 ) } ;
      char  strnum  [ busize  ] = "0x" ;
      int const l = snprintf  ( & strnum  [ 2 ] , busize - 2  , "%lx"  ,
        pxv ) ;
      if  ( l <= 0 or l >= busize - 2 ) {
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
          L"PLOStreamPushThreadIdC:l <= 0 or l >= busize - 2" ; 
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
      }
      StringConstrCStr ( Char ) ( strtmp , strnum , true ) ;
      { PLOSymbol * const symbolerror = isBADPushCStringToStreamFD  ( 
          StreamPointerToConstFD  ( & mep->str ) , strtmp ) ;
        if  ( symbolerror ) 
          inErrorS(PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        (PArray_wchar_c) { (wchar_t const(*)[])&
          L"PLOStreamPushThreadIdC:"});
          PLOStreamPushPLO(& Lisp2Dferr ,&(symbolerror->plo));)
      }
      Lisp2D_StackFree  ( oldstack_str ) ;
    } // std ::  string  st  ;
    break;
  case  StreamTypeName(No)  :
  default:
    inErrorS(PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        (PArray_wchar_c) { (wchar_t const(*)[])&
          L"PLOStreamPushThreadIdC: unknow stream type: "});
    PLOStreamPushStreamType(& Lisp2Dferr ,StreamStreType(&mep->str));)
  }
  return  mep ;
}

PLOStream const * PLOStreamPushUSize  ( PLOStream  const * const mep ,
  size_t const  xv ) {
  if(StreamStreType(&mep->str)  ==  StreamTypeName(No)) 
    return  mep;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    inErrorS  (
      PLOStreamPushPLO(& Lisp2Dferr ,&mep->plo);
      PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (wchar_t const(*)[])& L" is not writeable"});)
    return  mep ;
  }
  switch(StreamStreType(&mep->str)){
  case    StreamTypeName(FD)   :
    { ThreadStackPointerType const oldstack_str = Lisp2DThreadSP ( ) ;
      String(Char) * const strtmp = StackNew_StringChar ( ) ;
      if ( ( sizeof ( size_t ) not_eq 8 ) and ( sizeof ( size_t ) not_eq 4 ) ) {
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
          L"PLOStreamPushUSize:unknown size of size_t" ; 
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ; }
      // ceil ( log ( 10 , 2 ^ 64 ) ) + 1 = 21
      // ceil ( log ( 10 , 2 ^ 32 ) ) + 1 = 11
      enum  { busize  = ( sizeof ( size_t ) == 8 ? 21 : 11 ) } ;
      /* ! to static _Thread*/ char  strnum  [ busize  ] ;
      int const l = snprintf  ( & strnum  [ 0 ] , busize  , "%zu"  , xv  ) ;
      if  ( l <= 0 or l >= busize  ) {
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
          L"PLOStreamPushUSize:l <= 0 or l >= busize" ; 
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
      }
      StringConstrCStr ( Char ) ( strtmp , strnum , true ) ;
      { PLOSymbol * const symbolerror = isBADPushCStringToStreamFD  ( 
          StreamPointerToConstFD  ( & mep->str ) , strtmp ) ;
        if  ( symbolerror )   
          inErrorS  ( PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
              ( PArray_wchar_c ) { (wchar_t const(*)[])&
                L"PLOStreamPushUSize:"});
            PLOStreamPushPLO(& Lisp2Dferr ,&(symbolerror->plo));)
      }
      Lisp2D_StackFree  ( oldstack_str ) ;
    } // strtmp
    break;
  case   StreamTypeName(No)   :
  default :
    inErrorS(PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (wchar_t const(*)[])& 
          L"PLOStreamPushUSize: unknow stream type: "});
      PLOStreamPushStreamType(& Lisp2Dferr ,StreamStreType(&mep->str));)  }
  return  mep ; }

// ! to #define D(stream,long double const * ld,*ld)
PLOStream const * PLOStreamPushLDouble  ( PLOStream  const * const mep ,
  long double const ldv ) {
  if(StreamStreType(&mep->str)  ==  StreamTypeName(No)) 
    return  mep;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    inErrorS(
      PLOStreamPushPLO(& Lisp2Dferr ,&mep->plo);
    PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (wchar_t const(*)[])& L" is not writeable"});)
    return  mep;
  }
  switch(StreamStreType(&mep->str)){
  case    StreamTypeName(FD)   :
    { ThreadStackPointerType const oldstack_str = Lisp2DThreadSP ( ) ;
      String(Char) * const strtmp = StackNew_StringChar ( ) ;
      // mantissa size = 8 bytes , 64 bits
      // log ( 10 , 2 ^ 64 ) = 19.3 digits
      enum  { busize  = 30 } ;
      char  strnum  [ busize  ] ;
      int const l = snprintf  ( & strnum  [ 0 ] , busize  , "%.19Lg"  , ldv ) ;
      if  ( l <= 0 or l >= busize  ) {
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
          L"PLOStreamPushLDouble:l <= 0 or l >= busize" ; 
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
      }
      StringConstrCStr ( Char ) ( strtmp , strnum , true ) ;
      { PLOSymbol * const symbolerror = isBADPushCStringToStreamFD  ( 
          StreamPointerToConstFD  ( & mep->str ) , strtmp ) ;
        if  ( symbolerror ) {
          inErrorS(PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (wchar_t const(*)[])& L"PLOStreamPushLDouble:"});
          PLOStreamPushPLO(& Lisp2Dferr ,&(symbolerror->plo));)
          Lisp2D_StackFree  ( oldstack_str ) ;
          break ;
        }
      }
      Lisp2D_StackFree  ( oldstack_str ) ;
    } // strtmp
    break;
  case    StreamTypeName(No)   :
  default:
    inErrorS(PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (wchar_t const(*)[])& 
          L"PLOStreamPushLDouble: unknow stream type: "});
    PLOStreamPushStreamType(& Lisp2Dferr ,StreamStreType(&mep->str));)  }
  return  mep;
}

// stream read-line [place]
TaskForThread Lisp2Dbimstreamreadline2 ( StackClusterPrivate  * const stclpp ) {
  PLOStream * const mep = ParaLispObjectToPLOStream ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ParaLispObject  *   place=  CCAR(StackClusterPrivate_ArgTop(stclpp ));
  if(StreamReadable(&mep->str)){
    StreamType const streamtype = StreamStreType(&mep->str);
    if( streamtype == StreamTypeName(FD)){
      ThreadStackPointerType const oldstack0 = Lisp2DThreadSP ( ) ;
      String  ( Char ) * const buftmpc = StackNew_StringChar ( ) ;
      String_Init ( Char  ) ( buftmpc , true ) ;
      // read all chars up to the \n or \0000 or EOF
      do {
        char  cha = 0 ;
        ssize_t const  readsize  =
          read  (  StreamFDValue  (&mep  ->str ) , & cha , 1U  ) ;
        if  ( readsize  ==  -1  ) {
          Lisp2D_StackFree  ( oldstack0 ) ;
          return   SetResultErrnoLockGC  ( stclpp )  ;
        }
        if  ( readsize  ==  0 ) {
          StreamSetEofFD  (&mep  ->str ) ;
          break;
        }
        if  ((cha == 0) or (cha == '\n'))   
          break;
        String_AddChar  ( Char  ) ( buftmpc , cha ) ;
     } while ( true ) ;
     if ( String_Empty  ( Char  ) ( buftmpc ) ) {
        if(StreamEof(&mep->str)){   
          Lisp2D_StackFree  ( oldstack0 ) ;
          PassagePassIn ( passageGCCPub ) ;
          StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
          PassagePassOut ( passageGCCPub ) ;          
          return  TaskForThread_Analyse;
        }
        if  ( Stringp ( place ) )
          StringClear ( WideChar )  ( PLOString_Value2  ( ParaLispObjectToPLOString ( place ) ) ) ;
        else  {
          PLOStringPub  * const sp = PLOString_New ( ) ;
          place = & sp -> plo ;
        }
        PassagePassIn ( passageGCCPub ) ;
        StackClusterPrivate_SetResult ( stclpp , place ) ;
        PassagePassOut ( passageGCCPub ) ;
      }
      String  ( WideChar ) * const buftmp2c = StackNew_StringWideChar ( ) ;
      String_Init ( WideChar  ) ( buftmp2c , true ) ;
      if ( isBADReadCStringToWString2 ( buftmpc , buftmp2c ) ) {
        Lisp2D_StackFree  ( oldstack0 ) ;
        PassagePassIn ( passageGCCPub ) ;
        StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
        PassagePassOut ( passageGCCPub ) ;
        return  TaskForThread_Analyse ; 
      }
      if ( Stringp ( place ) )
        PLOStringMoveString ( ParaLispObjectToPLOString ( place ) , buftmp2c ) ;
      else {    
        PLOStringPub * const  places2 = PLOString_NewMoveString ( buftmp2c ) ;
        { jmp_buf jump_buffer ;
          jmp_buf * const oldjb = Lisp2DJBglobal  ;
          Lisp2DJBglobal = &  jump_buffer ;
          int const volatile exctype = setjmp  ( jump_buffer ) ;
          if  ( exctype ) {
            Lisp2DJBglobal = oldjb  ;
            Lisp2D_StackFree  ( oldstack0 ) ;
            longjmp ( * Lisp2DJBglobal , exctype ) ;
          }
          Lisp2DJBglobal = oldjb  ;
          place = & places2 -> plo ;
          PassagePassIn ( passageGCCPub ) ;
          StackClusterPrivate_SetResult ( stclpp , place  ) ;
          PassagePassOut ( passageGCCPub ) ;
        }
      }
      Lisp2D_StackFree  ( oldstack0 ) ;
      PassagePassIn ( passageGCCPub ) ;
      StackClusterPrivate_SetResult(stclpp ,  place);
      PassagePassOut ( passageGCCPub ) ;      
      return  TaskForThread_Analyse ; 
    } // type == FD
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_ExceptionSet(stclpp ,&PLOString_NewCopy(
      L"stream::read-line: stream type is unknown")->plo);
    PassagePassOut ( passageGCCPub ) ;
    return  TaskForThread_Analyse;   
  }
  PassagePassIn ( passageGCCPub )  ;
  PLOStringPub * const  st  = PLOString_NewCopyArr ( (PArray_wchar_c){(wchar_t const(*)[])&L" is not readable"} ) ;
  { ParaLispObject  * list [ ] = { &  mep ->  plo , & st  -> plo , 0 } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){ (ParaLispObject*(*)[])&list}));
  } // list
  PassagePassOut ( passageGCCPub ) ;  
  return  TaskForThread_Analyse;    
}

# define  PLOCoutFun  Lisp2D_PLOCoutFun
typedef void  PLOCoutFun ( ParaLispObject  const * me  ,
  PLOStream const * s , bool  const * flag2pprinted , Vector  ( StackPLOP ) * historyp ) ;

# define  bimstreamwriteprinc Lisp2D_bimstreamwriteprinc
// stream write/princ [{x}]
static  TaskForThread bimstreamwriteprinc ( PLOStream * const mep ,
  StackClusterPrivate * const stclp , PLOCoutFun fp ) {
  ParaLispObject  * j ;
  if  ( StackClusterPrivate_ClearflagCont ( stclp ) ) {
    StackClusterPrivate_Pop ( stclp ,&j )  ;
    goto  Next ; 
  }
  j = StackClusterPrivate_ArgTop (stclp );
  if(StackClusterPrivate_exceptionnotprocessed  ( stclp )) 
    return  TaskForThread_Analyse;
  if  ( not  StreamWriteable  ( & mep  -> str ) ) {
    PassagePassIn ( passageGCCPub ) ;
    ParaLispObject  * list [ ] = { &  mep->plo,  &  PLOString_NewCopy(
      L" is not writeable")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));  
    PassagePassOut ( passageGCCPub ) ;    
    return  TaskForThread_Analyse;
  }
// ! to remove history limit    
Next  :
  { ThreadStackPointerType const oldstack_myp = Lisp2DThreadSP ( ) ;
    Vector  ( StackPLOP ) * const historyp = Lisp2D_StackVar ( Lisp2DStackTypeStackPLOP ) ;
    VectorInit  ( StackPLOP ) ( historyp , false ) ;
    { unsigned  int tasks = maxListItemProcessing ;
      for ( ; ( tasks  > 0  ) and CConsp (  j ) ; --  tasks ) {
        bool  const ffalse  = false ;
        fp ( ClassCons_First ( & ParaLispObjectToPLOCons ( j ) ->  cc  ) , mep ,
          & ffalse , historyp ) ;
        j = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( j )->cc ) ; 
      } 
    }
    Lisp2D_StackFree  ( oldstack_myp ) ;
  } // history
  if(CConsp(  j)){
    StackClusterPrivate_setTaskForManager ( stclp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclp , 1U  ) ;
    StackClusterPrivate_Push ( stclp , j ) ;
    return  TaskForThread_Analyse;
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclp ,&mep->plo);
  PassagePassOut ( passageGCCPub ) ;  
  return  TaskForThread_Analyse;
}  
  
// stream write [{x}]
TaskForThread Lisp2Dbimstreamwrite2(StackClusterPrivate * const stclpp){
  PLOStream * const mep  = ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop ( stclpp ) ) ;
  return  bimstreamwriteprinc(mep,stclpp , ParaLispObject_CoutToHuman ) ; 
}
  
// stream writeln [{x}]
TaskForThread Lisp2Dbimstreamwriteln2(StackClusterPrivate * const stclpp){
  // ? if Analyse
  Lisp2Dbimstreamwrite2(stclpp);
  PLOStreamEndl ( ParaLispObjectToPLOStream(StackClusterPrivate_ThisTop( stclpp )));
  return  TaskForThread_Analyse;
}
  
// stream princ [{x}]
TaskForThread Lisp2Dbimstreamprinc2(StackClusterPrivate * const stclpp){
  PLOStream * const mep  = ParaLispObjectToPLOStream (
    StackClusterPrivate_ThisTop ( stclpp )  ) ;
  return  bimstreamwriteprinc(mep,stclpp , ParaLispObject_CoutToInterpret ) ;
}
  
// stream print [{x}]
TaskForThread Lisp2Dbimstreamprint2(StackClusterPrivate * const stclpp){
  Lisp2Dbimstreamprinc2(stclpp);
  // ? if error
  PLOStreamEndl ( ParaLispObjectToPLOStream ( StackClusterPrivate_ThisTop ( stclpp )  ) ) ;
  return  TaskForThread_Analyse ;
}

LISP2DMAPPUBINSERTBODY  ( FlessOpenModes )
LISP2DMAPPUBINSERTBODY  ( FlessMkdirModes )

void  OpenModes_Init ( OpenModes * const me ) {
  me -> flagread  = false ;
  me -> flagwrite = false ;
  me -> flagsFD = 0 ;
  me -> modeFD  = 0 ;
  me -> flagatend = false ;
}

LISP2DMAPSTACKATBODY ( FlessOpenModes )
LISP2DMAPSTACKATBODY ( FlessMkdirModes )

void  PLOStreamInit ( PLOStream * const me ) {
  StreamInit  ( & me  ->  str ) ;
  ParaLispObject_InitVTP  ( & me  ->  plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOStream } ) ;
}

LISP2DMAPSTACKPUSHFRONTVALUEBODY ( FlessOpenModes )
LISP2DMAPSTACKPUSHFRONTVALUEBODY ( FlessMkdirModes )
LISP2D_PROTO_MAP_STACK_FREE ( FlessOpenModes )
LISP2DMAPSTACKDESTRBODY ( FlessOpenModes )
LISP2D_MAP_STACK_FREE_BODY ( FlessOpenModes )
LISP2D_PROTO_MAP_STACK_FREE ( FlessMkdirModes )
LISP2DMAPSTACKDESTRBODY ( FlessMkdirModes )
LISP2D_MAP_STACK_FREE_BODY ( FlessMkdirModes )

PLOStream const * PLOStreamPushStringWideChar ( PLOStream  const * const sp  , String  ( WideChar  )  const * const ws ) {
  StreamPushWCharArray ( & sp ->  str , Lisp2DStringWideCharCArr ( ws  ) ) ;
  return sp  ;
}

# include "castb.h"
# include "widestring/inline.h"

inline  signed int castStreamTypeToSInt ( StreamType const s ) {
  return  ( signed int  ) s ;
}

inline  signed int castEvalAddrCont2ToSInt ( EvalAddrCont2 const ea ) {
  return  ( signed int  ) ea ;
}
# include "stat/inline.h"
