// Random value
// ©2019-23 by Alex Glebe

# include "random/class.h" // RandomInit
# include "random/next.h" // RandomNext2
Mutex2  RandomMutex ;
# include "define.h"
# ifdef LISP2D_DEBUG
bool  globalrandomstateOk = false ;
# endif
# include <math.h> // ldexpl
# include <iso646.h> // not
# include <stdio.h> // stderr
# include <stdlib.h>  // exit
# include <time.h>
# include "castb.h"

# define  LISP2D_RANDOM_getrandom
//# define  LISP2D_RANDOM_rand
// ! declare in Makefile USE_GNU_SOURCE = -D'_GNU_SOURCE'
//# define  LISP2D_RANDOM_syscall

# ifdef  LISP2D_RANDOM_rand

# include <stdint.h>

static  ssize_t Lisp2D_getrandom  ( void  * const buf , size_t  const len ) {
  static  bool  first = true  ;
  if ( first  ) {
    if  ( sizeof  ( unsigned  int ) ==  4 )
      srand ( lint_cast_ulint ( time  ( 0 ) ) % 0x100000000UL ) ;
    else  if  ( sizeof  ( unsigned  int ) ==  2 )
      srand ( lint_cast_ulint ( time  ( 0 ) ) % 0x10000UL ) ;
    else
      srand ( ulint_cast_uint ( lint_cast_ulint ( time  ( 0 ) ) ) ) ;
    first = false ;
  }
  uint8_t * p = buf ;
  size_t j = len ;
  while ( j ) {
    -- j ;
    * p = int_cast_uint8 ( rand  ( ) bitand  0xff ) ;    
    ++ p ;
  }
  return ulint_cast_lint ( len ) ;
}
# else
# ifdef  LISP2D_RANDOM_getrandom
# include <sys/random.h> // getrandom
// OR
// # include <linux/random.h> // getrandom
static  ssize_t Lisp2D_getrandom  ( void  * const buf , size_t  const len ) {
  return getrandom ( buf , len , 0 ) ;
}
# else
# ifdef  LISP2D_RANDOM_syscall
# include <unistd.h>
# include <sys/syscall.h> // syscall getrandom
static  ssize_t Lisp2D_getrandom  ( void  * const buf , size_t  const len ) {
  return  syscall ( SYS_getrandom , buf , len , 0 ) ;
}
# else
# error LISP2D_RANDOM no rand no getrandom no syscall
# endif
# endif
# endif

# include <wchar.h>
# include "mutex2/public.h"

// ! to put in Fraction
long  double  RandomNext2 ( void  ) {
# ifdef LISP2D_DEBUG  
  if  ( not globalrandomstateOk ) {
    if  ( fputws  ( L"RandomNext2 : not globalrandomstateOk\n"  , stderr  ) < 0 )
      exit  ( 1 ) ;
    RandomInit  ( ) ; 
  }
# endif
  static  _Thread_local uint8_t buf [ 8 ] = { 0 } ;
  Mutex2Lock  ( & RandomMutex ) ;
  ssize_t const r = Lisp2D_getrandom  ( & ( buf [ 0 ] ) , 8 ) ;
  Mutex2Unlock  ( & RandomMutex ) ;
  
  if ( r == -1L ) {
    perror  ( "getrandom" ) ;
    return  0 ; 
  }
  
  long  double  result =  0.5L  ;
  // push 63 bits , last bit is 1 : [ r r r 1 ]
  { int i = 0 ;
    do {
      result  = ldexpl  ( result  , -1  ) ;
      if ( ( buf [ i >> 3 ] >> ( i % 8 ) ) bitand 1 )
        result  +=  0.5L  ;
      ++ i ;
    } while ( i < 63 ) ; 
  } // i

  return  result  ; 
}
  
void  RandomInit  ( void ) {
# ifdef LISP2D_DEBUG
  if  ( not globalrandomstateOk ) {
# endif    
    Mutex2SetStateRaw  ( & RandomMutex ) ;
    Mutex2Constructor ( & RandomMutex , 0 ) ;
# ifdef LISP2D_DEBUG
    globalrandomstateOk = true  ; 
  } else {
    if  ( fputws  ( L"RandomInit : double initialization\n"  , stderr  ) < 0 )
      exit  ( 2 ) ; 
  }
# endif
}

void  RandomDestr  ( void ) {
# ifdef LISP2D_DEBUG
  if  ( not globalrandomstateOk ) {
    if  ( fputws  ( L"RandomDestr : double destruction\n"  , stderr  ) < 0 )
      exit  ( 3 ) ; 
  }
# endif
  Mutex2Destructor ( & RandomMutex ) ;
# ifdef LISP2D_DEBUG
  globalrandomstateOk = false ;
# endif 
}

# include "random/bim.h"
# include "plocomplex/public.h"
# include "stcl/classc.h"
# include "stcl/public.h"
# include "stcl/inline.h"

//  nil random
TaskForThread bimnilrandom  ( StackClusterPrivate  * const stclpp ) {
  { PLOComplex  * const res = NewComplexBareExc ( ) ;
    Complex_SetExactFromLD  ( PLOComplexToComplex ( res ) , RandomNext2 ( ) ) ; 
    StackClusterPrivate_SetResult ( stclpp  , & res ->  plo ) ;
  }
  return  TaskForThread_Analyse ; 
}

# ifdef LISP2D_BIMSTAT

# include "random/stat.h"

size_t  RandomStatWait  ( void )  {
  return  Mutex2StatWait ( & Lisp2DRandomMutex ) ;
}
# endif // LISP2D_BIMSTAT
