// Lisp2D - create BuildInMethods
// ©2014-23 by Alex Glebe

# include "define.h"
# include "inerrors.h"
# include "stringbim.h"
# include "boolean.h"
# include "setbims.h"
# include "complexbim.h"
# include "string-unix.h"
# include "host.h"
# include "newobject.h"
# include "drawline.h"
# include "drawsquare.h"
# include "signall.h"
# include "random/bim.h"
# include "object/bim.h"
# include "errorstra.h"
# include "dir/inline.h"
# include "quote/public.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "backquote/inline.h"
# include "unquotesplicing/public.h"
# include "comma/public.h"
# include "bytedeque/public.h"
# include "char/public.h"
# include "cons/inline.h"
# include "vector/public.h"
# include "lock/public.h"
# include "socket/public.h"
# include "stat/public.h"
# include "listplo/public.h"
# include "env/public.h"
# include "time/public.h"
# include "plostream/public.h"

bool  Lisp2DisOKSetBIMs ( void ) {  
  StackClusterPrivate  * const stcl = Constr ( ) ;
  NameSpaceDef  * const nsstd  = NameSpaceAssoc ( & plosstd ) ;
# ifdef  LISP2D_DEBUG
  if ( not nsstd ) {
    ErrorStra((PArray_wchar_c){(wchar_t const (*)[])& L"STD namespace is not defined"});
    return  false;
  }
# endif
  ParaLispObject  * const stdpro=  nsstd->prop;
  struct setbimargs{
// ! remove symbol , use ct = uint8_t Type...    
    wchar_t const * namec;
    int argt;
    FuncBiningP funcaddr2;
    BoolCheckThis fcheckthis  ;
    int  ct;
    BoolThisIsConst flagthisconst ;
    };
    
  { struct  setbimargs arrsetbimargs [ ] = {
    //complex
    
    {  L"number", NSBIF_ParaCalc, & bimcomplexcomplex,
      BoolCheckThis_True, TypeGlobalListMemberPLOComplex,
      BoolThisIsConst_False},
    {  L"numberp", NSBIF_ParaCalc,  & bimnilnumberp, BoolCheckThis_False,
      TypeGlobalListMemberPLONil, BoolThisIsConst_True},
    { L"realp", NSBIF_NotCalc,& bimcomplexrealp, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    {L"integerp", NSBIF_NotCalc,& bimcomplexintegerp, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    {L"-", NSBIF_ParaCalc,& bimcomplexminus, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,    BoolThisIsConst_True},
    // ! to add sinon L"÷" 
    {L"/", NSBIF_ParaCalc,& bimcomplexdivide, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    {L"/=", NSBIF_ParaCalc,& bimcomplexdividesetf, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_False},
    // ! to make copy with name "eq"
    {L"=", NSBIF_ParaCalc,& bimcomplexequal, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    // ! to make copy with name "not_eq"
    // ! to add sinon L"≠"    
    {L"<>", NSBIF_ParaCalc,& bimcomplexnotequal, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    {L"setf", NSBIF_ParaCalc,& bimcomplexsetf, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_False},
    {L"swap", NSBIF_ParaCalc,& bimcomplexswap, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_False},
    // ! to write abs=
    {L"abs", NSBIF_NotCalc,& bimcomplexabs, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    {L"<", NSBIF_ParaCalc,  & bimcomplexless, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    {L">", NSBIF_ParaCalc,  & bimcomplexmore, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    // ! to add sinon L"≤"    
    {L"<=", NSBIF_ParaCalc,& bimcomplexlessoreq, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    // ! to add sinon L"≥"  
    {L">=", NSBIF_ParaCalc,& bimcomplexmoreoreq, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    {L"infp", NSBIF_NotCalc,& bimcomplexinfp, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    {L"min", NSBIF_ParaCalc,& bimcomplexmin, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    {L"max", NSBIF_ParaCalc,& bimcomplexmax, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    { L"trunc"  , NSBIF_ParaCalc  , & bimcomplextrunc , BoolCheckThis_True  ,
      TypeGlobalListMemberPLOComplex ,
      BoolThisIsConst_True  } ,
    {L"ceil", NSBIF_ParaCalc,& bimcomplexceil, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    // ! to write ceil=
    {L"floor", NSBIF_ParaCalc,& bimcomplexfloor, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    // ! to write floor=
    {L"%", NSBIF_ParaCalc,& bimcomplexmod, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    {L"mod"  , NSBIF_ParaCalc  , & bimcomplexmod  , BoolCheckThis_True  ,
      TypeGlobalListMemberPLOComplex , BoolThisIsConst_True  } ,
    // ! to write %=
    {L"gcd", NSBIF_ParaCalc,& bimcomplexgcd, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},
    { L"sin"  , NSBIF_ParaCalc  , & bimcomplexsin , BoolCheckThis_True  ,
      TypeGlobalListMemberPLOComplex  , BoolThisIsConst_True  } ,
    // ! to write sin=
    { L"cos"  , NSBIF_ParaCalc  , & bimcomplexcos , BoolCheckThis_True  ,
      TypeGlobalListMemberPLOComplex  , BoolThisIsConst_True  } ,
    // ! to write cos=
    {L"arcsin", NSBIF_NotCalc,& bimcomplexarcsin, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    // ! to write ...=
    {L"arccos", NSBIF_NotCalc,& bimcomplexarccos, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
    {L"tan", NSBIF_NotCalc,& bimcomplextg, BoolCheckThis_True,
      TypeGlobalListMemberPLOComplex,         BoolThisIsConst_True},{
    
    
  L"arctan", NSBIF_NotCalc,& bimcomplexarctan, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"sinh", NSBIF_NotCalc,& bimcomplexsinh, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"cosh", NSBIF_NotCalc,& bimcomplexcosh, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"arsinh", NSBIF_NotCalc,& bimcomplexarsinh, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"arcosh", NSBIF_NotCalc,& bimcomplexarcosh, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"tanh", NSBIF_NotCalc,& bimcomplextanh, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"coth", NSBIF_NotCalc,& bimcomplexcoth, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"cot", NSBIF_NotCalc,& bimcomplexcot, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"artanh", NSBIF_NotCalc,& bimcomplexartanh, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  
  L"sqrt", NSBIF_ParaCalc,& bimcomplexsqrt, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"√", NSBIF_ParaCalc,& bimcomplexsqrt, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  
  L"exp", NSBIF_NotCalc,& bimcomplexexp, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"^", NSBIF_ParaCalc,& bimcomplexpow, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},
         
  { L"cbrt" , NSBIF_ParaCalc  , & bimcomplexcbrt  , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOComplex  , BoolThisIsConst_True } ,
  { L"∛"  , NSBIF_ParaCalc  , & bimcomplexcbrt  , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOComplex , BoolThisIsConst_True } ,       
    
         {
  
  L"random", NSBIF_NotCalc,& bimcomplexrandom, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"randomcomplex", NSBIF_NotCalc,& bimcomplexrandomcomplex, BoolCheckThis_True,
         TypeGlobalListMemberPLOComplex, BoolThisIsConst_True},
  { L"random-normal", NSBIF_ParaCalc,& bimcomplexrandomnormal ,
    BoolCheckThis_True,TypeGlobalListMemberPLOComplex,
    BoolThisIsConst_True},         
         {
  
    L"for", NSBIF_NotCalc, & bimcomplexfor, BoolCheckThis_True,
    TypeGlobalListMemberPLOComplex,
    BoolThisIsConst_True},{
    L"∀", NSBIF_NotCalc, & bimcomplexfor, BoolCheckThis_True,
    TypeGlobalListMemberPLOComplex,
    BoolThisIsConst_True},
    
  // ! argument [eps] -> required
  { L"log"  , NSBIF_ParaCalc  , & bimcomplexlog , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOComplex  , BoolThisIsConst_True  } ,
         
  { L"ln" , NSBIF_ParaCalc , & bimcomplexln , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOComplex  , BoolThisIsConst_True  } ,
  { L"for*", NSBIF_NotCalc,& bimcomplexforcons, BoolCheckThis_True,
    TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
// ! to add sinon L"∀*"  
  L"string", NSBIF_ParaCalc,& bimcomplexstring, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"round", NSBIF_ParaCalc,& bimcomplexround, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"!", NSBIF_NotCalc,& bimcomplexfactorial, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"char", NSBIF_NotCalc,& bimcomplexchar, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"nanp", NSBIF_NotCalc,& bimcomplexnanp, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"finitep", NSBIF_NotCalc,& bimcomplexfinitep, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"setuid", NSBIF_NotCalc,& bimcomplexsetuid, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"seteuid", NSBIF_NotCalc,& bimcomplexseteuid, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"setgid", NSBIF_NotCalc,& bimcomplexsetgid, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"setegid", NSBIF_NotCalc,& bimcomplexsetegid, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"evenp", NSBIF_NotCalc,& bimcomplexevenp, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"oddp", NSBIF_NotCalc,& bimcomplexoddp, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"uname", NSBIF_NotCalc,& bimcomplexuname, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"gid", NSBIF_NotCalc,& bimcomplexgid, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"home", NSBIF_NotCalc,& bimcomplexhome, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"shell", NSBIF_NotCalc,& bimcomplexshell, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"gname", NSBIF_NotCalc,& bimcomplexgname, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"gmem", NSBIF_NotCalc,& bimcomplexgmem, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"degrees", NSBIF_NotCalc,& bimcomplexdegrees, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"radians", NSBIF_NotCalc,& bimcomplexradians, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"num", NSBIF_NotCalc,& bimcomplexnum, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"denom", NSBIF_NotCalc,& bimcomplexdenom, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"nice", NSBIF_NotCalc,& bimcomplexnice, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"real", NSBIF_NotCalc,& bimcomplexreal, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"imag", NSBIF_NotCalc,& bimcompleximag, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"conj", NSBIF_NotCalc,& bimcomplexconj, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"polar", NSBIF_ParaCalc,& bimcomplexpolar, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         BoolThisIsConst_True},{
  L"arg", NSBIF_NotCalc,& bimcomplexarg, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},{
  L"erf", NSBIF_NotCalc,& bimcomplexerf, BoolCheckThis_True,
  TypeGlobalListMemberPLOComplex,
         
         BoolThisIsConst_True},
  {  L"iopl" , NSBIF_NotCalc , & bimcomplexiopl  , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOComplex , BoolThisIsConst_True  } ,
    
// ! to do : inb , outb , ioperm , ...    
  
// ! to write real memory access :    
// (addr get-byte) -> number 
// (addr set-byte byte) -> nil    
         {  
  
// ! to write : rounding, nano_sleep  , real part using , ...  
  
    
  // to write:
  // norm = abs^2
  // lcm,combin,combina

  // vector
  L"merge", NSBIF_ParaCalc,& Lisp2Dbimvectormerge2, BoolCheckThis_True,
    TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"elt", NSBIF_ParaCalc,& Lisp2Dbimvectorelt2, BoolCheckThis_False,
    TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"size", NSBIF_NotCalc,& Lisp2Dbimvectorsize2, BoolCheckThis_True,
    TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"reverse", NSBIF_NotCalc,& Lisp2Dbimvectorreverse2, BoolCheckThis_True,
    TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"push", NSBIF_ParaCalc,& Lisp2Dbimvectorpush2, BoolCheckThis_True,
    TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_False},
  { L"pop", NSBIF_ParaCalc,& bimvectorpop ,
    BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_False},
  {
  L"subseq", NSBIF_ParaCalc,&  Lisp2Dbimvectorsubseq2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"resize", NSBIF_ParaCalc,& Lisp2Dbimvectorresize2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_False},{
  L"clear", NSBIF_NotCalc,& Lisp2Dbimvectorclear2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_False},{
  L"setelt", NSBIF_ParaCalc,& Lisp2Dbimvectorsetelt2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_False},{
  L"transpose", NSBIF_NotCalc,& Lisp2Dbimvectortranspose2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_True},{
  L"list", NSBIF_NotCalc,& Lisp2Dbimvectorlist2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"empty", NSBIF_NotCalc,& Lisp2Dbimvectorempty2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_True},{
  L"insert", NSBIF_ParaCalc,& Lisp2Dbimvectorinsert2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_False},{
  L"setf", NSBIF_ParaCalc,& Lisp2Dbimvectorsetf2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_False},{
  
  
  L"for", NSBIF_NotCalc,& Lisp2Dbimvectorfor2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
// ! to add sinon L"∀"    
  L"for*", NSBIF_NotCalc,& Lisp2Dbimvectorforcons2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_True},{
// ! to add sinon L"∀*"  
  
// !! write (vector / number)
  

  
  L"vector", NSBIF_ParaCalc,& Lisp2Dbimvectorvector2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_False},{
  

  
  L"swap", NSBIF_ParaCalc,& Lisp2Dbimvectorswap2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_False},{
  L"vectorp", NSBIF_ParaCalc,& Lisp2Dbimnilvectorp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  L"vector", NSBIF_NotCalc,& Lisp2Dbimnilvector2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  L"first", NSBIF_NotCalc,& Lisp2Dbimvectorfirst2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_True},{
  L"last", NSBIF_NotCalc,& Lisp2Dbimvectorlast2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"map", NSBIF_ParaCalc,& Lisp2Dbimvectormap2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
  L"mapc", NSBIF_ParaCalc,& Lisp2Dbimvectormapc2, BoolCheckThis_True, TypeGlobalListMemberPLOVector,
         
         BoolThisIsConst_True},{
  L"copy-matrix", NSBIF_NotCalc,& Lisp2Dbimvectorcopymatrix2, BoolCheckThis_True,
    TypeGlobalListMemberPLOVector,
         BoolThisIsConst_True},{
//std::wclog<<" 0,7855."<<std::flush;

  // string
  L"+", NSBIF_ParaCalc,& bimstringplus, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  
  L"=", NSBIF_ParaCalc,& bimstringequal, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
// ! to add sinon L"≠"    
  L"<>", NSBIF_ParaCalc,& bimstringnotequal, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"elt", NSBIF_ParaCalc,& bimstringelt, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setelt", NSBIF_ParaCalc,& bimstringsetelt, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_False},{
  L"subseq", NSBIF_ParaCalc,& bimstringsubseq, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"size", NSBIF_NotCalc,& bimstringsize, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  
    
  
         
  L"symbol", NSBIF_NotCalc,& bimstringsymbol, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True} ,
// ? to ExtraThread 
  { L"system" , NSBIF_ParaCalc  , & bimstringsystem ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"getenv" , NSBIF_NotCalc , & bimstringgetenv  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"setenv", NSBIF_ParaCalc,& bimstringsetenv, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"unsetenv", NSBIF_NotCalc,& bimstringunsetenv, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},
  { L"accessf"  , NSBIF_NotCalc , & bimstringaccessf  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"accessx"  , NSBIF_NotCalc , & bimstringaccessx  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"accessr"  , NSBIF_NotCalc , & bimstringaccessr  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"accessw"  , NSBIF_NotCalc , & bimstringaccessw  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"setuid" , NSBIF_ParaCalc  , & bimstringsetuid ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"setgid" , NSBIF_ParaCalc  , & bimstringsetgid ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"setlgid"  , NSBIF_ParaCalc  , & bimstringsetlgid  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"setluid"  , NSBIF_ParaCalc  , & bimstringsetluid  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  ,
    BoolThisIsConst_True  } ,
  { L"chdir"  , NSBIF_NotCalc,& bimstringchdir, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"rmdir", NSBIF_NotCalc,& bimstringrmdir, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  
  L"rename", NSBIF_ParaCalc,& bimstringrename, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"ls", NSBIF_NotCalc,& bimstringls, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
    L"<", NSBIF_ParaCalc,& bimstringless, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,         BoolThisIsConst_True},{
  L">", NSBIF_ParaCalc,& bimstringmore, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,         BoolThisIsConst_True},{
// ! to add sinon L"≤"  
  L"<=", NSBIF_ParaCalc,& bimstringlessoreq, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,                  BoolThisIsConst_True},{
// ! to add sinon L"≥"    
    L">=", NSBIF_ParaCalc,  & bimstringmoreoreq, BoolCheckThis_True,
    TypeGlobalListMemberPLOString, BoolThisIsConst_True},{
  L"min", NSBIF_ParaCalc,& bimstringmin, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"max", NSBIF_ParaCalc,& bimstringmax, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"upper", NSBIF_NotCalc,& bimstringupper, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"lower", NSBIF_NotCalc,& bimstringlower, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"list", NSBIF_NotCalc,& bimstringlist, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"vector", NSBIF_NotCalc,& bimstringvector, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
   L"byte-vector", NSBIF_NotCalc, & bimstringbytevector, BoolCheckThis_True, TypeGlobalListMemberPLOString,
    BoolThisIsConst_True},{
  L"setf", NSBIF_ParaCalc,& bimstringsetf, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_False},{
  L"swap", NSBIF_ParaCalc,& bimstringswap, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_False},{
  
  
// ! to add sinon L"∀"    
  L"for", NSBIF_NotCalc,& bimstringfor, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"for*", NSBIF_NotCalc,& bimstringforcons, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
// ! to add sinon L"∀*"    
  

  
  L"vfsblocksize", NSBIF_NotCalc,& bimstringvfsblocksize, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"frsize", NSBIF_NotCalc,& bimstringfrsize, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"vfsblocks", NSBIF_NotCalc,& bimstringvfsblocks, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"bfree", NSBIF_NotCalc,& bimstringbfree, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"bavail", NSBIF_NotCalc,& bimstringbavail, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"files", NSBIF_NotCalc,& bimstringfiles, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"ffree", NSBIF_NotCalc,& bimstringffree, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"favail", NSBIF_NotCalc,& bimstringfavail, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"fsid", NSBIF_NotCalc,& bimstringfsid, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"rdonly", NSBIF_NotCalc,& bimstringrdonly, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"nosuid", NSBIF_NotCalc,& bimstringnosuid, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"namemax", NSBIF_NotCalc,& bimstringnamemax, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"string", NSBIF_ParaCalc,& bimstringstring, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_False},{
  L"resize", NSBIF_ParaCalc,& bimstringresize, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_False},{
  L"link", NSBIF_ParaCalc,& bimstringlink, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},{
  L"symlink", NSBIF_ParaCalc,& bimstringsymlink, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"readlink", NSBIF_ParaCalc,& bimstringreadlink, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         
         BoolThisIsConst_True},
         {
  L"setusrreadp", NSBIF_ParaCalc,& bimstringsetusrreadp, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setusrwritep", NSBIF_ParaCalc,& bimstringsetusrwritep, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setusrexecp", NSBIF_ParaCalc,& bimstringsetusrexecp, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setgrpreadp", NSBIF_ParaCalc,& bimstringsetgrpreadp, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setgrpwritep", NSBIF_ParaCalc,& bimstringsetgrpwritep, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setgrpexecp", NSBIF_ParaCalc,& bimstringsetgrpexecp, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setothreadp", NSBIF_ParaCalc,& bimstringsetothreadp, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setothwritep", NSBIF_ParaCalc,& bimstringsetothwritep, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
  L"setothexecp", NSBIF_ParaCalc,& bimstringsetothexecp, BoolCheckThis_True,
         TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},
  { L"setuidp", NSBIF_ParaCalc,& bimstringsetuidp, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,         BoolThisIsConst_True},
  { L"setgidp", NSBIF_ParaCalc,& bimstringsetgidp, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,         BoolThisIsConst_True},
  { L"setvtxp", NSBIF_ParaCalc,& bimstringsetvtxp, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,         BoolThisIsConst_True},
  {  L"stringp", NSBIF_ParaCalc,& bimnilstringp, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,         BoolThisIsConst_True},
  {  L"mkdir", NSBIF_ParaCalc,& bimstringmkdir, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,         BoolThisIsConst_True},
  { L"settime", NSBIF_ParaCalc,& bimstringsettime, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,         BoolThisIsConst_True},
  {    L"nsubstitute", NSBIF_ParaCalc,
    & bimstringnsubstitute, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_False},{
  L"chroot", NSBIF_ParaCalc,& bimstringchroot, BoolCheckThis_True,
    TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},{
//std::wclog<<" 0,786."<<std::flush;
// !! to make string::map, mapc, sort  
  
  // cons
  
  L"size", NSBIF_NotCalc,& Lisp2Dbimconssize2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
    BoolThisIsConst_True},{
  L"first", NSBIF_NotCalc,& Lisp2Dbimconsfirst2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_True},{
  L"second", NSBIF_NotCalc,& Lisp2Dbimconssecond2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_True},{
  L"third", NSBIF_NotCalc,& Lisp2Dbimconsthird2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_True},{
  L"rest", NSBIF_NotCalc,& Lisp2Dbimconsrest2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
    BoolThisIsConst_True},{
  L"last", NSBIF_NotCalc,& Lisp2Dbimconslast2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
    BoolThisIsConst_True},{
  
  L"setfirst", NSBIF_ParaCalc,& Lisp2Dbimconssetfirst2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         
         BoolThisIsConst_False},{
  L"setrest", NSBIF_ParaCalc,& Lisp2Dbimconssetrest2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         
         BoolThisIsConst_False},
  { L"+=" , NSBIF_ParaCalc  , & Lisp2Dbimconsnconc2 , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOCons , BoolThisIsConst_False } ,
  { L"copy-tree", NSBIF_NotCalc,& Lisp2Dbimconscopytree2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         
         BoolThisIsConst_True},{
  L"copy-list", NSBIF_NotCalc,& Lisp2Dbimconscopylist2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         
         BoolThisIsConst_True},{
  L"copy-list-protected", NSBIF_NotCalc,& Lisp2Dbimconscopylistprotected2, BoolCheckThis_True,
         TypeGlobalListMemberPLOCons, BoolThisIsConst_True},{
  L"vector", NSBIF_NotCalc,& Lisp2Dbimconsvector2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_True},{
// !? to make list: string (write OR princ) ?!
  L"setf", NSBIF_ParaCalc,&  Lisp2Dbimconssetf2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_False},{
  
  L"swap", NSBIF_ParaCalc,&  Lisp2Dbimconsswap2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_False},{
  
// ! to make parallel cycle for  
// ! to add sinon L"∀"    
  L"for*", NSBIF_NotCalc,& Lisp2Dbimconsforcons2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_True},{
// ! to add sinon L"∀*"    
  
  L"cons", NSBIF_ParaCalc,&  Lisp2Dbimconscons2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_False},{
  L"map", NSBIF_ParaCalc,& Lisp2Dbimconsmap2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
    BoolThisIsConst_True},{
  L"mapc", NSBIF_ParaCalc,&  Lisp2Dbimconsmapc2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_True},{
  L"subseq", NSBIF_ParaCalc,&  Lisp2Dbimconssubseq2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         BoolThisIsConst_True},{
  L"without", NSBIF_ParaCalc,& Lisp2Dbimconswithout2, BoolCheckThis_True, TypeGlobalListMemberPLOCons,
         
         BoolThisIsConst_True},{
// !! to make list: sort     

  // nil
  L"first", NSBIF_NotCalc,& Lisp2Dbimnilfirst2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},
  { L"man"  , NSBIF_NotCalc , & bimnilman , BoolCheckThis_False ,
    TypeGlobalListMemberPLONil  , BoolThisIsConst_True  } ,
  {
  L"second", NSBIF_NotCalc,& Lisp2Dbimnilfirst2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{ // the same
  L"third", NSBIF_NotCalc,& Lisp2Dbimnilfirst2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},  // the same
#ifndef  LISP2D_DONTUSEX11
           {L"display-size", NSBIF_NotCalc,& Lisp2Dbimnildisplaysize2, BoolCheckThis_False,
              TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"display-sizemm", NSBIF_NotCalc,& Lisp2Dbimnildisplaysizemm2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,
         
         BoolThisIsConst_True},{
  L"screen-count", NSBIF_NotCalc,& Lisp2Dbimnilscreencount2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},
#endif
      { L"rest", NSBIF_NotCalc,& Lisp2Dbimnilrest2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  L"cons", NSBIF_ParaCalc,& Lisp2Dbimnilcons2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  L"atom", NSBIF_ParaCalc,& Lisp2Dbimnilatom2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  
    L"not", NSBIF_ParaCalc,  & bimnilnot, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},{
    L"¬", NSBIF_ParaCalc,  & bimnilnot, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True
    },{
  
  L"list", NSBIF_ParaCalc,& Lisp2Dbimnillist2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  L"list*", NSBIF_ParaCalc,& Lisp2Dbimnillistc2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"last", NSBIF_NotCalc,& Lisp2Dbimnillast2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  L"string", NSBIF_NotCalc,& bimnilstring, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"eq", NSBIF_ParaCalc,& bimnileq, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  L"type-of", NSBIF_ParaCalc,& bimniltypeof, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"listp", NSBIF_ParaCalc,& Lisp2Dbimnillistp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"objectp", NSBIF_ParaCalc,& Lisp2Dbimnilobjectp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"acons", NSBIF_ParaCalc,& Lisp2Dbimnilacons23, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},
    { L"or", NSBIF_ParaCalc, & bimnilor, BoolCheckThis_False,
      TypeGlobalListMemberPLONil, BoolThisIsConst_True},
    { L"∨", NSBIF_ParaCalc,  & bimnilor, BoolCheckThis_False,
      TypeGlobalListMemberPLONil, BoolThisIsConst_True},
    { L"nor", NSBIF_ParaCalc,  & bimnilnor, BoolCheckThis_False
      , TypeGlobalListMemberPLONil, BoolThisIsConst_True},
    { L"⊽", NSBIF_ParaCalc,  & bimnilnor, BoolCheckThis_False,
      TypeGlobalListMemberPLONil, BoolThisIsConst_True},
    {
  
// ! to write bitxor , bitand , bitor , bitnot ?      
      
    L"xor", NSBIF_ParaCalc,  & bimnilxor, BoolCheckThis_False,
      TypeGlobalListMemberPLONil, BoolThisIsConst_True},
    { L"⊻", NSBIF_ParaCalc,  & bimnilxor, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},
    { L"and", NSBIF_ParaCalc,  & bimniland, BoolCheckThis_False,
      TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
    L"∧", NSBIF_ParaCalc,  & bimniland, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True
      },{
  
    L"nand", NSBIF_ParaCalc, & bimnilnand, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},{
    L"⊼", NSBIF_ParaCalc,  & bimnilnand, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},{
  
    L"imply", NSBIF_ParaCalc,  & bimnilimply, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},
    { L"->", NSBIF_ParaCalc,  & bimnilimply, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},
    { L"localtime", NSBIF_NotCalc,& Lisp2Dbimnilsystime2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"geteuid", NSBIF_NotCalc,& bimnilgeteuid, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"getegid", NSBIF_NotCalc,& bimnilgetegid, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},
  { L"getuid" , NSBIF_NotCalc , & bimnilgetuid  , BoolCheckThis_False ,
    TypeGlobalListMemberPLONil  , BoolThisIsConst_True  } ,
  { L"getgid", NSBIF_NotCalc,& bimnilgetgid, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"sysname", NSBIF_NotCalc,& bimnilsysname, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"release", NSBIF_NotCalc,& bimnilrelease, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"version", NSBIF_NotCalc,& bimnilversion, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"machine", NSBIF_NotCalc,& bimnilmachine, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"nodename", NSBIF_NotCalc,& bimnilnodename, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"uptime", NSBIF_NotCalc,& bimniluptime, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"totalram", NSBIF_NotCalc,& bimniltotalram, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"freeram", NSBIF_NotCalc,& bimnilfreeram, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"getcwd", NSBIF_NotCalc,& bimnilgetcwd, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  
// ! to add sinon L"∀*"    
  L"for*", NSBIF_NotCalc,& Lisp2Dbimnilforcons2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"for", NSBIF_NotCalc,& Lisp2Dbimnilfor2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
// ! to add sinon L"∀"    
  
  L"=", NSBIF_ParaCalc,& bimnilequal, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  L"size", NSBIF_NotCalc,& Lisp2Dbimnilsize2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  L"daylightp", NSBIF_NotCalc,& Lisp2Dbimnildaylightp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         
         BoolThisIsConst_True},{
  L"timezone", NSBIF_NotCalc,& Lisp2Dbimniltimezone2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"tzname", NSBIF_NotCalc,& Lisp2Dbimniltzname2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"setdaylightp", NSBIF_NotCalc,& Lisp2Dbimnilsetdaylightp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"settimezone", NSBIF_NotCalc,& Lisp2Dbimnilsettimezone2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"settzname", NSBIF_NotCalc,& Lisp2Dbimnilsettzname2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         
         BoolThisIsConst_True},{
  L"boundp", NSBIF_NotCalc,& bimnilboundp, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"defmethod", NSBIF_NotCalc,& bimnildefmethod, BoolCheckThis_False, TypeGlobalListMemberPLONil,
                  BoolThisIsConst_True},{
  L"getmethod", NSBIF_ParaCalc,& bimnilgetmethod, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,                  BoolThisIsConst_True},{
  L"get", NSBIF_ParaCalc,& bimnilget, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},
  {
  L"put", NSBIF_ParaCalc,& bimnilput, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},
  {
  L"defmacro", NSBIF_NotCalc,& bimnildefmacro, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,         BoolThisIsConst_True},
  {  
  L"remprop", NSBIF_ParaCalc,& bimnilremprop, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,         BoolThisIsConst_True},
  {  L"copy", NSBIF_NotCalc,& bimnilcopy, BoolCheckThis_False,
    TypeGlobalListMemberPLONil, BoolThisIsConst_True},
  {  L"copy-tree", NSBIF_NotCalc,& Lisp2Dbimnilcopytree2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,         BoolThisIsConst_True},
  {  L"copy-list", NSBIF_NotCalc,& Lisp2Dbimnilcopylist2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},
  {
  L"copy-list-protected", NSBIF_NotCalc,& Lisp2Dbimnilcopylist2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,
         
         BoolThisIsConst_True},{ // the same
  L"nprocs", NSBIF_NotCalc,& bimnilnprocs, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},
         {
  L"random", NSBIF_NotCalc,& Lisp2Dbimnilrandom, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},
         /*{
  L"*random-old*", NSBIF_NotCalc,& Lisp2Dbimnilrandom2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},         */
         {
  L"gensym", NSBIF_NotCalc,& Lisp2Dbimnilgensym2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  
// ! to add sinon L"≠"  
  L"<>", NSBIF_ParaCalc,& bimnilnotequal, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"getclass", NSBIF_NotCalc,& bimnilgetclass, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"do-while", NSBIF_NotCalc,& bimnildowhile, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{ // exception checked
  L"time", NSBIF_NotCalc,& Lisp2Dbimniltime2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  
  L"cond", NSBIF_NotCalc,& bimnilcond, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{  // exception checked
  L"apply", NSBIF_ParaCalc,& bimnilapply, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"if", NSBIF_NotCalc,& bimnilif, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{  // exception checked
  L"when", NSBIF_NotCalc,& bimnilwhen, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{  // exception checked
  L"parallel", NSBIF_ParaCalc,& bimnilparallel, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         
         BoolThisIsConst_True},{
  L"while", NSBIF_NotCalc,& bimnilwhile, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{  // exception checked
  L"eval", NSBIF_ParaCalc,& bimnileval, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True
        },{
  L"setq", NSBIF_NotCalc,& bimnilsetq, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{  // exception checked
  
// ! first: create envrinoment
// ! second: run task with PFirst of new env    
  
  L"let", NSBIF_NotCalc,& bimnillet, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{  // exception checked
  L"fork", NSBIF_NotCalc,& bimnilfork, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  
    L"try", NSBIF_NotCalc, & bimniltry, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True
  },{
    L"throw", NSBIF_ParaCalc,  & bimnilthrow, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},{
    L"exception", NSBIF_NotCalc, & bimnilexception, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},{
    L"exception-history", NSBIF_NotCalc, & bimnilexceptionhistory, BoolCheckThis_False,
         TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
    L"cycledp", NSBIF_NotCalc,& Lisp2Dbimnilcycledp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},{
    L"funcall", NSBIF_ParaCalc,& bimnilfuncall, BoolCheckThis_False, TypeGlobalListMemberPLONil,
      BoolThisIsConst_True},{
    L"getlogin", NSBIF_NotCalc,& bimnilgetlogin, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
    L"setdebug", NSBIF_ParaCalc,& bimnilsetdebug, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         
         BoolThisIsConst_True},{
  L"getdebug", NSBIF_NotCalc,& bimnilgetdebug, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"map", NSBIF_ParaCalc,& bimnilmap, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  L"mapc", NSBIF_ParaCalc,& bimnilmap, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{  // the same
  L"getpid", NSBIF_NotCalc,& bimnilgetpid, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"getppid", NSBIF_NotCalc,& bimnilgetppid, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"consp", NSBIF_ParaCalc,& Lisp2Dbimnilconsp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"bifp", NSBIF_ParaCalc,& Lisp2Dbimnilbifp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},
  { L"reverse", NSBIF_NotCalc,& Lisp2Dbimnilreverse2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil, BoolThisIsConst_True},
  { L"+", NSBIF_ParaCalc,& Lisp2Dbimnilplus2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil, BoolThisIsConst_True },
  { L"environment", NSBIF_NotCalc,& Lisp2Dbimnilenvironment2,
    BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},
  { L"environment-external", NSBIF_NotCalc,& Lisp2Dbimnilenvironmentexternal2,
    BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},
# ifdef LISP2D_DEBUG
  {L"*crash*", NSBIF_NotCalc,& bimnilcrash, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True}, 
# endif
  { L"sethostent" , NSBIF_ParaCalc  , & bimnilsethostent  ,
    BoolCheckThis_False , TypeGlobalListMemberPLONil ,
    BoolThisIsConst_True  } ,
  { L"gethostent" , NSBIF_NotCalc , & bimnilgethostent  ,
    BoolCheckThis_False , TypeGlobalListMemberPLONil ,
    BoolThisIsConst_True  } ,
  { L"endhostent" , NSBIF_NotCalc , & bimnilendhostent  ,
    BoolCheckThis_False , TypeGlobalListMemberPLONil ,
    BoolThisIsConst_True  } ,
  { L"constp", NSBIF_ParaCalc,& bimnilconstp, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  L"const", NSBIF_ParaCalc,&  bimnilconst, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  
// !? dismiss or leave
// dismiss: (const copy) and no more access to modify it
// leave: (nil unconst xconst),modify,(nil const xconst)
  L"unconst", NSBIF_ParaCalc,&  bimnilunconst, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
    
  L"subseq", NSBIF_ParaCalc,& bimnilsubseq, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  L"without", NSBIF_ParaCalc,&  bimnilwithout, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
// ! address-of -> pointer-of    
  L"address-of", NSBIF_ParaCalc,& bimniladdressof, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  
    L"private", NSBIF_NotCalc, & bimnilprivate, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
    L"open-protoent", NSBIF_NotCalc, & Lisp2Dbimnilopenprotoent2,
    BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{       
    L"close-protoent", NSBIF_NotCalc,
    & Lisp2Dbimnilcloseprotoent2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
    L"get-protoent", NSBIF_NotCalc,  & Lisp2Dbimnilgetprotoent2,
    BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
    L"log", NSBIF_ParaCalc,  & bimnillog, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
    L"log-list", NSBIF_NotCalc,  & bimnilloglist, BoolCheckThis_False,
    TypeGlobalListMemberPLONil, BoolThisIsConst_True},
    { L"environ-list" , NSBIF_NotCalc , & bimnilenvironlist , BoolCheckThis_False ,
      TypeGlobalListMemberPLONil , BoolThisIsConst_True  } ,
    {
  
  // symbol
    L"+", NSBIF_ParaCalc, & Lisp2Dbimsymbolplus2, BoolCheckThis_True,
    TypeGlobalListMemberPLOSymbol, BoolThisIsConst_True},{
    L"=", NSBIF_ParaCalc, & Lisp2Dbimsymbolequal2, BoolCheckThis_True,
    TypeGlobalListMemberPLOSymbol, BoolThisIsConst_True},{
    L"string", NSBIF_NotCalc,  & Lisp2Dbimsymbolstring2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         
         BoolThisIsConst_True},{
  L"properties", NSBIF_NotCalc,& Lisp2Dbimsymbolproperties2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         
         BoolThisIsConst_True},{
  L"set", NSBIF_ParaCalc,& Lisp2Dbimsymbolset2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         BoolThisIsConst_True},{
  L"defmethod", NSBIF_NotCalc,& Lisp2Dbimsymboldefmethod2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
                  BoolThisIsConst_True},{
  
  L"defmacro", NSBIF_NotCalc,& Lisp2Dbimsymboldefmacro2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         BoolThisIsConst_True},{
  L"boundp", NSBIF_ParaCalc,& Lisp2Dbimsymbolboundp2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         BoolThisIsConst_True},{
  L"defclass", NSBIF_NotCalc,& Lisp2Dbimsymboldefclass2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         BoolThisIsConst_True},{
// ! to add sinon L"≠"    
  L"<>", NSBIF_ParaCalc,& Lisp2Dbimsymbolnotequal2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         
         BoolThisIsConst_True},{
  L"<", NSBIF_ParaCalc,& Lisp2Dbimsymbolless3, BoolCheckThis_True,
    TypeGlobalListMemberPLOSymbol,         BoolThisIsConst_True},{
  L">", NSBIF_ParaCalc,& Lisp2Dbimsymbolmore3, BoolCheckThis_True,
    TypeGlobalListMemberPLOSymbol,         BoolThisIsConst_True},{
// ! to add sinon L"≤"    
  L"<=", NSBIF_ParaCalc,& Lisp2Dbimsymbollessoreq3, BoolCheckThis_True,
    TypeGlobalListMemberPLOSymbol,                  BoolThisIsConst_True},{
// ! to add sinon L"≥"  
  L">=", NSBIF_ParaCalc,& Lisp2Dbimsymbolmoreoreq3, BoolCheckThis_True,
    TypeGlobalListMemberPLOSymbol,                  BoolThisIsConst_True},{
  L"min", NSBIF_ParaCalc,& Lisp2Dbimsymbolmin2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         BoolThisIsConst_True},{
  L"max", NSBIF_ParaCalc,& Lisp2Dbimsymbolmax2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         BoolThisIsConst_True},{
  L"letset", NSBIF_ParaCalc,& Lisp2Dbimsymbolletset2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         BoolThisIsConst_True},{
  L"namespace", NSBIF_NotCalc,& Lisp2Dbimsymbolnamespace2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         
         BoolThisIsConst_True},{
  
// ! to remake : symbol -> symbol | (symbol  initvalue)  
  
  L"defvar", NSBIF_NotCalc,& Lisp2Dbimsymboldefvar2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         
         BoolThisIsConst_True},{
  L"defstatic", NSBIF_ParaCalc,& Lisp2Dbimsymboldefstatic2, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
         
         BoolThisIsConst_True},
  { L"newobject", NSBIF_NotCalc,& bimsymbolnewobject,
    BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
    BoolThisIsConst_True},
  {  // macro
  L"symbolp", NSBIF_ParaCalc,& Lisp2Dbimnilsymbolp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
    L"const", NSBIF_NotCalc,  & Lisp2Dbimsymbolconst, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
    BoolThisIsConst_True},{
    L"unconst", NSBIF_NotCalc,  & Lisp2Dbimsymbolunconst, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
    BoolThisIsConst_True},{
    L"constp", NSBIF_NotCalc, & Lisp2Dbimsymbolconstp, BoolCheckThis_True, TypeGlobalListMemberPLOSymbol,
    BoolThisIsConst_True},{
// ! to write: method-const, method-unconst, method-constp  
  
  // char
  L"=", NSBIF_ParaCalc,& Lisp2Dbimcharequal2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
    BoolThisIsConst_True},{
// ! to add sinon L"≤"    
  L"<=", NSBIF_ParaCalc,& Lisp2Dbimcharlessoreq3, BoolCheckThis_True,
    TypeGlobalListMemberPLOChar,         BoolThisIsConst_True},{
// ! rename code->number
  
    L"number", NSBIF_NotCalc, & Lisp2Dbimcharnumber, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
    BoolThisIsConst_True},{
// ! to write: (char byte-vector) // take from (string byte-vector)
  L"symbol", NSBIF_NotCalc,& Lisp2Dbimcharsymbol2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"string", NSBIF_NotCalc,& Lisp2Dbimcharstring2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
// ! to add sinon L"≠"    
  L"<>", NSBIF_ParaCalc,& Lisp2Dbimcharnotequal2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"<", NSBIF_ParaCalc,& Lisp2Dbimcharless3, BoolCheckThis_True,
    TypeGlobalListMemberPLOChar, BoolThisIsConst_True},{
  L">", NSBIF_ParaCalc,& Lisp2Dbimcharmore3, BoolCheckThis_True,
    TypeGlobalListMemberPLOChar, BoolThisIsConst_True},{
// ! to add sinon L"≥"  
  L">=", NSBIF_ParaCalc,& Lisp2Dbimcharmoreoreq3, BoolCheckThis_True,
    TypeGlobalListMemberPLOChar,         BoolThisIsConst_True},{
  L"min", NSBIF_ParaCalc,& Lisp2Dbimcharmin2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
    BoolThisIsConst_True},{
  L"max", NSBIF_ParaCalc,& Lisp2Dbimcharmax2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
    BoolThisIsConst_True},{
  L"alphap", NSBIF_NotCalc,& Lisp2Dbimcharalphap2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"upperp", NSBIF_NotCalc,& Lisp2Dbimcharupperp2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"lowerp", NSBIF_NotCalc,& Lisp2Dbimcharlowerp2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"digitp", NSBIF_NotCalc,& Lisp2Dbimchardigitp2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"spacep", NSBIF_NotCalc,& Lisp2Dbimcharspacep2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"controlp", NSBIF_NotCalc,& Lisp2Dbimcharcontrolp2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         
         BoolThisIsConst_True},{
  L"printp", NSBIF_NotCalc,& Lisp2Dbimcharprintp2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"upper", NSBIF_NotCalc,& Lisp2Dbimcharupper2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"lower", NSBIF_NotCalc,& Lisp2Dbimcharlower2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_True},{
  L"setf", NSBIF_ParaCalc,& Lisp2Dbimcharsetf2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_False},{
  L"swap", NSBIF_ParaCalc,& Lisp2Dbimcharswap2, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_False},{
  
  L"charp", NSBIF_ParaCalc,& Lisp2Dbimnilcharp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
    L"char", NSBIF_ParaCalc,  & Lisp2Dbimcharchar, BoolCheckThis_True, TypeGlobalListMemberPLOChar,
         BoolThisIsConst_False},
  {  L"byte-vector"  , NSBIF_NotCalc  , & Lisp2Dbimcharbytevector ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOChar  , BoolThisIsConst_True  } ,
         {
  /*
! to write:

  char - constructor

ispunct
    Check if character is a punctuation character (function )

isxdigit
    Check if character is hexadecimal digit (function )

isalnum
    Check if character is alphanumeric (function )

isgraph
    Check if character has graphical representation (function )

isblank
    Check if character is blank (function )*/
  
  // ! copy 

  
// ! to make in ExtraThread    
  L"close", NSBIF_NotCalc,& Lisp2Dbimstreamclose2, BoolCheckThis_True, TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False},{
  
// !? to write: good , eof,bad , fail  
  
  L"listen", NSBIF_NotCalc,& Lisp2Dbimstreamlisten2, BoolCheckThis_True, TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_True},{
  
  L"openp", NSBIF_NotCalc,& Lisp2Dbimstreamopenp2, BoolCheckThis_True, TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_True},
  {  L"chdir"  , NSBIF_NotCalc , & Lisp2Dbimstreamchdir  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStream  ,
    BoolThisIsConst_True  } ,
  
                  {
  
  L"streamp", NSBIF_ParaCalc,& Lisp2Dbimnilstreamp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  
  
  // ! copy ?

  // BoolCheckThis::True
  // boolean
  L"string", NSBIF_NotCalc,& bimbooleanstring, BoolCheckThis_False, TypeGlobalListMemberPLOBoolean,
         BoolThisIsConst_True},{
  L"booleanp", NSBIF_ParaCalc,& bimnilbooleanp, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},

  // BoolCheckThis::False

#ifndef  LISP2D_DONTUSEX11
  // window
         {  L"close"  , NSBIF_NotCalc , & Lisp2Dbimwindowclose2 , BoolCheckThis_True  , TypeGlobalListMemberPLOWindowPri ,
         
         BoolThisIsConst_False},{
  L"draw-pixel", NSBIF_ParaCalc,& Lisp2Dbimdrawpixel2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_False},{
  L"draw-square", NSBIF_ParaCalc,& bimdrawsquare, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_False},
  { L"draw-line"  , NSBIF_ParaCalc  , & bimwindowdrawline ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOWindowPri  ,
    BoolThisIsConst_False } ,
  { L"getpixel", NSBIF_ParaCalc,& Lisp2Dbimwindowgetpixel2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_True},{
  L"width", NSBIF_NotCalc,& Lisp2Dbimwindowwidth2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         
         BoolThisIsConst_True},{
  L"height", NSBIF_NotCalc,& Lisp2Dbimwindowheight2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         
         BoolThisIsConst_True},{
  L"pointer-coord", NSBIF_NotCalc,& Lisp2Dbimwindowpointercoord2, BoolCheckThis_True,
         TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_True},
#ifdef  LISP2D_DEBUG  
  // for font loading
           {L"*getpixel*", NSBIF_ParaCalc,& Lisp2Dbimwindowsysgetpixel2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_True},
#endif  
           {L"move", NSBIF_ParaCalc,& Lisp2Dbimwindowmove2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_False},{
  L"resize", NSBIF_ParaCalc,& Lisp2Dbimwindowresize2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_False},{
  L"map-resize", NSBIF_ParaCalc,& Lisp2Dbimwindowmapresize2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_False},{
  
// ! to make: map-resize - only map    
  
  L"rename", NSBIF_ParaCalc,& Lisp2Dbimwindowrename2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_False},{
  L"open", NSBIF_NotCalc,& Lisp2Dbimwindowopen2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_False},{
  L"openp", NSBIF_NotCalc,& Lisp2Dbimwindowopenp2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         
         BoolThisIsConst_True},{
  L"hide", NSBIF_NotCalc,& Lisp2Dbimwindowhide2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         BoolThisIsConst_False},
#ifdef  LISP2D_DEBUG  
         {L"*font*", NSBIF_ParaCalc,& Lisp2Dbimwindowsysloadfont2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
         
         BoolThisIsConst_False},
#endif  
         {L"windowp", NSBIF_ParaCalc,& Lisp2Dbimnilwindowp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
    BoolThisIsConst_True},{
  L"window", NSBIF_ParaCalc,& Lisp2Dbimwindowwindow2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_False},{ // constructor
  L"mapwindow", NSBIF_ParaCalc,& Lisp2Dbimnilmapwindow2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         
         BoolThisIsConst_True},{
  L"wait-event", NSBIF_NotCalc,&  Lisp2Dbimwindowwaitevent2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_True},{
  L"check-event", NSBIF_NotCalc,& Lisp2Dbimwindowcheckevent2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_True},{
  L"make-event", NSBIF_ParaCalc,& Lisp2Dbimwindowmakeevent2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_True},{
  L"pending", NSBIF_NotCalc,& Lisp2Dbimwindowpending2, BoolCheckThis_True, TypeGlobalListMemberPLOWindowPri,
    BoolThisIsConst_True},

#endif

  // time
      {L"setf", NSBIF_ParaCalc,& Lisp2Dbimtimesetf2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_False},{
  L"swap", NSBIF_ParaCalc,& Lisp2Dbimtimeswap2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_False},{
  
  L"year", NSBIF_NotCalc,& Lisp2Dbimtimeyear2, BoolCheckThis_True, TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
  L"month", NSBIF_NotCalc,& Lisp2Dbimtimemonth2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_True},{
  L"day", NSBIF_NotCalc,& Lisp2Dbimtimeday2, BoolCheckThis_True, TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
  L"hour", NSBIF_NotCalc,& Lisp2Dbimtimehour2, BoolCheckThis_True, TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
  L"minutes", NSBIF_NotCalc,& Lisp2Dbimtimeminutes2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_True},{
  L"seconds", NSBIF_NotCalc,& Lisp2Dbimtimeseconds2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_True},{
  L"weekday", NSBIF_NotCalc,& Lisp2Dbimtimeweekday2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_True},{
  L"yearday", NSBIF_NotCalc,& Lisp2Dbimtimeyearday2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_True},{
  L"dstp", NSBIF_NotCalc,& Lisp2Dbimtimedstp2, BoolCheckThis_True, TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
  L"number", NSBIF_NotCalc,& Lisp2Dbimtimenumber2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_True},{
  L"time", NSBIF_ParaCalc,& Lisp2Dbimtimetime2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_False},{
  L"setyear", NSBIF_ParaCalc,& Lisp2Dbimtimesetyear2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         
         BoolThisIsConst_False},{
  L"setmonth", NSBIF_ParaCalc,& Lisp2Dbimtimesetmonth2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         
         BoolThisIsConst_False},{
  L"setday", NSBIF_ParaCalc,& Lisp2Dbimtimesetday2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_False},{
  L"sethour", NSBIF_ParaCalc,& Lisp2Dbimtimesethour2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         
         BoolThisIsConst_False},{
  L"setminutes", NSBIF_ParaCalc,& Lisp2Dbimtimesetminutes2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_False},{
  L"setseconds", NSBIF_ParaCalc,& Lisp2Dbimtimesetseconds2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_False},{
  L"string", NSBIF_NotCalc,& Lisp2Dbimtimestring2, BoolCheckThis_True, TypeGlobalListMemberPLOTime,
         BoolThisIsConst_True},{
  L"timep", NSBIF_ParaCalc,& Lisp2Dbimniltimep2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
         BoolThisIsConst_True},{
  L"=", NSBIF_ParaCalc,& Lisp2Dbimtimeeq2, BoolCheckThis_True, TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
// ! to add sinon L"≠"    
  L"<>", NSBIF_ParaCalc,& Lisp2Dbimtimenoteq2, BoolCheckThis_True, TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
  L"<", NSBIF_ParaCalc,& Lisp2Dbimtimeless3, BoolCheckThis_True,
    TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
  L">", NSBIF_ParaCalc,& Lisp2Dbimtimemore3, BoolCheckThis_True,
    TypeGlobalListMemberPLOTime, BoolThisIsConst_True},{
// ! to add sinon L"≤"    
  L"<=", NSBIF_ParaCalc,& Lisp2Dbimtimelesseq3, BoolCheckThis_True,
    TypeGlobalListMemberPLOTime,         BoolThisIsConst_True},{
// ! to add sinon L"≥"  
  L">=", NSBIF_ParaCalc,& Lisp2Dbimtimemoreeq3, BoolCheckThis_True,
    TypeGlobalListMemberPLOTime,         BoolThisIsConst_True},
    
  { L"min", NSBIF_ParaCalc, & Lisp2Dbimtimemin, BoolCheckThis_True, TypeGlobalListMemberPLOTime,        BoolThisIsConst_True},
  { L"max", NSBIF_ParaCalc, & Lisp2Dbimtimemax, BoolCheckThis_True, TypeGlobalListMemberPLOTime,         BoolThisIsConst_True},  
    
    {
  
  // dir
  L"dir", NSBIF_ParaCalc,& Lisp2Dbimdirdir2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri, BoolThisIsConst_False},{
  
  L"swap", NSBIF_ParaCalc,&  Lisp2Dbimdirswap2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri, BoolThisIsConst_False},{
  L"dirp", NSBIF_ParaCalc,& Lisp2Dbimnildirp2, BoolCheckThis_True, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  L"open", NSBIF_ParaCalc,& Lisp2Dbimdiropen2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri, BoolThisIsConst_False},{
  L"close", NSBIF_NotCalc,& Lisp2Dbimdirclose2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri, BoolThisIsConst_False},{
  L"openp", NSBIF_NotCalc,& Lisp2Dbimdiropenp2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri, BoolThisIsConst_True},
  { L"read" , NSBIF_NotCalc , & Lisp2Dbimdirread2 , BoolCheckThis_True  ,
    TypeGlobalListMemberPLODirPri  , BoolThisIsConst_False } ,
  { L"rewind", NSBIF_NotCalc,& Lisp2Dbimdirrewind2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri,
          BoolThisIsConst_False},{
  L"seek", NSBIF_ParaCalc,& Lisp2Dbimdirseek2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri, BoolThisIsConst_False},{
  L"tell", NSBIF_NotCalc,& Lisp2Dbimdirtell2, BoolCheckThis_True, TypeGlobalListMemberPLODirPri, BoolThisIsConst_True},{
  
  // passage
  

// !? rename lock -> mutex  
  // lock
  L"progn", NSBIF_NotCalc,& Lisp2Dbimlockprogn2, BoolCheckThis_True, TypeGlobalListMemberPLOLock,
          BoolThisIsConst_False},{
  L"try", NSBIF_NotCalc,& Lisp2Dbimlocktry2, BoolCheckThis_True, TypeGlobalListMemberPLOLock, BoolThisIsConst_False},{

  L"set", NSBIF_NotCalc,& Lisp2Dbimlockset2, BoolCheckThis_True, TypeGlobalListMemberPLOLock, BoolThisIsConst_False},{
  
  L"unlock", NSBIF_NotCalc,& Lisp2Dbimlockunlock2, BoolCheckThis_True, TypeGlobalListMemberPLOLock,
          BoolThisIsConst_False},{
  L"lockp", NSBIF_ParaCalc,& Lisp2Dbimnillockp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  
  // signal
  L"wait", NSBIF_ParaCalc,& Lisp2Dbimsignalwait2, BoolCheckThis_True, TypeGlobalListMemberPLOSignal,
          BoolThisIsConst_True},{
  L"send", NSBIF_NotCalc,& Lisp2Dbimsignalsend2, BoolCheckThis_True, TypeGlobalListMemberPLOSignal,
          BoolThisIsConst_True},{
  L"signalp", NSBIF_ParaCalc,& Lisp2Dbimnilsignalp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  
  // environment
  L"envp", NSBIF_ParaCalc,& Lisp2Dbimnilenvp2, BoolCheckThis_False, TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  L"eval", NSBIF_ParaCalc,& Lisp2Dbimenveval2, BoolCheckThis_True, TypeGlobalListMemberPLOEnv, BoolThisIsConst_False},{
  L"apply", NSBIF_ParaCalc,& Lisp2Dbimenvapply2, BoolCheckThis_True, TypeGlobalListMemberPLOEnv,
          BoolThisIsConst_False},{
  L"funcall", NSBIF_ParaCalc,& Lisp2Dbimenvfuncall2, BoolCheckThis_True, TypeGlobalListMemberPLOEnv,
          BoolThisIsConst_False},{
  L"setq", NSBIF_NotCalc,& Lisp2Dbimenvsetq2, BoolCheckThis_True,
    TypeGlobalListMemberPLOEnv, BoolThisIsConst_False},{
  
// ! how to use ?  
  
  // quote
  L"quote", NSBIF_ParaCalc,& Lisp2Dbimquotequote2, BoolCheckThis_True,
    TypeGlobalListMemberPLOQuote,
          BoolThisIsConst_False},{
  L"quotep", NSBIF_ParaCalc,& Lisp2Dbimnilquotep2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  L"setf", NSBIF_ParaCalc,& Lisp2Dbimquotesetf2, BoolCheckThis_True,
    TypeGlobalListMemberPLOQuote,
          BoolThisIsConst_False},{
  L"swap", NSBIF_ParaCalc,& Lisp2Dbimquoteswap2, BoolCheckThis_True,
    TypeGlobalListMemberPLOQuote,
          BoolThisIsConst_False},{
  
  
// ! how to use ?
  
  // backquote
  L"backquote", NSBIF_ParaCalc,& Lisp2Dbimbackquotebackquote2, BoolCheckThis_True,
          TypeGlobalListMemberBackQuotePrivate, BoolThisIsConst_False},{
  L"backquotep", NSBIF_ParaCalc,& Lisp2Dbimnilbackquotep2, BoolCheckThis_False,
    TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  L"setf", NSBIF_ParaCalc,& Lisp2Dbimbackquotesetf2, BoolCheckThis_True,
          TypeGlobalListMemberBackQuotePrivate, BoolThisIsConst_False},{
  L"swap", NSBIF_ParaCalc,& Lisp2Dbimbackquoteswap2, BoolCheckThis_True,
          TypeGlobalListMemberBackQuotePrivate, BoolThisIsConst_False},{
  
  
// ! how to use ?  
  
  // unquote
  L"unquote", NSBIF_ParaCalc,& Lisp2Dbimcommacomma2, BoolCheckThis_True, TypeGlobalListMemberPLOComma,
          BoolThisIsConst_False},{
  L"unquotep", NSBIF_ParaCalc,& Lisp2Dbimnilcommap2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
          BoolThisIsConst_True},{
  L"setf", NSBIF_ParaCalc,& Lisp2Dbimcommasetf2, BoolCheckThis_True, TypeGlobalListMemberPLOComma,
          BoolThisIsConst_False},{
  L"swap", NSBIF_ParaCalc,& Lisp2Dbimcommaswap2, BoolCheckThis_True, TypeGlobalListMemberPLOComma,
          BoolThisIsConst_False},{
  
  
// ! how to use ?
  
  // unquote-splicing
  L"unquote-splicing", NSBIF_ParaCalc,
          & Lisp2DbimUnquoteSplicingUnquoteSplicing2, BoolCheckThis_True, TypeGlobalListMemberPLOUnquoteSplicing, BoolThisIsConst_False},{
  L"unquote-splicingp", NSBIF_ParaCalc,& Lisp2DbimnilUnquoteSplicingP2, BoolCheckThis_False,
          TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  L"setf", NSBIF_ParaCalc,& Lisp2DbimUnquoteSplicingsetf2, BoolCheckThis_True,
          TypeGlobalListMemberPLOUnquoteSplicing, BoolThisIsConst_False},{
  L"swap", NSBIF_ParaCalc,& Lisp2DbimUnquoteSplicingswap2, BoolCheckThis_True,
          TypeGlobalListMemberPLOUnquoteSplicing, BoolThisIsConst_False},{
  
  
  // byte-vector
  L"byte-vector", NSBIF_ParaCalc,& Lisp2Dbimbytevectorbytevector2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"byte-vectorp", NSBIF_ParaCalc,&  Lisp2DbimnilbytevectorP2, BoolCheckThis_False,
          TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
  L"setf", NSBIF_ParaCalc,& Lisp2DbimbytedequeSetf2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
// ! copy exists
  L"swap", NSBIF_ParaCalc,& Lisp2DbimbytedequeSwap2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  
  L"size", NSBIF_NotCalc,&  Lisp2Dbimbytedequesize2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"elt", NSBIF_ParaCalc,&  Lisp2DbimbytedequeElt2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"setelt", NSBIF_ParaCalc,& Lisp2DbimbytedequeSetelt2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"resize", NSBIF_ParaCalc,& Lisp2Dbimbytedequeresize2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"push-back", NSBIF_ParaCalc,&  Lisp2Dbimbytedequepushback2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"push-front", NSBIF_ParaCalc,& Lisp2Dbimbytedequepushfront2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"pop-back", NSBIF_ParaCalc,& Lisp2Dbimbytedequepopback2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"pop-front", NSBIF_ParaCalc,&  Lisp2Dbimbytedequepopfront2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"clear", NSBIF_NotCalc,& Lisp2Dbimbytedequeclear2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_False},{
  L"empty", NSBIF_NotCalc,& Lisp2Dbimbytedequeempty2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"back", NSBIF_ParaCalc,& Lisp2Dbimbytedequeback2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"front", NSBIF_ParaCalc,&  Lisp2Dbimbytedequefront2, BoolCheckThis_True,
          TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"vector", NSBIF_ParaCalc,& Lisp2DbimbytedequeVector2, BoolCheckThis_True,
    TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"string", NSBIF_ParaCalc,& Lisp2DbimbytedequeString2, BoolCheckThis_True,
    TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"list", NSBIF_NotCalc,&  Lisp2DbimbytedequeList2, BoolCheckThis_True,
    TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},{
  L"subseq", NSBIF_ParaCalc,& Lisp2Dbimbytedequesubseq2, BoolCheckThis_True,
    TypeGlobalListMemberPLOByteDeque, BoolThisIsConst_True},
  {  L"char" , NSBIF_ParaCalc  , & Lisp2Dbimbytevectorchar , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOByteDeque , BoolThisIsConst_True  } ,
    {
      
// !! to make byte-vector:: for, for*, map, mapc ,sort   
      
// !? to write real memory address : 
// (byte-vector data-addr) -> number | nil      
  
  
  // socket
  L"socket", NSBIF_NotCalc,& Lisp2Dbimsocketsocket2, BoolCheckThis_True,
    TypeGlobalListMemberPLOSocket, BoolThisIsConst_False},{
  
   L"socketp", NSBIF_ParaCalc,  & Lisp2Dbimnilsocketp2, BoolCheckThis_False, TypeGlobalListMemberPLONil,
               BoolThisIsConst_True},{
  
    L"socket-select", NSBIF_ParaCalc,  & Lisp2Dbimnilsocketselect, BoolCheckThis_True,
    TypeGlobalListMemberPLONil, BoolThisIsConst_True},{
    L"descriptor", NSBIF_NotCalc, & Lisp2Dbimsocketdescriptor, BoolCheckThis_True,
    TypeGlobalListMemberPLOSocket, BoolThisIsConst_False}  ,
  // stat
  { L"stat" , NSBIF_ParaCalc  , & Lisp2Dbimstatstat , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOStatPri  , BoolThisIsConst_False } ,
  { L"statp"  , NSBIF_ParaCalc  , & Lisp2Dbimnilstatp , BoolCheckThis_False ,
    TypeGlobalListMemberPLONil , BoolThisIsConst_True  } ,
  { L"setf" , NSBIF_ParaCalc  , & Lisp2Dbimstatsetf , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOStatPri  , BoolThisIsConst_False } ,
  { L"swap" , NSBIF_ParaCalc  , & Lisp2Dbimstatswap , BoolCheckThis_True  , 
    TypeGlobalListMemberPLOStatPri  , BoolThisIsConst_False } ,
  {  L"device" , NSBIF_NotCalc , & Lisp2Dbimstatdevice ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"inode" , NSBIF_NotCalc , & Lisp2Dbimstatinode  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"permissions" , NSBIF_NotCalc , & Lisp2Dbimstatpermissions  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"file-type" , NSBIF_NotCalc , & Lisp2Dbimstatfiletype ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"links" , NSBIF_NotCalc , & Lisp2Dbimstatlinks ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"uid" , NSBIF_NotCalc , & Lisp2Dbimstatuid ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"gid" , NSBIF_NotCalc , & Lisp2Dbimstatgid ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"rdevice" , NSBIF_NotCalc , & Lisp2Dbimstatrdevice ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"totalsize" , NSBIF_NotCalc , & Lisp2Dbimstattotalsize ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"blocksize" , NSBIF_NotCalc , & Lisp2Dbimstatblocksize ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"blocks" , NSBIF_NotCalc , & Lisp2Dbimstatblocks ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"access-time" , NSBIF_NotCalc , & Lisp2Dbimstataccesstime ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"modify-time" , NSBIF_NotCalc , & Lisp2Dbimstatmodifytime ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  } ,
  {  L"change-time" , NSBIF_NotCalc , & Lisp2Dbimstatchangetime ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOStatPri ,
    BoolThisIsConst_True  }
    };
# ifdef  LISP2D_DEBUG
  size_t  const setTypeSymbolNameTypeSize = MapPubSize  ( setTypeSymbolNameType ) (
      setTypeSymbolNameType_pub ) ;
# endif
  { enum { si = sizeof(arrsetbimargs)/sizeof(arrsetbimargs[0]) } ;
    struct  setbimargs const * const sien = & arrsetbimargs [ si ] ;
    struct  setbimargs const * ip = & arrsetbimargs [ 0 ] ;
    do {
      size_t  const jc  = MapPubFind  ( setTypeSymbolNameType ) ( setTypeSymbolNameType_pub , ( uint8_t ) ( ip -> ct ) ) ;
# ifdef  LISP2D_DEBUG
      if  ( jc  == setTypeSymbolNameTypeSize )
        fwprintf  ( stderr  , L"setTypeSymbolName . find ( %u ) == end\n" , ( unsigned int ) ( ip -> ct ) ) ;
      else
# endif
      { struct  sSetBIM a = { . stdclassnamep2 = MapPubCAt ( setTypeSymbolNameType ) ( setTypeSymbolNameType_pub , jc ) -> value ,
          . namec = ip -> namec , . argt = ip -> argt  , . funcaddr2 = ip -> funcaddr2 , . fcheckthis = ip->fcheckthis ,
          . ct = ip->ct, . stclp = stcl, . stdpropp = stdpro, . flagthisconst = ip->flagthisconst , . res = 0 } ;
        SetBIM ( & a ) ;
      }
      ++  ip  ;
    } while ( ip < sien ) ;
  }
  }

  // SetExtraThread
  {struct  setbimargs arrsetbimargs[]={
    {
    L"sleep", NSBIF_NotCalc, & bimcomplexsleep, BoolCheckThis_True,
    TypeGlobalListMemberPLOComplex,
    BoolThisIsConst_False},{
  L"read", NSBIF_NotCalc,& bimstringread, BoolCheckThis_True, TypeGlobalListMemberPLOString,
         BoolThisIsConst_True},
  
  {  L"open", NSBIF_ParaCalc,& Lisp2Dbimstringopenbinary2, BoolCheckThis_True,
         TypeGlobalListMemberPLOString, BoolThisIsConst_True

  },
  {  L"save", NSBIF_ParaCalc,& bimstringsave, BoolCheckThis_True,
  TypeGlobalListMemberPLOString,
                  BoolThisIsConst_True},
  { L"unlink", NSBIF_NotCalc,& bimstringunlink, BoolCheckThis_True, TypeGlobalListMemberPLOString,
                  BoolThisIsConst_True},
                  
  {  L"load"  , NSBIF_ParaCalc  , & bimstringload  , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  , BoolThisIsConst_True  } ,
                  {   
  // stream
    L"write", NSBIF_ParaCalc, & Lisp2Dbimstreamwrite2, BoolCheckThis_True,
    TypeGlobalListMemberPLOStream,
    BoolThisIsConst_False},{
  
// ! to make: read-bytes n  [byte-vector]
  
  L"read-byte", NSBIF_ParaCalc,& Lisp2Dbimstreamreadbyte2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False},{       
  L"writeln", NSBIF_ParaCalc,& Lisp2Dbimstreamwriteln2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
         BoolThisIsConst_False},{
  
// ! to make: wryte-bytes byte-vector [n]
  L"write-byte", NSBIF_ParaCalc,& Lisp2Dbimstreamwritebyte2,
  BoolCheckThis_True, TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False},{
  L"read", NSBIF_NotCalc,& Lisp2Dbimstreamread2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
         BoolThisIsConst_False},
  {  L"seek"  , NSBIF_ParaCalc  , & Lisp2Dbimstreamseek , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOStream  , BoolThisIsConst_False } ,

  {  L"tell"  , NSBIF_NotCalc  , & Lisp2Dbimstreamtell , BoolCheckThis_True  ,
    TypeGlobalListMemberPLOStream  , BoolThisIsConst_True } ,                
                  {
  L"terpri", NSBIF_NotCalc,& Lisp2Dbimstreamterpri2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False},{
  L"princ", NSBIF_ParaCalc,& Lisp2Dbimstreamprinc2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False},{
  L"print", NSBIF_ParaCalc,& Lisp2Dbimstreamprint2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False},{ 
  L"flush", NSBIF_NotCalc,& Lisp2Dbimstreamflush2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False } ,
  { L"read-line"  , NSBIF_ParaCalc  , & Lisp2Dbimstreamreadline2  , BoolCheckThis_True  , TypeGlobalListMemberPLOStream ,
    BoolThisIsConst_False } ,
  {
  
// ! to make: read-chars n  [string]
  
  L"read-char", NSBIF_ParaCalc,& Lisp2Dbimstreamreadchar2, BoolCheckThis_True,
  TypeGlobalListMemberPLOStream,
                  BoolThisIsConst_False},{       
  L"open", NSBIF_ParaCalc,& Lisp2Dbimsocketopen2,
    BoolCheckThis_True, TypeGlobalListMemberPLOSocket, BoolThisIsConst_False},{
  L"close", NSBIF_NotCalc,
    & Lisp2Dbimsocketclose2, BoolCheckThis_True, TypeGlobalListMemberPLOSocket,
    BoolThisIsConst_False},{
  L"connect", NSBIF_ParaCalc,
    & Lisp2Dbimsocketconnect2, BoolCheckThis_True,
    TypeGlobalListMemberPLOSocket,
    BoolThisIsConst_False},{
    L"bind", NSBIF_ParaCalc,  & Lisp2Dbimsocketbind, BoolCheckThis_True,
    TypeGlobalListMemberPLOSocket,
    BoolThisIsConst_False},{
    L"listen", NSBIF_ParaCalc,  & Lisp2Dbimsocketlisten, BoolCheckThis_True,
    TypeGlobalListMemberPLOSocket,
                BoolThisIsConst_False},{
    L"accept", NSBIF_NotCalc,  &  Lisp2Dbimsocketaccept2, BoolCheckThis_True,
    TypeGlobalListMemberPLOSocket, BoolThisIsConst_True},{
   L"read", NSBIF_ParaCalc,  & Lisp2Dbimsocketread2, BoolCheckThis_True,
   TypeGlobalListMemberPLOSocket,
               BoolThisIsConst_True},{
   L"write", NSBIF_ParaCalc, & Lisp2Dbimsocketwrite2, BoolCheckThis_True,
   TypeGlobalListMemberPLOSocket,
    BoolThisIsConst_True},{       
    L"socket-select", NSBIF_ParaCalc, & Lisp2Dbimconssocketselect,
    BoolCheckThis_True,
    TypeGlobalListMemberPLOCons, BoolThisIsConst_True} ,
  { L"truncate", NSBIF_ParaCalc,& Lisp2Dbimstringtruncate2,
    BoolCheckThis_True,  TypeGlobalListMemberPLOString, BoolThisIsConst_True},
  { L"truncate", NSBIF_ParaCalc,& Lisp2Dbimstreamtruncate,
    BoolCheckThis_True,  TypeGlobalListMemberPLOStream, BoolThisIsConst_False
    
  } ,
  {  L"in-byte" , NSBIF_NotCalc , & bimcomplexinbyte  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOComplex ,
    BoolThisIsConst_True  } ,
  {  L"in-byte-pause" , NSBIF_NotCalc , & bimcomplexinbytepause  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOComplex ,
    BoolThisIsConst_True  } ,
  {  L"out-byte" , NSBIF_ParaCalc , & bimcomplexoutbyte  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOComplex ,
    BoolThisIsConst_True  } ,
  {  L"out-byte-pause" , NSBIF_ParaCalc , & bimcomplexoutbytepause  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOComplex ,
    BoolThisIsConst_True  }  ,
  {  L"out-byte-vector" , NSBIF_ParaCalc , & bimcomplexoutbytevector  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOComplex ,
    BoolThisIsConst_True  } ,
  {  L"in-byte-vector" , NSBIF_ParaCalc , & bimcomplexinbytevector  ,
    BoolCheckThis_True  , TypeGlobalListMemberPLOComplex ,
    BoolThisIsConst_True  } ,

  { L"change-owner-group" , NSBIF_ParaCalc  , & Lisp2Dbimstringchownusergroup ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  , BoolThisIsConst_True  } ,
  { L"lchange-owner-group" , NSBIF_ParaCalc  , &
    Lisp2Dbimstringlchownusergroup ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  , BoolThisIsConst_True  } ,
  { L"change-owner-group" , NSBIF_ParaCalc  , & Lisp2Dbimstreamchownusergroup ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOStream  , BoolThisIsConst_True  } ,
  
  
  
  { L"stat" , NSBIF_ParaCalc  , & Lisp2Dbimstringstat ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  , BoolThisIsConst_True  } ,
  { L"lstat"  , NSBIF_ParaCalc  , & Lisp2Dbimstringlstat ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOString  , BoolThisIsConst_True  } ,
  { L"stat" , NSBIF_ParaCalc  , & Lisp2Dbimstreamstat  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOStream  , BoolThisIsConst_True  } ,
  { L"getwinsize" , NSBIF_NotCalc  , & Lisp2Dbimstreamgetwinsize  ,
    BoolCheckThis_True  ,
    TypeGlobalListMemberPLOStream  , BoolThisIsConst_True  }
    };
# ifdef  LISP2D_DEBUG
  size_t  const setTypeSymbolNameTypeSize = MapPubSize  (
    setTypeSymbolNameType ) ( setTypeSymbolNameType_pub ) ;
# endif
      { enum { si = sizeof(arrsetbimargs)/sizeof(arrsetbimargs[0]) } ;
    struct  setbimargs const * const sien = & arrsetbimargs [ si ] ;
    struct  setbimargs const * ip = & arrsetbimargs [ 0 ] ;
    do {
    
      size_t  const jc  = MapPubFind  ( setTypeSymbolNameType ) (
        setTypeSymbolNameType_pub , ( uint8_t ) ( ip -> ct ) ) ;
# ifdef  LISP2D_DEBUG
      if  ( jc  ==  setTypeSymbolNameTypeSize )
        fwprintf  ( stderr  , L"setTypeSymbolName . find ( %u ) == end\n" ,
          ( unsigned int ) ( ip -> ct ) ) ;
      else
# endif
      { struct  sSetBIM a = { . stdclassnamep2 = MapPubCAt ( setTypeSymbolNameType ) ( setTypeSymbolNameType_pub , jc ) -> value ,
          . namec = ip -> namec , . argt = ip -> argt  , . funcaddr2 = ip -> funcaddr2 , . fcheckthis = ip -> fcheckthis  ,
          . ct = ip -> ct  , . stclp = stcl , . stdpropp = stdpro , . flagthisconst = ip -> flagthisconst , . res = 0 } ;
        SetBIM  ( & a ) ;
        PLOBIF_SetExtraThread ( a . res , true  ) ;
      }
      ++  ip  ;
    } while ( ip < sien  ) ;
    }
  }
  
// ! to make static PLOBIF..

  struct setbimargs2{
    PLOBIF  * * const addrpointertobifp ;
    wchar_t const * namec;
    int argt;
    FuncBiningP funcaddr2;
    BoolCheckThis fcheckthis  ;
    int  ct;
    BoolThisIsConst flagthisconst ;
    };
  
  // variable = SetBIM2...
  {struct  setbimargs2 arrsetbimargs[]={
    {
    
  &plobimcomplexplus ,L"+", NSBIF_ParaCalc,& bimcomplexplus,
    BoolCheckThis_True,
                                 TypeGlobalListMemberPLOComplex,
                                 BoolThisIsConst_True},{
  
  &plobimcomplexplussetf ,   L"+=" , NSBIF_ParaCalc  ,
      & bimcomplexplussetf, BoolCheckThis_True, TypeGlobalListMemberPLOComplex,
      BoolThisIsConst_False},{
  
   &plobimcomplexminussetf , L"-=", NSBIF_ParaCalc,& bimcomplexminussetf,
   BoolCheckThis_True,
                                       TypeGlobalListMemberPLOComplex,
                                       BoolThisIsConst_False},{
  
 
// ! to add sinon L"×"
   &plobimcomplexmultiply , L"*", NSBIF_ParaCalc,& bimcomplexmultiply,
   BoolCheckThis_True,
                                     TypeGlobalListMemberPLOComplex,
                                     BoolThisIsConst_True},{
   &plobimcomplexmultiplysetf , L"*=", NSBIF_ParaCalc,& bimcomplexmultiplysetf,
                                         BoolCheckThis_True,
                                         TypeGlobalListMemberPLOComplex,
                                         BoolThisIsConst_False},{
  // ! to add sinon L"×"  
   &plobimvectormult  ,  L"*", NSBIF_ParaCalc,& Lisp2Dbimvectormult2,
   BoolCheckThis_True,
                                 TypeGlobalListMemberPLOVector,
                                 BoolThisIsConst_True},{
  // ! to add sinon L"×"  
   &plobimvectormultsetf , L"*=", NSBIF_ParaCalc,& Lisp2Dbimvectormultsetf2,
   BoolCheckThis_True,
                                     TypeGlobalListMemberPLOVector,
                                     BoolThisIsConst_False},{
   &plobimstringplussetf , L"+=", NSBIF_ParaCalc,& bimstringplussetf,
   BoolCheckThis_True,
                                     TypeGlobalListMemberPLOString,
                                     BoolThisIsConst_False},{
   &plobimnilprogn , L"progn", NSBIF_NotCalc,& bimnilprogn,
    BoolCheckThis_False,
                               TypeGlobalListMemberPLONil,
                               BoolThisIsConst_True}  // exception checked
  
  };
# ifdef  LISP2D_DEBUG
  size_t  const setTypeSymbolNameTypeSize = MapPubSize  (
    setTypeSymbolNameType ) ( setTypeSymbolNameType_pub ) ;
# endif
  { enum { si = sizeof(arrsetbimargs)/sizeof(arrsetbimargs[0]) } ;
    struct  setbimargs2 const * const sien = & arrsetbimargs [ si ] ;
    struct  setbimargs2 const * ip = & arrsetbimargs [ 0 ] ;
    do {
      size_t  const jc  = MapPubFind  ( setTypeSymbolNameType ) (
        setTypeSymbolNameType_pub , ( uint8_t ) ( ip -> ct ) ) ;
# ifdef  LISP2D_DEBUG
      if  ( jc  ==  setTypeSymbolNameTypeSize )
        fwprintf  ( stderr  , L"setTypeSymbolName . find ( %u ) == end\n" , ( unsigned int ) ( ip -> ct ) ) ;
      else
# endif
      { struct  sSetBIM a = { . stdclassnamep2 = MapPubCAt ( setTypeSymbolNameType ) ( setTypeSymbolNameType_pub , jc ) -> value ,
          . namec = ip->namec , . argt = ip->argt  , . funcaddr2 = ip->funcaddr2 , . fcheckthis =ip->fcheckthis  , . ct = ip->ct ,
          . stclp = stcl , . stdpropp = stdpro , . flagthisconst = ip->flagthisconst , . res = 0 } ;
        SetBIM ( & a ) ;
        * ( ip -> addrpointertobifp ) = a . res ;
      }
      Lisp2D_ListPushBack ( PLO ) ( list_global_atomsc  , & ( ( * ( ip -> addrpointertobifp ) ) ->base. plo ) ) ;
      ++  ip  ;
    } while( ip < sien ) ;
  }
  } // listsetbimargs
  
    
// ! file descriptor manipulation: fcntl(int fd,...) - to write
// ! to make flag O_APPEND, O_NONBLOCK, O_ASYNC and O_DIRECT  
// ! set-nonblock, unset-nonblock,...  
  
  
// ! to add : select - socket klient manipulation
  
  
// !? to write: unix-open with many FLAGS, and object: unix-stream
// !? works only on int fd, not fstream 
// !? and make method: fcntl  
  
  // Pointer
  
  // ! to write:  point->number point->object(NOT NEW) number->pointer
  // ? to write: = setf < > <= >= ......
  
  // ! ../test/lazy.lisp2d:
  // ! to write (nil laze ..)
  // ! ('lazy newobject ..)

  return  true;
}

# define  GetPropPairFromVector Lisp2D_GetPropPairFromVectorOhnJ
// returns NIL or (symbol . prop)
static  ParaLispObject  * GetPropPairFromVector(ParaLispObject  const *,
  ParaLispObject  const *,ParaLispObject  *, StackClusterPrivate  * ) ;
  
# include "string/public.h"
# include "env/public.h"
# include "stack/public.h"
  
// ! replace to new setbim.cpp  
// ! to return flag BAD  
// setting const variable  
void  SetBIM ( struct  sSetBIM * const s ) {
  PLOBIF * const addrf = aligned_alloc ( _Alignof ( PLOBIF ) , sizeof ( PLOBIF ) ) ;
  if ( not addrf ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"SetBIM:addrf = aligned_alloc(sizeof(PLOBIF))" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }

  PLOBIF_Init ( ( t_PLOBIF_Init ) { . me = addrf , . at = s -> argt  , . na = s -> namec , . fcheckthis =  s -> fcheckthis  ,
    . ct = s -> ct  , . flagthisconst = s -> flagthisconst } ) ;

  BIFPrivate  * addrfp  = & addrf -> base ;

  ParaLispObject_virtualPushAsNew ( & addrfp  ->  plo ) ;

  addrfp  ->  funcbining1  = s -> funcaddr2 ;
  addrfp  ->  symbolClassName  = s -> stdclassnamep2 ;
  ParaLispObject  const *   stdclass  ;
  if  ( s -> stdclassnamep2 == & plosnil )
    stdclass  = &  plonil  . plo  ;
  else if (  s -> stdclassnamep2 ==  & plostrue1  )
    stdclass  =  & plotrue . plo  ;
  else if ( s -> stdclassnamep2 == & plosfalse  )
    stdclass  = &  plofalse  . plo  ;
  else
    stdclass = & s -> stdclassnamep2 -> plo ;

  ParaLispObject  *   classj  = GetPropPairFromVector ( s -> stdpropp , stdclass  ,
    &  plossystemclass . plo  , s -> stclp  ) ;

# ifdef  LISP2D_DEBUG
  if  ( CConsp  (  classj ) )
    classj = ClassCons_Rest ( & ParaLispObjectToPLOCons ( classj  ) ->  cc ) ;
  else  {
    // to exit
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy (
      L"SetBIM: class " ) ->  plo , & s -> stdclassnamep2 ->  plo ,
      &  PLOString_NewCopy  ( L" is not defined"  ) ->  plo , 0 } ;
    StackClusterPrivate_ExceptionSet ( s -> stclp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    
    // to write
    
    bool  const flag=false;
    fputws(L"SetBIM: class ",stderr);
    { ThreadStackPointerType const oldstack_hi = Lisp2DThreadSP ( ) ;
      Vector  ( StackPLOP ) * const hip2 = StackNew_VectorPLOP ( ) ;
      VectorInit  ( StackPLOP ) ( hip2 , false ) ;
      ParaLispObject_CoutToInterpret  ( & s -> stdclassnamep2->plo , & Lisp2Dferr  ,
        & flag  , hip2 ) ;
      Lisp2D_StackFree  ( oldstack_hi ) ;
    }
    fputws(L" is not defined\n",stderr);
    s -> res = addrf ;
    return ;
  }
# else
  classj  =   CCDR ( classj );
# endif
  PLOCons * const envp3 = NewCons ( ClassGetMethods ( classj  ) , & plonil . plo  ) ;
  ParaLispObject  * classnewsymbolp ;
  String  ( WideChar ) * wsp  ;
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    wsp = Lisp2D_StackVar ( Lisp2DStackTypeStringWideChar ) ;
    StringConstrCStr ( WideChar )  ( wsp , s -> namec , true ) ;
    classnewsymbolp  = SymbolAddMoveString ( wsp ) ;
    Lisp2D_StackFree  ( oldstack ) ;
  }

# ifdef  LISP2D_DEBUG
  if  ( Assoc ( classnewsymbolp , & envp3 ->  plo , s -> stclp ) ) {
    // to exit
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(
      L"SetBIM: method ")->plo,& s -> stdclassnamep2->plo,
      &  PLOString_NewCopy(L"::")->plo,classnewsymbolp, & 
      PLOString_NewCopy(L" already defined")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet ( s -> stclp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    
    // to write
    
    fputws(L"SetBIM: method ",stderr);
    { ThreadStackPointerType const oldstack_hi = Lisp2DThreadSP ( ) ;
      Vector  ( StackPLOP ) * const hi2 = StackNew_VectorPLOP ( ) ;
      
      bool  const flag=false;
      VectorInit  ( StackPLOP ) ( hi2 , false ) ;
      ParaLispObject_CoutToInterpret  ( & s -> stdclassnamep2->plo   , &
        Lisp2Dferr  , & flag  , hi2 ) ;
      fputws(L"::",stderr);
      if  ( not  (  VectorEmpty ( StackPLOP ) ( hi2 ) ) ) {
        fputws(L" ! not  (hi.Empty()) ! ",stderr);
        fflush(stderr);
        Lisp2DStackPLOPClear ( hi2 ) ;
      }
      ParaLispObject_CoutToInterpret  ( classnewsymbolp   , & Lisp2Dferr  ,
        & flag  , hi2 ) ;
      Lisp2D_StackFree  ( oldstack_hi ) ;
    }
    fputws(L" already defined\n",stderr); 
  }
# endif
  if  ( IsNotEnvironmentLayer ( s -> stclp , ClassCons_First ( & envp3 -> cc  ) ) )  {
    s -> res = addrf ;
    return ;
  }
  if  ( EnvironmentLayerisBADPush (
      & ParaLispObjectToPLOEnvironmentLayer ( ClassCons_First (
      & envp3 -> cc ) ) -> envlay , classnewsymbolp  , & addrfp  ->  plo , true  ) ) {
    fputws(L"SetBIM:isBADPush:",stderr);
    ThreadStackPointerType const oldstack_hi = Lisp2DThreadSP ( ) ;
    Vector  ( StackPLOP ) * const hi2 = StackNew_VectorPLOP ( ) ;
    bool  const flag=false;
    VectorInit  ( StackPLOP ) ( hi2 , false ) ;
    ParaLispObject_CoutToInterpret  ( classnewsymbolp  , & Lisp2Dferr  , & flag  , hi2 ) ;
    Lisp2D_StackFree  ( oldstack_hi ) ;
    fputws(L"\n",stderr); 
  }
  // to make static member : (this += x) => (nil number::+= this x)
  ThreadStackPointerType const oldstack3 = Lisp2DThreadSP ( ) ;
  String  ( WideChar ) * const namecstaticp = Lisp2D_StackVar ( Lisp2DStackTypeStringWideChar ) ;
  String_ConstrCopy ( WideChar )  ( wsp , PLOSymbolCValue  ( s -> stdclassnamep2  ) ) ;
  StringAddCharArray  ( WideChar  ) ( namecstaticp , L"::" ) ;
  StringAddCharArray  ( WideChar  ) ( namecstaticp , s -> namec ) ;
  PLOBIF * const addrfstatic = aligned_alloc ( _Alignof ( PLOBIF ) , sizeof ( PLOBIF ) ) ;
  if ( not addrfstatic ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"SetBIM:addrfstatic = aligned_alloc(sizeof(PLOBIF))" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }

  PLOBIF_Init ( ( t_PLOBIF_Init ) { . me = addrfstatic , . at = s -> argt  , . na = StringCStr  ( WideChar  ) ( namecstaticp  ) ,
    . fcheckthis = s -> fcheckthis  , . ct = s -> ct  , . flagthisconst = s -> flagthisconst } ) ;

  BIFPrivate  * addrfpstatic  = & addrfstatic -> base ;
  ParaLispObject_virtualPushAsNew (&addrfpstatic  ->  plo ) ;
  addrfpstatic  ->  funcbining1  = s -> funcaddr2  ;
  addrfpstatic  ->  symbolClassName  = & plosnil  ;
  ParaLispObject  * classjstatic  = GetPropPairFromVector ( s -> stdpropp ,
    & plonil  . plo  , & ( plossystemclass  . plo ) , s -> stclp ) ;
  classjstatic  = CCDR (  classjstatic  )  ;
  PLOCons * const envstaticp  = NewCons ( ClassGetMethods (  classjstatic  ) , & plonil  . plo ) ;
  ParaLispObject  * const classnewsymbolstaticp= SymbolAddMoveString ( namecstaticp ) ;
  Lisp2D_StackFree  ( oldstack3 ) ;
  if  ( IsNotEnvironmentLayer ( s -> stclp , ClassCons_First(&envstaticp->cc ) ) ) {
    s -> res = addrf ;
    return  ;
  }
  if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
        ClassCons_First(&envstaticp ->cc) ) -> envlay ,
        classnewsymbolstaticp  ,  &  addrfpstatic ->  plo , true  ) ) {
    fputws(L"SetBIM:static:isBADPush:",stderr);
    ThreadStackPointerType const oldstack_histatic = Lisp2DThreadSP ( ) ;
    Vector  ( StackPLOP ) * const histatic2 = StackNew_VectorPLOP ( ) ;
    VectorInit  ( StackPLOP ) ( histatic2 , false ) ;
    { bool const f = false ;
      ParaLispObject_CoutToInterpret  ( classnewsymbolstaticp  , & Lisp2Dferr  , & f , histatic2 ) ;
    }
    Lisp2D_StackFree  ( oldstack_histatic ) ;
    fputws(L"\n",stderr);
  }
  s -> res = addrf ;
}
// !!! functions must use order PML -> PMU

# include "vector/public.h"

static  ParaLispObject  * GetPropPairFromVector (
  ParaLispObject  const * const pro , ParaLispObject const * const sp ,
  ParaLispObject  * const pp  , StackClusterPrivate  * const stclp
# ifndef  LISP2D_DEBUG
    __attribute__ ((unused))
# endif
  ) {
  if(not  Vectorp(pro))  
    return & plonil.plo;
  ParaLispObject  *   sl= PLOVector_Assoc3(ParaLispObjectToPLOVectorConst(pro),sp);
  if(not  CConsp( sl)) 
    return & plonil.plo;
  sl  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( sl  )->cc ) ;
# ifdef  LISP2D_DEBUG
  if(not  Vectorp( sl)){
    if(stclp){
      ParaLispObject  * list [ ] = { & PLOString_NewCopy(
        L"GetPropPairFromVector: ")->plo,
        sl,& PLOString_NewCopy(L" is not a vector")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclp,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    } else
      inErrorS(PLOStreamPushWideCharArray  ( &Lisp2Dferr ,
          (PArray_wchar_c) { (wchar_t const(*)[])& L"GetPropPair: "});
    PLOStreamPushPLO(& Lisp2Dferr ,  sl);
    PLOStreamPushWideCharArray  ( &Lisp2Dferr ,
      (PArray_wchar_c) { (wchar_t const(*)[])&  L" is not a vector"});)
    return & plonil.plo;
  }
# endif
  sl  =  PLOVector_Assoc3(ParaLispObjectToPLOVector(sl),pp);
  if(CConsp(  sl)) 
    return  sl;
  return & plonil.plo;
}
# include "widestring/inline.h"
# include "stat/inline.h"
# include "socket/inline.h"
# include "socket/inlinepri.h"
