// Lisp2D - Signal Lamp
// ©2008-23 by Alex Glebe
# include "define.h"
# include "signall.h"

# ifdef  LISP2D_CONSTRSTAT

_Atomic ( size_t ) PLOSignal_constrstat = 0 ;

size_t  PLOSignal_constrstat_Value  ( void ) {
  return  PLOSignal_constrstat  ;
}

void  PLOSignal_constrstat_Inc  ( void ) {
  ++  PLOSignal_constrstat  ;
}

# endif

ParaLispObject  * PLOSignal_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassdefsignal ;
}

ParaLispObject  * PLOSignal_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassparentssignal ;
}

# include "symbolc.h"

ParaLispObject  * PLOSignal_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return & plossignal . plo  ;
}

# include "plostream/public.h"

void  PLOSignal_staticCoutToInterpret ( ParaLispObject const * const _ __attribute__ ((unused)) ,
  PLOStream  const * const s , bool const _2 __attribute__ ((unused)) ,
  Vector ( StackPLOP )  * const _3 __attribute__ ((unused)) )  {
  PLOStreamPushWideCharArray  ( s , ( PArray_wchar_c ) { (  wchar_t const ( * ) [ ] ) & L"Signal" } ) ;
}

bool  PLOSignal_staticisBADCopyPrivate ( ParaLispObject const * _ __attribute__ ((unused)) ,
  ParaLispObject  * * const resultp  )  {
  PLOSignal * const res = aligned_alloc ( _Alignof ( PLOSignal ) , sizeof  ( PLOSignal ) ) ;
  if ( res ) {
    PLOSignal_Init  ( res ) ;
    ParaLispObject_virtualPushAsNew ( & res -> plo ) ;
    ( * resultp )  = &  res ->  plo ;
    return  false ;
  }
  return  true  ;
}

# include "boolean.h"
# include "cons/inline.h"
# include "stcl/public.h"
# include "stcl/inline.h"

// nil signalp obj
TaskForThread bimnilsignalp ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject * const xp =  CCAR ( StackClusterPrivate_ArgTop  ( stclpp  ) ) ;
  StackClusterPrivate_SetResult ( stclpp , ( Signalp ( xp ) ? xp : & plofalse  . plo  ) ) ;
  return  TaskForThread_Analyse ;
}

# include "sharedc.h"
# include "shared/inline.h"
# include "newobject.h"
# include "mutex2/public.h"

void  PLOSignal_Send  ( PLOSignal * const me ) { 
  Mutex2Lock ( & me -> mutexc )  ;
  if  ( QueueEmpty  ( VoidP ) ( & me -> stclpqueueC . pub ) ) {
    ++  ( me -> sended ) ;
    if  ( me -> sended ==  0  ) {
      --  ( me -> sended ) ;
      Mutex2Unlock  ( & ( me -> mutexc ) )  ;
      Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & L"PLOSignal_Send:sended count overflow" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    }
    Mutex2Unlock  ( & me -> mutexc ) ;
  } else  {
    { void  * stclpp ;
      // may be: out of memory 
      if  ( QueueIsBadPop ( VoidP ) (
          & me -> stclpqueueC . pub , & stclpp ) ) {
        Mutex2Unlock  ( & me -> mutexc )  ;
        Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
          L"PLOSignal_Send:stclqueue.Pop()" ;
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
      }
      Mutex2Unlock  ( & me -> mutexc )  ;  
      StackClusterPrivate_setTaskForManager (
        ( StackClusterPrivate  * ) ( stclpp ) , TaskRun ) ;
      TaskQueuePush ( ( StackClusterPrivate  * ) ( stclpp ) ) ;
    } /// stclp
    Mutex2Lock ( & Lisp2Dmutex2ManagerTasks ) ;
    if  ( flagManagerWaitSignal ) {
      flagManagerWaitSignal = false ;
      jmp_buf jb2 ;
      jmp_buf * const oldjb = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2  )  ;
      if  ( jbn ) {
        Lisp2DJBglobal  = oldjb ;
        Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
        longjmp ( * Lisp2DJBglobal  , jbn ) ;
      }
      ConditionSignal ( & Lisp2DsignalManager ) ;
      Lisp2DJBglobal  = oldjb ;
    }
    // nobody didnot signaling as flagManagerWaitSignal=false
    // manager will little wait
    Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
  }
}

# include "nilc.h"

// signal send
TaskForThread bimsignalsend ( StackClusterPrivate  * const stclpp  ) {
  PLOSignal_Send  (
    ParaLispObjectToPLOSignal ( StackClusterPrivate_ThisTop ( stclpp  ) ) ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo  ) ;
  return  TaskForThread_Analyse ; 
}

bool  PLOSignal_isOkGetSignal ( PLOSignal * const me ,
  StackClusterPrivate * const stclp ) {
  Mutex2Lock ( & ( me -> mutexc ) ) ;
  if  ( ( me -> sended ) not_eq  0  ) {
    --  ( me -> sended ) ;
    Mutex2Unlock  ( & me -> mutexc ) ;
    return  true  ;
  }
  if  ( QueueIsBadPushByValue ( VoidP ) (
      & me -> stclpqueueC . pub , stclp ) ) {
    Mutex2Unlock  ( & me -> mutexc ) ;
    Lisp2DExceptionStringGlobal . str = ( wchar_t const ( * ) [ ] ) & 
      L"PLOSignal_isOkGetSignal:QueueIsBadPushByValue" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
  Mutex2Unlock  ( & ( me -> mutexc ) ) ;
  return  false ;
}

void  PLOSignal_Destr ( PLOSignal * const me ) {
  jmp_buf jb  ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = &  jb ;
  int const volatile jbt = setjmp  ( jb  ) ;
  if  ( jbt  ==  0 )
    Mutex2Destructor  ( & me -> mutexc ) ; 
  // else remember and pass exception up
  QueueDestructor ( VoidP ) ( & me -> stclpqueueC . pub ) ;
  Lisp2DJBglobal = oldjb  ;
  if ( jbt == Lisp2DExcTypeString )
    fputws  ( & ( ( * Lisp2DExceptionStringGlobal . str ) [ 0 ] ) , stderr  ) ;
  ParaLispObject_staticDestr  ( & me  ->  plo ) ;
}

void  PLOSignal_Init ( PLOSignal * const me ) {
  me -> sended = 0 ;
  ParaLispObject_InitVTP  ( & me -> plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOSignal } ) ;
  { jmp_buf jb  ;
    jmp_buf * const oldjb = Lisp2DJBglobal  ;
    Lisp2DJBglobal = &  jb ;
    int const volatile jbt = setjmp  ( jb  ) ;
    if  ( jbt ) {
      Lisp2DJBglobal = oldjb ;
      longjmp ( * Lisp2DJBglobal  , jbt ) ; 
    }
    Mutex2Constructor  ( & me -> mutexc , NULL ) ;
    QueueConstructorDefault ( VoidP ) ( & me -> stclpqueueC . pub ) ;
    Lisp2DJBglobal = oldjb ;
  }
# ifdef  LISP2D_CONSTRSTAT
  PLOSignal_constrstat_Inc  ( ) ;
# endif
}

void  PLOSignal_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool flag2pprinted  , Vector ( StackPLOP ) * const h ) {
  PLOSignal_staticCoutToInterpret (  me ,  s ,  flag2pprinted  ,  h ) ;
}

bool  IsNotSignal ( StackClusterPrivate  * const stclp  , ParaLispObject  * const xp ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp ,
    . typetype  = TypeGlobalListMemberPLOSignal ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
  //return  UniversalIsNotType  ( stclp , xp , TypeGlobalListMemberPLOSignal  ) ;
}

# include "inerrors.h"
# include "lock/public.h"

// signal wait lock
TaskForThread Lisp2Dbimsignalwait2  ( StackClusterPrivate  * const stclpp  ) {
  { EvalAddrCont2 const  addr = StackClusterPrivate_ClearflagCont  ( stclpp ) ;
    if  ( addr ) {
      switch  ( addr  ) {
      case  1 :
        StackClusterPrivate_SetResult (stclpp ,& plonil.plo  ) ;
        goto  Exit  ;
      case  2 :
        goto  Next2 ;
      default :
# ifdef LISP2D_DEBUG
        inErrorS  ( PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
          ( PArray_wchar_c ) { (wchar_t const(*)[])& L"signal wait: addr="});
        PLOStreamPushEvalAddrCont(& Lisp2Dferr ,  addr ) ; )
# endif
        goto  Exit  ;
      } // switch addr
    } // if addr
  } // addr
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSignal ) ;
  StackClusterPrivate_Push (stclpp ,StackClusterPrivate_ThisTop(stclpp) ) ;
  StackClusterPrivate_Push ( stclpp , CCAR ( StackClusterPrivate_ArgTop  ( stclpp  ) ) ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
  goto  Exit  ;
Next2 :
  { ParaLispObject  * const lp = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) ) ;
    if  ( Lockp ( lp ) and
      // lock set
      ( not ( PLOLock_isOKTryLock ( ParaLispObjectToPLOLock  ( lp ) ) ) ) ) {
        // cant
      StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ; 
      StackClusterPrivate_setTaskForManager (
        stclpp , TaskWaitUnLock2 ) ;
      StackClusterPrivate_Push ( stclpp , lp ) ;
    }
  } // lp
Exit  :
  return  TaskForThread_Analyse ;
}

void PLOSignal_staticDestr  ( ParaLispObject * const me ) {
  PLOSignal_Destr ( ParaLispObjectToPLOSignal ( me  ) ) ;
}
