// Lisp2D - Socket internet server/user
// ©2015-23 by Alex Glebe
// # define  _POSIX_C_SOURCE 200112L
/*
  domain:
      PF_UNIX,PF_LOCAL   Локальная коммуникация               unix(7)
      PF_INET            IPv4, протоколы Интернет             ip(7)
      PF_INET6           IPv6, протоколы Интернет
      PF_IPX             IPX - протоколы Novell
      PF_NETLINK         Устройство       для       общения   netlink(7)
                          пользователя с ядром
      PF_X25             Протокол ITU-T X.25 / ISO-8208       x25(7)
      PF_AX25            Протокол AX.25, любительское радио
      PF_ATMPVC          ATM -- доступ к низкоуровневым PVC
      PF_APPLETALK       Appletalk                            ddp(7)
      PF_PACKET          Низкоуровневый пакетный интерфейс    packet(7)  
*/
/*
  type:
      SOCK_STREAM
              Обеспечивает  надежные,  двунаправленные  последовательные
              потоки  байтов, с поддержкой соединений.  Может также
              поддерживаться механизм вне-поточных данных.

      SOCK_DGRAM
              Обеспечивает датаграммы (ненадежные сообщения с ограниченной
              максимальной длиной, без поддержки соединения).

      SOCK_SEQPACKET
              Обеспечивает последовательный двунаправленный канал передачи
              датаграмм с поддержкой соединений; датаграммы  имеют
              ограниченную максимальную длину; от получателя требуется за один
              раз прочитать целый пакет.

      SOCK_RAW
              Обеспечивает доступ к низкоуровневому сетевому протоколу.

      SOCK_RDM
              Обеспечивает надежную доставку датаграмм без гарантии их
              последовательности.
*/
/*
  errno:
      EPROTONOSUPPORT
               Тип протокола или указанный протокол не поддерживаются в этом
               домене.

      ENFILE  Ядру не хватило памяти, чтобы создать новый сокет.

      EMFILE  Переполнение таблицы файлов процесса.

      EACCES  Не разрешено создание сокета указанного типа и/или протокола.

      ENOBUFS или ENOMEM
               Недостаточно памяти.  Сокет не может быть создан, пока не
               освободится память.

      EINVAL  Неизвестный протокол, или недоступный набор протоколов.
*/

# include "plo/struct.h" // ParaLispObject
# include "socket/public.h" // PLOSocket_staticCoutToInterpret

void  PLOSocket_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  , Vector(StackPLOP)  * const h ) {
  PLOSocket_staticCoutToInterpret ( me ,    s , flag2pprinted  , h ) ;
}

# include "symbolc.h" // plossocket
  
ParaLispObject  * PLOSocket_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  & plossocket  . plo ;
}
  
ParaLispObject  * PLOSocket_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassdefsocket ;
}
  
ParaLispObject  * PLOSocket_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassparentssocket ; 
}

# include "nilc.h"
# include "stcl/public.h"
# include "stcl/inline.h"

// socket SOCKET
TaskForThread Lisp2Dbimsocketsocket2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;
}

# include "define.h"
  
# ifdef  LISP2D_CONSTRSTAT

_Atomic ( size_t  ) PLOSocket_constrstat = 0 ;

size_t  PLOSocket_constrstatValue ( void  ) {
  return  PLOSocket_constrstat  ; }

void  PLOSocket_constrstatInc ( void  ) {
  ++  PLOSocket_constrstat  ;
}

# endif // LISP2D_CONSTRSTAT

# include "socket/private.h" // PLOSocket_Destr PLOSocketPriFromPub
# include "socket/public.h" // ParaLispObjectToPLOSocket

void PLOSocket_staticDestr  ( ParaLispObject * const me ) {
  PLOSocket_Destr(PLOSocketPriFromPub(ParaLispObjectToPLOSocket ( me  )));
}

# include <unistd.h> // close

void  PLOSocket_Destr(PLOSocketPri*const mepri) {
  if(mepri->descriptor not_eq  -1) {
    shutdown(mepri->descriptor,SHUT_RDWR);
    close(mepri->descriptor);
  }
  ParaLispObject_staticDestr(&mepri->pub.plo);
}

# include <netdb.h> // setprotoent
  
// nil open-protoent
// opens a connection to the database
TaskForThread Lisp2Dbimnilopenprotoent2( StackClusterPrivate * const stclpp){
  setprotoent(1);
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;}

// nil close-protoent
// closes the connection to the database
TaskForThread Lisp2Dbimnilcloseprotoent2(StackClusterPrivate  * const stclpp){
  endprotoent();
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;}

// nil get-protoent
// reads the next entry from the protocols database
// returns list(number name alias .. alias) or nil
TaskForThread Lisp2Dbimnilgetprotoent2(StackClusterPrivate  * const stclpp){
  struct protoent  *   strprot = getprotoent();
  if(strprot)
    StackClusterPrivate_SetResult(stclpp ,ProtoEntToList( strprot,stclpp));
  else  
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse;}

# include "streamc.h"
# include "plostream/public.h"
# include "plostream/struct.h"

void  PLOSocket_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool _1 __attribute__ ((unused)) ,
  Vector(StackPLOP)  * _2 __attribute__ ((unused)) ) {
  StreamPushWCharArray(&s->str,(PArray_wchar_c){(wchar_t const (*)[])(L"Socket(")});
  PLOSocketPri const * const so =
    PLOSocketCPriFromCPub(ParaLispObjectToPLOSocketConst(me));
  PLOStreamPushInt(s,so ->descriptor);
  PLOStreamPushWChar(s,L')');  }  

// socket read #[buf] [size [{string-flag}]]
// returns byte-received | string
TaskForThread Lisp2Dbimsocketread2(StackClusterPrivate  * const stclpp){
  void  * buf;
  return bimsocketread22( stclpp ,& buf); }

# include <errno.h> // EPROTONOSUPPORT
# include "string/public.h" // PLOString_NewCopy
# include "string/struct.h" // -> plo
# include "passage/public.h"

void bimsocketopen22(StackClusterPrivate * const stclpp ,int cie,
  wchar_t const * * const wc){
  switch(cie){
    case  EPROTONOSUPPORT:
      (*wc)  = L"PROTONOSUPPORT";
      break;
    case  ENFILE:
      (*wc)  = L"NFILE";
      break;
    case  EMFILE:
      (*wc)  = L"MFILE";
      break;
    case  EACCES:
      (*wc)  = L"ACCES";
      break;
    case  ENOBUFS:
      (*wc)  = L"NOBUFS";
      break;
    case  ENOMEM:
      (*wc)  = L"NOMEM";
      break;
    case  EINVAL:
      (*wc)  = L"INVAL";
      break;
    default :    
      (*wc)  = L"?";
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclpp ,&PLOString_NewCopy(*wc)->plo);
  PassagePassOut ( passageGCCPub ) ; 
}

void   bimsocketclose22( StackClusterPrivate  * const stclpp , wchar_t const * * const wc,int const intErrnoClose){
  switch(intErrnoClose){
      case  EBADF:
        (*wc)  = L"close:BADF";
        break;
      case  EINTR:
        (*wc)  = L"INTR";
        break;
      case  EIO:
        (*wc)  = L"IO";
        break;
      default:
        (*wc)  = L"close:?";
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclpp ,& PLOString_NewCopy (*wc)->plo);
  PassagePassOut ( passageGCCPub ) ;     
}

# include "sharedc.h"
# include "shared/inline.h" // OutOfMemoryLockGC

// socket accept
// returns error - string
// ok - new socket
TaskForThread Lisp2Dbimsocketaccept2  ( StackClusterPrivate  * const stclpp){
  PLOSocket * const mep = ParaLispObjectToPLOSocket( StackClusterPrivate_ThisTop(stclpp ));
  errno = 0 ;
// ! to return SymbolErrno  
  { int const   ui = accept(PLOSocket_Descriptor(mep),0,0) ;
  if(ui  < 0){
    int ierror = errno ;
    wchar_t const * ws  ;
    switch(ierror){
    case  EAGAIN:
      ws  = L"AGAIN";
      break;
    case  EBADF:
      ws  = L"BADF";
      break;
    case  ENOTSOCK:
      ws  = L"NOTSOCK";
      break;
    case  EOPNOTSUPP:
      ws  = L"OPNOTSUPP";
      break;
    case  EINTR:
      ws  = L"INTR";
      break;
    case  ECONNABORTED:
      ws  = L"CONNABORTED";
      break;
    case  EINVAL:
      ws  = L"INVAL";
      break;
    case  EMFILE:
      ws  = L"MFILE";
      break;
    case  ENFILE:
      ws  = L"NFILE";
      break;
    case  EFAULT:
      ws  = L"FAULT";
      break;
    case  ENOBUFS:
      ws  = L"NOBUFS";
      break;
    case  ENOMEM:
      ws  = L"NOMEM";
      break;
    case  EPROTO:
      ws  = L"PROTO";
      break;
    case  EPERM:
      ws  = L"PERM";
      break;
    case  ENOSR:
      ws  = L"NOSR";
      break;
    case  ESOCKTNOSUPPORT:
      ws  = L"SOCKTNOSUPPORT";
      break;
    case  EPROTONOSUPPORT:
      ws  = L"PROTONOSUPPORT";
      break;
    case  ETIMEDOUT:
      ws  = L"TIMEDOUT";
      break;
    default: 
      ws  = L"?";
    }
    PassagePassIn ( passageGCCPub ) ;
    { jmp_buf jb2  ;
      jmp_buf * const jbold = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2 ) ;
      if  ( jbn  ) {
        Lisp2DJBglobal  = jbold ;
        PassagePassOut ( passageGCCPub ) ;
        OutOfMemory ( stclpp ) ;
        return   TaskForThread_Analyse  ;
      }
      PLOStringPub * const newstr  = PLOString_NewCopy ( ws ) ;
      Lisp2DJBglobal  = jbold ;
      StackClusterPrivate_SetResult ( stclpp ,  & newstr -> plo ) ;
    } // newstr
    PassagePassOut ( passageGCCPub ) ;
    // ui  < 0
  } else {
    PLOSocketPri * const result = aligned_alloc ( _Alignof ( PLOSocketPri ) , sizeof ( PLOSocketPri ) ) ;
    if( result ){
      PLOSocket_InitD ( & result  ->  pub , ui  ) ;
      PassagePassIn ( passageGCCPub ) ;
      ParaLispObject_virtualPushAsNew(&result->pub.plo);
      StackClusterPrivate_SetResult(stclpp ,&  result->pub.plo);
      PassagePassOut ( passageGCCPub ) ;   
    }  else {
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
  }
  } // ui
  return TaskForThread_Analyse;
}

# include "cons/public.h" // CCAR
# include "boolean.h" // plofalse
  
// nil socketp
TaskForThread Lisp2Dbimnilsocketp2( StackClusterPrivate  * const stclpp){
  ParaLispObject  * const xp= CCAR  ( StackClusterPrivate_ArgTop(stclpp ));
  StackClusterPrivate_SetResult(stclpp , (SocketP(xp) ? xp : &plofalse.plo ));
  return TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimnilsocketselect( StackClusterPrivate * const stclpp ) {
  StackClusterPrivate_SetResult(stclpp ,& plonil.plo );
  return TaskForThread_Analyse;
}
  
# include <sys/un.h> // struct  sockaddr_un
# include "stack/public.h"

bool  PLOSocket_isBADConnectPort  ( PLOSocket * const mepub , uint16_t  const port  ,
  wchar_t volatile const * const addr, int  * const localerrorp , int * const intErrorp ) {
  * localerrorp  = 0;
  * intErrorp  = 0;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const addr2 = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( addr2 , true ) ; // "192.168.1.3"
  if ( isBADReadWCharToCharString ( addr , addr2 ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    * localerrorp  = 2;
    return  true  ;
  }
  { struct  addrinfo  * airesult;
    { unsigned  int const uiport =  port ;
      char  sport  [ 60  ] ;
      sprintf ( sport  , "%u"  , uiport  ) ;
      struct  addrinfo  hints;
      // Obtain address(es) matching host/port
      memset(&  hints,0,sizeof(struct addrinfo));
      hints.ai_family = AF_UNSPEC;    // Allow IPv4 or IPv6
      hints.ai_socktype = SOCK_STREAM;  // or SOCK_DGRAM; // Datagram socket
      hints.ai_flags  = 0;
      hints.ai_protocol = 0;          // Any protocol
      hints.ai_flags  = 0;
      //hints.ai_flags  = AI_PASSIVE;     // заполните мой IP-адрес за меня
      * intErrorp  = getaddrinfo  ( StringCStr ( Char  ) ( addr2  ) , sport , & hints ,
        & airesult  ) ;
    }
    Lisp2D_StackFree  ( oldstack_a ) ;
    if(*intErrorp)
      return  true;
    for(struct  addrinfo  * rp  = airesult;rp;rp  = (rp->ai_next)){
      int const   sfd = socket(rp->ai_family,rp->ai_socktype,      rp->ai_protocol);
      if(sfd  ==  -1)
        continue;
      if(connect(sfd,rp->ai_addr,rp->ai_addrlen)  not_eq  -1){
        PLOSocketPri * const me = PLOSocketPriFromPub(mepub);
        me -> descriptor  = sfd;
        // No longer needed
        freeaddrinfo(airesult);
        // Success
        return  false;
      }
      close(sfd);
    }
    // No address succeeded
    // No longer needed
    freeaddrinfo(airesult);
  }  // airesult
  * localerrorp  = 6;
  return  true;
}
  
bool  PLOSocket_isBADConnectPath(PLOSocket * const mepub ,
  wchar_t volatile const * const addr , int  * const localerrorp, int  * const aierrorp ) {
  * localerrorp  = 0;
  * aierrorp = 0;
  struct  sockaddr_un sa;
  { ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const addr2 = StackNew_StringChar ( ) ;
    String_Init ( Char  ) ( addr2 , true ) ; // "/home/alex/socket"
    if ( isBADReadWCharToCharString ( addr , addr2 ) ) {
      Lisp2D_StackFree  ( oldstack_a ) ;
      * localerrorp  = 1;
      return  true  ;
    }
    strcpy(sa.sun_path,StringCStr ( Char  ) ( addr2 ));
    Lisp2D_StackFree  ( oldstack_a ) ;
  }
  sa.sun_family = AF_UNIX;
  errno = 0 ;
  PLOSocketPri * const me = PLOSocketPriFromPub(mepub);
  if(connect(me -> descriptor, ( struct sockaddr  * )(  & sa),sizeof(sa))){
    * aierrorp = errno;
    return  true;
  }
  return  false;
}

TaskForThread Lisp2Dbimsocketclose2(StackClusterPrivate  * const stclpp){
  PLOSocket * const mep =ParaLispObjectToPLOSocket(
    StackClusterPrivate_ThisTop(stclpp ));
  bool  flagErrorShutdown;
  bool  flagErrorClose;
  int intErrnoShutdown=0;
  int intErrnoClose=0;
  int const descriptor = PLOSocket_Descriptor(mep);
  if(descriptor ==  -1){
    flagErrorShutdown = false;
    flagErrorClose  = false;}
  else{
    errno = 0 ;
    if(shutdown(descriptor,SHUT_RDWR)){
      flagErrorShutdown = true;
      intErrnoShutdown = errno;}
    else   
      flagErrorShutdown = false;
    errno =  0 ;
    if(close(descriptor)){
      flagErrorClose = true;
      intErrnoClose = errno;}
    else{
      PLOSocket_ClearDescriptor(mep);
      flagErrorClose = false;
    }
  }
// ! to return SymbolErrno      
  if(flagErrorShutdown){
    Lisp2DThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( WideChar  ) * const wstrres = StackNew_StringWideChar ( ) ;
    String_Init ( WideChar  ) ( wstrres , true ) ;
    switch(intErrnoShutdown){
    case  EBADF:
      StringSetArray(WideChar)(wstrres , L"shutdown:BADF");
      break;
    case  ENOTSOCK:
      StringSetArray(WideChar)(wstrres , L"NOTSOCK");
      break;
    case  ENOTCONN:
      StringSetArray(WideChar)(wstrres , L"NOTCONN");
      break;
    default:
      StringSetArray(WideChar)(wstrres , L"shutdown:?"); }
    if(flagErrorClose){
      switch(intErrnoClose){
      case  EBADF:
        StringAddCharArray(WideChar)(wstrres ,  L",close:BADF");
        break;
      case  EINTR:
        StringAddCharArray(WideChar)(wstrres , L",INTR");
        break;
      case  EIO:
        StringAddCharArray(WideChar)(wstrres , L",IO");
        break;
      default :     
        StringAddCharArray(WideChar)(wstrres , L",close:?");
      } 
    }
    PassagePassIn ( passageGCCPub ) ;
    { jmp_buf jb2  ;
      jmp_buf * const jbold = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2 ) ;
      if  ( jbn  ) {
        Lisp2DJBglobal  = jbold ;
        PassagePassOut ( passageGCCPub ) ;
        Lisp2D_StackFree  ( oldstack_a ) ;
        return  TaskForThread_Analyse ;
      }
      PLOStringPub * const newstr = PLOString_NewCopy ( StringCStr ( WideChar  )(wstrres) ) ;
      Lisp2DJBglobal  = jbold ;
      StackClusterPrivate_SetResult (stclpp ,&  newstr  ->  plo ) ;
    }
    PassagePassOut ( passageGCCPub ) ;   
    Lisp2D_StackFree  ( oldstack_a ) ;
    return  TaskForThread_Analyse ; 
  }
  if(flagErrorClose){
    wchar_t const * wc ;
    bimsocketclose22( stclpp ,& wc,intErrnoClose);
    return TaskForThread_Analyse;
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclpp ,& plonil.plo );
  PassagePassOut ( passageGCCPub ) ;
  return TaskForThread_Analyse;
}  

# include "stringbim.h" // IsNotStringLockGC
# include "string/public.h" // PLOString_CStr
# include <wctype.h> // towupper
# include "cast.h"

TaskForThread Lisp2Dbimsocketopen2( StackClusterPrivate * const stclpp){
  ParaLispObject  * const plostringdomainp = CCAR  ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotStringLockGC(   stclpp , plostringdomainp))    
    return  TaskForThread_Analyse;
  ParaLispObject  * const plostringtypep = CCAR  ( CCDR  ( StackClusterPrivate_ArgTop(stclpp )));
  if(IsNotStringLockGC(   stclpp , plostringtypep))    
    return  TaskForThread_Analyse;
  Lisp2DThreadStackPointerType const oldstack_b = Lisp2DThreadSP ( ) ;
  String  ( WideChar  ) * const stringdomain = StackNew_StringWideChar ( ) ;
  StringConstrCStr ( WideChar  ) ( stringdomain , PLOString_CStr  ( ParaLispObjectToPLOString ( plostringdomainp ) ) , true ) ;
  { wchar_t volatile  * i = String_Begin  ( WideChar  ) ( stringdomain  ) ;
    wchar_t volatile  const * e = String_CEnd ( WideChar  ) ( stringdomain  ) ;
    while ( i not_eq  e ) {
      * i = wint_cast_wchar ( towupper  ( wchar_cast_wint ( * i ) ) ) ;
      ++  i ;
    }
  }
  unsigned  short int usiDomainType ;
  { wchar_t volatile  const * const b = StringCStr  ( WideChar  ) ( stringdomain ) ;
    if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"UNIX"  ) ==  0 )
      usiDomainType = PF_UNIX;
    else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"LOCAL"  ) ==  0 )
      usiDomainType = PF_LOCAL;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"INET"  ) ==  0 )
    usiDomainType = PF_INET;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"INET6"  ) ==  0 )
    usiDomainType = PF_INET6;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"IPX"  ) ==  0 )
    usiDomainType = PF_IPX;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"NETLINK"  ) ==  0  )
    usiDomainType = PF_NETLINK;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"X25"  ) ==  0 )
    usiDomainType = PF_X25;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"AX25"  ) ==  0 )
    usiDomainType = PF_AX25;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"ATMPVC"  ) ==  0 )
    usiDomainType = PF_ATMPVC;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"APPLETALK"  ) ==  0 )
    usiDomainType = PF_APPLETALK;
  else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"PACKET"  ) ==  0 )
    usiDomainType = PF_PACKET;
  else  {   
    Lisp2D_StackFree  ( oldstack_b ) ;
    PassagePassIn ( passageGCCPub ) ;                        
    ParaLispObject  * list [ ] = { & PLOString_NewCopy(
        L"Unknown communication domain ")->plo,
      plostringdomainp , NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    PassagePassOut ( passageGCCPub ) ;                          
    return  TaskForThread_Analyse;
  }
  } // b
  Lisp2D_StackFree  ( oldstack_b ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( WideChar  ) * const stringtype = StackNew_StringWideChar ( ) ;
  StringConstrCStr ( WideChar  ) ( stringtype , PLOString_CStr  (
    ParaLispObjectToPLOString ( plostringtypep ) ) , true ) ;
  { wchar_t volatile  * i = String_Begin  ( WideChar  ) ( stringtype  ) ;
    wchar_t volatile  const * e = String_CEnd ( WideChar  ) ( stringtype  ) ;
    while ( i not_eq  e ) {
      * i = wint_cast_wchar ( towupper  ( wchar_cast_wint ( * i ) ) ) ;
      ++  i ;
    }
  }
  int inttype ;
  { wchar_t volatile  const * const b = StringCStr  ( WideChar  ) ( stringtype  ) ;
    if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"STREAM"  ) ==  0 )
      inttype = SOCK_STREAM;
    else  if ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"DGRAM"  ) ==  0 )
      inttype = SOCK_DGRAM;
    else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"SEQPACKET"  ) ==  0 )
      inttype = SOCK_SEQPACKET;
    else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"RAW"  ) ==  0 )
      inttype = SOCK_RAW;
    else  if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"RDM"  ) ==  0 )
      inttype = SOCK_RDM;
    else  {
      Lisp2D_StackFree  ( oldstack_a ) ;
      PassagePassIn ( passageGCCPub ) ;
      ParaLispObject  * list [ ] = { & PLOString_NewCopy (
        L"Unknown communication type ")->plo, plostringtypep, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      PassagePassOut ( passageGCCPub ) ;
      return TaskForThread_Analyse;
    }
  } // b
  Lisp2D_StackFree  ( oldstack_a ) ;
// ! to return SymbolErrno  
  int newdescriptor = socket(usiDomainType,inttype,0);
  bool  flagerror;
  int interrno;
  if(newdescriptor  ==  -1){
    flagerror = true;
    interrno  = errno;
  } else 
    flagerror = false;
            
  if(flagerror){
    int cie = interrno ;
    wchar_t const * wc;
    
    bimsocketopen22( stclpp , cie,& wc);
    
    return TaskForThread_Analyse;
  }
  { PLOSocket * const socketmep =ParaLispObjectToPLOSocket(
      StackClusterPrivate_ThisTop(stclpp ));
    PLOSocket_SetDescriptor(socketmep,newdescriptor);
    PLOSocket_SetUSIntDomain(socketmep,usiDomainType);
    PLOSocket_SetType(socketmep,inttype); 
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclpp ,& plonil.plo );
  PassagePassOut ( passageGCCPub ) ;
  return TaskForThread_Analyse;
}

# include "consiter/public.h"
# include "consiter/struct.h"
# include "cast.h"

bool  isBADWriteAnalyseMode(StackClusterPrivate * const stclp ,
  ParaLispObject * const listModep, int  * const flagsp){
  { ThreadStackPointerType const oldstack_i = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const i = NewStackCConsIterator ( ) ;
    CConsIteratorInit ( i , listModep ) ;
  for(;CConsIteratorHaveItem( i ) ; CConsIteratorNext( i )){
    // flag processing
    ParaLispObject  * const ploflagip = CConsIteratorItem( i);
    if(IsNotStringLockGC(  stclp ,ploflagip))  {
      Lisp2D_StackFree  ( oldstack_i ) ;
      return  true; 
    }

    // ! to up cycle ?

    ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( WideChar  ) * const wsflagi = StackNew_StringWideChar ( ) ;

    String_ConstrCopy ( WideChar  ) ( wsflagi ,
      PLOString_CValue  ( ParaLispObjectToPLOString ( ploflagip ) ) ) ;

    { wchar_t volatile * j = String_Begin  ( WideChar  ) ( wsflagi  ) ;
      wchar_t volatile const * e = String_CEnd ( WideChar  ) ( wsflagi  ) ;
      while ( j not_eq  e ) {
        * j = wint_cast_wchar ( towupper  ( wchar_cast_wint ( * j ) ) ) ;
        ++  j ;
      } 
    }
    wchar_t volatile const * const f = StringCStr  ( WideChar  ) ( wsflagi ) ;

    if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( f ) ,  L"OOB"  ) ==  0 )
      (*flagsp) |=  MSG_OOB;
    else if( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( f ) ,  L"DONTROUTE"  ) ==  0 )
      (*flagsp) |=  MSG_DONTROUTE;
    else if( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( f ) ,  L"DONTWAIT"  ) ==  0 )
      (*flagsp) |=  MSG_DONTWAIT;
    else if( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( f ) ,  L"NOSIGNAL"  ) ==  0 )
        (*flagsp) |=  MSG_NOSIGNAL;
      else if ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( f ) ,  L"CONFIRM"  ) ==  0 )
        (*flagsp) |=  MSG_CONFIRM;
      else  {
        Lisp2D_StackFree  ( oldstack_i ) ;
        PassagePassIn ( passageGCCPub ) ;
        ParaLispObject  * list [ ] = {
          ploflagip, & PLOString_NewCopy (L" flag is not known")->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        PassagePassOut ( passageGCCPub ) ;
        return  true;
      }
      Lisp2D_StackFree  ( oldstack_a ) ;
    }   // for i
    Lisp2D_StackFree  ( oldstack_i ) ;
  } // i
  return  false;
}

# include "plocomplex/public.h"

TaskForThread Lisp2Dbimsocketdescriptor(StackClusterPrivate  * const stclpp){
  PLOComplex  *  const cres=NewComplexBareExc( );
  Complex_SetInt  ( PLOComplexToComplex( cres),
    PLOSocket_Descriptor(ParaLispObjectToPLOSocket(
    StackClusterPrivate_ThisTop(stclpp))));
  StackClusterPrivate_SetResult(stclpp ,&  cres->plo);
  return  TaskForThread_Analyse;
}

# include "bytedeque/public.h"
# include "complex/inline.h"
# include "cons/inline.h"
  
// socket read #[buf] [size [{string-flag}]]
// returns byte-received | string
TaskForThread bimsocketread22 ( StackClusterPrivate  * const stclpp , void  * * const bufp ) {
  ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const bdbufp = CCAR  ( arglistp);
  if  ( IsNotByteDequeLockGC  ( stclpp , bdbufp ) ) 
    return TaskForThread_Analyse;
  if(IsConstLockGC(   stclpp ,bdbufp)) 
    return TaskForThread_Analyse;
  size_t  realsize;
  size_t  wasbufsize = PLOByteDeque_Size(
    ParaLispObjectToPLOByteDeque(bdbufp));
  int flags =0;
  if(CConsp(CCDR(arglistp))){
    ParaLispObject  * const plosizep = CCAR  ( CCDR  ( arglistp));
    if(IsNotNumberLockGC(  stclpp ,plosizep))      
      return TaskForThread_Analyse;
    if(IsNotRealLockGC(  stclpp ,ParaLispObjectToPLOComplex(plosizep)))     
      return TaskForThread_Analyse;
    if(  FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex(
        plosizep))), &realsize)) {
      PassagePassIn ( passageGCCPub ) ;      
      { ParaLispObject  * list [ ] = { plosizep, 
          & PLOString_NewCopy (L" has wrong size value")->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      }
      PassagePassOut ( passageGCCPub ) ;
      return TaskForThread_Analyse;
    }
    ( * bufp  ) = malloc  ( realsize  ) ;
    if  ( not ( * bufp  ) ) {
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
    ParaLispObject  * const listModep = CCDR  ( CCDR  ( arglistp));
    if(CConsp(listModep)){
// ! to use ConsIterator      
      PLOCons * first=   ParaLispObjectToPLOCons(listModep);
      PLOCons * last;
      do{
        // flag processing
        { ParaLispObject  * const ploflagip = ClassCons_First(&first->cc);
          if(IsNotStringLockGC(  stclpp ,ploflagip)){
            free(*bufp);
            return TaskForThread_Analyse;
          }
          ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
          String  ( WideChar  ) * const wsflagi = StackNew_StringWideChar ( ) ;
          StringConstrCStr ( WideChar  ) ( wsflagi , PLOString_CStr  (
            ParaLispObjectToPLOString ( ploflagip)) , true ) ;
          { wchar_t volatile * i = String_Begin  ( WideChar  ) ( wsflagi  ) ;
            wchar_t volatile const * const e = String_CEnd ( WideChar  ) ( wsflagi  ) ;
            while ( i not_eq  e ) {
              * i = wint_cast_wchar ( towupper  ( wchar_cast_wint ( * i ) ) ) ;
              ++  i ; 
            } 
          }
          wchar_t volatile const * const b = StringCStr  ( WideChar  ) ( wsflagi ) ;
          if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"OOB"  ) ==  0 )
            flags |=  MSG_OOB;
          else if  ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"PEEK"  ) ==  0 )
            flags |=  MSG_PEEK;
          else if ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"WAITALL"  ) ==  0 )
            flags |=  MSG_WAITALL;
          else if ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"TRUNC"  ) ==  0 )
            flags |=  MSG_TRUNC;
          else if ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"ERRQUEUE"  ) ==  0 )
            flags |=  MSG_ERRQUEUE;
          else if ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( b ) ,  L"NOSIGNAL"  ) ==  0 )
            flags |=  MSG_NOSIGNAL;
          else  {
            Lisp2D_StackFree  ( oldstack_a ) ;
            PassagePassIn ( passageGCCPub ) ;
              ParaLispObject  * list [ ] = { ploflagip,
                & PLOString_NewCopy (L" flag is not known")->plo, NULL } ;
              StackClusterPrivate_ExceptionSet(stclpp ,
                ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
            PassagePassOut ( passageGCCPub ) ;  
            free (*bufp);
            return TaskForThread_Analyse; 
          }
          Lisp2D_StackFree  ( oldstack_a ) ;
        }
        // go to next flag
        { ParaLispObject  * const firstnextp = ClassCons_Rest  ( &first->cc ) ;
          if(CConsp(firstnextp)){
            last  = first;
            first =   ParaLispObjectToPLOCons(firstnextp);
          } else
            break ;
        }  // from a while
        // cycle chek
        { bool  flagstop = false ;
          { PLOCons * i =   ParaLispObjectToPLOCons(listModep) ;
            for( ; i  not_eq  last;
              i = ParaLispObjectToPLOCons ( ClassCons_Rest  ( &i ->cc) ) )
              if(i  ==  first) {
                flagstop  = true;
                break;
              }   // from a for i
            } // i
          if(flagstop)
            break;
        }  // from a while
        // i == last
        if(last ==  first)
          break;  // from a while  
      } while ( true  ) ;
    }  //  if(Consp(listMode))
  } else  {  //  if(Consp(CCDR(arglist)))
    realsize  = wasbufsize;
    ( * bufp  ) = malloc  ( realsize  ) ;
    if  ( not ( * bufp  ) ) {
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
  }
  PLOSocket * const mep = ParaLispObjectToPLOSocket(
    StackClusterPrivate_ThisTop(stclpp ));
  errno = 0 ;
// ! to return SymbolErrno  
  ssize_t const   ssizebytereceived = recv(
    PLOSocket_Descriptor(mep),*bufp,realsize,flags) ;
  if(ssizebytereceived  < 0){
    int interrno = errno ;
    free(*bufp);
      wchar_t const * ws  ;
      switch(interrno){
      case  EBADF:
        ws  = L"BADF";
        break;
      case  ECONNREFUSED:
        ws  = L"CONNREFUSED";
        break;
      case  ENOTCONN:
        ws  = L"NOTCONN";
        break;
      case  ENOTSOCK:
        ws  = L"NOTSOCK";
        break;
      case  EAGAIN:
        ws  = L"AGAIN";
        break;
      case  EINTR:
        ws  = L"INTR";
        break;
      case  EFAULT:
        ws  = L"FAULT";
        break;
      case  EINVAL:
        ws  = L"INVAL";
        break;
      default : ws  = L"?" ; 
      }  // switch
    PassagePassIn ( passageGCCPub ) ;
    { jmp_buf jb2  ;
      jmp_buf * const jbold = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2 ) ;
      if  ( jbn  ) {
        Lisp2DJBglobal  = jbold ;
        PassagePassOut ( passageGCCPub ) ;
        return  TaskForThread_Analyse ;
      }
      PLOStringPub * const newstr  = PLOString_NewCopy ( ws ) ;
      Lisp2DJBglobal  = jbold ;
      StackClusterPrivate_SetResult (stclpp ,&  newstr  ->  plo  ) ;
    }
    PassagePassOut ( passageGCCPub ) ;
    return TaskForThread_Analyse;
  }
  size_t  sizebytereceived = (size_t)(ssizebytereceived) ;
  // when bdbuf < buf -> increase bdbuf size
  if(wasbufsize < sizebytereceived)
    PLOByteDeque_Resize ( ParaLispObjectToPLOByteDeque  ( bdbufp ) ,
      sizebytereceived  ) ;
  // insert buf to bdbuf  
  PLOByteDeque_SetFrontNotCheckNotResize  ( 
    ParaLispObjectToPLOByteDeque  ( bdbufp ) , (uint8_t  * )(*bufp),
    sizebytereceived);
  free(*bufp);
  PassagePassIn ( passageGCCPub ) ;
  { PLOComplex  * const  res =  NewComplexBareExc (  ) ;
    Complex_SetUSize  ( PLOComplexToComplex( res) ,sizebytereceived);
    StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  }
  PassagePassOut ( passageGCCPub ) ;  
  return TaskForThread_Analyse;
}

// !? to make:
// !? (socket write) -> empty message

// socket write #[buf] [size [{string-flag}]]
// or
// socket write string-buf [{string-flag}]

// returns byte-sended | string  
TaskForThread Lisp2Dbimsocketwrite2 ( StackClusterPrivate * const stclpp){
  ParaLispObject  * const arglistp  = StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const bdbufp = CCAR  ( arglistp ) ;
  int flags = 0 ;
  uint8_t * bufm  ;
  size_t  realsize;
  if(Stringp(bdbufp)){
    ThreadStackPointerType const oldstack_myp = Lisp2DThreadSP ( ) ;
    Deque ( Byte  ) * const dqbufp  = Lisp2D_StackVar ( StackType ( Deque_Byte  ) ) ;
    Deque_Init ( Byte ) ( dqbufp ) ;
    if  ( isBADReadWCharArrayToByteDeque (  PLOString_CStr  (
      ParaLispObjectToPLOString ( bdbufp  ) ) , dqbufp  ) ) {
      PassagePassIn ( passageGCCPub ) ;
        ParaLispObject  * list [ ] = {
           bdbufp,  & PLOString_NewCopy(L" string is bad")->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      PassagePassOut ( passageGCCPub ) ;
      Lisp2D_StackFree  ( oldstack_myp ) ;
      return  TaskForThread_Analyse;
    }
    realsize  = Deque_Size ( Byte )(dqbufp);
    if(isBADWriteAnalyseMode(stclpp , CCDR  ( CCDR  ( arglistp)),&flags)){
      Lisp2D_StackFree  ( oldstack_myp ) ;
      return  TaskForThread_Analyse;
    }
    bufm  = malloc  ( realsize  ) ;
    if  ( not bufm  ) {
      Lisp2D_StackFree  ( oldstack_myp ) ;
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
    // copy dqbuf to buf of size=realsize
    { uint8_t * bufi  = (bufm  + realsize);
      DequeConstIterator ( Byte ) j=Deque_CEnd ( Byte ) (dqbufp);
      for(size_t  i=realsize;i > 0;){
        --  i;
        Deque_ConstIterator_Dec ( Byte ) ( & j ) ;
        --  bufi;
        ( * bufi) = ( * Deque_ConstIterator_Place ( Byte )(j));
      }
    }
    Lisp2D_StackFree  ( oldstack_myp ) ;
    goto  Next;
  } // if(Stringp(&bdbuf))
  if  ( IsNotByteDequeLockGC  ( stclpp , bdbufp ) )    
    return  TaskForThread_Analyse;
  { size_t  wasbufsize  = PLOByteDeque_Size (
    ParaLispObjectToPLOByteDeque(bdbufp));
  if(CConsp(  CCDR  ( arglistp))){
    ParaLispObject  * const plosizep = CCAR  ( CCDR  ( arglistp));
    if(IsNotNumberLockGC(  stclpp ,plosizep))      
      return  TaskForThread_Analyse;
    if(IsNotRealLockGC(  stclpp ,ParaLispObjectToPLOComplex(plosizep)))
      return  TaskForThread_Analyse;
    if(  FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex(
        plosizep))),  &realsize)){
      PassagePassIn ( passageGCCPub ) ;
        ParaLispObject  * list [ ] = { plosizep,  & PLOString_NewCopy(
          L" has wrong size value")->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      PassagePassOut ( passageGCCPub ) ;
      return  TaskForThread_Analyse;
    }
    // write smallbuf bigsize -> writes small
    // !? to make error ?!
    if(realsize > wasbufsize)
      realsize  = wasbufsize;
    bufm  = malloc  ( realsize  ) ;
    if  ( not bufm  ) {
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
    if(isBADWriteAnalyseMode( stclpp , CCDR  ( CCDR  ( arglistp)),&flags)){
      free  ( bufm  ) ;
      return  TaskForThread_Analyse ; 
    }
  //  if(Consp(CCDR(arglist)))
  } else {
    realsize  = wasbufsize;
    bufm  = malloc  ( realsize  ) ;
    if  ( not bufm  ) {
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
  } 
  }  // wasbufsize
  // copy dbbuf to buf of size=realsize
  { uint8_t * bufi  = (bufm  + realsize);
    DequeConstIterator  ( Byte  ) j = Deque_CEnd  ( Byte  ) ( 
      PLOByteDeque_CValue ( ParaLispObjectToPLOByteDeque  ( bdbufp ) ) ) ;
    size_t  i = realsize ;
    while ( i > 0 ) {
      --  i ;
      Deque_ConstIterator_Dec ( Byte  ) ( & j ) ;
      --  bufi  ;
      ( * bufi) = ( * Deque_ConstIterator_Place ( Byte ) ( j ) ) ;
    }
  }
Next:      
  { PLOSocket * const mep  = ParaLispObjectToPLOSocket (
      StackClusterPrivate_ThisTop ( stclpp  ) ) ;
  errno = 0 ;
// ! to return SymbolErrno  
  ssize_t const   ressize=send(PLOSocket_Descriptor(mep),bufm,realsize,flags);
  free  ( bufm  ) ;
  if(ressize  < 0){
    int const interrno=errno;
      wchar_t const * ws  ;
      switch(interrno){
      case  EBADF:
        ws  = L"BADF";
        break;
      case  ENOTSOCK:
        ws  = L"NOTSOCK";
        break;
      case  EFAULT:
        ws  = L"FAULT";
        break;
      case  EMSGSIZE:
        ws  = L"MSGSIZE";
        break;
      case  EAGAIN:
        ws  = L"AGAIN";
        break;
      case  ENOBUFS:
        ws  = L"NOBUFS";
        break;
      case  EINTR:
        ws  = L"INTR";
        break;
      case  ENOMEM:
        ws  = L"NOMEM";
        break;
      case  EINVAL:
        ws  = L"INVAL";
        break;
      case  EPIPE:
        ws  = L"PIPE";
        break;
      default:
        ws  = L"?";
      }  // switch
    PassagePassIn ( passageGCCPub ) ;
    { jmp_buf jb2  ;
      jmp_buf * const jbold = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2 ) ;
      if  ( jbn  ) {
        Lisp2DJBglobal  = jbold ;
        PassagePassOut ( passageGCCPub ) ;
        return  TaskForThread_Analyse ;
      }
      PLOStringPub * const newstr = PLOString_NewCopy ( ws ) ;
      Lisp2DJBglobal  = jbold ;
      StackClusterPrivate_SetResult (stclpp ,&  newstr->plo ) ;
    }
    PassagePassOut ( passageGCCPub ) ; 
    // recv < 0
  } else {
    PassagePassIn ( passageGCCPub ) ;
    { PLOComplex  * const  res=NewComplexBareExc(  );
      { size_t const  sizebytesended=ssize_cast_size(ressize);
        Complex_SetUSize  ( PLOComplexToComplex( res),sizebytesended); 
      }
      StackClusterPrivate_SetResult(stclpp ,&  res->plo);
    }
    PassagePassOut ( passageGCCPub ) ; 
  } 
  } // mep
  return  TaskForThread_Analyse ; 
}

// socket connect ( port addr | path)
TaskForThread Lisp2Dbimsocketconnect2( StackClusterPrivate  * const stclpp){
  PLOSocket * const mep =
    ParaLispObjectToPLOSocket(StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  * const plonumberportp = CCAR  ( StackClusterPrivate_ArgTop(stclpp ));
  if(PLOSocket_USIntDomain(mep) ==  PF_UNIX){
    if(IsNotStringLockGC(   stclpp ,plonumberportp))      
      return TaskForThread_Analyse;
    int localerror;
    int interror;
    wchar_t volatile const * const ws2 = PLOString_CStr (
      ParaLispObjectToPLOString ( plonumberportp ) ) ;
    if  ( PLOSocket_isBADConnectPath  ( mep , ws2 , & localerror  , & interror  ) ) {
      if(localerror){
        PassagePassIn ( passageGCCPub ) ;
        // !!! to catch exception
        StackClusterPrivate_SetResult(stclpp ,
          & PLOString_NewCopy (L"BADPATH")->plo);
        PassagePassOut ( passageGCCPub ) ;  
        return TaskForThread_Analyse;
      }
      if(interror){
        wchar_t const * ws ;
        switch(interror){
        case  EBADF:
          ws  = L"BADF";
          break;
        case  EFAULT:
          ws  = L"FAULT";
          break;
        case  ENOTSOCK:
          ws  = L"NOTSOCK";
          break;
        case  EISCONN:
          ws  = L"ISCONN";
          break;
        case  ECONNREFUSED:
          ws  = L"CONNREFUSED";
          break;
        case  ETIMEDOUT:
          ws  = L"TIMEDOUT";
          break;
        case  ENETUNREACH:
          ws  = L"NETUNREACH";
          break;
        case  EADDRINUSE:
          ws  = L"ADDRINUSE";
          break;
        case  EINPROGRESS:
          ws  = L"INPROGRESS";
          break;
        case  EALREADY:
          ws  = L"ALREADY";
          break;
        case  EAGAIN:
          ws  = L"AGAIN";
          break;
        case  EAFNOSUPPORT:
          ws  = L"AFNOSUPPORT";
          break;
        case  EACCES:
          ws  = L"ACCES";
          break;
        case  EPERM:
          ws  = L"PERM";
          break;
        case  EADDRNOTAVAIL:
          ws  = L"ADDRNOTAVAIL";
          break;
        case  EINVAL:
          ws  = L"INVAL";
          break;
        case  EINTR:
          ws  = L"INTR";
          break;
        case  EPROTOTYPE:
          ws  = L"PROTOTYPE";
          break;
        case  ENOSR:
          ws  = L"NOSR";
          break;
        default :    
          ws  = L"?";
        }
  PassagePassIn ( passageGCCPub ) ;
  { jmp_buf jb2  ;
    jmp_buf * const jbold = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jb2 ;
    int const volatile jbn = setjmp  ( jb2 ) ;
    if  ( jbn  ) {
      Lisp2DJBglobal  = jbold ;
      PassagePassOut ( passageGCCPub ) ;
      return  TaskForThread_Analyse ;
    }
    PLOStringPub * const newstr  = PLOString_NewCopy ( ws ) ;
    Lisp2DJBglobal  = jbold ;
    StackClusterPrivate_SetResult (stclpp ,&  newstr->plo  ) ;
  }
        PassagePassOut ( passageGCCPub ) ;
        return TaskForThread_Analyse;
      }
      PassagePassIn ( passageGCCPub ) ;
      // !!! to catch exception
      StackClusterPrivate_SetResult (stclpp ,& PLOString_NewCopy (L"??")->plo);
      PassagePassOut ( passageGCCPub ) ;
      return TaskForThread_Analyse;
    }  //  if(me.isBADConnect)
    //  if me.USIntDomain() ==  PF_UNIX
    } else { //  me.USIntDomain() not_eq  PF_UNIX
    if( IsNotNumberLockGC(  stclpp ,plonumberportp))      
      return TaskForThread_Analyse;
    ParaLispObject  * const plostringaddrp= CCAR  (
      CCDR  ( StackClusterPrivate_ArgTop(stclpp )));
    if(IsNotStringLockGC(   stclpp ,plostringaddrp))      
      return TaskForThread_Analyse;
    uint16_t  port;
    if(Complex_IsBADUI16Value(PLOComplexToComplexConst(
      ParaLispObjectToPLOComplexConst(plonumberportp)),&port)){
  PassagePassIn ( passageGCCPub ) ;
  /// !!! to catch exception
  ParaLispObject  * list [ ] = {
    & PLOString_NewCopy (L"port is wrong : ")->plo,  plonumberportp, NULL } ;
    StackClusterPrivate_ExceptionSet(
      stclpp ,ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      PassagePassOut ( passageGCCPub ) ;
      return TaskForThread_Analyse;
    }
    int localerror;
    int interror;
    wchar_t volatile const * const ws3 = PLOString_CStr  (
      ParaLispObjectToPLOString ( plostringaddrp ) ) ;
    if  ( PLOSocket_isBADConnectPort  ( mep , port  , ws3 , & localerror  ,  & interror ) ) {
      if(interror){
        // ? char  const *   strerror=(gai_strerror(interror));
        wchar_t const * ws ;
        switch(interror){
        case  EAI_ADDRFAMILY:
          ws  = L"ADDRFAMILY";
          break;
        case  EAI_AGAIN:
          ws  = L"AGAIN";
          break;
        case  EAI_BADFLAGS:
          ws  = L"BADFLAGS";
          break;
        case  EAI_FAIL:
          ws  = L"FAIL";
          break;
        case  EAI_FAMILY:
          ws  = L"FAMILY";
          break;
        case  EAI_MEMORY:
          ws  = L"MEMORY";
          break;
        case  EAI_NODATA:
          ws  = L"NODATA";
          break;
        case  EAI_NONAME:
          ws  = L"NONAME";
          break;
        case  EAI_SERVICE:
          ws  = L"SERVICE";
          break;
        case  EAI_SOCKTYPE:
          ws  = L"SOCKTYPE";
          break;
        case  EAI_SYSTEM:
          ws  = L"SYSTEM";
          break;
        default:
          ws  = L"??";
        }      
  PassagePassIn ( passageGCCPub ) ;
  { jmp_buf jb2  ;
    jmp_buf * const jbold = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jb2 ;
    int const volatile jbn = setjmp  ( jb2 ) ;
    if  ( jbn  ) {
      Lisp2DJBglobal  = jbold ;
      PassagePassOut ( passageGCCPub ) ;
      return  TaskForThread_Analyse ;
    }
    PLOStringPub * const newstr  = PLOString_NewCopy ( ws ) ;
    Lisp2DJBglobal  = jbold ;
    StackClusterPrivate_SetResult (stclpp ,&  newstr ->plo ) ;
  }
        PassagePassOut ( passageGCCPub ) ; 
        return TaskForThread_Analyse;
      }
      if(localerror){
        wchar_t const * ws  ;
        switch(localerror){
        case  6:
          ws  = L"NOADDRSUCCEEDED";
          break;
        case  2:
          ws  = L"BADADDR";
          break;
        default:
          ws  = L"?";
        }
  PassagePassIn ( passageGCCPub ) ;
  { jmp_buf jb2  ;
    jmp_buf * const jbold = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jb2 ;
    int const volatile jbn = setjmp  ( jb2 ) ;
    if  ( jbn  ) {
      Lisp2DJBglobal  = jbold ;
      PassagePassOut ( passageGCCPub ) ;
      return  TaskForThread_Analyse ;
    }
    PLOStringPub * const newstr  = PLOString_NewCopy ( ws )  ;
    Lisp2DJBglobal  = jbold ;
    StackClusterPrivate_SetResult (stclpp ,&  newstr ->plo ) ;
  }
        PassagePassOut ( passageGCCPub ) ;
        return TaskForThread_Analyse;
      } //  if localerror
  PassagePassIn ( passageGCCPub ) ;
  // !!! to catch exception
      StackClusterPrivate_SetResult(stclpp ,& PLOString_NewCopy (L"???")->plo);
      PassagePassOut ( passageGCCPub ) ;
      return TaskForThread_Analyse;
    } // if flagbadconnet
    } //  if  me.USIntDomain() not_eq  PF_UNIX
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult(stclpp ,& plonil.plo );
  PassagePassOut ( passageGCCPub ) ;
  return TaskForThread_Analyse;
}

// socket bind port|string
TaskForThread Lisp2Dbimsocketbind ( StackClusterPrivate * const stclpp ) {
  PLOSocket * const mep = ParaLispObjectToPLOSocket  (
    StackClusterPrivate_ThisTop ( stclpp )  ) ;
  sa_family_t family  ;
  switch  ( PLOSocket_USIntDomain ( mep ) ) {
  case  PF_UNIX :
    family  = AF_UNIX ;
    break ;
  case  PF_INET :
    family  = AF_INET ;
    break ;
  case  PF_INET6  :
    family  = AF_INET6  ;
    break ;
  case  PF_IPX  :
    family  = AF_IPX  ;
    break ;
  case  PF_NETLINK  :
    family  = AF_NETLINK  ;
    break ;
  case  PF_X25  :
    family  = AF_X25  ;
    break ;
  case  PF_AX25 :
    family  = AF_AX25 ;
    break ;
  case  PF_ATMPVC :
    family  = AF_ATMPVC ;
    break ;
  case  PF_APPLETALK  :
    family  = AF_APPLETALK  ;
    break ;
  case  PF_PACKET :
    family  = AF_PACKET ;
    break ;
  default :
    PassagePassIn ( passageGCCPub ) ;
    // !!! to catch exception
    StackClusterPrivate_SetResult ( stclpp ,  & PLOString_NewCopy (
      L"usintdomain?" ) ->  plo ) ;
    PassagePassOut ( passageGCCPub ) ;
    return TaskForThread_Analyse  ; 
  }
  ParaLispObject * const plonumberportp = CCAR  ( StackClusterPrivate_ArgTop  ( stclpp ) ) ;
  int res ;
  if  ( family ==  AF_UNIX  ) {
    struct  sockaddr_un saddr ;
    saddr . sun_family  = family  ;
    if  ( IsNotStringLockGC ( stclpp ,  plonumberportp  ) )      
      return TaskForThread_Analyse  ;
    ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const strtmpp = StackNew_StringChar ( ) ;
    String_Init ( Char  ) ( strtmpp , true ) ;
    if ( isBADReadWCharToCharString ( PLOString_CStr  (
      ParaLispObjectToPLOString ( plonumberportp ) ) , strtmpp ) ) {
      Lisp2D_StackFree  ( oldstack_a ) ;
      PassagePassIn ( passageGCCPub ) ;
      BadString ( stclpp ,  ParaLispObjectToPLOString ( plonumberportp  ) ) ;
      PassagePassOut ( passageGCCPub ) ;
      return TaskForThread_Analyse  ; 
    }
    strcpy  ( saddr . sun_path  , String_CBegin ( Char  ) ( strtmpp ) ) ;
    Lisp2D_StackFree  ( oldstack_a ) ;
    res = bind  ( PLOSocket_Descriptor  ( mep ) ,
      ( struct  sockaddr  * ) ( & saddr ) , sizeof  ( saddr ) ) ;
    // family ==  AF_UNIX
  } else  {
    struct  sockaddr_in saddr ;
    saddr . sin_family  = family  ;
    if  ( IsNotNumberLockGC ( stclpp ,  plonumberportp ) )      
      return TaskForThread_Analyse  ;
    uint16_t  port  ;
    if  ( Complex_IsBADUI16Value  ( PLOComplexToComplexConst(
        ParaLispObjectToPLOComplexConst  ( plonumberportp  )) , &port  ) ) {
      PassagePassIn ( passageGCCPub ) ;
      // !!! to catch exception
      ParaLispObject  * list [ ] = { & PLOString_NewCopy ( 
        L"port is wrong : " ) ->  plo , plonumberportp  , NULL } ;
      StackClusterPrivate_ExceptionSet  ( stclpp ,
        ListOfPLO ( ( PArrayPLOP ) {
        ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;
      PassagePassOut ( passageGCCPub ) ; 
      return TaskForThread_Analyse  ; 
    }
    saddr . sin_port  = htons ( port  ) ;
    saddr . sin_addr  . s_addr = htonl  ( INADDR_ANY  ) ;
    errno = 0 ;
    res = bind  ( PLOSocket_Descriptor  ( mep ) , ( struct  sockaddr  * ) (
      & saddr ) , sizeof  ( saddr ) ) ; }
  if  ( res  < 0  ) {
// ! to return SymbolErrno    
    int const le  = errno ;
    wchar_t const * ws  ;
    switch  ( le  ) {
    case  EBADF :
      ws  = L"BADF" ;
      break ;
    case  EINVAL  :
      ws  = L"INVAL"  ;
      break ;
    case  EACCES  :
      ws  = L"ACCES"  ;
      break ;
    case  ENOTSOCK  :
      ws  = L"NOTSOCK"  ;
      break ;
    case  EROFS :
      ws  = L"ROFS" ;
      break ;
    case  EFAULT  :
      ws  = L"FAULT"  ;
      break ;
    case  ENAMETOOLONG  :
      ws  = L"NAMETOOLONG"  ;
      break ;
    case  ENOENT  :
      ws  = L"NOENT"  ;
      break ;
    case  ENOMEM  :
      ws  = L"NOMEM"  ;
      break ;
    case  ENOTDIR :
      ws  = L"NOTDIR" ;
      break ;
    case  ELOOP :
      ws  = L"LOOP" ;
      break ;
    default :
      ws  = L"?"  ; 
    }
    PassagePassIn ( passageGCCPub ) ;
    { jmp_buf jb2  ;
      jmp_buf * const jbold = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2 ) ;
      if  ( jbn  ) {
        Lisp2DJBglobal  = jbold ;
        PassagePassOut ( passageGCCPub ) ;
        return  TaskForThread_Analyse ;
      }
      PLOStringPub * const newstr  = PLOString_NewCopy ( ws ) ;
      Lisp2DJBglobal  = jbold ;
      StackClusterPrivate_SetResult ( stclpp ,  &  newstr ->  plo ) ; 
    }
    PassagePassOut ( passageGCCPub ) ;
    return TaskForThread_Analyse  ; 
  } // res  < 0
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & plonil .  plo ) ;
  PassagePassOut ( passageGCCPub ) ;
  return TaskForThread_Analyse  ;
}

// socket listen n
TaskForThread Lisp2Dbimsocketlisten ( StackClusterPrivate * const stclpp ) {
  PLOSocket * const mep = ParaLispObjectToPLOSocket (
    StackClusterPrivate_ThisTop ( stclpp )  ) ;  
  ParaLispObject  * const plonumbernp = CCAR  ( StackClusterPrivate_ArgTop  (
    stclpp )  ) ;
  if  (  IsNotNumberLockGC ( stclpp ,  plonumbernp  ) )
    return TaskForThread_Analyse  ;
  int n ;
  if  ( Complex_IsBADIntValue (PLOComplexToComplexConst(
      ParaLispObjectToPLOComplexConst  ( plonumbernp )), &n ) ) {
    PassagePassIn ( passageGCCPub ) ;
      ParaLispObject  * list [ ] = { &
        PLOString_NewCopy (  L"count is wrong : "  ) -> plo ,
        plonumbernp , NULL } ;
      StackClusterPrivate_ExceptionSet  ( stclpp , 
        ListOfPLO ( ( PArrayPLOP ) {
        ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;
    PassagePassOut ( passageGCCPub ) ;    
    return TaskForThread_Analyse  ; 
  }
// ! to return SymbolErrno  
  if  ( listen  ( PLOSocket_Descriptor  ( mep ) , n ) < 0 ) {
    int const interrno  = errno ;
    wchar_t const * wse ;
    switch  ( interrno  ) {
    case  EADDRINUSE  :
      wse = L"ADDRINUSE"  ;
      break ;
    case  EBADF :
      wse = L"BADF" ;
      break ;
    case  ENOTSOCK  :
      wse = L"NOTSOCK"  ;
      break ;
    case  EOPNOTSUPP  :
      wse = L"OPNOTSUPP"  ;
      break ;
    default :
      wse = L"?"  ; 
    }
    PassagePassIn ( passageGCCPub ) ;
    { jmp_buf jb2  ;
      jmp_buf * const jbold = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2 ) ;
      if  ( jbn  ) {
        Lisp2DJBglobal  = jbold ;
        PassagePassOut ( passageGCCPub ) ;
        return  TaskForThread_Analyse ;
      }
      PLOStringPub * const newstr = PLOString_NewCopy ( wse ) ;
      Lisp2DJBglobal  = jbold ;
      StackClusterPrivate_SetResult ( stclpp ,  & newstr  ->  plo ) ;
    }
    PassagePassOut ( passageGCCPub ) ;
    return TaskForThread_Analyse  ; 
  }
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
  PassagePassOut ( passageGCCPub ) ;
  return TaskForThread_Analyse  ; 
}

# include "minmaxint.h" // Lisp2DMaxInt
# include "fraction/public.h"
// ! to write selest-read, select-write, select-error

// OLD list-of-sockets select-read [timeout]
// list(list-of-sockets-for-read list-of-sockets-for-write
//  list-of-sockets-for-error) socket-select [timeout]
// returns List of sockets to read from or String with error name
TaskForThread Lisp2Dbimconssocketselect (
  StackClusterPrivate  * const stclpp ) {
  PLOCons * const listoflistp =
    ParaLispObjectToPLOCons ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  fd_set  read_set  ;
  FD_ZERO (  & read_set ) ;
  bool  flagemptysetr = true ;
  ParaLispObject  * const lofsrp = ClassCons_First ( & listoflistp -> cc ) ;
  int fd_hwm = -1 ;
  { ThreadStackPointerType const oldstack_ci = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const ci = NewStackCConsIterator ( ) ;
    CConsIteratorInit ( ci , lofsrp ) ;
  for ( ; CConsIteratorHaveItem ( ci ) ; CConsIteratorNext ( ci ) ) {
    ParaLispObject * const xp = CConsIteratorItem ( ci ) ;
    if  ( IsNotSocket ( stclpp ,  xp  ) ) {  
      Lisp2D_StackFree  ( oldstack_ci ) ;
      return TaskForThread_Analyse  ; 
    }
    int const xd  = PLOSocket_Descriptor  ( ParaLispObjectToPLOSocket ( xp  ) ) ;
    if  ( xd >=  0  ) {
      flagemptysetr = false ;
      fd_hwm  = Lisp2DMaxInt  ( fd_hwm  , xd  ) ;
      FD_SET  ( xd  , & read_set  ) ; 
    } 
  }  //  for ci
    Lisp2D_StackFree  ( oldstack_ci ) ;
  } // cip
  fd_set  write_set ;
  FD_ZERO ( & write_set ) ;
  bool  flagemptysetw = true ;
  ParaLispObject  * const listoflist2p = ClassCons_Rest  (
    & listoflistp -> cc  ) ;
  ParaLispObject  * lofsw ;
  ParaLispObject  * listoflist3 ;
  if  ( CConsp  ( listoflist2p  ) ) {
    listoflist3 = ClassCons_Rest  (
      & ParaLispObjectToPLOCons ( listoflist2p )  ->  cc ) ;
    lofsw = ClassCons_First ( & ParaLispObjectToPLOCons ( listoflist2p  )
      ->  cc  ) ;
    { ThreadStackPointerType const oldstack_ci = Lisp2DThreadSP ( ) ; 
      CConsIteratorPub * const ci = NewStackCConsIterator ( ) ;
      CConsIteratorInit ( ci , lofsw ) ;
    for ( ; CConsIteratorHaveItem ( ci ) ; CConsIteratorNext ( ci ) ) {
      ParaLispObject  * const xp = CConsIteratorItem  ( ci ) ;
      if  ( IsNotSocket ( stclpp ,  xp  ) ) {
        Lisp2D_StackFree  ( oldstack_ci ) ;
        return TaskForThread_Analyse  ; 
      }
      int const  xd = PLOSocket_Descriptor  ( ParaLispObjectToPLOSocket ( xp  ) ) ;
      if  ( xd >=  0  ) {
        flagemptysetw = false ;
        fd_hwm  = Lisp2DMaxInt  ( fd_hwm  , xd  ) ;
        FD_SET  ( xd  , & write_set ) ;
      }
    }
      Lisp2D_StackFree  ( oldstack_ci ) ;
    } // ci
  // consp(listoflist2)
  } else  {
    lofsw = & plonil  . plo ;
    listoflist3 = & plonil  . plo ;
  }
  fd_set  except_set  ;
  FD_ZERO ( & except_set  ) ;
  bool  flagemptysete = true ;
  ParaLispObject  * lofse ;
  if  ( CConsp  ( listoflist3 ) ) {
    lofse = ClassCons_First ( & ParaLispObjectToPLOCons ( listoflist3 ) ->  cc  ) ;
    { ThreadStackPointerType const oldstack_ci = Lisp2DThreadSP ( ) ; 
      CConsIteratorPub * const ci = NewStackCConsIterator ( ) ;
      CConsIteratorInit ( ci , lofse ) ;
    for ( ; CConsIteratorHaveItem ( ci ) ; CConsIteratorNext ( ci ) ) {
      ParaLispObject  * const xp = CConsIteratorItem  ( ci ) ;
      if  ( IsNotSocket ( stclpp ,  xp  ) ) {
        Lisp2D_StackFree  ( oldstack_ci ) ;
        return TaskForThread_Analyse  ;
      }
      int const  xd = PLOSocket_Descriptor  ( ParaLispObjectToPLOSocket ( xp  ) ) ;
      if  ( xd >=  0  ) {
        flagemptysete = false ;
        fd_hwm  = Lisp2DMaxInt  ( fd_hwm  , xd  ) ;
        FD_SET  ( xd  , & except_set  ) ;
      }
    } 
      Lisp2D_StackFree  ( oldstack_ci ) ;
    } // ci
    //  consp(listoflist3)
  } else   
    lofse = & plonil  . plo ;
  if  ( fd_hwm < 0  ) {
    // set of closed sockets
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
    PassagePassOut ( passageGCCPub ) ;    
    return TaskForThread_Analyse  ; 
  }
  ParaLispObject  * const argsp = StackClusterPrivate_ArgTop  ( stclpp )  ;
  int resselect = -1 ;
  if  ( CConsp  ( argsp ) ) {
    ParaLispObject  * const timeoutp = 
      ClassCons_First ( & ParaLispObjectToPLOCons ( argsp ) ->  cc  ) ;
    if  ( IsNotNumber ( stclpp ,  timeoutp  ) )
      return TaskForThread_Analyse  ;
    struct  timeval tv = { 0 , 0 } ;
    { ThreadStackPointerType const oldstack_secp = Lisp2DThreadSP ( ) ;
      FractionPub * const sec = NewStackFraction  ( ) ;
      FractionConstrCopy ( sec , 
         Complex_CRe  (  PLOComplexToComplexConst  (
          ParaLispObjectToPLOComplexConst  ( timeoutp ) ) ) ) ;
      tv  . tv_sec = FractionLValue  ( sec ) ;
      if  ( ( tv  . tv_sec  ) < 0 ) {
        Lisp2D_StackFree  ( oldstack_secp ) ;
        { PLOStringPub * const newstr  = PLOString_NewCopy ( L" < 0" ) ;
          ParaLispObject  * list [ ] = { timeoutp , & newstr -> plo  , NULL } ;
          StackClusterPrivate_ExceptionSet  ( stclpp ,
            ListOfPLO ( ( PArrayPLOP ) { ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;
        }
        return  TaskForThread_Analyse ;
      }
      FractionMod  ( sec ) ;
      FractionMultULInt ( sec ,  1000000UL ) ; 
      FractionRound  ( sec ) ;
      tv  . tv_usec = FractionLValue ( sec ) ;
      if  ( ( tv  . tv_usec ) ==  1000000L  ) {
        ++  ( tv  . tv_sec  ) ;
        tv  . tv_usec  = 0  ; 
      }
      Lisp2D_StackFree  ( oldstack_secp ) ;
    }  // sec
    errno = 0 ;
    resselect = select  ( fd_hwm  + 1 , flagemptysetr ? 0 : & read_set  ,
      flagemptysetw ? 0 : & write_set ,
      flagemptysete ? 0 : & except_set  , & tv  ) ;
// ? to return time left : tv
    } else  {
    errno = 0 ;
    resselect = select  ( fd_hwm  + 1 , flagemptysetr ? 0 :   & read_set  ,
      flagemptysetw ? 0 : & write_set ,
      flagemptysete ? 0 : & except_set  , 0 ) ; 
  }
  if  ( resselect  ==  0  ) {
    // timeout
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
    PassagePassOut ( passageGCCPub ) ;
    return TaskForThread_Analyse  ; 
  }
  if  ( resselect  < 0  ) {
// ! to return SymbolErrno    
    int interrno  = errno ;
    { wchar_t const * wse ;
    switch  ( interrno  ) {
    case  EBADF :
      wse = L"BADF" ;
      break ;
    case  EINTR :
      wse = L"INTR" ;
      break ;
    case  EINVAL  :
      wse = L"INVAL"  ;
      break ;
    case  ENOMEM  :
      wse = L"NOMEM"  ;
      break ;
    default :
      wse = L"?"  ; 
    }
    PassagePassIn ( passageGCCPub ) ;
    { jmp_buf jb2  ;
      jmp_buf * const jbold = Lisp2DJBglobal  ;
      Lisp2DJBglobal  = & jb2 ;
      int const volatile jbn = setjmp  ( jb2 ) ;
      if  ( jbn  ) {
        Lisp2DJBglobal  = jbold ;
        PassagePassOut ( passageGCCPub ) ;
        return  TaskForThread_Analyse ;
      }
      PLOStringPub * const newstr = PLOString_NewCopy ( wse ) ;
      Lisp2DJBglobal  = jbold ;
      StackClusterPrivate_SetResult ( stclpp ,  &  newstr ->  plo ) ;
    } 
    } // wse
    PassagePassOut ( passageGCCPub ) ;
    return TaskForThread_Analyse  ; 
  }
  ParaLispObject  * resultr = & plonil . plo ;
  PLOCons * lastcons = 0 ;
  { ThreadStackPointerType const oldstack_ci = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const ci = NewStackCConsIterator ( ) ;
    CConsIteratorInit ( ci , lofsrp ) ;
  for ( ; CConsIteratorHaveItem ( ci ) ; CConsIteratorNext ( ci ) ) {
    ParaLispObject  * const xp = CConsIteratorItem ( ci ) ;
    int const xd  = PLOSocket_Descriptor  ( 
      ParaLispObjectToPLOSocket ( xp  ) ) ;
    if  ( xd >=  0  )
      if  ( FD_ISSET  ( xd  , & read_set  ) ) {
        PLOCons * const newcons =  NewCons  ( xp  , & plonil  . plo ) ;
        if  ( lastcons ==  0 ) {
          resultr = & newcons ->  plo ;
          lastcons  = newcons ; 
        } else  {
          ClassCons_SetRest ( & lastcons  ->  cc  , & newcons ->  plo ) ;
          lastcons  = newcons ; 
        } 
      } 
    } // for int fd
    Lisp2D_StackFree  ( oldstack_ci ) ;
  } // cip
  ParaLispObject  * resultw = & plonil  . plo ;
  lastcons  = 0 ;
  { ThreadStackPointerType const oldstack_ci = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const ci = NewStackCConsIterator ( ) ;
    CConsIteratorInit ( ci , lofsw ) ;
  for ( ; CConsIteratorHaveItem ( ci ) ; CConsIteratorNext ( ci ) ) {
    ParaLispObject  * const xp = CConsIteratorItem  ( ci ) ;
    int const xd  = PLOSocket_Descriptor  ( ParaLispObjectToPLOSocket ( xp  ) ) ;
    if  ( xd >=  0  )
      if  ( FD_ISSET  ( xd  , & write_set ) ) {
        PLOCons * const  newcons = NewCons  ( xp  , & plonil  . plo ) ;
        if  ( lastcons ==  0 ) {
          resultw = & newcons ->  plo ;
          lastcons  = newcons ; 
        } else  {
          ClassCons_SetRest ( & lastcons  ->  cc  , & newcons ->  plo ) ;
          lastcons  = newcons ; 
        } 
      } 
  } // for int fd
    Lisp2D_StackFree  ( oldstack_ci ) ;
  } // cip
  ParaLispObject  * resulte = & plonil  . plo ;
  lastcons  = 0 ;
  { ThreadStackPointerType const oldstack_ci = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const ci = NewStackCConsIterator ( ) ;
    CConsIteratorInit ( ci , lofse ) ;
  for ( ; CConsIteratorHaveItem ( ci ) ;
    CConsIteratorNext ( ci ) ) {
    ParaLispObject  * const xp = CConsIteratorItem  ( ci ) ;
    int const xd  = PLOSocket_Descriptor  ( 
      ParaLispObjectToPLOSocket ( xp  ) ) ;
    if  ( xd >=  0  )
      if  ( FD_ISSET  ( xd  , & except_set  ) ) {
        PLOCons * const  newcons  = NewCons ( xp  , & plonil  . plo ) ;
        if  ( lastcons ==  0 ) {
          resulte = & newcons ->  plo ;
          lastcons  = newcons ; 
        } else  {
          ClassCons_SetRest ( & lastcons  ->  cc  , & newcons ->  plo ) ;
          lastcons  = newcons ; 
        } 
      } 
    } // for int fd
    Lisp2D_StackFree  ( oldstack_ci ) ;
  } // cip
  PLOCons * result  = NewCons ( resulte , & plonil  . plo ) ;
  result  =  NewCons  ( resultw , & result  ->  plo ) ;
  result  =  NewCons  ( resultr , & result  ->  plo ) ;
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate_SetResult ( stclpp ,  &  result ->  plo ) ;
  PassagePassOut ( passageGCCPub ) ;
  return TaskForThread_Analyse  ; 
}

// returns list(number name alias .. alias) or nil  
ParaLispObject  * ProtoEntToList  ( struct protoent  * const strprot2p  ,
  StackClusterPrivate * const stclp ) {
  PLOComplex  *  const pcomplexProto=NewComplexBareExc( );
  Complex_SetInt  ( PLOComplexToComplex( pcomplexProto),strprot2p->p_proto);
  PLOCons * const consRes = NewCons( & pcomplexProto->plo,&plonil.plo);
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( WideChar ) * const wstrnamec = StackNew_StringWideChar ( ) ;
  String_Init ( WideChar  ) ( wstrnamec , true ) ;
  String  ( Char ) * const strtmpc = StackNew_StringChar ( ) ;
  StringConstrCStr ( Char  ) ( strtmpc , strprot2p  ->  p_name , true ) ;
  if ( isBADReadCStringToWString2 ( strtmpc , wstrnamec ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    BadStringN(stclp );
    return & plonil.plo ;
  }
  PLOCons * consLast=consRes;
  PLOCons * consNext;
  { PLOStringPub * const  strName2 = PLOString_NewMoveString ( wstrnamec ) ;
    consNext  =   NewCons(&( strName2->plo),&plonil.plo);
    ClassCons_SetRest(&consLast->cc,& consNext->plo);
    consLast  = consNext  ;
  }
  { char  * * listStr = strprot2p -> p_aliases;
    for ( ; *  listStr  ; ++  listStr ) {
      StringSetArray  ( Char  ) ( strtmpc , * listStr ) ;
      StringClear ( WideChar  ) ( wstrnamec ) ;
      if ( isBADReadCStringToWString2 ( strtmpc , wstrnamec ) ) {
        Lisp2D_StackFree  ( oldstack ) ;
        BadStringN(stclp );
        return & plonil.plo ;
      }
      { PLOStringPub * const  strName2 = PLOString_NewMoveString ( wstrnamec ) ;
        consNext  =   NewCons(&( strName2->plo),&plonil.plo);
        ClassCons_SetRest(&consLast->cc,& consNext->plo);
        consLast  = consNext  ;
      }
    } // for listStr
    Lisp2D_StackFree  ( oldstack ) ;
  } // listStr
  return & (  consRes->plo ) ;
}

bool  IsNotSocket ( StackClusterPrivate  * const stclp , ParaLispObject * const  xp  ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp ,
    . typetype  = TypeGlobalListMemberPLOSocket ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
}

void  PLOSocket_InitD ( PLOSocket * const me , int const d ) {
  PLOSocketPri * const mepri = PLOSocketPriFromPub ( me ) ;
  mepri -> descriptor  = d ;
  mepri -> usintdomain = 0 ;
  mepri -> intDomainType = 0 ;
  VirtualTypePointer vtp = { . addr = { . p = mepri } , . type = TypeGlobalListMemberPLOSocket } ;
  ParaLispObject_InitVTP(&mepri->pub.plo,vtp);
# ifdef  LISP2D_CONSTRSTAT
  PLOSocket_constrstatInc ( ) ;
# endif
}

void PLOSocket_Init ( PLOSocket * const me ) {
  PLOSocketPri * const mepri = PLOSocketPriFromPub ( me ) ;
  mepri -> descriptor  = -1  ;
  mepri -> usintdomain = 0 ;
  mepri -> intDomainType = 0 ;
  VirtualTypePointer vtp =  { . addr = { . p = mepri } , . type = TypeGlobalListMemberPLOSocket };
  ParaLispObject_InitVTP(&mepri->pub.plo,vtp);
# ifdef  LISP2D_CONSTRSTAT
  PLOSocket_constrstatInc ( ) ;
# endif
}

# include "castb.h"
# include "widestring/inline.h"
# include "socket/inline.h"
