// Lisp2D - Stack universal types
// ©2019-23 by Alex Glebe

# include <setjmp.h>
# include <iso646.h>
# include <stdlib.h>
# include "excc.h"
# include <stdint.h>

// ! how to set _Alignof Objects and _Alignof type ?

/*
SP - new :
    uint8_t type
        Object
SP - old :
 */

# include "stack/private.h"

// its Stack , individual for all threads
_Thread_local uint8_t * Lisp2DThreadStack = 0 ;

# include "stack/enum.h"
# include "stack/type.h"

// Stack pointer begins from bottom address
_Thread_local Lisp2DThreadStackPointerType  Lisp2D_thread_stack_sp = Lisp2DThreadStackSIZE ;
// size of each types may be in stack
Lisp2DThreadStackSizeType  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeSIZE ] ;
// array of destructor for all types (may be NULL )
StackDestructorType Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeSIZE ] ;

# define  uint32tcaststackpt  Lisp2D_uint32tcaststackpt
inline  Lisp2DThreadStackPointerType  uint32tcaststackpt  ( uint32_t  s ) ;

# include "stack/public.h"

void * Lisp2D_StackVar ( ThreadStackTypeType const t ) {
  if  ( not Lisp2DThreadStack ) {
    Lisp2DExcStackGlobal . value = L"StackVar:not ThreadStack" ;
    longjmp(*Lisp2DJBglobal,Lisp2DExcTypeStack) ; 
  }
  { Lisp2DThreadStackSizeType const ts = ( Lisp2DThreadStackTypeSize [ t ] ) ;
    if ( Lisp2D_thread_stack_sp < ( ts + sizeof ( Lisp2DThreadStackTypeType ) ) ) {
      Lisp2DExcStackGlobal . value = L"StackVar:stack overflow" ;
      longjmp(*Lisp2DJBglobal,Lisp2DExcTypeStack);
    }
    Lisp2D_thread_stack_sp -=  ts ; 
  }
  void * const res = Lisp2DThreadStack + Lisp2D_thread_stack_sp ;
  Lisp2D_thread_stack_sp -= uint32tcaststackpt ( sizeof ( Lisp2DThreadStackTypeType ) ) ;
  ( * uint8pcastLisp2DThreadStackTypeTypep ( Lisp2DThreadStack + Lisp2D_thread_stack_sp  ) ) = t ;
  return res ; 
}

# include "define.h"

void  Lisp2D_StackFree  ( ThreadStackPointerType  const old ) {
  // old very small , is upper
  if ( Lisp2D_thread_stack_sp > old ) {
    Lisp2DExcStackGlobal . value = L"StackFree:thread_stack_sp > old" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeStack  ) ; 
  }
  // old very big , out of stack border
  if ( old > Lisp2DThreadStackSIZE ) {
    Lisp2DExcStackGlobal . value = L"StackFree:old > ThreadStackSIZE" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeStack  ) ; 
  }
  if  ( not Lisp2DThreadStack ) {
    Lisp2DExcStackGlobal . value = L"StackFree:not ThreadStack" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeStack  ) ; 
  }
  // [SP..old..SIZE]
  while ( Lisp2D_thread_stack_sp < old ) {
    ThreadStackTypeType t = ( * uint8pcastLisp2DThreadStackTypeTypep (
      Lisp2DThreadStack + Lisp2D_thread_stack_sp  ) ) ;
# ifdef LISP2D_DEBUG
    if  ( t >=  Lisp2DStackTypeSIZE ) {
      Lisp2DExcStackGlobal . value = L"StackFree:t >=  Lisp2DStackTypeSIZE" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeStack  ) ;
    }
# endif      
    Lisp2D_thread_stack_sp += uint32tcaststackpt (
      sizeof ( ThreadStackTypeType ) ) ;
    { StackDestructorType d = Lisp2DThreadStackTypeDestructor [ t ] ;
      if  ( d ) 
        d ( Lisp2DThreadStack + Lisp2D_thread_stack_sp ) ; 
    }
    Lisp2D_thread_stack_sp +=  Lisp2DThreadStackTypeSize [ t ] ; 
  }
  // stack was broken
  if ( Lisp2D_thread_stack_sp not_eq old ) {
    Lisp2DExcStackGlobal . value = L"StackFree:thread_stack_sp not_eq old" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeStack  ) ; 
  } 
}

inline  Lisp2DThreadStackPointerType  uint32tcaststackpt  ( uint32_t  const s ) {
  return  ( Lisp2DThreadStackPointerType ) s ;
}
