// Lisp2D - Stack universal types
// all types characteristics for stack
// ©2019-23 by Alex Glebe
# include "deque/public-wchar.h"
# include "deque/public-byte.h"
# include "vector-unslongint.h"
# include "complex/private.h"
# include "fraction/inlinepri.h"
# include "ulong/private.h"
# include "consiter/struct.h"
# include "stack-plop.h"
# include "slong/private.h"
# include "stack/type.h"

# define  ulongdest_cast_stack  Lisp2D_ulongdest_cast_stack
inline  Lisp2D_StackDestructorType  ulongdest_cast_stack ( void  ( * f )  ( ULong * ) ) ;

# define  slongdest_cast_stack  Lisp2D_slongdest_cast_stack
inline  Lisp2D_StackDestructorType  slongdest_cast_stack ( void  ( * f )  ( Lisp2DSLong * ) ) ;

# define  strchardest_cast_stack  Lisp2D_strchardest_cast_stack
inline  Lisp2D_StackDestructorType  strchardest_cast_stack ( void  ( * f )  ( String ( Char ) * ) ) ;

# define  strwchardest_cast_stack  Lisp2D_strwchardest_cast_stack
inline  Lisp2D_StackDestructorType  strwchardest_cast_stack ( void  ( * f )  ( String ( WideChar ) * ) ) ;

# define  fractiondest_cast_stack  Lisp2D_fractiondest_cast_stack
inline  Lisp2D_StackDestructorType  fractiondest_cast_stack ( void  ( * f )  ( Fraction2 * ) ) ;

# define  vectorstackunsintdest_cast_stack Lisp2D_vectorstackunsintdest_cast_stack
inline  Lisp2D_StackDestructorType  vectorstackunsintdest_cast_stack ( void  ( * f )  ( Vector ( StackUnsInt ) * ) ) ;

# define  vectorstackplopdest_cast_stack  Lisp2D_vectorstackplopdest_cast_stack
inline  Lisp2D_StackDestructorType  vectorstackplopdest_cast_stack ( void  ( * f )  ( Vector ( StackPLOP ) * ) ) ;

# define  dequewchardest_cast_stack  Lisp2D_dequewchardest_cast_stack
inline  Lisp2D_StackDestructorType  dequewchardest_cast_stack ( void  ( * f )  ( Deque  ( WChar ) * ) ) ;

# define  dequebytedest_cast_stack  Lisp2D_dequebytedest_cast_stack
inline  Lisp2D_StackDestructorType  dequebytedest_cast_stack ( void  ( * f )  ( Deque  ( Byte ) * ) ) ;

# define  vectorunslongintdest_cast_stack Lisp2D_vectorunslongintdest_cast_stack
inline  Lisp2D_StackDestructorType  vectorunslongintdest_cast_stack ( void  ( * f )  ( Vector ( UnsLongInt ) * ) ) ;

# define  complexdest_cast_stack  Lisp2D_complexdest_cast_stack
inline  Lisp2D_StackDestructorType  complexdest_cast_stack ( void  ( * f )  ( ComplexPri * ) ) ;

# define  stack_destr_zero  Lisp2D_stack_destr_zero
static  StackDestructorType const stack_destr_zero  = 0 ;

# include "stack/public.h"

void  StackTypesInit ( void ) {
  
  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeULong  ] = sizeof  ( ULong ) ;
  Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeULong  ] = ulongdest_cast_stack  ( ULongDestr  ) ;
    
  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeSLong ] = sizeof ( Lisp2DSLong ) ;
  Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeSLong ] = slongdest_cast_stack ( Lisp2DSLongDestr ) ;
    
  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeStringChar ] = sizeof  ( String  ( Char  ) ) ;
  Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeStringChar ] = strchardest_cast_stack ( Lisp2DStringCharDestr ) ;
    
  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeStringWideChar ] = sizeof  ( String  ( WideChar  ) ) ;
  Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeStringWideChar ] = strwchardest_cast_stack ( StringDestr ( WideChar  ) ) ;

  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeFraction ] = sizeof  ( Fraction2 ) ;
  Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeFraction ] = fractiondest_cast_stack ( FractionDestrPri ) ;
    
  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeStackUnsInt ] = sizeof  ( Vector  ( StackUnsInt ) ) ;
  Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeStackUnsInt ] =
    vectorstackunsintdest_cast_stack  ( Vector_Destr  ( StackUnsInt ) ) ;
    
  Lisp2DThreadStackTypeSize [ Lisp2DStackTypeStackPLOP ] = sizeof  ( Vector  ( StackPLOP ) ) ;
  Lisp2DThreadStackTypeDestructor [ Lisp2DStackTypeStackPLOP ] = vectorstackplopdest_cast_stack ( Vector_Destr  ( StackPLOP ) ) ;

  Lisp2DThreadStackTypeSize [ StackType ( Deque_WChar ) ] = sizeof  ( Deque  ( WChar ) ) ;
  Lisp2DThreadStackTypeDestructor [ StackType ( Deque_WChar ) ] = dequewchardest_cast_stack ( Deque_Destr ( WChar ) ) ;

  Lisp2DThreadStackTypeSize [ StackType ( Deque_Byte  ) ] = sizeof  ( Deque  ( Byte ) ) ;
  Lisp2DThreadStackTypeDestructor [ StackType ( Deque_Byte  ) ] = dequebytedest_cast_stack ( Deque_Destr ( Byte  ) ) ;

  Lisp2DThreadStackTypeSize [ StackType ( VectorUnsignedLongInt  ) ] = sizeof  ( Vector  ( UnsLongInt ) ) ;
  Lisp2DThreadStackTypeDestructor [ StackType ( VectorUnsignedLongInt  ) ] =
    vectorunslongintdest_cast_stack ( Vector_Destr ( UnsLongInt ) ) ;
   
  Lisp2DThreadStackTypeSize [ StackType ( ComplexPri ) ] = sizeof ( ComplexPri  ) ;
  Lisp2DThreadStackTypeDestructor [ StackType ( ComplexPri ) ] = complexdest_cast_stack ( ComplexPri_Destr ) ;
    
  Lisp2DThreadStackTypeSize [ StackType ( CConsIterator ) ] = sizeof ( CConsIterator  ) ;
  Lisp2DThreadStackTypeDestructor [ StackType ( CConsIterator ) ] = stack_destr_zero ;

}

inline  Lisp2D_StackDestructorType  ulongdest_cast_stack ( void  ( * const f )  ( ULong * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  slongdest_cast_stack ( void  ( * const f )  ( Lisp2DSLong * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  strchardest_cast_stack ( void  ( * const f )  ( String ( Char ) * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  strwchardest_cast_stack ( void  ( * const f )  ( String ( WideChar ) * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  fractiondest_cast_stack ( void  ( * const f )  ( Fraction2 * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  vectorstackunsintdest_cast_stack ( void  ( * const f )  ( Vector ( StackUnsInt ) * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  vectorstackplopdest_cast_stack ( void  ( * const f )  ( Vector ( StackPLOP ) * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  dequewchardest_cast_stack ( void  ( * const f )  ( Deque  ( WChar ) * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  dequebytedest_cast_stack ( void  ( * const f )  ( Deque  ( Byte ) * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  vectorunslongintdest_cast_stack ( void  ( * const f )  ( Vector ( UnsLongInt ) * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}

inline  Lisp2D_StackDestructorType  complexdest_cast_stack ( void  ( * const f )  ( ComplexPri * ) ) {
  return  ( Lisp2D_StackDestructorType ) f ;
}
