// Lisp2D - Stat
// ©2017-23 by Alex Glebe

# include "stat/public.h"
# include "stat/structpri.h"
# include "stcl/classc.h"
# include "stat/private.h"

PLOStat * NewStat ( StackClusterPrivate * const stclpp , struct stat const * const ss ) {
  PLOStatPri * const res = aligned_alloc ( _Alignof ( PLOStatPri ) , sizeof ( PLOStatPri ) ) ;
  if ( not res ) {
    OutOfMemory ( stclpp  ) ;
    return  0 ;
  }
  PLOStat_InitStruct  ( & res -> pub , ss  ) ;
  ParaLispObject_virtualPushAsNew ( & res -> pub  . plo ) ;
  return & res  ->  pub ;
}

# include "plostream/public.h"

void  PLOStat_staticCoutToInterpret ( ParaLispObject const * _ __attribute__ ((unused)),
  PLOStream  const * const s , bool _2 __attribute__ ((unused)), Vector  ( StackPLOP ) * _3 __attribute__ ((unused))) {
  PLOStreamPushWideCharArray  ( s , ( PArray_wchar_c ) { (  wchar_t const ( * ) [ ] ) & L"Stat" } ) ;
}

ParaLispObject  * PLOStat_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  & plosstat . plo  ;
}

ParaLispObject  * PLOStat_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassdefstat ;
}

ParaLispObject  * PLOStat_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   ploclassparentsstat ;
}

# include "boolean.h"
# include "cons/public.h"
# include "cons/inline.h"
# include "stcl/public.h"
# include "stcl/inline.h"

// nil statp [x]
TaskForThread Lisp2Dbimnilstatp ( StackClusterPrivate * const stclpp  ) {
  ParaLispObject  * const arglistp =  StackClusterPrivate_ArgTop  ( stclpp  ) ;
  ParaLispObject  *   res = & plofalse .  plo ;
  if  ( CConsp ( arglistp ) ) {
    ParaLispObject  * const xp = ClassCons_First  ( & ParaLispObjectToPLOCons ( arglistp )  ->  cc ) ;
    if  ( StatP ( xp ) )
      res = xp ;
  }
  StackClusterPrivate_SetResult ( stclpp , res ) ;
  return   TaskForThread_Analyse  ;
}

//  stat setf [stat]
TaskForThread Lisp2Dbimstatsetf ( StackClusterPrivate  * const stclpp  ) {
  PLOStat * const mep  =  ParaLispObjectToPLOStat ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ParaLispObject  * const arglistp =  StackClusterPrivate_ArgTop  ( stclpp ) ;
  if  ( CConsp ( arglistp ) ) {
    ParaLispObject  * const yp = ClassCons_First  ( & ParaLispObjectToPLOCons ( arglistp )  ->  cc ) ;
    if  ( IsNotStat ( yp , stclpp ) )
      goto  Exit  ;
    PLOStat_SetCopy ( mep ,  ParaLispObjectToPLOStat  ( yp ) ) ;
  }
  StackClusterPrivate_SetResult ( stclpp ,  & mep ->  plo  ) ;
Exit  :
  return   TaskForThread_Analyse  ;
}

# include "newobject.h"

bool  IsNotStat ( ParaLispObject * const  xp , StackClusterPrivate * const stclp  ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp  ,
    . typetype  = TypeGlobalListMemberPLOStatPri ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
}

# include "sharedc.h"
# include "shared/inline.h"

//  stat swap [stat]
TaskForThread Lisp2Dbimstatswap ( StackClusterPrivate  * const stclpp  ) {
  PLOStat * const mep  =  ParaLispObjectToPLOStat ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ParaLispObject  * const xp =  StackClusterPrivate_ArgTop  ( stclpp ) ;
  if  ( CConsp ( xp ) ) {
    ParaLispObject  * const yp =  ClassCons_First ( & ParaLispObjectToPLOCons ( xp )  ->  cc ) ;
    if  ( IsNotStat ( yp , stclpp ) )
      goto  Exit  ;
    if  ( IsConst (  stclpp  , yp ) )
      goto  Exit  ;
    PLOStat_Swap ( mep , ParaLispObjectToPLOStat  ( yp ) ) ;
  }
  StackClusterPrivate_SetResult ( stclpp ,  & mep ->  plo  ) ;
Exit  :
  return   TaskForThread_Analyse  ;
}

// stat file-type  
TaskForThread Lisp2Dbimstatfiletype ( StackClusterPrivate  * const stclpp  ) {
  PLOStat * const mep  =  ParaLispObjectToPLOStat ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  mode_t  const  ft  = ( * ( PLOStat_CMode ( mep ) ) )  bitand  S_IFMT  ;
  PLOSymbol * res ;
  switch  ( ft  ) {
  case  S_IFSOCK  :
    res = & plossocket  ;
    break ;
  case  S_IFLNK :
    res = & ploslink ;
    break ;
  case  S_IFREG :
    res = & plosregular ;
    break ;
  case  S_IFBLK :
    res = & plosblock ;
    break ;
  case  S_IFDIR :
    res = & plosdirectory ;
    break ;
  case  S_IFCHR :
    res = & ploscharacter ;
    break ;
  case  S_IFIFO :
    res = & plosfifo  ;
    break ;
  default :
    res = & plosquestion  ;
  }
  StackClusterPrivate_SetResult ( stclpp , &  res ->  plo ) ;
  return  TaskForThread_Analyse ;
}

# include "time/public.h"
# include "time/struct.h"

// stat access-time
TaskForThread Lisp2Dbimstataccesstime ( StackClusterPrivate  * const stclpp ) {
  { PLOStat * const mep  = ParaLispObjectToPLOStat(StackClusterPrivate_ThisTop(stclpp )) ;
    PLOTime * const res = aligned_alloc ( _Alignof ( PLOTime ) , sizeof ( PLOTime ) ) ;
    if  ( res ) {
      PLOTime_Inittimespec(res,* ( PLOStat_CATime ( mep ) ));
      StackClusterPrivate_SetResult(stclpp , ParaLispObject_virtualPushAsNew(&res-> pub . plo));
    } else
      OutOfMemory( stclpp );
  }
  return  TaskForThread_Analyse;
}

// stat modify-time
TaskForThread Lisp2Dbimstatmodifytime ( StackClusterPrivate  * const stclpp  )  {
  { PLOStat * const mep  = ParaLispObjectToPLOStat(StackClusterPrivate_ThisTop(stclpp )) ;
    PLOTime * const res = aligned_alloc ( _Alignof ( PLOTime ) , sizeof ( PLOTime ) ) ;
    if  ( res ) {
      PLOTime_Inittimespec(res,* ( PLOStat_CMTime ( mep ) ));
      StackClusterPrivate_SetResult(stclpp , ParaLispObject_virtualPushAsNew(&res-> pub . plo));
    } else
      OutOfMemory( stclpp );
  }
  return  TaskForThread_Analyse;
}

// stat change-time
TaskForThread Lisp2Dbimstatchangetime (
  StackClusterPrivate  * const stclpp  )  {
  { PLOStat * const mep  = ParaLispObjectToPLOStat(StackClusterPrivate_ThisTop(stclpp )) ;
    PLOTime * const res = aligned_alloc ( _Alignof ( PLOTime ) , sizeof ( PLOTime ) ) ;
    if  ( res ) {
      PLOTime_Inittimespec(res,* ( PLOStat_CCTime ( mep ) ));
      StackClusterPrivate_SetResult(stclpp ,
        ParaLispObject_virtualPushAsNew(&res-> pub . plo));
    } else
      OutOfMemory( stclpp );
  }
  return  TaskForThread_Analyse;
}

void  PLOStat_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool flag2pprinted  ,
  Vector(StackPLOP)  * const h ) {
  PLOStat_staticCoutToInterpret ( me , s , flag2pprinted  , h ) ;
}

void PLOStat_Destr  ( PLOStat * const me ) {
  ParaLispObject_staticDestr  ( & me  ->  plo ) ;
}

# define  PLOStat_staticDestr Lisp2D_PLOStat_staticDestr
void  PLOStat_staticDestr  ( ParaLispObject * const me ) {
  PLOStat_Destr(ParaLispObjectToPLOStat ( me ) ) ;
}

# include "sharedc.h"

//  stat permissions
TaskForThread Lisp2Dbimstatpermissions ( StackClusterPrivate  * const stclpp  ) {
  PLOStat * const mep = ParaLispObjectToPLOStat ( StackClusterPrivate_ThisTop ( stclpp  ) ) ;
  { mode_t  const ft  = * PLOStat_CMode ( mep ) ;
    ParaLispObject  * res = & plonil  . plo ;
    for ( int i = 0 ; i < array_mode_symbol_size  ; ++  i )
      if  ( ft bitand ( ( * arrayp_mode_symbol  ) [ i ] . mo ) )
        res = & NewCons ( & ( ( * arrayp_mode_symbol  ) [ i ] . sp  ->  plo ) , res ) -> plo ;
    StackClusterPrivate_SetResult ( stclpp , res ) ;
  } // ft
  return  TaskForThread_Analyse ;
}
  
# include "plostream/public.h"
# include "string/public.h"

// (stat stat ) -> stat
// stat0 stat stat1 -> stat0 setf stat1
// stat stat string -> string stat thisstat
// stat stat stream -> stream stat thisstat
TaskForThread Lisp2Dbimstatstat ( StackClusterPrivate  * const stclpp  ) {
  PLOStat * const mep  = ParaLispObjectToPLOStat( StackClusterPrivate_ThisTop(stclpp)) ;
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp) ;
  if  ( CConsp ( arglistp ) ) {
    ParaLispObject  * const yp =ClassCons_First(& ParaLispObjectToPLOCons( arglistp )->cc ) ;
    switch  (    ParaLispObject_Type  (yp )  ) {
    case  TypeGlobalListMemberPLOStream  :
      return  bimstreamstat ( stclpp , ParaLispObjectToPLOStream( yp ) , mep  )  ;
    case  TypeGlobalListMemberPLOString  :
      return  bimstringstat ( stclpp , ParaLispObjectToPLOString( yp ) , mep  )  ;
    case  TypeGlobalListMemberPLOStatPri  :
      PLOStat_SetCopy ( mep ,  ParaLispObjectToPLOStat( yp ) ) ;
      break ;
    default :
      StackClusterPrivate_ExceptionSet  (stclpp,& PLOString_NewCopy ( 
        L"stat::stat: argument type not supported"  )->plo ) ;
      goto  Exit  ;
    }
  }
  StackClusterPrivate_SetResult (stclpp,& mep ->plo ) ;    
Exit  :
  return   TaskForThread_Analyse  ;
}
  
# include "plocomplex/public.h"
# include "cast.h"

static  unsigned int Lisp2D_major ( unsigned long long int const dev ) {
  return (ullint_cast_uint((dev >> 8) bitand 0xfff)) bitor
    (ullint_cast_uint (dev >> 32) bitand ( compl 0xfffU));
}
  
static  unsigned int Lisp2D_minor ( unsigned long long int const dev ) {
  return (ullint_cast_uint(dev bitand 0xff)) bitor
    (ullint_cast_uint (dev >> 12) bitand (compl 0xffU));
}

//  stat device
TaskForThread Lisp2Dbimstatdevice ( StackClusterPrivate  * const stclpp  ) {
  { PLOComplex  * const resmajor =  NewComplexBareExc  (   ) ;
    if  ( not resmajor  )
      goto  Exit  ;
    PLOComplex  * const resminor = NewComplexBareExc  (   ) ; 
    if  ( not resminor  )
      goto  Exit  ;
    ParaLispObject  * list [ ] = { &resmajor->plo  , &resminor->plo  , NULL } ;
    StackClusterPrivate_SetResult (
      stclpp ,ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    PLOStat * const mep  =
      ParaLispObjectToPLOStat(StackClusterPrivate_ThisTop(stclpp ))  ;
    unsigned  int   tmp = Lisp2D_major ( * ( PLOStat_CDev ( mep )  ) ) ;
    Complex_SetUInt ( PLOComplexToComplex( resmajor) , tmp  ) ;
    tmp = Lisp2D_minor ( * ( PLOStat_CDev ( mep )  ) ) ;
    Complex_SetUInt ( PLOComplexToComplex( resminor) , tmp ) ;
  }
Exit  :
  return  TaskForThread_Analyse ;
}

//  stat rdevice
TaskForThread Lisp2Dbimstatrdevice ( StackClusterPrivate  * const stclpp  ) {
  { PLOComplex  * const resmajor = NewComplexBareExc  (   ) ;
    PLOComplex  * const resminor  = NewComplexBareExc  (   ) ; 
    ParaLispObject  * list [ ] = { &resmajor->plo  , &resminor->plo  , NULL } ;
    StackClusterPrivate_SetResult (stclpp ,
      ListOfPLO(( PArrayPLOP){(ParaLispObject*(*)[])&list}));
    PLOStat * const mep  = ParaLispObjectToPLOStat(
      StackClusterPrivate_ThisTop(stclpp )) ;
    unsigned  int   tmp = Lisp2D_major ( * ( PLOStat_CRDev ( mep )  )  ) ;
    Complex_SetUInt ( PLOComplexToComplex( resmajor) , tmp  ) ;
    tmp = Lisp2D_minor ( * ( PLOStat_CRDev ( mep )  ) ) ;
    Complex_SetUInt ( PLOComplexToComplex( resminor) , tmp ) ;
  }
  return  TaskForThread_Analyse ;
}
  
# include "complex/inline.h"

//  stat inode
TaskForThread Lisp2Dbimstatinode ( StackClusterPrivate  * const stclpp  ) {
  PLOComplex  *  const res = NewComplexBareExc  (   )  ;  
  StackClusterPrivate_SetResult (stclpp ,&  res->plo ) ;
  Complex * const res2  = PLOComplexToComplex ( res ) ;
  { PLOStat * const mep  = ParaLispObjectToPLOStat(
      StackClusterPrivate_ThisTop(stclpp )) ;
    enum  { bitsize = sizeof  ( ino_t ) <<  3 } ;
    FractionSetFromBits ( Complex_Re(res2),
      PLOStat_CInode ( mep )  , bitsize ) ;  
  }
  FractionSet0  (Complex_Im(res2));
  return  TaskForThread_Analyse; 
}

// stat links
TaskForThread Lisp2Dbimstatlinks ( StackClusterPrivate  * const stclpp  ) {
  PLOComplex  * const  res =  NewComplexBareExc( );
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  Complex * const res2  = PLOComplexToComplex ( res ) ;
  { PLOStat * const mep  = ParaLispObjectToPLOStat(
      StackClusterPrivate_ThisTop(stclpp )) ;
    enum { bitsize = sizeof( nlink_t ) <<  3 } ;
    FractionSetFromBits (Complex_Re(res2),
      PLOStat_CLinks ( mep ),bitsize); 
  }
  FractionSet0  (Complex_Im(res2) );
  return  TaskForThread_Analyse;
}  

// stat uid
TaskForThread Lisp2Dbimstatuid ( StackClusterPrivate  * const stclpp  )  {
  PLOComplex  * const  res = NewComplexBareExc ( );
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  Complex * const res2  = PLOComplexToComplex ( res ) ;
  { PLOStat * const mep  =ParaLispObjectToPLOStat(
      StackClusterPrivate_ThisTop(stclpp ))  ;
    enum { bitsize =sizeof( uid_t ) <<  3 } ;
    FractionSetFromBits (Complex_Re(res2),
      PLOStat_CUID ( mep ),bitsize);  
  }
  FractionSet0  (Complex_Im(res2));
  return  TaskForThread_Analyse; 
}  

// stat gid
TaskForThread Lisp2Dbimstatgid ( StackClusterPrivate  * const stclpp  )  {
  PLOComplex  * const  res = NewComplexBareExc ( );
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  Complex * const res2  = PLOComplexToComplex ( res ) ;
  { PLOStat * const mep  =ParaLispObjectToPLOStat(
      StackClusterPrivate_ThisTop(stclpp )) ;
    enum { bitsize=sizeof( gid_t ) <<  3 } ;
    FractionSetFromBits (Complex_Re(res2),
      PLOStat_CGID ( mep ),bitsize);  
  }
  FractionSet0  (Complex_Im(res2));
  return  TaskForThread_Analyse;
}  

// stat totalsize
TaskForThread Lisp2Dbimstattotalsize ( StackClusterPrivate  * const stclpp  )  {
  PLOComplex  * const  res =  NewComplexBareExc ( );
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  Complex * const res2  = PLOComplexToComplex ( res ) ;
  { PLOStat * const mep  =ParaLispObjectToPLOStat(
    StackClusterPrivate_ThisTop(stclpp )) ;
    enum { bitsize=sizeof(off_t) <<  3 } ;
    FractionSetFromBits (Complex_Re(res2),
      PLOStat_CSize ( mep ),bitsize); 
  }
  FractionSet0(Complex_Im(res2));
  return  TaskForThread_Analyse;
}  

// stat blocksize
TaskForThread Lisp2Dbimstatblocksize ( StackClusterPrivate  * const stclpp  )  {
  PLOComplex  * const  res =  NewComplexBareExc ( );
  StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  Complex * const res2  = PLOComplexToComplex ( res ) ;
  { PLOStat * const mep  = ParaLispObjectToPLOStat(
      StackClusterPrivate_ThisTop(stclpp )) ;
    enum { bitsize=sizeof(blksize_t) <<  3 } ;
    FractionSetFromBits (Complex_Re(res2),
      PLOStat_CBSize ( mep ) ,bitsize); 
  }
  FractionSet0(Complex_Im(res2));
  return  TaskForThread_Analyse; 
}  

// stat blocks
TaskForThread Lisp2Dbimstatblocks ( StackClusterPrivate  * const stclpp  )  {
  PLOComplex  * const  res =  NewComplexBareExc ( );
  StackClusterPrivate_SetResult(stclpp ,&  res->plo); 
  Complex * const res2  = PLOComplexToComplex ( res ) ;
  { PLOStat * const mep  =ParaLispObjectToPLOStat(
      StackClusterPrivate_ThisTop(stclpp )) ;
    enum { bitsize =sizeof(blkcnt_t) <<  3 } ;
    FractionSetFromBits (Complex_Re(res2),
      PLOStat_CBlocks ( mep ),bitsize); 
  }
  FractionSet0(Complex_Im(res2));
  return  TaskForThread_Analyse; 
}

# include "plo/public.h"

void  PLOStat_InitStruct  (  PLOStat * const me , struct stat const * const s ) {
  PLOStatPri * const p = PLOStat_toPrivate  ( me ) ;
  VirtualTypePointer  tmp = { . addr = { . p = p } , . type = TypeGlobalListMemberPLOStatPri } ;
  ParaLispObject_InitVTP  ( & me -> plo , tmp ) ;
  p -> mystat = * s ;
}

void  PLOStat_Init ( PLOStat * const me ) {
  VirtualTypePointer  tmp = { . addr = { . p = PLOStat_toPrivate  ( me ) } , . type = TypeGlobalListMemberPLOStatPri } ;
  ParaLispObject_InitVTP ( & me -> plo , tmp ) ;
}

void  PLOStat_Swap  ( PLOStat * const me , PLOStat * const sp ) {
  if ( me not_eq   sp  ) {
    struct stat const tmp = PLOStat_toPrivate(me) -> mystat ;
    PLOStat_toPrivate(me) -> mystat = PLOStat_toPrivate(sp) -> mystat ;
    PLOStat_toPrivate(sp) -> mystat = tmp ; 
  } 
}

void  PLOStat_SetCopy ( PLOStat * const me , PLOStat const * const s ) {
  if ( me not_eq s ) 
    PLOStat_toPrivate(me) -> mystat  = ( PLOStat_toCPrivate(s) ->  mystat  ) ; 
}

void  PLOStat_SetValue ( PLOStat * const me , struct stat const * const s ) {
  if ( ( & PLOStat_toCPrivate(me) -> mystat ) not_eq s )
    PLOStat_toPrivate(me) -> mystat  = ( * s )  ; 
}

# include "castb.h"
# include "stat/inline.h"
