// Lisp2D interpreter - Statistics
// ©2016-23 by Alex Glebe
# include "define.h"
# ifdef  LISP2D_BIMSTAT
# include <math.h>  // sqrtl logl , Link with -lm.
# ifndef  M_PIl
# define  M_PIl 3.141592653589793238462643383279502884L
# endif
# include <iso646.h>  // not not_eq
# include "statistics.h"

// sqrtl  ( M_PIl ) / 2.0L
# define  LISP2DHALFSQRTPI \
  0.8862269254527580136554694434369849886934389360249042510986328125L

// logl ( M_PIl / 2.0L  )
# define  LISP2DLOGHALFPI \
  0.451582705289454864748706353072549291027826257050037384033203125L

static  long double  erfinv_positive2 ( long double  x ) {
  if  ( x < 0.66L )
    return  x *  LISP2DHALFSQRTPI ; 
  x  = logl ( 1.0L  - x ) ;
  return sqrtl  (  - ( x + ( ( logl ( - ( LISP2DLOGHALFPI + ( 2.0L  * x ) ) ) + LISP2DLOGHALFPI ) / 2.0L  ) ) ) ;
}
    
# include "cast.h"

// ! to run Exception      
bool Lisp2DStatisticsIsBADInit ( struct  Lisp2DStatistics * const me , size_t const s ) {
# ifdef LISP2D_DEBUG  
  switch  ( me ->  flagConstructed )  {
  case Lisp2DStatisticsLive :
    break ;
  case Lisp2DStatisticsNotConstructed :
    fputws  ( L"Lisp2DStatisticsIsBADInit:NotConstructed\n",stderr);
    return true ;
  case Lisp2DStatisticsDestructed :
    fputws  ( L"Lisp2DStatisticsIsBADInit:Destructed\n",stderr);
    return true ;
  default : 
    fputws  ( L"Lisp2DStatisticsIsBADInit:dead memory\n",stderr);
    return true ; 
  }
# endif
  // constructors for all sections with zero values
  { struct Lisp2DStatisticsSection * newsectionsStat = 0 ;
    if  ( s  > 0  ) {
      newsectionsStat = aligned_alloc ( _Alignof ( struct Lisp2DStatisticsSection [ s ] ) ,
        sizeof  ( struct Lisp2DStatisticsSection [ s ] ) ) ;
      if  ( not newsectionsStat )
        return  true  ;
      { struct Lisp2DStatisticsSection * i = newsectionsStat + s ;
        do  {
          --  i ;
          Lisp2DStatisticsSectionConstructor  ( i ) ;
        } while ( i not_eq  newsectionsStat ) ; 
      } // i
      if  ( s  > 1  ) {
        // no constructors for long doubles
        me -> lderfp = aligned_alloc ( _Alignof ( long  double [ s - 1 ] ) , sizeof  ( long  double [ s - 1 ] ) ) ;
        if  ( not ( me -> lderfp ) ) {
          // if lderf is null then newsectionsStat must be null
          free ( newsectionsStat ) ;
          newsectionsStat = 0 ;
          return true ; 
        } 
      } else
        me -> lderfp = 0 ;
    } // if  (  s   > 0  )
    me -> sectionsStat  = newsectionsStat ;
    if  ( not ( newsectionsStat ) ) {
      me -> sizeset  =  0 ;
      return  true  ; 
    } 
  } // newsectionsStat
  me -> sizeset = s ;
  for ( size_t  k = ( s >>  1 ) , i = 0 ; k > 0 ; --  k , ++  i ) {
    long  double  ierf  = 1.0L  - ( size_cast_longdouble ( i + 1 ) / size_cast_longdouble ( s ) ) * 2.0L ;
    ierf  = erfinv_positive2 ( ierf ) ;
    (*(me -> lderfp)) [ ( s - 2 ) - i ] = ierf  ;
    (*(me -> lderfp)) [ i ] = ( - ierf  ) ; 
  } // for i , k
  return  false ; 
}

void  StatisticsSectionPush2  ( struct  Lisp2DStatisticsSection * const s , long  double  const xv , long  double  const x2v ) {
  ( s ->  sumx ) +=  xv ;
  ( s ->  sumx2 ) +=  x2v ;
  ++ ( s ->  count ) ;
}

# define  StatisticsSectionGlobalPush Lisp2D_StatisticsSectionGlobalPush
static  void  StatisticsSectionGlobalPush  ( struct  Lisp2DStatisticsSectionGlobal * const g ,
  long  double  const xv , long  double  const x2v ) {
  StatisticsSectionPush2  ( &  ( g ->  s ) , xv , x2v ) ;
  g ->  average = Lisp2DStatisticsSectionCalcAverage  ( & ( g ->  s ) ) ;
  g ->  deviation = StatisticsSectionCalcDeviation ( & ( g ->  s ) , g ->  average ) ;
}
  
// the same
// Dispersy = (1/(n-1))*(Sum(x^2)-Aver(x)*Summ(x))  
long  double  StatisticsSectionCalcDeviation ( struct  Lisp2DStatisticsSection const * const s , long  double  const avev ) {
  size_t  const  n = Lisp2DStatisticsSectionCount  ( s ) ;
  if  ( n > 1 )
    return sqrtl  (
      ( StatisticsSectionPointerSumX2 ( s ) - ( StatisticsSectionPointerSumX  ( s ) * avev ) ) / ( n - 1 ) ) ;
  return 0 ;
}
    
void Lisp2DStatisticsConstructor  ( struct  Lisp2DStatistics * const me ) {
  Lisp2DStatisticsSectionGlobalConstructor  ( & ( me  ->  globalStat  ) ) ;
  me  ->  sizeset = 0 ;
  me  ->  sectionsStat = NULL ;
  me  ->  lderfp = NULL ;
  Lisp2D_atomic_mutex_init(& ( me  ->  amutex ));
# ifdef LISP2D_DEBUG  
  me ->  flagConstructed = Lisp2DStatisticsLive  ;
# endif  
}

void  StatisticsPush  ( struct  Lisp2DStatistics * const me , long  double  const xv ) {
# ifdef LISP2D_DEBUG  
  switch  ( me ->  flagConstructed )  {
  case Lisp2DStatisticsLive : 
    break ;
  case Lisp2DStatisticsNotConstructed :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DStatisticsPush:NotConstructed" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  case Lisp2DStatisticsDestructed :
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DStatisticsPush:Destructed" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  default : 
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])& L"Lisp2DStatisticsPush:dead memory" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  long double const x2 = xv * xv ;
  Lisp2D_atomic_mutex_lock  ( & ( me  ->  amutex )  ) ;
  StatisticsSectionGlobalPush  ( &  ( me ->  globalStat ) , xv , x2 ) ;
  // 0: (-inf)..(average+(lderf[0]*deviation))
  // 1: (average+(lderf[0]*deviation))..(average+(lderf[1]*deviation))
  // ...
  // n-2: (average+(lderf[n-3]*deviation))..(average+(lderf[n-2]*deviation))
  // n-1: (average+(lderf[n-2]*deviation))..(+inf)
  if  ( ( me  ->  sizeset ) > 1 ) {
    if  ( Lisp2DStatisticsCount ( me ) > 1 ) {
      bool  flagfound = false ;
      for ( size_t  i = 0 ; i < ( ( me  ->  sizeset ) - 1 ) ; ++  i )
        if  ( xv <=  ( StatisticsPointerAverage ( me ) +
            ( ( *(me -> lderfp) ) [ i ] * StatisticsPointerDeviation  ( me ) ) ) ) {
          StatisticsSectionPush2  ( ( me  ->  sectionsStat  ) + i , xv , x2 ) ;
          flagfound = true  ;
          break ; 
        }
      if  ( not flagfound )
        StatisticsSectionPush2  ( ( me  ->  sectionsStat  ) + (  me  ->  sizeset ) - 1 , xv , x2 ) ;
    } else
      StatisticsSectionPush2  ( ( me  ->  sectionsStat  ) + ( (  me  ->  sizeset ) >> 1 ) , xv , x2 ) ;
  } else
    //  sizeset is 0 or 1
    if  ( ( me  ->  sizeset ) ==  1 ) 
      StatisticsSectionPush2  ( me  ->  sectionsStat , xv , x2 ) ;
  Lisp2D_atomic_mutex_unlock  ( & ( me  ->  amutex )  ) ;
}

void  Lisp2DStatisticsSectionGlobalConstructor  ( struct  Lisp2DStatisticsSectionGlobal * const me  ) {
  Lisp2DStatisticsSectionConstructor ( & ( me -> s ) ) ;
  me -> average = me -> deviation = 0 ; 
}
  
void  Lisp2DStatisticsSectionConstructor  ( struct  Lisp2DStatisticsSection * const me  ) {
  me -> count = 0 ;
  me -> sumx = me -> sumx2 = 0 ; 
}
# else
_Bool STATISTICSCsomething = 0 ;
# endif

# ifdef  LISP2D_BIMSTAT

  struct Lisp2DStatistics  GCstat  ;
  
  struct Lisp2DStatistics  GCstatSymbols;
    
  struct Lisp2DStatistics  statComplexOperatorPlusEq ;
  struct Lisp2DStatistics  statComplexOperatorMulEq ;
  struct Lisp2DStatistics  statComplexOperatorDivEq ;
  struct Lisp2DStatistics  statComplexDelete ;
  
  struct Lisp2DStatistics  statULongDivWithOstBig  ;
  struct Lisp2DStatistics  statULongDivWithOstBigFast  ;
  
  struct Lisp2DStatistics  statULongDivWithOstBig2 ;

struct Lisp2DStatistics  statCreateLinks2 ;
struct  Lisp2DStatistics  statULongSquare ;

void StatisticsDestructor ( struct  Lisp2DStatistics * const s ) {
# ifdef LISP2D_DEBUG
  switch  ( s ->  flagConstructed ) {
  case  Lisp2DStatisticsLive :
    s ->  flagConstructed = Lisp2DStatisticsDestructed;
    break;
  case  Lisp2DStatisticsNotConstructed :
    fputws(L"StatisticsDestructor:NotConstructed\n",stderr);
    return;
  case  Lisp2DStatisticsDestructed  :
    fputws(L"StatisticsDestructor:Destructed\n",stderr);
    return;
  default :
    fputws(L"StatisticsDestructor:dead memory\n",stderr);
    return;
  }
# endif
  free ( s -> sectionsStat  ) ;
  free ( s -> lderfp ) ;
}

bool  StatisticsIsBADElt  ( struct  Lisp2DStatistics  const * const s , size_t const  i ,
  struct Lisp2DStatisticsSection * * const x ) {
  if  ( i < ( s ->  sizeset ) ) {
    ( * x ) = ( s ->  sectionsStat  ) + i ;
    return  false ;
  }
  return  true  ;
}

# include "castb.h"

# endif // LISP2D_BIMSTAT
