// Lisp2D Eval History inside StackCluster
// ©2012-23 by Alex Glebe
# include <stdlib.h>
# include <errno.h>
# include <stdbool.h>
# include <stddef.h>
# include <setjmp.h>
# include <iso646.h>
# include "define.h"
# ifdef LISP2D_DEBUG
# include <wchar.h> // fputws
# include <stdio.h> // stderr
# endif
# include "stclhistory.h"
# include "excc.h"
# include "plo/struct.h"

  // is empty: indexpush=0 isfull=false havelostdata=false
  // 1 element: indexpush=1 isfull=false havelostdata=false
  // ...
  // n-1 element: indexpush=n-1 isfull=false havelostdata=false
  // n element: indexpush=0 isfull=true havelostdata=false
  // n+1 element: indexpush=1 isfull=true havelostdata=true
  
void  Lisp2DStackClusterHistoryPush  ( struct  Lisp2DStackClusterHistory * const h ,
  struct Lisp2DStackClusterHistoryStep * const s ) {
# ifdef LISP2D_DEBUG
  switch  ( h -> state ) {
  case  Lisp2DStackClusterHistoryLive :
    break;
  case  Lisp2DStackClusterHistoryNotConstructed :
    fputws(L"Lisp2DStackClusterHistoryPush:NotConstructed\n",stderr);
    return;
  case  Lisp2DStackClusterHistoryDestructed  :
    fputws(L"Lisp2DStackClusterHistoryPush:Destructed\n",stderr);
    return;
  default :
    fputws(L"Lisp2DStackClusterHistoryPush:dead memory\n",stderr);
    return;
  }
# endif  
  if  ( h -> sizeMemory  )  {
    h ->  history  [ h  -> indexPush ] = ( * s ) ;
    ++  ( h -> indexPush  ) ;
    if  ( h ->  isFull  ) {
      if  ( h ->  indexPush  ==  (  h ->  sizeMemory  ) ) h ->  indexPush = 0 ;
      h ->  haveLostData  = true  ;
      // only once
    } else
      if  ( h ->  indexPush  ==  (  h ->  sizeMemory )  ) {
        h ->  indexPush = 0 ;
        h ->  isFull  = true  ;
      }
  }
}
        
void  StackClusterHistoryConstructor  ( struct  Lisp2DStackClusterHistory * const h , size_t  const memsize ) {
  h ->  indexPush = 0 ;
  h ->  haveLostData  = false ;
  h ->  sizeMemory = memsize ;
  h ->  isFull  = ( memsize  ==  0  ) ;
  if  ( memsize == 0  ) {
    h ->  history = 0 ;
    goto Ok ;
  }
  h ->  history = aligned_alloc ( _Alignof ( StackClusterHistoryStep [ memsize ] ) , sizeof  ( StackClusterHistoryStep [ memsize ] ) ) ;
  if  ( not ( h -> history ) ) {
# ifdef LISP2D_DEBUG
    h -> state = Lisp2DStackClusterHistoryNotConstructed ;
# endif      
    Lisp2DExcOutOfMemoryGlobal . value =  
      L"Lisp2DStackClusterHistoryConstructor:h ->  history = "
      L"aligned_alloc ( sizeof  ( struct  Lisp2DStackClusterHistoryStep ) * memsize )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
Ok :
# ifdef LISP2D_DEBUG
  h -> state = Lisp2DStackClusterHistoryLive
# endif  
  ;
}

void  Lisp2DStackClusterHistoryConstructorDefault ( struct  Lisp2DStackClusterHistory * const h ) {
  h ->  indexPush = 0 ;
  h ->  haveLostData  = false ;
  h ->  sizeMemory = 0 ;
  h ->  isFull  = true ;
  h ->  history = NULL ;
# ifdef LISP2D_DEBUG
  h -> state = Lisp2DStackClusterHistoryLive ;
# endif    
}
  
void  Lisp2DStackClusterHistoryDestructor ( struct  Lisp2DStackClusterHistory * const h ) {
# ifdef LISP2D_DEBUG
  switch  ( h -> state ) {
  case  Lisp2DStackClusterHistoryLive :
    h -> state = Lisp2DStackClusterHistoryDestructed;
    break;
  case  Lisp2DStackClusterHistoryNotConstructed :
    fputws(L"Lisp2DStackClusterHistoryDestructor:NotConstructed\n",stderr);
    return;
  case  Lisp2DStackClusterHistoryDestructed  :
    fputws(L"Lisp2DStackClusterHistoryDestructor:Destructed\n",stderr);
    return;
  default :
    fputws(L"Lisp2DStackClusterHistoryDestructor:dead memory\n",stderr);
    return;
  }
# endif  
  free ( h  ->  history ) ;
}
  
void  Lisp2DStackClusterHistoryClear ( struct  Lisp2DStackClusterHistory * const h ) {
# ifdef LISP2D_DEBUG
  switch  ( h -> state ) {
  case  Lisp2DStackClusterHistoryLive :
    break;
  case  Lisp2DStackClusterHistoryNotConstructed :
    fputws(L"Lisp2DStackClusterHistoryClear:NotConstructed\n",stderr);
    return;
  case  Lisp2DStackClusterHistoryDestructed  :
    fputws(L"Lisp2DStackClusterHistoryClear:Destructed\n",stderr);
    return;
  default :
    fputws(L"Lisp2DStackClusterHistoryClear:dead memory\n",stderr);
    return;
  }
# endif  
  free  ( h ->  history ) ;
  h ->  history = NULL  ;
}
  
// after Garbage
void  Lisp2DStackClusterHistoryInit_ExcStrMem ( struct  Lisp2DStackClusterHistory * const s ,
  size_t const newsize ) {
# ifdef LISP2D_DEBUG
  switch  ( s -> state ) {
  case  Lisp2DStackClusterHistoryLive :
    break;
  case  Lisp2DStackClusterHistoryNotConstructed :

    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DStackClusterHistoryInit:NotConstructed" ; 
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    
  case  Lisp2DStackClusterHistoryDestructed  : 
    
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DStackClusterHistoryInit:Destructed" ; 
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
    
  default :
    
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DStackClusterHistoryInit:dead memory" ; 
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeString ) ;
  }
# endif  
  s ->  indexPush = 0 ;
  s ->  isFull  = ( newsize  ==  0  ) ;
  s ->  haveLostData  = false ;
  if  ( newsize ) {
    errno = 0 ;
    struct Lisp2DStackClusterHistoryStep * const newmem = realloc ( s -> history , sizeof (
      struct Lisp2DStackClusterHistoryStep ) * newsize ) ;
    if  ( errno ) {
      Lisp2DExcOutOfMemoryGlobal . value =
        L"Lisp2DStackClusterHistoryInit:void * newmem = "
        L"realloc ( s -> history , sizeof (struct Lisp2DStackClusterHistoryStep ) * newsize ) ;" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    s -> history = newmem ;
  } else  {
    free  ( s -> history ) ;
    s -> history = NULL ;
  }
  s -> sizeMemory  = newsize ;
}
  
void  Lisp2DStackClusterHistoryGCDeclareContent ( struct  Lisp2DStackClusterHistory const * const h ) {
  struct  Lisp2DStackClusterHistoryStep const * is =
    (  h  ->  history + ( ( h ->  isFull  ) ? ( h ->  sizeMemory  ) : ( h ->  indexPush ) ) ) ;
  struct  Lisp2DStackClusterHistoryStep const * const h0 = ( h ->  history ) ;
  while ( is  not_eq  h0  ) {
    --  is  ;
    StackClusterHistoryDeclareForGC  ( is ) ;
  }
}
    
// returns error    
bool  Lisp2DStackClusterHistoryisBADResize ( struct  Lisp2DStackClusterHistory * const s  ,
  size_t const newsize ) {
  // new vectors to protect from wrong newsize
  if  ( newsize ) {
    errno = 0 ;
    struct Lisp2DStackClusterHistoryStep * newhistory =
      realloc  ( s -> history , newsize * sizeof  ( struct Lisp2DStackClusterHistoryStep ) ) ;
    if  ( errno )
      return true ;
    s -> history = newhistory ;
  } else {
    free ( s -> history ) ;
    s -> history = NULL ;
  }
  s ->  sizeMemory  = newsize ;
  s ->  indexPush = 0 ;
  s ->  isFull  = ( newsize  ==  0  ) ;
  s ->  haveLostData  = false ;
  // copying old data is NOT writened yet
  // finish work
  return  false ;
}

# include "plo/public.h"

void  StackClusterHistoryDeclareForGC ( struct  Lisp2DStackClusterHistoryStep const * const s ) {
  ParaLispObject_virtualDeclareForGC ( s ->  This ) ;
  { ParaLispObject * const m = s -> Meth ;
    if  ( m )
      ParaLispObject_virtualDeclareForGC ( m ) ;
  }
  { ParaLispObject * const a = s -> Arg ;
    if  ( a )
      ParaLispObject_virtualDeclareForGC ( a ) ;
  }
# ifdef LISP2D_DEBUGENV    
  ParaLispObject_virtualDeclareForGC ( s -> Env ) ;
  ParaLispObject_virtualDeclareForGC ( s -> EnvExt) ;
# endif    
}  

# include "stclhistoryp.h"
# include "stcl/pushhistory.h"
# include "stcl/public.h"
# include "stcl/inline.h"

void  PushHistory ( StackClusterPrivate  * const stclpp  , ParaLispObject  * const xp ) {
  { struct Lisp2DStackClusterHistoryStep  st = {  xp , NULL , NULL
# ifdef  LISP2D_DEBUGENV
      , StackClusterPrivate_Env ( stclpp )
      , StackClusterPrivate_ExtEnv ( stclpp )
# endif
      } ;
    Lisp2DStackClusterHistoryPush  ( StackClusterPrivate_myHistory  ( stclpp ) , & st ) ;
  }
}
# include "stclhistory/inline.h"
