// Lisp2D - String Unix
// ©2006-23 by Alex Glebe
# include <stddef.h>
# include <stdlib.h>
# include <unistd.h>
# include "string-unix.h"

# define  DeleteCharInd Lisp2DDeleteCharIndnSyC
static  void  DeleteCharInd ( char  * * const aa  , size_t const ind ) {
  { char  * * aai = aa  ;
    for ( size_t  j2  = 0 ; j2  < ind ; ++  j2  , ++  aai )
      free ( * aai ) ; 
  }
  free ( aa  ) ; 
}

# include "plocomplex/public.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "cons/inline.h"
# include "complex/inline.h"
# include "string/public.h"
# include "stringbim.h"
# include "stcl/public.h"
# include "stcl/inline.h"
# include "stack/public.h"

//  string system [{string}]
TaskForThread bimstringsystem ( StackClusterPrivate   * const stclpp  ) {
  ParaLispObject  * i2  ;
  PLOComplex  * s ;
  { EvalAddrCont2 const  c = StackClusterPrivate_ClearflagCont  ( stclpp ) ;
    if  ( c ) {
      switch  ( c ) {
      case  1 :
        goto  Exit  ;
      case  2 :
        { ParaLispObject * tmp  ;
          StackClusterPrivate_Pop ( stclpp ,&tmp ) ;
          s = ParaLispObjectToPLOComplex ( tmp ) ;
        }
        StackClusterPrivate_Pop (stclpp ,& i2  ) ;
        if  ( StackClusterPrivate_exceptionnotprocessed  ( stclpp ) )
          goto  Exit  ;
        goto  Next  ;
      default : ;
      }
    } // if c
  } // c
  s  = NewComplexBareExc (  ) ;
  Complex_Set0  ( PLOComplexToComplex ( s ) ) ;  
  i2  = StackClusterPrivate_ArgTop ( stclpp ) ;
Next  :
  { unsigned  int tasks2  = maxListItemProcessing ;
    for ( ; ( tasks2  > 0 ) and CConsp (  i2  ) ;
      i2  = ClassCons_Rest(&ParaLispObjectToPLOCons  ( i2  )->cc ) , -- tasks2 )
        FractionAdd1  (Complex_Re  (PLOComplexToComplex(s) )); 
  } // tasks2
  if  ( CConsp (  i2  ) ) {
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont  ( stclpp , 2U  ) ; 
    StackClusterPrivate_Push (stclpp , i2  ) ;
    StackClusterPrivate_Push (stclpp ,&  s->plo ) ;
    goto  Exit  ; 
  }
  char  * *  aa ;
  { size_t  ind ;
    if( FractionIsBadSizeValue ( Complex_CRe  ( PLOComplexToComplexConst(s) ) , &ind ) )
      goto Exit ;
    aa  = aligned_alloc ( _Alignof ( char * ) , sizeof  ( char * ) * ( ind + 2 ) ) ;
    if  ( not aa ) {
      OutOfMemory (  stclpp ) ;
      goto  Exit  ;
    }
  } // ind
  // aa[0]=this
  // aa[1]= first arg
  // aa[s]= last arg
  // aa[s+1]=0
  
  { bool  flagexit  = false ;
    { ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;  
      String  ( Char  ) * const strtmpc = StackNew_StringChar( ) ;
      String_Init ( Char  ) ( strtmpc , true ) ;
      if ( isBADReadWCharToCharString  ( PLOString_CStr ( ParaLispObjectToPLOString (
              StackClusterPrivate_ThisTop( stclpp ) ) ) , strtmpc ) ) {
        BadString ( stclpp , ParaLispObjectToPLOString (
          StackClusterPrivate_ThisTop( stclpp ) ) ) ;
        flagexit  = true  ;
      } else {
        char  * const dop =
          aligned_alloc ( _Alignof ( char ) , sizeof  ( char ) * ( String_Size ( Char  ) ( strtmpc ) + 1U ) ) ;
        if  ( dop ) {
          strcpy  ( dop , StringCStr  ( Char  ) ( strtmpc ) ) ;
          aa  [ 0 ] = dop ; 
        } else  {
          OutOfMemory (  stclpp ) ;
          flagexit  = true  ;
        }  
      }
      Lisp2D_StackFree  ( oldstack_a ) ;
    } // strtmp
    if  ( flagexit  ) {
      free ( aa  ) ;
      goto  Exit  ; 
    } 
  } // flagexit
  { size_t  indlast ;
    if( FractionIsBadSizeValue ( Complex_CRe  (
      PLOComplexToComplexConst(s) ) , &indlast )) 
      goto Exit ;
    { ParaLispObject  * j = StackClusterPrivate_ArgTop ( stclpp ) ;
      char  * * aaind = & ( aa  [ 1 ] ) ;
      for ( size_t  ind = 1U ; ind <=  indlast ; ++  ind , ++  aaind ) {
        ParaLispObject  * const ip = CCAR (  j ) ;
// !! to write: obj->MakeStringValue->OldCharValue
        { bool  flagexit  = false ;
          if  ( IsNotString ( stclpp , ip ) )
            flagexit  = true  ;
          else {
            ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;  
            String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
            String_Init ( Char  ) ( strtmpc , true ) ;
            if ( isBADReadWCharToCharString ( PLOString_CStr (
                  ParaLispObjectToPLOString ( ip ) ) , strtmpc ) ) {
              BadString ( stclpp , ParaLispObjectToPLOString ( ip ) ) ;
              flagexit  = true  ; 
            } else {
              char  * const dop =
                aligned_alloc ( _Alignof ( char ) , sizeof  ( char ) * ( String_Size ( Char  ) ( strtmpc ) + 1U ) ) ;
              if  ( dop ) {
                strcpy  ( dop , StringCStr  ( Char  ) ( strtmpc ) ) ;
                ( * aaind ) = dop ; 
              } else  {
                OutOfMemory (  stclpp ) ;
                flagexit  = true  ;
              } 
            }
            Lisp2D_StackFree  ( oldstack_a ) ;
          } // strtmp
          if  ( flagexit  ) {
            DeleteCharInd ( aa  , ind ) ;
            goto  Exit  ; 
          } 
        } // flagexit
        j = CCDR (  j ) ; 
      } 
    } // j
    aa  [ indlast + 1 ] = 0 ;
    int const ifork = fork  ( ) ;
    if  ( ifork ==  -1  ) {
      OutOfMemory (  stclpp ) ;
      DeleteCharInd ( aa  , indlast ) ;
      goto  Exit  ; 
    }
    if  ( ifork ) {
      DeleteCharInd ( aa  , indlast ) ;
      StackClusterPrivate_setTaskForManager ( stclpp , 
        TaskWaitSystemTask ) ;
      StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ; 
      PLOComplex * const cpid  =  NewComplexBareExc (  )  ;
      Complex_SetInt  ( PLOComplexToComplex( cpid) , ifork ) ;  
      StackClusterPrivate_Push (stclpp ,&  cpid->plo ) ;
      goto  Exit  ;
    }
  } // indlast
  execvp  ( aa  [ 0 ] , aa  ) ;
//fputws( L"exit(1)\n",stderr);
  // can't run the task
  //_exit(1); // ? always returns 1
              // system returns NIL (here) or INTEGER (after task finished)
  abort ( ) ;   // ? may be trapped by a signal handler
  // std::quick_exit // new C++ version
  //exit(1);  // !! my personal destructors called
Exit  :
  return  TaskForThread_Analyse ; 
}

# include "boolean.h"

# define  bimstringaccessflag Lisp2d_bimstringaccessflag
static  TaskForThread bimstringaccessflag  ( StackClusterPrivate   * const stclpp , int const flag  ) {
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  { ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
    String_Init ( Char  ) ( strtmpc , true ) ;
    if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) )
      BadString ( stclpp ,  mep ) ;
    else
      StackClusterPrivate_SetResult ( stclpp ,  access  ( StringCStr  ( Char  ) ( strtmpc ) , flag ) ?
        & plofalse  . plo  : &  mep ->  plo ) ;
    Lisp2D_StackFree  ( oldstack_a ) ;
  }
  return  TaskForThread_Analyse ; 
}

//  string accessf
TaskForThread bimstringaccessf  ( StackClusterPrivate   * const stclpp ) {
  return  bimstringaccessflag ( stclpp  , F_OK  ) ; }
  
//  string accessx
TaskForThread bimstringaccessx  ( StackClusterPrivate   * const stclpp ) {
  return  bimstringaccessflag ( stclpp  , X_OK  ) ; }

//  string accessr
TaskForThread bimstringaccessr  ( StackClusterPrivate   * const stclpp ) {
  return  bimstringaccessflag ( stclpp  , R_OK  ) ; }

//  string accessw
TaskForThread bimstringaccessw  ( StackClusterPrivate   * const stclpp ) {
  return  bimstringaccessflag ( stclpp  , W_OK  ) ; }

typedef int Lisp2D_funchown ( char const *  , uid_t , gid_t ) ;
typedef Lisp2D_funchown  funchown ;

# define  int_cast_uid  Lisp2D_int_cast_uid
inline  uid_t int_cast_uid ( int i ) ;

//  string setgid number
# define  bimstringsetgidfun  Lisp2D_bimstringsetgidfun
static  TaskForThread bimstringsetgidfun ( 
  StackClusterPrivate   * const stclpp , funchown  fun ) {
  ParaLispObject  * const xp  = CCAR ( StackClusterPrivate_ArgTop ( stclpp )  ) ;
  if  ( IsNotNumber ( stclpp ,  xp  ) )
    return  TaskForThread_Analyse ;
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp ,  mep ) ;
    return  TaskForThread_Analyse ; 
  }
  int const res = fun ( StringCStr  ( Char  ) ( strtmpc )  , int_cast_uid ( -1  ) ,
    FractionUIValue  ( Complex_CRe ( PLOComplexToComplexConst  (
          ParaLispObjectToPLOComplexConst ( xp  ) ) ) ) ) ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( res ? 
    & plonil . plo :
    & mep ->  plo ) ) ;
  return  TaskForThread_Analyse ; 
}

//  string setgid number
TaskForThread bimstringsetgid ( StackClusterPrivate   * const stclpp ) {
  return  bimstringsetgidfun  ( stclpp , chown  ) ; 
}

//  string setlgid number
TaskForThread bimstringsetlgid (StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetgidfun  ( stclpp , lchown ) ; 
}
  
# define  int_cast_gid  Lisp2D_int_cast_gid
inline  gid_t int_cast_gid ( int i ) ;
  
//  string setuid number
# define  bimstringsetuidfun  Lisp2D_bimstringsetuidfun
static  TaskForThread bimstringsetuidfun (
  StackClusterPrivate   * const stclpp , funchown * const funp ) {
  ParaLispObject  * const xp  = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) ) ;
  if  ( IsNotNumber ( stclpp ,  xp  ) )
    return  TaskForThread_Analyse ;
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp ,  mep ) ;
    return  TaskForThread_Analyse ; 
  }
  int const res = ( * funp ) ( StringCStr  ( Char  ) ( strtmpc ) ,
    FractionUIValue  ( Complex_CRe ( PLOComplexToComplexConst  (
          ParaLispObjectToPLOComplexConst ( xp  ) ) ) ) ,
    int_cast_gid ( -1  ) ) ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  StackClusterPrivate_SetResult ( stclpp ,  ( res ?
    &  plonil . plo :
    & mep ->  plo ) ) ;
  return  TaskForThread_Analyse ; 
}

//  string setuid number
TaskForThread bimstringsetuid ( StackClusterPrivate   * const stclpp ) {
  return  bimstringsetuidfun  ( stclpp , chown  ) ; 
}

//  string setluid number
TaskForThread bimstringsetluid (StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetuidfun  ( stclpp , lchown ) ; 
}

typedef int Lisp2D_funchdir ( char const * ) ;
typedef Lisp2D_funchdir  funchdir ;

# define  bimstringchdirfun Lisp2D_bimstringchdirfun
static  TaskForThread bimstringchdirfun (  StackClusterPrivate   * const stclpp ,
  funchdir  fun ) {
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( ParaLispObjectToPLOString (
    StackClusterPrivate_ThisTop ( stclpp )  ) ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp ,  ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop (
      stclpp )  ) ) ;
    return  TaskForThread_Analyse ;
  }
  { int const  e = fun ( StringCStr  ( Char  ) ( strtmpc ) ) ;
    Lisp2D_StackFree  ( oldstack_a ) ;
    if  ( e )
      return   SetResultErrnoNotLockGC  ( stclpp )  ;
    StackClusterPrivate_SetResult (stclpp , & plonil  . plo  ) ;
  } // int e
  return  TaskForThread_Analyse ;
}

//  string chdir
TaskForThread bimstringchdir ( StackClusterPrivate   * const stclpp ) {
  return  bimstringchdirfun  ( stclpp , chdir  ) ;
}

//  string rmdir
TaskForThread bimstringrmdir (StackClusterPrivate   * const stclpp  ) {
  return  bimstringchdirfun  ( stclpp , rmdir ) ; 
}

# include <errno.h>
# include "passage/public.h"

//  string unlink
TaskForThread bimstringunlink ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp  ) ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    PassagePassIn ( passageGCCPub ) ;
    BadString ( stclpp ,  mep ) ;
    PassagePassOut ( passageGCCPub ) ; 
    return  TaskForThread_Analyse ; 
  }
  int const  e = unlink ( StringCStr  ( Char  ) ( strtmpc ) ) ;
  int er  ;
  if ( e )
    er  = errno ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  PassagePassIn ( passageGCCPub )  ;
  StackClusterPrivate_SetResult ( stclpp  , ( e ?
    & SymbolFromIntErrno  ( er  ) -> plo :
    & plonil  . plo ) ) ;
  PassagePassOut ( passageGCCPub ) ;
  return  TaskForThread_Analyse ; 
}

typedef int Lisp2D_funlink ( char const * , char const * ) ;
typedef Lisp2D_funlink  funlink ;

# define  bimstringlinkfun  Lisp2D_bimstringlinkfun
static  TaskForThread bimstringlinkfun ( StackClusterPrivate * const stclpp ,
  funlink fun ) {
  ParaLispObject  * const newpathp  = CCAR ( StackClusterPrivate_ArgTop ( stclpp )  ) ;
  if  ( IsNotString ( stclpp , newpathp ) )  
    return  TaskForThread_Analyse ;
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp , mep  ) ;
    return  TaskForThread_Analyse ;
  }
  String  ( Char  ) * const strtmp2c = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmp2c , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( ParaLispObjectToPLOString (
          newpathp  ) ) , strtmp2c ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp ,  ParaLispObjectToPLOString ( newpathp  ) ) ;
    return  TaskForThread_Analyse ;
  }
  int const res = fun ( StringCStr  ( Char  ) ( strtmpc ) ,
    StringCStr  ( Char  ) ( strtmp2c ) )  ;
  int er  ;
  if ( res )
    er  = errno ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  StackClusterPrivate_SetResult ( stclpp ,
    ( res ? & SymbolFromIntErrno  ( er ) -> plo : ( & plonil  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

//  string link string
TaskForThread bimstringlink ( StackClusterPrivate   * const stclpp ) {
  return  bimstringlinkfun  ( stclpp , link  ) ;
}

//  string symlink string
TaskForThread bimstringsymlink (StackClusterPrivate   * const stclpp  ) {
  return  bimstringlinkfun  ( stclpp , symlink ) ;
}

# include <sys/statvfs.h>
  
# define  toQuitLoadStatVfs Lisp2DtoQuitLoadStatVfsbN87
static  bool  toQuitLoadStatVfs  ( StackClusterPrivate  * const stclpp , struct  statvfs * const stp ) {
  { ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
    String_Init ( Char  ) ( strtmpc , true ) ;
    PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) ) ;
    if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) )  {
      Lisp2D_StackFree  ( oldstack_a ) ;
      BadString ( stclpp , mep  ) ;
      return  true  ;
    }
    int const res = statvfs ( StringCStr  ( Char  ) ( strtmpc ) , stp ) ;
    int er  ;
    if ( res )
      er  = errno ;
    Lisp2D_StackFree  ( oldstack_a ) ;
    if  ( res ) {
      StackClusterPrivate_SetResult ( stclpp , & SymbolFromIntErrno  ( er ) -> plo ) ;
      return  true  ;
    }
  }
  return  false ;
}

# define  toQuitLoadStatVfsMakeComplex Lisp2D_toQuitLoadStatVfsMakeComplex
static  bool  toQuitLoadStatVfsMakeComplex  ( StackClusterPrivate  * const st ,
  struct  statvfs * const stp , PLOComplex * * const cres ) {
  if ( not  toQuitLoadStatVfs  ( st , stp ) ) {
    * cres = NewComplexBareExc ( ) ;
    StackClusterPrivate_SetResult ( st ,  & ( ( * cres ) ->  plo ) ) ;
    return  false ;
  }
  return  true  ;
}
  
// string vfsblocksize
TaskForThread bimstringvfsblocksize ( StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_bsize ) ;
  return  TaskForThread_Analyse ;
}

// string frsize
TaskForThread bimstringfrsize ( StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_frsize  ) ;
  return  TaskForThread_Analyse ;
}
  
// string vfsblocks
TaskForThread bimstringvfsblocks (  StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_blocks  ) ;
  return  TaskForThread_Analyse ;
}

// string bfree
TaskForThread bimstringbfree (  StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_bfree ) ;
  return  TaskForThread_Analyse ;
}

// string bavail
TaskForThread bimstringbavail ( StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_bavail  ) ;
  return  TaskForThread_Analyse ;
}

// string files
TaskForThread bimstringfiles (  StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_files ) ;
  return  TaskForThread_Analyse ;
}

// string ffree
TaskForThread bimstringffree (  StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_ffree ) ;
  return  TaskForThread_Analyse ;
}

// string favail
TaskForThread bimstringfavail ( StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_favail  ) ;
  return  TaskForThread_Analyse ;
}

// string fsid
TaskForThread bimstringfsid ( StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_fsid  ) ;
  return  TaskForThread_Analyse ;
}

// string namemax
TaskForThread bimstringnamemax (  StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  PLOComplex  * pret  ;
  if  ( not toQuitLoadStatVfsMakeComplex  ( stclpp  , & st , & pret ) )
    Complex_SetULInt  ( PLOComplexToComplex ( pret  ) , st  . f_namemax ) ;
  return  TaskForThread_Analyse ;
}

// string rdonly
TaskForThread bimstringrdonly ( StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  if  ( toQuitLoadStatVfs  ( stclpp , & st ) ) 
    return  TaskForThread_Analyse ;
  StackClusterPrivate_SetResult ( stclpp , ( ( st  . f_flag bitand ST_RDONLY ) ?
    ( & plotrue . plo ) : ( & plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

// string nosuid
TaskForThread bimstringnosuid ( StackClusterPrivate   * const stclpp  ) {
  struct  statvfs st  ;
  if  ( toQuitLoadStatVfs  ( stclpp , & st ) )
    return  TaskForThread_Analyse ;
  StackClusterPrivate_SetResult ( stclpp ,  ( ( st  . f_flag bitand ST_NOSUID ) ?
    ( & plotrue . plo ) : ( & plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

# include <sys/stat.h>  // mkdir

# define  bimstringsetusrmode Lisp2D_bimstringsetusrmode
static  TaskForThread bimstringsetusrmode (
  StackClusterPrivate * const stclpp  , mode_t  const mod ) {
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp  ) ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp  , mep ) ;
    return  TaskForThread_Analyse ;
  }
  struct  stat  buf ;
  { int const sres = stat  ( StringCStr  ( Char  ) ( strtmpc ) , & buf ) ;
    int er  ;
    if ( sres )
      er  = errno ;
    if  ( sres ) {
      Lisp2D_StackFree  ( oldstack_a ) ;
      StackClusterPrivate_SetResult ( stclpp , & SymbolFromIntErrno  ( er ) -> plo ) ;
      return  TaskForThread_Analyse ;
    }
  }
  { ParaLispObject  * const  xp = CCAR ( StackClusterPrivate_ArgTop ( stclpp )  ) ;
    if  ( ( xp  ==  ( & plonil  . plo ) )  or  ( xp  ==  (  &  plofalse . plo ) ) )
      // ! to optimize : if already NO go out
      buf . st_mode and_eq  ( mode_t  ) ( compl ( mod ) ) ;
    else  
      // ! to optimize : if already YES go out
      buf . st_mode or_eq ( mod ) ;
  }
  { int const cres  = chmod ( StringCStr  ( Char  ) ( strtmpc ) , buf . st_mode ) ;
    int er  ;
    if  ( cres  )
      er  = errno ;
    Lisp2D_StackFree  ( oldstack_a ) ;
    StackClusterPrivate_SetResult ( stclpp ,
      ( cres ? ( & SymbolFromIntErrno  ( er ) -> plo ) : ( & mep ->  plo ) ) ) ;
  }
  return  TaskForThread_Analyse ;
}

// string setusrreadp
TaskForThread bimstringsetusrreadp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IRUSR ) ;
}

// string setusrwritep
TaskForThread bimstringsetusrwritep (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IWUSR ) ;
}

// string  setusrexecp
TaskForThread bimstringsetusrexecp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IXUSR ) ;
}
  
// string setgrpreadp
TaskForThread bimstringsetgrpreadp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IRGRP ) ;
}

// string setgrpwritep
TaskForThread bimstringsetgrpwritep ( StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IWGRP ) ;
}

// string setgrpexecp
TaskForThread bimstringsetgrpexecp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IXGRP ) ;
}
  
// string setothreadp
TaskForThread bimstringsetothreadp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IROTH ) ;
}

// string setothwritep
TaskForThread bimstringsetothwritep ( StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IWOTH ) ;
}
  
// string setothexecp
TaskForThread bimstringsetothexecp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_IXOTH ) ;
}

// string setuidp
TaskForThread bimstringsetuidp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_ISUID ) ;
}

// string setgidp
TaskForThread bimstringsetgidp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_ISGID ) ;
}
  
// string setvtxp
TaskForThread bimstringsetvtxp (  StackClusterPrivate   * const stclpp  ) {
  return  bimstringsetusrmode  ( stclpp  , S_ISVTX ) ;
}

# include <utime.h>
# include "time/public.h"
  
// string settime time-access time-modify
TaskForThread bimstringsettime (StackClusterPrivate   * const stclpp){
  PLOStringPub * const  mep = ParaLispObjectToPLOString(StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  * const lp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const xp= CCAR  ( lp);
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString(stclpp , mep);
    return  TaskForThread_Analyse ;
  }
  if  ( Timep ( xp ) ) {
    ParaLispObject  * const yp= CCAR  ( CCDR  ( lp));
    struct  utimbuf time2 ;
    if  ( Timep ( yp ) )
      time2 = (struct  utimbuf){PLOTime_CValue(ParaLispObjectToPLOTime(xp)).tv_sec,
        PLOTime_CValue(ParaLispObjectToPLOTime(yp)).tv_sec};
    else
      time2=(struct  utimbuf){PLOTime_CValue(ParaLispObjectToPLOTime(xp)).tv_sec,
        PLOTime_CValue(ParaLispObjectToPLOTime(xp)).tv_sec};
    int const cres  = utime ( StringCStr  ( Char  ) ( strtmpc ) , & time2 ) ;
    int er  ;
    if  ( cres  )
      er  = errno ;
    Lisp2D_StackFree  ( oldstack_a ) ;
    StackClusterPrivate_SetResult ( stclpp ,  ( cres ?
        &  ( SymbolFromIntErrno  ( er ) -> plo) :
        (&plonil.plo ) ) ) ;
    return  TaskForThread_Analyse ;
  }
  int const cres  = utime(StringCStr  ( Char  ) ( strtmpc ), (const struct utimbuf *)0) ;
  int er  ;
  if  ( cres  )
    er  = errno ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  StackClusterPrivate_SetResult(stclpp ,( cres ? &(
      SymbolFromIntErrno  ( er ) -> plo) :
      (&plonil.plo)));
  return  TaskForThread_Analyse ;
}

TaskForThread bimstringchroot (StackClusterPrivate   * const stclpp){
  PLOStringPub * const  mep = ParaLispObjectToPLOString( StackClusterPrivate_ThisTop(stclpp ));
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
    BadString(stclpp ,mep);
    return  TaskForThread_Analyse ; }
  int const cres  = chroot ( StringCStr  ( Char  ) ( strtmpc ) ) ;
  int er  ;
  if  ( cres  )
    er  = errno ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  StackClusterPrivate_SetResult(stclpp ,(cres ?
      &( SymbolFromIntErrno  ( er ) -> plo)  :
      &  plonil . plo));
  return  TaskForThread_Analyse ;
}

//  string readlink
TaskForThread bimstringreadlink ( StackClusterPrivate   * const stclpp  ) {
  size_t  nmax  = 0x100UL ;
  char  * cres = aligned_alloc ( _Alignof ( char ) , sizeof  ( char ) * nmax ) ;
  if  ( not cres  ) {
    OutOfMemory ( stclpp )  ;
    return  TaskForThread_Analyse ;
  }
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  bool  flagcont  ;
  ssize_t n ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp , mep  ) ;
    return  TaskForThread_Analyse ;
  }
  do  {
    n = readlink  ( StringCStr  ( Char  ) ( strtmpc ) , cres  , nmax  ) ;
    if  ( n  < 0  ) {
      int const err = errno ;
      Lisp2D_StackFree  ( oldstack_a ) ;
      free  ( cres  ) ;
      StackClusterPrivate_SetResult ( stclpp , & SymbolFromIntErrno  ( err ) -> plo ) ;
      return  TaskForThread_Analyse ;
    }
    flagcont  = false ;
    if  ( ( ( size_t  ) n ) >=  nmax  ) {
      free  ( cres  ) ;
      size_t  const nmaxold = nmax  ;
      nmax  <<= 1 ;
      if  ( nmax < nmaxold  ) {
        Lisp2D_StackFree  ( oldstack_a ) ;
        ParaLispObject  * list [ ] = {  & PLOString_NewCopy ( L"nmax < nmaxold" ) ->
          plo , NULL } ;
        StackClusterPrivate_ExceptionSet  ( stclpp ,
          ListOfPLO ( ( PArrayPLOP ) { ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;
        return  TaskForThread_Analyse ;
      }
      cres = aligned_alloc ( _Alignof ( char ) , sizeof  ( char ) * nmax ) ;
      if  ( not cres  ) {
        Lisp2D_StackFree  ( oldstack_a ) ;
        OutOfMemory ( stclpp )  ;
        return  TaskForThread_Analyse ;
      }
      flagcont  = true  ;
    }
  } while ( flagcont  ) ;
  cres  [ n ] = '\0'  ;
  String  ( Char  ) * const strcresc = StackNew_StringChar ( ) ;
  StringConstrCStr ( Char  ) ( strcresc , cres , true ) ;
  free  ( cres  ) ;
  String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ;
  String_Init ( WideChar  ) ( wanc , true ) ;
  if  ( isBADReadCStringToWString2  ( strcresc , wanc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadStringN  ( stclpp )  ;
    return  TaskForThread_Analyse ;
  }
  PLOStringPub * const  res20p = PLOString_NewMoveString ( wanc ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & res20p -> plo  ) ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  return  TaskForThread_Analyse ;
}

# include <dirent.h>  // DIR
  
//  string ls
TaskForThread bimstringls ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString ( stclpp ,  mep ) ;
    return  TaskForThread_Analyse ;
  }
  DIR * const dir = opendir ( StringCStr  ( Char  ) ( strtmpc ) ) ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  if  ( not dir  ) {
    int const res = errno ;
    StackClusterPrivate_SetResult ( stclpp , & SymbolFromIntErrno  ( res ) -> plo ) ;
    return  TaskForThread_Analyse ;
  }
  ParaLispObject  *   res = & plonil  . plo ;
  ParaLispObject  *   resback = & plonil  . plo ;
  struct  dirent  * entry ;
  ThreadStackPointerType const oldstack_b = Lisp2DThreadSP ( ) ;
  String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ;
  String_Init ( WideChar  ) ( wanc , true ) ;
  String  ( Char  ) * const strtmp2c = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmp2c , true ) ;
  while ( ( entry  = readdir  ( dir ) ) ) {
    StringSetArray ( Char  ) ( strtmp2c , entry -> d_name ) ;
    if  ( isBADReadCStringToWString2  ( strtmp2c , wanc ) ) {
      Lisp2D_StackFree  ( oldstack_b ) ;
      BadStringN(stclpp );
      closedir(dir);
      return  TaskForThread_Analyse ;
    }
    PLOStringPub * const  ress3 = PLOString_NewMoveString ( wanc ) ;
    PLOCons * const rest2 = NewCons( & ress3->plo,&plonil.plo);
    if(res  ==  (&  plonil.plo))  //  Null
      res = (&rest2->plo);
    else   
      ClassCons_SetRest(&ParaLispObjectToPLOCons(resback)->cc,&  rest2->plo);
    resback = (&rest2->plo ) ;
  } // while
  Lisp2D_StackFree  ( oldstack_b ) ;
  closedir(dir);
  StackClusterPrivate_SetResult(stclpp ,  res);
  return  TaskForThread_Analyse ; 
}

# define  bimstringmkdir2c  Lisp2Dbimstringmkdir2cmp5m
static  void  bimstringmkdir2c ( StackClusterPrivate * const stclpp  ,
  ParaLispObject  * const kp  ) {
  PassagePassIn ( passageGCCPub ) ;
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile ex = setjmp  ( jump_buffer ) ;
  if  ( not ex ) {
    ParaLispObject  * list [ ] = { & PLOString_NewCopy (
        L"string::mkdir:unknown flag " )->plo , kp , NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
  } else
    OutOfMemory ( stclpp ) ;
  Lisp2DJBglobal = oldjb  ;
  PassagePassOut ( passageGCCPub ) ; 
}

# include "consiter/public.h"
# include "consiter/struct.h"
# include "plostream/public.h"

// string mkdir mode mode ... mode
TaskForThread bimstringmkdir (StackClusterPrivate   * const stclpp){
  PLOStringPub * const  mep = ParaLispObjectToPLOString(StackClusterPrivate_ThisTop(stclpp ));
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack_a ) ;
    BadString(stclpp , mep);
    return  TaskForThread_Analyse; 
  }
  mode_t  accessmode  = 0U ;
  { ThreadStackPointerType const oldstack_j = Lisp2DThreadSP ( ) ; 
    CConsIteratorPub * const j = NewStackCConsIterator ( ) ;
    CConsIteratorInit ( j ,
      StackClusterPrivate_ArgTop  ( stclpp  ) ) ;
    for ( ; CConsIteratorHaveItem  ( j ) ; CConsIteratorNext  ( j ) ) {
      ParaLispObject  * const kp = CConsIteratorItem  ( j ) ;
      size_t  const i = MapPubFind  ( FlessMkdirModes ) ( symboltomkdirmodes_pub , kp ) ;
      if  ( i  ==  MapPubSize ( FlessMkdirModes ) (
        symboltomkdirmodes_pub ) ) {
        Lisp2D_StackFree  ( oldstack_a ) ;
        bimstringmkdir2c ( stclpp , kp ) ;
        return   TaskForThread_Analyse  ; 
      }
      MapPubCAt ( FlessMkdirModes ) ( symboltomkdirmodes_pub , i ) -> value (
        & accessmode ) ;
    }
    Lisp2D_StackFree  ( oldstack_j ) ;
  } // jp
  if  ( accessmode ==  0  )
    accessmode  = ( S_IRWXU bitor S_IXGRP bitor S_IRGRP  bitor  S_IXOTH bitor
      S_IROTH /*0x1ff*/ ) ;
  int const er  = mkdir ( StringCStr  ( Char  ) ( strtmpc ) , accessmode  ) ;
  int err ;
  if ( er )
    err = errno ;
  Lisp2D_StackFree  ( oldstack_a ) ;
  if  ( er  ==  0  ) 
    StackClusterPrivate_SetResult ( stclpp , & plonil.plo  ) ;
  else  
    StackClusterPrivate_SetResult ( stclpp , & SymbolFromIntErrno  ( err ) -> plo ) ;
  return  TaskForThread_Analyse ;
}
# include "widestring/inline.h"
inline  uid_t int_cast_uid ( int const i ) {
  return  ( uid_t ) i ;
}
inline  gid_t int_cast_gid ( int const i ) {
  return  ( gid_t ) i ;
}
