// Lisp2D - String
// ©2006-23 by Alex Glebe
# include "stringbim.h"
# include "string/public.h"
# include "cons/inline.h"
# include "complex/inline.h"
# include "plocomplex/public.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "stcl/public.h"
# include "stcl/inline.h"

// string subseq [indexfirst [indexend]] => [first..end-1]
TaskForThread bimstringsubseq  ( StackClusterPrivate   * const stclpp  ) {
  { PLOStringPub * const mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
    size_t  ind1  ;
    { ParaLispObject  * const arglistp =  StackClusterPrivate_ArgTop  ( stclpp )  ;
      if  ( not  CConsp  ( arglistp ) ) {
        PLOStringPub  * const result = PLOString_NewCopy ( PLOString_CStr ( mep ) ) ;
        StackClusterPrivate_SetResult ( stclpp ,  & result  ->  plo ) ;
        return  TaskForThread_Analyse ;
      } // if not consp arglist
      { ParaLispObject  * const indp = ClassCons_First(&ParaLispObjectToPLOCons ( arglistp )->cc ) ;
        if  ( IsNotNumber ( stclpp , indp ) )
          return  TaskForThread_Analyse ;
        if  ( not  FractionIntegerP ( Complex_CRe  (  PLOComplexToComplexConst  (
                ParaLispObjectToPLOComplexConst ( indp ) ) ) ) ) {
          ParaLispObject  * list [ ] = {  indp  , & PLOString_NewCopy ( L" is not integer"  )->plo , NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          return  TaskForThread_Analyse ;
        }
        if  ( FractionLessUInt(Complex_CRe  (PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst ( indp )) ), 0U) ) {
          ParaLispObject  * list [ ] = {  indp  , & PLOString_NewCopy ( L" < 0" )->plo , NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          return  TaskForThread_Analyse ;
        }  // zero
        if  ( FractionIsBadSizeValue  ( Complex_CRe ( PLOComplexToComplexConst  (
          ParaLispObjectToPLOComplexConst ( indp  ) ) ) , & ind1  ) ) {
          ParaLispObject  * list [ ] = {  indp  , & PLOString_NewCopy ( L" is too big"  ) ->plo , NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          return  TaskForThread_Analyse ;
        }
        { ParaLispObject  * const arglist2p  =  ClassCons_Rest  ( & ParaLispObjectToPLOCons ( arglistp )->cc ) ;
          if  ( CConsp ( arglist2p  ) ) {
            ParaLispObject  * const indendp  =  ClassCons_First ( & ParaLispObjectToPLOCons ( arglist2p  )->cc ) ;
            if  ( IsNotNumber ( stclpp , indendp  ) )
              return  TaskForThread_Analyse ;
            if  ( not  FractionIntegerP ( Complex_CRe  (PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst ( indendp  )) ) ) ) {
              ParaLispObject  * list [ ] = {  indendp , &  PLOString_NewCopy  ( L" is not integer"  )->plo , NULL } ;
              StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){ (ParaLispObject*(*)[])&list}));
              return  TaskForThread_Analyse ;
            }
            if  ( FractionLess  ( Complex_CRe  (  PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst ( indendp  )) ) ,
              Complex_CRe  (PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst  ( indp )) ) )  ==  Lisp2DBoolTrue  ) {
              ParaLispObject  * list [ ] = {  indendp , &  PLOString_NewCopy  ( L" < "  )-> plo ,   indp  , NULL } ;
              StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
              return  TaskForThread_Analyse ;
            }
            size_t  ind2  ;
            if  ( FractionIsBadSizeValue  ( Complex_CRe  (PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst ( indendp  )) ) ,
              &ind2  ) ) {
              ParaLispObject  * list [ ] = {  indendp , &  PLOString_NewCopy  ( L" is too big"  )->plo , NULL } ;
              StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
              return  TaskForThread_Analyse ;
            }
            PLOStringPub * const res2p  = PLOString_NewStrBegEnd ( ( t_PLOString_NewStrBegEnd ) {
              . vp = mep , . indexfirst = ind1 , . indexend = ind2 } ) ;
            StackClusterPrivate_SetResult(stclpp , & res2p  -> plo ) ;
            return  TaskForThread_Analyse ; 
          }  // if consp arglist2
        } // arglist2
      } // ind
    } // arglist
    PLOStringPub * const  res3p  = PLOString_NewStrBeg ( mep , ind1 ) ;
    StackClusterPrivate_SetResult ( stclpp , &  res3p  -> plo ) ;
  } // ind1 me res3
  return  TaskForThread_Analyse ; 
}  

//  nil man
TaskForThread bimnilman ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  resp  = PLOString_NewCopy ( L"nil - nothing or empty list" ) ;
  StackClusterPrivate_SetResult ( stclpp , & resp -> plo ) ;
  return  TaskForThread_Analyse ;
}

//  nil string
TaskForThread bimnilstring ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  resp  = PLOString_NewCopy ( StringCStr  ( WideChar  ) ( PLOSymbolCValue  ( & plosnil  ) ) ) ;
  StackClusterPrivate_SetResult ( stclpp , & resp -> plo ) ;
  return  TaskForThread_Analyse ;
}

# include "boolean.h"
  
//  nil stringp
TaskForThread bimnilstringp  ( StackClusterPrivate   * const stclpp  ) {
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop(stclpp ))  ;
  StackClusterPrivate_SetResult(stclpp ,( Stringp ( xp ) ? xp :
    &plofalse.plo  ) ) ;
  return  TaskForThread_Analyse ;
}

# include "inerrors.h"
# include "passage/public.h"
  
bool  IsNotStringLockGC ( StackClusterPrivate  * const stclp , ParaLispObject * const xp ) {
  if  ( Stringp ( xp ) )
    return  false ;
  { static  wchar_t const wc [ ] = L" is not string" ;
    if  ( stclp  ) {
      PassagePassIn ( passageGCCPub ) ;
      PLOStringPub * const mess = PLOString_NewCopy ( wc ) ;
// ! to catch exception
      { ParaLispObject  * list [ ] = {  xp  , & mess  ->  plo , 0 } ;
        StackClusterPrivate_ExceptionSet  ( stclp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      }
      PassagePassOut ( passageGCCPub ) ; 
      // if stcl
    } else
      inErrorS (
        PLOStreamPushPLO(& Lisp2Dferr ,xp);
        PLOStreamPushWideCharArray  ( & Lisp2Dferr , (PArray_wchar_c ){ & wc } ) ; )
  } // wc
  return  true  ; 
}  

# include "newobject.h"
  
bool  IsNotString ( StackClusterPrivate  * const stclp  , ParaLispObject * const xp ) {
  tUniversalIsNotType ar  = {
    . stclp = stclp ,
    . xp  = xp ,
    . typetype  = TypeGlobalListMemberPLOString ,
    . res = false ,
  } ;
  UniversalIsNotTypess  ( & ar  ) ;
  return  ar  . res ;
  //return  UniversalIsNotType  ( stclp , xp ,  TypeGlobalListMemberPLOString ) ;
}

# include "char/public.h"
  
// string nsubstitute uint uint char|string
// string nsubstitute char char
TaskForThread bimstringnsubstitute  ( StackClusterPrivate  * const stclpp ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  ParaLispObject  * const lp =StackClusterPrivate_ArgTop(stclpp )  ;
  ParaLispObject  * const ip = CCAR ( lp )  ;
  StackClusterPrivate_SetResult(stclpp ,& plonil.plo  ) ;
  if  ( Numberp ( ip ) ) {
    if  ( not  FractionIntegerP (
        Complex_CRe  (PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst
        ( ip ))) ) ) {
      { ParaLispObject  * list [ ] = {    ip  , & PLOString_NewCopy (
          L" is not integer"  )->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      }
      return  TaskForThread_Analyse ; 
    } // if not integerp (i .re)
    if  ( FractionLessUInt(
        Complex_CRe (PLOComplexToComplexConst(ParaLispObjectToPLOComplex  ( ip
          )) ), 0U) ) {
      { ParaLispObject  * list [ ] = {    ip  , & PLOString_NewCopy ( L"<0" )
          ->plo ,  NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      }
      return  TaskForThread_Analyse ; 
    } // if i.re<0
    // i>=0
    ParaLispObject  * const restp  = CCDR ( lp )  ;
    ParaLispObject  * const np = CCAR ( restp  )  ;
    if  ( IsNotNumber ( stclpp , np ) )
      return  TaskForThread_Analyse ;
    if  ( not  FractionIntegerP (
        Complex_CRe  (PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst (
          np )) ) ) ) {
      { ParaLispObject  * list [ ] = {   np  , & PLOString_NewCopy (
          L" is not integer"  )->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); }
      return  TaskForThread_Analyse ; 
    }
    size_t    mys = String_Size(WideChar)(PLOString_CValue(mep)) ;
    if  ( FractionMoreUSize(
        Complex_CRe  (PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst ( ip ))  ),  mys )) {
      PLOComplex  * const  res2  =NewComplexBareExc  (  ) ;
        Complex_SetUSize ( PLOComplexToComplex( res2), mys ) ;
        { ParaLispObject  * list [ ] = {    ip  , & PLOString_NewCopy ( L">"  )
            ->plo , &res2->plo  , NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
        }
      return  TaskForThread_Analyse ; 
    } // if i.re > mys
    // i=[0..mys]
    size_t  ind ;
    if  ( ULongIsBADSizeValue ( FractionCNump  (
       Complex_CRe  (PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst (
         ip ) ) ) ) ,  & ind ) ) {
      { ParaLispObject  * list [ ] = {   ip  , & PLOString_NewCopy (
        L" is very big" )  ->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      }
      return  TaskForThread_Analyse ; 
    } // if not (i.re.chast isoksizeValue)
    size_t  s ;
    if  ( ULongIsBADSizeValue ( FractionCNump  (
       Complex_CRe  (PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst (
         np ) ) ) ) , & s ) ) {
      { ParaLispObject  * list [ ] = {   np  , & PLOString_NewCopy (
          L" is very big" ) ->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      }
      return  TaskForThread_Analyse ; 
    } // if not (n.re.chast isoksizeValue)
    if  ( FractionLessUInt ( Complex_CRe (
      PLOComplexToComplexConst(
      ParaLispObjectToPLOComplex  ( np )) ) ,  0U  ) ) {
      // n<0
      if  ( ind  < s  ) {
        PLOComplex  * const  res2  =NewComplexBareExc  ( ) ;
                   Complex_SetUSize ( PLOComplexToComplex( res2), ind ) ;
          PLOComplex  * const  res3  =NewComplexBareExc  (  ) ;
                       Complex_SetUSize ( PLOComplexToComplex( res3), s ) ;
          { ParaLispObject  * list [ ] = { &res2->plo  ,
              &  PLOString_NewCopy  ( L"<"  )->plo , &res3->plo  , NULL } ;
            StackClusterPrivate_ExceptionSet(stclpp ,
              ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
          }
            // if res2
        return  TaskForThread_Analyse ; 
      } // if ind < s
      ind -=  s ;
    } // if n.re<0
    // ind=[0..s]
    // s=[0..mys]
    if  ( ( ind + s )  > mys  )
      s = ( mys  - ind  ) ;
    // s=[0..mys-ind]
    ParaLispObject  * const objp = CCAR ( CCDR ( restp  ) ) ;
    if  ( Charp ( objp ) )
      StringReplaceChars ( WideChar )(PLOString_Value2 (mep ),ind , s , 1 ,
        PLOChar_Value(ParaLispObjectToPLOChar ( objp ) ));
    else {
      if  ( Stringp ( objp ) ) {
        struct  sPLOString_Replace a = { . me = mep , . index = ind , . length = s , . str = ParaLispObjectToPLOString ( objp ) } ;
        PLOString_Replace ( & a ) ;
        //PLOString_Replace ( mep  , ind , s , ParaLispObjectToPLOString ( objp ) ) ;
      } else  {
        { ParaLispObject  * list [ ] = {   objp  ,
            & PLOString_NewCopy ( L" is not char nor string"  )->plo , NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        }
        return  TaskForThread_Analyse ;
      } // else not stringp obj
    }
    StackClusterPrivate_SetResult(stclpp ,& mep->plo  ) ;
    return  TaskForThread_Analyse ;
  }  // if Numberp(i)
  if  ( Charp ( ip ) ) {
    ParaLispObject  * const jp = CCAR ( CCDR ( lp ) )  ;
    if  ( IsNotChar ( stclpp , jp ) )
      return  TaskForThread_Analyse ;
    { wchar_t const sci = PLOChar_Value(ParaLispObjectToPLOChar ( ip )  )  ;
      wchar_t const scj = PLOChar_Value(ParaLispObjectToPLOChar ( jp )  )  ;
      wchar_t volatile * kp  = String_Begin  ( WideChar ) ( PLOString_Value2 ( mep ) ) ;
      wchar_t volatile const * ke  = String_CEnd ( WideChar ) ( PLOString_Value2 ( mep ) ) ;
      while ( kp not_eq ke ) {
        if  ( ( * kp  )  ==  sci )
          ( * kp  ) = scj ;
        ++  kp  ;
      }
    } // sci , scj
    StackClusterPrivate_SetResult(stclpp ,& mep->plo  ) ;
    return  TaskForThread_Analyse ;
  }  // if charp i
  { ParaLispObject  * list [ ] = {   ip  , & PLOString_NewCopy (
      L" is not number nor char"  )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
  }
  return  TaskForThread_Analyse ; 
}

// string resize
TaskForThread bimstringresize  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp )  ;
  ParaLispObject  * const np = CCAR ( arglistp )  ;
  if  ( IsNotNumber ( stclpp , np ) )
    return  TaskForThread_Analyse ;
  if  ( FractionLessUInt(
      Complex_CRe (PLOComplexToComplexConst(ParaLispObjectToPLOComplex  ( np )) ),0U) ) {
    { ParaLispObject  * list [ ] = {    np  , & PLOString_NewCopy ( L"<0" )->
        plo , NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
        (ParaLispObject*(*)[])&list})); 
    }
    return  TaskForThread_Analyse ;
  } // if n.re < 0
  { size_t  us  ;
    if( FractionIsBadSizeValue ( Complex_CRe  (
      PLOComplexToComplexConst(
      ParaLispObjectToPLOComplexConst ( np )) ) , &us  ) )
      return  TaskForThread_Analyse;
    ParaLispObject  * const arg2p  = CCDR ( arglistp )  ;
    if  ( CConsp ( arg2p  ) ) {
      ParaLispObject  * const cp =
        ClassCons_First(&ParaLispObjectToPLOCons ( arg2p  )->cc ) ;
      if  ( IsNotChar ( stclpp , cp ) ) 
        StringResize ( WideChar )(PLOString_Value2 (mep ),us  , L' ');
      else 
        StringResize ( WideChar )(PLOString_Value2 (mep ),us  ,
          PLOChar_Value(ParaLispObjectToPLOChar ( cp ) )); 
    } // if consp arg2
    else
      StringResize ( WideChar )(PLOString_Value2 (mep ),us  , L' '); 
  } // us
  StackClusterPrivate_SetResult(stclpp ,& mep->plo  ) ;
  return  TaskForThread_Analyse ; 
}  

/*('string  defmethod string(x  c)
  (nil  cond
    ((nil numberp x)
      (this setf  (x  make-string c)))
    ((nil stringp x)
      (this setf  x)))
  this)*/
TaskForThread bimstringstring  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp )  ;
  ParaLispObject  * const xp = CCAR ( arglistp )  ;
  switch  (   ParaLispObject_Type  ( xp)  ) {
  case  TypeGlobalListMemberPLOComplex :
    if  ( FractionLessUInt( Complex_CRe (PLOComplexToComplexConst(ParaLispObjectToPLOComplex  ( xp )) ), 0U) ) {
      { ParaLispObject  * list [ ] = {
          &  PLOString_NewCopy  ( L" the size " )->plo ,
          xp , & PLOString_NewCopy ( L"<0" ) ->plo, NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));  
      }
      return  TaskForThread_Analyse ; 
    }
    { size_t  us  ;
      if  ( FractionIsBadSizeValue (
          Complex_CRe  (PLOComplexToComplexConst(
            ParaLispObjectToPLOComplexConst ( xp ) ) ) , &  us  ) )
        return  TaskForThread_Analyse ;
      ParaLispObject  * const cp = CCAR ( CCDR ( arglistp ) )  ;
      StringClear ( WideChar )  ( PLOString_Value2  ( mep ) ) ;
      if  ( Charp ( cp ) )
        StringResize ( WideChar )(PLOString_Value2 (mep ),us  ,
          PLOChar_Value(ParaLispObjectToPLOChar ( cp ) ));
      else     
        StringResize ( WideChar )(PLOString_Value2 (mep ),us  , L' '); 
    } // us
    break ;
  case  TypeGlobalListMemberPLOString  :
    { ParaLispObject * const  arg2p = CCDR ( arglistp )  ;
      ParaLispObject  * const indp = CCAR ( arg2p  )  ;
      if  ( Numberp ( indp ) ) {
        ParaLispObject  * const endp = CCAR ( CCDR ( arg2p  ) )  ;
        if  ( Numberp ( endp ) )
          PLOString_AssignSubSeq  ( mep ,
            ParaLispObjectToPLOString (   xp ) ,
            ParaLispObjectToPLOComplex ( indp ) ,
            ParaLispObjectToPLOComplex ( endp ) , stclpp ) ;
        else
          PLOString_AssignSubSeqEnd  ( mep ,
            ParaLispObjectToPLOString (   xp ) ,
            ParaLispObjectToPLOComplex ( indp ) ,
            stclpp ) ;

      } else
        // if  ( not Numberp ( ind ) )
        PLOString_Set(mep,ParaLispObjectToPLOString (   xp )) ;
    } // arg2 , ind
    break ;
    default : ;
  }  // switch x.type
  StackClusterPrivate_SetResult(stclpp ,& mep->plo  ) ;
  return  TaskForThread_Analyse ; 
}

# include "env/public.h"

// ! to make function "compile" -> returns list (nil progn ...)  
  
// ! load -> don't run ?   

// string load [{args}]
TaskForThread bimstringload  ( StackClusterPrivate   * const stclpp  ) {
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) )
    return  TaskForThread_Analyse ; 
  PassagePassIn ( passageGCCPub ) ;
  StackClusterPrivate  * const dopstclpp = StackClusterPrivate_NewStCl4  ( stclpp )  ;
  StackClusterPrivate_EnvNewLevel2(dopstclpp) ;
  StackClusterPrivate_SetExtEnv ( dopstclpp , StackClusterPrivate_Env ( dopstclpp ) ) ;
  // new program have own ExternalEnv
  ParaLispObject  * const lp = CCAR ( StackClusterPrivate_Env(dopstclpp ) )  ;
  if  ( IsNotEnvironmentLayer ( stclpp , lp ) ) 
    goto PassOutAndAnalyse ;
  { ParaLispObject  * const mep  = StackClusterPrivate_ThisTop( stclpp ) ;
    { jmp_buf jump_buffer ;
      jmp_buf * const oldjb = Lisp2DJBglobal  ;
      Lisp2DJBglobal = &  jump_buffer ;
      int const volatile ex = setjmp  ( jump_buffer ) ;
      if  ( ex ) {
        Lisp2DJBglobal = oldjb  ;
        goto  PassOutAndMemOut  ; 
      }
      PLOCons * const  nc  =   NewCons (  mep , StackClusterPrivate_ArgTop  ( stclpp ) ) ;
      if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
            lp ) -> envlay , & ploarg  . plo , & nc  ->  plo , true  ) ) {
        HasConstLink  ( stclpp , & ploarg . plo ) ;
        Lisp2DJBglobal  = oldjb  ;
        goto PassOutAndAnalyse ; 
      }
      Lisp2DJBglobal  = oldjb  ; 
    }
    StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSons ) ;
    // ! TaskWaitSonsAlmost - not works
    StackClusterPrivate_SetflagCont ( stclpp , 1U ) ;
     // father of pointer is StackClusterPrivate  
    { ParaLispObject * * const owp = StackClusterPrivate_resultp ( stclpp ) ;
       SetResultPlace (  dopstclpp ,  owp  , *owp ) ; 
    }
    ParaLispObject  * const pr2  =  StackClusterPrivate_LoadProgramS  (
      dopstclpp , PLOString_CStr (ParaLispObjectToPLOString  ( mep ) ) )  ;
    StackClusterPrivate_CreateTask (dopstclpp , pr2  ) ;
  }
  PassagePassOut ( passageGCCPub )  ;
  return  TaskForThread_Wait  ; 
PassOutAndMemOut :
  PassagePassOut ( passageGCCPub ) ;
  OutOfMemory (  stclpp ) ;
  return  TaskForThread_Analyse ;
PassOutAndAnalyse :
  PassagePassOut ( passageGCCPub ) ;
  return  TaskForThread_Analyse ; 
}

// string for* symbol [{body}]
TaskForThread bimstringforcons ( StackClusterPrivate   * const stclpp  ) {
  ParaLispObject  * varC  ;
  PLOComplex  * varA  ;
  PLOStringPub *  me ;
  ParaLispObject  * arglist ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )
    goto  Next  ;
  me  =   ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) )  ;
  arglist = StackClusterPrivate_ArgTop(stclpp ) ;
  if  ( IsNotSymbol ( stclpp ,  CCAR (   arglist  ) ) )
    return  TaskForThread_Analyse ;
  if  ( String_Empty  ( WideChar  ) ( PLOString_CValue ( me ) ) or (
      not  CConsp ( ClassCons_Rest ( &ParaLispObjectToPLOCons ( arglist )->cc )
      ) ) ) {
    StackClusterPrivate_SetResult(stclpp ,& plonil.plo  ) ;
    return  TaskForThread_Analyse ; 
  }
  varA  = NewComplexBareExc  (  ) ;
  Complex_Set0  ( PLOComplexToComplex( varA)) ;  
  StackClusterPrivate_EnvNewLevel2(stclpp ) ;
  for ( size_t  ind ;
    (not FractionIsBadSizeValue ( Complex_CRe  ( PLOComplexToComplexConst( varA) ) ,
    &ind )) and (ind < ( String_Size(WideChar)(PLOString_CValue ( me) ) )) ;
    FractionAdd1  (Complex_Re  (PLOComplexToComplex(varA) )) ) {
    if  ( IsNotEnvironmentLayer ( stclpp , CCAR ( StackClusterPrivate_Env( stclpp ) ) ) )
      return  TaskForThread_Analyse ; 
    { PLOCharPriv  * const newchar = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
      if ( not newchar )  {
        OutOfMemory ( stclpp ) ;
        break ; 
      }
      PLOChar_InitValue ( newchar , String_EltCopy  ( WideChar  ) ( PLOString_CValue (  me ) , ind ));
      if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
            CCAR ( StackClusterPrivate_Env(stclpp ) ) ) -> envlay , CCAR (
            arglist  ) , ParaLispObject_virtualPushAsNew (  & newchar -> pub . plo ) , false ) ) {
        HasConstLink  ( stclpp , CCAR (   arglist  ) ) ;
        return  TaskForThread_Analyse ; 
      }
    } // newchar
    StackClusterPrivate_Push (stclpp ,&  varA->plo  );
    varC  = CCDR  (  arglist )  ;
    for ( ; CConsp (  varC  ) ; 
         varC  = ClassCons_Rest ( &ParaLispObjectToPLOCons ( varC  )->cc ) ) {
      StackClusterPrivate_Push (stclpp ,  varC  ) ;
      StackClusterPrivate_Push (stclpp ,
        ClassCons_First(&ParaLispObjectToPLOCons ( varC  )->cc ) ) ; 
      StackClusterPrivate_SetflagCont ( stclpp , 1U ) ; 
      return  TaskForThread_Analyse ;
Next  :
      varC = StackClusterPrivate_PopValue  ( stclpp ) ;
      if  ( StackClusterPrivate_exceptionnotprocessed  ( stclpp ) ) {
        //varA
        StackClusterPrivate_Popn (  stclpp ) ;
        StackClusterPrivate_EnvCDR(stclpp );
        return  TaskForThread_Analyse ; 
      }
    } // for varC
    varA  = ParaLispObjectToPLOComplex ( StackClusterPrivate_PopValue ( stclpp ) ) ;
    me  =    ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) ) ;
    arglist = ( StackClusterPrivate_ArgTop(stclpp ) ) ; 
  } // for ind
  //old environment restoring
  StackClusterPrivate_EnvCDR(stclpp ) ;
  return  TaskForThread_Analyse ; 
}  

# include "bimc.h"
  
// string for i [{body}]
TaskForThread bimstringfor ( StackClusterPrivate   * const stclpp  ) {
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) ) {
    StackClusterPrivate_SetResult(stclpp ,& plonil.plo  ) ;
    return  TaskForThread_Analyse ;
  }
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  ParaLispObject  * arglist =StackClusterPrivate_ArgTop(stclpp ) ;
  ParaLispObject  * const varBp  =  CCAR (  arglist )  ;
  if  ( IsNotSymbol ( stclpp , varBp  ) )
    return  TaskForThread_Analyse ;
  arglist = ClassCons_Rest ( &ParaLispObjectToPLOCons ( arglist )->cc ) ;
  if  ( String_Empty  ( WideChar  ) ( PLOString_CValue (  mep ) ) or (  not CConsp (  arglist ) ) ) {
    StackClusterPrivate_SetResult(stclpp ,& plonil.plo  ) ;
    return  TaskForThread_Analyse ;
  }
  ParaLispObject  *  task3  ;
  if  ( not CConsp ( ClassCons_Rest ( &ParaLispObjectToPLOCons ( arglist )->cc ) ) )
    // if body has one task : "str" for i (...)  
    task3 = ClassCons_First(&ParaLispObjectToPLOCons ( arglist )->cc ) ;
  else {
    // if body has many tasks : "str" for i t1 .. tn
    PLOCons * const task2 =  NewCons  ( &    plobimnilprogn->base.plo  , arglist )  ;
    task3 =   &NewCons  (& plonil.plo  , & task2->plo )->plo  ;
  } // if  Consp(arglist->Rest)
  { size_t   j = String_Size(WideChar)(PLOString_CValue (mep ) ) ;
    while (   j  >  0   ) {
      --  j ;
      StackClusterPrivate  * const dopstclpp = StackClusterPrivate_NewStCl4  (stclpp ) ;
      StackClusterPrivate_EnvNewLevel2(dopstclpp) ;
      { ParaLispObject  * const newenvlayerp = CCAR ( StackClusterPrivate_Env(dopstclpp) ) ;
        if  ( IsNotEnvironmentLayer ( stclpp , newenvlayerp ) )
          return  TaskForThread_Analyse ;
        { PLOCharPriv  * const newchar = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
          if ( not newchar )  {
            OutOfMemory ( stclpp ) ;
            break ;
          }
          PLOChar_InitValue ( newchar , PLOString_CStr ( mep )  [ j ] ) ;
          if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer ( newenvlayerp ) -> envlay ,
              varBp , ParaLispObject_virtualPushAsNew ( & newchar -> pub . plo ) , false ) ) {
            HasConstLink  ( stclpp , varBp  ) ;
            return  TaskForThread_Analyse ;
          }
        }
      } // newenvlayer
      StackClusterPrivate_CreateTask (dopstclpp , task3  ) ;
    } // while j
  } // j
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U ) ;
  return  TaskForThread_Analyse ;
}

//  string swap string
TaskForThread bimstringswap  ( StackClusterPrivate   * const stclpp  ) {
  ParaLispObject  * const xp  =  CCAR ( StackClusterPrivate_ArgTop( stclpp ))  ;
  if  ( IsNotString ( stclpp , xp ) or IsConst ( stclpp , xp ) )
    return  TaskForThread_Analyse ;
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  PLOString_Swap  ( mep , ParaLispObjectToPLOString (   xp ) ) ;
  StackClusterPrivate_SetResult(stclpp ,& mep->plo  ) ;
  return  TaskForThread_Analyse ;
}

//  string setf string
TaskForThread bimstringsetf  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  ParaLispObject  * const xp =  CCAR ( StackClusterPrivate_ArgTop(stclpp ))  ;
  if  ( IsNotString ( stclpp , xp ) )
    return  TaskForThread_Analyse ;
  PLOString_Set ( mep , ParaLispObjectToPLOString (   xp ) ) ;
  StackClusterPrivate_SetResult(stclpp ,& mep->plo  ) ;
  return  TaskForThread_Analyse ;
}

# define  castCharCtoByteC  Lisp2D_castCharCtoByteC
inline  uint8_t  const *  castCharCtoByteC ( char const * s ) ;
  
# include "bytedeque/public.h"
# include "bytedeque/struct.h"
# include "stack/public.h"
  
// string byte-vector
TaskForThread bimstringbytevector  ( StackClusterPrivate   * const stclpp  ) {
  PLOByteDeque  * v2  ;
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const me2c = StackNew_StringChar ( ) ;
    String_Init ( Char  ) ( me2c , true ) ;
    { PLOStringPub * const mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
      if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , me2c ) ) {
        Lisp2D_StackFree  ( oldstack ) ;
        ParaLispObject  * list [ ] = {  & mep-> plo  , & PLOString_NewCopy ( L" task is BAD" )->plo , 0 } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        return  TaskForThread_Analyse ;
      }
    } // me
    uint8_t const * const meuintstr = castCharCtoByteC ( StringCStr  ( Char  ) ( me2c ) ) ;
    size_t    n = 0 ;
    for ( uint8_t const * j = meuintstr ; (  * j  ) ; ++ j  )   
      ++  n ;
    v2 = aligned_alloc ( _Alignof ( PLOByteDeque ) , sizeof ( PLOByteDeque ) ) ;
    if ( not v2 ) {
      Lisp2D_StackFree  ( oldstack ) ;
      OutOfMemory (  stclpp ) ;
      return  TaskForThread_Analyse ;
    }
    PLOByteDeque_InitSize ( v2 , n ) ;
    ParaLispObject_virtualPushAsNew (&v2  ->  plo ) ;
    PLOByteDeque_SetFrontNotCheckNotResize ( v2 , meuintstr , n ) ;
    Lisp2D_StackFree  ( oldstack ) ;
  }
  StackClusterPrivate_SetResult ( stclpp ,  & v2 -> plo  ) ;
  return  TaskForThread_Analyse ;
}

# include "vector/public.h"

// string vector
TaskForThread bimstringvector  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub const * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  size_t  const   n = String_Size(WideChar)( PLOString_CValue (mep ) ) ;
  PLOVector * const v2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if  ( not v2 ) {
    OutOfMemory ( stclpp ) ;
    return  TaskForThread_Analyse ;
  }
  PLOVector_InitSizeFill(v2,n , &plonil.plo );
  PLOVector_staticPushAsNew ( &v2->plo ) ;
  for ( ssize_t   i2  = size_cast_ssize ( n )  - 1 ; i2 >=  0  ;
    --  i2  ) {
    size_t const   i3  = ssize_cast_size ( i2  ) ;
    PLOCharPriv * const newchar = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
    if  ( not newchar ) {
      OutOfMemory ( stclpp ) ;
      break ;
    }
    PLOChar_InitValue ( newchar ,  PLOString_CStr (mep ) [ ssize_cast_size (i2)  ]);
    PLOVector_SetElt  ( v2  , i3  , ParaLispObject_virtualPushAsNew (  & newchar -> pub . plo ) ) ;
  } // for i2
  StackClusterPrivate_SetResult(stclpp ,&   v2->plo ) ;
  return  TaskForThread_Analyse ;
}

// string list
TaskForThread bimstringlist  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  ParaLispObject  *   res = & plonil.plo   ;
  for ( ssize_t   i2  = size_cast_ssize (
    String_Size(WideChar)( PLOString_CValue (mep ) ) )  - 1   ; i2 >=  0  ;
    --  i2  ) {
    PLOCharPriv * const cres = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
    if  ( not cres ) {
      OutOfMemory ( stclpp ) ;
      return  TaskForThread_Analyse ;
    }
    PLOChar_InitValue ( cres  , String_EltCopy  ( WideChar  ) ( PLOString_CValue (  mep ) , ssize_cast_size ( i2  ) ));
    ParaLispObject  * const res2 = &(cres -> pub . plo) ;
    ParaLispObject_virtualPushAsNew ( res2) ;
    res = &(  NewCons  (   res2 ,   res  )->plo ) ;
  } // for i2
  StackClusterPrivate_SetResult(stclpp ,  res  ) ;
  return  TaskForThread_Analyse ;
}

# include <wctype.h>
  
// string lower
TaskForThread bimstringlower ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const me0p = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  PLOStringPub * const  me3 = PLOString_NewCopyPLOString ( me0p ) ;
  for ( size_t  i2  = String_Size ( WideChar  ) ( PLOString_CValue ( me3 ) ) ;
    i2 > 0 ; ) {
    --  i2  ;
    ( * StringEltAddr ( WideChar  ) ( PLOString_Value2 ( me3 ) , i2 ) ) =
      wint_cast_wchar ( towlower (
        wchar_cast_wint ( String_EltCopy  ( WideChar  ) ( PLOString_CValue ( me3 ) ,
          i2 ) ) ) ) ;
  }
  StackClusterPrivate_SetResult ( stclpp , & me3 -> plo ) ;
  return  TaskForThread_Analyse ; 
}

// string upper
TaskForThread bimstringupper ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const me0p  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop(  stclpp ) ) ;
  PLOStringPub * const  me3 = PLOString_NewCopyPLOString ( me0p ) ;
  for ( size_t  i2  = String_Size ( WideChar  ) ( PLOString_CValue ( me3 ) ) ;
    i2 > 0 ; ) {
    --  i2  ;
    ( * StringEltAddr ( WideChar  ) ( PLOString_Value2 ( me3 ) , i2 ) ) =
      wint_cast_wchar ( towupper (
        wchar_cast_wint ( String_EltCopy  ( WideChar  ) ( PLOString_CValue ( me3 ) ,
          i2 ) ) ) ) ;
  }
  StackClusterPrivate_SetResult ( stclpp , & me3 -> plo ) ;
  return  TaskForThread_Analyse ; 
}

# include "minmax.h"
  
# define  StringMore  Lisp2DStringMoreUPsj
static  Lisp2DBoolType StringMore  ( ParaLispObject * const xp  ,
  ParaLispObject * const yp  , StackClusterPrivate  * const stclpp  ) {
  if  ( IsNotString (  stclpp  , yp ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOString_More (ParaLispObjectToPLOStringConst (   xp ),
    ParaLispObjectToPLOStringConst (   yp )) )
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ; }

// string max [{string}]
TaskForThread bimstringmax ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalMinMax  (  & StringMore , stclpp  ) ; }   

# include "lessmore.h"
  
// string > [{string}]
TaskForThread bimstringmore  ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalLessMore  (  & StringMore , stclpp  ) ; }   

# define  StringLess  Lisp2DStringLesssHOn
static  Lisp2DBoolType StringLess  ( ParaLispObject * const xp  ,
  ParaLispObject * const yp  , StackClusterPrivate  * const stclpp  ) {
  if  ( IsNotString (  stclpp  , yp ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOString_Less (ParaLispObjectToPLOStringConst (   xp ),
    ParaLispObjectToPLOStringConst (  yp )) )
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ; }
  
// string min [{string}]
TaskForThread bimstringmin ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalMinMax  (  & StringLess , stclpp  ) ; }

// string < [{string}]
TaskForThread bimstringless  ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalLessMore  (  & StringLess , stclpp  ) ; }   

# define  StringMoreEq  Lisp2DStringMoreEqRILs
static  Lisp2DBoolType StringMoreEq  ( ParaLispObject * const xp  ,
  ParaLispObject * const yp  ,
                               StackClusterPrivate  * const stclpp  ) {
  if  ( IsNotString (  stclpp  , yp ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOString_MoreEq (ParaLispObjectToPLOStringConst (   xp ),
    ParaLispObjectToPLOStringConst ( yp ) ))
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ; } 
  
// string >= [{string}]
TaskForThread bimstringmoreoreq  ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalLessMore  (  & StringMoreEq , stclpp  ) ; }  

# define  StringLessEq  Lisp2DStringLessEqsrkY
static  Lisp2DBoolType StringLessEq  ( ParaLispObject * const xp  ,
  ParaLispObject * const yp  ,
                               StackClusterPrivate  * const stclpp  ) {
  if  ( IsNotString (  stclpp  , yp ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOString_LessEq (ParaLispObjectToPLOStringConst (   xp ),
    ParaLispObjectToPLOStringConst ( yp ) ))
    return  Lisp2DBoolTrue  ;
  return  Lisp2DBoolFalse ; }  
  
// string <= [{string}]
TaskForThread bimstringlessoreq  ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalLessMore  (  & StringLessEq , stclpp  ) ;
}

# include <errno.h>
  
//  string rename string
TaskForThread bimstringrename  ( StackClusterPrivate   * const stclpp  ) {
  ParaLispObject  * const newnamep = CCAR ( StackClusterPrivate_ArgTop  (
    stclpp ) ) ;
  if  ( IsNotString ( stclpp , newnamep ) )
    return  TaskForThread_Analyse ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  { PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop (  stclpp ) ) ;
    if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep ) , strtmpc ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      BadString ( stclpp , mep  ) ;
      return  TaskForThread_Analyse ;
    }
  } // mep
  String  ( Char  ) * const strtmp2c = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmp2c , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr (
        ParaLispObjectToPLOString ( newnamep ) ) , strtmp2c ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    BadString ( stclpp  , ParaLispObjectToPLOString (   newnamep ) ) ;
    return  TaskForThread_Analyse ;
  }
  ParaLispObject  * res ;
  { int const an  = rename  (
      StringCStr  ( Char  ) ( strtmpc ) , StringCStr  ( Char  ) ( strtmp2c ) ) ;
    Lisp2D_StackFree  ( oldstack ) ;
    if ( an == 0 )
      res = & plonil  . plo ;
    else
      res = & SymbolFromIntErrno  ( errno ) -> plo ; } // an
  StackClusterPrivate_SetResult ( stclpp ,  res ) ;
  return  TaskForThread_Analyse ;
}

//  string unsetenv
TaskForThread bimstringunsetenv  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop (  stclpp ) ) ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char  ) * const strtmpc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr  ( mep  ) , strtmpc ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    BadString ( stclpp  , mep  ) ;
    return  TaskForThread_Analyse ;
  }
  int const an  = unsetenv  ( StringCStr  ( Char  ) ( strtmpc ) ) ;
  Lisp2D_StackFree  ( oldstack ) ;
  ParaLispObject  * res ;
  if ( an == 0 )
    res = & plonil  . plo ;
  else
    res = & SymbolFromIntErrno  ( errno ) -> plo ;
  StackClusterPrivate_SetResult(stclpp ,res ) ;
  return  TaskForThread_Analyse ;
}

//  string setenv
TaskForThread bimstringsetenv  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char ) * const strtmp2 = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmp2 , true ) ;
  if  ( isBADReadWCharToCharString ( PLOString_CStr ( mep ) , strtmp2 ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    BadString ( stclpp , mep  ) ;
    return  TaskForThread_Analyse ;
  }
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop(stclpp )) ;
  if  ( IsNotString ( stclpp ,  xp ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    return  TaskForThread_Analyse ;
  }
  String  ( Char  ) * const strtmpxc = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmpxc , true ) ;
  if  ( isBADReadWCharToCharString  ( PLOString_CStr (
      ParaLispObjectToPLOString ( xp ) ) , strtmpxc ) ) {
    Lisp2D_StackFree  ( oldstack ) ;
    BadString ( stclpp  , ParaLispObjectToPLOString ( xp ) ) ;
    return  TaskForThread_Analyse ;
  }
  int const  an  = setenv  ( StringCStr ( Char ) ( strtmp2 ) ,
    StringCStr  ( Char  ) ( strtmpxc ) , 1 ) ;
  Lisp2D_StackFree  ( oldstack ) ;
  ParaLispObject  * res ;
  if ( an == 0 )
    res = & plonil  . plo ;
  else
    res = & SymbolFromIntErrno  ( errno ) -> plo ;
  StackClusterPrivate_SetResult ( stclpp , res ) ;
  return  TaskForThread_Analyse ;
}

//  string getenv
TaskForThread bimstringgetenv  ( StackClusterPrivate   * const stclpp  ) {
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( Char ) * const strtmp = StackNew_StringChar ( ) ;
  String_Init ( Char  ) ( strtmp , true ) ;
  { PLOStringPub * const mep  = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
    if  ( isBADReadWCharToCharString (
      PLOString_CStr ( mep ) , strtmp  ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      BadString ( stclpp , mep ) ;
      return  TaskForThread_Analyse ;
    }
  }
      
  // !! getenv_s    C11
  /*
   #define __STDC_WANT_LIB_EXT1__ 1
#include <stdlib.h>
errno_t getenv_s(size_t * restrict len,
  char * restrict value, rsize_t maxsize,
  const char * restrict name);
   */
      
  { char const * const  an  = getenv  ( StringCStr ( Char  ) ( strtmp ) ) ;
    if  ( an ) {
      StringSetArray  ( Char  ) ( strtmp  , an  ) ;
      String  ( WideChar ) * const wan2 = StackNew_StringWideChar ( ) ;
      String_Init ( WideChar  ) ( wan2 , true ) ;
      if  ( isBADReadCStringToWString2  ( strtmp , wan2 ) ) {
        Lisp2D_StackFree  ( oldstack ) ;
        BadStringN ( stclpp ) ;
        return  TaskForThread_Analyse ;
      }
      { PLOStringPub * const  res2 = PLOString_NewMoveString ( wan2 ) ;
        StackClusterPrivate_SetResult ( stclpp ,  &res2 -> plo  ) ;
      }
      // if an
    } else
      StackClusterPrivate_SetResult(stclpp ,& plonil.plo  ) ;
  } // an
  Lisp2D_StackFree  ( oldstack ) ;
  return  TaskForThread_Analyse ;
}

# define  SaveObject  Lisp2DSaveObjectoEJt
static  bool  SaveObject ( ParaLispObject  const * const xp ,
  PLOStream * const sp ) {
  { ThreadStackPointerType const oldstack_hi = Lisp2DThreadSP ( ) ;
    Vector  ( StackPLOP ) * const hip2 = StackNew_VectorPLOP ( ) ;
    VectorInit  ( StackPLOP ) ( hip2 , false ) ;
    bool  const ffalse  = false ;  
    ParaLispObject_CoutToInterpret  (  xp  ,  sp , & ffalse  , hip2 ) ;
    Lisp2D_StackFree  ( oldstack_hi ) ;
  }
  int e = 0 ;
  StreamClose ( & sp -> str , & e ) ;
  return  not ( e ) ;
}
  
# define  SaveObjectLockGC  Lisp2DSaveObjectLockGCyie5
static  bool  SaveObjectLockGC  ( StackClusterPrivate * const stclpp ,
  String  ( WideChar  ) const * const nmp2 , ParaLispObject  const * const xp ) {
  PLOStream  * const s = aligned_alloc ( _Alignof ( PLOStream ) , sizeof ( PLOStream ) ) ;
  if  ( s ) {
    PLOStreamInit ( s ) ;
    PassagePassIn ( passageGCCPub ) ;
    ParaLispObject_virtualPushAsNew (&s -> plo ) ;
    PassagePassOut ( passageGCCPub ) ;  
  } else  {
    OutOfMemory (  stclpp ) ;
    return  false ; 
  }
  if  ( StreamisBADOpenWOutLockGC ( & s -> str, nmp2 , stclpp ) ) 
    return  false ; 
  return  SaveObject ( xp ,  s ) ; 
}
  
// string save obj
TaskForThread bimstringsave  ( StackClusterPrivate   *const stclpp  ) {
  { ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) ) ;
    String  ( WideChar  ) const * const wstr = PLOString_CValue  (
      ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ) ;
    bool const bres  = SaveObjectLockGC  ( stclpp , wstr ,  xp ) ;
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult(stclpp , (bres ? xp : &plonil.plo )) ;
    PassagePassOut ( passageGCCPub ) ; 
  }
  return  TaskForThread_Analyse ; 
}

// string symbol
TaskForThread bimstringsymbol  ( StackClusterPrivate   * const stclpp  ) {
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( WideChar ) * const wsp =  StackNew_StringWideChar ( ) ;
  StringConstrCStr ( WideChar )  ( wsp , PLOString_CStr (
    ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) ) ) , true ) ;
  StackClusterPrivate_SetResult ( stclpp , SymbolAddMoveString ( wsp ) ) ;
  Lisp2D_StackFree  ( oldstack ) ;
  return  TaskForThread_Analyse ;
}

# include "deque/public-wchar.h"
  
//  string read
TaskForThread bimstringread  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  size_t const  n = String_Size ( WideChar ) ( PLOString_CValue ( mep ) ) ;
  if  ( n  ==  0  ) {
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_SetResult(stclpp ,& plonil.plo  ) ;
    PassagePassOut ( passageGCCPub ) ;
    return  TaskForThread_Analyse ; 
  }
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  Deque ( WChar ) * const dqstp = Lisp2D_StackVar ( StackType ( Deque_WChar  ) ) ;
  Deque_Init ( WChar ) ( dqstp ) ;
  Deque_Resize ( WChar ) ( dqstp , n  + 1 ) ;
  for ( size_t  il  = 0 ; il  < n ; ++  il  )
    ( * Deque_AtFast ( WChar ) ( dqstp , il ) ) =
      String_EltCopy  ( WideChar  ) ( PLOString_CValue (  mep ) , il  ) ;
  ( * Deque_AtFast ( WChar ) ( dqstp , n ) ) = L' '  ;
  { ParaLispObject  * r =  & plonil.plo ;
    PassagePassIn ( passageGCCPub ) ;
    StackClusterPrivate_LoadObjectS (stclpp , &r , 0 , dqstp ) ;
    StackClusterPrivate_SetResult(stclpp , r ) ;
    PassagePassOut ( passageGCCPub ) ; 
  }
  Lisp2D_StackFree  ( oldstack ) ;
  return  TaskForThread_Analyse ; 
}

// string size
TaskForThread bimstringsize  ( StackClusterPrivate   * const stclpp  ) {
  size_t const  vs = String_Size ( WideChar ) ( PLOString_CValue (
    ParaLispObjectToPLOString  ( StackClusterPrivate_ThisTop ( stclpp ) ) ) ) ;
  PLOComplex  *  const res =NewComplexBareExc  ( ) ;
  Complex_SetUSize ( PLOComplexToComplex( res), vs  ) ;
  StackClusterPrivate_SetResult(stclpp ,&  res->plo ) ; 
  return  TaskForThread_Analyse ; 
}

//  string setelt
TaskForThread bimstringsetelt  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const  mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) ) ;
  { ParaLispObject  const * const  lp  = StackClusterPrivate_ArgTop(stclpp ) ;
    ParaLispObject  * const ip = CCAR ( lp ) ;
    if  ( IsNotNumber ( stclpp , ip ) )
      return  TaskForThread_Analyse ;
    if  ( not  FractionIntegerP ( Complex_CRe  (
      PLOComplexToComplexConst  ( ParaLispObjectToPLOComplexConst ( ip ) )
        ) ) ) {
      { ParaLispObject  * list [ ] = {   ip  , & PLOString_NewCopy (
          L" is not integer"  )->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      }
      return  TaskForThread_Analyse ;
    }
    if  ( FractionLessUInt ( Complex_CRe (
      PLOComplexToComplexConst  (
      ParaLispObjectToPLOComplex  ( ip )) ),0U) ) {
      { ParaLispObject  * list [ ] = {   ip  , & PLOString_NewCopy ( L" < 0" )
          ->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      }
      return  TaskForThread_Analyse ; 
    }
    ParaLispObject  * const cp = CCAR ( CCDR ( lp ) ) ;
    if  ( IsNotChar ( stclpp , cp ) )
      return  TaskForThread_Analyse ;
    size_t  ind ;
    if  ( ULongIsBADSizeValue ( FractionCNump  (
       Complex_CRe  ( PLOComplexToComplexConst  (
         ParaLispObjectToPLOComplexConst ( ip ) ) ) ) , & ind ) ) {
      { ParaLispObject  * list [ ] = {
          ip  , & PLOString_NewCopy ( L" is very big" )->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
      }
      return  TaskForThread_Analyse ; 
    }
    ( * StringEltAddr ( WideChar ) ( PLOString_Value2 ( mep ) , ind ) ) =
      PLOChar_Value(ParaLispObjectToPLOChar ( cp ) ) ;
  }
  StackClusterPrivate_SetResult(stclpp ,& mep->plo  ) ;
  return  TaskForThread_Analyse ; 
}

//  string elt
TaskForThread bimstringelt ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub const * const mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ParaLispObject  * const ploindp  = CCAR ( StackClusterPrivate_ArgTop  ( stclpp ) ) ;
  if  ( IsNotNumber ( stclpp , ploindp  ) )
    return  TaskForThread_Analyse ;
  if  ( not  FractionIntegerP ( Complex_CRe  (
    PLOComplexToComplexConst
    ( ParaLispObjectToPLOComplexConst ( ploindp  ) ) ) ) ) {
    { ParaLispObject  * list [ ] = {   ploindp ,
      &  PLOString_NewCopy  ( L" is not integer"  )->plo , NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    }
    return  TaskForThread_Analyse ; 
  }
  if  ( FractionLessUInt ( Complex_CRe (
    PLOComplexToComplexConst  (
    ParaLispObjectToPLOComplex  ( ploindp  ) ) ) , 0U ) ) {
    { ParaLispObject  * list [ ] = {   ploindp ,
        &  PLOString_NewCopy  ( L"<0" )->plo , NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    }
    return  TaskForThread_Analyse ; 
  }
  size_t  s ;
  if  ( ULongIsBADSizeValue ( FractionCNump ( Complex_CRe  (
    PLOComplexToComplexConst  ( ParaLispObjectToPLOComplexConst ( ploindp ) ) ) ) , & s ) ) {
    { ParaLispObject  * list [ ] = {   ploindp , 
        &  PLOString_NewCopy  ( L" is very big" )->plo , NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    }
    return  TaskForThread_Analyse ; 
  }
  if  ( s  >=  String_Size(WideChar)(PLOString_CValue(mep)) ) {
    StackClusterPrivate_SetResult(stclpp ,& plonil.plo  ) ;
    return  TaskForThread_Analyse ;
  }
  PLOCharPriv  * const res = aligned_alloc ( _Alignof ( PLOCharPriv ) , sizeof ( PLOCharPriv ) ) ;
  if ( not res )
    OutOfMemory ( stclpp ) ;
  else  {
    PLOChar_InitValue ( res , PLOString_CStr (  mep ) [ s ] ) ;
    StackClusterPrivate_SetResult ( stclpp ,   ParaLispObject_virtualPushAsNew (  & res -> pub . plo ) ) ;
  }
  return  TaskForThread_Analyse ; 
}

// string <>
# define  StringNotEq Lisp2DStringNotEqDoAZ
static  Lisp2DBoolType StringNotEq  ( ParaLispObject * const  xp  ,
  ParaLispObject * const yp  ) {
  if  ( not  Stringp  ( yp ) )
    return  Lisp2DBoolTrue  ;
  if  ( PLOString_Eq(ParaLispObjectToPLOStringConst  ( xp ) , 
    ParaLispObjectToPLOStringConst ( yp ) ))
    return  Lisp2DBoolFalse ;
  return  Lisp2DBoolTrue  ; 
}
  
# include "eqnoteq.h"
  
TaskForThread bimstringnotequal ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalEqNotEq(  & StringNotEq  , stclpp  ) ;
}

// string = [{x}]
# define  StringEq  Lisp2DStringEqt5OX
static  Lisp2DBoolType StringEq  ( ParaLispObject * const xp  , ParaLispObject * const yp  ) {
  if  ( not  Stringp  ( yp ) )
    return  Lisp2DBoolFalse ;
  if  ( PLOString_NotEq ( ParaLispObjectToPLOStringConst  ( xp) ,
    ParaLispObjectToPLOStringConst ( yp ) ))
    return  Lisp2DBoolFalse ;
  return  Lisp2DBoolTrue  ;
}
  
TaskForThread bimstringequal ( StackClusterPrivate   * const stclpp  ) {
  return  bimUniversalEqNotEq (  & StringEq  , stclpp  ) ;
}

// string +=
TaskForThread bimstringplussetf  ( StackClusterPrivate   * const stclpp  ) {
  PLOStringPub * const mep = ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ParaLispObject  * j ;
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) ) {
    j = StackClusterPrivate_PopValue ( stclpp ) ;
    goto  Next  ;
  }
  StackClusterPrivate_SetResult ( stclpp ,& mep->plo  ) ;
  j = StackClusterPrivate_ArgTop  ( stclpp ) ;
Next  :
  { unsigned  int tasks = maxListItemProcessing ;
  for ( ; ( tasks  > 0  ) and CConsp (  j ) ;
    j  = ClassCons_Rest ( & ParaLispObjectToPLOCons ( j ) ->  cc ) ,
    --  tasks ) {
    ParaLispObject  * const xp = ClassCons_First  ( & ParaLispObjectToPLOCons (
      j ) ->  cc ) ;
    switch  (    ParaLispObject_Type  ( xp)  ) {
    case  TypeGlobalListMemberPLOString  :
      PLOString_Add (mep, ParaLispObjectToPLOString ( xp )) ;
      break ;
    case  TypeGlobalListMemberPLOChar  :
      String_AddChar(WideChar)(PLOString_Value2 (mep ),
        PLOChar_Value(ParaLispObjectToPLOChar ( xp ) ));
      break ;
    default :
      { ParaLispObject  * list [ ] = { xp  , & PLOString_NewCopy ( 
          L" is not string nor char"  )->plo , NULL } ;
        StackClusterPrivate_ExceptionSet(stclpp ,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      }
      return  TaskForThread_Analyse ;
    }
    }
  }
  if  ( CConsp (   j  ) ) {
    StackClusterPrivate_setTaskForManager ( stclpp , 
      TaskPause ) ;    
    StackClusterPrivate_SetflagCont  ( stclpp , 1U ) ;
    StackClusterPrivate_Push (stclpp , j ) ;
  }
  return  TaskForThread_Analyse ;
}

// ! to merge code for + and += 
  
// string + [{string}]
TaskForThread bimstringplus  ( StackClusterPrivate  * const stclpp  ) {
  ParaLispObject  * j ;
  PLOStringPub *  me ;
  if  ( StackClusterPrivate_ClearflagCont  ( stclpp ) ) {
    me  = ParaLispObjectToPLOString ( StackClusterPrivate_PopValue ( stclpp ) ) ;
    j = StackClusterPrivate_PopValue ( stclpp ) ;
    goto  Next  ;
  }
  me  =  ParaLispObjectToPLOString ( StackClusterPrivate_ThisTop( stclpp ) ) ;
  { ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop(stclpp ) ;
    if  ( not  CConsp  (  arglistp ) ) {
      StackClusterPrivate_SetResult(stclpp ,&  me->plo ) ;
      return  TaskForThread_Analyse ;
    }
    j =    arglistp   ;
  }
  me = PLOString_NewCopy ( PLOString_CStr ( me ) ) ;
  StackClusterPrivate_SetResult(stclpp ,&   me->plo ) ;
Next  :
  { unsigned  int tasks = maxListItemProcessing ;
    for ( ; ( tasks  > 0  ) and CConsp (   j  ) ;
      j  = ClassCons_Rest( &ParaLispObjectToPLOCons ( j )->cc ) , --  tasks ) {
    ParaLispObject  * const xp = ClassCons_First  ( & ParaLispObjectToPLOCons (
      j ) ->  cc ) ;
    switch  (  ParaLispObject_Type  (xp )  ) {
    case  TypeGlobalListMemberPLOString  :
      PLOString_Add (   me , ParaLispObjectToPLOString ( xp ) ) ;
      break ;
    case  TypeGlobalListMemberPLOChar  :
      String_AddChar(WideChar)(PLOString_Value2 (me ),
        PLOChar_Value(ParaLispObjectToPLOChar ( xp ) ));
      break ;
    default :
        { ParaLispObject  * list [ ] = {
            xp  , & PLOString_NewCopy ( L" is not string nor char"  ) ->  plo , NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        }
        return  TaskForThread_Analyse ;
      }
    }
  }
  if  ( CConsp (   j  ) ) {
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont  ( stclpp , 1U ) ;
    StackClusterPrivate_Push (stclpp , j ) ;
    StackClusterPrivate_Push (stclpp ,&  me->plo  ) ;
  }
  return  TaskForThread_Analyse ;
}

# include "castb.h"
# include "widestring/inline.h"
inline  uint8_t  const *  castCharCtoByteC ( char const * const s ) {
  return  ( uint8_t  const * ) s ;
}
