// Lisp2D - String
// ©2006-23 by Alex Glebe
# include "define.h"
# include "string/public.h"

# ifdef  LISP2D_CONSTRSTAT
_Atomic ( size_t  ) PLOString_constrstat = 0 ;

size_t  PLOString_constrstatValue ( void  ) {
  return  PLOString_constrstat  ;
}

void  PLOString_constrstatInc ( void  ) {
  ++  PLOString_constrstat  ;
}

# endif // LISP2D_CONSTRSTAT

# include "plostream/public.h"

void  PLOString_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool _ __attribute__ ((unused)) , Vector ( StackPLOP )  * _2 __attribute__ ((unused)) ) {
  PLOStreamPushWChar(s,L'"');
  PLOStreamPushWideCharArray  ( s , ( PArray_wchar_c ) { (wchar_t const(*)[])
    PLOString_CStr ( ParaLispObjectToPLOStringConst  (  me  ) ) } ) ;
  PLOStreamPushWChar(s,L'"');  
} 

void  PLOString_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool _ __attribute__ ((unused)) ,
  Vector  ( StackPLOP )  * _2 __attribute__ ((unused))  ) {
  PLOStreamPushWideCharArray  ( s , ( PArray_wchar_c ) { (wchar_t const(*)[])
    PLOString_CStr ( ParaLispObjectToPLOStringConst  (  me  ) ) } ) ;   
}

bool  PLOString_staticisBADCopyPrivate ( ParaLispObject const * me ,   ParaLispObject  * * const resultp  ) {
  PLOStringPub * const  res0 = PLOString_NewCopy ( PLOString_CStr ( ParaLispObjectToPLOStringConst ( me ) ) ) ;
  ( * resultp ) = & res0  ->  plo ;
  return false ;
}

# include "minmaxint.h"

bool  PLOString_Less ( PLOStringPub const * const  sp  , PLOStringPub  const  * const mep ) {
  if  (  mep  ==   sp  )
    return  false ;
  { size_t  const   mel = String_Size  ( WideChar  ) ( PLOString_CValue (  mep  ) ) ;
    size_t  const   sl  = String_Size  ( WideChar  ) ( PLOString_CValue (  sp ) ) ;
    { size_t  const  l  = Lisp2DMinSize ( mel , sl  ) ;
      wchar_t volatile const * meit = PLOString_CStr (mep ) ;
      wchar_t volatile const * sit  = PLOString_CStr (sp ) ;
      size_t  i = 0 ;
      while ( i  < l ) {
        if  ( (  * meit ) < ( *  sit  ) )
          return  false ;
        if  ( (  * meit ) > ( *  sit  ) )
          return  true  ;
        ++  meit  ;
        ++  sit ;
        ++  i ;
      }
    } // l
    if  ( mel  > sl )
      return  true  ;
  } // mel , sl
  return  false ;
}

# include "symbolc.h"

ParaLispObject  * PLOString_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return &  plosstring . plo  ;
}

# include "string/private.h"

// returns new PLOString or exception
PLOStringPub * PLOString_New ( void ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_New" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal  = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_Init ( res ) ;
  Lisp2DJBglobal = oldjb  ;
  return   ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew (  & res -> pub . plo ) ) ;
}

// returns new PLOString or exception
PLOStringPub * PLOString_NewCopy ( wchar_t volatile const * const s ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_NewCopy  ( wchar_t  volatile const  * )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_Initwcharstr  ( res , s ) ;
  Lisp2DJBglobal = oldjb  ;
  return   ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew (  & res -> pub . plo ) ) ;
}

// returns new PLOString or exception
PLOStringPub * PLOString_NewWChar ( wchar_t const ch ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_NewWChar ( wchar_t ch )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_InitWChar ( res , ch ) ;
  Lisp2DJBglobal = oldjb  ;
  return   ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew (  & res -> pub . plo ) ) ;
}

// returns new PLOString or exception
PLOStringPub * PLOString_NewCopyPLOString ( PLOStringPub * const sp ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_NewCopyPLOString ( PLOStringPub * const sp )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_InitCopy  ( res , sp ) ;
  Lisp2DJBglobal = oldjb  ;
  return   ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ) ;
}

// returns new PLOString or exception
PLOStringPub * PLOString_NewMoveString ( String  ( WideChar ) * const sp ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_NewMoveString ( String  ( WideChar ) * )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_InitMoveString  ( res , sp ) ;
  Lisp2DJBglobal = oldjb  ;
  return   ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ) ;
}
  
PLOStringPub * PLOString_NewCopyArr ( PArray_wchar_c const s ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_NewCopyArr ( PArray_wchar_c s )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_InitPArray_wchar_c  ( res , s ) ;
  Lisp2DJBglobal = oldjb  ;
  return  ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ) ;
}

PLOStringPub * PLOString_NewStrBeg ( PLOStringPub  const * const vp , size_t const indexfirst ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_NewStrBeg (  PLOStringPub  const * vp , size_t indexfirst )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_InitStrBeg ( res , vp , indexfirst  ) ;
  Lisp2DJBglobal = oldjb  ;
  return  ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ) ;
}

PLOStringPub * PLOString_NewStrBegEnd ( t_PLOString_NewStrBegEnd  const arg ) {
  PLOStringPri  * const res = aligned_alloc ( _Alignof ( PLOStringPri ) , sizeof ( PLOStringPri ) ) ;
  if ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOString_NewStrBegEnd ( t_PLOString_NewStrBegEnd arg )" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  jmp_buf jump_buffer ;
  jmp_buf * const oldjb = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jump_buffer ;
  int const volatile exctype = setjmp  ( jump_buffer ) ;
  if  ( exctype ) {
    free  ( res ) ;
    Lisp2DJBglobal = oldjb  ;
    longjmp ( * Lisp2DJBglobal , exctype ) ;
  }
  PLOString_InitStrBegEnd ( res , arg . vp , arg . indexfirst , arg . indexend ) ;
  Lisp2DJBglobal = oldjb  ;
  return  ParaLispObjectToPLOString ( ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ) ;
}

void  PLOString_Initwcharstr ( PLOStringPri * const me , wchar_t volatile const * const s ) {
  StringConstrCStr ( WideChar  ) ( & me -> value , s , true ) ;
  ParaLispObject_InitVTP  ( & me  -> pub . plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
}

void  PLOString_InitMoveString ( PLOStringPri * const me , String ( WideChar ) * const swc ) {
  String_ConstrMove ( WideChar  ) ( & me -> value , swc ) ;
  ParaLispObject_InitVTP  ( & me  -> pub .  plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
}

void  PLOString_InitPArray_wchar_c ( PLOStringPri * const me , PArray_wchar_c const s ) {
  StringConstrCStr ( WideChar  ) ( & me -> value , * s . p , true ) ;
  ParaLispObject_InitVTP  ( & me -> pub . plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
}

bool  Stringp ( ParaLispObject const * const xp  ) {
  return  ParaLispObject_Type(xp)==TypeGlobalListMemberPLOString;
}

void PLOString_Add ( PLOStringPub  * const  mep , PLOStringPub  const * const  xp ) {
  StringAddString ( WideChar )  ( PLOString_Value2 (  mep )  , PLOString_CValue (  xp ) ) ;
}

void  PLOString_Addws ( PLOStringPub  * const mep , wchar_t  volatile const * const  xp ) {
  StringAddCharArray  ( WideChar )  ( PLOString_Value2 ( mep ) , xp  ) ;
}

# include "string/inlinepri.h"

void  PLOStringMoveString ( PLOStringPub * const me , String ( WideChar ) * const ws ) {
  PLOStringPri * const mep = StringPub_cast_StringPri ( me ) ;
  StringSwap  ( WideChar ) ( & mep -> value , ws  ) ;
}
  
# include "cast.h"

bool  PLOString_Eq  ( PLOStringPub const * const me  , PLOStringPub const * const sp  ) {
  return  ( me  ==  sp ) or ( wcscmp  ( wcharconstvolatp_cast_wcharconstp ( PLOString_CStr  ( me  ) ) ,
    wcharconstvolatp_cast_wcharconstp ( PLOString_CStr (  sp ) ) ) == 0 ) ;
}

bool  PLOString_NotEq ( PLOStringPub const * const me  , PLOStringPub const * const sp  ) {
  return  ( me  not_eq  sp ) and wcscmp  ( wcharconstvolatp_cast_wcharconstp ( PLOString_CStr ( me  ) ) ,
    wcharconstvolatp_cast_wcharconstp ( PLOString_CStr (  sp )  ) ) ;
}

void  PLOString_SetCStr ( PLOStringPub * const me , wchar_t const * const ws ) {
  if  ( PLOString_CStr  ( me ) not_eq  ws )
    StringSetArray  ( WideChar ) (  PLOString_Value2 ( me ) , ws  ) ; 
}

void  PLOString_InitCopy ( PLOStringPri * const me , PLOStringPub  const * const sp ) {
  ParaLispObject_InitVTP  ( & me  ->  pub . plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
  String_ConstrCopy ( WideChar  ) ( & me -> value , PLOString_CValue ( sp ) ) ;
}

void  PLOString_Destr ( PLOStringPri * const me ) {
  StringDestr ( WideChar  ) ( & me -> value ) ;
  ParaLispObject_staticDestr  ( & me -> pub . plo ) ;
}

void PLOString_staticDestr  ( ParaLispObject * const  me  ) {
  PLOString_Destr ( StringPub_cast_StringPri ( ParaLispObjectToPLOString ( me  ) ) ) ;
}

ParaLispObject  * PLOString_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassdefstring  ;
}

ParaLispObject  * PLOString_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassparentsstring  ;
}

# define  PLOString_SubSeq  Lisp2D_PLOString_SubSeq  
// e <= v.size
static  void  PLOString_SubSeq  ( PLOStringPub * const me , PLOStringPub const * const vp ,
  size_t  const indexfirst  , size_t const e  ) {
  if  ( indexfirst < e  ) {
    PLOString_Resize  ( me , e - indexfirst  ) ; 
    wchar_t volatile * j = String_Begin  ( WideChar  ) ( PLOString_Value2 ( me ) ) ;
    wchar_t volatile const * i = String_CBegin ( WideChar  ) ( PLOString_CValue ( vp ) ) +
      ( ptrdiff_t ) ( indexfirst  ) ;
    wchar_t volatile const * const je  = String_CEnd ( WideChar  ) ( PLOString_CValue ( me ) ) ;
    do  {
      ( *  j  )  = (  *  i) ;
      ++  j ;
      ++  i ;
    } while ( j  not_eq  je ) ;
  }
}

void PLOString_InitStrBeg ( PLOStringPri * const me , PLOStringPub  const * const vp , size_t  const indexfirst  ) {
  String_Init ( WideChar  ) ( & me -> value , true ) ;
  ParaLispObject_InitVTP  ( & me -> pub . plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString } ) ;
  size_t  const   e = String_Size(  WideChar ) (PLOString_CValue ( vp )) ;
  PLOString_SubSeq  ( & me -> pub , vp , indexfirst  , e ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
}

void  PLOString_InitStrBegEnd ( PLOStringPri * const me , PLOStringPub  const * const vp , size_t const  indexfirst ,
  size_t const indexend ) {
  String_Init ( WideChar  ) ( & me -> value , true ) ;
  ParaLispObject_InitVTP  ( & ( me -> pub . plo ) ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString } ) ;
  size_t  const   e = Lisp2DMinSize ( indexend  , String_Size(  WideChar ) (PLOString_CValue ( vp )) ) ;
  PLOString_SubSeq  ( & me -> pub , vp , indexfirst  , e ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
}

void  PLOString_Init ( PLOStringPri * const me ) {
  String_Init ( WideChar  ) ( & me ->value , true ) ;
  ParaLispObject_InitVTP  ( & me -> pub . plo,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString });
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
}    

# include "char/public.h"
  
void  PLOStringAddPLOChar ( PLOStringPub  * const mep , PLOCharPub  const * const  xp ) {
  String_AddChar  ( WideChar  ) ( PLOString_Value2 (  mep )  , PLOChar_Value (  xp ) ) ;
}

# include "plocomplex/public.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "complex/type.h"
# include "complex/inline.h"

void PLOString_AssignSubSeq  ( PLOStringPub * const me , PLOStringPub  * const  vp , PLOComplex * const indp  ,
  PLOComplex * const endp , StackClusterPrivate  * const stp  ) {
  Complex const * const cindp = PLOComplexToComplexConst ( indp  ) ;
  Complex const * const cendp = PLOComplexToComplexConst ( endp  ) ;
  if  ( not  Complex_Integerp ( cindp ) ) {
    ParaLispObject  * list [ ] = {
      &  indp->plo  , & PLOString_NewCopy ( L" is not integer"  )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return ; 
  }
  if  ( not  Complex_Integerp ( cendp ) ) {
    ParaLispObject  * list [ ] = {
      &  endp->plo  , & PLOString_NewCopy ( L" is not integer"  )->plo , 
      NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return ; 
  }
  if  ( FractionLessUInt(Complex_CRe ( cindp ) ,0U) ) {
    ParaLispObject  * list [ ] = {
      &  indp->plo  , & PLOString_NewCopy ( L" < 0" )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return ; 
  }
  if  ( FractionLessUInt(Complex_CRe ( cendp ),0U) ) {
    ParaLispObject  * list [ ] = {
      &  endp->plo  , & PLOString_NewCopy ( L" < 0" )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  ; 
  } 
  if  ( FractionLess  ( Complex_CRe ( cendp ) ,
    Complex_CRe ( cindp ) )  == Lisp2DBoolTrue  ) {
    ParaLispObject  * list [ ] = {
      &  endp->plo  , &  PLOString_NewCopy  ( L" < "  )->plo , & indp->plo ,
      NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return ; 
  }
  if  ( me == vp ) {
    // me=[0..ind-1,ind,ind+1..s-1] -> [ind,..,end-1]
    if  ( FractionIs0 ( Complex_CRe ( cindp ) ) ) {
      // me=[0..ind-1,ind,ind+1..s-1] -> [0,..,end-1]
      size_t  end2  ;
      if( FractionIsBadSizeValue ( Complex_CRe ( cendp ) , & end2  ))
        return ;
      PLOString_ResizeLet  ( me , end2  , L' '  ) ; 
      return  ;
    } // if ind.re=0
    size_t  const   vs  = String_Size(  WideChar ) (PLOString_CValue ( me )) ;  
    size_t  ind2  ;
    if  (   FractionIsBadSizeValue  ( Complex_CRe  ( cindp ) , & ind2  ) ) {
      ParaLispObject  * list [ ] = {
        &  indp->plo  , & PLOString_NewCopy  ( L" is bad"  )->plo , NULL } ;
      StackClusterPrivate_ExceptionSet  ( stp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return ; 
    }
    size_t  end2  ;
    if  (  FractionIsBadSizeValue  ( Complex_CRe  ( cendp ) , & end2  ) ) {
      ParaLispObject  * list [ ] = {
        &  endp->plo  , & PLOString_NewCopy  ( L" is bad"  )->plo , NULL } ;
      StackClusterPrivate_ExceptionSet  ( stp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return ; 
    }
    size_t  const  newsize = end2 - ind2 ;
    if  ( FractionMoreEqUSize(Complex_CRe  ( cindp ),vs)  ) {
      PLOString_Clear ( me ) ;
      PLOString_ResizeLet  ( me , newsize , L' '  ) ;
      return  ; 
    } // if ind.re>=s
    if  ( newsize  > vs ) 
      PLOString_ResizeLet  ( me , newsize , L' '  ) ; 
    wchar_t volatile * jtarget = String_Begin (  WideChar ) ( PLOString_Value2 ( me ) ) ;
    wchar_t volatile * jsource = jtarget + size_cast_ptrdiff ( ind2  ) ;
    do  {
      ( *  jtarget  )  = (  *  jsource  ) ;
      ++  jtarget ;
      ++  jsource ;
      ++  ind2  ;
    } while ( ( ind2 < vs )  and (  ind2  < end2  ) ) ;
    PLOString_Resize  ( me , newsize ) ;
    return  ; 
  } // if me == vp
  size_t  const   vvs = String_Size(  WideChar ) (PLOString_CValue ( vp )) ;    
  size_t  ind2  ;
  if  (  FractionIsBadSizeValue  ( Complex_CRe  ( cindp ) , & ind2  ) ) {
    ParaLispObject  * list [ ] = { &  indp->plo  ,
      & PLOString_NewCopy  ( L" is bad"  )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return ; 
  }
  size_t  end2  ;
  if  ( FractionIsBadSizeValue  ( Complex_CRe  ( cendp ) , & end2  ) ) {
    ParaLispObject  * list [ ] = {
      &  endp->plo  , & PLOString_NewCopy  ( L" is bad"  )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return ; 
  }
  size_t  const  newsize = end2 - ind2 ;
  PLOString_Clear ( me ) ;
  PLOString_ResizeLet  (me, newsize , L' '  ) ; 
  if  ( ind2 >=  vvs  )
    return ;  
  wchar_t volatile * jtarget = String_Begin (  WideChar ) ( PLOString_Value2 ( me ) ) ;
  wchar_t volatile * jsource = String_Begin (  WideChar ) ( PLOString_Value2 ( vp ) ) +
    size_cast_ptrdiff ( ind2  ) ;
  while ( ( ind2 < vvs  )  and (  ind2  < end2  ) ) {
    ( *  jtarget  )  = (  *  jsource  ) ;
    ++  jtarget ;
    ++  jsource ;
    ++  ind2  ;
  } 
}  

void PLOString_AssignSubSeqEnd  ( PLOStringPub * me , PLOStringPub  * const  vp , PLOComplex * const indp  ,
  StackClusterPrivate  * const stp  ) {
  if  ( not  Complex_Integerp ( PLOComplexToComplexConst(indp) ) ) {
    ParaLispObject  * list [ ] = {
      &  indp -> plo  , & PLOString_NewCopy ( L" is not integer"  )->plo , 
      NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  ; 
  }
  if  ( FractionLessUInt ( Complex_CRe (PLOComplexToComplexConst( indp) ),0U)  ) {
    ParaLispObject  * list [ ] = {
      &  indp -> plo  , & PLOString_NewCopy ( L" < 0" )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  ; 
  }
  size_t  const vvs  = String_Size(  WideChar ) (PLOString_CValue ( vp )) ;      
  if  ( FractionMoreUSize(Complex_CRe  (
    PLOComplexToComplexConst(indp) ), vvs ) ) {
    PLOComplex  * const  res = NewComplexBareExc  ( ) ;
    if  ( res ) {
      Complex_SetUSize (  PLOComplexToComplex(res), vvs ) ;
      ParaLispObject  * list [ ] = {
        &  indp->plo  , &  PLOString_NewCopy  ( L" > "  )->plo , &res->plo ,
        NULL } ;
      StackClusterPrivate_ExceptionSet(stp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    }
    return  ;
  } // if indp->re>vs
  if  ( me == vp ) {
    // me=[0..*indp-1,*indp,*indp+1..s-1] -> [*indp,..,end-1]
    if  ( FractionIs0(Complex_CRe  (
      PLOComplexToComplexConst(indp) ) ) )
      // me=[0..*indp-1,*indp,*indp+1..s-1] -> [0,..,end-1]
      return  ;
    size_t  ind2  ;
    if  (  FractionIsBadSizeValue  (
      Complex_CRe  (PLOComplexToComplexConst(indp) ) , & ind2  ) ) {
      ParaLispObject  * list [ ] = {
        &  indp->plo  , &  PLOString_NewCopy ( L" is bad"  )->plo , NULL } ;
      StackClusterPrivate_ExceptionSet  ( stp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return ; 
    }    
    if  ( FractionMoreEqUSize(Complex_CRe  ( PLOComplexToComplexConst(indp) ),  vvs)  ) {
      PLOString_Clear ( me ) ;
      return ; 
    }
    size_t const   newsize = vvs - ind2  ;
    wchar_t volatile * jtarget = String_Begin (  WideChar ) ( PLOString_Value2 ( me ) ) ;
    // ! .begin() long calls twice
    wchar_t volatile * jsource = jtarget + size_cast_ptrdiff ( ind2  ) ;
    do  {
      ( *  jtarget  )  = (  *  jsource  ) ;
      ++  jtarget ;
      ++  jsource ;
      ++  ind2  ;
    } while ( ind2 <   vvs ) ;
    PLOString_Resize  ( me , newsize ) ;
    return  ; 
  } // if me == vp
  size_t  ind2  ;
  if  (  FractionIsBadSizeValue  (
    Complex_CRe  (PLOComplexToComplexConst(indp) ) , &ind2  ) ) {
    ParaLispObject  * list [ ] = {
      &  indp->plo  , &  PLOString_NewCopy  ( L" is bad"  )->plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  ; 
  }
  PLOString_Clear ( me ) ;
  size_t const   newsize =  vvs  - ind2  ;  
  PLOString_ResizeLet  ( me,newsize , L' '  ) ; 
  wchar_t volatile * jtarget = String_Begin (  WideChar ) ( PLOString_Value2 ( me ) ) ;
  wchar_t volatile * jsource = String_Begin (  WideChar ) ( PLOString_Value2 ( vp ) ) +
    size_cast_ptrdiff ( ind2  ) ;
  while ( ind2 <   vvs  ) {
    ( *  jtarget  )  = (  *  jsource  ) ;
    ++  jtarget ;
    ++  jsource ;
    ++  ind2  ; 
  } 
}

void PLOString_Resize  ( PLOStringPub * const me , size_t const s  ) {
  PLOStringPri * const mep = StringPub_cast_StringPri ( me ) ;
  StringResize  ( WideChar  ) ( & mep -> value , s , L'\00'  ) ;
}

void  PLOString_Set ( PLOStringPub * const me , PLOStringPub const * const xp ) {
  if  ( me not_eq  xp ) {
    PLOStringPri * const mep = StringPub_cast_StringPri ( me ) ;
    PLOStringPri const * const x = StringPub_cast_StringPriConst ( xp ) ;
    StringSet ( WideChar  ) ( & mep -> value , & x -> value ) ;
  }
}

void  PLOString_Swap  ( PLOStringPub * const me , PLOStringPub * const xp  ) {
  if  ( me not_eq  xp ) {
    PLOStringPri * const mep = StringPub_cast_StringPri ( me ) ;
    PLOStringPri * const x = StringPub_cast_StringPri ( xp ) ;
    StringSwap  ( WideChar  ) ( & mep -> value , & x -> value ) ;
  }
}

void  PLOString_InitWChar ( PLOStringPri * const me , wchar_t const c ) {
  String_ConstrFillChar ( WideChar  ) ( & me -> value , 1 , c , true ) ;
  VirtualTypePointer vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOString } ;
  ParaLispObject_InitVTP  ( & me -> pub . plo , vtp ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOString_constrstatInc ( ) ;
# endif
}

bool  PLOString_LessEq  ( PLOStringPub  const * const xp  , PLOStringPub const * const yp ) {
  return not  PLOString_Less( yp , xp ) ;
}

bool  PLOString_MoreEq ( PLOStringPub  const * const xp  , PLOStringPub const * const yp ) {
  return not  PLOString_Less( xp  , yp  ) ;
}

void  PLOString_Replace ( struct  sPLOString_Replace const * const a ) {
  PLOStringPri * const mep = StringPub_cast_StringPri ( a -> me ) ;
  PLOStringPri const * const strp = StringPub_cast_StringPriConst ( a -> str ) ;
  StringReplace  ( WideChar ) ( & mep -> value , a -> index , a -> length  , & strp -> value ) ;
}
# include "widestring/inline.h"
