// Threads for all processors
// Thread for console task: getch
// ©2007-23 by Alex Glebe

# include <wchar.h>
# include <errno.h>
# include <stdio.h>
# include "threadc.h"

RunningPthread  *  runpthread = NULL ;
# include "define.h"
# ifdef LISP2D_CONSTRSTAT
_Atomic ( size_t  ) RunningPthreadconstrstat = 0 ;
# endif

void  RunningPthreadInit ( RunningPthread * const me ,
  void * ( * const f ) ( void  *  ) , void  * const arg ) {
  { int const e = pthread_create(&me->pth, 0,f, arg);
    if(e){
      wchar_t const * s ;
      switch(e){
      case  EAGAIN  :
        s = L"pthread_create:Insufficient resources to create another thread. "
        L"system-imposed limit on the number of threads was encountered. "
        L"There are a number of limits that may trigger this error: "
        L"the RLIMIT_NPROC soft resource limit (set via setrlimit(2)), "
        L"which limits the number of processes and  threads for  a  real  user  ID, "
        L"was  reached;  the  kernel's  system-wide  limit  on  the  number "
        L"of processes and threads, /proc/sys/kernel/threads-max, was reached "
        L"(see proc(5)); or the maximum number of PIDs, /proc/sys/kernel/pid_max, "
        L"was reached (see proc(5))." ;
        break;
      case  EINVAL  :
        s = L"pthread_create:Invalid settings in attr.";
        break;
      case  EPERM :
        s = L"pthread_create:No permission to set the scheduling policy and"
          L" parameters specified in attr." ;
        break;
      default: s = L"pthread_create:?";
      }
      fputws(s,stderr);
    }
  }
# ifdef  LISP2D_CONSTRSTAT
  ++  RunningPthreadconstrstat ;
# endif
}

void  RunningPthreadDestr  ( RunningPthread * const me ) {
  int const e = pthread_join(me -> pth, 0) ;
  if ( e ) {
    wchar_t  const  * s ;
    switch(e){
    case  EDEADLK:
      s = L"pthread_join:A deadlock was detected (e.g., two threads tried to join "
      L"with each other); or thread specifies the calling thread." ;
      break;
    case  EINVAL:
      return; // was not joinable
      /*s = L"pthread_join:thread is not a joinable thread."
       L"/Another thread is already waiting to join with this thread."
      break;*/
    case  ESRCH:
      s = L"pthread_join:No thread with the ID thread could be found.";
      break;
    default:
      s = L"pthread_join:?";
    }
    fputws(s,stderr);
  }
}
    
# ifdef LISP2D_CONSTRSTAT
size_t  RunningPthreadStat  ( void ) {
  return  RunningPthreadconstrstat ;
}
# endif

void RunningPthreadSetArray ( RunningPthread x [ const ] ) {
  runpthread = x ;
}

RunningPthread (*RunningPthreadArray ( void ))[] {
  return (RunningPthread (*)[])runpthread ;
}

# include "stcl/classc.h"
# include "define.h"
# include "stcl/exceptiontryset.h"
# include "stcl/public.h"
# include "stcl/inline.h"
# include "inner-history/inline.h"

// if was exception -> don't touch returns false else -> set new returns true
void  ExceptionTrySetPs ( tExceptionTrySetP * const ar  ) {
  jmp_buf jb0 ;
  jmp_buf * oldjb0 = Lisp2DJBglobal  ;
  Lisp2DJBglobal = & jb0 ;
  int const volatile jbn0 = setjmp  ( jb0  ) ;
  if  ( jbn0 ) {
    Lisp2DJBglobal = oldjb0 ;
    fwprintf  ( stderr , L"ExceptionTrySet ( tExceptionTrySetP * const ar  )"
      L":Lisp2DMutexLock ( & ( stclpr  . mutexExceptionC ) ):crash error=%d" , jbn0 ) ;
    return  ;
  }
  Lisp2DMutexLock ( StackClusterPrivate_mutexExceptionC  ( ar -> st  ) ) ;
  Lisp2DJBglobal = oldjb0 ;
  jmp_buf jb ;
  oldjb0 = Lisp2DJBglobal  ;
  Lisp2DJBglobal = &  jb ;
  int const volatile jbn = setjmp  ( jb  ) ;
  if  ( jbn  ) {
    Lisp2DMutexUnLock ( StackClusterPrivate_mutexExceptionC  ( ar -> st  ) ) ;
    fwprintf  ( stderr ,
      L"ExceptionTrySet ( tExceptionTrySetP * const ar  ):crash error=%d" , jbn ) ;
    Lisp2DJBglobal = oldjb0 ;
    return  ;
  }
  if  ( not  StackClusterPrivate_exceptionnotprocessed ( ar -> st  ) ) {
# ifdef  LISP2D_INNERHISTORY
    static  wchar_t const  Lisp2Dwsih_except1 [ ] =
      L"ExceptionTrySet( tExceptionTrySetP * const ar  ):"
      L"not  exceptionnotprocessed" ;
    Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_except1  ) ;
# endif
    StackClusterPrivate_setexceptionvalue ( ar -> st  , ar  ->  xp  ) ;
    StackClusterPrivate_setexceptionhistory ( ar -> st  , ar  ->  hp ) ;
    StackClusterPrivate_setexceptionnotprocessed  ( ar -> st  , true ) ;
    Lisp2DMutexUnLock ( StackClusterPrivate_mutexExceptionC  ( ar -> st  ) ) ;
    Lisp2DJBglobal = oldjb0 ;
    return  ;
  }
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t const  Lisp2Dwsih_except2 [ ] =
    L"ExceptionTrySet( tExceptionTrySetP * const ar  ):"
    L"exceptionnotprocessed" ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_except2  ) ;
# endif
  Lisp2DMutexUnLock ( StackClusterPrivate_mutexExceptionC  ( ar -> st  ) ) ;
  Lisp2DJBglobal = oldjb0 ;
}

# include <sys/wait.h>  // waitpid
# include "string/public.h"
# include "string/struct.h"
# include "plocomplex/public.h"
# include "sharedc.h"
# include "shared/inline.h"
# include "thread.h"
# include "passage/public.h"
# include "cons/inline.h"
# include "nilc.h"
# include "cons/struct.h"
# include "stack/private.h"
# include "mutex2/public.h"

// ... dismissed ... must to do: PMU(stclp->mutexTask)
// ! to remove mutex & condition   , use promise               
void * pthSystemTask ( void * const stclp0 ) {
  StackClusterPrivate  * const stclp  = stclp0  ;
  PLOComplex * cpid ;
  { ParaLispObject * tmp  ;
    StackClusterPrivate_Pop ( stclp , & tmp ) ; 
    cpid = ParaLispObjectToPLOComplex ( tmp ) ; 
  }
  int pid ;
  if  ( Complex_IsBADIntValue ( PLOComplexToComplexConst(cpid) , &pid ) ) {
    StackClusterPrivate_ExceptionSet  ( stclp ,
      & PLOString_NewCopy (
        L"pthSystemTask2:Complex_IsBADIntValue" ) ->  plo ) ;
    return 0 ; 
  }
  jmp_buf jb ;
  jmp_buf * const oldjb0 = Lisp2DJBglobal  ;
  Lisp2DJBglobal = &  jb ;
  int const volatile jbn = setjmp  ( jb  ) ;
  if  ( jbn ) {
    Lisp2DJBglobal = oldjb0 ;
    //OutOfMemory ( stclp  ) ;
    //StackClusterPrivate_ExceptionSet  ( stclp , & plonil . plo ) ;
    fwprintf  ( stderr ,
      L"pthSystemTask2( StackClusterPrivate  * stclp ,int const  pid)"
      L":Lisp2DInnerHistoryPush:crash error=%d" , jbn ) ;
    return 0 ;
  }
  Lisp2DThreadStack = malloc ( Lisp2DThreadStackSIZE ) ;
  if ( not Lisp2DThreadStack ) {
    OutOfMemory ( stclp  ) ;
    return 0 ;
  }
    
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t  const  Lisp2Dwsih_pthsystemtask[] = L"void  pthSystemTask2( StackClusterPrivate  * ,int):start" ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_pthsystemtask  ) ;
# endif
  {
Again : ;
    int status  = 0x55555555  ;
    int const waitres = waitpid ( pid , & status  , 0 ) ;
    PassagePassIn ( passageGCCPub ) ;
    if  ( waitres ==  -1  ) {
      PLOStringPub * const newstr  = PLOString_NewCopy ( L"pthSystemTask:(waitpid(pid,& status,0)  ==  -1)" ) ;
      StackClusterPrivate_ExceptionSet  ( stclp , &  newstr ->  plo  ) ;
    } else {
      int is  = -1  ;
      if  ( WIFEXITED ( status  ) ) {
        is  = WEXITSTATUS ( status  ) ;
        PLOComplex  * const  s  = NewComplexBareExc ( ) ;
        Complex_SetInt  ( PLOComplexToComplex ( s ) , is  ) ;
        StackClusterPrivate_SetResult ( stclp , & s ->  plo ) ;
      } else  if  ( WIFSIGNALED ( status  ) ) {
        is  = WTERMSIG  ( status  ) ;
        PLOComplex  * const  s  = NewComplexBareExc ( ) ;
        Complex_SetInt  ( PLOComplexToComplex ( s ) , is  ) ;
        PLOCons * const res2 = NewCons ( & s -> plo , & plonil  . plo ) ;
        PLOCons * const res = NewCons ( & plossignal  . plo , & res2 -> plo ) ;
        StackClusterPrivate_SetResult ( stclp , & res ->  plo ) ;
      } else  if  ( WIFSTOPPED  ( status  ) ) {
        /*is  = WSTOPSIG  ( status  ) ;
        PLOComplex  * const  s  = NewComplexBareExc ( ) ;
        Complex_SetInt  ( PLOComplexToComplex ( s ) , is  ) ;
        StackClusterPrivate_SetResult ( stclp , & s ->  plo ) ;*/
        PassagePassOut ( passageGCCPub ) ;
        goto  Again ;
      }
/*
fwprintf  ( stderr  , L"waitpid : status = %d\n"  , status  ) ; // 134
fwprintf  ( stderr  , L"waitpid : waitres = %d\n" , waitres ) ;
fwprintf  ( stderr  , L"is = %d\n"  , is  ) ;
fwprintf  ( stderr  , L"WIFEXITED = %d\n"  , WIFEXITED ( status  ) ? 1 : 0 ) ;
fwprintf  ( stderr  , L"WIFSIGNALED = %d\n"  , WIFSIGNALED ( status  ) ? 1 : 0 ) ;
fwprintf  ( stderr  , L"WIFSTOPPED = %d\n"  , WIFSTOPPED ( status  ) ? 1 : 0 ) ;
*/
    }
    PassagePassOut ( passageGCCPub ) ;
  }  // status
// !? to find place for lock protection if GarbageCollector runs
// !? mutexGC.UnLock();      
  StackClusterPrivate_setTaskForManager ( stclp ,  TaskRun ) ;
  TaskQueuePush ( stclp ) ;
  Mutex2Lock  ( & Lisp2Dmutex2ManagerTasks ) ;
  if ( flagManagerWaitSignal ) {
    flagManagerWaitSignal = false ;
    jmp_buf jb2 ;
    jmp_buf * oldjb = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jb2 ;
    int const volatile jbn2 = setjmp  ( jb2  ) ;
    if  ( jbn2  ) {
      Lisp2DJBglobal  = oldjb ;
      Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
      longjmp ( * Lisp2DJBglobal  , jbn2 ) ;
    }
    ConditionSignal ( & Lisp2DsignalManager ) ;
    Lisp2DJBglobal  = oldjb ;
  }
  // nobody didnot signaling as flagManagerWaitSignal=false
  // manager will little wait
  countExtraThreads2_Dec  ( ) ;
  Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
# ifdef  LISP2D_INNERHISTORY
  static  wchar_t  const  Lisp2Dwsih_pthsystemtask2[] = 
    L"void  pthSystemTask2( StackClusterPrivate  * ,int):return" ;
  Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_pthsystemtask2  )
    ;
# endif    
  Lisp2DJBglobal = oldjb0 ;
  free(Lisp2DThreadStack);
  return  0 ;
}
  
# include "gc.h"

# ifdef  LISP2D_DEBUG  
# define  isOKSetToBeInGarbageAndClear  Lisp2DisOKSetToBeInGarbageAndClear9fSb
static  bool  isOKSetToBeInGarbageAndClear  ( StackClusterPrivate * const stclpp ) {
  if  ( StackClusterPrivate_wanttobeingarbage ( stclpp  ) ) 
    return  false ;
  if  ( StackClusterIsBADClearBeforeGarbage  ( stclpp ) ) 
    return false ;
  StackClusterPrivate_setwanttobeingarbage  ( stclpp , true ) ;
  countStackClusterWantToBeInGarbage_Inc ( ) ;
  return true ;
}
# else

# define  SetToBeInGarbageAndClear  Lisp2DSetToBeInGarbageAndClear9fSb
static  void  SetToBeInGarbageAndClear  ( StackClusterPrivate * const stclpp ) {
  if  ( Lisp2DStackClusterIsBADClearBeforeGarbage  ( stclpp ) ) 
    return ;
  StackClusterPrivate_setwanttobeingarbage  ( stclpp  , true ) ;
  countStackClusterWantToBeInGarbage_Inc ( ) ;
}
# endif
  
# include "lock/public.h"
# include "signall.h"
# include "nilc.h"
# include "window/public.h"
  
void * pthEval (  void * const  rpt  ) {
  RunningPthArgs * const rpa = rpt ;
  jmp_buf jb ;
  jmp_buf * const oldjb0 = Lisp2DJBglobal  ;
  Lisp2DJBglobal = &  jb ;
  int const volatile jbn = setjmp  ( jb  ) ;
  if  ( jbn ) {
    Lisp2DJBglobal = oldjb0 ;
    
// !! longjmp generate Real CRASH ??
    
    switch ( jbn ) {
    case  Lisp2DExcTypeOutOfMemory :
      fwprintf ( stderr , L"pthEval:Out of memory = \"%ls\"\n" ,
        Lisp2DExcOutOfMemoryGlobal . value ) ;
      break ;
    case  Lisp2DExcTypeStack  :
      fwprintf ( stderr , L"pthEval:Stack crash = \"%ls\"\n" ,
        Lisp2DExcStackGlobal  . value ) ;
      break ;
    default :
      fwprintf  ( stderr , L"pthEval(void * rpt):crash error=%d\n" , jbn ) ;
    }
    return 0 ;
  }
  Lisp2DThreadStack = malloc ( Lisp2DThreadStackSIZE ) ;
  if (  not Lisp2DThreadStack ) {
    Lisp2DExcOutOfMemoryGlobal . value = 
      L"pthEval2:Lisp2DThreadStack = malloc ( Lisp2DThreadStackSIZE )";
    longjmp ( * Lisp2DJBglobal ,  Lisp2DExcTypeOutOfMemory ) ;
  }
  
  // ? A new thread inherits a copy of its creator's signal mask.
  // ? to change the action 
  
  StackClusterPrivate  * stclp  ;
  
  Mutex2Lock  ( & Lisp2Dmutex2ManagerTasks ) ;
  do {
    if  ( ! ( rpa ->  taskp ) ) {
TryAgain:
      rpa -> flagWaitSignal = true ;
      if ( flagManagerWaitSignal ) {
        flagManagerWaitSignal = false ;
        jmp_buf jb2 ;
        jmp_buf * oldjb = Lisp2DJBglobal  ;
        Lisp2DJBglobal  = & jb2 ;
        int const volatile jbn2 = setjmp  ( jb2  ) ;
        if  ( jbn2  ) {
          Lisp2DJBglobal  = oldjb ;
          Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
          longjmp ( * Lisp2DJBglobal  , jbn2 ) ;
        }
        ConditionSignal ( & Lisp2DsignalManager ) ;
        Lisp2DJBglobal  = oldjb ;
      }
      // situation local thread.wait=true &
      //  manager.wait=false - must be bringed in locked position       
      int const jbn2 = pthread_cond_wait ( & rpa ->
        condrunning2 , Mutex2Addr  ( & Lisp2Dmutex2ManagerTasks ) ) ;
      if ( jbn2 ) {
        fwprintf  ( stderr ,
          L"Manager:1.pthread_cond_wait ( & rpa -> privat. condrunning2,"
          L"Lisp2DMutexAddr ( & mutexManagerTasks ) ) = %d" , jbn2 ) ;
        Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"Manager:1.pthread_cond_wait ( & rpa -> privat. condrunning2" ;
        longjmp  ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
      }
      // task may be 0
    } // if (task == nullptr)
    if(flagThreadsShutDown){
      if ( flagManagerWaitSignal ) {
        flagManagerWaitSignal = false ;
        jmp_buf jb2 ;
        jmp_buf * const oldjb = Lisp2DJBglobal  ;
        Lisp2DJBglobal  = & jb2 ;
        int const volatile jbn2 = setjmp  ( jb2  ) ;
        if  ( jbn2  ) {
          Lisp2DJBglobal  = oldjb ;
          Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
          longjmp ( * Lisp2DJBglobal  , jbn2 ) ;
        }
        ConditionSignal ( & Lisp2DsignalManager ) ;
        Lisp2DJBglobal  = oldjb ;
        // nobody didnot signaling as flagManagerWaitSignal=false
        // manager will little wait
      }
      
      Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;

      pthread_mutex_lock(& rpa ->ready_mutex);
      rpa ->promise_state = true  ;
      pthread_cond_signal(& rpa ->ready_cond);
      pthread_mutex_unlock(& rpa ->ready_mutex);
      
      free(Lisp2DThreadStack);
      return 0 ;
    }
    // task may be nullptr
    if(!(rpa->taskp ))
      goto  TryAgain;

    Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
TaskRunning : ;
    TaskForThread tft = TaskForThread_Analyse  ;
# ifdef LISP2D_DEBUG        
    rpa ->  debugStartEval = true ;
# endif       
// ! to make: void Eval() -> enum Eval()
// ! OR                   -> void Eval(RunningPthArgs  * rpt)
// ! to return different value to pthEval2()
// ! if StackCluster in Queue was again twice:
// ! first Thread: make extraThread, extraThread Push(stclp) in Queue
// ! second Thread: stclp processed and make TaskForManagerToQuit or Run or Else
// ! first Thread: MUST to IGNORE stclp.TaskForManager and
//      get NEXT task from the queue
    tft = StackClusterPrivate_Eval2 ( rpa -> taskp ) ;
# ifdef LISP2D_DEBUG          
    rpa ->  debugStartEval = false  ;
# endif
    if  ( flagThreadsShutDown ) {
# ifdef  LISP2D_INNERHISTORY
      static  wchar_t  const Lisp2Dwsih_ptheval10[] = 
        L"pthEval2(void *  rpt):after Eval:if(flagThreadsShutDown)" ;
      Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_ptheval10  ) ;
# endif                
      Mutex2Lock ( & Lisp2Dmutex2ManagerTasks ) ;
      if  ( flagManagerWaitSignal ) {
        flagManagerWaitSignal = false ;
        jmp_buf jb2 ;
        jmp_buf * const oldjb = Lisp2DJBglobal  ;
        Lisp2DJBglobal  = & jb2 ;
        int const volatile jbn2 = setjmp  ( jb2  ) ;
        if  ( jbn2  ) {
          Lisp2DJBglobal  = oldjb ;
          Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
          longjmp ( * Lisp2DJBglobal  , jbn2 ) ;
        }
        ConditionSignal ( & Lisp2DsignalManager ) ;
        Lisp2DJBglobal  = oldjb ; 
      }
      Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;

      pthread_mutex_lock(& rpa ->ready_mutex);
      rpa ->promise_state = true  ;
      pthread_cond_signal(& rpa ->ready_cond);
      pthread_mutex_unlock(& rpa ->ready_mutex);
      
      free(Lisp2DThreadStack);
      return 0 ;
    } // flagThreadsShutDown
    //  TaskRun,TaskTerminate,TaskWaitSonsAlmost,
    // TaskGetChar,TaskWinGetChar,TaskWaitUnLock
// ! to rewrite: mutexTask -> mutexSons

    if  ( tft  ==  TaskForThread_Analyse ) {
      stclp  =  rpa -> taskp  ;
      
      Lisp2DMutexLock ( StackClusterPrivate_mutexTask  ( stclp ) ) ;
# ifdef LISP2D_DEBUG
# ifdef  LISP2D_INNERHISTORY
      if  ( StackClusterPrivate_exceptionnotprocessed  ( stclp ) ) {
        static  wchar_t  const Lisp2Dwsih_ptheval12[] =
          L"pthEval2(void *  rpt):170:Exception exist" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory ,
          Lisp2Dwsih_ptheval12  ) ; 
      }
# endif            
# endif  
      switch  ( StackClusterPrivate_TaskForManager  ( stclp ) ) {
      case  TaskWaitSonsAlmost  : ;
# ifdef  LISP2D_INNERHISTORY
        static  wchar_t  const Lisp2Dwsih_ptheval2[] =
          L"pthEval2(void *  rpt):TaskWaitSonsAlmost" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory , Lisp2Dwsih_ptheval2 ) ;
# endif                
        if  ( StackClusterPrivate_sonscount ( stclp ) ) {
          StackClusterPrivate_setTaskForManager ( stclp , TaskWaitSons ) ;
          Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  (
            stclp ) ) ;
        } else  {
          StackClusterPrivate_setTaskForManager ( stclp , TaskRun ) ;
          Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( stclp ) ) ;
          // trying to be faster: don't touch queueStCl
          // pause to check GC
          // if we forget -> get trouble
          if(flagStopToGC or  Lisp2DTimeToGC()){
            // don't forget task
            TaskQueuePush ( stclp ) ;
            goto  SignalToManager;
          }
          goto  TaskRunning;
        } // stclp->sonscount  ==  0
        break;
      case  TaskPause:
        StackClusterPrivate_setTaskForManager ( stclp , TaskRun ) ;
        Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( stclp ) ) ;
        TaskQueuePush ( stclp ) ;
        // don't forget task
        if(flagStopToGC or  Lisp2DTimeToGC())
          goto  SignalToManager;
        // if locked queue then evaluate current stclp
        // else task:=(pop new) and push stclp to the end
        if  ( not  isOKTaskQueuePopPrivate ( & rpa ->  taskp  ) )
          goto  SignalToManager ; 
        goto  TaskRunning ;
# ifndef  LISP2D_DONTUSEX11
      case  TaskWinWaitEvent:
        if  ( PLOWindow_isBADEventQueueLockPop  ( ParaLispObjectToPLOWindow (
              StackClusterPrivate_ThisTop ( stclp ) ) , stclp ) ) {
          // event to found, thread transfer to new task
          Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( stclp ) ) ;
          break;
        }
        // event found, task continue
        StackClusterPrivate_setTaskForManager ( stclp , TaskRun ) ;
        Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( stclp ) ) ;
        // if locked queue then evaluate current stclp
        // else task:=(pop new) and push stclp to the end
        TaskQueuePush ( stclp ) ;
        if  ( isOKTaskQueuePopPrivate  ( & rpa->taskp ) )
          goto  TaskRunning ; 
        goto  SignalToManager ; 
# endif        
      case  TaskTerminate :
        Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( stclp ) ) ;
        //here we must delet_e it from fathers sonslist
        // locked with mutexBoss by another task
        { StackClusterPrivate  * const  fathp  = StackClusterPrivate_scFatherp ( stclp ) ;
          if  ( fathp ) {
// ! to rewrite: mutexTask -> mutexSons
            Lisp2DMutexLock ( StackClusterPrivate_mutexTask  ( fathp ) ) ;
            // atomic
            StackClusterPrivate_sonscountdec  ( fathp ) ;
            // no difference, mutexSons not used -> to dismiss
            if  ( StackClusterPrivate_exceptionnotprocessed  ( stclp ) ) {
              tExceptionTrySetP ar = {
                . st = fathp ,
                . xp = StackClusterPrivate_exceptionvalue ( stclp ) ,
                . hp = StackClusterPrivate_exceptionhistory ( stclp ) ,
              } ;
              // protection from triple work
              ExceptionTrySetPs ( & ar ) ;
            }
          if  ( not StackClusterPrivate_sonscount ( fathp ) ) {
            // was last
            if  ( StackClusterPrivate_TaskForManager  ( fathp )  ==  TaskWaitSons  ) {
              StackClusterPrivate_setTaskForManager ( fathp , TaskRun ) ;
              if  ( flagStopToGC or  Lisp2DTimeToGC ( ) ) {
                Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( fathp ) ) ;
                // don't forget task
                TaskQueuePush ( fathp ) ;
# ifdef  LISP2D_DEBUG        
                if  ( not  (  isOKSetToBeInGarbageAndClear  ( stclp ) ) ) {
                  pthread_mutex_lock( & rpa ->ready_mutex);
                  rpa ->promise_state = true  ;
                  rpa ->promise_exc = true  ;
                  pthread_cond_signal(& rpa ->ready_cond);
                  pthread_mutex_unlock(& rpa ->ready_mutex);
                  free(Lisp2DThreadStack);
                  return 0 ;
                }
# else
                SetToBeInGarbageAndClear  ( stclp ) ;
# endif                    
                goto  SignalToManager ;
              }
# ifdef  LISP2D_DEBUG        
              if  ( not  (  isOKSetToBeInGarbageAndClear  ( stclp ) ) ) {
                pthread_mutex_lock(& rpa ->ready_mutex);
                rpa ->promise_state = true  ;
                rpa ->promise_exc = true  ;
                pthread_cond_signal(& rpa ->ready_cond);
                pthread_mutex_unlock(& rpa ->ready_mutex);
                  
                  free(Lisp2DThreadStack);
                  return 0 ;
              }
# else
              SetToBeInGarbageAndClear  ( stclp ) ;
# endif                                        
              // if locked queue then evaluate father yourself
              // else task:=(pop new) and push father to the end
              
              
              Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( fathp ) ) ;
              TaskQueuePush ( fathp ) ;
              if  ( isOKTaskQueuePopPrivate  ( & rpa -> taskp ) )
                goto  TaskRunning ; 
              goto  SignalToManager ;
            } // if wait sons
              
          } //  if fath->sonscount  ==  0
          Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( fathp ) ) ;
        } // if fathp
          
        } // if(fath)
# ifdef  LISP2D_DEBUG        
        if  ( not  (  isOKSetToBeInGarbageAndClear  ( stclp ) ) ) {
          pthread_mutex_lock(& rpa ->ready_mutex);
          rpa ->promise_state = true  ;
          rpa ->promise_exc = true  ;
          pthread_cond_signal(& rpa ->ready_cond);
          pthread_mutex_unlock(& rpa ->ready_mutex);
          
          free(Lisp2DThreadStack);
          return 0 ;
        }
# else
        SetToBeInGarbageAndClear  ( stclp ) ;
# endif          
        break ;
      case  TaskWaitUnLock2:
        Lisp2DMutexUnLock ( StackClusterPrivate_mutexTask  ( stclp ) ) ;
        { ParaLispObject  * lock  = & plonil  . plo ;
          StackClusterPrivate_Pop ( stclp,&lock);
          if(IsNotLock(stclp , lock))  
            break;
          if  ( PLOLock_isOKTryLock3 (  ParaLispObjectToPLOLock ( lock  ) ,
            stclp ) ) {
            TaskQueuePush ( stclp ) ;
            // don't forget task
            if(flagStopToGC or  Lisp2DTimeToGC())
              goto  SignalToManager;
            // trying to replace task to another, else calculate this stclp  
            if(isOKTaskQueuePopPrivate(&rpa->taskp))
              goto  TaskRunning; 
            goto  SignalToManager;
          } // if(trylock3)
        } // lock
        break;
      case  TaskWaitSignal: ;
# ifdef  LISP2D_INNERHISTORY
        static  wchar_t  const Lisp2Dwsih_ptheval3[] =
          L"pthEval2(void *  rpt):TaskWaitSignal" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory ,
          Lisp2Dwsih_ptheval3  ) ;
# endif                        
        Lisp2DMutexUnLock (  StackClusterPrivate_mutexTask ( stclp ) ) ;
        { ParaLispObject  * lock3 = & plonil  . plo ;
          StackClusterPrivate_Pop(stclp,&lock3);
          ParaLispObject  * signal2 = & plonil  . plo ;
          StackClusterPrivate_Pop(stclp,&signal2);
          if  ( IsNotSignal ( stclp , signal2 ) ) {
            if(Lockp( lock3)){
              if(PLOLock_isBADUnLock(ParaLispObjectToPLOLock(   lock3)))
                StackClusterPrivate_ExceptionSet(stclp,
                  &PLOString_NewCopy(L"Lock is not locked")->plo);
            }
            break;
          }
          if  ( PLOSignal_isOkGetSignal (
              ParaLispObjectToPLOSignal ( signal2 ) , stclp ) ) {
            StackClusterPrivate_setTaskForManager ( stclp , TaskRun ) ;
            TaskQueuePush ( stclp ) ;
            if(flagStopToGC or  Lisp2DTimeToGC()){
              // don't forget task
              if(Lockp( lock3)){
                if(PLOLock_isBADUnLock(ParaLispObjectToPLOLock(   lock3)))
                  StackClusterPrivate_ExceptionSet(stclp,
                    &PLOString_NewCopy(L"Lock is not locked")->plo);
              }
              goto  SignalToManager;
            }
            // trying to replace task to another, else calculate this stclp  
            if(Lockp( lock3)){
              if(PLOLock_isBADUnLock(ParaLispObjectToPLOLock(   lock3)))
                StackClusterPrivate_ExceptionSet(stclp,
                  &PLOString_NewCopy(L"Lock is not locked")->plo);
            }
            if(isOKTaskQueuePopPrivate(&rpa->taskp))
              goto  TaskRunning; 
            goto  SignalToManager;
          } // if GetSignal
          if(Lockp( lock3)){
            if(PLOLock_isBADUnLock(ParaLispObjectToPLOLock(   lock3)))
              StackClusterPrivate_ExceptionSet(stclp,
                &PLOString_NewCopy(L"Lock is not locked")->plo);
          }
        } // lock2
        break;
      case  TaskWaitSystemTask: ;
# ifdef  LISP2D_INNERHISTORY
        static  wchar_t  const Lisp2Dwsih_ptheval5[] =          L"pthEval2(void *  rpt):TaskWaitSystemTask" ;
        Lisp2DInnerHistoryPush  ( & Lisp2DinnerHistory ,          Lisp2Dwsih_ptheval5  ) ;
# endif                           
        Lisp2DMutexUnLock (  StackClusterPrivate_mutexTask ( stclp ) ) ;
        // join not needed
        {              
// ! to remove mutex & condition , use promise                             
              // links,links...
              pthread_t  tc ;
              switch ( pthread_create ( & tc , (pthread_attr_t const *)0,
                 & pthSystemTask , stclp ) ) {
              case  0 :
                goto  Detach  ;
              case  EAGAIN  :
                StackClusterPrivate_ExceptionSet  ( stclp , & plosagain . plo ) ;
                break ;
              case  EINVAL  :
                StackClusterPrivate_ExceptionSet  ( stclp , & plosinval . plo ) ;
                break ;
              case  EPERM :
                StackClusterPrivate_ExceptionSet  ( stclp , & plosperm  . plo ) ;
                break ;
              default :
                StackClusterPrivate_ExceptionSet  ( stclp , & plonil . plo ) ;
              }
              break ;
Detach :
              countExtraThreads2_Inc  ( ) ;
              switch  ( pthread_detach ( tc ) ) {
              case  0 :
                break ;
              case  EINVAL  :
                StackClusterPrivate_ExceptionSet  ( stclp , & plosinval . plo ) ;
                break ;
              case  ESRCH :
                StackClusterPrivate_ExceptionSet  ( stclp , & plossrch . plo ) ;
                break ;
              default :
                StackClusterPrivate_ExceptionSet  ( stclp , & plonil . plo ) ;
              } // switch detach
        } // tc
        break;
      case   TaskRun :
        Lisp2DMutexUnLock (  StackClusterPrivate_mutexTask ( stclp ) ) ;
        // trying to be faster: don't touch queueStCl
        // BIM(StackCluster,CreateTask:queueStCl.push(this);) n=3860264 fulltime=15.2
        // avrtime=3.95e-06 -> n=3862030 fulltime=6.13 avrtime=1.59e-06
        // BIM(StackCluster,CreateTask)                       n=3860264 fulltime=24.6
        // avrtime=6.37e-06 -> n=3862030 fulltime=15.6 avrtime=4.04e-06
        // mutexqueueStCl2= 25249671/31633643=79.8% -> 9289910/10275929=90.4%
        // simple successive cycles is VERY fast
        // task processing now is NOT in pure horizontal (parallel) manner
        // TaskPause works in old style (as must to be)
        // GlobalListOfObjects becomes very popular
        // GlobalListOfObjects= 24000389/25785197=93.1% -> 22785506/25719221=88.6%
        // pause to check GC
        // if we forget -> get trouble
        if(flagStopToGC or  Lisp2DTimeToGC()){
          // don't forget task
          TaskQueuePush ( stclp ) ;
          goto  SignalToManager;
        }
        goto  TaskRunning;
      case  TaskWaitSons:
        pthread_mutex_lock(& rpa ->ready_mutex);
        rpa ->promise_state = true  ;
        rpa ->promise_exc = true  ;
        pthread_cond_signal(& rpa ->ready_cond);
        pthread_mutex_unlock(& rpa ->ready_mutex);
      
          free(Lisp2DThreadStack);
          return 0 ; 
      case  TaskWaitExtraThread:
        pthread_mutex_lock(& rpa ->ready_mutex);
        rpa ->promise_state = true  ;
        rpa ->promise_exc = true  ;
        pthread_cond_signal(& rpa ->ready_cond);
        pthread_mutex_unlock(& rpa ->ready_mutex);
          free(Lisp2DThreadStack);
        return 0; 
      default :
        fwprintf  ( stderr  ,
          L"pthEval2:switch(stclp->TaskForManager2):default:unknown case:%i\n"
          , StackClusterPrivate_TaskForManager  ( stclp ) ) ;
          pthread_mutex_lock(& rpa ->ready_mutex);
          rpa ->promise_state = true  ;
          rpa ->promise_exc = true  ;
          pthread_cond_signal(& rpa ->ready_cond);
          pthread_mutex_unlock(& rpa -> ready_mutex);
          free(Lisp2DThreadStack);
          return 0 ;
      } // switch
      } // if(tft  ==  TaskForThread::Analyse)
      // pause to check GC
      // if we forget -> get trouble
      if(flagStopToGC or  Lisp2DTimeToGC()) 
        goto  SignalToManager;
      // get next task from the queue
      { StackClusterPrivate * scp ;
        if(isOKTaskQueuePop(&scp)){
          stclp = scp ;
          rpa->taskp = scp ;
          goto  TaskRunning ;
        }
      }
SignalToManager:
    Mutex2Lock  ( & Lisp2Dmutex2ManagerTasks ) ;
    rpa->taskp = (StackClusterPrivate *)0;
    runpthcount_Dec ( ) ;
  }while(not  flagThreadsShutDown);
  // manager is wicked about flagWaitSignal=false
  rpa->flagWaitSignal = true;
  pthread_mutex_lock(& rpa ->ready_mutex);
  rpa ->promise_state = true  ;
  pthread_cond_signal(& rpa ->ready_cond);
  pthread_mutex_unlock(& rpa ->ready_mutex);
  
  Mutex2Unlock  ( & Lisp2Dmutex2ManagerTasks ) ;
  Lisp2DJBglobal = oldjb0 ;
  free(Lisp2DThreadStack); 
  return 0 ;
}

# include "window/inline.h"
