// Lisp2D - Time type
// ©2008-23 by Alex Glebe
# include <stddef.h>
# include "time/public.h"
# include "define.h"

# ifdef  LISP2D_CONSTRSTAT

_Atomic ( size_t ) PLOTime_constrstat = 0 ;

void  PLOTime_constrstat_Inc  ( void  ) {
  ++  PLOTime_constrstat  ;
}

size_t  PLOTime_constrstat_Val  ( void  ) {
  return  PLOTime_constrstat  ;
}

# endif

# include "plo/struct.h"
# include "plo/public.h"

bool  Timep ( ParaLispObject const * const xp  ) {
  return ParaLispObject_Type  ( xp  ) ==  TypeGlobalListMemberPLOTime ;
}

IsNotTypeTemplBody  ( time , TypeGlobalListMemberPLOTime )

# include "time/public.h"

bool  PLOTime_staticisBADCopyPrivate  ( ParaLispObject const * const me , ParaLispObject  * * const resultp  ) {
  PLOTime * const res = aligned_alloc ( _Alignof ( PLOTime ) , sizeof ( PLOTime ) ) ;
  if ( not res )
    return  true  ;
  PLOTime_InitCopy  ( res , ParaLispObjectToPLOTimeConst  ( me  ) ) ;
  ParaLispObject_virtualPushAsNew ( & res -> pub . plo ) ;
  ( * resultp )  = & res -> pub . plo ;
  return  false ;
}

# include "symbolc.h"
# include "plostream/public.h"

void  PLOTime_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool  _ __attribute__ ((unused)) ,
  Vector ( StackPLOP )  * _2 __attribute__ ((unused)) ) {
  PLOTime const * const met = ParaLispObjectToPLOTimeConst(me);
  struct  tm  ptm;
  localtime_r ( & ( met -> tv . tv_sec  ) , & ptm ) ;
  { wchar_t const space = ' ' ;
    int   y2 = (int)(yearstart) + (ptm.tm_year);
    int   m2 = 1 + (ptm.tm_mon);
    long  double  const  t2 = 
      ( ( long double ) ( met -> tv . tv_nsec ) /
        (long double)(ulinanokoeff)) +
      (long  double)(ptm.tm_sec);
    PLOStreamPushWideCharArray  ( s ,
      ( PArray_wchar_c ) { (wchar_t const(*)[])&L"Time("})  ;
    PLOStreamPushInt(s,y2);
    PLOStreamPushWChar(s,  space);
    PLOStreamPushInt(s,m2);
    PLOStreamPushWChar(s,  space);
    PLOStreamPushInt(s,ptm.tm_mday);
    PLOStreamPushWChar(s,  space);
    PLOStreamPushInt(s, ptm.tm_hour);
    PLOStreamPushWChar(s,  space);
    PLOStreamPushInt(s, ptm.tm_min);
    PLOStreamPushWChar(s,  space) ;
    PLOStreamPushLDouble  ( s,  t2);
    PLOStreamPushWChar(s,  space);
    PLOStreamPushInt(s, ptm.tm_wday);
    PLOStreamPushWChar(s,  space);
    PLOStreamPushInt(s, ptm.tm_yday);
    PLOStreamPushWChar(s,  space);
  }
  if(ptm.tm_isdst)
    if(ptm.tm_isdst > 0)        
      PLOStreamPushPLO  ( s , & ( plostrue1 . plo ) ) ;  // ⊨ or true
    else       
      PLOStreamPushPLO(s ,&(  plosnil . plo));
  else      
    PLOStreamPushPLO  ( s , & ( plosfalse . plo ) ) ;
  PLOStreamPushWChar(s,  L')');
}

bool  PLOTime_Les ( PLOTime const * const me , PLOTime const * const xp ) {
  if  ( me ==  xp )
    return  false ;
  return ( me -> tv . tv_sec == ( xp -> tv . tv_sec  ) ) ?
    ( me -> tv . tv_nsec < ( xp -> tv . tv_nsec ) ) :
    ( me -> tv . tv_sec <  ( xp -> tv . tv_sec  ) ) ;
}

void  PLOTime_Inittimespec ( PLOTime * const me , struct timespec const t ) {
  me -> tv = t ;
  ParaLispObject_InitVTP  ( & me -> pub . plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOTime } ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOTime_constrstat_Inc  ( ) ;
# endif
}

# include "cons/public.h"
# include "stcl/public.h"
# include "stcl/inline.h"

TaskForThread bimtimesetf ( StackClusterPrivate  * const stclpp ) {
  PLOTime * const mep = ParaLispObjectToPLOTime ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop ( stclpp ) ) ;
  if  ( IsNotType ( time  ) ( stclpp  , xp ) )
    return  TaskForThread_Analyse ;
  PLOTime_Set ( mep , ParaLispObjectToPLOTime ( xp ) ) ;
  StackClusterPrivate_SetResult ( stclpp , & mep -> pub . plo ) ;
  return  TaskForThread_Analyse ;
}

# include "sharedc.h"
# include "shared/inline.h"

TaskForThread bimtimeswap ( StackClusterPrivate  * const stclpp ) {
  PLOTime * const mep = ParaLispObjectToPLOTime ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  ParaLispObject  * xp = CCAR ( StackClusterPrivate_ArgTop  ( stclpp )  ) ;
  if  ( IsNotType ( time  ) ( stclpp  ,  xp ) )
    return  TaskForThread_Analyse ;
  if  ( IsConst (  stclpp , xp  ) )
    return  TaskForThread_Analyse ;
  PLOTime_Swap  (  mep  , ParaLispObjectToPLOTime (  xp ) ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & mep -> pub . plo ) ;
  return  TaskForThread_Analyse ;
}

# include "undefbool.h"

# define  TimeEq  Lisp2D_TimeEq_r5t_
static  Lisp2DBoolType TimeEq  ( ParaLispObject * const xp  , ParaLispObject * const yp  ) {
  if  ( not Timep  ( yp ) )   
    return   Lisp2DBoolFalse ;
  if  ( PLOTime_NotEqu ( ParaLispObjectToPLOTimeConst  ( xp ) ,
    ParaLispObjectToPLOTimeConst ( yp ) ) )
    return   Lisp2DBoolFalse ;
  return   Lisp2DBoolTrue ;
}

# include "eqnoteq.h"

TaskForThread bimtimeeq ( StackClusterPrivate * const stclpp  ) {
  return   bimUniversalEqNotEq(  & TimeEq  ,  stclpp  )  ;
}

# define  TimeNotEq Lisp2D_TimeNotEqe7mD
static  Lisp2DBoolType TimeNotEq  ( ParaLispObject * const xp  , ParaLispObject * const yp  ) {
  if  ( not Timep  ( yp ) )    
    return   Lisp2DBoolTrue   ;
  if  ( PLOTime_Equ ( ParaLispObjectToPLOTimeConst  ( xp ) ,
    ParaLispObjectToPLOTimeConst ( yp ) )  )
    return   Lisp2DBoolFalse  ;
  return   Lisp2DBoolTrue   ;
}
  
TaskForThread bimtimenoteq ( StackClusterPrivate * const stclpp  ) {
  return   bimUniversalEqNotEq(  & TimeNotEq  ,  stclpp  )  ;
}

# define  TimeLess  Lisp2D_TimeLessaajl
static  Lisp2DBoolType TimeLess(ParaLispObject * const xp,
  ParaLispObject * const yp, StackClusterPrivate  * const stclpp){
  if  ( IsNotType ( time  ) ( stclpp  , yp  ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOTime_Les ( ParaLispObjectToPLOTimeConst  ( xp  ) ,
    ParaLispObjectToPLOTimeConst  ( yp  ) ) )
    return  Lisp2DBoolTrue;
  return  Lisp2DBoolFalse;
}
  
# include "lessmore.h"

// time < [{time}]
TaskForThread Lisp2Dbimtimeless3(StackClusterPrivate * const stclpp){
  return  bimUniversalLessMore(  & TimeLess,stclpp);
}

# include "minmax.h"

// time min [{time}]
TaskForThread Lisp2Dbimtimemin(StackClusterPrivate * const stclpp){
  return  bimUniversalMinMax(  & TimeLess,stclpp);
}

# define  TimeMore  Lisp2D_TimeMoretidD
static  Lisp2DBoolType TimeMore(ParaLispObject * const xp,
  ParaLispObject * const yp, StackClusterPrivate  * const stclpp ){
  if  ( IsNotType ( time  ) ( stclpp , yp ) )
    return  Lisp2DBoolUndef;
  if  ( PLOTime_Mor ( ParaLispObjectToPLOTimeConst  ( xp  ) ,
    ParaLispObjectToPLOTimeConst  ( yp  ) ) )
    return  Lisp2DBoolTrue;
  return  Lisp2DBoolFalse;
}
  
// time > [{time}]
TaskForThread Lisp2Dbimtimemore3(StackClusterPrivate * const stclpp){
  return  bimUniversalLessMore(  & TimeMore,stclpp);
}

// time max [{time}]
TaskForThread Lisp2Dbimtimemax(StackClusterPrivate * const stclpp){
  return  bimUniversalMinMax(  & TimeMore,stclpp);
}

# define  TimeLessEq  Lisp2D_TimeLessEqH2SK
static  Lisp2DBoolType TimeLessEq(ParaLispObject * const xp,
  ParaLispObject * const yp, StackClusterPrivate  * const stclpp){
  if  ( IsNotType ( time  ) ( stclpp  , yp  ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOTime_LessEq  ( ParaLispObjectToPLOTimeConst  ( xp  ) ,
    ParaLispObjectToPLOTimeConst  ( yp  ) ) )
    return  Lisp2DBoolTrue;
  return  Lisp2DBoolFalse;
}
  
// time <= [{time}]
TaskForThread Lisp2Dbimtimelesseq3(StackClusterPrivate * const stclpp){
  return  bimUniversalLessMore(  & TimeLessEq,stclpp);
}

# define  TimeMoreEq  Lisp2D_TimeMoreEqoK1Q
static  Lisp2DBoolType TimeMoreEq(ParaLispObject * const xp,
  ParaLispObject * const yp, StackClusterPrivate  * const stclpp){
  if  ( IsNotType ( time  ) ( stclpp  , yp  ) )
    return  Lisp2DBoolUndef ;
  if  ( PLOTime_MorEq ( ParaLispObjectToPLOTimeConst  ( xp  ) ,
    ParaLispObjectToPLOTimeConst  ( yp  ) ) )
    return  Lisp2DBoolTrue;
  return  Lisp2DBoolFalse;
}
  
// time >= [{time}]
TaskForThread Lisp2Dbimtimemoreeq3(StackClusterPrivate * const stclpp){
  return  bimUniversalLessMore(  & TimeMoreEq,stclpp);
}

// nil localtime
TaskForThread Lisp2Dbimnilsystime2  ( StackClusterPrivate  * const stclpp){
  struct  timespec  t;
  clock_gettime(CLOCK_REALTIME,&  t);
  PLOTime * const res = aligned_alloc ( _Alignof ( PLOTime ) , sizeof ( PLOTime ) ) ;
  if (  res ) {
    PLOTime_Inittimespec(res,t);
    StackClusterPrivate_SetResult(stclpp ,
      (ParaLispObject_virtualPushAsNew(&res-> pub . plo)));
  } else
    OutOfMemory ( stclpp ) ;
  return  TaskForThread_Analyse;  
}

# include "boolean.h"
# include "nilc.h"

TaskForThread Lisp2Dbimtimedstp2(StackClusterPrivate  * const stclpp){
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(ParaLispObjectToPLOTime(
    StackClusterPrivate_ThisTop(stclpp )))->tv_sec),&  ptm);
  if(ptm.tm_isdst){
    if(ptm.tm_isdst > 0){
      StackClusterPrivate_SetResult(stclpp ,&plotrue.plo);
      return  TaskForThread_Analyse;
    }
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  StackClusterPrivate_SetResult(stclpp ,&plofalse.plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimniltimep2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  StackClusterPrivate_SetResult ( stclpp ,
    ( Timep (  xp ) ?
      xp :
      & plofalse  . plo ) ) ;
  return  TaskForThread_Analyse;
}

ParaLispObject  * PLOTime_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return & plostime . plo ;
}

ParaLispObject  * PLOTime_staticClassDef  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassdeftime ;
}

ParaLispObject  * PLOTime_staticClassParents  ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return  ploclassparentstime ;
}

//  nil daylightp
TaskForThread Lisp2Dbimnildaylightp2(StackClusterPrivate  * const stclpp){
  StackClusterPrivate_SetResult(stclpp ,(daylight ?
    &(plotrue.plo) :
    &plofalse.plo));
  return  TaskForThread_Analyse;
}

//  nil setdaylightp
TaskForThread Lisp2Dbimnilsetdaylightp2(StackClusterPrivate * const stclpp){
  ParaLispObject  * const ap = CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  daylight  = ( (( ap  ==  (&  plonil.plo))  or  ( ap  ==  (&  plofalse.plo))) ?
    0 :
    1);
  tzset();
  StackClusterPrivate_SetResult(stclpp , ap);
  return  TaskForThread_Analyse;
}

# include "plocomplex/public.h"
  
TaskForThread Lisp2Dbimtimeyear2(StackClusterPrivate  * const stclpp) {
  struct  tm  ptm;
  localtime_r ( & ( PLOTime_CValueP (
      ParaLispObjectToPLOTime(StackClusterPrivate_ThisTop(stclpp )))->tv_sec), & ptm ) ;
  int r=(int)yearstart + ptm.tm_year;
  PLOComplex  * const  cr=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( cr),r);
  StackClusterPrivate_SetResult(stclpp ,&  cr->plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimemonth2(StackClusterPrivate * const stclpp){
  struct  tm  ptm;
  localtime_r ( & ( PLOTime_CValueP ( ParaLispObjectToPLOTime (
    StackClusterPrivate_ThisTop(stclpp )))->tv_sec),&  ptm);
  int r=1 + ptm.tm_mon;
  PLOComplex  * const  cr=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( cr),r);
  StackClusterPrivate_SetResult(stclpp ,&  cr->plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimeday2(StackClusterPrivate * const stclpp){
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(ParaLispObjectToPLOTime(
    StackClusterPrivate_ThisTop(stclpp )))->tv_sec),&  ptm);
  PLOComplex  * const  cr=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( cr),ptm.tm_mday);
  StackClusterPrivate_SetResult(stclpp ,&  cr->plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimehour2(StackClusterPrivate  * const stclpp){
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(ParaLispObjectToPLOTime(
    StackClusterPrivate_ThisTop(stclpp )))->tv_sec),&  ptm);
  PLOComplex  * const  cr=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( cr),ptm.tm_hour);
  StackClusterPrivate_SetResult(stclpp ,&  cr->plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimeminutes2(StackClusterPrivate * const stclpp){
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(ParaLispObjectToPLOTime(
    StackClusterPrivate_ThisTop(stclpp )))->tv_sec),&  ptm);  
  PLOComplex  * const  cr=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( cr),ptm.tm_min);
  StackClusterPrivate_SetResult(stclpp ,&  cr->plo);
  return  TaskForThread_Analyse;
}

# include "complex/inline.h"
# include "cast.h"

TaskForThread Lisp2Dbimtimeseconds2(StackClusterPrivate * const stclpp){
  PLOTime * const  mep  = ParaLispObjectToPLOTime ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(mep)->tv_sec),& ptm);
  PLOComplex  * const  pres=NewComplexBareExc();
  Complex_SetLInt ( PLOComplexToComplex( pres),PLOTime_CValue(mep).tv_nsec);
  FractionDivULInt ( Complex_Re  ( PLOComplexToComplex ( pres  ) ) ,  ulinanokoeff  ) ;
  FractionAddUInt ( Complex_Re  (PLOComplexToComplex(pres) ) , int_cast_uint ( ptm.tm_sec  ) ) ;
  StackClusterPrivate_SetResult(stclpp ,&  pres->plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimeweekday2(StackClusterPrivate * const stclpp){
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(ParaLispObjectToPLOTime(
    StackClusterPrivate_ThisTop(stclpp )))->tv_sec),&  ptm);
  PLOComplex  * const  cr=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( cr),ptm.tm_wday);
  StackClusterPrivate_SetResult(stclpp ,&  cr->plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimeyearday2(StackClusterPrivate * const stclpp){
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(ParaLispObjectToPLOTime( StackClusterPrivate_ThisTop(stclpp )))->tv_sec),&  ptm);
  PLOComplex  * const  cr=NewComplexBareExc();
  Complex_SetInt  ( PLOComplexToComplex( cr),ptm.tm_yday);
  StackClusterPrivate_SetResult(stclpp ,&  cr->plo);
  return  TaskForThread_Analyse;
}

# include "stack/public.h"

TaskForThread Lisp2Dbimtimenumber2(StackClusterPrivate  * const stclpp){
  PLOTime * const  mep  = ParaLispObjectToPLOTime ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  PLOComplex  * const  pres=NewComplexBareExc();
  Complex_SetLInt ( PLOComplexToComplex( pres),PLOTime_CValue(mep).tv_nsec);
  FractionDivULInt ( Complex_Re  ( PLOComplexToComplex ( pres  ) ) , ulinanokoeff  ) ;
  { ThreadStackPointerType const oldstack_fsec = Lisp2DThreadSP ( ) ;
    FractionPub * const fsec = NewStackFraction ( ) ;
    FractionConstrLInt ( fsec , PLOTime_CValue(mep).tv_sec ) ;
    FractionAddMoveFraction ( Complex_Re  ( PLOComplexToComplex ( pres  ) ) , fsec  ) ;
    Lisp2D_StackFree  ( oldstack_fsec ) ;
  }
  StackClusterPrivate_SetResult(stclpp ,&  pres->plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimetime2(StackClusterPrivate  * const stclpp){
  PLOTime * const mep=ParaLispObjectToPLOTime(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  const * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  switch(ParaLispObject_Type(xp)){
  case  TypeGlobalListMemberPLOTime:
    PLOTime_Set(mep,ParaLispObjectToPLOTimeConst(xp)); 
    break;
  case  TypeGlobalListMemberPLOComplex:
    { ThreadStackPointerType const oldstack_fs2 = Lisp2DThreadSP ( ) ;
      Complex * const frsec2p = Complex_FromStack ( ) ;
      Complex_InitCopy  ( frsec2p , PLOComplexToComplexConst(
         ParaLispObjectToPLOComplexConst ( xp ) ) ) ;
      FractionAbsFloor  (Complex_Re(  frsec2p ));
      struct  timespec  t;
      t.tv_sec = FractionLValue(Complex_CRe(  frsec2p ));
      // kernel has type long
      { Complex * const xcp = Complex_FromStack ( ) ;
        Complex_InitCopy( xcp ,PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst (xp)));
        Complex_Subtract  ( xcp , frsec2p ) ;
        // ? clear memory optimization ?
        FractionMultULInt ( Complex_Re  (  xcp ) ,
          ulinanokoeff ) ;
        t.tv_nsec =  FractionLValue(Complex_CRe(  xcp ));
        Lisp2D_StackFree  ( oldstack_fs2 ) ;
        // kernel has type long
        PLOTime_SetValue  ( mep  , t ) ;
      } 
    }
    break;
  default : ; 
  }
  StackClusterPrivate_SetResult(stclpp ,&mep-> pub . plo);
  return  TaskForThread_Analyse;
}

# include "string/public.h"
# include "string/struct.h"

TaskForThread Lisp2Dbimtimesetyear2(StackClusterPrivate * const  stclpp){
  PLOTime * const mep=ParaLispObjectToPLOTime(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,xp))
    return  TaskForThread_Analyse;
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(mep)->tv_sec),& ptm);
  if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        xp))))){
    ParaLispObject  * list [ ] = {
      xp,& PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { unsigned  short usyear;
    if  ( ULongisBADUShortValue ( FractionCNump  (
       Complex_CRe (PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst (
         xp ) ) ) ) , & usyear  ) ) {
      ParaLispObject  * list [ ] = {
        xp,& PLOString_NewCopy(L" is very big")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    ptm.tm_year = (usyear -  yearstart);
  }
  { time_t  const   res=mktime(&  ptm);
    if(res  ==  -1)    { 
      ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"?")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    } else   
      PLOTime_SetSec(mep,res);
  }
  StackClusterPrivate_SetResult(stclpp ,&mep-> pub . plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimesetmonth2(StackClusterPrivate  * const stclpp){
  PLOTime * const  mep=ParaLispObjectToPLOTime(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,xp))
    return  TaskForThread_Analyse;
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(mep)->tv_sec),& ptm);
  if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        xp))))){
    ParaLispObject  * list [ ] = {
      xp,& PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { unsigned  short usmonth;
    if  ( ULongisBADUShortValue ( FractionCNump  (
       Complex_CRe (  PLOComplexToComplexConst  (
         ParaLispObjectToPLOComplexConst ( xp ) ) ) ) , & usmonth ) ) {
      ParaLispObject  * list [ ] = {
        xp,& PLOString_NewCopy(L" is very big")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    ptm.tm_mon  = (usmonth  - 1);
  }
  { time_t  const   res=mktime(&  ptm);
    if(res  ==  -1)   { 
      ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"?")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    } else  
      PLOTime_SetSec(mep,res);
  }
  StackClusterPrivate_SetResult(stclpp ,&mep-> pub . plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimesetday2(StackClusterPrivate  * const stclpp){
  PLOTime * const  mep=ParaLispObjectToPLOTime(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,xp))
    return  TaskForThread_Analyse;
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(mep)->tv_sec),& ptm);
  if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        xp))))){
    ParaLispObject  * list [ ] = {
      xp,& PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { unsigned  short usday;
    if  ( ULongisBADUShortValue ( FractionCNump  (
       Complex_CRe (  PLOComplexToComplexConst  (
         ParaLispObjectToPLOComplexConst ( xp ) ) ) ) , & usday ) ) {
      ParaLispObject  * list [ ] = {
        xp,& PLOString_NewCopy(L" is very big")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    ptm.tm_mday = usday;
  }
  { time_t  const   res=mktime(&  ptm);
    if(res  ==  -1)     {
      ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"?")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    } else    
      PLOTime_SetSec(mep,res);
  }
  StackClusterPrivate_SetResult(stclpp ,&mep-> pub . plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimesethour2(StackClusterPrivate * const stclpp){
  PLOTime * const  mep=ParaLispObjectToPLOTime(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,xp)) 
    return  TaskForThread_Analyse;
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(mep)->tv_sec),& ptm);
  if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        xp))))){
    ParaLispObject  * list [ ] = {
      xp,& PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { unsigned  short ushhour;
    if  ( ULongisBADUShortValue ( FractionCNump  (
       Complex_CRe (  PLOComplexToComplexConst  (
         ParaLispObjectToPLOComplexConst ( xp ) ) ) ) , & ushhour ) ) {
      ParaLispObject  * list [ ] = {
        xp,& PLOString_NewCopy(L" is very big")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    ptm.tm_hour = ushhour;
  }
  { time_t  const   res=mktime(&  ptm);
    if(res  ==  -1)   { 
      ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"?")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
    } else    
      PLOTime_SetSec(mep,res);
  }
  StackClusterPrivate_SetResult(stclpp ,&mep-> pub . plo);
  return  TaskForThread_Analyse;
}

TaskForThread Lisp2Dbimtimesetminutes2(StackClusterPrivate  * const stclpp){
  PLOTime * const  mep=ParaLispObjectToPLOTime(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,xp))  
    return  TaskForThread_Analyse;
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(mep)->tv_sec),& ptm);
  if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        xp))))){
    ParaLispObject  * list [ ] = {
      xp,& PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { unsigned  short ushminut;
    if  ( ULongisBADUShortValue  ( FractionCNump  (
       Complex_CRe (  PLOComplexToComplexConst  (
         ParaLispObjectToPLOComplexConst ( xp ) ) ) ) , & ushminut  ) ) {
      ParaLispObject  * list [ ] = {
        xp,& PLOString_NewCopy(L" is very big")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    ptm.tm_min  = ushminut;
  }
  { time_t  const  res=mktime(&  ptm);
    if(res  ==  -1)     {
      ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"?")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); }
    else
      PLOTime_SetSec(mep,res);
  }
  StackClusterPrivate_SetResult(stclpp ,&mep-> pub . plo);
  return  TaskForThread_Analyse;
}

# include "fraction/public.h"
  
TaskForThread Lisp2Dbimtimesetseconds2(StackClusterPrivate  * const stclpp){
  PLOTime * const  mep=ParaLispObjectToPLOTime(
    StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  * const xp= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,xp)) 
    return  TaskForThread_Analyse;
  struct  tm  ptm;
  localtime_r(& (PLOTime_CValueP(mep)->tv_sec),& ptm);
  ThreadStackPointerType const oldstack_sec = Lisp2DThreadSP ( ) ;
  FractionPub * const sec = NewStackFraction  ( ) ;
  FractionConstrCopy ( sec ,  Complex_CRe (
    PLOComplexToComplexConst  ( ParaLispObjectToPLOComplex  ( xp  ) ) ) ) ;
  FractionFloor (sec);
  { unsigned  short ushsecond;
    if  ( ULongisBADUShortValue ( FractionCNump  (
      sec ) ,  & ushsecond ) ) {
      ParaLispObject  * const list [ ] = {
        xp,& PLOString_NewCopy(L" is very big")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      Lisp2D_StackFree  ( oldstack_sec ) ;
      return  TaskForThread_Analyse;
    }
    ptm.tm_sec  = ushsecond;
  }
  time_t  const   res=mktime(&  ptm);
  if(res  ==  -1) { 
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"?")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
  } else  
    PLOTime_SetSec(mep,res);
  { ThreadStackPointerType const oldstack_nsec = Lisp2DThreadSP ( ) ;
    FractionPub * const nsec = NewStackFraction ( ) ;
    FractionConstrCopy ( nsec ,
       Complex_CRe ( PLOComplexToComplexConst  (
        ParaLispObjectToPLOComplex  ( xp  ) ) ) ) ;
    FractionSubtractMove  (  nsec  , sec ) ;
    FractionMultULInt ( nsec ,  ulinanokoeff ) ;
    { long  int const nsec2 = FractionLValue(nsec);
      Lisp2D_StackFree  ( oldstack_nsec ) ;
      PLOTime_SetNSec ( mep  , nsec2 ) ;
    }
  }
  Lisp2D_StackFree  ( oldstack_sec ) ;
  StackClusterPrivate_SetResult(stclpp ,&mep-> pub . plo);
  return  TaskForThread_Analyse;
}

//  nil timezone
TaskForThread Lisp2Dbimniltimezone2(StackClusterPrivate * const stclpp){
  PLOComplex  *  const pres=NewComplexBareExc();
  Complex_SetLInt ( PLOComplexToComplex( pres),timezone);
  StackClusterPrivate_SetResult(stclpp ,&  pres->plo);
  return  TaskForThread_Analyse;
}

//  nil settimezone
TaskForThread Lisp2Dbimnilsettimezone2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const ap= CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  if(IsNotNumber(  stclpp ,ap)) 
    return  TaskForThread_Analyse;
  timezone = FractionLValue(Complex_CRe(PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(ap))));
  tzset();
  StackClusterPrivate_SetResult(stclpp ,ap);
  return  TaskForThread_Analyse;
}

# include "cons/inline.h"
  
// nil time
TaskForThread Lisp2Dbimniltime2(StackClusterPrivate * const stclpp){
  ParaLispObject  * varC;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) )
    goto  Next  ;
  { struct  timespec  tv;
    clock_gettime(CLOCK_REALTIME,&  tv);
    { PLOComplex  * const  pres=NewComplexBareExc();
      Complex_SetLInt ( PLOComplexToComplex( pres),tv.tv_sec);
      { ThreadStackPointerType const oldstack_ttmp = Lisp2DThreadSP ( ) ;
        FractionPub * const ttm = NewStackFraction ( ) ;
        FractionConstrLInt (  ttm ,tv.tv_nsec);
        FractionDivULInt  (ttm,ulinanokoeff );
        FractionAddMoveFraction  ( Complex_Re  (
          PLOComplexToComplex ( pres  ) ) , ttm ) ;
        Lisp2D_StackFree  ( oldstack_ttmp ) ;
      }
      StackClusterPrivate_Push(stclpp ,&   pres->plo); 
    } 
  }
  varC  = StackClusterPrivate_ArgTop(stclpp );
  for ( ; CConsp  ( varC  ) ; varC  = ClassCons_Rest  ( &
    ParaLispObjectToPLOCons ( varC  )->cc ) ) {
    StackClusterPrivate_Push(stclpp , varC);
    StackClusterPrivate_Push(stclpp , ClassCons_First(&
    ParaLispObjectToPLOCons(varC)->cc));
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    return  TaskForThread_Analyse;
Next:
    StackClusterPrivate_Pop(stclpp ,&varC);
  }
  { struct  timespec  tv;
    clock_gettime(CLOCK_REALTIME,&  tv);
    PLOComplex  * const  ptt2=NewComplexBareExc();
    Complex_SetLInt ( PLOComplexToComplex( ptt2),tv.tv_sec);
    { ThreadStackPointerType const oldstack_ttm2p = Lisp2DThreadSP ( ) ;
      FractionPub * const ttm2 = NewStackFraction ( ) ;
      FractionConstrLInt (ttm2,tv.tv_nsec);
      FractionDivULInt  (ttm2,ulinanokoeff );
      FractionAddMoveFraction  ( Complex_Re  ( 
        PLOComplexToComplex ( ptt2  ) ) , ttm2 ) ;
      Lisp2D_StackFree  ( oldstack_ttm2p ) ;
    }
    { ParaLispObject * tmp;
      StackClusterPrivate_Pop(stclpp ,&tmp);
      Complex_Subtract  ( PLOComplexToComplex( ptt2),
        PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst(tmp)));
    }
    StackClusterPrivate_SetResult(stclpp ,&  ptt2->plo);
  }
  return  TaskForThread_Analyse;
}

# include "stringbim.h"
# include "string/public.h"
  
//  nil settzname
TaskForThread Lisp2Dbimnilsettzname2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const ap= CCAR ( arglistp);
  ParaLispObject  * const bp= CCAR ( CCDR ( arglistp));
  if(IsNotString(  stclpp,ap))  
    return  TaskForThread_Analyse;
  if(IsNotString(  stclpp,bp)) 
    return  TaskForThread_Analyse;
  if(flagdeletetzname02) {
    free(tzname[0]);
    tzname[0] = 0 ;
  }
  { ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const strtmp2 = StackNew_StringChar ( ) ;
    String_Init ( Char  ) ( strtmp2 , true ) ;
    if ( isBADReadWCharToCharString  ( PLOString_CStr ( ParaLispObjectToPLOString ( ap  )
      ) , strtmp2 ) ) {
      BadString(stclpp ,ParaLispObjectToPLOString(ap));
      return  TaskForThread_Analyse;
    }
    char  * const dop =
      aligned_alloc ( _Alignof ( char ) , sizeof  ( char  )  * ( String_Size ( Char  ) ( strtmp2 )  + 1 ) ) ;
    if ( not dop ) {
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
    strcpy(dop, StringCStr ( Char  ) ( strtmp2 ) ) ;
    Lisp2D_StackFree  ( oldstack_a ) ;
    tzname[0] = dop;
    flagdeletetzname02 = true;
  } // strtmp
  if(flagdeletetzname12) {
    free(tzname[1]);
    tzname[1] = 0 ;
  } // strtmp
  { ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    String  ( Char  ) * const strtmp2 = StackNew_StringChar ( ) ;
    String_Init ( Char  ) ( strtmp2 , true ) ;
    if ( isBADReadWCharToCharString  ( PLOString_CStr ( ParaLispObjectToPLOString ( ap  )
      ) , strtmp2 ) ) {
      BadString(stclpp ,ParaLispObjectToPLOString(bp));
      return  TaskForThread_Analyse;
    }
    char  * const dop =
      aligned_alloc ( _Alignof ( char ) , sizeof  ( char  )  * ( String_Size ( Char  ) ( strtmp2 )  + 1 ) ) ;
    if ( not dop ) {
      OutOfMemory ( stclpp ) ;
      return   TaskForThread_Analyse  ;
    }
    strcpy(dop,StringCStr ( Char  ) ( strtmp2 ));
    Lisp2D_StackFree  ( oldstack_a ) ;
    tzname[1] = dop;
    flagdeletetzname12 = true;
  }
  StackClusterPrivate_SetResult(stclpp ,arglistp);
  return  TaskForThread_Analyse;
}

# include "errorstra.h"

void  PLOTime_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const os , bool _ __attribute__ ((unused)) ,
  Vector ( StackPLOP )  * _2 __attribute__ ((unused)) ) {
  char  ct  [ stringsizestandardlimit ] ;
  ctime_r ( & ( ParaLispObjectToPLOTimeConst  ( me  ) -> tv . tv_sec  ) ,
    ct  ) ;
  bool  flagfind  = false ;
  { unsigned  char  i = 0 ;
    char  * cti = ct  ;
    for ( ; i  < stringsizestandardlimit ;  ++ i  , ++  cti )
      if  ( ( * cti ) ==  '\n'  ) {
        ( * cti ) = '\0'  ;
        flagfind  = true  ;
        break ;
      }
  }
  if  ( flagfind  ) {
    ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    String  ( Char ) * const stc = StackNew_StringChar ( ) ;
    StringConstrCStr ( Char  ) ( stc , ct , true ) ;
    String  ( WideChar ) * const wstc = StackNew_StringWideChar ( ) ;
    String_Init ( WideChar  ) ( wstc , true ) ;
    if ( isBADReadCStringToWString2 ( stc , wstc ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      ErrorStra((PArray_wchar_c){(wchar_t const (*)[])&L"PLOTime::Cout:bad string"});
      return ;
    }
    PLOStreamPushWideCharArray  ( os ,
      (PArray_wchar_c) { (wchar_t const(*)[])StringCStr(WideChar)(wstc)});
    Lisp2D_StackFree  ( oldstack ) ;
  } else
    ErrorStra (
      (PArray_wchar_c){(wchar_t const (*)[])&L"PLOTime::Cout:very big string"});
}

// time string
TaskForThread Lisp2Dbimtimestring2(StackClusterPrivate  * const stclpp){
  char  ct[ stringsizestandardlimit  ];
  ctime_r(& (PLOTime_CValueP(ParaLispObjectToPLOTime(
    StackClusterPrivate_ThisTop(stclpp )))->tv_sec),ct);
  char  * cti=ct;
  bool    flagfind=false;
  for(unsigned  char    i=0;i  <  stringsizestandardlimit  ;++ i,++  cti)
    if((* cti)  ==  '\n'){
      (* cti) = '\0';
      flagfind  = true;
      break;
    }
  if ( flagfind ) {
    ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ;
    String_Init ( WideChar  ) ( wanc , true ) ;
    { String  ( Char ) * const tmpctc = StackNew_StringChar ( ) ;
      StringConstrCStr ( Char  ) ( tmpctc , ct , true ) ;
      if ( isBADReadCStringToWString2 ( tmpctc , wanc ) ) {
        Lisp2D_StackFree  ( oldstack ) ;
        BadStringN(stclpp );
        return  TaskForThread_Analyse ;
      }
    }
    PLOStringPub * const  resp = PLOString_NewMoveString ( wanc ) ;
    Lisp2D_StackFree  ( oldstack ) ;
    StackClusterPrivate_SetResult ( stclpp ,  & resp  ->  plo ) ;
  } else
    StackClusterPrivate_ExceptionSet(
      stclpp ,&PLOString_NewCopy(L"time:string:very big string")->plo);
  return  TaskForThread_Analyse;
}

//  nil tzname
TaskForThread Lisp2Dbimniltzname2 ( StackClusterPrivate * const stclpp ) {
  tzset ( ) ;
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  String  ( WideChar ) * const wanc = StackNew_StringWideChar ( ) ;
  String_Init ( WideChar  ) ( wanc , true ) ;
  { String  ( Char ) * const tzname0c = StackNew_StringChar ( ) ;
    StringConstrCStr ( Char  ) ( tzname0c , tzname  [ 0 ] , true ) ;
    if ( isBADReadCStringToWString2 ( tzname0c , wanc ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      BadStringN(stclpp );
      return  TaskForThread_Analyse ;
    }
  }
  String  ( WideChar ) * const wan1c = StackNew_StringWideChar ( ) ;
  String_Init ( WideChar  ) ( wan1c , true ) ;
  { String  ( Char ) * const tzname1c = StackNew_StringChar ( ) ;
    StringConstrCStr ( Char  ) ( tzname1c , tzname  [ 1 ] , true ) ;
    if ( isBADReadCStringToWString2 ( tzname1c , wan1c ) ) {
      Lisp2D_StackFree  ( oldstack ) ;
      BadStringN(stclpp );
      return  TaskForThread_Analyse ;
    }
  }
  { PLOStringPub * const  resp = PLOString_NewMoveString ( wanc ) ;
    { PLOStringPub * const  res1p = PLOString_NewMoveString ( wan1c ) ;
      ParaLispObject  * list [ ] = { &resp->plo, &res1p->plo, 0 } ;
      StackClusterPrivate_SetResult(
        stclpp ,ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list } ) ) ;
    }
  }
  Lisp2D_StackFree  ( oldstack ) ;
  return  TaskForThread_Analyse ;
}

void  PLOTime_Init ( PLOTime * const me ) {
  me -> tv . tv_sec = 0 ;
  me -> tv . tv_nsec = 0 ;
  VirtualTypePointer  vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOTime } ;
  ParaLispObject_InitVTP  ( & me -> pub . plo , vtp ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOTime_constrstat_Inc  ( ) ;
# endif
}

void  PLOTime_InitCopy ( PLOTime * const me , PLOTime const * const tp ) {
  me -> tv = tp -> tv ;
  VirtualTypePointer vtp = { . addr = { . p = me } , . type = TypeGlobalListMemberPLOTime } ;
  ParaLispObject_InitVTP  ( & me -> pub . plo , vtp ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOTime_constrstat_Inc  ( ) ;
# endif
}

bool  PLOTime_Equ ( PLOTime  const * const me , PLOTime  const * const xp  ) {
  if  ( me == xp )
    return  true  ;
  return (  me -> tv . tv_sec == ( xp -> tv . tv_sec  ) ) and
    ( me -> tv . tv_nsec ==  ( xp -> tv . tv_nsec ) ) ;
}

void PLOTime_staticDestr  ( ParaLispObject * const me ) {
  PLOTime_statDestr ( ParaLispObjectToPLOTime ( me  ) ) ;
}

bool  PLOTime_NotEqu ( PLOTime  const * const me , PLOTime  const * const xp ) {
  return not  PLOTime_Equ ( me , xp ) ;
}

bool  PLOTime_LessEq ( PLOTime const * const me , PLOTime const * const xp ) {
  return not  PLOTime_Les ( xp  , me ) ;
}

bool  PLOTime_MorEq  ( PLOTime const * const me , PLOTime const * const xp ) {
  return not  PLOTime_Les ( me , xp  ) ;
}
# include "widestring/inline.h"
# include "castb.h"
