// Lisp2D unsigned integer
// ©2019-23 by Alex Glebe
# include <stdatomic.h>
# include <stddef.h>
# include <iso646.h>
# include <limits.h>
# include <sys/types.h>
# include <unistd.h>
# include "define.h"
# include "ulong/private.h"

void  ULong_Simplify  ( ULong * const mep ) {
  if  ( ( VectorEmpty ( StackUnsInt ) ( & ( mep -> valuec ) ) ) or
    ( VectorBackCopy  ( StackUnsInt ) ( & ( mep -> valuec ) ) ) )
    return  ;
  { size_t  newsize = VectorSizeCopy  ( StackUnsInt ) (
      & ( mep  -> valuec   ) ) - 1UL ;
    unsigned  int const *  i = Vector_CRBegin ( StackUnsInt ) (
      & (  mep  ->  valuec  ) ) ;
    while ( newsize ) {
      --  i ;
      if  ( * i ) {
        Vector_Resize ( StackUnsInt ) ( & ( mep  -> valuec   ) , newsize ) ;
        return  ; 
      }
      --  newsize ; 
    } 
  }
  VectorClear ( StackUnsInt ) ( &  ( mep  -> valuec   ) ) ;
}

// ! me > l  
void  ULongAddPrivate  ( ULong  * const  mep  , ULong  const * const lp  ) {
# ifdef LISP2D_DEBUG
  if  ( mep == lp ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongAddPrivate:mep == lp" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  bool pere = false ;
  unsigned  int *   iterj = VectorBegin ( StackUnsInt ) ( & mep ->  valuec ) ;
  unsigned  int const *   iterlj  = VectorCBegin  ( StackUnsInt ) (
    & ( lp  ->  valuec   ) ) ;
  unsigned  int const * const  iterljend = VectorCEnd ( StackUnsInt ) (
    & ( lp -> valuec  ) ) ;
  for ( ; iterlj not_eq  iterljend  ; ++ iterj  , ++  iterlj  ) {
    if  ( pere  ) {
      if  ( *  iterlj ) {
        unsigned  int const s = ( ( * iterj )  + (  *  iterlj ) + 1U  ) ;
        pere  = ( s  <=  (  *  iterj  ) ) ;
        ( *  iterj  ) = s ; 
      } else
        pere  = ( ( ++ (  * iterj ) )  ==  0  ) ;
    } else {
      if  ( *  iterlj ) {
        unsigned  int const s = ( ( * iterj )  + (  *  iterlj ) ) ;
        pere  = ( s  < (  *  iterj  ) ) ;
        ( *  iterj  ) = s ; 
      }
    }
  }
  if  ( pere  ) {
    unsigned  int const * iterjend  = VectorCEnd  ( StackUnsInt ) (
      & ( mep ->  valuec   ) ) ;
    for ( ; iterj  not_eq  iterjend ; ++ iterj  )
      if  ( ++ (  *  iterj  ) )
        return  ;
    Vector_PushByValue ( StackUnsInt ) ( & ( mep ->  valuec   ) , 1U  ) ;
  }
}  

# include "cast.h"

// cout off low int's count and low bit's count
void  ULongCutOff  ( t_ULongCutOff const * const arg )  {
  Vector_RemoveFront ( StackUnsInt )  ( & ( arg -> mep  -> valuec   ) , arg -> intscount ) ;
  if  ( arg -> bitscount ) {
    unsigned int * j =  VectorBegin ( StackUnsInt ) ( & ( arg -> mep  -> valuec   ) ) ;
    unsigned  int const * const end = Vector_CRBegin  ( StackUnsInt ) ( & ( arg -> mep  -> valuec  ) ) ;
    unsigned  int const step  = ulint_cast_uint ( ( sizeof  ( unsigned  int ) <<  3 ) - arg -> bitscount ) ;
    while ( j not_eq end  ) {
      ( * j ) = ( ( ( * j ) >>  arg -> bitscount ) bitor ( ( * ( j + 1 ) ) <<  step  ) ) ;
      ++  j ;
    }
    ( * j ) >>=  arg -> bitscount ;
    ULong_Simplify  ( arg -> mep ) ;
  }
}

# include "ulong/public.h"

# ifdef  LISP2D_CONSTRSTAT  

_Atomic ( size_t  ) Lisp2DULongconstrstat = ATOMIC_VAR_INIT( 0UL ) ;  
  
size_t  Lisp2DULongconstrstatGetValue  ( void ) {
  return Lisp2DULongconstrstat ; 
}

void  Lisp2DULongconstrstatAddOne ( void ) {
  ++  Lisp2DULongconstrstat  ; 
}

# endif

# include "ulong/private.h"

void  ULong_Init ( ULongPub * const mel ) {
  ULong * const melp = ULong_toPrivate  ( mel ) ;
  VectorInit  ( StackUnsInt ) ( & ( melp ->  valuec  ) , false ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DULongconstrstat  ;
# endif  
}

void  ULongDestrPub  ( ULongPub * const my ) {
  ULong * const myp = ULong_toPrivate  ( my ) ;
  ULongDestr  ( myp ) ;
}

void  ULongDestr  ( ULong * const myp ) {
  Vector_Destr  ( StackUnsInt ) ( & ( myp ->  valuec  ) ) ;
}

void  ULong_ConstrCopy ( ULongPub * const my , ULongPub const * const x ) {
# ifdef  LISP2D_DEBUG
  if  ( my ==  x )  {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongConstrCopy:my == x" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  Vector_ConstrCopy  ( StackUnsInt ) ( & ( ULong_toPrivate ( my  ) ->
    valuec  ) , & ( ULong_toCPrivate  ( x ) ->  valuec  ) ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DULongconstrstat  ;
# endif
}

void  ULongConstrUInt ( ULongPub * const my , unsigned int const x ) {
  ULong * const myp = ULong_toPrivate  ( my ) ;
  if  ( x )
    VectorInitOne ( StackUnsInt ) ( & ( myp ->  valuec  ) , x , false ) ;
  else 
    VectorInit  ( StackUnsInt ) ( & ( myp ->  valuec  ) , false ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DULongconstrstat  ;
# endif
}
  
void  ULongConstrMove ( ULongPub * const my , ULongPub * const x ) {
# ifdef  LISP2D_DEBUG
  if  ( my ==  x )  {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongConstrMove:my == x" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  ULong * const myp = ULong_toPrivate  ( my ) ;
  ULong * const xp = ULong_toPrivate  ( x ) ;
  VectorInitMove  ( StackUnsInt ) ( & ( myp  ->  valuec  ) , & ( xp  ->  valuec  ) ) ;
# ifdef  LISP2D_CONSTRSTAT
  ++  Lisp2DULongconstrstat  ;
# endif
}

// /= (2^pow)    
void  ULongDiv2Power  ( ULongPub * const my , size_t pow ) {
  if  ( pow ==  0 )
    return  ;
  ULong * const myp = ULong_toPrivate  ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp -> valuec   ) )
    return  ;
  { enum { intbitsize = sizeof  ( unsigned  int ) * 8U  } ;
    // pow>0 size>0
    { size_t const bigpow = pow  / intbitsize ;
      if  ( bigpow  ) {
        // >>= (sizeof(unsigned int) * 8 * bigpow)    
        Vector_RemoveFront ( StackUnsInt )(&myp-> valuec  , bigpow ) ;
        pow -=  bigpow  * intbitsize ;
        if  ( pow ==  0 )
          return  ;
        if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec   ) )
          return  ; 
      }
    } // bigpow
    // size > 0
    // pow = 1 .. 31
    if  ( VectorSizeCopy(StackUnsInt)(& myp->valuec   ) ==  1UL ) {
      unsigned  int * fp = VectorFront(StackUnsInt) ( &myp->valuec ) ;
      (*fp) >>= pow ;
      if  ( (*fp) ==  0 )
        VectorClear ( StackUnsInt ) ( &myp->valuec  ) ;
      return  ; 
    }
    // valuec.size >= 2
    unsigned  int const bitsup  = intbitsize - size_cast_uint ( pow ) ;
    unsigned  int * const ilast = Vector_Back ( StackUnsInt )  ( &myp->valuec  ) ;
    for ( unsigned  int * i =  VectorBegin ( StackUnsInt ) ( &myp->valuec  )  ;
      i not_eq  ilast ; ++  i ) {
      ( * i ) >>= pow ;
      ( * i ) or_eq ( ( * ( i + 1 ) ) <<  bitsup  ) ; 
    }
    ( * ilast ) >>= pow ;
    if  ( ( * ilast ) ==  0 )
      VectorPop ( StackUnsInt ) (& myp->valuec   ) ;
  } // intbitsize
}  
  
// /= 2
void  ULongDiv2  ( ULongPub * const my ) {
  ULong * const myp = ULong_toPrivate  ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & ( myp -> valuec  ) ) )
    return  ;
  { enum { intbitsize  = sizeof  ( unsigned  int ) * 8U } ;
    // size > 0
    if  ( VectorSizeCopy  ( StackUnsInt ) ( & (myp->valuec )  ) ==  1UL ) {
      unsigned  int * fp = VectorFront  ( StackUnsInt ) ( &(myp->valuec )) ;
      ( * fp  ) >>= 1U ;
      if  ( (*fp) ==  0 )
        VectorClear ( StackUnsInt ) ( &(myp->valuec ) ) ;
      return  ; 
    }
    // valuec.size >= 2
    enum  { bitsup  = intbitsize - 1U } ;
    unsigned  int * const ilast = 
      Vector_Back ( StackUnsInt )  ( &(myp->valuec ) )   ;
    for ( unsigned  int * i =  VectorBegin(StackUnsInt) ( &(myp->valuec ) )  ;
      i not_eq  ilast ; ++  i ) {
      ( * i ) >>= 1U ;
      ( * i ) or_eq ( ( * ( i + 1 ) ) <<  bitsup  ) ; 
    }
    ( * ilast ) >>= 1U ;
    if  ( ( * ilast ) ==  0 ) 
      VectorPop(StackUnsInt) (& myp->valuec   ) ;  
  } // intbitsize
}

bool  ULongisEqUInt ( ULongPub const * const my , unsigned  int const x ) {
  ULong const * const myp = ULong_toCPrivate  ( my ) ;
  switch  ( VectorSizeCopy  ( StackUnsInt ) ( & ( myp -> valuec  ) ) ) {
  case  0 :
    return  x  ==  0 ;
  case  1 :
    return  Vector_FrontCopy ( StackUnsInt ) ( & ( myp -> valuec  ) ) ==  x ;
  default :
    return  false ; 
  } 
}
    
bool  ULongLessUInt ( ULongPub const * const my , unsigned  int const x ) {
  ULong const * const myp = ULong_toCPrivate  ( my ) ;
  switch  ( VectorSizeCopy  ( StackUnsInt ) ( & myp ->  valuec )  ) {
  case  0:
    return  x  not_eq  0  ;
  case  1:  
# ifdef  LISP2D_DEBUG
    if ( Vector_FrontCopy ( StackUnsInt )(&myp->valuec ) ==  0 ) {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"Lisp2DULongLessUInt:VectorSizeCopy(StackUnsInt)(&myp->valuec)==1:"
        L"Vector_FrontCopy ( StackUnsInt )(&myp->valuec) ==  0" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
    }
# endif    
    return  Vector_FrontCopy ( StackUnsInt )(&myp->valuec ) < x;
  default :
# ifdef  LISP2D_DEBUG
    if(VectorBackCopy(StackUnsInt)(& myp->valuec ) ==  0){
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"Lisp2DULongLessUInt:VectorSizeCopy(StackUnsInt)(&myp->valuec)>1:"
        L"Vector_FrontCopy ( StackUnsInt )(&myp->valuec) ==  0" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
    }
# endif        
    return  false ;
  } 
}
    
bool  ULongMoreUInt ( ULongPub const * const my ,
  unsigned  int const x ) {
  ULong const * const myp = ULong_toCPrivate  ( my ) ;
  switch ( VectorSizeCopy ( StackUnsInt ) ( & myp -> valuec  ) ) {
  case  0 :
    return  false ;
  case  1 :
    return  Vector_FrontCopy ( StackUnsInt ) ( & myp -> valuec  ) > x ;
  default :
    return  true ; 
  } 
}
  
unsigned  int ULongMod2  ( ULongPub const * const my ) {
  ULong const * const myp = ULong_toCPrivate  ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp -> valuec  ) )
    return  0 ;
  return  Vector_FrontCopy ( StackUnsInt ) ( & myp -> valuec  ) bitand 1U ;
}

bool  ULongisEqULong ( ULongPub  const * const me ,
  ULongPub  const * const x ) {
  if ( me ==  x )
    return  true  ;
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  ULong const * const xp = ULong_toCPrivate  ( x ) ;
  return  Lisp2DStackUnsIntisEq ( & ( mep -> valuec  ) ,
    & ( xp -> valuec  ) ) ;
}

bool  ULongIs1 ( ULongPub  const * const me ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  Vector  ( StackUnsInt ) const * const myvaluec = & mep  ->  valuec  ;
  if  ( ( VectorSizeCopy  ( StackUnsInt ) ( myvaluec ) ) ==  1UL )
    return  Vector_FrontCopy ( StackUnsInt ) ( myvaluec ) ==  1U  ;
  return  false ;
}

bool  ULongIsBADUIValue  ( ULongPub  const * const me ,
  unsigned int * const resultp ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  size_t  const   s = VectorSizeCopy(StackUnsInt) ( & mep ->  valuec   ) ;
  if  ( s  > 1  )    
    return  true  ;
  if  ( s ) 
    (  * resultp )  = Vector_FrontCopy ( StackUnsInt ) ( & mep -> valuec  ) ;
  else
    ( * resultp )  = 0  ;
  return  false  ; 
}  

bool  ULongisBADByteValue  ( ULongPub  const * const me ,
  uint8_t  * const resp  )  {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  switch  ( VectorSizeCopy  ( StackUnsInt ) ( & mep ->  valuec ) ) {
  case  0 :  
    ( * resp  ) = 0 ;
    return  false ;
  case  1 :  {
    unsigned  int const uitmp = Vector_FrontCopy ( StackUnsInt )  (
      & mep ->  valuec   ) ;
    if  ( uitmp  <=  0xff ) {
      ( * resp  ) = uint_cast_uint8 ( uitmp ) ;
      return  false ; 
    }
    return  true  ; 
  }
  default :
    return  true ; 
  } 
}

// used by : stream write-byte number|byte-vector  
bool  ULongisBADUCharValue ( ULongPub const * const me ,
  unsigned char  * const resp  )  {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  switch  ( VectorSizeCopy(StackUnsInt) ( & mep ->  valuec  ) ) {
  case  0 :  
    ( * resp ) = 0 ;
    return  false ;
  case  1 :  {
    unsigned  int const uitmp = Vector_FrontCopy ( StackUnsInt )  (
      & mep ->  valuec   ) ;
    if  ( uitmp  <= UCHAR_MAX ) {
      ( * resp ) = uint_cast_uchar ( uitmp  ) ;
      return  false ; 
    }
    return  true  ; 
  }
  default :
    return  true ; 
  } 
}

# if  USHRT_MAX ==  UINT_MAX
  
bool  ULongisBADUShortValue ( ULongPub const * const me ,
  unsigned short int * const resultp  ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  { size_t  const  s  = VectorSizeCopy(StackUnsInt) ( & mep ->  valuec ) ;
    if  ( s  > 1  )   
      return  true ;
    if  ( s  ==  0  ) {
      ( * resultp )  = 0 ;
      return  false ; 
    }
  }
  ( * resultp )  = Vector_FrontCopy ( StackUnsInt )  ( & mep ->  valuec   ) ;
  return  false ; 
}
  
# else

bool  ULongisBADUShortValue ( ULongPub  const *  const me ,
  unsigned short int * const  resultp  ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  size_t  const  s  = VectorSizeCopy(StackUnsInt) ( & mep ->  valuec   ) ;
  if  ( s  > 1  )   
    return  true ;
  if  ( s ) {
    unsigned  int const fr  = Vector_FrontCopy ( StackUnsInt )  (
      & mep ->  valuec   ) ;
    unsigned short int const  tmp  = uint_cast_ushort ( fr  ) ;
    if  ( tmp not_eq  fr  ) 
      return  true ;
    ( * resultp ) = tmp ;  
  } else
    ( * resultp )  = 0  ;
  return  false ; 
}
  
# endif

bool  ULongIsBADUI16Value  ( ULongPub  const * const me ,
  uint16_t *  const resultp ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  size_t  const  s = VectorSizeCopy(StackUnsInt) ( & mep  ->  valuec   ) ;
  if  ( s  > 1  ) 
    return  true  ;
  if  ( s ) {
    unsigned int const fc =
      Vector_FrontCopy ( StackUnsInt )  ( & mep ->  valuec   ) ;
    ( * resultp ) = uint_cast_uint16 ( fc  ) ;
    if  ( ( * resultp ) not_eq  fc  )
      return  true ; 
  } else
    ( * resultp ) = 0 ;
  return  false  ; 
}
  
# if UINT_MAX  ==  SIZE_MAX

bool  ULongIsBADSizeValue  ( ULongPub  const * const me ,
  size_t * const resultp  ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  size_t  const s = VectorSizeCopy(StackUnsInt) ( & mep ->  valuec   ) ;
  if  ( s  > 1  )
    return  true ;
  if  ( s  ==  0  )
    ( * resultp )  = 0 ;
  else
    (  * resultp ) = Vector_FrontCopy ( StackUnsInt )  ( & mep ->  valuec   ) ;
  return  false  ; 
}
  
# else

bool  ULongIsBADSizeValue ( ULongPub const * const me ,
  size_t  * const resultp ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  { size_t  const s = VectorSizeCopy  ( StackUnsInt ) ( & mep ->  valuec   ) ;
    enum { n = ( sizeof  ( size_t  )  / sizeof ( unsigned int  ) ) } ;
    if  ( s  > n  )
      return  true ; 
  }
  size_t  resultlocal = 0 ;
  { unsigned  int const * j = VectorCEnd(StackUnsInt) ( & mep ->  valuec   ) ;
    unsigned  int const * const j0 = VectorCBegin(StackUnsInt)  (
      & mep ->  valuec   ) ;
    while ( j not_eq  j0  ) {
      --  j ;
      enum { intbits = ( sizeof  ( unsigned  int )  <<  3  ) } ;  
      resultlocal <<= intbits  ;
      resultlocal or_eq ( *  j  ) ; 
    } 
  } // j j0
  ( * resultp ) = resultlocal ;
  return  false  ; 
}
  
# endif   

# if ULONG_MAX  ==  UINT_MAX    

bool  ULongisOKULValue  ( ULongPub const * const me ,
  unsigned long  int * const resultp ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  size_t  const s = VectorSizeCopy(StackUnsInt) ( & mep -> valuec  ) ;
  if ( s  > 1 )
    return  false ;
  if  ( s )
    ( * resultp ) = Vector_FrontCopy ( StackUnsInt )  ( & mep -> valuec  ) ;
  else
    ( * resultp ) = 0 ;
  return  true ; 
}
  
# else

bool  ULongisOKULValue ( ULongPub  const * const me ,
  unsigned long  int * const resultp ) {
  ULong const * const mep = ULong_toCPrivate  ( me ) ;
  enum { n = ( sizeof  ( unsigned long  int  )  /
    sizeof ( unsigned int  ) ) } ;
  size_t  const s = VectorSizeCopy(StackUnsInt) ( & mep -> valuec  ) ;
  if ( s  > n )
    return  false ;
  unsigned  long  int resultlocal = 0 ;
  enum { intbits = ( sizeof ( unsigned  int )  <<  3 ) } ;  
  unsigned  int const * j = VectorCEnd(StackUnsInt) ( & mep -> valuec  ) ;
  unsigned  int const * const j0 = VectorCBegin(StackUnsInt) (
    & mep -> valuec  ) ;
  while ( j not_eq  j0 ) {
    --  j ;
    resultlocal <<= intbits ;
    resultlocal or_eq (  * j ) ; 
  }
  ( * resultp )  = resultlocal ;
  return  true  ; 
}
  
# endif

# if WCHAR_MAX <= UINT_MAX  

void  ULongSetWChar ( ULongPub * const me , wchar_t const x  ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  if  ( x ) {
    Vector_Resize1 ( StackUnsInt ) ( & mep ->  valuec  ) ;
    { unsigned  int const x2  = wchar_cast_uint ( x ) ;
      ( * VectorAt ( StackUnsInt ) ( & mep ->  valuec   , 0UL ) ) = x2  ;
    } 
  } else
    VectorClear ( StackUnsInt ) ( &  mep ->  valuec  ) ;
}

# else

void  ULongSetWChar  ( ULongPub * const me , wchar_t  x  ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  VectorClear ( StackUnsInt ) ( &  mep ->  valuec  ) ;
  if  ( x ) {
    do  {
      unsigned  int const x2  = wchar_cast_uint ( x ) ;
      Vector_PushByValue ( StackUnsInt ) ( & mep ->  valuec   , x2  ) ;
      enum { uibs = ( sizeof  ( unsigned int  )  <<  3  ) } ;
      x >>= uibs ;
    } while ( x ) ; 
  } 
}

# endif

# if SIZE_MAX ==  UINT_MAX

void ULongSetUSize ( ULongPub * const me , size_t const x ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  VectorClear ( StackUnsInt ) ( &  mep ->  valuec  ) ;
  if  ( x ) {
    unsigned  int const x2  = size_cast_uint ( x ) ;
    Vector_PushByValue ( StackUnsInt )  ( & mep ->  valuec   , x2  ) ;
  } 
}
  
# else    

# define  ULongSetUSizeNonZero  Lisp2D_ULongSetUSizeNonZero
static  void ULongSetUSizeNonZero ( ULong  * const mep  ,
  size_t x ) {
  do  {
    Vector_PushByValue ( StackUnsInt ) ( & mep ->  valuec   ,
      size_cast_uint ( x ) ) ;
    enum { uibs = ( sizeof  ( unsigned int  )  <<  3  ) } ;
    x >>= uibs ;
  } while ( x ) ; 
}
  
void  ULongSetUSize ( ULongPub * const me , size_t  const x ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  VectorClear ( StackUnsInt ) ( & mep  ->  valuec  ) ;
  if  ( x )
    ULongSetUSizeNonZero ( mep , x ) ; 
}
  
# endif

# if ULONG_MAX ==  UINT_MAX
  
void  ULongSetULInt  ( ULongPub * const me ,
  unsigned long  int const x ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  VectorClear ( StackUnsInt ) ( & mep -> valuec  ) ;
  if ( x ) {
    unsigned  int const x2 = ulint_cast_uint ( x ) ;
    Vector_PushByValue ( StackUnsInt )  ( & mep ->  valuec   , x2  ) ;
  }  
}  
  
# else

// ! to optimize as in SetUInt64NonZero

# define  ULongSetULIntNonZero  Lisp2DULongSetULIntNonZero
static void ULongSetULIntNonZero ( ULong  * const mep ,
  unsigned long  int x ) {
  do {
    Vector_PushByValue ( StackUnsInt ) ( & mep -> valuec  ,
      ulint_cast_uint ( x ) ) ;
    enum { uibs = ( sizeof  ( unsigned int  )  <<  3  ) } ;
    x >>= uibs ;
  } while ( x ) ;
}

void ULongSetULInt ( ULongPub * const  me ,
  unsigned long  int const x ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  VectorClear ( StackUnsInt ) ( & mep -> valuec  ) ;
  if ( x )
    ULongSetULIntNonZero  ( mep , x ) ;
}
  
# endif  

void  ULongSet1 ( ULongPub * const me ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  Vector_Resize1 ( StackUnsInt )  ( &  mep  ->  valuec  )  ;
  ( * VectorFront ( StackUnsInt ) ( &  mep ->  valuec   ) )  = 1U ;
}
  
# include "ulong/inline.h"
  
bool  ULongEvenP ( ULongPub const * const me ) {
  return not  ULongMod2  ( me ) ; 
} // (x&1)==0

void  ULongSetFromBits ( ULongPub * const  me ,
  void const * const  x  , size_t const bits ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  enum { intbits = ( sizeof  ( unsigned int  )  <<  3  ) } ;
  size_t   news = bits  / intbits ;
  bool  flagHalfDigit = false ;
  if  ( ( news  * intbits )  not_eq  bits ) {
    flagHalfDigit = true  ;
    ++  news  ; 
  }
  Vector_Resize ( StackUnsInt )  ( & mep ->  valuec   , news  ) ;
  unsigned  int const * xp = voidcp_cast_uintcp ( x ) ;
  unsigned  int * iter = VectorBegin(StackUnsInt) ( & mep ->  valuec ) ;
  for ( size_t  j = 0 ; j < news ; ++ j ) {
    ( *  iter ) = ( *  xp ) ;
    ++  iter  ;
    ++  xp  ;
  }
  if  ( flagHalfDigit ) {
    --  iter  ;
    ( *  iter ) and_eq
      ( ( 1U  <<  ( bits - (  news - 1  )  * intbits  ) ) - 1 ) ; 
  }
  ULong_Simplify  ( mep ) ;
}
  
// me - clear   , swap(me,l)
void  ULongClearSwap ( ULongPub  * const me , ULongPub  * const  l ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  ULong * const lp = ULong_toPrivate  ( l ) ;
  // don't compare pointers. Swap - checks pointers yourself
  Vector_Swap ( StackUnsInt ) ( &  mep ->  valuec  , &  lp  ->  valuec  ) ;
  VectorClear ( StackUnsInt ) ( &  lp  ->  valuec  ) ;
}

// me - clear   , move(me,l)
void  ULongClearMove ( ULongPub  * const me , ULongPub  * const  l ) {
  ULong * const lp = ULong_toPrivate  ( l ) ;
  // don't compare pointers. Swap - checks pointers yourself
  Vector_Swap ( StackUnsInt ) ( & ULong_toPrivate  ( me ) ->  valuec  , &  lp  ->  valuec  ) ;
  VectorClear ( StackUnsInt ) ( &  lp  ->  valuec  ) ;
}

  // g++ - doesnot clear argument value    
  
void  ULongAdd1  ( ULongPub * const  me ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  size_t const s = VectorSizeCopy ( StackUnsInt ) ( & mep ->  valuec ) ;
  if  ( s  ==  0  ) {
    Vector_PushByValue ( StackUnsInt ) ( & mep ->  valuec   , 1U  ) ;
    return ;
  }
  unsigned  int * const p = VectorFront ( StackUnsInt ) ( & mep ->  valuec )  ;
  if ( * p == UINT_MAX  ) {
    * p = 0 ;
    size_t i  = 1 ;
    do  {
      if  ( i  ==  s  ) {
        Vector_PushByValue ( StackUnsInt ) ( & mep ->  valuec , 1U  ) ;
        return  ;
      }
      unsigned  int * const pi  = Vector_EltAddr ( StackUnsInt ) ( & mep ->  valuec , i ) ;
      if ( * pi == UINT_MAX )
        * pi = 0 ;
      else {
        ++  ( * pi ) ;
        break ;
      }
      ++  i ;
    } while ( true ) ;
  } else
    ++  ( * p ) ;
}
  
void  ULongSetFromBitsNegativeSize  ( ULongPub * const me ,
  void const * const  x  , size_t const bits  ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  enum { intbits = ( sizeof  ( unsigned int  )  <<  3  ) } ;
  size_t   news = bits  / intbits ;
  bool  flagHalfDigit = false ;
  if  ( ( news  * intbits )  not_eq  bits ) {
    flagHalfDigit = true  ;
    ++  news  ; 
  }
  Vector_Resize ( StackUnsInt )  ( & mep ->  valuec   , news  ) ;
  unsigned  int const *  xp = voidcp_cast_uintcp ( x ) ;
  unsigned  int * iter = VectorBegin(StackUnsInt) ( & mep ->  valuec ) ;
  for ( size_t  j = 0 ; j < news  ; ++ j  ) {
    ( * iter )  = (  compl  (  *  xp ) ) ;
    ++  iter  ;
    ++  xp  ; 
  }
  if  ( flagHalfDigit ) {
    --  iter  ;
    ( * iter )  and_eq  (  (
        1U  <<  ( bits - (  news - 1  )  * intbits  ) ) - 1 ) ; 
  }
  ULong_Simplify  ( mep ) ;
  ULongAdd1  ( ULongToPub ( mep ) ) ; 
}

void  ULongSubUInt  ( ULongPub * const me , unsigned int const ui ) {
  if  ( ui ==  0  )
    return ;
  ULong * const mep = ULong_toPrivate  ( me ) ;
# ifdef  LISP2D_DEBUG
  if  ( VectorEmpty ( StackUnsInt ) ( & mep ->  valuec ) ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongSubUInt:I'm zero" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  { unsigned  int * const fp  = VectorFront ( StackUnsInt ) ( & mep ->  valuec ) ;
    if ( ( * fp ) >=  ui )
      ( * fp ) -=  ui ;
    else {
      ( * fp ) +=  ( UINT_MAX  - ( ui  - 1 ) ) ;
# ifdef  LISP2D_DEBUG    
      size_t  const s = VectorSizeCopy  ( StackUnsInt ) ( & mep ->  valuec ) ;
# endif    
      size_t  i = 1UL ;
      do {
# ifdef  LISP2D_DEBUG
        if  ( i ==  s ) {
          Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
            L"Lisp2DULongSubUInt:I'm less" ;
          longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
        }
# endif
        unsigned int * const vi = Vector_EltAddr ( StackUnsInt ) ( & mep ->  valuec , i ) ;
        if ( ( * vi ) == 0 )
          ( * vi ) = UINT_MAX  ;
        else  {
          --  ( * vi ) ;
          break ;
        }
        ++  i ;
      } while ( true ) ;
    }
  }
  ULong_Simplify  ( mep ) ;
}

void  ULongSetUInt ( ULongPub * const me ,  unsigned  int const  x ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  if  ( x ) {
    Vector_Resize1 ( StackUnsInt ) (  & mep ->valuec ) ;
    ( * ( VectorFront ( StackUnsInt ) ( & mep ->  valuec ) )  ) = x ; 
  } else
    VectorClear ( StackUnsInt ) ( &mep ->valuec ) ;
}

// *= 2^bits OR <<= bits
void  ULongMul2Power ( ULongPub * const me , size_t const bits ) {
  if  ( bits ==  0 ) 
    return ;
  ULong * const mep = ULong_toPrivate  ( me ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & mep -> valuec ) ) 
    return  ;
  // this>0
  enum { intbitsize  = ( sizeof  ( unsigned  int )  * 8UL  ) } ;
  size_t const bigpow = bits  / intbitsize ;
  if  ( bigpow  )
    // <<= (sizeof(unsigned int) * 8 * bigpow)    
    Vector_InsertFront ( StackUnsInt ) ( & mep ->  valuec , 0U , bigpow ) ;
  { size_t const pow2 = bits % intbitsize ;
    if  ( pow2 ) {
      unsigned  int pere2 = 0U ;
      size_t  const sdvig = intbitsize - pow2 ;
      { unsigned  int const * const end = VectorCEnd ( StackUnsInt ) ( & mep -> valuec  ) ;
        for ( unsigned  int * j2 = ( VectorBegin(StackUnsInt) (
          & mep -> valuec  ) +
          ( ( ptrdiff_t ) bigpow  ) ) ; j2  not_eq  end ; ++  j2  ) {
          unsigned  int const pere  = ( * j2  ) >>  sdvig ;
          ( *  j2 ) <<= pow2  ;
          ( *  j2 ) or_eq  pere2  ;
          pere2 = pere ; 
        } 
      }
      if  ( pere2 )
        Vector_PushByValue ( StackUnsInt ) ( & mep ->  valuec  ,
          pere2 )  ; 
    } 
  }
  ULong_Simplify  ( mep ) ;
}  

void  ULongDifferenceEq ( ULongPub *  const me ,
  ULongPub  const * const  l ) {
  ULong * const mep = ULong_toPrivate  ( me ) ;
  if  ( me ==  l ) {
    ULongSet0  ( me ) ;
    return;  
  }
  ULong const * const lp = ULong_toCPrivate  ( l ) ;
  size_t const ls  = VectorSizeCopy ( StackUnsInt ) ( & lp  ->  valuec ) ;
  if  ( ls ==  0 ) 
    return ;
  size_t const mys = VectorSizeCopy ( StackUnsInt ) ( & mep ->  valuec )  ;
# ifdef  LISP2D_DEBUG
  if  ( mys  ==  0  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongDifferenceEq:mys  ==  0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
  if  ( mys  < ls ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongDifferenceEq:mys < ls" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  // mys >= ls
  if  ( ls ==  1  ) {
    bool  flagout ;
    unsigned  int * j = VectorBegin ( StackUnsInt ) ( & mep ->  valuec ) ;
    unsigned  int const * const jend  = VectorCEnd  ( StackUnsInt ) (
      & mep->valuec  ) ;
    unsigned  int const lv  = Vector_FrontCopy ( StackUnsInt )  ( & lp  ->  valuec ) ;
    if  ( ( * j ) < lv )  {
      ( * j ) +=  ( UINT_MAX  - ( lv  - 1 ) ) ;
      flagout = true  ;
    } else  {
      ( * j ) -=  lv  ;
      flagout = false ;
    }
    ++  j ;
    while ( flagout ) {
# ifdef  LISP2D_DEBUG
      if  ( j  ==  jend ) {
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"ULongDifferenceEq:0.me < l" ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
      }
# endif
      if  ( ( * j ) ==  0 ) {
        flagout = true  ;
        ( * j ) = UINT_MAX  ;
      } else  {
        flagout = false ;
        --  ( * j ) ;
      }
      ++  j ;
    } // while(flagout)
    if ( ( j  ==  jend ) and
      ( VectorBackCopy  ( StackUnsInt ) ( & mep ->  valuec ) ==  0 ) )
      VectorPop ( StackUnsInt ) ( & mep ->  valuec ) ;
    return  ;
  }
  unsigned  int * i = VectorBegin ( StackUnsInt ) ( & mep ->  valuec ) ;
  { bool flagpere = false ;
    { unsigned  int const * const liend =
        VectorCEnd  ( StackUnsInt ) ( & ( lp  ->  valuec ) ) ;
      for ( unsigned  int const * li =
        VectorCBegin  ( StackUnsInt ) ( & ( lp  ->  valuec ) ) ;
          li not_eq  liend  ; ++ i , ++  li  ) {
        unsigned  int s ;
        if  ( ( * i ) >=  ( * li  ) )
          s = ( * i ) - ( * li  ) ;
        else
          s = ( ( UINT_MAX  - ( * li  ) ) + ( * i ) + 1 ) ;
        if  ( flagpere  ) {
          if  ( s )
            --  s ;
          else
            s = UINT_MAX  ;
          flagpere  = ( s >=  ( * i ) ) ; 
        } else
          flagpere  = ( ( ( * li  ) not_eq  0 ) and ( s  > ( * i ) ) ) ;
        ( * i ) = s ;
      }
    } // li, liend
    unsigned  int const * const iend  = VectorCEnd  ( StackUnsInt ) (
      & mep ->  valuec ) ;
    for ( ; flagpere and ( i not_eq  iend  ) ; ++  i ) {
      if  ( * i ) {
        flagpere  = false ;
        --  ( * i ) ;
      } else
        ( * i ) = UINT_MAX ;
    }
# ifdef  LISP2D_DEBUG
    if  ( flagpere  ) {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"ULongDifferenceEq:1.me < l" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
    }
# endif
  } // flagpere
  // clear zeros ? Simplify
  i = VectorEnd ( StackUnsInt ) ( &  mep ->  valuec  ) ;
  --  i ;
  if  ( * i ) 
    return ;
  size_t j = ( mys - 1U  ) ;
  do  {
    if  ( j  ==  0  ) {
      VectorClear ( StackUnsInt ) ( &  mep ->  valuec  ) ;
      return  ;
    }
    --  j ;
    --  i ;
  } while ( ( * i )  ==  0  ) ;
  Vector_Resize ( StackUnsInt )  ( & mep ->  valuec  , j + 1 ) ;
}
  
void  ULongAddUInt  ( ULongPub * const me , unsigned int const x ) {
  if  ( x ==  0 )
    return  ;
  ULong * const mep = ULong_toPrivate  ( me ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & mep ->  valuec ) ) {
    Vector_PushByValue ( StackUnsInt ) ( & mep ->  valuec , x ) ;
    return ;
  }
  unsigned  int * iteri = VectorBegin ( StackUnsInt ) ( & mep ->  valuec )  ;
  if  ( ( * iteri ) > UINT_MAX - x )
    ( * iteri ) -=  ( ( UINT_MAX - x ) + 1 ) ;
  else {
    ( * iteri ) +=  x ;
    return  ;
  }
  unsigned  int const * const iteriend = VectorCEnd ( StackUnsInt ) (
    & ( mep ->  valuec )  ) ;
  do  {
    ++  iteri ;
    if  ( iteri  ==  iteriend ) {
      Vector_PushByValue ( StackUnsInt ) ( & mep ->  valuec ,  1U  ) ;
      return  ;
    }
    if  ( ( * iteri ) ==  UINT_MAX  )
      ( * iteri ) = 0 ;
    else  {
      ++  ( * iteri ) ;
      return  ;
    }
  } while ( true ) ;
}

void  ULongAddTemp ( ULongPub  *  const me , ULongPub  *  const l ) {
  if  ( me  ==  l ) {
    ULongMul2  ( me ) ;
    return  ; 
  }
  ULong * const mep = ULong_toPrivate  ( me ) ;
  ULong * const lp = ULong_toPrivate  ( l ) ;
  { size_t const s2 = VectorSizeCopy  ( StackUnsInt ) ( & lp  ->  valuec  ) ;
    if  ( s2 ==  0  )
      return ;
    if  ( VectorSizeCopy  ( StackUnsInt ) ( & mep ->  valuec  ) < s2  )
      ULongSwap  ( me , l ) ;
  } // s2
  // s1 >= s2      
  ULongAddPrivate ( mep , lp ) ;
}

void  ULongAddConst  ( ULongPub * const me , ULongPub  const * const l ) {
  if  ( me == l ) {
    ULongMul2  ( me ) ;
    return  ; 
  }
  ULong * const mep = ULong_toPrivate  ( me ) ;
  ULong const * const lp = ULong_toCPrivate  ( l ) ;
  { size_t const s2 = VectorSizeCopy  ( StackUnsInt ) ( & lp  ->  valuec )  ;
    if  ( s2 ==  0  )    
      return  ;
    size_t s1 = VectorSizeCopy  ( StackUnsInt ) ( & mep ->  valuec )  ;
    if  ( s1 ==  0  ) {
      VectorSet ( StackUnsInt ) ( & mep ->  valuec  , & ( lp -> valuec  ) ) ;
      return ; 
    }
    if  ( s1 < s2 ) {
      Vector_Resize ( StackUnsInt ) ( & mep  ->  valuec ,  s2 ) ;
      do  {
        ( * VectorAt ( StackUnsInt ) ( & mep ->  valuec ,  s1  ) ) = 0U  ;
        ++  s1  ;
      } while ( s1  < s2  ) ;
    }
  }
  ULongAddPrivate ( mep , lp  ) ;
}  
  
bool  ULongLess ( ULongPub  const * const  my , ULongPub const * const x ) {
  if  ( my ==  x )   
    return  false  ;
  ULong const * const myp = ULong_toCPrivate  ( my ) ;
  ULong const * const xp = ULong_toCPrivate  ( x ) ;
  { size_t  const s = VectorSizeCopy  ( StackUnsInt ) ( & myp ->  valuec )  ;
    size_t  const xs = VectorSizeCopy ( StackUnsInt ) ( & xp  ->  valuec )  ;
    if  ( s  not_eq  xs )     
      return  s  < xs ;
    // s == xs
    if  ( s  ==  0  )
      return  false ;  // 0 < 0
    // this is NOT 0
  }
  unsigned  int const * j =
    VectorCEnd  ( StackUnsInt ) ( & myp ->  valuec ) - 1  ;
  unsigned  int const * xj = 
    VectorCEnd ( StackUnsInt ) ( & ( xp  ->  valuec )  )  - 1  ;
  unsigned  int const * const jbegin = VectorCBegin ( StackUnsInt ) (
    & myp ->  valuec ) ;
  while ( j not_eq  jbegin  ) {
    { unsigned int const jv = ( * j ) ;
      unsigned int const xjv = ( * xj ) ;
      if  ( jv  not_eq  xjv )
        return  jv  < xjv ; 
    }
    --  j ;
    --  xj  ;
  }
  // this is NOT 0 - checked
  return  ( * j ) < ( * xj  ) ;
  // is equal
}

# include "stack/public.h"

bool  ULongMoreOrEqSizet ( ULongPub  const * const my , size_t const x ) {
  bool result ;
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    ULongPub * const x2lp = StackNew_ULong ( ) ;
    ULong_Init  ( x2lp ) ; // no memory
    ULongSetUSize ( x2lp , x ) ;
    result = ULongMoreOrEq  ( my , x2lp ) ;
    Lisp2D_StackFree  ( oldstack ) ;
  }
  return result ;
}

/*
!! Multiply by HALF
a*10+b
x
c*10+d
---
res = b*d+(b*c+a*d)*10+a*c*100=C0+C1*10+C2*100
C0=b*d
C2=a*c
C1=b*c+a*d=b*c+(b*d-b*d)+a*d+(a*c-a*c)=C0+b*c-b*d+C2+a*d-a*c=C0+C2+b*(c-d)+a*(d-c)=C0+C2+b*(c-d)-a(c-d)=
 = C0 + C2 + (c-d)*(b-a)
OR
 = C0 + C2 + (d-c)*(a-b)
OR
 = C0 + C2 - (c-d)*(a-b)
OR
 = C0 + C2 - (d-c)*(b-a) 
 */  

/*
a*3+b
x
c*5+d
---
res = b*d+b*c*5+a*d*3+a*c*15=C0+C1*5+C2*3+C3*15
C0=b*d
C3=a*c
C1=b*c=b*c+(b*d-b*d)=C0+b*c-b*d=C0+b*(c-d)
C2=a*d=a*d+(a*c-a*c)=C3+a*d-a*c=C3+a*(d-c)=C3-a*(c-d)
C1*5+C2*3=(C0+b*(c-d))*5+(C3-a*(c-d))*3=C0*5+C3*3+(c-d)*(b*5-a*3)


C1=b*c+a*d=b*c+(b*d-b*d)+a*d+(a*c-a*c)=C0+b*c-b*d+C2+a*d-a*c=C0+C2+b*(c-d)+a*(d-c)=C0+C2+b*(c-d)-a(c-d)=
 = C0 + C2 + (c-d)*(b-a)
OR
 = C0 + C2 + (d-c)*(a-b)
OR
 = C0 + C2 - (c-d)*(a-b)
OR
 = C0 + C2 - (d-c)*(b-a) 
 */      
      
void  ULongMultiplyConst ( ULongPub * const mep ,
  ULongPub  const * const l ) {
  Lisp2DThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  ULongPub * const mul = StackNew_ULong ( ) ;
  ULong_ConstrCopy ( mul , l ) ;
  ULongMultiplyMove ( mep , mul ) ;
  Lisp2D_StackFree  ( oldstack ) ;
}
      
void  ULongMultiplyMove ( ULongPub * const restrict me ,
  ULongPub * const  restrict l ) {
  ULong * const mep = ULong_toPrivate ( me ) ;
  if  ( me == l ) {
    ULongSquare ( me ) ;
    return ;
  }
  size_t  const * const currentsize0p = VectorSizeLink  ( StackUnsInt ) (
    & ( mep ->  valuec ) ) ;
  if  ( ( * currentsize0p ) ==  0 )
    return  ;
  ULong * const lp = ULong_toPrivate ( l ) ;
  size_t  const * const currentsize1p = VectorSizeLink  ( StackUnsInt ) (
    & ( lp  ->  valuec ) ) ;
  if  ( ( * currentsize1p ) ==  0 ) {
    VectorClear ( StackUnsInt ) ( &  mep ->  valuec ) ;
    return   ; 
  }
  // all > 0
  if  ( ( * currentsize1p ) ==  1 ) {
    unsigned int const  fc =
      Vector_FrontCopy ( StackUnsInt ) ( & ( lp  ->  valuec )  ) ;
    // operator*=(UINT - checks itself for 0 and 1
    ULongMultiplyUInt ( ULongToPub ( mep ) , fc  ) ;
    return ; 
  }
  if  ( ( * currentsize0p ) ==  1 ) {
    unsigned  int const x = Vector_FrontCopy ( StackUnsInt )  ( & mep ->  valuec )  ;
    ULongSwap ( me , l ) ;
    // operator*=(UINT - checks itself for 0 and 1
    ULongMultiplyUInt ( ULongToPub ( mep ) , x ) ;
    return ; 
  }
  if  ( ( * currentsize0p ) < ( * currentsize1p ) )  
    ULongSwap ( me , l ) ;
  // currentsize0 >= currentsize1
  size_t  const bs = (  ( * currentsize0p ) >>  1 ) ;
  ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
  ULongPub * const aplp = StackNew_ULong ( ) ;
  ULong_Init  ( aplp ) ; // no memory
  ULong * const apl = ULong_toPrivate ( aplp ) ;
  { size_t  as = (  ( * currentsize0p ) - bs  ) ;
    Vector_Resize ( StackUnsInt )  ( & apl ->  valuec ,  as  ) ;
    { size_t  i2 = (  * currentsize0p ) ;
      do  {
        --  as  ;
        --  i2  ;
        ( * VectorAt ( StackUnsInt ) ( & apl ->  valuec ,  as  ) ) =
          VectorAtCopy ( StackUnsInt ) ( & mep ->  valuec ,  i2  ) ;
      } while ( as > 0  ) ;
    }
  } // i2 , as
  // make size b without leading zeros
  size_t  newsizeb = bs ;
  do {
    if  ( VectorAtCopy ( StackUnsInt ) ( & mep ->  valuec ,  newsizeb  - 1U  )
      not_eq  0 )
      break ;
    --  newsizeb  ;
  } while ( newsizeb  not_eq 0 ) ;
  if  ( newsizeb  ) {
    Vector_Resize ( StackUnsInt )  ( & mep ->  valuec ,  newsizeb  ) ;
    if  ( ( * currentsize1p ) > bs  ) {
      ThreadStackPointerType const oldstack_c = Lisp2DThreadSP ( ) ;
      ULongPub * const cplp = StackNew_ULong ( ) ;
      ULong_Init  ( cplp ) ; // no memory
      ULong * const cpl = ULong_toPrivate ( cplp ) ;
      { size_t  cs = ( ( * currentsize1p ) - bs  ) ;
        Vector_Resize ( StackUnsInt )  ( & cpl ->  valuec ,  cs  ) ;
        { size_t  i2 = (  * currentsize1p ) ;
          do  {
            --  cs  ;
            --  i2  ;
            ( * VectorAt ( StackUnsInt ) ( & cpl ->  valuec ,  cs  ) ) =
              VectorAtCopy ( StackUnsInt ) ( & lp  ->  valuec ,  i2  ) ;
          } while ( cs > 0  ) ;
        }
      } // i2 , cs
      // make size b without leading zeros
      size_t  newsized = bs ;        
      do {
        if  ( VectorAtCopy ( StackUnsInt ) ( & lp  ->  valuec ,  newsized  - 1U  )
          not_eq  0 )
          break ;
        --  newsized  ;
      } while ( newsized  not_eq 0 ) ;
      if  ( newsized  ) {
        // d is l
        Vector_Resize ( StackUnsInt )  ( & lp  ->  valuec ,  newsized  ) ;
        // C0=b*d        
        ThreadStackPointerType const oldstack_c0 = Lisp2DThreadSP ( ) ;
        ULongPub * const c0 = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( c0 , ULongCToPub ( mep ) ) ;
        ULongMultiplyConst ( c0 , ULongCToPub ( lp ) ) ;
        // C2=a*c
        ULongPub * const c2 = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( c2 , aplp ) ;
        ULongMultiplyConst ( c2 , cplp ) ;
        ULongPub * const c1plp = StackNew_ULong ( ) ;
        ULong_Init  ( c1plp ) ; // no memory
        ULong * const c1pl = ULong_toPrivate ( c1plp ) ;
        if  ( ULongLess ( cplp , ULongCToPub ( lp ) ) ) {
          if  ( ULongLess ( ULongCToPub ( mep ) , aplp ) ) {
            // C1 = C0 + C2 + (d-c)*(a-b)
            ULongSwap ( c1plp ,  l ) ;
            ULongDifferenceEq  ( c1plp , cplp ) ;
            ULongDifferenceEq  ( aplp , ULongCToPub ( mep ) ) ;
            ULongMultiplyMove ( c1plp , aplp ) ;
            ULongAddConst  ( c1plp , c2 ) ;
            ULongAddConst  ( c1plp , c0 ) ;
            // (  * this) < a
          } else {
// ! if (me == a) then  C1 = C0 + C2          
            // not ((  * this) < a)
            // (a  < b)
            // C1 = C0 + C2 - (d-c)*(b-a)
            //c1  = c0;
            ULongSet (  c1plp ,  c0 ) ;
            ULongAddConst  ( c1plp , c2 ) ;
            ULongDifferenceEq  ( ULongToPub ( lp ) , cplp ) ;
            ULongDifferenceEq  ( ULongToPub ( mep ) , aplp ) ;
            ULongMultiplyMove ( ULongToPub ( lp ) , ULongToPub ( mep ) ) ;
            ULongDifferenceEq  ( c1plp , ULongCToPub ( lp ) ) ;
          }  // not ((  * this) < a)
// ! if (c == l) then  C1 = C0 + C2                      
        } else { // not (c  < l)
          // (d  < c)
          if  ( ULongLess ( ULongCToPub ( mep ) , aplp ) ) {
            // C1 = C0 + C2 - (c-d)*(a-b)
            //c1  = c0;
            ULongSet (  c1plp ,  c0 ) ;
            ULongAddConst  ( c1plp , c2 ) ;
            ULongDifferenceEq  ( cplp , ULongCToPub ( lp ) ) ;
            ULongDifferenceEq  ( aplp , ULongCToPub ( mep ) ) ;
            ULongMultiplyMove ( cplp , aplp ) ;
            ULongDifferenceEq  ( c1plp , cplp ) ;
            // (  * this) < a
          } else {
// ! if (me == a) then  C1 = C0 + C2                      
            // not ((  * this) < a)
            // (a  < b)
            // C1 = C0 + C2 + (c-d)*(b-a)
            ULongMove ( c1plp ,  cplp ) ;
            ULongDifferenceEq  ( c1plp , ULongCToPub ( lp ) ) ;
            ULongDifferenceEq  ( ULongToPub ( mep ) , aplp ) ;
            ULongMultiplyMove ( c1plp , ULongToPub ( mep ) ) ;
            ULongAddConst  ( c1plp , c2 ) ;
            ULongAddConst  ( c1plp , c0 ) ;
          } // not ((  * this) < a)
        }
        // res = C0+C1*10+C2*100
        ULongMove ( me , c2 ) ;
        // <<= (sizeof(unsigned int) * 8 * power)
        Vector_InsertFront ( StackUnsInt ) ( & mep ->  valuec , 0 ,
          bs  <<  1 ) ;
        // power
        Vector_InsertFront ( StackUnsInt ) ( & c1pl ->valuec , 0 , bs ) ;
        //  zero
        ULongAddTemp ( ULongToPub ( mep ) , c1plp )  ;
        ULongAddTemp ( ULongToPub ( mep ) , c0 ) ; 
        Lisp2D_StackFree  ( oldstack_c0 ) ;
        // if newsized of d
      } else { // not newsized
        // d is zero
        // a*c*100+b*c*10=10*(a*c*10+b*c)
        ULongMultiplyConst ( aplp , cplp ) ;
        ULongMultiplyMove ( ULongToPub ( mep ) , cplp ) ;
        // <<= (sizeof(unsigned int) * 8 * power)
        Vector_InsertFront ( StackUnsInt ) ( & apl ->  valuec  , 0  , bs ) ;
        ULongAddTemp ( ULongToPub ( mep ) , aplp ) ;
        Vector_InsertFront ( StackUnsInt ) ( & mep ->  valuec  , 0  , bs ) ;
      }
      // d is zero
      Lisp2D_StackFree  ( oldstack_c ) ;
      // if(currentsize1 > bs)
    } else { // not (currentsize1 > bs)
      // c is zero, d == l
      // a*d*10+b*d
      ULongMultiplyConst ( aplp , ULongCToPub ( lp ) ) ;
      ULongMultiplyMove ( ULongToPub ( mep ) , ULongToPub ( lp ) ) ;
      // <<= (sizeof(unsigned int) * 8 * power)
      Vector_InsertFront ( StackUnsInt ) ( & apl ->  valuec  , 0  , bs ) ;
      ULongAddTemp ( ULongToPub ( mep ) , aplp ) ;   
    }  // c is zero
    // if newsizeb of b
  } else  {
    // b is zero
    // a*c*100+a*d*10=10*(a*c*10+a*d)=10*a*(c*10+d)
    if  ( ( * currentsize1p ) > bs  ) {
      ThreadStackPointerType const oldstack_c = Lisp2DThreadSP ( ) ;
      ULongPub * const cplp = StackNew_ULong ( ) ;
      ULong_Init  ( cplp ) ; // no memory
      ULong * const cpl = ULong_toPrivate ( cplp ) ;
      { size_t  cs = (  ( * currentsize1p ) - bs  ) ;
        Vector_Resize ( StackUnsInt )  ( & cpl ->  valuec ,  cs  ) ;
        { size_t  i2 = (  * currentsize1p ) ;
          do  {
            --  cs  ;
            --  i2  ;
            ( * VectorAt ( StackUnsInt ) ( & cpl ->  valuec ,  cs  ) ) =
              VectorAtCopy ( StackUnsInt ) ( & lp  ->  valuec ,  i2  ) ;
          } while ( cs > 0  ) ;
        }
      } // cs
      // make size b without leading zeros
      size_t  newsized = bs ;        
      do {
        if  ( VectorAtCopy ( StackUnsInt ) ( & lp  ->  valuec ,  newsized  - 1U  )
          not_eq  0 )
          break ;
        --  newsized  ;
      } while ( newsized not_eq 0 ) ;
      if  ( newsized  ) {
        Vector_Resize ( StackUnsInt )  ( & lp  ->  valuec ,  newsized  ) ;
        ULongMultiplyConst ( cplp , aplp ) ;
        ULongMultiplyMove ( ULongToPub ( lp ) , aplp ) ;
        // <<= (sizeof(unsigned int) * 8 * power)
        Vector_InsertFront ( StackUnsInt ) ( & cpl ->  valuec  , 0  , bs  ) ;
        ULongAddTemp ( cplp , ULongToPub ( lp ) ) ;
        Vector_InsertFront ( StackUnsInt ) ( & cpl ->  valuec  , 0  , bs  ) ;
        // zero
        ULongMove ( me , cplp ) ;
        // if newsized
      } else {
        // b is zero , d is zero
        ULongMultiplyMove ( aplp , cplp ) ;
        { size_t  const  power = (  bs  <<  1 ) ;
          // <<= (sizeof(unsigned int) * 8 * power)
          Vector_InsertFront ( StackUnsInt ) ( & apl ->  valuec  , 0  ,
            power ) ;          
        }  // power
        ULongMove ( me , aplp ) ;
      } // b is zero , d is zero
      Lisp2D_StackFree  ( oldstack_c ) ;
      //  if(currentsize1 > bs) , b is zero , c
    } else {
      // b is zero , c is zero , d is l
      ULongMultiplyMove ( aplp , ULongToPub ( lp ) ) ;
      Vector_InsertFront ( StackUnsInt ) ( & apl ->  valuec  , 0  , bs  ) ;
      ULongMove ( me , aplp ) ;
    } // b is zero , c is zero , d is l
  }  // b is zero
  Lisp2D_StackFree  ( oldstack_a ) ;
}

# define  UIntSwapValue Lisp2D_UIntSwapValue
inline  void UIntSwapValue  ( unsigned  int * a  , unsigned  int * b  ) ;
  
/*
!? Multiply by HALF
b
x
c*10+d
---
res = b*d+(b*c)*10=C0+C1*10
C0=b*d
C1=b*c=b*c+(b*d-b*d)=C0+b*c-b*d=C0+b*(c-d)
?!
how to speed up here ?
bonus only with big-int > half integer size
 */  
  
void  ULongMultiplyUInt ( ULongPub * const my , unsigned int const ui  ) {
  ULong * const myp = ULong_toPrivate ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec   ) )
    return  ;
  switch  ( ui  ) {
  case  0 :
    VectorClear ( StackUnsInt ) ( &  myp ->  valuec  ) ;
    return  ;
  case  1 :
    return  ;
  case  2 :
    ULongMul2  ( ULongToPub ( myp ) ) ;
    return  ;
  default : ;
  }
  { size_t  const * const currentvaluesizep = VectorSizeLink  ( StackUnsInt ) (
      & myp ->  valuec ) ;
  if  ( ( * currentvaluesizep ) ==  1 ) {
    unsigned  int * const vfp = VectorFront ( StackUnsInt ) (
      & myp ->  valuec )  ;
# ifdef  LISP2D_DEBUG
    if  ( ( * vfp ) ==  0 ) {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"Lisp2DULongMultiplyUInt_ExcMem:"
        L"(*VectorFront(StackUnsInt)(&myp->valuec))==0" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
    }
# endif    
    if  ( ( * vfp ) ==  1 ) {
      ( * vfp )  = ui ;
      return  ;
    }
    if  ( ( * vfp ) ==  2 ) {
      ( * vfp )  = ui ;
      ULongMul2  ( ULongToPub ( myp ) ) ;
      return  ;
    }
  }  // if(currentvaluesize ==  1)
  if  ( ui ==  2  ) {
    ULongMul2 ( ULongToPub ( myp ) ) ;
    return  ;
  }
  // ui > 2 , this > 2
  { unsigned  int ul2 = ui  ;
  
// ! to remove currentmep - BUG danger  
    unsigned  int * currentmep = VectorFront  ( StackUnsInt ) (
      & myp ->  valuec )  ;
  
    if  ( Lisp2DULongLessUInt ( ULongCToPub ( myp ) , ul2 ) )
      UIntSwapValue ( & ul2 , currentmep  ) ;
  // 2 < ul2 <= this
  if  ( ( * currentvaluesizep ) ==  1 ) {
    /*
      a*10+b
      c*10+d
      c0=b*d <100 >=0
      c1=a*c <100 >=0
      c2=b*c+a*d=b*c+(b*d-b*d)+a*d+(a*c-a*c)=c0+(b*c-b*d)+c1+(a*d-a*c)=
        =c0+c1+b*(c-d)+a*(d-c)=c0+c1+b*(c-d)-a*(c-d)
        =c0+c1+(c-d)*(b-a)
        =c0+c1-(c-d)*(a-b) <200 >=0
      result = c0 + c2*10 + c1*100
    */
    
    enum { halfBitSizeOfUnsInt = (sizeof(unsigned int)  <<  2) } ;
    unsigned  int const mehigh =  ( ( * currentmep  )  >> 
      halfBitSizeOfUnsInt ) ;
    unsigned  int const melow = ( ( ( * currentmep  ) <<
      halfBitSizeOfUnsInt ) >> halfBitSizeOfUnsInt  ) ;
    unsigned  int const uihigh = (  ul2  >>  halfBitSizeOfUnsInt  ) ;
    unsigned  int const uilow = ( ( ul2 <<  halfBitSizeOfUnsInt ) >> 
      halfBitSizeOfUnsInt ) ;
    unsigned  int reshigh = mehigh * uihigh ;
    
    if  ( reshigh ) {
      
      Vector_PushByValue ( StackUnsInt ) ( & myp ->  valuec ,  reshigh ) ;
      
      // ! currentmep now is not valid
      currentmep = VectorFront  ( StackUnsInt ) ( & myp ->  valuec )  ;
    }
    // currentvaluesize ==  2
    unsigned  int const reslow = melow * uilow ;
    ( * currentmep  ) = reslow  ;
    unsigned  int resmedium ;
    bool signminus =  true  ;
    if  ( mehigh < melow  ) {
      resmedium = melow  - mehigh ;
      signminus = false ; 
    } else    
      resmedium = mehigh - melow ;
    if  ( uihigh < uilow  ) {
      resmedium *=  ( uilow  - uihigh ) ;
      signminus = not signminus ;
    } else
      resmedium *=  ( uihigh - uilow  ) ;
    bool medoverflow = false ;
    if  ( signminus ) {
      // resmedium = (reshigh - resmedium) + reslow;
      unsigned  int backupreshigh = reshigh ;
      reshigh -=  resmedium ;
      if  ( reshigh  > backupreshigh  ) 
        reshigh +=  reslow  ;
      else  {
        backupreshigh = reshigh ;
        reshigh +=  reslow  ;
        if  ( reshigh  < backupreshigh  )
          medoverflow = true;
      }
      resmedium = reshigh ;
    } else {
      // resmedium +=  (reshigh + reslow);
      unsigned  int backupresmedium = resmedium ;
      resmedium +=  reshigh ;
      if  ( resmedium  < backupresmedium  ) {
        medoverflow = true  ;
        resmedium +=  reslow  ;
      } else {
        backupresmedium = resmedium ;
        resmedium +=  reslow  ;
        if  ( resmedium  < backupresmedium  )
          medoverflow = true  ;
      }
    }
    unsigned  int mediumhigh = resmedium >>  halfBitSizeOfUnsInt ;
    if  ( medoverflow ) 
      mediumhigh  +=  ( 1U <<  halfBitSizeOfUnsInt  ) ;
    if  ( mediumhigh  ) {
      if  ( ( * currentvaluesizep ) ==  1 ) {
        
        Vector_PushByValue ( StackUnsInt ) ( & myp ->  valuec ,
          mediumhigh  ) ;
        
        // currentvaluesize ==  2
        // ! currentmep now is not valid
        currentmep = VectorFront  ( StackUnsInt ) ( & myp ->  valuec )  ;  
      }
      else
        ( * VectorAt ( StackUnsInt ) ( & myp ->  valuec ,  1UL ) )  +=  mediumhigh ;
    }
    unsigned  int const mediumlow = resmedium  <<  halfBitSizeOfUnsInt ;
    //( * this) +=  mediumlow;
    unsigned  int const backuplow = ( * currentmep  ) ;
    ( * currentmep  ) +=  mediumlow ;
    if  ( ( * currentmep  ) < backuplow ) {
      if  ( ( * currentvaluesizep ) ==  1 )
        Vector_PushByValue ( StackUnsInt ) ( & myp ->  valuec ,  1U  ) ;
      else
        ++  ( * VectorAt ( StackUnsInt ) ( & myp ->  valuec ,  1UL ) ) ;
    }
    return  ;
  } // if currentvaluesize == 1
  // currentvaluesize > 1
# ifdef  LISP2D_DEBUG
  if  ( VectorBackCopy  ( StackUnsInt ) ( & myp ->  valuec )  ==  0 ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongMultiplyUInt_ExcMem:"
      L"VectorBackCopy(StackUnsInt)(& myp->valuec)  ==  0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  /*
      a*10+b
      d
      parallel:
      c0=b*d <100 >=0
      c2=a*d <100 >=0
      result = c0 + c2*10
    */
  { // not ( nthreads  > 1 )
    // conseq
    size_t  const bs = ( * currentvaluesizep ) >>  1 ;
    // bs >= 1
    ThreadStackPointerType const oldstack_a = Lisp2DThreadSP ( ) ;
    ULongPub * const aplp = StackNew_ULong ( ) ;
    ULong_Init  ( aplp ) ; // no memory
    ULong * const apl = ULong_toPrivate ( aplp ) ;
    { size_t  const as = (  * currentvaluesizep ) - bs ;
      Vector_Resize ( StackUnsInt )  ( & apl ->  valuec ,  as  ) ;
      for ( size_t  i = as  , i2 = (  * currentvaluesizep ) ; i > 0 ; ) {
        --  i ;
        --  i2  ;
        ( * VectorAt ( StackUnsInt ) ( & apl ->  valuec ,  i ) ) =
          VectorAtCopy ( StackUnsInt ) ( & myp ->  valuec ,  i2  ) ;
      }
# ifdef  LISP2D_DEBUG      
      if  ( VectorBackCopy  ( StackUnsInt ) ( & apl ->  valuec )  ==  0 ) {
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"Lisp2DULongMultiplyUInt_ExcMem:"
          L"VectorBackCopy(StackUnsInt)(& ap->valuec)  ==  0" ;
        Lisp2D_StackFree  ( oldstack_a ) ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
      }
# endif
    }  // as          
    // make size b without leading zeros
    { size_t  newsize = bs  ;
      do {
        if  ( VectorAtCopy ( StackUnsInt ) ( & myp ->  valuec ,  newsize  - 1U ) not_eq  0 )
          break ;
        --  newsize ;
      } while ( newsize not_eq 0  ) ;
      // BUG !! base.valuec[bs-1] may be ZERO
      Vector_Resize ( StackUnsInt )  ( & myp ->  valuec ,  newsize ) ;
    } // newsize
# ifdef  LISP2D_DEBUG      
    if  ( not  VectorEmpty ( StackUnsInt ) ( & myp ->  valuec ) )
      if  ( VectorBackCopy  ( StackUnsInt ) ( & myp ->  valuec )  ==  0) {
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"Lisp2DULongMultiplyUInt_ExcMem:"
          L"1.VectorBackCopy(StackUnsInt)(& myp->valuec)  ==  0" ;
        Lisp2D_StackFree  ( oldstack_a ) ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
      }
# endif      
    // ! recursion make wrong statistics time    
    ULongMultiplyUInt ( aplp , ul2 ) ;
    ULongMultiplyUInt ( ULongToPub ( myp ) , ul2 ) ;
# ifdef  LISP2D_DEBUG      
    if  ( not VectorEmpty ( StackUnsInt ) ( & myp ->  valuec ) )
      if  ( VectorBackCopy  ( StackUnsInt ) ( & myp ->  valuec )  ==  0 ) {
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"Lisp2DULongMultiplyUInt_ExcMem:"
          L"2.VectorBackCopy(StackUnsInt)(& myp->valuec)  ==  0" ;
        Lisp2D_StackFree  ( oldstack_a ) ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
      }
# endif   
    // ! as has wrong base.valuec now
    { size_t  const * const currentavsp =
        VectorSizeLink ( StackUnsInt ) ( & ( apl  ->  valuec ) ) ;
# ifdef  LISP2D_DEBUG
    if  ( * currentavsp ) {
      if  ( VectorBackCopy  ( StackUnsInt ) ( & apl ->  valuec )  ==  0 ) {
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"Lisp2DULongMultiplyUInt_ExcMem:"
          L"VectorBackCopy(StackUnsInt)(&ap->valuec)  ==  0" ;
        Lisp2D_StackFree  ( oldstack_a ) ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
      }
    } else  {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"Lisp2DULongMultiplyUInt_ExcMem:(*currentavsp)  ==  0" ;
      Lisp2D_StackFree  ( oldstack_a ) ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
    }
# endif      
    size_t  const wasavs = (  * currentavsp ) ;  
    { size_t  const abs = ( * currentavsp ) + bs ;
      Vector_Resize ( StackUnsInt )  ( & apl ->  valuec ,  abs ) ;
      for ( size_t  i = wasavs  , i2 =  abs ; i > 0 ; ) {
        --  i ;
        --  i2  ;
        ( * VectorAt ( StackUnsInt ) ( & apl ->  valuec ,  i2  ) ) =
          VectorAtCopy ( StackUnsInt ) ( & apl ->  valuec ,  i ) ;
      }
    } // abs
    for ( size_t  i = bs  ; i > 0 ; ) {
      --  i ;
      ( * VectorAt ( StackUnsInt ) ( & apl ->  valuec ,  i ) ) = 0 ;
    }
# ifdef  LISP2D_DEBUG
    if  ( VectorBackCopy  ( StackUnsInt ) ( & apl ->  valuec )  ==  0 ) {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"Lisp2DULongMultiplyUInt_ExcMem:"
        L"VectorBackCopy(StackUnsInt)(&ap->valuec)  ==  0" ;
      Lisp2D_StackFree  ( oldstack_a ) ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
    }
# endif            
    } // currentavs
    //( * this) +=  b;
    //( * this) +=  a;
    // a - is big , me is small
    ULongAddTemp ( ULongToPub ( myp ) , aplp ) ;
# ifdef  LISP2D_DEBUG      
    if  ( not  (  VectorEmpty ( StackUnsInt ) ( & myp ->  valuec   ) ) )
      if  ( VectorBackCopy  ( StackUnsInt ) ( & myp ->  valuec )  ==  0 ) {
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"Lisp2DULongMultiplyUInt_ExcMem:"
          L"3.VectorBackCopy(StackUnsInt)(&myp->valuec)  ==  0" ;
        Lisp2D_StackFree  ( oldstack_a ) ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
      }
# endif       
    Lisp2D_StackFree  ( oldstack_a ) ;
  } // bs , a
    
  // a - new , b - new , average wrong , to see fulltime
  // n=29453838 fulltime=8.69 avrtime=2.95e-07 (2.11e-07 .. 3.79e-07)
  
  // a - new , b = this , average wrong , to see fulltime
  // n=29614975 fulltime=8.6 avrtime=2.9e-07 (2.07e-07 .. 3.73e-07)
  
  // make method Multiply
  // n=22011660 fulltime=7.81 avrtime=3.55e-07 (2.17e-07 .. 4.93e-07)
    
    } // ul2 res
  } //  currentvaluesize
}
  
void  ULongFactorial ( ULongPub * const my ) {
  ULong * const myp = ULong_toPrivate ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec ) )    
    // 0!=1
    Vector_PushByValue ( StackUnsInt ) ( & myp ->  valuec ,  1U  ) ;
  else  {
    if  ( VectorSizeCopy  ( StackUnsInt ) ( & myp ->  valuec )  ==  1UL ) {
      unsigned  int const fc = Vector_FrontCopy ( StackUnsInt ) (
        & myp ->  valuec ) ;
      if  ( fc ==  1U )
        // 1!=1
        return ;
      if  ( fc ==  2U )
        // 2!=2
        return  ; 
    }
    size_t  y ;
    { ThreadStackPointerType const oldstack_x = Lisp2DThreadSP ( ) ;
      ULongPub * const x = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( x , ULongCToPub ( myp ) ) ;
      ULong * const xp = ULong_toPrivate ( x ) ;
      if  ( ULongOddP ( ULongCToPub ( myp ) ) ) {
        // ((this-1)/2)!*this!!*(2^((this-1)/2))
        ULongSub1 ( x ) ;
        ULongDiv2 ( x ) ;
        if  ( VectorSizeCopy  ( StackUnsInt ) ( & xp  ->  valuec )  > 1UL ) {
          // infinity
          VectorClear ( StackUnsInt ) ( &  myp ->  valuec  ) ;
          Lisp2D_StackFree  ( oldstack_x ) ;
          return  ; 
        }
        if  ( ULongIsBADSizeValue  ( x , & y ) ) {
          Lisp2D_StackFree  ( oldstack_x ) ;
          return ; 
        }
        ULongFactorial ( x ) ;
      } else {
        // (this/2)!*(this-1)!!*(2^(this/2))
        ULongDiv2 ( x ) ;
        if  ( VectorSizeCopy  ( StackUnsInt ) ( & xp  ->  valuec )  > 1UL ) {
          // infinity
          VectorClear ( StackUnsInt ) (  & myp ->  valuec  ) ;
          Lisp2D_StackFree  ( oldstack_x ) ;
          return ; 
        }
        if  ( ULongIsBADSizeValue  ( x , & y ) ) {
          Lisp2D_StackFree  ( oldstack_x ) ;
          return ; 
        }
        ULongFactorial ( x ) ;
        ULongSub1 ( my ) ; 
      }
      Lisp2DULongPrivateFactorial2_ExcMem ( myp ) ;
      ULongMultiplyMove ( ULongToPub ( myp ) , x ) ;
      Lisp2D_StackFree  ( oldstack_x ) ;
    } // x
    ULongMul2Power ( ULongToPub ( myp ) , y ) ;
  } 
}

// !!x=x*(x-2)*(x-4)*...*3     x-odd , >2
void Lisp2DULongPrivateFactorial2_ExcMem ( ULong * const  mep  ) {
  if ( Lisp2DULongMoreUInt ( ULongCToPub ( mep ) , 3U  ) ) {
    ThreadStackPointerType const oldstack_y = Lisp2DThreadSP ( ) ;
    ULongPub * const y = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( y , ULongCToPub ( mep ) ) ;
    do  {
      ULongSubUInt  ( y , 2U  ) ;
      ULongMultiplyConst ( ULongToPub ( mep ) , y ) ;
    } while ( Lisp2DULongMoreUInt ( y , 3U ) ) ;
    Lisp2D_StackFree  ( oldstack_y ) ;
  }
}

// me=Floor[sqrt(myp)]
void  ULongISqrt ( ULongPub * const my ) {
  ULong * const myp = ULong_toPrivate ( my ) ;
  size_t const  s = VectorSizeCopy  ( StackUnsInt ) ( & myp ->  valuec )  ;
  if  ( s ) {
    ThreadStackPointerType const oldstack_bit = Lisp2DThreadSP ( ) ;
    ULongPub * const bitplp = StackNew_ULong ( ) ;
    ULong_Init  ( bitplp ) ; // no memory
    ULong * const bitpl = ULong_toPrivate ( bitplp ) ;
    Vector_Resize ( StackUnsInt )  ( & bitpl ->  valuec ,  s - 1 ) ;
    for ( size_t  i = ( s - 1 ) ; i > 0 ; ) {
      -- i  ;
      ( * VectorAt ( StackUnsInt ) ( & bitpl ->  valuec ,  i ) ) = 0U  ;
    }
    Vector_PushByValue ( StackUnsInt ) ( & bitpl ->  valuec ,
      1U <<  (  ( sizeof  ( unsigned int  ) <<  3 ) - 2 ) ) ;
    // bit.last = [010000]
    while ( ULongMore ( bitplp , my ) )
      ULongDiv2Power  ( bitplp , 2UL ) ;
    // bit <= this
    ULongPub * const num = StackNew_ULong ( ) ;
    ULongConstrMove ( num , my ) ;
    ULongPub * const thisplusbitplp = StackNew_ULong ( ) ;
    ULong_Init  ( thisplusbitplp ) ;
    while ( not ( VectorEmpty ( StackUnsInt ) ( & bitpl ->  valuec   ) ) ) {
      ULongSet (  thisplusbitplp , my ) ;
      ULongAddConst  ( thisplusbitplp , bitplp ) ;
      if  ( ULongMoreOrEq ( num , thisplusbitplp ) ) {
        ULongDifferenceEq  ( num , thisplusbitplp ) ;
        ULongDiv2 ( my ) ; // this=(this>>1)+bit
        ULongAddConst  ( my , bitplp ) ;
      } else
        ULongDiv2 ( my ) ;
      ULongDiv2Power  ( bitplp , 2UL ) ;
    } // while not empty bit
    Lisp2D_StackFree  ( oldstack_bit ) ;
  }
}

/*

sqrt(S)=a/b

a[0]=1
b[0]=1

a[1]=1+S
b[1]=2

a[2]=1+3*S
b[2]=3+S

a[3]=1+6*S+S^2
b[3]=4+4*S

a[n]=a[n-1]+b[n-1]*S
b[n]=a[n-1]+b[n-1]

abs(eps) < 1/(2*(b^2))

! very large A and B

*/

/*

sqrt(S) : quadratically

x[0]=1

x[1]=(1+S)/2

x[k]=(x[k-1]+(S/x[k-1]))/2

! very hard number -> very HARD answer with few steps

*/

// returns  GCD, not disturb itself
//Evclide
unsigned  int ULongGCD3 ( ULongPub const * const my ,
  unsigned  int x ) {
  ULong const * const myp = ULong_toCPrivate ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec  ) or  ( x ==  0 ) )
    return 0 ;
  if  ( x ==  1 ) 
    return 1  ;
  unsigned  int result  ;
  { unsigned  int y ;
    if  ( Lisp2DULongMoreUInt ( ULongCToPub ( myp ) , x ) ) {
      ThreadStackPointerType const oldstack_me = Lisp2DThreadSP ( ) ;
      ULongPub * const me = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( me , ULongCToPub ( myp ) ) ;
      ULong * const mep = ULong_toPrivate ( me ) ;
      ULongSetRemainderUInt ( mep , x ) ; 
      if  ( ULongIs0 ( me ) ) {
        result  = x ;
        Lisp2D_StackFree  ( oldstack_me ) ;
        goto  ExitInner  ;
      }
      y =  Vector_FrontCopy ( StackUnsInt ) ( & mep ->  valuec  ) ;
      Lisp2D_StackFree  ( oldstack_me ) ;
      // this > x , me
    } else
      y =  Vector_FrontCopy ( StackUnsInt ) ( & myp ->  valuec  )  ;
    // this>0
    do  {
      x %=  y  ;
      if  ( x  ==  0 ) {
        result  = y ;
        goto  ExitInner  ; 
      }
      y %=  x  ;
    } while ( y not_eq 0  ) ;
    result  = x  ;
ExitInner : ;
  } // y  
  return  result  ; 
}

# define  ULongHasPowerOfTwo  Lisp2D_ULongHasPowerOfTwo
size_t  ULongHasPowerOfTwo ( ULong  const * xp ) ;

// 7*(2^10) -> this = 7 , result = 10
// returns power
// exceptions - may be
# define  ULongRemovePowerOfTwo Lisp2DULongRemovePowerOfTwo
static  size_t  ULongRemovePowerOfTwo  ( ULong * const x ) {
  size_t  const powme = ULongHasPowerOfTwo ( x ) ;
  ULongDiv2Power  ( ULongToPub ( x ) , powme ) ;
  return  powme ;
}

bool  ULongIsx2x5 ( ULongPub const * const my ) {
  enum { five  = 5U } ;
  ULong const * const myp = ULong_toCPrivate ( my ) ;
  { size_t  const  mysize = VectorSizeCopy  ( StackUnsInt ) (
      & myp ->  valuec  )  ;
    // 0 1 2
    if  ( mysize  ==  0 )
      return  true  ; //0
    if  ( mysize  ==  1 ) {
      unsigned  int myvalue = Vector_FrontCopy ( StackUnsInt ) (
        & myp ->  valuec )  ;
      if  ( myvalue < 3 )
        return  true  ; //1 2
      while ( ( myvalue bitand 1  ) ==  0 )
        myvalue  >>= 1  ;
      if  ( myvalue ==  1 )
        return  true  ;
      while ( ( myvalue % five  ) ==  0 )
        myvalue  /=  five ;
      return  ( myvalue ==  1 ) ; 
    } 
  }  // mysize
  { ThreadStackPointerType const oldstack_mycopy = Lisp2DThreadSP ( ) ;
    ULongPub * const mycopy = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( mycopy , ULongCToPub ( myp ) ) ;
// ! very slow - NxDiv2  
// ! to make: calc N-0s and >>=N
    ULongRemovePowerOfTwo  ( ULong_toPrivate ( mycopy ) ) ;
    while (  Lisp2DULongMoreUInt ( mycopy , 1U ) )
      if  ( ULongDividingAndReturnsRemainder ( mycopy , five  ) ) {
        Lisp2D_StackFree  ( oldstack_mycopy ) ;
        return  false ; 
      }
    Lisp2D_StackFree  ( oldstack_mycopy ) ;
  } // mycopy
  return  true  ;
}

# if 0

// slow

# define  mul_add Lisp2D_mul_add
static  void  mul_add ( ULongPub const * const factor , unsigned  int const term  ,
  ULongPub  * const number ) {
  ThreadStackPointerType const oldstack_carry = Lisp2DThreadSP ( ) ;
  ULongPub * const carry = StackNew_ULong ( ) ;
  ULongConstrUInt ( carry , term  ) ;
  Vector  ( StackUnsInt ) * const ve  = & ULong_toPrivate ( number  ) ->  valuec  ;
  ULongPub * const p = StackNew_ULong ( ) ;
  ULong_Init  ( p ) ;
  Vector  ( StackUnsInt ) * const vp  = & ULong_toPrivate ( p ) ->  valuec  ;
  { unsigned  int const * const digitend  = VectorCEnd  ( StackUnsInt ) ( ve  ) ;
    for ( unsigned int * digit = VectorBegin  ( StackUnsInt ) ( ve  ) ;
      digit < digitend ; ++ digit ) {
      ULongSet ( p , factor ) ;
      ULongMultiplyUInt ( p , * digit ) ;
      ULongAddTemp  ( p , carry ) ;
      if ( not VectorEmpty ( StackUnsInt ) ( vp ) ) {
        * digit = Vector_FrontCopy ( StackUnsInt ) ( vp ) ;
        Vector_RemoveFront ( StackUnsInt ) ( vp , 1 ) ;
      } else
        * digit = 0 ;
      ULongSwap ( carry , p ) ;
    }
  }
  Vector  ( StackUnsInt ) * const vc  = & ULong_toPrivate ( carry ) ->  valuec  ;
  while ( not ULongIs0 ( carry ) ) {
    Vector_PushByValue  ( StackUnsInt ) ( ve  , Vector_FrontCopy ( StackUnsInt ) ( vc ) ) ;
    Vector_RemoveFront ( StackUnsInt ) ( vc , 1 ) ;
  }
  Lisp2DStackFree_Exc ( oldstack_carry ) ;
}

# endif

# include "sharedc.h"
# include "shared/inline.h"
# include "statistics.h"

void  ULongSquare ( ULongPub * const me ) {
# ifdef  LISP2D_BIMSTAT
  struct  timespec  timeBegin ;
  clock_gettime ( CLOCK_REALTIME  , & timeBegin ) ;
# endif
# if  1
// BIM(ULong,Square) n=115167 fulltime=1.15 M=9.95e-06 σ=9.41e-05 ( n=100774 ( 87.5% ) M=2e-06
//  σ=1.36e-06 ,  n=14393 ( 12.5% ) M=6.56e-05 σ=0.000259 ) mutexstat = 113319/127415=88.9%
// ! to optimize
//(a*10+b)^2 = (a*10+b)*(a*10+b) = a^2*100 + 2*a*b*10 + b^2 =
//  or ?
//  = a^2*100 + (b^2 + a^2 - |a-b|^2)*10 + b^2
  ThreadStackPointerType const oldstack_me2 = Lisp2DThreadSP ( ) ;
  ULongPub * const me2 = StackNew_ULong ( ) ;
  ULong_ConstrCopy ( me2 , me ) ;
  ULongMultiplyMove ( me , me2 ) ;
  Lisp2D_StackFree  ( oldstack_me2 ) ;
# else

  // slow

// BIM(ULong,Square) n=114380 fulltime=1.83 M=1.6e-05 σ=7.24e-05 ( n=80085 ( 70% ) M=8.28e-06
//  σ=6.01e-06 ,  n=34295 ( 30% ) M=3.41e-05 σ=0.00013 ) mutexstat = 112779/131559=85.7%
  ThreadStackPointerType const oldstack_res = Lisp2DThreadSP ( ) ;
  ULongPub * const res = StackNew_ULong ( ) ;
  // res = 0
  ULong_Init  ( res ) ;
  // 100(n) -> ?(2^32)
  mul_add ( me  , 1 , res ) ;
  mul_add ( me  , 0 , res ) ;
  mul_add ( me  , 0 , res ) ;
  ULongSwap ( me  , res ) ;
  Lisp2DStackFree_Exc ( oldstack_res ) ;
# endif
# ifdef  LISP2D_BIMSTAT
  struct  timespec  timeEnd ;
  clock_gettime ( CLOCK_REALTIME  , &  timeEnd  ) ;
  IncreaseStat5  (  & timeBegin , & timeEnd , & statULongSquare  ) ;
# endif
}

// this=this^n
void  ULongPowerSize ( ULongPub * const my , size_t const  n )  {
# if 0

  // slow

  ThreadStackPointerType const oldstack_res = Lisp2DThreadSP ( ) ;
  ULongPub * const res = StackNew_ULong ( ) ;
  // res = 0
  ULong_Init  ( res ) ;
  mul_add ( my , 1 , res ) ;
  for ( size_t  i = 0 ; i < n ; ++  i )
    mul_add ( my  , 0 , res ) ;
  ULongSwap ( my  , res ) ;
  Lisp2DStackFree_Exc ( oldstack_res ) ;
# else
  ULongPowerSizet ( my  , n ) ;
# endif
}

// this=this^n
void  ULongPowerSizet ( ULongPub * const my , size_t const  n ) {
  switch  ( n ) {
  case  0 :
    ULongSet1 ( my ) ;
    break ;
  case  1 :
    break ;
  case  2 :
    ULongSquare ( my ) ;
    break ;
  default :
    // 0^3 == 0 , 1^3 == 1
    if  ( ULongIs0  ( my  ) or ULongIs1  ( my  ) )
      return  ;
    // 2^3 == 2<<(3-1)
    if ( ULongisEqUInt ( my , 2U  ) ) {
      ULongMul2Power  ( my  , n - 1 ) ;
      return  ;
    }
    if  ( n  bitand 1U  ) {
        ThreadStackPointerType const oldstack_half = Lisp2DThreadSP ( ) ;
        ULongPub * const half = StackNew_ULong ( ) ;
        ULong_ConstrCopy ( half , my ) ;
        ULongPowerSizet ( half , n >> 1  ) ;
        ULongSquare ( half ) ;
        ULongMultiplyConst ( half , my ) ;
        ULongSwap ( my , half ) ;
        Lisp2D_StackFree  ( oldstack_half ) ;
    } else {
        ULongPowerSizet ( my , n >> 1  ) ;
        ULongSquare ( my ) ;
    }
  }
}

// ! class wstring needed

/*

Result = 0

2^(m-1) <= M < 2^m          2^(n-1) <= N < 2^n
 
2^(m-n-1)  <  M/N < 2^(m-n+1)
  
dResult = [ M >> n ]

Result += dResult

dResult is near < 4x

M -= ( N * dResult )
  
goto Start ;
  
 */  
  

/*  

SLOW

Result = 0

UP:

M/N = (2^m+a)/(2^n+b) > (2^m)/(2^(n+1)) = 2^(m-n-1)
  
while m > n 
  
  M -= (N << (m-n-1))  ;  Result += 2^(m-n-1)

  goto UP ;  
  
end while
  
if m = n then
  
 while M >= N
   
   M -= N ; Result += 1
   
 end while  
 
end if m = n
   
return Result ;   
   
*/  

# define  SizetMin  Lisp2DSizetMin
inline size_t SizetMin  ( size_t a , size_t b ) ;

// binary Evclide
// ! to add ULong &&  
void ULongGCD ( ULongPub * restrict const my ,
  ULongPub const * restrict const x ) {
  if  ( my  ==  x )
    goto  Exit  ;
  ULong * const myp = ULong_toPrivate ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec  ) )
    goto  Exit  ;
  ULong const * const xp = ULong_toCPrivate ( x ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & xp  ->  valuec  ) ) {
    ULongSet0  ( my ) ;
    goto  Exit  ;    
  }
  if  ( VectorSizeCopy  ( StackUnsInt ) ( & xp  ->  valuec  ) ==  1UL ) {
    ULongSetUInt ( my , ULongGCD3 ( my ,
      * VectorCFront  ( StackUnsInt ) ( &  ( xp ->  valuec  ) ) ) ) ;  
    goto  Exit  ; 
  }
  if  ( VectorSizeCopy  ( StackUnsInt ) ( & myp ->  valuec ) ==  1UL ) {
    ULongSetUInt ( my , ULongGCD3 ( x ,
      * VectorCFront  ( StackUnsInt ) ( & myp->valuec  ) ) ) ;
    goto  Exit  ;
  }
  { size_t  powall  ;
    { ThreadStackPointerType const oldstack_x2 = Lisp2DThreadSP ( ) ;
      ULongPub * const x2plp = StackNew_ULong ( ) ;
      ULong_Init  ( x2plp ) ;
      ULong * const x2pl = ULong_toPrivate ( x2plp ) ;
      { size_t  const powme = ULongRemovePowerOfTwo  ( myp )  ;
        ULongSet ( x2plp , x ) ;
        size_t  const powx  = ULongRemovePowerOfTwo  ( x2pl )  ;
        powall  = Lisp2DSizetMin ( powme , powx  ) ;
      }
Again :
      // this is odd and x2 is odd
      if  ( ULongMoreOrEq ( my , x2plp ) ) {
        ULongDifferenceEq ( my , x2plp ) ;
        if  ( ULongIs0 ( my ) )
          goto  Result  ;
        ULongRemovePowerOfTwo  ( myp ) ;
      } else  {
        ULongDifferenceEq ( x2plp , my ) ;
        ULongRemovePowerOfTwo  ( x2pl ) ;
      }
      goto  Again ;
Result  :
      ULongSwap  ( my , x2plp ) ;
      Lisp2D_StackFree  ( oldstack_x2 ) ;
    } // x2    
    ULongMul2Power ( my , powall ) ;
  }
Exit  : ;
}

// WideChar or Char  
// wchar_t or char
// L'0' or '0'
# define LISP2DULONGADDTOWITHOUTEXP(  STRNAME , CTYPE , ZERO  ) \
void  Lisp2DULongAddToStr ##  STRNAME ##  WithoutExp ( \
  ULongPub const * const my , String ( STRNAME ) * const stdsp  ) { \
  ULong const * const myp = ULong_toCPrivate ( my ) ; \
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec ) ) { \
    String_AddChar ( STRNAME ) ( stdsp , ZERO ) ; \
    return  ; \
  } \
  /* this is NOT 0 */ \
  ThreadStackPointerType const oldstack_di = Lisp2DThreadSP ( ) ; \
  String  ( STRNAME ) * const dip = Lisp2D_StackVar ( Lisp2DStackTypeString ##  STRNAME ) ; \
  String_Init ( STRNAME ) (  dip , true ) ; \
  { enum { ten = 10U } ; \
    if  ( VectorSizeCopy  ( StackUnsInt ) ( & myp ->  valuec )  ==  1UL  ) { \
      unsigned  int icd = Vector_FrontCopy ( StackUnsInt )  ( & myp ->  valuec ) ; \
      do  { \
        String_AddChar ( STRNAME ) (  dip , \
          ( CTYPE ) ( ZERO + (  icd  % ten  ) ) ) ; \
        icd /=  ten ; \
      } while ( icd ) ; \
    } else  { \
      ThreadStackPointerType const oldstack_cd = Lisp2DThreadSP ( ) ; \
      ULongPub * const cd = StackNew_ULong ( ) ; \
      ULong_ConstrCopy ( cd , ULongCToPub ( myp ) ) ; \
      ULong * const cdp = ULong_toPrivate ( cd ) ; \
      do  { \
        unsigned  int const ic = \
          ULongDividingAndReturnsRemainder ( cd , ten ) ; \
        String_AddChar ( STRNAME ) (  dip , ( CTYPE ) ( ZERO + ic ) ) ; \
      } while ( VectorSizeCopy  ( StackUnsInt ) ( & cdp -> valuec ) > 1 ) ; \
      if  ( VectorSizeCopy  ( StackUnsInt ) ( & cdp  -> valuec ) ==  1UL ) { \
        unsigned  int icd = Vector_FrontCopy ( StackUnsInt ) ( & cdp -> valuec ) ; \
        do { \
          String_AddChar ( STRNAME ) (  dip , \
            ( CTYPE ) ( ZERO + (  icd  % ten  ) ) ) ; \
          icd /=  ten ; \
        } while ( icd ) ; \
      } \
      Lisp2D_StackFree  ( oldstack_cd ) ; \
    } \
  } /* ten */ \
  { CTYPE const * dii = String_CEnd ( STRNAME ) (  dip ) - 1UL ; \
    { CTYPE const * const diend = String_CBegin ( STRNAME ) ( dip ) - 1UL ; \
      while ( dii not_eq  diend ) { \
        String_AddChar ( STRNAME ) ( stdsp , * dii ) ; \
        --  dii ; \
      } \
    } \
  } /* dii */ \
  Lisp2D_StackFree  ( oldstack_di ) ; \
}  
  
LISP2DULONGADDTOWITHOUTEXP  ( WideChar  , wchar_t volatile , L'0'  )

LISP2DULONGADDTOWITHOUTEXP  ( Char  , char  , '0' )

// WideChar or Char
# define LISP2DPROMOSTRINGADDFOREXP2( STRNAME ) \
void  Lisp2DString  ##  STRNAME ##  AddForExp2_ExcMem ( \
  String  ( STRNAME ) *  stdsp , unsigned  long  int * exp2p ) ;

LISP2DPROMOSTRINGADDFOREXP2 ( WideChar  )

LISP2DPROMOSTRINGADDFOREXP2 ( Char  )  

// L'e' or 'e'
# define LISP2DULONGADDTO(  STRNAME , CTYPE , ZERO  , EXPON ) \
void  Lisp2DULongAddToStr ##  STRNAME ( ULongPub const * const my , \
  String  ( STRNAME ) * const stdsp ) { \
  ULong const * const myp = ULong_toCPrivate ( my ) ; \
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec ) ) { \
    String_AddChar ( STRNAME ) ( stdsp , ZERO ) ; \
    return  ; \
  } \
  /* this is NOT 0  */ \
  ThreadStackPointerType const oldstack_di = Lisp2DThreadSP ( ) ; \
  String  ( STRNAME ) * const dip = Lisp2D_StackVar ( Lisp2DStackTypeString ##  STRNAME ) ; \
  String_Init ( STRNAME ) (  dip , true ) ; \
  { enum { ten = 10U } ; \
    if  ( VectorSizeCopy  ( StackUnsInt ) ( & myp ->  valuec )  ==  1UL  ) { \
      unsigned  int icd = Vector_FrontCopy ( StackUnsInt )  ( & myp ->  valuec ) ; \
      do  { \
        String_AddChar ( STRNAME ) (  dip , \
          ( CTYPE ) ( ZERO + ( icd  % ten  ) ) ) ; \
        icd /=  ten ;  \
      } while ( icd ) ; \
    } else  { \
      ThreadStackPointerType const oldstack_cd = Lisp2DThreadSP ( ) ; \
      ULongPub * const cd = StackNew_ULong ( ) ;  \
      ULong_ConstrCopy ( cd , ULongCToPub ( myp ) ) ; \
      ULong * const cdp = ULong_toPrivate ( cd ) ; \
      do  { \
        unsigned  int const ic  = \
          ULongDividingAndReturnsRemainder ( cd , ten ) ; \
        String_AddChar ( STRNAME ) (  dip , ( CTYPE ) ( ZERO + ic ) ) ; \
      } while ( VectorSizeCopy  ( StackUnsInt ) ( \
        & ( cdp -> valuec ) ) > 1 ) ;  \
      if  ( VectorSizeCopy  ( StackUnsInt ) ( \
        & ( cdp -> valuec ) ) ==  1 ) {  \
        unsigned  int icd = Vector_FrontCopy ( StackUnsInt )  ( \
          & ( cdp -> valuec )  )  ; \
        /*memory economy*/  \
        Lisp2D_StackFree  ( oldstack_cd ) ; \
        do  { \
          String_AddChar ( STRNAME ) (  dip , \
            ( CTYPE ) ( ZERO + ( icd  % ten  ) ) ) ; \
          icd /=  ten ;  \
        } while ( icd ) ; \
      } else \
        Lisp2D_StackFree  ( oldstack_cd ) ; \
    } \
  } /* ten */ \
  { unsigned  long  int exp2  = 0  ; \
    { CTYPE const * restrict dii = String_CBegin ( STRNAME ) (  dip ) ; \
      CTYPE const * const restrict die = String_CEnd ( STRNAME ) (  dip ) ; \
      while ( dii not_eq  die ) { \
        if  ( ( * dii )  not_eq  ZERO ) \
          break ;  \
        ++  exp2  ; \
        ++  dii ; \
      } \
    } /* dii  die */ \
    if  ( exp2 > 2  ) { \
      if  ( not  String_Empty ( STRNAME ) (  dip ) ) \
        for ( size_t  j = String_Size ( STRNAME ) (  dip ) - 1 ; \
             j  >=  exp2 ; --  j ) \
          String_AddChar ( STRNAME ) ( stdsp , \
            String_EltCopy ( STRNAME ) (  dip  , j ) ) ; \
      String_AddChar ( STRNAME ) ( stdsp , EXPON ) ; \
      Lisp2DString  ##  STRNAME ##  AddForExp2_ExcMem ( stdsp , & exp2 ) ; \
      Lisp2D_StackFree  ( oldstack_di ) ; \
      return  ; \
    } \
  } \
  { CTYPE const * restrict dii = String_CEnd ( STRNAME ) (  dip ) - 1UL ; \
    { CTYPE const * const restrict diend = \
        String_CBegin ( STRNAME ) (  dip ) - 1UL ; \
      while ( dii not_eq  diend ) { \
        String_AddChar ( STRNAME ) ( stdsp , * dii ) ; \
        --  dii ; \
      } \
    }  /* diend  */ \
  } /* dii */ \
  Lisp2D_StackFree  ( oldstack_di ) ; \
} /* this is NOT 0 - checked  */  
  
LISP2DULONGADDTO  ( WideChar  , wchar_t volatile , L'0'  , L'e'  )

LISP2DULONGADDTO  ( Char  , char  , '0' , 'e' )  
  
// WideChar or Char
# define LISP2DSTRINGADDFOREXP2(  STRNAME , CTYPE , ZERO  ) \
void  Lisp2DString  ##  STRNAME ##  AddForExp2_ExcMem ( \
  String  ( STRNAME ) * const stdsp , unsigned  long  int * const exp2p ) { \
  { ThreadStackPointerType const oldstack_di = Lisp2DThreadSP ( ) ; \
    String  ( STRNAME ) * const dip = Lisp2D_StackVar ( Lisp2DStackTypeString ##  STRNAME ) ; \
    String_Init ( STRNAME ) (  dip , true ) ; \
    do  { \
      String_AddChar ( STRNAME ) (  dip , \
        ( CTYPE ) ( ZERO + ( ( * exp2p ) % 10 ) ) ) ; \
      ( * exp2p )  /=  10  ; \
    } while ( * exp2p  ) ;  \
    /* di is NOT empty  */  \
    CTYPE const * restrict dii = String_CEnd ( STRNAME ) (  dip ) - 1UL ; \
    { CTYPE const * const restrict diend = \
        String_CBegin ( STRNAME ) (  dip ) - 1UL ; \
      while ( dii not_eq  diend ) { \
        String_AddChar ( STRNAME ) ( stdsp , * dii ) ; \
        --  dii ; \
      } \
    }  /* diend  */  \
    Lisp2D_StackFree  ( oldstack_di ) ; \
  }  /* di was NOT empty */ \
}

LISP2DSTRINGADDFOREXP2  ( WideChar  , wchar_t volatile , L'0'  )

LISP2DSTRINGADDFOREXP2  ( Char  , char  , '0' )

ULongPub * StackNew_ULong ( void  ) {
  return ULongToPub ( Lisp2D_StackVar ( Lisp2DStackTypeULong ) ) ;
}

# include "castb.h"

bool  ULongMoreOrEq  ( ULongPub  const * const my , ULongPub  const * const  x ) {
  return  not  ULongLess  ( my , x ) ;
}

bool  ULongOddP  ( ULongPub const * const my ) {
  return ULongMod2 ( my ) ;
} // (x&1)==1

void  ULongSub1 ( ULongPub * const me ) {
  ULongSubUInt  ( me , 1U ) ;
}

void  ULongMul2 ( ULongPub  * const me ) {
  ULongMul2Power ( me , 1UL ) ;
}

bool  ULongLessOrEq  ( ULongPub  const * const restrict  my , ULongPub  const * const restrict x ) {
  return  not  ULongLess  ( x , my ) ;
}

void  ULongSetPointer ( ULongPub * const my , void  * const x ) {
  // x is the pointer
  // (& x) is the place where pointer is (in stack with 64 bits size)
  // needs pointer yourself
  ULongSetFromBitsNegativeSize ( my , & x , sizeof  ( void  * )  <<  3  ) ;
}
# include "widestring/inline.h"
inline  void UIntSwapValue  ( unsigned  int * const restrict a  , unsigned  int * const restrict b  ) {
  unsigned  int const tmp = ( * a ) ;
  ( * a ) = ( * b ) ;
  ( * b ) = tmp ;
}

size_t  ULongHasPowerOfTwo ( ULong  const * const xp ) {
  size_t  result  = 0 ;
  { Vector  ( StackUnsInt ) const * const arrp =
      ULongCValue  ( ULongCToPub ( xp ) ) ;
    unsigned int const * arri = VectorCBegin  ( StackUnsInt ) ( arrp ) ;
    unsigned int const * const  arre  =  VectorCEnd ( StackUnsInt ) ( arrp  ) ;
    enum { itemsizemax = ( sizeof  ( unsigned int  ) * 8LU ) } ;
    while ( arri not_eq arre  ) {
      { unsigned int  i = ( * arri  ) ;
        if  ( i ) {
          { size_t  bitshift  = ( itemsizemax >>  1U  ) ;
            do  {
              { unsigned int const  i2 = ( i >>  bitshift )  ;
                if  ( ( i2  <<  bitshift  ) ==  i ) {
                  result  +=  bitshift  ;
                  i = i2  ;
                }
              }
              bitshift  >>= 1U  ;
            } while ( bitshift  ) ;
          }
          goto Exit ;
        }
      }
      result  +=  itemsizemax ;
      ++  arri  ;
    }
  }
Exit  :
  return  result  ;
}

inline size_t SizetMin  ( size_t const  a , size_t const  b ) {
  if  ( a < b )
    return a  ;
  return b  ;
}
