// Lisp2D unsigned integer dividing
// ©2005-23 by Alex Glebe

// returns 0..32      
# define  CalcBitZerosInTheEnd  Lisp2DCalcBitZerosInTheEnd
static  unsigned int CalcBitZerosInTheEnd ( unsigned  int ) ;

# define  CalcBitSize Lisp2DCalcBitSize
static  unsigned int CalcBitSize ( unsigned int ) ;

# include "ulong/type.h"

// me := [me/by] ; ost := me%by ; by - modified
# define  ULongDivisionFast Lisp2DULongDivisionFast
static  void  ULongDivisionFast  ( ULong * mep , ULong *  byp  , ULong * ostp  ) ;

# include "define.h"
# include "ulong/private.h"
# include "ulong/public.h"
# include "cast.h"
# include "stack/public.h"

unsigned  int ULongDividingAndReturnsRemainder ( ULongPub * const me , unsigned  int divisor ) {
  ULong * const mep = ULong_toPrivate ( me ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & mep ->  valuec  ) )
    goto  ReturnZero  ;
# ifdef  LISP2D_DEBUG
  if  ( divisor  ==  0  ){
    Lisp2DExceptionStringGlobal . str = (wchar_t const (*)[])&
      L"Lisp2DULongDividingAndReturnsRemainder:divisor  ==  0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  if  ( divisor  ==  1  )
    goto  ReturnZero  ;
  { size_t  const s = VectorSizeCopy(StackUnsInt)(&mep->valuec  )  ;
    if  ( s  ==  1  ) {
      unsigned  int x =  Lisp2DStackUnsIntFrontCopy(&mep->valuec  )  ;
      if  ( x  < divisor  ) {
        Lisp2DStackUnsIntClear ( &mep->valuec  ) ;
        return  x ; 
      }
      unsigned  int const ost = ( x % divisor ) ;
      x /=  divisor ;
      // x>0
      (*VectorFront(StackUnsInt) ( &mep->valuec  ) )  = x  ;
      return  ost ; 
    }
    enum { halfdigitsize = ( sizeof  ( unsigned int  )  <<  2  ) } ;
    // size>=2
    if  ( divisor  < (  1U <<  halfdigitsize  ) ) {
      // divisor - small number
      ThreadStackPointerType const oldstack3  = Lisp2DThreadSP ( ) ;
      ULongPub * const ost3p = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( ost3p , me ) ;
      (*Lisp2DStackUnsIntBack(& mep->valuec  )) /=  divisor ;
      ULong * const ost3 = ULong_toPrivate ( ost3p ) ;
      (*Lisp2DStackUnsIntBack(& ost3 ->valuec  )) %=  divisor ;
      if  ( s  > 2  )
        for ( size_t j = ( s - 2 ) ; j > 0 ; --  j ) {
          size_t  jplus1 = ( j + 1 ) ;
          unsigned  int xj = ( ( Lisp2DStackUnsIntAtCopy(& ost3-> valuec  ,
            j ) >> halfdigitsize )  bitor
            ( Lisp2DStackUnsIntAtCopy(&ost3 -> valuec  , jplus1  )  <<
              halfdigitsize  ) ) ;
          (*Lisp2DStackUnsIntAt  ( &mep->valuec  , j )) =
            ( ( xj  / divisor ) <<  halfdigitsize ) ;
          xj  = ( ( Lisp2DStackUnsIntAtCopy(&ost3-> valuec  , j ) <<
            halfdigitsize )
            >>  halfdigitsize  )  bitor
            ( ( xj  % divisor ) <<  halfdigitsize ) ;
          (*Lisp2DStackUnsIntAt  ( &ost3-> valuec  , jplus1 ) )  = 0  ;
          (*Lisp2DStackUnsIntAt  ( &mep->valuec  , j )) or_eq  (  xj /
            divisor  ) ;
          (*Lisp2DStackUnsIntAt  ( &ost3-> valuec  , j )) =
            ( xj % divisor  ) ; 
        } // for j
      if  ( s  > 1  ) {
        unsigned  int xj = ( ( Lisp2DStackUnsIntAtCopy(& ost3-> valuec  , 0UL )
          >> halfdigitsize )  bitor
          ( Lisp2DStackUnsIntAtCopy(&ost3-> valuec  , 1UL )  <<  halfdigitsize ) ) ;
        (*Lisp2DStackUnsIntAt  ( &mep->valuec  , 0UL )) = ( ( xj / divisor  ) <<
          halfdigitsize ) ;
        xj  = ( ( Lisp2DStackUnsIntAtCopy(&ost3-> valuec  , 0UL ) << halfdigitsize )
          >> halfdigitsize  )  bitor
          ( ( xj  % divisor ) <<  halfdigitsize ) ;
        (*Lisp2DStackUnsIntAt  ( &ost3-> valuec  , 1UL )) = 0 ;
        (*Lisp2DStackUnsIntAt  ( &mep->valuec  , 0UL )) or_eq  (  xj / divisor ) ;
        (*Lisp2DStackUnsIntAt  ( &ost3-> valuec  , 0UL )) = ( xj % divisor  ) ;
      }
      // s > 1
      ULong_Simplify  ( mep ) ;
# ifdef LISP2D_DEBUG
      if  ( VectorEmpty ( StackUnsInt ) ( & ost3-> valuec  ) ) {
        Lisp2D_StackFree  ( oldstack3 ) ;
        Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
          L"Lisp2DULongDividingAndReturnsRemainder:"
          L"Lisp2DStackUnsIntEmpty(& ost2. valuec  )" ;
        longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
      }
# endif    
      unsigned int const result = Lisp2DStackUnsIntFrontCopy(& ost3-> valuec ) ;
      Lisp2D_StackFree  ( oldstack3 ) ;
      return result ; 
    } // if divisor  <  (1U <<  halfdigitsize)
    // divisor  >=  (1U <<  halfdigitsize)
    { unsigned int const  bitendzeros = CalcBitZerosInTheEnd ( divisor ) ;
      if  ( CalcBitSize ( divisor ) - bitendzeros < halfdigitsize ) {
        unsigned int myostatok ;
        { unsigned int const shiftup = size_cast_uint ( ( sizeof  (
            unsigned int  )  <<  3 ) - bitendzeros ) ;
          myostatok = ( ( Lisp2DStackUnsIntFrontCopy(&mep->valuec  ) ) <<
            shiftup ) >>  shiftup ; 
        }
        ULongDiv2Power  ( me , bitendzeros);
        divisor >>= bitendzeros ;
        return ( ULongDividingAndReturnsRemainder ( me ,
          divisor ) << bitendzeros ) bitor myostatok ;
      } 
    }
    // divisor  >=  (1U <<  halfdigitsize)
  } // size_t s
  { ThreadStackPointerType const oldstack3  = Lisp2DThreadSP ( ) ;
    ULongPub * const res3p  = StackNew_ULong ( ) ;
    ULong_Init ( res3p ) ;
    { ThreadStackPointerType const oldstackd3  = Lisp2DThreadSP ( ) ;
      ULongPub * const d3p  = StackNew_ULong ( ) ;
      ULong_Init ( d3p ) ;
      ULongSetUInt ( d3p , divisor ) ;
      ULongDivisionFast  ( mep , ULong_toPrivate ( d3p ) , ULong_toPrivate ( res3p ) ) ;
      Lisp2D_StackFree  ( oldstackd3 ) ;
    } // d
    { unsigned  int result  ;
      if  ( not ULongIsBADUIValue ( res3p , & result  ) ) {
        Lisp2D_StackFree  ( oldstack3 ) ;
        return  result  ; 
      } 
    } // result
    Lisp2D_StackFree  ( oldstack3 ) ;
  } //  ULong res ;
  Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
    L"Lisp2DULongDividingAndReturnsRemainder:"
    L"Lisp2DULongisOKUIValue ( &  res2 , & result  )==false" ;
  longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
ReturnZero  :
  return  0 ; 
}

// 0..32  
static  unsigned int CalcBitZerosInTheEnd ( unsigned  int x ) {
  if  ( x ==  0 ) {
    enum { bits = sizeof  ( unsigned  int ) <<  3U } ;
    return bits ;
  }
  unsigned  int zerobitscount = 0  ;
  enum { halfbits = sizeof  ( unsigned  int ) <<  2U } ;
  unsigned  int bitdiff = halfbits ;
  // 4 * 4 = 16 bits , half of unsigned int
  do  {
    { unsigned  int workplace2 = ( x <<  bitdiff ) ;
      if  ( workplace2  )
        x = workplace2  ;
      else
        zerobitscount +=  bitdiff ; 
    }
    bitdiff >>= 1U  ;
  } while ( bitdiff ) ; 
  return zerobitscount  ; 
}

// returns 0..32    

static  unsigned int CalcBitSize ( unsigned int x ) {
  if  ( x == 0 )
    return 0 ;
  unsigned  int bitscount = 1U  ;
  unsigned  int bitdiff =
    size_cast_uint ( sizeof  ( unsigned  int ) <<  2U ) ;
  // 4 * 4 = 16 bits , half of unsigned int
  do  {
    { unsigned  int const workplace2 = ( x >>  bitdiff ) ;
      if  ( workplace2  ) {
        x = workplace2  ;
        bitscount +=  bitdiff ; 
      } 
    }
    bitdiff >>= 1U  ;
  } while ( bitdiff ) ;
  return  bitscount ; 
}

# define  ULongDivideFront  Lisp2DULongDivideFront
static  void  ULongDivideFront ( ULong * mep , ULong * byp , ULong * ostp ) ;

# include "ulong/inline.h"

/*
Divizor BY removing last zero bit's
ME / BY000 = (MEh*1000+MEl) / (BYh*1000) =
           = [MEh / BYh] + ((MEh%BYh)*1000 + MEl)/(BYh*1000)
*/    
  
// me := [me/by] ; ost := me%by ; by - modified
static  void  ULongDivisionFast ( ULong * const mep , ULong * const byp , ULong * const ostp ) {
  if  ( ULongValueSize ( ULongCToPub ( byp ) ) ==  0  ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongDivisionFast:Lisp2DULongValueSize ( byp)==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
  size_t const mysize = ULongValueSize ( ULongCToPub ( mep ) )  ;
  if  ( mysize ==  0  ) {
    ULongSet0  ( ULongToPub(ostp) ) ;
    return;
  }
  if  ( ULongLess ( ULongCToPub ( mep ) , ULongCToPub ( byp ) ) ) {
    ULongMove ( ULongToPub(ostp) , ULongToPub(mep) ) ;
    ULongSet0  ( ULongToPub(mep) ) ;
    return ;
  }
  if  ( mysize  ==  1U  ) {
    ULongDivideFront ( mep,byp , ostp ) ;
    return ;
  }
  if  ( ULongOddP ( ULongCToPub ( byp ) ) ) {
    t_ULongPrivateDivideByOdd const arg = {
      . mep = mep ,
      . byp = byp ,
      . ostp = ostp ,
    } ;
    ULongPrivateDivideByOdd ( & arg ) ;
    return ;
  }
  size_t  zerointscount = 0  ;
  unsigned  int zerobitscount ;
  // counting all zeros
  { unsigned  int const * j = VectorCBegin(StackUnsInt)  ( & ( byp->valuec  ) ) ;
    while ( ( * j ) ==  0 ) {
      ++  j ;
      ++  zerointscount ; 
    }
    // *j <> 0
    zerobitscount = CalcBitZerosInTheEnd  ( * j ) ;
  } // j
  // me divide save to ostatok
  ThreadStackPointerType const oldstack2 = Lisp2DThreadSP ( ) ;
  ULongPub * const oldostatok2p = StackNew_ULong ( ) ;
  ULong_Init ( oldostatok2p ) ;
  jmp_buf jump_buffer ;
  jmp_buf * jbold = Lisp2DJBglobal  ;
  Lisp2DJBglobal  = & jump_buffer ;
  int const volatile  jbn = setjmp  ( jump_buffer ) ;
  if  ( jbn  ) {
    Lisp2DJBglobal  = jbold ;
    Lisp2D_StackFree  ( oldstack2 ) ;
    longjmp ( * Lisp2DJBglobal  , jbn ) ;
  }
  ULong * const oldostatok2 = ULong_toPrivate ( oldostatok2p ) ;
  { size_t zerointscount2 = zerointscount  ;
    if  ( zerobitscount )
      ++  zerointscount2  ;
    Lisp2DStackUnsIntResize_ExcMem ( & oldostatok2  ->valuec  , zerointscount2  ) ;
    memcpy  ( VectorBegin(StackUnsInt) ( &oldostatok2 -> valuec  ) ,
      VectorCBegin(StackUnsInt)  ( & mep->valuec  ) , zerointscount2  * sizeof  ( unsigned  int ) ) ;
  }
  { unsigned  int const step  = ulint_cast_uint ( ( sizeof  ( unsigned  int ) <<  3 ) - zerobitscount ) ;
    if  ( zerobitscount ) {
      unsigned  int * const lastp  = Lisp2DStackUnsIntBack  ( &  oldostatok2 -> valuec  ) ;
      (*lastp)  = ( ( (*lastp)  <<  step  ) >>  step  ) ; 
    } 
  }
  ULong_Simplify  ( oldostatok2 ) ;
  { t_ULongCutOff const arg = {
      . mep = mep ,
      . intscount = zerointscount ,
      . bitscount = zerobitscount ,
    } ;
    ULongCutOff  ( &  arg ) ;
  }
  { t_ULongCutOff const arg = {
      . mep = byp ,
      . intscount = zerointscount ,
      . bitscount = zerobitscount ,
    } ;
    ULongCutOff  ( & arg ) ;
  }
  if  ( ULongValueSize ( ULongCToPub ( mep ) ) ==  1U  )
    ULongDivideFront (mep, byp , ostp  ) ;
  else {
    t_ULongPrivateDivideByOdd const arg = {
      . mep = mep ,
      . byp = byp ,
      . ostp = ostp ,
    } ;
    ULongPrivateDivideByOdd ( & arg ) ;
  }
  if  ( not ( VectorEmpty ( StackUnsInt ) ( & ostp  -> valuec  ) ) ) {
    Lisp2DStackUnsIntInsertFront_ExcMem(&ostp -> valuec  , 0U , zerointscount ) ;
    ULongMul2Power ( ULongToPub ( ostp ) , zerobitscount ) ;
    ULongAddTemp  ( ULongToPub ( ostp ) , oldostatok2p ) ;
    // if(not (ost.valuec.Empty()))
  } else 
    ULongMove ( ULongToPub(ostp) , oldostatok2p ) ;
  Lisp2D_StackFree  ( oldstack2 ) ;
  Lisp2DJBglobal  = jbold ; 
}

static  void  ULongDivideFront ( ULong * const mep , ULong * const byp ,
  ULong * const ostp ) {
  unsigned  int me2 = Lisp2DStackUnsIntFrontCopy  ( & mep -> valuec   )  ;
  unsigned  int by2 = Lisp2DStackUnsIntFrontCopy  ( & byp  -> valuec   )  ;
  ULongSetUInt  ( ULongToPub ( ostp ) , me2 % by2 ) ;
  ULongSetUInt  ( ULongToPub ( mep ) ,  me2 / by2 ) ; 
}

/*
  if me.size<by.size*2
  vector size normalize : me.sizebits == by.sizebits*2
  by removing last bits (equal both to ME and BY)
  call DivideNormalized
  Adds +1 when needed
  or
  divide big ME by school manner with normalized big part of ME
  call DivideNormalized
  cycle while ME still big
*/  
  
void  ULongPrivateDivideByOdd ( t_ULongPrivateDivideByOdd const * const arg ) {
  size_t const mes  = VectorSizeCopy(StackUnsInt)(& arg -> mep-> valuec   )  ;
  size_t const bys  = VectorSizeCopy(StackUnsInt)(&arg -> byp-> valuec   )  ;
  unsigned int const intbitsize = ulint_cast_uint ( ( sizeof  ( unsigned  int ) << 3U  ) ) ;
# ifdef LISP2D_DEBUG  
  if  ( bys ==  0 ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideByOdd_ExcMem:"
      L"VectorSizeCopy(StackUnsInt)(&byp-> valuec  )==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  if  ( bys == 1 ) {
    unsigned int const lastint =  VectorBackCopy(StackUnsInt)(& arg -> byp-> valuec   )  ;
    unsigned  int const by2 = CalcBitSize ( lastint )  ;
    if  ( by2 <=  ( intbitsize  >>  1 ) ) {
      ULongSetUInt  ( ULongToPub ( arg -> ostp ) ,
        ULongDividingAndReturnsRemainder ( ULongToPub ( arg -> mep ) , lastint ) ) ;
      return ;  
    }
  }
  // bys == 1  
  if  ( ( mes -  bys  + 2UL ) <=   bys   ) {
    Lisp2DULongPrivateDivideSmall_ExcMem ( arg -> mep , arg -> byp , arg -> ostp ) ;
    return ; 
  }
    // ( mes -  bys  + 2UL ) <=   bys
  if  ( ( mes - bys ) >=  ( bys + 1UL ) ) {
    Lisp2DULongPrivateDivideBig_ExcMem ( arg -> mep ,  arg -> byp  , arg -> ostp ) ;
    return ; 
  }
    // ( mes - bys ) >=  ( bys + 1UL )
  // mes-1 < bys*2 < mes+2
  // bys*2 == mes XOR mes+1
  unsigned  int const my2 = CalcBitSize ( VectorBackCopy(StackUnsInt)(    &arg -> mep-> valuec   ) )  ;
  unsigned  int const by2 = CalcBitSize ( VectorBackCopy(StackUnsInt)(    &arg -> byp-> valuec   ) )  ;
  int const my3 = uint_cast_int  ( my2 + intbitsize  - by2 - by2 )  ;
  int const by3 = uint_cast_int  ( ( bys == (  mes - bys ) ) ?
    0 :
    intbitsize  ) ;
  if  ( my3 > by3 ){
    Lisp2DULongPrivateDivideBig_ExcMem ( arg -> mep ,  arg -> byp   , arg -> ostp ) ;
    return ; 
  }// my3>by3
  if  ( my3 < by3 ){
    Lisp2DULongPrivateDivideSmall_ExcMem ( arg -> mep ,  arg -> byp  , arg -> ostp ) ;
    return ; 
  }// my3<by3
  Lisp2DULongPrivateDivideNormalized_ExcMem  ( arg -> mep  , arg -> byp , arg -> ostp ) ;
}

/*
  if me.size<by.size*2
  vector size normalize : me.sizebits == by.sizebits*2
  by removing last bits (equal both to ME and BY)
  call DivideNormalized
  Adds +1 when needed
*/
    
void  Lisp2DULongPrivateDivideSmall_ExcMem ( ULong * const mep  ,
  ULong * const byp  , ULong * const ostp ) {
// ! to make vars public
  size_t const mes  = VectorSizeCopy  ( StackUnsInt ) ( & mep -> valuec   )  ;
# ifdef LISP2D_DEBUG    
  if  ( mes ==  0 ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideSmall_ExcMem:"
      L"VectorSizeCopy(StackUnsInt)(& mep-> valuec  )==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
  unsigned  int const my2 = CalcBitSize ( VectorBackCopy(StackUnsInt)(
    &mep-> valuec   ) )  ;
  size_t const bys  = VectorSizeCopy(StackUnsInt)(&byp-> valuec   )  ;
# ifdef LISP2D_DEBUG    
  if  ( bys ==  0 ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideSmall_ExcMem:"
      L"VectorSizeCopy(StackUnsInt)(&byp-> valuec  )==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
  unsigned  int const bitint = size_cast_uint ( sizeof  ( unsigned  int ) <<  3U ) ;
  unsigned  int const by2 = CalcBitSize ( VectorBackCopy(StackUnsInt)( &byp-> valuec   ) )  ;
  /*
   diffs := ((2 * bybitsize) - mebitsize))
   mecopy := me
   ostatok swap me
   (mecopy  >>= diffs) divideuniversal/normalized ((bycopy  >> diffs) + 1)
   me := mecopy
   ostatok -= (mecopy *= by)
   if ostatok >= by {me++; ostatok-=by;}
  */
  size_t const  diffbit  =
    ( ( bys + bys - mes  - 1  )  * bitint  + by2 + by2 - my2 ) ;
  { ThreadStackPointerType const oldstack2 = Lisp2DThreadSP ( ) ;
    ULongPub * const mecopy2p = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( mecopy2p , ULongCToPub ( mep ) ) ;
    ULongSwap  ( ULongToPub(mep) , ULongToPub(ostp) ) ;
    //mecopy >>=  diffbit  ;
    ULongDiv2Power  ( mecopy2p , diffbit ) ;
    ULong * const mecopy2 = ULong_toPrivate ( mecopy2p ) ;
    { ThreadStackPointerType const oldstack3 = Lisp2DThreadSP ( ) ;
      ULongPub * const bycopy2p = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( bycopy2p , ULongCToPub ( byp ) ) ;
      ULongDiv2Power  ( bycopy2p , diffbit ) ;
      ULongAdd1 ( bycopy2p ) ;
      { ULongPub * const ost2lp = StackNew_ULong  ( ) ;
        ULong_Init  ( ost2lp ) ;
        ULongDivisionFast ( mecopy2 , ULong_toPrivate ( bycopy2p ) , ULong_toPrivate ( ost2lp ) ) ;
      } // ost2
      Lisp2D_StackFree  ( oldstack3 ) ;
    } // bycopy
    ULongSet  ( ULongToPub ( mep ) , mecopy2p ) ;
    ULongMultiplyConst ( mecopy2p , ULongCToPub ( byp ) ) ;
    ULongDifferenceEq ( ULongToPub ( ostp ) , mecopy2p ) ;
    Lisp2D_StackFree  ( oldstack2 ) ;
  } // mecopy
  while ( ULongMoreOrEq ( ULongCToPub(ostp) , ULongCToPub(byp)) ) {
    ULongAdd1 ( ULongToPub  ( mep ) ) ;
    ULongDifferenceEq ( ULongToPub ( ostp ) , ULongCToPub ( byp ) ) ;
  }
}

# define  ULongDivWithOstSetMoveArg Lisp2DULongDivWithOstSetMoveArg
static  void  ULongDivWithOstSetMoveArg ( ULongPub * myp , ULongPub * lp  ,
  ULongPub * ostp ) ;

/*
 if me.size>by.size*2
  divide big ME by school manner with normalized big part of ME
  call DivideNormalized
  cycle while ME still big
 */      
      
void  Lisp2DULongPrivateDivideBig_ExcMem ( ULong * const  mep  ,
  ULong * const  byp  , ULong * const ostp ) {
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  ULongPub * const resultlp = StackNew_ULong  ( ) ;
  ULong_Init  ( resultlp ) ;
  size_t bys  = VectorSizeCopy  ( StackUnsInt ) ( & byp -> valuec ) ;
# ifdef LISP2D_DEBUG    
  if  ( bys ==  0 ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideBig_ExcMem:"
      L"VectorSizeCopy(StackUnsInt)(&byp-> valuec  )==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  enum { bitint = ( sizeof  ( unsigned  int ) <<  3U ) } ; 
  unsigned  int const by2 =
    CalcBitSize ( VectorBackCopy(StackUnsInt)(&byp-> valuec   ) )  ;
# ifdef LISP2D_DEBUG    
  if  ( ( bys ==  1 ) and ( by2  <= ( bitint  >>  1 ) ) ){
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideBig_ExcMem:"
      L"( bys ==  1 ) and ( by2  <= ( bitint  >>  1 ) )" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
DivAgain  : ;
  volatile size_t mes  = VectorSizeCopy(StackUnsInt)(&mep-> valuec   )  ;
# ifdef LISP2D_DEBUG    
  if  ( mes ==  0 ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideBig_ExcMem:"
      L"VectorSizeCopy(StackUnsInt)(&mep-> valuec  )==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif  
  unsigned  int my2 =
    CalcBitSize ( VectorBackCopy(StackUnsInt)(&mep-> valuec   ) )  ;
# ifdef LISP2D_DEBUG
  { size_t const  mes2  = (  ( mes - 1UL ) * bitint  ) ;
    size_t const  bys3  = ( (  bys - 1UL ) * bitint  ) ;
    if  ( ( bys3  + by2 ) * 2 >=  ( mes2  + my2 ) ) {
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"Lisp2DULongPrivateDivideBig_ExcMem:( bys3  + by2 ) * 2 >= "
        L"( mes2  + my2 )" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
    }
  }
# endif 
  // diffbits = mybitsize - 2*bybitsize
  // ( A*2^diffs+C ) / B  ,  A/B=x+s/B  ,  result += x * 2^diffs
  // ( s*2^diffs+C ) / B  ,  ...
  // (mes-1-(bys-1)*2)*32+my2-by2*2 => (mes+1-bys*2)*32+my2-by2*2
  // my2 = 1..32 by2 = 1..32 my2-by2*2=-63..30
  size_t volatile diffuint = ( mes + 1 - bys - bys ) ;
  unsigned int diffbit = my2  ;
  while  ( diffbit < by2 + by2 ) {
    diffbit += bitint ;
    -- diffuint ; 
  }
  diffbit -= ( by2 + by2 ) ;
  { ThreadStackPointerType const oldstackmecopy = Lisp2DThreadSP ( ) ;
    ULongPub * const mecopylp = StackNew_ULong ( ) ;
    ULong_Init  ( mecopylp ) ;
    jmp_buf jump_buffer ;
    jmp_buf * const jbold = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jump_buffer ;
    int const volatile  jbn = setjmp  ( jump_buffer ) ;
    if  ( jbn ) {
      Lisp2DJBglobal  = jbold ;
      Lisp2D_StackFree  ( oldstack ) ;
      longjmp ( * Lisp2DJBglobal  , jbn ) ; 
    }
    ULong * const mecopyl = ULong_toPrivate ( mecopylp ) ;
    { size_t const bys2 = mes - diffuint ;  
      Lisp2DStackUnsIntResize_ExcMem ( &  mecopyl -> valuec  , bys2 );
      memcpy  ( VectorBegin(StackUnsInt) ( & mecopyl -> valuec  ) ,
        VectorCBegin(StackUnsInt) ( & ( mep-> valuec  ) ) + diffuint ,
        bys2 * sizeof  ( unsigned  int ) ) ;
    }
    // mes*32=(mes-diffuint)*32+diffuint*32    OR
    // mes*32=(mes-diffuint-1)*32+(32-diffbit)+diffbit+diffuint*32
    ULongDiv2Power  ( mecopylp , diffbit ) ;
    { ThreadStackPointerType const oldstacktmpby = Lisp2DThreadSP ( ) ;
      ULongPub * const tmpby2p = StackNew_ULong ( ) ;
      ULong_ConstrCopy ( tmpby2p , ULongCToPub ( byp ) ) ;
      Lisp2DULongPrivateDivideNormalized_ExcMem  ( mecopyl , ULong_toPrivate ( tmpby2p ) , ostp ) ;
      Lisp2D_StackFree  ( oldstacktmpby ) ;
    }
    // mecopy was small , adding zeros
    ULongMul2Power ( mecopylp , diffbit ) ;
    Lisp2DStackUnsIntInsertFront_ExcMem(&  mecopyl-> valuec  , 0 , diffuint ) ;
    // mecopy is big result and ost is big remainder
    // me=mebig|diff : mebig replace by ost  
# ifdef LISP2D_DEBUG  
    if  ( VectorSizeCopy(StackUnsInt)(& mep-> valuec    ) <=  diffuint  ){
      Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
        L"Lisp2DULongPrivateDivideBig_ExcMem:VectorSizeCopy(StackUnsInt)(& "
        L"mep-> valuec   ) <=  diffuint" ;
      longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
    }
# endif
    { size_t newsize  ;
      { unsigned int * jp =  Lisp2DStackUnsIntAt ( &mep-> valuec  ,
          diffuint  )   ;
        unsigned int *  jost ;
        // me last zeroing & replace with ost
        if  ( VectorEmpty ( StackUnsInt ) ( &  ostp -> valuec    ) ) {
          if  ( diffbit ) {
            ( * jp  ) and_eq  ( ( 1U  <<  diffbit ) - 1U ) ;
            newsize = 1 ; 
          }
          else 
            newsize = 0 ; 
        } else
          if  ( diffbit ) {
            ULongMul2Power  ( ULongToPub ( ostp ) , diffbit ) ;
            jost = VectorBegin(StackUnsInt) ( & ostp-> valuec  ) ;
            ( * jp  ) = ( ( ( * jp  ) bitand ( ( 1U  <<  diffbit ) - 1U ) )
              bitor  ( * jost ) ) ;
            ++  jost  ;
            ++ jp ;
            newsize = ( VectorSizeCopy(StackUnsInt)(&    ostp-> valuec   ) ) ;
            memcpy  ( jp , jost ,
              ( newsize - 1U ) * sizeof  ( unsigned  int ) ) ;  
          } else  {
            jost = VectorBegin(StackUnsInt) (&  ostp-> valuec  ) ;
            newsize = ( VectorSizeCopy(StackUnsInt)(&   ostp-> valuec   ) ) ;
            memcpy  ( jp , jost , newsize * sizeof  ( unsigned  int ) ) ; 
          } 
      } // jp , jost
      newsize += diffuint ;
      Lisp2DStackUnsIntResize_ExcMem ( & mep-> valuec  , newsize ) ;
    } // newsize
    // may be zero's in the end
    ULong_Simplify  (  mep  ) ;
    ULongAddTemp  ( resultlp , mecopylp ) ;
    Lisp2DJBglobal  = jbold ; 
    Lisp2D_StackFree  ( oldstackmecopy ) ;
  } // mecopy
  // if me is big goto Try Again else call DivWithOst
  mes = VectorSizeCopy(StackUnsInt) (&mep-> valuec   ) ;
  if  ( mes >= bys + bys + 1 )
    goto  DivAgain ;
  if  ( ( mes + 2UL ) > ( bys + bys ) ) {    
    if  ( mes )
      my2 = CalcBitSize ( VectorBackCopy(StackUnsInt)(& mep-> valuec   ) ) ;
    else
      my2 = 0 ;                                                             
    // mes = bys*2-1 .. bys*2
    if  ( ( ( bys + bys - mes ) * bitint  ) + by2 + by2 < bitint  + my2 )
      goto  DivAgain  ;
  }
  ULongDivWithOstSetMoveArg ( ULongToPub ( mep ) , ULongToPub ( byp ) , ULongToPub ( ostp ) ) ;
  ULongClear ( ULongToPub ( byp ) ) ;
  ULongAddTemp  ( ULongToPub ( mep ) , resultlp ) ;  
  Lisp2D_StackFree  ( oldstack ) ;
}
  
/*
 if me.bitsize==by.bitsize*2
  A/B B:=b*k+l  b>k>l
  k:=2^((by.bitsize - 1) floor 2)
  c1 := [[A / k] / (b+1)]
  A/B := c1 + [(c1-[(c1*l)/k])/(b+1)]+ O(1)
  while ost > by : me+=1 , ost -= by
 */  
  
void  Lisp2DULongPrivateDivideNormalized_ExcMem  ( ULong * const mep  ,
  ULong * const byp  , ULong * const ostp ) {
// ! to make vars public 
  enum { bitint = ( sizeof  ( unsigned  int ) << 3U ) } ; 
  size_t const bys  = VectorSizeCopy(StackUnsInt) (&byp-> valuec   )  ;
# ifdef LISP2D_DEBUG    
  if  ( bys ==  0 ){
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideNormalized_ExcMem:"
      L"VectorSizeCopy(StackUnsInt) (&byp-> valuec  )==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  size_t const mesize = VectorSizeCopy(StackUnsInt) (&mep-> valuec   )  ;
  unsigned  int const bylastbitsize = CalcBitSize (
    VectorBackCopy(StackUnsInt)( & byp-> valuec   ) )  ;
# ifdef LISP2D_DEBUG
  if  ( mesize  ==  0 ){
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongPrivateDivideNormalized_ExcMem:"
      L"VectorSizeCopy(StackUnsInt) (&mep-> valuec  )==0" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif  
  unsigned int byhalflastbit  ;
  size_t byhalfsize ;
  //{byhalfsize,byhalflastbit}:= {bys,bylastbitsize}-1bit;
  if  ( bylastbitsize > 1U )  {
    byhalflastbit = bylastbitsize - 1U ;
    byhalfsize = bys  ;  
  } else  {
    byhalflastbit = bitint ;
    byhalfsize  = bys - 1 ; 
  }
  // {(byhalfsize-1)*32+byhalflastbit} floor 2 => {[(byhalfsize-1)/2]*32,[byhalflastbit/2]}
  // if (byhalfsize-1) % 2 == 0
  //                                           or {[(byhalfsize-1)/2]*32,16+[byhalflastbit/2]}
  // {(byhalfsize-1)*32+1} floor 2 => {[byhalfsize/2]*32,32}      if (byhalfsize-1) % 2 == 0
  //                               or {[byhalfsize/2]*32,16}
  if  ( byhalflastbit > 1U  ) {
    byhalflastbit >>=  1U  ;
    if  ( ( byhalfsize  - 1U  ) bitand 1U )
      byhalflastbit  +=  ( bitint  >>  1U  ) ; 
    byhalfsize  = ( ( byhalfsize  + 1U  ) >> 1U ) ; 
  } else  {
    byhalflastbit = ( ( ( byhalfsize  - 1U  ) bitand  1U  ) ?
      ( bitint  >>  1U  ) : bitint  ) ;
    byhalfsize  >>= 1U  ;
  }
  // bylow:=by & {byhalfsize,byhalflastbit}
  ThreadStackPointerType const oldstack_b = Lisp2DThreadSP ( ) ;
  ULongPub * const bylowlp = StackNew_ULong ( ) ;
  ULong_Init  ( bylowlp ) ;
  ULong * const bylowl = ULong_toPrivate ( bylowlp ) ;
  { jmp_buf jump_buffer ;
    jmp_buf * jbold = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jump_buffer ;
    int const volatile  jbn = setjmp  ( jump_buffer ) ;
    if  ( jbn ) {
      Lisp2DJBglobal  = jbold ;
      Lisp2D_StackFree  ( oldstack_b ) ;
      longjmp ( * Lisp2DJBglobal  , jbn ) ; 
    }
    Lisp2DStackUnsIntResize_ExcMem ( & bylowl -> valuec  , byhalfsize ) ;
    Lisp2DJBglobal  = jbold ;
  }
  memcpy  ( VectorBegin(StackUnsInt) ( &   bylowl -> valuec  ) ,
    VectorCBegin(StackUnsInt)  ( & ( byp-> valuec  ) ) ,
    byhalfsize  * sizeof  ( unsigned  int ) ) ;
  if  ( bitint > byhalflastbit ) {
    unsigned  int diff  = ( bitint  - byhalflastbit ) ;
    (*Lisp2DStackUnsIntBack ( &  bylowl -> valuec  )) =
      ( ( VectorBackCopy(StackUnsInt)(&    bylowl -> valuec  ) << diff ) >>  diff  ) ;
  }
  ULong_Simplify  ( bylowl ) ;
  ULongPub * const byhighlp = StackNew_ULong ( ) ;
  ULong_Init  ( byhighlp ) ;
  ULong * const byhighl = ULong_toPrivate ( byhighlp ) ;
  { jmp_buf jump_buffer2 ;
    jmp_buf * const jbold = Lisp2DJBglobal  ;
    Lisp2DJBglobal  = & jump_buffer2 ;
    int const volatile  jbn = setjmp  ( jump_buffer2  ) ;
    if  ( jbn  ) {
      Lisp2DJBglobal  = jbold ;
      Lisp2D_StackFree  ( oldstack_b ) ;
      longjmp ( * Lisp2DJBglobal  , jbn ) ;
    }
    { size_t highsize = bys - byhalfsize ;
      if  ( bitint > byhalflastbit )
        ++ highsize ;  
      Lisp2DStackUnsIntResize_ExcMem ( & byhighl -> valuec  , highsize ) ;                                                              
      memcpy  ( VectorBegin(StackUnsInt) ( &   byhighl -> valuec  ) ,
        VectorCBegin(StackUnsInt)  ( & ( byp-> valuec  ) ) +
        ( bys - highsize ) , highsize  * sizeof  ( unsigned  int ) ) ;
      if  ( bitint > byhalflastbit )
        ULongDiv2Power  ( byhighlp , byhalflastbit ) ;
    }
    Lisp2DJBglobal  = jbold ;
  } // highsize has lost to be true
  ULongAdd1 ( byhighlp ) ;
  // c1 /= k
  ULongPub * const c1lp = StackNew_ULong ( ) ;
  ULong_Init  ( c1lp ) ;
  jmp_buf jump_buffer3 ;
  jmp_buf * const jbold = Lisp2DJBglobal  ;
  Lisp2DJBglobal  = & jump_buffer3 ;
  int const volatile  jbn = setjmp  ( jump_buffer3  ) ;
  if  ( jbn ) {
    Lisp2DJBglobal  = jbold ;
    Lisp2D_StackFree  ( oldstack_b ) ;
    longjmp ( * Lisp2DJBglobal  , jbn ) ; 
  }
  ULong * const c1l = ULong_toPrivate ( c1lp ) ;
  { size_t  mesizecut = mesize - byhalfsize ;
    if  ( bitint > byhalflastbit )
      ++ mesizecut ;  
    Lisp2DStackUnsIntResize_ExcMem ( & c1l -> valuec  , mesizecut ) ;
    memcpy  ( VectorBegin ( StackUnsInt ) ( &   c1l -> valuec  ) ,
      VectorCBegin  ( StackUnsInt ) ( & ( mep-> valuec  ) ) +
        ( mesize - mesizecut ) , mesizecut  * sizeof  ( unsigned  int ) ) ;
    if  ( bitint > byhalflastbit )
      ULongDiv2Power  ( c1lp , byhalflastbit ) ;
  } // mesizecut has lost to be true
  Lisp2DJBglobal  = jbold ;
  // c1 /= (b+1)
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    ULongPub * const byhighcopy2p = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( byhighcopy2p , byhighlp ) ;
    ULongDivisionFast ( c1l , ULong_toPrivate ( byhighcopy2p ) , ostp ) ;
    Lisp2D_StackFree  ( oldstack ) ;
  }
  // A/B := c1 + [(c1-[(c1*l)/k])/(b+1)]+ O(1)
  ULongMultiplyConst ( bylowlp , c1lp ) ;
  // (c1*l) /= k
  if  ( bitint > byhalflastbit ) {
    Lisp2DStackUnsIntRemoveFront(&    bylowl -> valuec  , byhalfsize - 1 ) ;
    ULongDiv2Power  ( bylowlp  , byhalflastbit ) ; 
  } else
    Lisp2DStackUnsIntRemoveFront(&   bylowl -> valuec  , byhalfsize ) ;
  // (c1 - c1*l/k)/(b+1)
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    ULongPub * const c3p = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( c3p , c1lp ) ;
    ULongDifferenceEq ( c3p , bylowlp ) ;
    ULongClear (bylowlp ) ;
    ULongDivisionFast ( ULong_toPrivate ( c3p ) , byhighl , ostp ) ;
    ULongClear ( byhighlp ) ;
    ULongAddTemp ( c1lp , c3p ) ; 
    Lisp2D_StackFree  ( oldstack ) ;
  } // c3
  // while ost > by : result+=1 , ost -= by
  ULongSet  ( ULongToPub ( ostp ) , c1lp ) ;
  ULongMultiplyConst ( ULongToPub ( ostp ) , ULongCToPub ( byp ) ) ;
  ULongDifferenceEq ( ULongToPub ( mep ) , ULongCToPub ( ostp ) ) ;
  while ( ULongMoreOrEq(ULongCToPub(mep) , ULongCToPub(byp)) ) {
    ULongAdd1 ( c1lp ) ;
    ULongDifferenceEq ( ULongToPub ( mep ) , ULongCToPub ( byp ) ) ;
  }
  ULongClear  ( ULongToPub(byp)) ;
  ULongSwap ( ULongToPub(mep), ULongToPub(ostp) ) ;
  ULongMove ( ULongToPub(mep) , c1lp ) ;
  Lisp2D_StackFree  ( oldstack_b ) ;
} 
  
static  void  ULongDivWithOstSetMoveArg ( ULongPub * const my ,
  ULongPub * const l , ULongPub * const ost ) {
  if  ( my == ost ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongDivWithOstSetMoveArg:myp ==    ostp" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  if ( l == ost ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongDivWithOstSetMoveArg:lp ==    ostp" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  ULong * const ostp = ULong_toPrivate  ( ost ) ;
  if  ( my == l ) {
    ULongSet1 ( my ) ;
    ULongSet0  ( ost ) ;
    goto  Exit  ;
  }
  ULong * const myp = ULong_toPrivate  ( my ) ;
  // 0/x
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec   ) ) {
    Lisp2DStackUnsIntClear ( &ostp->valuec  ) ;
    goto  Exit  ;
  }
  ULong * const lp = ULong_toPrivate  ( l ) ;
# ifdef  LISP2D_DEBUG
  // this/0
  if  ( VectorEmpty ( StackUnsInt ) ( & lp  ->  valuec   ) ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"Lisp2DULongDivWithOstSetMoveArg:VectorEmpty(StackUnsInt)"
      L"(&lp->valuec  )" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
# endif
  if  ( VectorSizeCopy(StackUnsInt)(&lp->valuec   )  ==  1UL  ) {
    ULongSetUInt ( ULongToPub ( ostp ) ,
      ULongDividingAndReturnsRemainder ( ULongToPub ( myp ) ,
        Lisp2DStackUnsIntFrontCopy  ( & ( lp-> valuec  ) ) ) ) ;
    goto  Exit  ;
  }
  if  ( ULongLess ( ULongCToPub ( myp ) , ULongCToPub ( lp ) ) ) {
    ULongClearSwap ( ULongToPub ( ostp ) , ULongToPub ( myp ) ) ;
    goto  Exit  ;
  }
  ULongDivisionFast ( myp , lp , ostp ) ;
Exit  : ;
}
  
// x%0  [x/0]*0+result=x result=x-[x/0]*0=0
// x%1  [x/1]*1+result=x result=x-[x/1]*1=0
void  ULongSetRemainderUInt ( ULong * const myp , unsigned  int const l ) {
  if  ( l ==  0 ) {
    Lisp2DStackUnsIntClear ( &myp->valuec  ) ;
    return ;
  }
  switch  ( VectorSizeCopy(StackUnsInt)(&myp->valuec   ) ) {
  case  0 :
    return ;
  case  1 :
    ( *VectorFront(StackUnsInt) ( &myp->valuec   ) ) %=  l ;
    if  ( Lisp2DStackUnsIntFrontCopy(&myp->valuec   ) ==  0UL )
      Lisp2DStackUnsIntClear ( &myp->valuec  ) ;
    return ;
  default :
    ULongSetUInt ( ULongToPub ( myp ) ,
      ULongDividingAndReturnsRemainder ( ULongToPub ( myp ) , l ) ) ;    
  } 
}
      
void ULongDivConst ( ULongPub * const my , ULongPub const * const byp ) {
  if ( my ==  byp ) { 
    ULongSet1  ( my ) ;
    return;
  }
  ULong * const myp = ULong_toPrivate ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec   ) ) 
    return  ;
  ULong const * const by = ULong_toCPrivate ( byp ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & by  ->  valuec   ) ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongDivConst:VectorEmpty(StackUnsInt)(&by->valuec)" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ;
  }
  if(VectorSizeCopy(StackUnsInt)(&by->valuec )  ==  1UL){
    ULongDividingAndReturnsRemainder ( ULongToPub ( myp ) ,
      Lisp2DStackUnsIntFrontCopy(& by -> valuec  ) ) ;
    return;
  }
  if(VectorSizeCopy(StackUnsInt)(&myp->valuec )  ==  1UL)  {
    Lisp2DStackUnsIntClear ( &myp->valuec  ) ;
    return;
  }    
  ThreadStackPointerType const oldstack_o = Lisp2DThreadSP ( ) ;
  ULongPub * const ostlp = StackNew_ULong ( ) ;
  ULong_Init  ( ostlp ) ;
  // ! to catch exception to destroy ost ?
  ULongDivWithOstSet ( ULongToPub ( myp ) , ULongCToPub ( by ) , ostlp ) ; 
  Lisp2D_StackFree  ( oldstack_o ) ;
}

void ULongDivWithOstSet ( ULongPub * const my ,
  ULongPub const * const l , ULongPub * const ost ) {
  if  ( my == ost ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongDivWithOstSet:my == ost" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
  ULong * const ostp = ULong_toPrivate ( ost ) ;
  if  ( my ==  l ) {
    ULongSet1  ( my ) ;
    ULongSet0  ( ost ) ;
    goto  Exit  ; 
  }
  if  ( l  ==  ost ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongDivWithOstSet:l  ==  ost" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
  ULong * const myp = ULong_toPrivate ( my ) ;
  // 0/x
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec   ) ) {
    Lisp2DStackUnsIntClear ( & ostp->valuec  ) ;
    goto  Exit  ; 
  }
  ULong const * const lp = ULong_toCPrivate ( l ) ;
# ifdef  LISP2D_DEBUG
  // this/0
  if  ( VectorEmpty ( StackUnsInt ) ( & lp  ->  valuec   ) ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongDivWithOstSet:VectorEmpty(StackUnsInt)(&lp->valuec  )" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  if  ( VectorSizeCopy(StackUnsInt)(&lp->valuec   )  ==  1UL  ) {
    ULongSetUInt ( ost ,
      ULongDividingAndReturnsRemainder ( my ,
        Lisp2DStackUnsIntFrontCopy  ( & ( lp->valuec  ) ) ) ) ;
    goto  Exit  ; 
  }
  if  ( ULongLess ( my , l ) ) {
    ULongClearSwap ( ost , my ) ;
    goto  Exit  ; 
  }
  { ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
    ULongPub * const ll2p = StackNew_ULong ( ) ;
    ULong_ConstrCopy ( ll2p , l ) ;
    // ! to catch to destroy l2
    ULongDivisionFast ( myp , ULong_toPrivate ( ll2p ) , ostp ) ;
    Lisp2D_StackFree  ( oldstack ) ;
  } // l2
Exit  :  
  ; 
}

// ! (*lp)<>0 !
void ULongDivMove ( ULongPub * const my , ULongPub * const l ) {
  if  ( my ==  l  ) {
    ULongSet1 ( my ) ;
    return;
  }
  ULong * const myp = ULong_toPrivate ( my ) ;
  if  ( VectorEmpty ( StackUnsInt ) ( & myp ->  valuec   ) ) 
    return ;
  ULong * const lp = ULong_toPrivate ( l ) ;
# ifdef  LISP2D_DEBUG
  if  ( VectorEmpty ( StackUnsInt ) ( & lp  ->  valuec   ) ) {
    Lisp2DExceptionStringGlobal . str = (wchar_t const(*)[])&
      L"ULongDivMove:VectorEmpty(StackUnsInt)(&lp->valuec  )" ;
    longjmp ( * Lisp2DJBglobal  , Lisp2DExcTypeString ) ; 
  }
# endif
  if  ( VectorSizeCopy(StackUnsInt)(&lp->valuec   )  ==  1UL  ) {
    ULongDividingAndReturnsRemainder ( ULongToPub ( myp ) ,
      Lisp2DStackUnsIntFrontCopy(&lp->valuec   ) ) ;
    return ;
  }
  if  ( VectorSizeCopy(StackUnsInt)(&myp->valuec   ) ==  1UL ){
    Lisp2DStackUnsIntClear ( &myp->valuec  ) ;
    return; 
  }
  ThreadStackPointerType const oldstack = Lisp2DThreadSP ( ) ;
  ULongPub * const ostlp = StackNew_ULong ( ) ;
  ULong_Init  ( ostlp ) ;
  // ! to catch to destroy ost
  ULongDivWithOstSetMoveArg ( ULongToPub( myp ) , ULongToPub ( lp ) , ostlp ) ;
  Lisp2D_StackFree  ( oldstack ) ;
}

# include "castb.h"
