// Lisp2D - Vector #()
// ©2011-23 by Alex Glebe
# include "define.h"
# include "vector/public.h"

# ifdef  LISP2D_CONSTRSTAT
_Atomic ( size_t  ) PLOVector_constrstat = 0 ;

size_t  PLOVector_constrstatCValue ( void  ) {
  return  PLOVector_constrstat  ; 
}

void  PLOVector_constrstat_Inc ( void  ) {
  ++  PLOVector_constrstat  ;
}
  
# endif // LISP2D_CONSTRSTAT

_Atomic ( size_t  ) PLOVector_gcvectorslength = 0 ;

void  PLOVector_gcvectorslength_Inc ( void  ) {
  ++  PLOVector_gcvectorslength  ;
}

void  PLOVector_gcvectorslength_Dec ( void  ) {
  --  PLOVector_gcvectorslength  ;
}

void  PLOVector_gcvectorslength_Add ( size_t const x ) {
  PLOVector_gcvectorslength += x ;
}

void  PLOVector_gcvectorslength_Sub ( size_t const x ) {
  PLOVector_gcvectorslength -= x ;
}

size_t  PLOVector_gcvectorslength_CValue  ( void  ) {
  return  PLOVector_gcvectorslength ;
}

void  PLOVector_gcvectorslength_Clear ( void  ) {
  PLOVector_gcvectorslength = 0 ;
}

# include "vector/struct.h"

void  PLOVector_Push  ( PLOVector * const me , ParaLispObject  * const xp )  {
  Vector_PushByValue  ( StackPLOP ) ( & me  ->  value  , xp  ) ;
  PLOVector_gcvectorslength_Inc ( ) ;
}

# include "inerrors.h"
  
void  PLOVector_Insert  ( PLOVector * const me , size_t const  i  , ParaLispObject  * const xp  ) {
# ifdef  LISP2D_DEBUG
  size_t  const vs  = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
  if  ( i  > vs ) {
    inErrorS  (
      PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
        ( PArray_wchar_c ) { (wchar_t const(*)[])&L"PLOVector_Insert: i=" });
      PLOStreamPushUSize(& Lisp2Dferr , i );
      PLOStreamPushWideCharArray  ( & Lisp2Dferr , ( PArray_wchar_c ) {
        (wchar_t const(*)[])&L" s=" } ) ;
      PLOStreamPushUSize(& Lisp2Dferr ,   vs  ) ; )
    return  ;
  }
# endif
  VectorPushAtValue ( StackPLOP ) ( & me -> value  , i , xp ) ;
  PLOVector_gcvectorslength_Inc ( ) ;
}

# include "string/public.h"
# include "string/struct.h"
  
void  PLOVector_PushToIndex(PLOVector * const me ,
  ParaLispObject * const xp,size_t index , StackClusterPrivate * const stclpp ){
  size_t  const   vs = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
  if(index  > vs){
    StackClusterPrivate_ExceptionSet  ( stclpp ,  & PLOString_NewCopy (
      L"PLOVectorPushToIndex:index  > vs" ) ->  plo ) ;
    return  ;
  }
  VectorPushAtValue ( StackPLOP ) ( & me -> value  , index , xp ) ;
  PLOVector_gcvectorslength_Inc ( ) ;
}

# include "sharedc.h"
# include "shared/inline.h"
  
# define  PLOVector_CoutPri  Lisp2D_PLOVector_CoutPri 
static  void  PLOVector_CoutPri ( PLOVector const * const me ,
  FuncCoutToInterpretHuman  funccout  , PLOStream  const * const osp,
  bool  flag2pprinted,Vector(StackPLOP)  * const looking2p) {
  bool  flagpprinted = flag2pprinted ;
  if(Printed3Points(&me->plo,osp,&flagpprinted,looking2p))  
    return;
  PLOStreamPushWideCharArray  ( osp ,( PArray_wchar_c) { (wchar_t const(*)[]) & L"#("  } ) ;
  { size_t  const   s = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
    if(s){
      Vector_PushByValue  ( StackPLOP ) ( looking2p  , &((PLOVector * )me)->plo);
      size_t  const   sizem1= s  - 1;
      ParaLispObject const * xinext = Vector_FrontCopy(StackPLOP)(&me->value );
      for(size_t    i = 0 ;i  < sizem1;++  i){
        funccout( xinext , osp  , & flagpprinted , looking2p  ) ;
        xinext  = Vector_EltCopy(StackPLOP)(&me->value ,i + 1);
        PLOStreamPushWChar(osp,L' ');
      }
      funccout( Vector_EltCopy(StackPLOP)(&me->value ,sizem1) , osp  , & flag2pprinted , looking2p  ) ;
      VectorPop(StackPLOP)(looking2p);
    } // s>0
  } // s
  PLOStreamPushWChar(osp,L')');
}

# include "vector/public.h"

void  PLOVector_staticCoutToInterpret ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  , Vector(StackPLOP)  * const h ){
  PLOVector_CoutPri(ParaLispObjectToPLOVectorConst(  me  ), ParaLispObject_CoutToInterpret,s,flag2pprinted,h);
}

void  PLOVector_staticCoutToHuman ( ParaLispObject const * const me ,
  PLOStream  const * const s , bool const flag2pprinted  , Vector(StackPLOP)  * const h ) {
  PLOVector_CoutPri ( ParaLispObjectToPLOVectorConst(  me  ) ,
    ParaLispObject_CoutToHuman , s , flag2pprinted , h ) ; 
}

# include "errorstra.h"
# include "nilc.h"

ParaLispObject  * PLOVector_PopLisp ( PLOVector * const me  ) {
# ifdef  LISP2D_DEBUG
  if  ( VectorEmpty ( StackPLOP ) ( & me  ->  value  ) ) {
    ErrorStra ( ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) &
      L"PLOVector_PopLisp: vector is empty" } ) ;
    return & plonil . plo ; 
  }
# endif
  ParaLispObject  * const xp =  Vector_PopValue ( StackPLOP ) ( & me  ->  value  ) ;
  PLOVector_gcvectorslength_Dec ( ) ;
  return   xp ;
}

ParaLispObject  * PLOVector_PopLispInd ( PLOVector * const myp , size_t const ind ) {
  if  ( VectorEmpty ( StackPLOP ) ( & myp ->  value  ) )  {
    ErrorStra ( ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) &
      L"PLOVectorPopLispInd: vector is empty" } ) ;
    return & plonil . plo ;
  }
  if(ind  >= VectorSizeCopy  ( StackPLOP ) ( & myp ->  value  ) ){
    ErrorStra ( ( PArray_wchar_c  ) { ( wchar_t const ( * ) [ ] ) &
      L"PLOVectorPopLispInd: index not less then vector size" } ) ;
    return & plonil . plo ;
  }
  ParaLispObject  * const x = Vector_EltCopy  ( StackPLOP ) ( & myp ->  value  , ind ) ;
  Vector_EraseInd ( StackPLOP ) ( & myp ->  value  , ind ) ;
  PLOVector_gcvectorslength_Dec ( ) ;
  return  x ;
}

void  PLOVector_SetElt  ( PLOVector * const me  , size_t  const i , ParaLispObject  * const rp  ) {
  size_t  const   vs = VectorSizeCopy  ( StackPLOP ) ( & me ->  value  ) ;
  if  ( i  < vs )
    * Vector_EltAddr  ( StackPLOP ) ( & me ->  value  , i ) = rp  ;
  else  {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOVector_SetElt:i >= vs" ;
    longjmp ( * Lisp2DJBglobal ,   Lisp2DExcTypeString ) ;
  }
}

bool  PLOVector_staticisBADCopyPrivate ( ParaLispObject const * const me , ParaLispObject  * * const resultp  )  {
  PLOVector * res2  ;
  if  ( PLOVector_isBADCopyStatic ( ParaLispObjectToPLOVectorConst  ( me  ) , & res2  ) )
    return true ;
  ( * resultp ) = ( & res2  ->  plo ) ;
  return false  ;
}

bool  PLOVector_isBADCopyStatic ( PLOVector const * const me ,
  PLOVector  * * const resultp ) {
  PLOVector * const  res  = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if  ( res ) {
    PLOVector_InitCopy  ( res , me  ) ;
    PLOVector_staticPushAsNew ( & res ->  plo ) ;
    * resultp  = res ;
    return  false ;
  }
  return  true  ;
}

// real copy without error
# define  PLOVector_CopyMatrix  Lisp2D_PLOVector_CopyMatrixk66B
static  ParaLispObject  * PLOVector_CopyMatrix  ( PLOVector * const me ,
  StackClusterPrivate * const stclp , Vector  ( StackPLOP ) * const historyp )  {
// !! to do : cycle check with LINEAR complexity
//  bool  tocheck
//  pointer slow
  { size_t  const hs = VectorSizeCopy  ( StackPLOP ) ( historyp  ) ;
    for ( size_t  i = 0 ; i  < hs ; ++ i  )
      if  ( Vector_EltCopy  ( StackPLOP ) ( historyp  , i )  ==  &  me  ->  plo ) {
        ParaLispObject  * list [ ] = { &  PLOString_NewCopy ( L"vector is cycled" ) ->
          plo , NULL } ;
        StackClusterPrivate_ExceptionSet  ( stclp ,
          ListOfPLO ( ( PArrayPLOP  ) {
              ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;      
        return & plonil . plo ;
      }
  }
  PLOVector * const newa2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if  ( newa2 ) {
    PLOVector_InitCopy  ( newa2 , me  ) ;
    PLOVector_staticPushAsNew ( & newa2 ->  plo ) ;
  } else  {
    Lisp2DExcOutOfMemoryGlobal . value = L"PLOVector_CopyMatrix" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  Vector_PushByValue  ( StackPLOP ) ( historyp  , & me  ->  plo ) ;
  if  ( not  VectorEmpty  ( StackPLOP ) ( & me  ->  value  ) ) {
    ParaLispObject  * * i = VectorFront ( StackPLOP ) ( & me  ->  value  ) ;
    for(  ; i  not_eq  Vector_CBack ( StackPLOP ) ( &  me ->  value  ) ; ++ i  )
      if  ( Vectorp ( * i ) )
        ( * i ) = PLOVector_CopyMatrix  ( ParaLispObjectToPLOVector ( * i ) , stclp , historyp  ) ;
    if  ( Vectorp ( * i ) )
      ( * i ) = PLOVector_CopyMatrix  (
        ParaLispObjectToPLOVector ( * i ) , stclp , historyp  ) ;
  }
  VectorPop ( StackPLOP ) ( historyp  ) ;
  return  & newa2 ->  plo ;
}

# include "stcl/public.h"
# include "stcl/inline.h"

// vector copy-matrix
TaskForThread Lisp2Dbimvectorcopymatrix2  ( StackClusterPrivate  * const stclpp ) {
  PLOVector * const mep = ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  Vector  ( StackPLOP ) history ;
  VectorInit  ( StackPLOP ) ( & history , false ) ;
  StackClusterPrivate_SetResult ( stclpp , PLOVector_CopyMatrix ( mep , stclpp  , & history  ) ) ;
  Vector_Destr  ( StackPLOP ) ( & history  ) ;
  return  TaskForThread_Analyse ;
}
  
# include "cons/public.h"
# include "cons/inline.h"

// vector merge
TaskForThread Lisp2Dbimvectormerge2 ( StackClusterPrivate * const stclpp  ) {
  PLOVector * const mep = ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp ) ) ;
  PLOVector * res2  ;
  ParaLispObject  * j ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) not_eq Lisp2DEvalAddrContZero2 ) {
    res2  = ParaLispObjectToPLOVector ( StackClusterPrivate_PopValue  ( stclpp )  ) ;
    j = StackClusterPrivate_PopValue  ( stclpp )  ;
    goto  Next  ;
  }
  { ParaLispObject * const  arglistp =  StackClusterPrivate_ArgTop  ( stclpp )  ;
    if  ( not  CConsp ( arglistp  ) ) {
      StackClusterPrivate_SetResult ( stclpp ,  & mep ->  plo ) ;
      return TaskForThread_Analyse  ;
    }
    res2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
    if  ( res2  ) {
      PLOVector_InitCopy  ( res2  , mep ) ;
      PLOVector_staticPushAsNew ( & res2  ->  plo ) ;
    } else  {
      Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectormerge2" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    j = arglistp  ;
  }
Next  :
  { unsigned  int tasks = maxListItemProcessing ;
    for ( ; ( tasks  > 0  ) and CConsp (  j ) ;
      j = ClassCons_Rest  ( & ParaLispObjectToPLOCons ( j ) ->  cc ) , --  tasks ) {
      ParaLispObject  * const arrjp = ClassCons_First ( & ParaLispObjectToPLOCons ( j ) ->  cc  ) ;
      if  ( IsNotType ( vector  ) ( stclpp ,  arrjp ) )
        return TaskForThread_Analyse  ;
      size_t  k = 0 ;
      for ( ; k  < VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue (
        ParaLispObjectToPLOVector ( arrjp ) ) ) ; ++  k )
        PLOVector_Push  ( res2  ,
          Vector_EltCopy ( StackPLOP )  ( PLOVector_CValue  ( ParaLispObjectToPLOVector ( arrjp ) ) , k ) ) ;
    }
  }
  if ( not CConsp  ( j ) )
    StackClusterPrivate_SetResult ( stclpp ,  &  res2 ->  plo ) ;
  else {
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    StackClusterPrivate_Push  ( stclpp , j ) ;
    StackClusterPrivate_Push  ( stclpp ,  & res2  ->  plo ) ;
  }
  return  TaskForThread_Analyse ;
}

//  vector reverse
TaskForThread Lisp2Dbimvectorreverse2 ( StackClusterPrivate * const stclpp  ) {
  PLOVector * const me = ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  if  ( not  me )
    return  TaskForThread_Analyse ;
  size_t  const s = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  ( me  ) ) ;
  PLOVector * const res2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if  ( res2  ) {
    PLOVector_InitSizeFill  ( res2  , s , & plonil  . plo ) ;
    PLOVector_staticPushAsNew ( & res2  ->  plo ) ;
  } else  {
    Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectorreverse2" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  if  ( s ) {
    size_t  j1 = s - 1 ;
    size_t  j0 = 0 ;
    for ( ; j0  < s ; ++  j0  , -- j1 )
      PLOVector_SetElt  ( res2  , j1  , Vector_EltCopy ( StackPLOP ) (
        PLOVector_CValue  ( me  ) , j0  ) ) ;
  } // if s
  StackClusterPrivate_SetResult ( stclpp ,  & res2  ->  plo ) ;
  return  TaskForThread_Analyse ;
}

// vector clear
TaskForThread Lisp2Dbimvectorclear2 ( StackClusterPrivate * const stclpp  ) {
  StackClusterPrivate_SetResult ( stclpp , & PLOVector_Clear ( ParaLispObjectToPLOVector (
      StackClusterPrivate_ThisTop ( stclpp ) ) ) ->  plo ) ;
  return  TaskForThread_Analyse ;
}

# include "minmaxint.h"

// vector transpose
TaskForThread Lisp2Dbimvectortranspose2 ( StackClusterPrivate * const stclpp  ) {
  PLOVector * const  mep = ParaLispObjectToPLOVector  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  size_t  maxwidth = 0 ;
  size_t const  s = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  ( mep  ) ) ;
  for ( size_t  i = 0 ; i  < s  ; ++  i ) {
    ParaLispObject  * const rowip  = Vector_EltCopy  ( StackPLOP ) (
      PLOVector_CValue  ( mep ) , i ) ;
    if  ( Vectorp ( rowip ) )
      maxwidth  = Lisp2DMaxSize ( VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue (
          ParaLispObjectToPLOVector ( rowip  ) ) ) , maxwidth  ) ;
  }
  PLOVector * const res2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if  ( res2 )  {
    PLOVector_InitSizeFill  ( res2  , maxwidth  , & plonil  . plo ) ;
    PLOVector_staticPushAsNew ( & res2  ->  plo ) ;
  } else  {
    Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectortranspose2 I" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  for ( size_t  i = 0 ; i  < maxwidth ; ++ i  ) {
    PLOVector * const resi2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
    if  ( resi2 ) {
      PLOVector_InitSizeFill  ( resi2 , maxwidth  , & plonil  . plo ) ;
      PLOVector_staticPushAsNew ( & resi2 ->  plo ) ;
    } else  {
      Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectortranspose2 II" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    PLOVector_SetElt  ( res2  , i , & resi2 ->  plo ) ;
    for(size_t  j = 0 ; j  < maxwidth ; ++ j  ) {
      ParaLispObject  * ij = &  plonil . plo ;
      ParaLispObject  * const rowjp  = Vector_EltCopy  ( StackPLOP ) (
        PLOVector_CValue  ( mep  ) , j ) ;
      if  ( Vectorp ( rowjp ) )
        ij  = Vector_EltCopy  ( StackPLOP ) ( PLOVector_CValue  (
          ParaLispObjectToPLOVector ( rowjp  ) ) , i ) ;
      PLOVector_SetElt  ( resi2 , j , ij  ) ;
    }
  }
  StackClusterPrivate_SetResult ( stclpp ,  & res2  ->  plo ) ;
  return  TaskForThread_Analyse ;
}

void  PLOVector_ResizeFill  ( PLOVector * const me ,  size_t  const n , ParaLispObject  * const xp  ) {
  size_t const s  = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
  if  ( s  not_eq  n  ) {
    Vector_ResizeFill ( StackPLOP ) ( & me  ->  value  , n , xp ) ;
    PLOVector_gcvectorslength_Add  ( n ) ;
    PLOVector_gcvectorslength_Sub  ( s  ) ;
  }
}

void  PLOVector_Resize  ( PLOVector * const me , size_t  const n  ) {
  size_t const s  = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
  if  ( s  not_eq  n  ) {
    Vector_Resize ( StackPLOP ) ( & me  ->  value  , n ) ;
    PLOVector_gcvectorslength_Add  ( n ) ;
    PLOVector_gcvectorslength_Sub  ( s  ) ;
  }
}

PLOVector *  PLOVector_Clear  ( PLOVector * const me ) {
  size_t const  s  = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
  if  ( s  not_eq  0  ) {
    PLOVector_gcvectorslength_Sub  ( s  ) ;
    VectorClear ( StackPLOP ) ( & me  ->  value  ) ; }
  return  me  ;
}

# include "boolean.h"
  
// returns TRUE or FALSE
ParaLispObject  * PLOVector_RemProp ( PLOVector * const me , ParaLispObject * const  sp  ) {
  size_t vs  = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
  if  ( vs > 0  ) {
    if  ( vs ==  1  ) {
      if  ( sp  ==  CCAR ( Vector_FrontCopy ( StackPLOP ) ( & me  ->  value  ) ) ) {
        VectorClear ( StackPLOP ) ( & me  ->  value  ) ;
        PLOVector_gcvectorslength_Dec ( ) ;
        return & plotrue  . plo ;
      }
    } else  {
      if  ( vs ==  2  ) {
        if  ( sp  ==  CCAR ( Vector_FrontCopy ( StackPLOP ) ( & me  ->  value  ) ) ) {
          Vector_EraseInd ( StackPLOP ) ( & me  ->  value  , 0 ) ;
          PLOVector_gcvectorslength_Dec ( ) ;
          return & plotrue  . plo ;
        }
        if  ( sp  ==  CCAR ( * Vector_Back ( StackPLOP ) ( & me  ->  value  ) ) ) {
          VectorPop ( StackPLOP ) ( & me  ->  value  ) ;
          PLOVector_gcvectorslength_Dec ( ) ;
          return & plotrue  . plo ;
        }
      } else  {
        // size > 2
        if  ( sp  ==  CCAR ( Vector_FrontCopy ( StackPLOP ) ( & me  ->  value  ) ) ) {
          Vector_EraseInd ( StackPLOP ) ( & me  ->  value  , 0 ) ;
          PLOVector_gcvectorslength_Dec ( ) ;
          return & plotrue  . plo ;
        }
        if  ( sp  ==  CCAR ( * Vector_Back ( StackPLOP ) ( & me  ->  value  ) )  ) {
          VectorPop ( StackPLOP ) ( & me  ->  value  ) ;
          PLOVector_gcvectorslength_Dec ( ) ;
          return & plotrue  . plo ;
        }
        size_t  uj0 = 0 ;
        size_t  uj1 = vs  - 1 ;
        size_t  uj2 ;
        while ( uj1 > ( 1 + uj0 ) ) {
          uj2 = ( ( uj0 + uj1 ) >>  1 ) ;
          if  ( sp  ==  CCAR ( Vector_EltCopy ( StackPLOP ) ( & me  ->  value  , uj2 ) ) ) {
            Vector_EraseInd ( StackPLOP ) ( & me  ->  value  , uj2 ) ;
            PLOVector_gcvectorslength_Dec ( ) ;
            return & plotrue  . plo ;
          }
          if  ( sp  < CCAR ( Vector_EltCopy ( StackPLOP ) ( & me  ->  value  , uj2 ) ) )
            uj1 = uj2 ;
          else      
            uj0 = uj2 ;
        }
      }
    }
  }
  return & plofalse . plo ;
}

// vector list
TaskForThread Lisp2Dbimvectorlist2  ( StackClusterPrivate  * const stclpp ) {
  Vector  ( StackPLOP ) const * const me =
    PLOVector_CValue ( ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp )  ) ) ;
  ParaLispObject  *  res = & plonil . plo ;
// to write: for(=end; !=begin; --)
  { size_t  i = VectorSizeCopy  ( StackPLOP ) ( me  ) ;
    while ( i  >  0 ) {
      --  i ;
      res = & ( NewCons ( Vector_EltCopy  ( StackPLOP ) ( me  , i ) , res ) ->  plo ) ;
    }
  } // i
  StackClusterPrivate_SetResult ( stclpp ,  res ) ;
  return  TaskForThread_Analyse ;
}

// vector empty
TaskForThread Lisp2Dbimvectorempty2 ( StackClusterPrivate * const stclpp  ) {
  PLOVector * const mep = ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  StackClusterPrivate_SetResult ( stclpp , (  ( VectorEmpty ( StackPLOP ) (
    & mep ->  value  ) ) ?
    & mep ->  plo :
    & ( plofalse  . plo ) ) ) ;
  return  TaskForThread_Analyse ;
}

//  vector setf vector
TaskForThread Lisp2Dbimvectorsetf2  ( StackClusterPrivate  * const stclpp ) {
  ParaLispObject  * const argp = CCAR ( StackClusterPrivate_ArgTop  ( stclpp )  ) ;
  if  ( IsNotType ( vector  ) ( stclpp ,  argp  ) )
    return  TaskForThread_Analyse ;
  PLOVector * const  mep = ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  PLOVector_Set ( mep , ParaLispObjectToPLOVector ( argp  ) ) ;
  StackClusterPrivate_SetResult ( stclpp ,  & mep ->  plo ) ;
  return  TaskForThread_Analyse ;
}

void  PLOVector_Set ( PLOVector * const me  ,
  PLOVector const * const  xp ) {
  if  ( me ==  xp )
    return  ;
  { size_t const vs = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
    size_t const xvs  = VectorSizeCopy  ( StackPLOP ) ( & xp  ->  value  ) ;
    PLOVector_gcvectorslength_Sub  ( vs ) ;
    PLOVector_gcvectorslength_Add  ( xvs ) ;
  }
// ? insert under mutex -> speed -> decrease ?
  VectorSet ( StackPLOP ) ( & me  ->  value  , & xp  ->  value  ) ;
}

// returns nil | (s . val)
ParaLispObject * PLOVector_Assoc3 ( PLOVector const * const me ,
  ParaLispObject  const * const sp ) {
  size_t const vsize = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  ( me  ) ) ;
  if  ( vsize ) {
    { ParaLispObject  * const cellp = Vector_FrontCopy  ( StackPLOP ) (
        PLOVector_CValue  ( me  ) ) ;
      ParaLispObject  * const symbolp = CCAR ( cellp ) ;
        if  ( sp  == symbolp ) 
          return  cellp ;
      if  ( ( vsize ==  1 ) or  (  sp  <   symbolp  ) )     
        return & plonil . plo ;
    }
    { ParaLispObject * const cell2p = * Vector_CBack ( StackPLOP ) ( &  me ->  value  ) ;
      ParaLispObject  * const symbol2p =  CCAR ( cell2p ) ;
      if  ( sp  ==    symbol2p )       
        return  cell2p  ;
      if  ( symbol2p  < sp  )   
        return & plonil . plo ;
    }
    size_t  i0 = 0 ;
    size_t  i2 = vsize  - 1 ;
    while ( ( i0 + 1  ) < i2  ) {
      size_t  const i1 = (  i0  + i2  ) >>  1 ;
      ParaLispObject  * const cellp =  Vector_EltCopy ( StackPLOP ) (
        & me  ->  value  , i1  ) ;
      ParaLispObject  * const cellsymbolp = CCAR ( cellp )  ;
      if  ( sp  ==    cellsymbolp )      
        return  cellp ;
      if  ( sp  <   cellsymbolp )   
        i2  = i1  ;
      else      
        i0  = i1  ;
    }
  }
  return & plonil . plo ;
}

void  PLOVector_PushLink  ( PLOVector * const me , ParaLispObject  * const ap ,
  ParaLispObject  * const bp , StackClusterPrivate  * const stclp ) {
  // #(... (object . value) ...)
  if  ( VectorEmpty ( StackPLOP ) ( & me  ->  value  ) ) {
    PLOCons * const newcons2 =   NewCons ( ap  , bp  ) ;
    PLOVector_Push  ( me  , &  newcons2 ->  plo ) ;
    return  ;
  }
  // length>0
  ParaLispObject  * arr0  = Vector_FrontCopy  ( StackPLOP ) ( PLOVector_CValue  ( me  ) ) ;
  if  ( IsNotCons ( stclp , arr0  ) ) 
    return  ;
  // already exist
  ParaLispObject  * symbenv = ClassCons_First ( & ParaLispObjectToPLOCons ( arr0  ) ->  cc  ) ;
  if  ( ap  ==  symbenv ) {
    ClassCons_SetRest ( & ParaLispObjectToPLOCons ( arr0  ) ->  cc  , bp  ) ;
    return  ;
  }
  // less then [0]
  if  ( ap  < symbenv ) {
    PLOCons * const newcons2 = NewCons ( ap  , bp  ) ;
    PLOVector_Insert  ( me  , 0 , & newcons2  ->  plo ) ;
    return  ;
  }
  if  ( VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  ( me  ) )  ==  1  ) {
    PLOCons * const newcons2 =   NewCons ( ap  , bp  ) ;
    PLOVector_Insert  ( me  , 1 , & newcons2  ->  plo ) ;
    return  ;
  }
  // already exist
  arr0  = * Vector_CBack ( StackPLOP ) ( &  me ->  value  ) ;
  if  ( IsNotCons ( stclp , arr0  ) )
    return  ;
  symbenv = ClassCons_First ( & ParaLispObjectToPLOCons ( arr0  ) ->  cc ) ;
  if  ( ap  ==  symbenv ) {
    ClassCons_SetRest ( & ParaLispObjectToPLOCons ( arr0  ) ->  cc  , bp  ) ;
    return  ;
  }
  // more then [length-1]
  if  ( symbenv  < ap ) {
    PLOCons * const newcons2 = NewCons ( ap  , bp  ) ;
    PLOVector_Push  ( me  , &  newcons2 ->  plo ) ;
    return  ;
  }
  size_t    j0 = 0  ;
  size_t    j1 = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  ( me  ) )  - 1 ;
  while ( ( j1 - j0 ) > 1 ) {
    size_t  const   j2 = (  j0  + j1  ) >>  1 ;
    // already exist
    arr0  = Vector_EltCopy  ( StackPLOP ) ( & me  ->  value  , j2  ) ;
    if  ( IsNotCons ( stclp , arr0  ) )
      return  ;
    symbenv = ClassCons_First ( & ParaLispObjectToPLOCons ( arr0  ) ->  cc ) ;
    if  ( ap  ==  symbenv ) {
      ClassCons_SetRest ( & ParaLispObjectToPLOCons ( arr0  ) ->  cc  , bp  ) ;
      return  ;
    }
    // less then [j2]
    if  ( ap  < symbenv )    
      j1  = j2  ;
    else 
      j0  = j2  ;
  }
  PLOCons * const newcons2 = NewCons ( ap  , bp  ) ;
  PLOVector_Insert  ( me  , j1  , & newcons2  ->  plo ) ;
}

// vector first
TaskForThread Lisp2Dbimvectorfirst2 ( StackClusterPrivate * const stclpp  ) {
  PLOVector * const  mep = ParaLispObjectToPLOVector  ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  if  ( VectorEmpty ( StackPLOP ) ( & mep ->  value  ) ) {
    ParaLispObject  * list [ ] = { & PLOString_NewCopy  ( L"vector is empty"  )
      ->  plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stclpp ,  ListOfPLO ( ( PArrayPLOP ) {
      ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;
    return  TaskForThread_Analyse ;
  }
  StackClusterPrivate_SetResult ( stclpp , Vector_FrontCopy ( StackPLOP ) (
    PLOVector_CValue  ( mep ) ) ) ;
  return  TaskForThread_Analyse ;
}

// vector last
TaskForThread Lisp2Dbimvectorlast2  ( StackClusterPrivate  * const stclpp ) {
  PLOVector * const mep = ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  if  ( VectorEmpty ( StackPLOP ) ( & mep ->  value  ) ) {
    ParaLispObject  * list [ ] = { & 
      PLOString_NewCopy ( L"vector is empty"  ) -> plo , NULL } ;
    StackClusterPrivate_ExceptionSet  ( stclpp ,  ListOfPLO ( ( PArrayPLOP  ) {
      ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;
    return  TaskForThread_Analyse ;
  }
  StackClusterPrivate_SetResult ( stclpp , * Vector_CBack ( StackPLOP ) ( & mep -> value  ) )  ;
  return  TaskForThread_Analyse ; 
}

# include "consiter/public.h"
# include "consiter/struct.h"
# include "stack/public.h"

/*(nil  defmethod map (objseq func  &rest arglist &env  env)  \
  (nil  cond  \
    ((nil vectorp objseq) \
      (nil  let ((n (objseq size))) \
        (n  for depth \
          (nil  apply (objseq elt depth)  func  \
            (arglist  for vectori \
              (nil  if  (depth  < (vectori  size))  \
                (vectori  elt depth)))  \
            env)))) \
    ((nil consp objseq) \
      (nil  cons  \
        (nil  apply (objseq first)  func  \
          (arglist  for i \
            (i  first)) \
          env)  \
        (nil  apply nil 'map  \
          (nil  list* (objseq rest) func  \
            (arglist  for i \
              (i  rest))) \
          env)))))*/

// vector map func {vector}
TaskForThread Lisp2Dbimvectormap2 ( StackClusterPrivate * const stclpp  ) {
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) {
    StackClusterPrivate_SetResult ( stclpp , StackClusterPrivate_PopValue ( stclpp )  ) ;
    return  TaskForThread_Analyse ;
  }
  PLOVector * const mep = ParaLispObjectToPLOVector ( StackClusterPrivate_ThisTop ( stclpp )  ) ;
  size_t  const mys = VectorSizeCopy  ( StackPLOP ) ( & mep -> value  )  ;
  PLOVector * const res2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if  ( res2   )  {
    PLOVector_InitSizeFill  ( res2  , mys , & plonil  . plo ) ;
    PLOVector_staticPushAsNew ( & res2  ->  plo ) ;
  } else  {
    Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectormap2" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  if  ( mys  ==  0  ) {
    // #() map func #() .. -> #()
    StackClusterPrivate_SetResult ( stclpp ,  & res2  ->  plo ) ;
    return  TaskForThread_Analyse ;
  }
  ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop ( stclpp )  ;
  ParaLispObject  * const funcp =  CCAR ( arglistp  ) ;
  ParaLispObject  * const vectorlistp = CCDR ( arglistp ) ;
  for ( size_t  i = mys ; i  > 0  ; ) {
    --  i ;
    PLOCons * lastcons2 = NewCons ( funcp , & plonil  . plo ) ;
    PLOCons *  taski2 = NewCons (
      Vector_EltCopy  ( StackPLOP ) ( & mep ->  value  , i ) ,
      & lastcons2 ->  plo ) ;
// !!  remove ConsIterator & make TaskPause
    { ThreadStackPointerType const oldstack_j = Lisp2DThreadSP ( ) ; 
      CConsIteratorPub * const j = NewStackCConsIterator ( ) ;
      CConsIteratorInit ( j , vectorlistp ) ;
    for ( ; CConsIteratorHaveItem ( j ) ; CConsIteratorNext ( j ) ) {
      ParaLispObject  * const vp = CConsIteratorItem  ( j ) ;
      ParaLispObject  *  xj = & plonil  . plo ;
      if  ( Vectorp ( vp  ) )
        if  ( i  < VectorSizeCopy  ( StackPLOP ) ( &
          ParaLispObjectToPLOVector ( vp ) ->  value  ) )
          xj  = Vector_EltCopy  ( StackPLOP ) ( &
            ParaLispObjectToPLOVector ( vp ) ->  value  , i ) ;
      PLOCons * const next2 = NewCons ( xj  , & plonil  . plo ) ;
      ClassCons_SetRest ( & lastcons2 ->  cc  , & next2 ->  plo ) ;
      lastcons2 = next2 ; 
    }
      Lisp2D_StackFree  ( oldstack_j ) ;
    } // jp_
    StackClusterPrivate  * const dopstclp = StackClusterPrivate_NewStCl4  ( stclpp )  ;
    SetResultPlace  (  dopstclp ,  PLOVector_AddrElt ( res2 , i ) , & res2  ->  plo ) ;
    StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
    StackClusterPrivate_SetEnv  ( dopstclp ,  StackClusterPrivate_ExtEnv  ( stclpp )  ) ;
    StackClusterPrivate_CreateTask  ( dopstclp ,  & taski2  ->  plo ) ;
  }
  StackClusterPrivate_Push  ( stclpp ,  & res2  ->  plo ) ;
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse ; 
}

// vector mapc func {vector}
TaskForThread Lisp2Dbimvectormapc2  ( StackClusterPrivate  * const stclpp ) {
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) {
    StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
    return  TaskForThread_Analyse ; }
  PLOVector * const mep = ParaLispObjectToPLOVector (
    StackClusterPrivate_ThisTop ( stclpp )  ) ;
  size_t  const   mys = VectorSizeCopy  ( StackPLOP ) ( & mep -> value  ) ;
  if  ( mys  ==  0  ) {
    // #() map func #() .. -> nil
    StackClusterPrivate_SetResult ( stclpp ,  & plonil  . plo ) ;
    return  TaskForThread_Analyse ; 
  }
  ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop ( stclpp )  ;
  ParaLispObject  * const funcp = CCAR ( arglistp ) ;
  ParaLispObject  * const vectorlistp = CCDR ( arglistp ) ;
// !! TaskPause
  for ( size_t  i = mys ; i ; ) { 
    -- i  ;
    PLOCons *  lastcons2  ;
    lastcons2 =   NewCons ( funcp , & plonil  . plo ) ;
    PLOCons *  taski2 ;
    taski2  = NewCons ( Vector_EltCopy  ( StackPLOP ) ( & mep ->  value  ,
      i ) , & lastcons2 ->  plo ) ;
// !! TaskPause
    { ThreadStackPointerType const oldstack_j = Lisp2DThreadSP ( ) ; 
      CConsIteratorPub * const j = NewStackCConsIterator ( ) ;
      CConsIteratorInit ( j , vectorlistp ) ;
      for ( ; CConsIteratorHaveItem ( j ) ; CConsIteratorNext ( j ) ) {
        ParaLispObject  * const vp = CConsIteratorItem  ( j ) ;
        ParaLispObject  * xj  = &  plonil . plo ;
        if  ( Vectorp ( vp  ) )
          if  ( i < VectorSizeCopy  ( StackPLOP ) ( &
            ParaLispObjectToPLOVector ( vp  ) -> value  )  )
            xj  = Vector_EltCopy  ( StackPLOP ) ( & ParaLispObjectToPLOVector
              ( vp ) ->  value  , i ) ;
        PLOCons * const next2 = NewCons ( xj  , & plonil  . plo ) ;
        ClassCons_SetRest ( & lastcons2 ->  cc  , & next2 ->  plo ) ;
        lastcons2 = next2 ;
      }
      Lisp2D_StackFree  ( oldstack_j ) ;
    } // j
    StackClusterPrivate  * const dopstclp = StackClusterPrivate_NewStCl4  ( stclpp )  ;
    StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
    StackClusterPrivate_SetEnv  ( dopstclp ,  StackClusterPrivate_ExtEnv  ( stclpp )  ) ;
    StackClusterPrivate_CreateTask  ( dopstclp ,  & taski2  ->  plo ) ; 
  }
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse ; 
}

void  PLOVector_staticDeclareForGC  ( ParaLispObject * const me ) {
  if  ( not  me -> flagGCPassed ) {
    me -> flagGCPassed = true  ;
    GlobalListPopNotLock  ( & GlobalListOfObjectsGarbage , & me -> base )  ;
    GlobalListPushNotLock ( & GlobalListOfObjects , & me -> base ) ;
    PLOVector_gcvectorslength_Add  ( Vector_MemorySizeCopy ( StackPLOP ) (
        & ParaLispObjectToPLOVector ( me  ) -> value  ) ) ;
  }
}

void  PLOVector_staticGCDeclareContent ( ParaLispObject * const me ) {
  PLOVector * const v = ParaLispObjectToPLOVector ( me  ) ;
  ParaLispObject * * const e = VectorEnd  ( StackPLOP ) ( & v ->  value  ) ;
  for ( ParaLispObject  * * i = VectorBegin ( StackPLOP ) ( & v ->  value  ) ; i not_eq e  ; ++  i )
    ParaLispObject_virtualDeclareForGC  ( * i ) ;
}

ParaLispObject  * PLOVector_staticTypeOf ( ParaLispObject const * _ __attribute__ ((unused)) ) {
  return   &  plosvector . plo ;
}

void  PLOVector_InitSizeFill  ( PLOVector * const me  , size_t const n , ParaLispObject  * const initp ) {
  ParaLispObject_InitVTP  ( & me  ->  plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOVector } ) ;
  VectorInit  ( StackPLOP ) ( & me  ->  value , false ) ;
  Vector_ResizeFill ( StackPLOP ) ( & me  ->  value  , n , initp ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOVector_constrstat_Inc  ( ) ;
# endif
}

void  PLOVector_InitSize  ( PLOVector * const me , size_t const n ) {
  ParaLispObject_InitVTP(&me->plo , ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOVector });
  VectorInit  ( StackPLOP ) ( & me->value  , false ) ;
  Vector_Resize ( StackPLOP ) ( & me->value  , n ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOVector_constrstat_Inc  ( ) ;
# endif
}

void  PLOVector_InitCopy  ( PLOVector * const me , PLOVector  const * const vp ) {
  ParaLispObject_InitVTP  ( & me  ->  plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOVector } ) ;
  VectorInit  ( StackPLOP ) ( & me -> value  , false ) ;
  VectorSet ( StackPLOP ) ( & me -> value  , & vp -> value  ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOVector_constrstat_Inc  ( ) ;
# endif
}

# define  PLOVector_SubSeqPri Lisp2D_PLOVector_SubSeqPri
// e <= vp->size and DOESNOT check vp=me
static  void  PLOVector_SubSeqPri ( PLOVector * const me , PLOVector const * const vp ,
  size_t  const indexfirst  , size_t const e  ) {
  if(indexfirst < e){
    { size_t  const  news = e - indexfirst ;
      PLOVector_Resize(me,news);
    }
    ParaLispObject  * * j = VectorBegin(StackPLOP)(&me->value );
    ParaLispObject  * const * i = VectorCBegin(StackPLOP)(&vp->value )+indexfirst ;
    ParaLispObject  * const * const je = VectorCEnd(StackPLOP)(&me->value ) ;
    do{
      (*  j)  = (*  i);
      ++  j;
      ++  i;
    }while(j  not_eq  je);
  }
}

void  PLOVector_InitFirst ( PLOVector * const me , PLOVector  const * const vp , size_t  const indexfirst ) {
  ParaLispObject_InitVTP  ( & me  ->  plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOVector } ) ;
  VectorInit  ( StackPLOP ) ( & me -> value , false  ) ;
  size_t  const e = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  ( vp ) ) ;
  PLOVector_SubSeqPri ( me  , vp  , indexfirst  , e ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOVector_constrstat_Inc  ( ) ;
# endif
}

void  PLOVector_InitFirstEnd  ( PLOVector * const me , PLOVector  const * const vp , size_t  const indexfirst ,
  size_t const indexend ) {
  ParaLispObject_InitVTP  ( & me -> plo , ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOVector } ) ;
  VectorInit  ( StackPLOP ) ( & me -> value , false  ) ;
  size_t  const   e = Lisp2DMinSize ( indexend  , VectorSizeCopy  ( StackPLOP ) ( & vp -> value  ) ) ;
  PLOVector_SubSeqPri ( me  , vp  , indexfirst  , e ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOVector_constrstat_Inc  ( ) ;
# endif
}

void  PLOVector_Destr ( PLOVector * const me){
  Vector_Destr(StackPLOP)(&me->value );
  ParaLispObject_staticDestr(&me->plo);
}

void PLOVector_staticDestr  ( ParaLispObject * const me) {
  PLOVector_Destr (ParaLispObjectToPLOVector ( me  ));
}

ParaLispObject  * PLOVector_staticPushAsNew ( ParaLispObject * const me ) {
  GlobalListPush ( & GlobalListOfObjects , & me -> base ) ;
  PLOVector_gcvectorslength_Add  (VectorSizeCopy  ( StackPLOP ) (
    PLOVector_CValue  ( ParaLispObjectToPLOVector(me) ) ) );
  return  me ;
}

ParaLispObject  * PLOVector_staticClassDef  ( ParaLispObject const * _
  __attribute__ ((unused)) ) {
  return  ploclassdefvector ;
}

ParaLispObject  * PLOVector_staticClassParents  ( ParaLispObject const * _
  __attribute__ ((unused)) ) {
  return  ploclassparentsvector ;
}

// nil vectorp
TaskForThread Lisp2Dbimnilvectorp2(StackClusterPrivate  * const stclpp){
  ParaLispObject  * const xp = CCAR ( StackClusterPrivate_ArgTop(stclpp ));
  StackClusterPrivate_SetResult(stclpp ,(Vectorp(xp) ? xp : &plofalse.plo));
  return  TaskForThread_Analyse;
}

// nil vector
TaskForThread Lisp2Dbimnilvector2 ( StackClusterPrivate * const stclpp){
  PLOVector * const res = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if  ( not res ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimnilvector2" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  PLOVector_Init(res);
  StackClusterPrivate_SetResult(stclpp ,
    PLOVector_staticPushAsNew(&res->plo));
  return  TaskForThread_Analyse;
}

void  PLOVector_Init ( PLOVector * const me ) {
  ParaLispObject_InitVTP(&me->plo ,
    ( VirtualTypePointer ) { . addr = { . p = me } , . type = TypeGlobalListMemberPLOVector });
  VectorInit  ( StackPLOP ) ( & me->value , false  ) ;
# ifdef  LISP2D_CONSTRSTAT
  PLOVector_constrstat_Inc  ( ) ;
# endif
}

# include "complex/public.h"
# include "bimc.h"

// vector *=
TaskForThread Lisp2Dbimvectormultsetf2(StackClusterPrivate  * const stclpp){
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) ) {
    StackClusterPrivate_SetResult(stclpp , StackClusterPrivate_PopValue(stclpp ));
    return  TaskForThread_Analyse;
  }
  // (v *=)
  { PLOVector * const mep = ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(stclpp ));
    size_t  const  s = VectorSizeCopy  ( StackPLOP ) ( & mep -> value  );
    if(s  ==  0){
      StackClusterPrivate_SetResult(stclpp ,&mep->plo);
      return  TaskForThread_Analyse;
    }
    ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop(stclpp );
    if(not  CConsp(arglistp)){
      StackClusterPrivate_SetResult(stclpp ,&mep->plo);
      return  TaskForThread_Analyse;
    }
    if(IsNotNumber(  stclpp , CCAR ( arglistp)))
      return  TaskForThread_Analyse;
    
// !! TaskPause
  
    { for(size_t  i=s;i  >  0;){
        -- i;
        // (v[i]  *=  args)
        ParaLispObject  * const me2p = Vector_EltCopy(StackPLOP)(&mep->value ,i);
        switch(ParaLispObject_Type(me2p)){
        case  TypeGlobalListMemberPLOComplex:
          { StackClusterPrivate * const  dopstclp = StackClusterPrivate_NewStCl4(stclpp );
            SetResultPlace  (  dopstclp ,  PLOVector_AddrElt ( mep , i ) , & mep -> plo ) ;
            StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
            PLOCons * const newcons3 = NewCons( &plobimcomplexmultiplysetf->base.plo,arglistp);
            PLOCons * const newcons2  =   NewCons(me2p, & newcons3->plo);
            StackClusterPrivate_CreateTask(dopstclp ,&  newcons2->plo);
          }
          break;
        case  TypeGlobalListMemberPLOVector:
          { StackClusterPrivate * const  dopstclp = StackClusterPrivate_NewStCl4(stclpp );
            SetResultPlace  (  dopstclp ,  PLOVector_AddrElt ( mep , i ) , & mep -> plo ) ;
            StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
            PLOCons * const newcons3 =   NewCons( &plobimvectormultsetf->base.plo,arglistp);
            PLOCons * const newcons2 =   NewCons(me2p, & newcons3->plo);
            StackClusterPrivate_CreateTask(dopstclp ,&  newcons2->plo);
          }
          break;
        default:;
          ParaLispObject  * list [ ] = { me2p ,
            & PLOString_NewCopy(L" is not number nor vector")->plo, NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        }
      }
      StackClusterPrivate_Push(stclpp ,&mep->plo);
    }
  }  // me
  StackClusterPrivate_setTaskForManager ( stclpp  , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
}

# include "complex/inline.h"
# include "plocomplex/public.h"

//  vector elt
TaskForThread Lisp2Dbimvectorelt2 ( StackClusterPrivate * const stclpp  ) {
  ParaLispObject  * s ;
  ParaLispObject  * j ;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp ) not_eq Lisp2DEvalAddrContZero2 ) {
    s = StackClusterPrivate_PopValue  ( stclpp )  ;
    j = StackClusterPrivate_PopValue  ( stclpp )  ;
    goto  Next  ; 
  }
  if  ( not  (  s  = StackClusterPrivate_ThisTop  ( stclpp )  ) )   
    return  TaskForThread_Analyse ;
  { ParaLispObject  * const arglistp = StackClusterPrivate_ArgTop ( stclpp )  ;
    if  ( not  CConsp ( arglistp  ) ) {
      StackClusterPrivate_SetResult ( stclpp ,  s ) ;
      return  TaskForThread_Analyse ; 
    }  // this
    j = arglistp  ; 
  }
Next  :
  { unsigned  int tasks = maxListItemProcessing ;
    for ( ; ( tasks  > 0  ) and CConsp (  j ) ;
      j  = ClassCons_Rest ( & ParaLispObjectToPLOCons ( j ) ->  cc ) , --  tasks ) {
      ParaLispObject  * const jcarp = ClassCons_First ( & ParaLispObjectToPLOCons ( j ) ->  cc  ) ;
      if  ( IsNotNumber ( stclpp , jcarp  ) )
        return  TaskForThread_Analyse ;
      if  ( IsNotType ( vector  ) ( stclpp ,  s ) )
        return  TaskForThread_Analyse ;
      if  ( FractionLessUInt (
        Complex_CRe (PLOComplexToComplexConst(
          ParaLispObjectToPLOComplex  ( jcarp  )) ) , 0U  ) ) {
        ParaLispObject  * list [ ] = { jcarp  ,
          &  PLOString_NewCopy  ( L"<0" ) ->  plo , NULL } ;
        StackClusterPrivate_ExceptionSet  ( stclpp ,
          ListOfPLO ( (PArrayPLOP)  {
          ( ParaLispObject  * ( * ) [ ] ) & list  } ) ) ;
        return  TaskForThread_Analyse ;
      }
    { size_t svs = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  (
        ParaLispObjectToPLOVector ( s ) ) ) ;
      if  ( FractionMoreEqUSize  ( 
        Complex_CRe  (PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst  ( jcarp  ))) , svs ) ) {
        PLOComplex  * const res =   NewComplexBareExc ( ) ;
        Complex_SetUSize (  PLOComplexToComplex( res), svs ) ;
        { ParaLispObject  * list [ ] = { jcarp  ,
            &  PLOString_NewCopy  ( L">=" ) ->  plo , & res ->  plo , NULL } ;
          StackClusterPrivate_ExceptionSet  ( stclpp ,  ListOfPLO (
            (PArrayPLOP)  { (
            ParaLispObject  * ( * ) [ ] ) & list  } ) ) ; 
        }
        return  TaskForThread_Analyse ; 
      } 
    }
    size_t  ind ;    
    if  ( FractionIsBadSizeValue ( 
        Complex_CRe  (PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst  ( jcarp )) ) , & ind ) ) {
      ParaLispObject  * list [ ] = { jcarp  , &
        PLOString_NewCopy  ( L" is bad" ) ->  plo , NULL } ;
      StackClusterPrivate_ExceptionSet  ( stclpp ,
        ListOfPLO ( (PArrayPLOP)  { ( ParaLispObject  * ( * ) [ ] ) & list  }
        ) ) ;
      return  TaskForThread_Analyse ; 
    }
    s = Vector_EltCopy ( StackPLOP )  ( PLOVector_CValue  (
      ParaLispObjectToPLOVector ( s ) ) , ind ) ; 
    }
  }
  if  ( CConsp  ( j ) ) {
    StackClusterPrivate_setTaskForManager ( stclpp , TaskPause ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    StackClusterPrivate_Push  ( stclpp , j  ) ;
    StackClusterPrivate_Push  ( stclpp , s  ) ;
    return  TaskForThread_Analyse ; }
  StackClusterPrivate_SetResult ( stclpp ,  s ) ;
  return  TaskForThread_Analyse ; }

//  vector size
TaskForThread Lisp2Dbimvectorsize2(StackClusterPrivate  * const stclpp){
  PLOVector  * const  me  =ParaLispObjectToPLOVector (
    StackClusterPrivate_ThisTop(stclpp ));
  if(me){
    size_t  const  mvs = VectorSizeCopy  ( StackPLOP ) (
      PLOVector_CValue  ( me  ) ) ;
    PLOComplex  * const  res = NewComplexBareExc();
    Complex_SetUSize  ( PLOComplexToComplex( res),mvs);
    StackClusterPrivate_SetResult(stclpp ,&  res->plo);
  }
  return  TaskForThread_Analyse; 
}

//  vector push what [index] 
TaskForThread Lisp2Dbimvectorpush2(StackClusterPrivate  * const stclpp){
  PLOVector * const me=ParaLispObjectToPLOVector (
    StackClusterPrivate_ThisTop ( stclpp ));
  if(not  me) 
    return  TaskForThread_Analyse;
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const indexp= CCAR ( CCDR ( arglistp));
  if(not  Numberp(indexp)){
    PLOVector_Push(me,  CCAR (  arglistp));
    StackClusterPrivate_SetResult(stclpp ,&  me->plo);
    return  TaskForThread_Analyse;
  }
  if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indexp))))){
    ParaLispObject  * list [ ] = { indexp,& PLOString_NewCopy(
      L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  if(FractionLessUInt(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplex(
        indexp))),0U)){
    ParaLispObject  * list [ ] = { indexp,& PLOString_NewCopy(L"<0")->plo,
      NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  { size_t  ind;
    if( FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indexp))), &ind))
      return  TaskForThread_Analyse;
    PLOVector_PushToIndex(me , CCAR ( arglistp),ind,stclpp ); 
  } // ind
  StackClusterPrivate_SetResult(stclpp ,&  me->plo);
  return  TaskForThread_Analyse;
}

//  vector pop [index_place]
TaskForThread bimvectorpop  (StackClusterPrivate * const stclpp){
  PLOVector * const  mep=ParaLispObjectToPLOVector(
    StackClusterPrivate_ThisTop(stclpp ));
  if  ( VectorEmpty ( StackPLOP ) ( PLOVector_CValue  ( mep  ) ) ) {
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"vector is empty")
      ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO(
      (PArrayPLOP){(ParaLispObject*(*)[])&list}));
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  if(CConsp(arglistp)){
    ParaLispObject  * const indp=ClassCons_First(&ParaLispObjectToPLOCons(
      arglistp) ->cc);
    if(IsNotNumber(  stclpp ,indp))
      return  TaskForThread_Analyse;
    if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indp))))){
        ParaLispObject  * list [ ] = { indp,& PLOString_NewCopy(
          L" is not integer")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    if(FractionLessUInt(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          indp))), 0U)){
      ParaLispObject  * list [ ] = { indp,& PLOString_NewCopy(L" < 0")->plo,
        NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    size_t  ind1;
    if(  FractionIsBadSizeValue(Complex_CRe(
      PLOComplexToComplexConst(
      ParaLispObjectToPLOComplexConst(indp))), &ind1)){
      ParaLispObject  * list [ ] = { indp,& PLOString_NewCopy(
        L" is bad size value")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse ; 
    }
    ParaLispObject  * const r = PLOVector_PopLispInd ( mep , ind1 ) ;
    StackClusterPrivate_SetResult(stclpp ,r); 
  } else  
    StackClusterPrivate_SetResult(stclpp ,PLOVector_PopLisp(mep));
  return  TaskForThread_Analyse;
}

// vector subseq [indexfirst [indexend]] => [first..end-1]
TaskForThread Lisp2Dbimvectorsubseq2  ( StackClusterPrivate  * const stclpp){
  PLOVector * const mep=
    ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  if(not  CConsp(arglistp)){
    PLOVector  * result;
    if  ( PLOVector_isBADCopyStatic ( mep  , & result  ) ) {
      Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectorsubseq2 I" ;
      longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
    }
    StackClusterPrivate_SetResult(stclpp ,&  result->plo);
    return  TaskForThread_Analyse ; 
  }
  ParaLispObject  * const indp=ClassCons_First(&ParaLispObjectToPLOCons(
    arglistp)->cc);
  if(IsNotNumber(  stclpp ,indp))   
    return  TaskForThread_Analyse;
  if(not  FractionIntegerP(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indp))))){
    ParaLispObject  * list [ ] = { indp,& PLOString_NewCopy(
      L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { if(FractionLessUInt(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          indp))), 0U)){
      ParaLispObject  * list [ ] = { indp,& PLOString_NewCopy(L" < 0")->plo,
        NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
        (ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
  }
  size_t  ind1;
  if(  FractionIsBadSizeValue(Complex_CRe(
    PLOComplexToComplexConst(
        ParaLispObjectToPLOComplexConst(indp))),&ind1)){
    ParaLispObject  * list [ ] = { indp,& PLOString_NewCopy(L" is very big")
      ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  ParaLispObject  * const arglist2p  = ClassCons_Rest  (
    &ParaLispObjectToPLOCons ( arglistp )->cc ) ;
  if(CConsp(arglist2p)){
    ParaLispObject  * const indendp=ClassCons_First(&
      ParaLispObjectToPLOCons(arglist2p)->cc);
    if(IsNotNumber(  stclpp ,indendp))    
      return  TaskForThread_Analyse;
    if(not  FractionIntegerP(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          indendp ))))){
      ParaLispObject  * list [ ] = { indendp,&  PLOString_NewCopy(
        L" is not integer")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
        (ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    if(FractionLess(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indendp))),
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indp))))  == Lisp2DBoolTrue) {
      ParaLispObject  * list [ ] = { indendp,&  PLOString_NewCopy(L" < ")->plo,
        indp, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
        (ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    size_t  ind2;
    if(  FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        indendp))), &ind2)){
      ParaLispObject  * list [ ] = { indendp,&  PLOString_NewCopy(
        L" is too big") ->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    { PLOVector * const res2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
      if  ( res2 ) {
        PLOVector_InitFirstEnd(res2,mep,ind1,ind2);
        PLOVector_staticPushAsNew(&res2->plo); 
      } else  {
        Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectorsubseq2 II" ;
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
      }
      StackClusterPrivate_SetResult(stclpp ,&  res2->plo);  
    } // res2
    return  TaskForThread_Analyse;
  }
  PLOVector * const res2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if ( not res2 ) {
    Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectorsubseq2 III" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  PLOVector_InitFirst ( res2  , mep  , ind1  ) ;
  PLOVector_staticPushAsNew(&res2->plo); 
  StackClusterPrivate_SetResult(stclpp ,&  res2->plo);
  return  TaskForThread_Analyse;
}

// vector resize n new
TaskForThread Lisp2Dbimvectorresize2(StackClusterPrivate  * const stclpp){
  PLOVector * const  mep=ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const np= CCAR ( arglistp);
  if(IsNotNumber(  stclpp ,np))   
    return  TaskForThread_Analyse;
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
        ParaLispObjectToPLOComplex(np))),0U)){
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"size ")->plo, np,
      & PLOString_NewCopy(L"<0")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { size_t  ind;
    if( FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        np))),&ind))
      return  TaskForThread_Analyse;
    PLOVector_ResizeFill(mep,ind,  CCAR ( CCDR ( arglistp)));
  }
  StackClusterPrivate_SetResult(stclpp ,&mep->plo);
  return  TaskForThread_Analyse;
}

// vector setelt i j k new
TaskForThread Lisp2Dbimvectorsetelt2(StackClusterPrivate  * const stclpp){
  PLOVector * const  mep=ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(
    stclpp ));
  ParaLispObject  * arri=&  mep->plo;
  
// !! TaskPause
  
  ParaLispObject  * ijk = StackClusterPrivate_ArgTop(stclpp );
  StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  // (#() setelt) -> nil
  if  ( not CConsp (  ijk ) )
    return  TaskForThread_Analyse ;
  ParaLispObject  * ivalue = ClassCons_Rest( &ParaLispObjectToPLOCons ( ijk )
    ->cc ) ;
  // (#() setelt i) -> error
  if  ( not CConsp (  ivalue )  ) {
    StackClusterPrivate_ExceptionSet(stclpp ,&PLOString_NewCopy(
      L"value to set is absent")->plo);
    return  TaskForThread_Analyse ; 
  }
  // (#() setelt i v)
  // another ijk
  { unsigned  int tasks = maxListItemProcessing ;
  for ( ; ( tasks  > 0  ) and CConsp  ( ClassCons_Rest  (
    &ParaLispObjectToPLOCons ( ivalue ) ->cc) ) ;
    // (i v ?) -> (.. i v)
    ijk  = ivalue ,
    ivalue = ClassCons_Rest ( &ParaLispObjectToPLOCons ( ivalue )->cc ) ,
    --  tasks ) {
    // (#() setelt i v)
    ParaLispObject  * const ip=ClassCons_First(&ParaLispObjectToPLOCons(ijk)
      ->cc);
    if(IsNotNumber(  stclpp ,ip))     
      return  TaskForThread_Analyse;
    if(not  FractionIntegerP(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          ip))))){
      ParaLispObject  * list [ ] = { ip,& PLOString_NewCopy(
        L" is not integer")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){(
        ParaLispObject*(*)[])&list}));          
      return  TaskForThread_Analyse;
    }
    if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
          ParaLispObjectToPLOComplex(ip))),0U)){
      ParaLispObject  * list [ ] = { ip,& PLOString_NewCopy(L"<0")->plo, NULL
        } ;
      StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){(
        ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    { size_t  uind;
      if( FractionIsBadSizeValue(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          ip))), &uind))
        return  TaskForThread_Analyse;
      size_t  arris = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  (
        ParaLispObjectToPLOVector ( arri  ) ) ) ;
      if(uind >=  arris){
        PLOComplex  * const  res = NewComplexBareExc();
        Complex_SetUSize  ( PLOComplexToComplex( res),uind);
        { PLOComplex  * const  res2 = NewComplexBareExc();
          Complex_SetUSize  ( PLOComplexToComplex( res2),arris);
          {  ParaLispObject  * list [ ] = { &res->plo,& PLOString_NewCopy(
              L" is more than size=")->plo, &res2->plo, NULL } ;
            StackClusterPrivate_ExceptionSet(stclpp ,
              ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
          }
        }
        return  TaskForThread_Analyse;
      }
      arri  = Vector_EltCopy  ( StackPLOP ) ( PLOVector_CValue  (
        ParaLispObjectToPLOVector ( arri  ) ) , uind  ) ; 
    }
      if  ( IsNotType ( vector  ) (  stclpp ,  arri ) )
        return  TaskForThread_Analyse ;
    }  // for
  } // tasks
  if  ( CConsp ( ClassCons_Rest ( &ParaLispObjectToPLOCons ( ivalue )->cc ) ) ) {
    StackClusterPrivate_ExceptionSet(stclpp ,&PLOString_NewCopy(
      L"index big depth is not supported")->plo);
    return  TaskForThread_Analyse ; 
  }
    
// !! TaskPause
      
  ParaLispObject  * const ip=ClassCons_First(&ParaLispObjectToPLOCons(ijk)
    ->cc);
  if(IsNotNumber(  stclpp ,ip)) 
    return  TaskForThread_Analyse;
  if(not  FractionIntegerP(Complex_CRe(PLOComplexToComplexConst(
        ParaLispObjectToPLOComplexConst(ip))))){
    ParaLispObject  * list [ ] = { ip,& PLOString_NewCopy(L" is not integer")->
      plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){(
      ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
        ParaLispObjectToPLOComplex(ip))),0U)){
    ParaLispObject  * list [ ] = { ip,& PLOString_NewCopy(L"<0")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse;
  }
  { size_t  uind;
    if( FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        ip))),&uind))
      return  TaskForThread_Analyse;
    size_t const arris = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  (
      ParaLispObjectToPLOVector ( arri  ) ) ) ;
    if(uind >=  arris){
      PLOComplex  * const  res=NewComplexBareExc();
      { Complex_SetUSize  ( PLOComplexToComplex( res),uind);
        PLOComplex  * const  res2=NewComplexBareExc();
        { Complex_SetUSize  ( PLOComplexToComplex( res2),arris);
          ParaLispObject  * list [ ] = { &res->plo,& PLOString_NewCopy(
            L" is more than size=")->plo, &res2->plo, NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
        }
      }
      return  TaskForThread_Analyse;
    }
    PLOVector_SetElt(ParaLispObjectToPLOVector(arri),uind,  CCAR ( ivalue));
  }
  StackClusterPrivate_SetResult(stclpp ,&mep->plo);
  return  TaskForThread_Analyse;
}

# include "env/public.h"

// vector for i {body}
// i - is const variable
TaskForThread Lisp2Dbimvectorfor2(StackClusterPrivate * const stclpp){
  PLOVector * const  mep=ParaLispObjectToPLOVector(
    StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const varBp= CCAR ( arglistp);
  PLOComplex  * j4 ;
  ParaLispObject  * task2 ;
  { EvalAddrCont2  const c  = StackClusterPrivate_ClearflagCont ( stclpp ) ;
  if(c  not_eq  Lisp2DEvalAddrContZero2){
    switch(c){
    case  1:
      StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
      return  TaskForThread_Analyse;
    default:
      task2=StackClusterPrivate_PopValue(stclpp );
      j4  =  ParaLispObjectToPLOComplex(StackClusterPrivate_PopValue(stclpp ));
      if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
        StackClusterPrivate_setTaskForManager ( stclpp ,
          TaskWaitSonsAlmost ) ;
        StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
        return  TaskForThread_Analyse;
      }
      goto  TaskNext; 
    }
    return  TaskForThread_Analyse; 
  } 
  }
  { size_t const  s = VectorSizeCopy  ( StackPLOP ) ( & mep -> value  );
    if(s  ==  0)   
      return  TaskForThread_Analyse;
    if(IsNotSymbol(  stclpp ,varBp))     
      return  TaskForThread_Analyse;
    task2 =   &NewCons( &plobimnilprogn->base.plo,  CCDR ( arglistp))->plo;
    task2 =   &NewCons(&plonil.plo, task2)->plo; 
// !! TaskPause
    size_t const   sminus1 = s  - 1U ;
    
    j4  = NewComplexBareExc();
    
    Complex_SetUSize  ( PLOComplexToComplex( j4),sminus1);      
  }
TaskNext:
    { unsigned  int tasks = maxTasksCreate ;
    for(;(tasks  > 0) and (FractionMoreEqUInt(
      Complex_CRe(PLOComplexToComplexConst(j4)),0U));
        FractionSub1  (Complex_Re(PLOComplexToComplex(j4)))
        ,-- tasks){
      StackClusterPrivate  * const dopstclp =
        StackClusterPrivate_NewStCl4(stclpp );
      StackClusterPrivate_EnvNewLevel2(dopstclp);
      if(IsNotEnvironmentLayer(  stclpp ,
         CCAR ( StackClusterPrivate_Env(dopstclp)))){
        StackClusterPrivate_EnvCDR(dopstclp);
        return  TaskForThread_Analyse;
      }
      { size_t  ind ;
        if ( FractionIsBadSizeValue ( Complex_CRe (
          PLOComplexToComplexConst ( j4 ) ) , & ind ) ) 
          return  TaskForThread_Analyse;
        if  ( EnvironmentLayerisBADPush ( & 
          ParaLispObjectToPLOEnvironmentLayer (
              CCAR ( StackClusterPrivate_Env(dopstclp ) ) ) -> envlay ,
            varBp  ,
            Vector_EltCopy(StackPLOP)(&mep -> value ,ind) , false ) ) {
          HasConstLink(  stclpp , varBp);
          return  TaskForThread_Analyse;
        }
      }
        StackClusterPrivate_CreateTask(dopstclp ,  task2); 
      }  // for
    } // tasks
    if(FractionMoreEqUInt(Complex_CRe(PLOComplexToComplexConst(
      j4)),0U)){
      StackClusterPrivate_setTaskForManager ( stclpp ,  TaskPause ) ;
      StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
      StackClusterPrivate_Push(stclpp ,& j4->plo);
      StackClusterPrivate_Push(stclpp , task2);
      return  TaskForThread_Analyse; 
    }
  StackClusterPrivate_setTaskForManager ( stclpp , TaskWaitSonsAlmost ) ;
  StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
  return  TaskForThread_Analyse;
}
  
// vector for* symbol {body}
// symbol - is const variable
TaskForThread Lisp2Dbimvectorforcons2(StackClusterPrivate * const stclpp){
  ParaLispObject  * varC;
  PLOComplex  * vari;
  PLOVector *  me;
  ParaLispObject  * arglist;
  if  ( StackClusterPrivate_ClearflagCont ( stclpp )  not_eq 
    Lisp2DEvalAddrContZero2 )
    goto  Next;
  me  =   ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(stclpp ));
  if(VectorEmpty(StackPLOP)(&me->value )){
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
    return  TaskForThread_Analyse;
  }
  arglist = StackClusterPrivate_ArgTop(stclpp ) ;
  
// !! to make varibale for : CCAR( *  arglist)      
  
  if ( IsNotSymbol(  stclpp ,  CCAR ( arglist)))
    return  TaskForThread_Analyse;
  vari  = NewComplexBareExc();
  Complex_Set0  ( PLOComplexToComplex( vari));    
  StackClusterPrivate_EnvNewLevel2(stclpp );
  do {
    { size_t  const   mvs = VectorSizeCopy  ( StackPLOP ) (
        & me  ->  value  ) ;
      if(not  FractionLessUSize (Complex_CRe(
          PLOComplexToComplexConst(vari)),mvs))
        break;
    } // mvs
// !! to make varibale for : CCAR(stcl.Env())
    if(IsNotEnvironmentLayer(  stclpp ,
      CCAR ( StackClusterPrivate_Env(stclpp )))){
      StackClusterPrivate_EnvCDR(stclpp );
      return  TaskForThread_Analyse;
    }
    { size_t  ind;
      if( FractionIsBadSizeValue(Complex_CRe(
          PLOComplexToComplexConst(vari)),&ind)) 
        return  TaskForThread_Analyse;
      if  ( EnvironmentLayerisBADPush ( & ParaLispObjectToPLOEnvironmentLayer (
            CCAR ( StackClusterPrivate_Env(stclpp ) ) ) -> envlay , CCAR (
            arglist ) , Vector_EltCopy(StackPLOP)(&me->value ,ind) , false ) )
      {
          HasConstLink(  stclpp ,  CCAR ( arglist));
          return  TaskForThread_Analyse; 
      } 
    } // ind
    StackClusterPrivate_Push(stclpp ,& vari->plo);
    varC  = CCDR ( arglist);
    while ( CConsp  ( varC  ) ) {
      StackClusterPrivate_Push(stclpp , varC);
      StackClusterPrivate_Push(stclpp ,
        ClassCons_First(&ParaLispObjectToPLOCons(varC)->cc));
      StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
      return  TaskForThread_Analyse;
Next:
      varC=StackClusterPrivate_PopValue(stclpp );
      if(StackClusterPrivate_exceptionnotprocessed  ( stclpp )){
        // vari
        StackClusterPrivate_Popn(stclpp );
        StackClusterPrivate_EnvCDR(stclpp );
        return  TaskForThread_Analyse;
      }
      varC  = ClassCons_Rest  ( &ParaLispObjectToPLOCons ( varC  )->cc ) ;
    }
    vari = ParaLispObjectToPLOComplex(StackClusterPrivate_PopValue(stclpp ));
    me  =   ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(stclpp ));
    arglist = StackClusterPrivate_ArgTop(stclpp ) ; 
    FractionAdd1  (Complex_Re(PLOComplexToComplex(vari)));
  } while ( true  ) ;
  //old environment restoring
  StackClusterPrivate_EnvCDR(stclpp );
  return  TaskForThread_Analyse; 
}

// !!! rewrite to usual function -> next bimnumbermultiply
// vector *
TaskForThread Lisp2Dbimvectormult2  ( StackClusterPrivate  * const stclpp){
  ParaLispObject  * varD;
  PLOVector *  me;
  ParaLispObject  * arglist;
  size_t  s;
  { EvalAddrCont2 const  addr = StackClusterPrivate_ClearflagCont ( stclpp ) ;
  if(addr){
    switch(addr){
    case  1:
      goto  Next1;
    case  2:
      goto  Next2;
    case  3:
      return  TaskForThread_Analyse;
    default:
      inErrorS(
        PLOStreamPushWideCharArray  ( & Lisp2Dferr ,
          (PArray_wchar_c) { (wchar_t const(*)[])&L"vector *: addr=" });
        PLOStreamPushEvalAddrCont(& Lisp2Dferr , addr ) ; )
    }
  }
  } // addr
  // (v *)
  me  =   ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(stclpp ));
  arglist = StackClusterPrivate_ArgTop(stclpp );
  if(not  CConsp( arglist)){
    StackClusterPrivate_SetResult(stclpp ,&  me->plo);
    return  TaskForThread_Analyse;
  }
  varD  = CCAR ( arglist);
  if(Vectorp(  varD)){
    // #(x y z) * #(a b c) = x*a+y*b+z*c
    { size_t  const   s2  = Lisp2DMinSize ( VectorSizeCopy  ( StackPLOP ) ( &
        ParaLispObjectToPLOVector ( varD  ) ->  value  ) ,
        VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ) ;
      if(s2 ==  0){
        PLOComplex  * const  result=NewComplexBareExc();
        Complex_Set0  ( PLOComplexToComplex( result));    
          StackClusterPrivate_SetResult(stclpp ,&  result->plo);
        return  TaskForThread_Analyse;
      }
      // (nil BIM(+))
      { PLOCons * const varB2 = NewCons( &  plobimcomplexplus->
        base.plo,&plonil.plo);
        PLOCons * const vartask2 = NewCons(&plonil.plo,& varB2->plo);
      { StackClusterPrivate  * const  dopstclp =
          StackClusterPrivate_NewStCl4(stclpp );
        SetResultPlace  (  dopstclp ,
          ClassCons_PFirst ( &vartask2 ->cc) , & vartask2  ->  plo ) ;
        StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
        // (v[i]  * args)
        ParaLispObject  * const xip = Vector_FrontCopy(StackPLOP)(
          PLOVector_CValue(me));
        if(Numberp(xip)){
          ParaLispObject  * list [ ] = { xip,&plobimcomplexmultiply->base.plo,
            Vector_FrontCopy(StackPLOP)(
              PLOVector_CValue(ParaLispObjectToPLOVector(
              varD))) , NULL } ;
          StackClusterPrivate_CreateTask(dopstclp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        } else
          if(Vectorp(xip)){
            ParaLispObject  * list [ ] = { xip,&plobimvectormult->base.plo,
              Vector_FrontCopy(StackPLOP)(PLOVector_CValue(ParaLispObjectToPLOVector(
              varD))) , NULL } ;
            StackClusterPrivate_CreateTask(dopstclp ,
              ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          } else {
            ParaLispObject  * list [ ] = { xip,
              &  PLOString_NewCopy(L" is not number nor vector")->plo, NULL } ;
            StackClusterPrivate_ExceptionSet(stclpp ,
              ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
          }
      } // dopstcl
          
// !! TaskPause
          
      for(size_t  i=1;i  < s2;++ i){
        PLOCons * const vartop2 = NewCons(&plonil.plo,&plonil.plo);
        ClassCons_SetRest(&varB2->cc,&  vartop2->plo);
        StackClusterPrivate  * const dopstclp =
          StackClusterPrivate_NewStCl4(stclpp );
        SetResultPlace  (  dopstclp , ClassCons_PFirst ( &vartop2->cc ) ,
          & vartop2 ->  plo ) ;
        StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
      ParaLispObject  * const xip = Vector_EltCopy(StackPLOP)(&me->value ,i);
      if(Numberp(xip)){
        ParaLispObject  * list [ ] = { xip,&plobimcomplexmultiply->base.plo,
          Vector_EltCopy(StackPLOP)(&ParaLispObjectToPLOVector(varD)->value ,i)
          , NULL } ;
        StackClusterPrivate_CreateTask(dopstclp,
          ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      } else
        if(Vectorp(xip)){
          ParaLispObject  * list [ ] = { xip,&plobimvectormult->base.plo,
            Vector_EltCopy(StackPLOP)(&ParaLispObjectToPLOVector(varD)->value ,i)
            , NULL } ;
          StackClusterPrivate_CreateTask(dopstclp,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        } else{
          ParaLispObject  * list [ ] = { xip ,
            &  PLOString_NewCopy(L" is not number nor vector")->plo, NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
        }
      }
    StackClusterPrivate_Push(stclpp ,& vartask2->plo);
      }
    } // s2
    // (nil BIM(+) nil ... nil)
    StackClusterPrivate_setTaskForManager ( stclpp , 
      TaskWaitSonsAlmost ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 2U  ) ;
    return  TaskForThread_Analyse;
Next2:
    // (x BIM(+) y ... z)
    StackClusterPrivate_setflagApply  ( stclpp , true  ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 3U  ) ;
    return  TaskForThread_Analyse;
  }
  s = VectorSizeCopy  ( StackPLOP ) ( & me  ->  value  ) ;
  if(s){
    { PLOVector  * mec;
      if  ( PLOVector_isBADCopyStatic ( me  , & mec ) ) {
        Lisp2DExcOutOfMemoryGlobal . value = L"Lisp2Dbimvectormult2" ;
        longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
      }
      PLOVector * const varresp = mec ;
      
    if(IsNotNumber(  stclpp , varD)) 
      return  TaskForThread_Analyse;
    
// !! TaskPause
    
    { size_t  i = s ;
    while ( i  >  0 ) {
      -- i;
      StackClusterPrivate  * const dopstclp =
        StackClusterPrivate_NewStCl4(stclpp );
      SetResultPlace  (  dopstclp , PLOVector_AddrElt ( varresp , i ) ,
        & varresp  -> plo ) ;
      StackClusterPrivate_setflagApply  ( dopstclp , true  ) ;
      // (v[i]  * args)
      ParaLispObject  * const xip = Vector_EltCopy(StackPLOP)(&varresp->
        value ,i);
      if(Numberp(xip)){
        PLOCons * newcons3 = NewCons(
            &plobimcomplexmultiply->base.plo, arglist);
        PLOCons * const newcons2 = NewCons(xip,& newcons3->plo);
        StackClusterPrivate_CreateTask(dopstclp ,&  newcons2->plo);
      } else
        if(Vectorp(xip)){
          PLOCons * const newcons3 = NewCons( & plobimvectormult->base.plo,
            arglist);
          PLOCons * const newcons2 = NewCons(xip,& newcons3->plo);
          StackClusterPrivate_CreateTask(dopstclp ,&  newcons2->plo); 
        } else {
          ParaLispObject  * list [ ] = { xip,
            &  PLOString_NewCopy(L" is not number nor vector")->plo, NULL } ;
          StackClusterPrivate_ExceptionSet(stclpp ,
            ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list})); 
        }
      }
    }
    StackClusterPrivate_Push(stclpp ,&varresp->plo);
    } // mec
    StackClusterPrivate_setTaskForManager ( stclpp , 
      TaskWaitSonsAlmost ) ;
    StackClusterPrivate_SetflagCont ( stclpp , 1U  ) ;
    return  TaskForThread_Analyse;
Next1:
    StackClusterPrivate_SetResult(stclpp ,
      StackClusterPrivate_PopValue(stclpp ));
  } else
    StackClusterPrivate_SetResult(stclpp ,&plonil.plo);
  return  TaskForThread_Analyse; 
}

# define  MakeVector  Lisp2D_MakeVectorbBNT
static  ParaLispObject  * MakeVector  ( ParaLispObject * const dimsp ,
  ParaLispObject  * const  inip , StackClusterPrivate * const fatherp ,
  Vector ( StackPLOP ) * const historyp ) {
  size_t  const historys  = VectorSizeCopy ( StackPLOP ) ( historyp ) ;
  for(size_t i = 0; i  not_eq  historys;++  i)
    if(Vector_EltCopy(StackPLOP)(historyp, i)  == dimsp){
      ParaLispObject  * list [ ] = {  dimsp,&  PLOString_NewCopy(L" is cycled")
        ->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(fatherp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return & plonil.plo;
    }
  //0-dimension=atom
  if( dimsp ==   & plonil.plo ) //  Null
    return  inip;
  Vector_PushByValue(StackPLOP)(historyp,dimsp);
  if(Numberp(dimsp)){
    VectorPop(StackPLOP)(historyp);
    if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
      ParaLispObjectToPLOComplex(dimsp))),0U)){
      ParaLispObject  * list [ ] = { dimsp,&  PLOString_NewCopy(L"<0")->plo,
        NULL } ;
      StackClusterPrivate_ExceptionSet(fatherp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return & plonil.plo;
    }
    size_t  ind;
    if( FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
        dimsp))), &ind)) 
      return & plonil.plo;
    { PLOVector * const res = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
      if( res ){
        PLOVector_InitSizeFill(res,ind,inip);
        return  PLOVector_staticPushAsNew(&res->plo);
      }
    }
    Lisp2DExcOutOfMemoryGlobal . value = L"MakeVector I" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  ParaLispObject  * const dimsCARp =  CCAR ( dimsp);
  if(IsNotNumber(  fatherp ,dimsCARp)){
    VectorPop(StackPLOP)(historyp);
    return & plonil.plo;
  }
  if(not  FractionIntegerP(Complex_CRe(PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(dimsCARp))))){
    VectorPop(StackPLOP)(historyp);
    ParaLispObject  * list [ ] = { dimsCARp,& PLOString_NewCopy(
      L" is not integer") ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(fatherp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return & plonil.plo;
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(dimsCARp))),0U)){
    VectorPop(StackPLOP)(historyp);
    ParaLispObject  * list [ ] = { dimsCARp,& PLOString_NewCopy(L"<0")->plo,
      NULL } ;
    StackClusterPrivate_ExceptionSet(fatherp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return & plonil.plo;
  }
  size_t  usi;
  if( FractionIsBadSizeValue(
    Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
      dimsCARp))), &usi))
    return & plonil.plo;
  PLOVector * const dop2 = aligned_alloc ( _Alignof ( PLOVector ) , sizeof ( PLOVector ) ) ;
  if(dop2  ){
    PLOVector_InitSizeFill(dop2,usi,&plonil.plo);
    PLOVector_staticPushAsNew(&dop2->plo);
  } else{
    VectorPop(StackPLOP)(historyp);
    Lisp2DExcOutOfMemoryGlobal . value = L"MakeVector II" ;
    longjmp ( * Lisp2DJBglobal , Lisp2DExcTypeOutOfMemory ) ;
  }
  ParaLispObject  * const dimsCDRp = CCDR (  dimsp);
  for(size_t    uil = 0 ;uil < usi;++  uil)
    PLOVector_SetElt(dop2,uil,MakeVector( dimsCDRp,inip,fatherp,historyp));
  VectorPop(StackPLOP)(historyp);
  return &  dop2->plo; 
}

# define  PLOVector_MakeMatrix  Lisp2D_PLOVector_MakeMatrix
static  PLOVector *  PLOVector_MakeMatrix(PLOVector * const me ,
  ParaLispObject * const dimsp,
  ParaLispObject * const  inip,StackClusterPrivate * const fatherp ) {
  ParaLispObject  * const dimsCARp = CCAR (  dimsp);
  if(IsNotNumber(  fatherp ,dimsCARp))  
    return  me ;
  if(not  FractionIntegerP(Complex_CRe(PLOComplexToComplexConst(
    ParaLispObjectToPLOComplexConst(dimsCARp))))){
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(
      L"PLOVector::MakeMatrix : ") ->plo, dimsCARp,
      & PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(fatherp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  me ;
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst( ParaLispObjectToPLOComplexConst(dimsCARp))),0U)){
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(
      L"PLOVector::MakeMatrix : ") ->plo, dimsCARp,& PLOString_NewCopy(L"<0")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(fatherp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  me ;
  }
  { size_t  si;
    if( FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst( dimsCARp))), &si))
      return me ;
    PLOVector_ResizeFill(me,si,&plonil.plo);
    if(si){
      ThreadStackPointerType const oldstack_h = Lisp2DThreadSP ( ) ;
      Vector ( StackPLOP ) * const  histor  = StackNew_VectorPLOP ( ) ;
      VectorInit ( StackPLOP ) ( histor , false ) ;
      ParaLispObject  * const dimsCDRp = CCDR (  dimsp);
      // ? iterator
      while(si  > 0){
        --  si;
        PLOVector_SetElt(me,si,MakeVector(dimsCDRp,inip,fatherp,histor )); 
      }
      Lisp2D_StackFree  ( oldstack_h ) ;
    }
  }
  return  me ; 
}

# define  PLOVector_AssignSubSeqInd Lisp2D_PLOVector_AssignSubSeqInd
static  PLOVector * PLOVector_AssignSubSeqInd(PLOVector * const me ,
  PLOVector  * const  vp , PLOComplex * const indp ,
  StackClusterPrivate  * const stp )  ;
  
# define  PLOVector_AssignSubSeqIndEnd  Lisp2D_PLOVector_AssignSubSeqIndEnd
static  PLOVector * PLOVector_AssignSubSeqIndEnd  ( PLOVector * me ,
  PLOVector  * vp , PLOComplex * indp ,
  PLOComplex  * endp , StackClusterPrivate * stp ) ;
  
/*('vector  defmethod vector(x  l)
  (nil  cond
    ((nil numberp x)
      (this resize  x l))
    ((nil consp x )
      ...)
    ((nil vectorp x)
      (this setf  x)))
  this)*/
TaskForThread Lisp2Dbimvectorvector2(StackClusterPrivate  * const stclpp){
  PLOVector * const  mep =  ParaLispObjectToPLOVector (
    StackClusterPrivate_ThisTop ( stclpp ) ) ;
  ParaLispObject  * const arglistp =StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const xp = CCAR ( arglistp);
  switch(ParaLispObject_Type(xp)){
  case  TypeGlobalListMemberPLOComplex:
    if(not  FractionIntegerP(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst
        ( xp))))){
      ParaLispObject  * list [ ] = { xp,& PLOString_NewCopy(L" is not integer")
        ->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse; 
    }
    if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst( xp))),0U)){
      ParaLispObject  * list [ ] = { xp,& PLOString_NewCopy(L"<0")->plo, NULL
        } ;
      StackClusterPrivate_ExceptionSet(stclpp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
    PLOVector_Clear(mep);
    { size_t  ind;
      if( FractionIsBadSizeValue(
        Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst(
          xp))), &ind))
        return  TaskForThread_Analyse;
      PLOVector_ResizeFill(mep,ind,  CCAR ( CCDR ( arglistp))); 
    } // ind
    break;
  case  TypeGlobalListMemberPLOCons:
    PLOVector_Clear(mep);
    PLOVector_MakeMatrix(mep,xp,  CCAR ( CCDR ( arglistp)),stclpp );
    break;
  case  TypeGlobalListMemberPLOVector:
    {ParaLispObject * const  arg2p= CCDR ( arglistp);
    ParaLispObject  * const indp = CCAR ( arg2p);
    if(Numberp(indp)){
      ParaLispObject  * const endp= CCAR ( CCDR ( arg2p));
      if(Numberp(endp))
        PLOVector_AssignSubSeqIndEnd(mep,ParaLispObjectToPLOVector(xp),
          ParaLispObjectToPLOComplex(indp),
          ParaLispObjectToPLOComplex(endp),stclpp );
      else
        PLOVector_AssignSubSeqInd(mep,ParaLispObjectToPLOVector(xp),
          ParaLispObjectToPLOComplex(indp),stclpp); }
    else
      PLOVector_Set(mep,ParaLispObjectToPLOVector(xp)); }
    break;
  default:;
  }
  StackClusterPrivate_SetResult(stclpp ,&mep->plo);
  return  TaskForThread_Analyse;
}

// ! to rewrite n is not size, but as subseq is end
  
PLOVector * PLOVector_AssignSubSeqIndEnd  ( PLOVector * const me ,
  PLOVector  * const  vp , PLOComplex * const indp ,
  PLOComplex  * const endp , StackClusterPrivate * const stp ) {
  if(not  Complex_Integerp(PLOComplexToComplexConst(indp))){
    ParaLispObject  * list [ ] = { &  indp->plo,& PLOString_NewCopy(
      L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ;}
  if(not  Complex_Integerp(PLOComplexToComplexConst(endp))){
    ParaLispObject  * list [ ] = { &  endp->plo,& PLOString_NewCopy(
      L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me;
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
    indp)) ,0U)){
    ParaLispObject  * list [ ] = { &  indp->plo,& PLOString_NewCopy(L" < 0")
      ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ;
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
    endp)) ,0U)){
    ParaLispObject  * list [ ] = { &  endp->plo,& PLOString_NewCopy(L" < 0")
      ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me;
  }
  if(FractionLess(Complex_CRe(PLOComplexToComplexConst(endp)),
    Complex_CRe(PLOComplexToComplexConst(indp)))  ==
    Lisp2DBoolTrue){
    ParaLispObject  * list [ ] = { &  endp->plo,&  PLOString_NewCopy(L" < ")
      ->plo,      & indp->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return me ;
  }
  if(me ==  vp){
    // me=[0..ind-1,ind,ind+1..s-1] -> [ind,..,end-1]
    if(FractionIs0(Complex_CRe(PLOComplexToComplexConst(
      indp)))){
      // me=[0..ind-1,ind,ind+1..s-1] -> [0,..,end-1]
      size_t  end2;
      if( FractionIsBadSizeValue(Complex_CRe(
        PLOComplexToComplexConst(endp)), &end2))
        return me ;
      PLOVector_ResizeFill(me,end2,&plonil.plo);
      return  me ;
    }
    size_t  const vs = VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  (
      me  ) ) ;
    size_t  ind2 ;
    if(  FractionIsBadSizeValue(Complex_CRe(
      PLOComplexToComplexConst(indp)),        &ind2)){
      ParaLispObject  * list [ ] = { &  indp->plo,&  PLOString_NewCopy(
        L" is bad")    ->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
        (ParaLispObject*(*)[])&list}));
      return  me ;
    }
    size_t  end2;
    if(  FractionIsBadSizeValue(Complex_CRe(
      PLOComplexToComplexConst( endp)), &end2)){
      ParaLispObject  * list [ ] = { &  endp->plo,&  PLOString_NewCopy(
        L" is bad") ->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
        (ParaLispObject*(*)[])&list}));
      return  me ;
    }
    size_t  const  newsize=end2 - ind2;
    if(FractionMoreEqUSize(Complex_CRe(
      PLOComplexToComplexConst(indp) ),vs)){
      PLOVector_Clear(me);
      PLOVector_ResizeFill(me,newsize,&plonil.plo);
      return  me ;
    }
    if(newsize  > vs)
      PLOVector_ResizeFill(me,newsize,&plonil.plo);
    ParaLispObject  * * jtarget=VectorBegin(StackPLOP)(&me->value );
    ParaLispObject  * * jsource=VectorBegin(StackPLOP)(&me->value ) + ind2;
    do{
      (*  jtarget)  = (*  jsource);
      ++  jtarget;
      ++  jsource;
      ++  ind2;
    }while((ind2 < vs)  and (ind2  < end2));
    PLOVector_Resize(me,newsize);
    return  me ;
  } // this == v
  size_t  const   vvs=VectorSizeCopy  ( StackPLOP ) (
    PLOVector_CValue  ( vp ) );  
  size_t const vs=vvs;
  size_t  ind2;
  if(  FractionIsBadSizeValue(Complex_CRe(
    PLOComplexToComplexConst( indp)), &ind2)){
    ParaLispObject  * list [ ] = { &  indp->plo,&  PLOString_NewCopy(
      L" is bad") ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ;
  }
  size_t  end2;
  if(  FractionIsBadSizeValue(Complex_CRe(
    PLOComplexToComplexConst( endp)), &end2)){
    ParaLispObject  * list [ ] = { &  endp->plo,&  PLOString_NewCopy(
      L" is bad") ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ;
  }
  size_t  const  newsize=end2 - ind2;
  PLOVector_Clear(me);
  PLOVector_ResizeFill(me,newsize,&plonil.plo);
  if(ind2 >=  vs )
    return  me ;  
  ParaLispObject  * * jtarget=VectorBegin(StackPLOP)(&me->value );
  ParaLispObject  * * jsource=VectorBegin(StackPLOP)(&vp->value )+ind2;
  while((ind2 < vs )  and (ind2  < end2)){
    (*  jtarget)  = (*  jsource);
    ++  jtarget;
    ++  jsource;
    ++  ind2; 
  }
  return  me ;
}

PLOVector * PLOVector_AssignSubSeqInd(PLOVector * const me ,
  PLOVector  * const  vp , PLOComplex * const indp ,
  StackClusterPrivate  * const stp ) {
  if(not  Complex_Integerp(PLOComplexToComplexConst(indp))){
    ParaLispObject  * list [ ] = { &  indp->plo,
      & PLOString_NewCopy(L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  me;
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
    indp)) ,0U)){
    ParaLispObject  * list [ ] = { &  indp->plo,& PLOString_NewCopy(L" < 0")
      ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  me;
  }
  size_t  const  vvs=VectorSizeCopy  ( StackPLOP ) (
    PLOVector_CValue  ( vp ) );  
  if(FractionMoreUSize(Complex_CRe(PLOComplexToComplexConst(
    indp) ), vvs)){
    PLOComplex  * const  res=NewComplexBareExc();
      Complex_SetUSize  ( PLOComplexToComplex( res),vvs);
      ParaLispObject  * list [ ] = { &  indp->plo,
        &  PLOString_NewCopy(L" > ")->plo,&res->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  me;
  }
  if(me ==    vp){
    // me=[0..ind-1,ind,ind+1..s-1] -> [ind,..,end-1]
    if(FractionIs0(Complex_CRe(PLOComplexToComplexConst(
      indp))))
      // me=[0..ind-1,ind,ind+1..s-1] -> [0,..,end-1]
      return  me;
    size_t  ind2;
    if(  FractionIsBadSizeValue(Complex_CRe(
      PLOComplexToComplexConst(indp)),        &ind2)){
      ParaLispObject  * list [ ] = { &  indp->plo,
        &  PLOString_NewCopy(L" is bad")->plo, NULL } ;
      StackClusterPrivate_ExceptionSet(stp ,
        ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  me;
    }    
    if(FractionMoreEqUSize(Complex_CRe(
      PLOComplexToComplexConst(indp) ),vvs)){
      PLOVector_Clear(me);
      return  me;
    }
    size_t  const  newsize=vvs - ind2;
    ParaLispObject  * * jtarget=VectorBegin(StackPLOP)(&me->value );
    ParaLispObject  * * jsource=VectorBegin(StackPLOP)(&me->value ) + ind2;
    do{
      (*  jtarget)  = (*  jsource);
      ++  jtarget;
      ++  jsource;
      ++  ind2;
    }while(ind2 < vvs);
    PLOVector_Resize(me,newsize);
    return  me;
  } // this == v
  size_t  ind2;
  if(  FractionIsBadSizeValue(Complex_CRe(
    PLOComplexToComplexConst( indp)), &ind2)){
    ParaLispObject  * list [ ] = { &  indp->plo,
      &  PLOString_NewCopy(L" is bad")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  me;
  }
  PLOVector_Clear(me);
  size_t  const  newsize=vvs - ind2;  
  PLOVector_ResizeFill(me,newsize,&plonil.plo);
  ParaLispObject  * * jtarget=VectorBegin(StackPLOP)(&me->value );
  ParaLispObject  * * jsource=VectorBegin(StackPLOP)(&vp->value ) + ind2;
  while(ind2 < vvs){
    (*  jtarget)  = (*  jsource);
    ++  jtarget;
    ++  jsource;
    ++  ind2;
  }
  return  me;
}

# define  PLOVector_SwapInd Lisp2D_PLOVector_SwapInd
static  PLOVector *  PLOVector_SwapInd(PLOVector * me ,
  PLOComplex * ip , PLOComplex  * jp ,
  StackClusterPrivate * stp ) ;

// vector swap i j
// vector swap vector
TaskForThread Lisp2Dbimvectorswap2(StackClusterPrivate  * const stclpp){
  PLOVector * const  mep=
    ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const ip= CCAR ( arglistp);
  if(Vectorp(ip)){
    if(IsConst(  stclpp ,ip)) 
      return  TaskForThread_Analyse;
    // vector swap vector
    PLOVector_Swap(mep,ParaLispObjectToPLOVector(ip));
    StackClusterPrivate_SetResult(stclpp ,&mep->plo);
    return  TaskForThread_Analyse; }
  if(not Numberp(ip)){
    ParaLispObject  * list [ ] = { ip,
      & PLOString_NewCopy(L" is not vector nor number")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse; 
  }
  ParaLispObject  * const jp= CCAR ( CCDR ( arglistp));
  if(IsNotNumber(  stclpp ,jp))    
    return  TaskForThread_Analyse;
  StackClusterPrivate_SetResult(stclpp , & PLOVector_SwapInd ( mep  ,
    ParaLispObjectToPLOComplex  ( ip),ParaLispObjectToPLOComplex(jp), stclpp )
    ->plo);
  return  TaskForThread_Analyse; 
}

static  PLOVector *  PLOVector_SwapInd(PLOVector * const me ,
  PLOComplex * const ip , PLOComplex  * const jp ,
  StackClusterPrivate * const stp ) {
  if( ip  ==    jp) 
    return  me;
  if(not  Complex_Integerp(PLOComplexToComplexConst(ip))){
    ParaLispObject  * list [ ] = { &  ip->plo,&
      PLOString_NewCopy(L" is not integer") ->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ; 
  }
  if(not  Complex_Integerp(PLOComplexToComplexConst(jp))){
    ParaLispObject  * list [ ] = { &  jp->plo,& PLOString_NewCopy(
      L" is not integer")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me; 
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
    ip)) ,0U)){
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"index ")->plo,
      & ip->plo,
      & PLOString_NewCopy(L"<0")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ; 
  }
  size_t const   s=VectorSizeCopy  ( StackPLOP ) ( PLOVector_CValue  ( me  ) );
  if  ( ULongMoreOrEqSizet  ( FractionCNump  ( Complex_CRe (
    PLOComplexToComplexConst  ( ip  ) ) ) , s ) ) {
    PLOComplex  * const  res=NewComplexBareExc();
    Complex_SetUSize  ( PLOComplexToComplex( res),s);
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"index ")->plo,
      & ip->plo,
      & PLOString_NewCopy(L">=")->plo,&res->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ; 
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
    jp)) ,0U)){
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"index ")->plo,
      & jp->plo,
      & PLOString_NewCopy(L"<0")->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ; 
  }
  if ( ULongMoreOrEqSizet ( FractionCNump  ( Complex_CRe (
    PLOComplexToComplexConst  ( jp  ) ) ) , s ) ) {
    PLOComplex  * const  res=NewComplexBareExc(  );
    Complex_SetUSize  ( PLOComplexToComplex( res),s);
    ParaLispObject  * list [ ] = { &  PLOString_NewCopy(L"index ")->plo,
      & jp->plo,
      & PLOString_NewCopy(L">=")->plo,&res->plo, NULL } ;
    StackClusterPrivate_ExceptionSet(stp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  me ; 
  }
  { size_t  ui;
    ULongIsBADSizeValue  ( FractionCNump  ( Complex_CRe (
         PLOComplexToComplexConst( ip) ) ) , & ui  ) ; // always good
    size_t  uj;
    ULongIsBADSizeValue  ( FractionCNump  ( Complex_CRe (
         PLOComplexToComplexConst( jp) ) ) , & uj  ) ; // always good
    { Vector ( StackPLOP ) * const  meva  = & me ->  value  ;
      ParaLispObject  * const tmp =
        Vector_EltCopy  ( StackPLOP ) ( meva , ui ) ;
      * Vector_EltAddr ( StackPLOP ) ( meva , ui ) = 
        Vector_EltCopy ( StackPLOP ) ( meva , uj ) ;
      * Vector_EltAddr ( StackPLOP ) ( meva , uj ) = tmp ; 
    } 
  } // ui uj
  return  me ; 
}

// vector insert ind val
TaskForThread Lisp2Dbimvectorinsert2(StackClusterPrivate  * const stclpp){
  PLOVector * const  mep=
    ParaLispObjectToPLOVector(StackClusterPrivate_ThisTop(stclpp ));
  ParaLispObject  * const arglistp=StackClusterPrivate_ArgTop(stclpp );
  ParaLispObject  * const ip= CCAR (  arglistp);
  if(IsNotNumber(  stclpp ,ip))    
    return  TaskForThread_Analyse;
  if(not  FractionIntegerP(Complex_CRe(PLOComplexToComplexConst(
        ParaLispObjectToPLOComplexConst(ip))))){
    ParaLispObject  * list [ ] = {   ip, & PLOString_NewCopy(L" is not integer")->plo, 0 } ;
    StackClusterPrivate_ExceptionSet(stclpp ,ListOfPLO((PArrayPLOP){
      (ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse; 
  }
  if(FractionLessUInt(Complex_CRe(PLOComplexToComplexConst(
        ParaLispObjectToPLOComplex(ip))),0U)){
    ParaLispObject  * list [ ] = {  ip,& PLOString_NewCopy(L"<0")->plo, 0 } ;
    StackClusterPrivate_ExceptionSet(stclpp ,
      ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
    return  TaskForThread_Analyse; 
  }
  { size_t  const  s=VectorSizeCopy  ( StackPLOP ) ( & mep -> value  );
    if(FractionMoreUSize(Complex_CRe(PLOComplexToComplexConst(
          ParaLispObjectToPLOComplexConst(ip))),s)){
      PLOComplex  * const  res=NewComplexBareExc();
      Complex_SetUSize  ( PLOComplexToComplex( res),s);
      ParaLispObject  * list [ ] = { ip,& PLOString_NewCopy(L">")->plo, &res->plo, 0 } ;
      StackClusterPrivate_ExceptionSet(stclpp , ListOfPLO((PArrayPLOP){(ParaLispObject*(*)[])&list}));
      return  TaskForThread_Analyse;
    }
  } // s
  { size_t  ind;
    if( FractionIsBadSizeValue(
      Complex_CRe(PLOComplexToComplexConst(ParaLispObjectToPLOComplexConst( ip))),&ind))
      return  TaskForThread_Analyse;
    PLOVector_Insert(mep,ind,  CCAR ( CCDR ( arglistp))); 
  } // ind
  StackClusterPrivate_SetResult(stclpp ,&mep->plo);
  return  TaskForThread_Analyse; 
}

IsNotTypeTemplBody  ( vector , TypeGlobalListMemberPLOVector )
