// Lisp2D - Queue (small OR great) - few trouble with memory
// ©2012-23 by Alex Glebe
# include <stdio.h>
# include <wchar.h> // fputws
# include <string.h> // memcpy
# include <stdbool.h>
# include <stddef.h> // size_t
# include <stdlib.h> // free
# include <stdint.h> // uint8_t
# include "define.h"
/*
size=0 memorysize= new 0 or 0..2
 +1 ok
size=1 memorysize= new 2 or 1..3
 -1 ok
 +1 ok
size=2 memorysize= new 3 or 2..7
 -1 1/3 ok
 if memorysize=4 then -1 -> 1/4 task to delete
 +1 ok
size=3 memorysize= new 6 or 3..11
 -1 1/3 ok
 +1 2/3 ok
size=s memorysize= new s*2 or s..(s*4-1)
*/

// ! Queue - works without constructors/destructors for elements

# ifdef LISP2D_DEBUG

# define QueueDefineENumName( QUEUEN ) \
  enum  Lisp2D ##  QUEUEN  ##  State
  
# define  QueueNotConstructed(  QUEUEN  )  Lisp2D ##  QUEUEN  ##  NotConstructed
# define  QueueLive(  QUEUEN  )  Lisp2D ##  QUEUEN  ##  Live
# define  QueueDestructed(  QUEUEN  ) Lisp2D  ##  QUEUEN  ##  Destructed

# define QueueDefineNum( QUEUEN ) QueueDefineENumName ( QUEUEN  )  { \
  QueueNotConstructed (  QUEUEN  ) = 0x1111 , \
  QueueLive (  QUEUEN  ) = 0x5555 , \
  QueueDestructed (  QUEUEN  ) = 0x3333 , \
} ;

# else

# define QueueDefineNum( QUEUEN )
  
# define QueueDefineENumName( QUEUEN )

# endif  

/* [firstindex..firstindex+size]  \
   or \
   [0..firstindex-(memorysize-size)] [firstindex..memorysize-1] \
   push to the end  \
   pop from the start */
# define  Queue(  N )  Lisp2DQueue  ##  N
# define  QueuePub(  N )  Lisp2DQueuePub  ##  N
# define  Queue_toCPrivate( N ) Lisp2D_Queue_toCPrivate ##  N
# define  Queue_toPrivate(  N ) Lisp2D_Queue_toPrivate  ##  N
# define  QueueEmpty( N ) Lisp2DQueue ##  N ##  Empty
# define  QueueClear( N ) Lisp2DQueue ##  N ##  Clear
# define  QueueSize(  N ) Lisp2DQueue ##  N ##  Size
# define  QueueMemorySize(  N ) Lisp2DQueue ##  N ##  MemorySize
# define  QueueFirstIndex(  N ) Lisp2DQueue ##  N ##  FirstIndex
# define  QueueAtValue( N ) Lisp2DQueue ##  N ##  AtValue
# define  QueueAtAddr(  N ) Lisp2DQueue ##  N ##  AtAddr
# define  QueueFrontValue(  N ) Lisp2DQueue ##  N ##  FrontValue
# define  QueueFrontAddr( N ) Lisp2DQueue ##  N ##  FrontAddr
# define  QueueFrontProtected(  N ) Lisp2DQueue ##  N ##  FrontProtected

# ifdef LISP2D_DEBUG
  # define QUEUECLASS( QUEUEN  , QUEUET  ) \
    LISP2DQUEUECLASS  ( QUEUEN  , QUEUET , QueueDefineENumName ( QUEUEN ) state ; )
# else
  # define  QUEUECLASS( QUEUEN  , QUEUET  ) \
    LISP2DQUEUECLASS ( QUEUEN  , QUEUET , )
# endif  

// ! to enum type struct public .. .h

# define LISP2DQUEUECLASS( QUEUEN  , QUEUET , DeclareState ) \
QueueDefineNum( QUEUEN ) \
typedef struct  sLisp2D ##  QUEUEN  Queue ( QUEUEN  ) ; \
typedef struct  sLisp2Dpub ##  QUEUEN QueuePub ( QUEUEN  ) ; \
struct  sLisp2Dpub ##  QUEUEN { \
  char  _ ; \
} ; \
struct  sLisp2D ##  QUEUEN { \
  QUEUET * data ;  \
  size_t  size  ;  \
  size_t  memorysize  ;  \
  size_t  firstindex  ; \
  DeclareState \
  QueuePub  ( QUEUEN  ) pub ; \
} ; \
inline  Queue ( QUEUEN  ) const * Queue_toCPrivate ( QUEUEN  )  ( QueuePub ( QUEUEN  ) const * ) ; \
inline  Queue ( QUEUEN  ) * Queue_toPrivate ( QUEUEN  )  ( QueuePub ( QUEUEN  ) * ) ; \
inline  bool  QueueEmpty  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * ) ; \
inline  void QueueClear ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * ) ; \
inline  size_t  QueueSize ( QUEUEN  )  ( QueuePub  ( QUEUEN  ) const * ) ; \
inline  size_t  QueueMemorySize ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * ) ; \
inline  size_t  QueueFirstIndex ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * ) ; \
inline  QUEUET QueueAtValue ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * , size_t index ) ; \
inline  QUEUET * QueueAtAddr  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * , size_t index ) ; \
inline  QUEUET QueueFrontValue  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * ) ; \
inline  QUEUET * QueueFrontAddr ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * ) ; \
inline  bool  QueueFrontProtected ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * , QUEUET  * result  ) ; \
inline  Queue ( QUEUEN  ) const * Queue_toCPrivate ( QUEUEN  )  ( QueuePub ( QUEUEN  ) const * const p ) { \
  return  ( Queue ( QUEUEN  ) const * ) ( ( ( uint8_t * ) p ) - offsetof ( Queue ( QUEUEN  ) , pub ) ) ; \
} \
inline  Queue ( QUEUEN  ) * Queue_toPrivate ( QUEUEN  )  ( QueuePub ( QUEUEN  ) * const p ) { \
  return  ( Queue ( QUEUEN  ) * ) ( ( ( uint8_t * ) p ) - offsetof ( Queue ( QUEUEN  ) , pub ) ) ; \
} \
inline  bool  QueueEmpty  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s ) {  \
  return Queue_toCPrivate ( QUEUEN  ) ( s ) ->  size ==  0 ; \
}  \
inline  void QueueClear ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * const qp ) { \
  Queue ( QUEUEN  ) * const q = Queue_toPrivate ( QUEUEN  ) ( qp ) ; \
  q ->  firstindex = 0 ; \
  q ->  size  = 0 ; \
  if  ( q ->  memorysize  > 2 ) { \
    free ( q  ->  data  ) ; \
    q ->  memorysize = 0  ; \
    q ->  data = NULL ; \
  } \
} \
inline  size_t  QueueSize ( QUEUEN  )  ( QueuePub  ( QUEUEN  ) const * const s ) {  \
  return Queue_toCPrivate ( QUEUEN  ) ( s )  ->  size ; \
} \
inline  size_t  QueueMemorySize ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s ) {  \
  return Queue_toCPrivate ( QUEUEN  ) ( s ) -> memorysize ; \
}  \
inline  size_t  QueueFirstIndex ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s ) {  \
  return Queue_toCPrivate ( QUEUEN  ) ( s ) -> firstindex ; \
}  \
inline  QUEUET QueueAtValue ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s , size_t const  index ) { \
  return  *  ( ( Queue_toCPrivate ( QUEUEN  ) ( s ) -> data ) + index ) ; \
}  \
inline  QUEUET * QueueAtAddr  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s , size_t const  index ) { \
  return  ( Queue_toCPrivate ( QUEUEN  ) ( s ) -> data ) + index ; \
}  \
inline  QUEUET QueueFrontValue  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s ) { \
  Queue ( QUEUEN  ) const * const p = Queue_toCPrivate ( QUEUEN  ) ( s ) ; \
  return  ( p -> data ) [ p ->  firstindex  ] ; \
}  \
inline  QUEUET * QueueFrontAddr ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s ) { \
  Queue ( QUEUEN  ) const * const p = Queue_toCPrivate ( QUEUEN  ) ( s ) ; \
  return  p -> data + ( p ->  firstindex  ) ; \
}  \
inline  bool  QueueFrontProtected ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const s , QUEUET  * const result  ) { \
  Queue ( QUEUEN  ) const * const p = Queue_toCPrivate ( QUEUEN  ) ( s ) ; \
  if  ( p ->  size  ) { \
    ( * result  ) = ( p ->  data  )  [ p ->  firstindex  ] ; \
    return false  ; \
  }  \
  return true ; \
}

# define  QueueConstructorDefault( N  ) Lisp2DQueue ##  N ##  ConstructorDefault
  
# define QUEUEPROTOCONSTRUCTORDEFAULT( QUEUEN )  \
void QueueConstructorDefault  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * ) ;

# ifdef LISP2D_DEBUG

# define QUEUECONSTRUCTORDEFAULT( QUEUEN )  \
void QueueConstructorDefault  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * const me ) { \
  Queue ( QUEUEN  ) * const p = Queue_toPrivate ( QUEUEN  ) ( me ) ; \
  p -> size = 0UL ; \
  p -> data = NULL ; \
  p -> memorysize = 0UL ; \
  p -> firstindex = 0 ; \
  p -> state = QueueLive ( QUEUEN  ) ; \
}

# else

# define QUEUECONSTRUCTORDEFAULT( QUEUEN )  \
void QueueConstructorDefault  ( QUEUEN  ) ( QueuePub  ( QUEUEN  ) * const me ) { \
  Queue ( QUEUEN  ) * const p = Queue_toPrivate ( QUEUEN  ) ( me ) ; \
  p -> memorysize = 0UL ; \
  p -> data = NULL ; \
  p -> firstindex = 0 ; \
  p -> size = 0UL ; \
}

# endif  
  
# define  QueueDestructor(  N ) Lisp2DQueue ##  N ##  Destructor
  
# define  PROTOQUEUEDESTRUCTOR( QUEUEN  ) \
void QueueDestructor  ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * )  ;
  
# ifdef LISP2D_DEBUG

# define  QUEUEDESTRUCTOR( QUEUEN  ) \
void QueueDestructor  ( QUEUEN  ) ( QueuePub ( QUEUEN  )  * const s ) {  \
  Queue ( QUEUEN  ) * const p = Queue_toPrivate ( QUEUEN  ) ( s ) ; \
  switch  ( p -> state  ) {  \
  case QueueLive  ( QUEUEN  ) : \
    break ; \
  case QueueNotConstructed  ( QUEUEN  ) : \
    fputws  ( L"QueueDestructor(QUEUEN):NotConstructed\n" , stderr  ) ; \
    return ;  \
  case QueueDestructed  ( QUEUEN  ) : \
    fputws  ( L"QueueDestructor(QUEUEN):Destructed\n" , stderr  ) ; \
    return ; \
  default : \
    fputws  ( L"QueueDestructor(QUEUEN)::dead memory\n" , stderr  ) ; \
    return ; \
  } \
  p -> state = QueueDestructed  ( QUEUEN  ) ; \
  free  ( p ->  data  ) ; \
}

# else

# define  QUEUEDESTRUCTOR( QUEUEN  ) \
void QueueDestructor  ( QUEUEN  ) ( QueuePub ( QUEUEN  )  * const s ) {  \
  free  ( Queue_toPrivate ( QUEUEN  ) ( s ) ->  data  ) ; \
}
  
# endif  
  
# define  QueueIsBadElt(  N ) Lisp2DQueue ##  N ##  IsBadElt
  
# define PROTOQUEUEISBADELT( QUEUEN  , QUEUET  ) \
bool  QueueIsBadElt ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * q , \
  size_t index , QUEUET * result ) ;  
  
# define QUEUEISBADELT( QUEUEN  , QUEUET  ) \
bool  QueueIsBadElt ( QUEUEN  ) ( QueuePub ( QUEUEN  ) const * const q , \
  size_t const index , QUEUET * const result ) { \
  Queue ( QUEUEN  ) const * const p = Queue_toCPrivate ( QUEUEN  ) ( q ) ; \
  if  ( index  < p -> size ) {  \
    size_t  const spacetotheend = \
      ( p ->  memorysize  - ( p ->  firstindex  ) ) ;  \
    if  ( spacetotheend <= index  ) \
      ( * result  ) = ( ( p ->  data  ) [ index - spacetotheend ] ) ; \
    else \
      ( * result  ) = ( ( p ->  data  ) [ ( p ->  firstindex  ) + index  ] ) ; \
    return  false ; \
  } \
  return  true  ; \
}

# define  QueueIsBadPushByValue(  N ) Lisp2DQueue ##  N ##  IsBadPushByValue
    
# define PROTOQUEUEISBADPUSHBYVALUE( QUEUEN  , QUEUET  ) \
bool  QueueIsBadPushByValue ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * q , QUEUET x ) ;

# include <iso646.h> // not

# define QUEUEISBADPUSHBYVALUE( QUEUEN  , QUEUET  ) \
bool  QueueIsBadPushByValue ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * const q , QUEUET const  x ) { \
  Queue ( QUEUEN  ) * const p = Queue_toPrivate ( QUEUEN  ) ( q ) ; \
  if  ( p ->  memorysize  ) { \
    if  ( p ->  size == ( p ->  memorysize  ) ) { \
      /* is full       prepare memory */ \
      size_t  news  ; \
      /* 1/1 -> 2/3 \
       2/2 -> 3/6 \
       3/3 -> 4/8 */  \
      if  ( p ->  size ==  1  ) \
        news  = 3 ;  \
      else  \
        news  = ( p ->  size + 1  ) <<  1 ; \
      if  ( news < ( p ->  memorysize) ) { \
        /* may be impossible */ \
        return  true  ; \
      }  \
      { QUEUET * const  datanew  = aligned_alloc ( _Alignof ( QUEUET [ news ] ) , sizeof  ( QUEUET [ news ] ) ) ;  \
        if  ( not datanew ) \
          return  true  ; \
        if  ( p ->  firstindex == 0 ) \
          memcpy  ( datanew , p ->  data  , sizeof  ( QUEUET  ) * ( p ->  memorysize  ) ) ; \
        else  { \
          size_t  const spacetotheend = \
            ( p ->  memorysize  - ( p ->  firstindex  ) ) ;  \
          /* source = end part , target = beginning \
 ? to use std::copy              */ \
          memcpy  ( datanew , p ->  data + ( p ->  firstindex  ) , sizeof  ( QUEUET  ) * spacetotheend ) ;  \
          /* source = beginning , target = continue \
 ? to use std::copy              */ \
          memcpy  ( datanew  + spacetotheend  , p ->  data  , sizeof  ( QUEUET  ) * ( p ->  firstindex  ) ) ; \
          /* firstindex==size-spacetoend */  \
          p ->  firstindex  = 0 ; \
        } \
        free ( p  ->  data  ) ; \
        p ->  data = datanew  ; \
      }  \
      p ->  memorysize  = news  ; \
    }  \
    size_t  const spacetotheend = p ->  memorysize  - ( p ->  firstindex  ) ;  \
    /* push to the end */ \
    if  ( p ->  size < spacetotheend  ) \
      ( p ->  data  ) [ p ->  firstindex + ( p ->  size  ) ] = x ;  \
      /* push to start part */  \
    else \
      ( p ->  data  ) [ p ->  firstindex - ( p ->  memorysize  - ( p ->  size  ) ) ] = x ; \
    ++ ( p ->  size  ) ;  \
    return  false ; \
  } \
  /* make first element \
   ! dont make memory=1 : if next will be Push - double work */ \
  p ->  data  = aligned_alloc ( _Alignof ( QUEUET [ 2 ] ) , sizeof  ( QUEUET [ 2 ] ) ) ;  \
  if ( not ( p  ->  data ) ) \
    return  true  ; \
  p ->  memorysize = 2 ; \
  p ->  firstindex  = 0 ;  \
  ( * ( p ->  data  ) ) = x ; \
  p ->  size  = 1 ; \
  return  false ; \
}

# define  QueueIsBadPop(  N ) Lisp2DQueue ##  N ##  IsBadPop
    
# define PROTOQUEUEISBADPOP( QUEUEN  , QUEUET  ) \
bool  QueueIsBadPop ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * s , QUEUET * result ) ;
    
# define QUEUEISBADPOP( QUEUEN  , QUEUET  ) \
bool  QueueIsBadPop ( QUEUEN  ) ( QueuePub ( QUEUEN  ) * const s , QUEUET * const result ) { \
  Queue ( QUEUEN  ) * const p = Queue_toPrivate ( QUEUEN  ) ( s ) ; \
  if  ( p ->  size > 1 ) { \
    ( * result  ) = p ->  data  [ p ->  firstindex  ] ;  \
    /* ! if firstindex is in the end then must to be ZERO */  \
    if  ( p ->  memorysize == p ->  firstindex + 1 ) \
      p ->  firstindex  = 0 ; \
    else  \
      ++  ( p ->  firstindex  ) ; \
    --  ( p ->  size  ) ; \
    /* ! if first take result then : ask for <  \
     ! if take result later : ask for <= */ \
    if  ( ( p ->  size  <<  2 ) <= p  ->  memorysize ) { \
      /* /4 = protection from next Push      */  \
      size_t  newmems ;  \
      /* 1/4 -> 1/2 \
       2/8 -> 2/3 \
       3/12 -> 3/6 */ \
      if  ( p ->  size ==  2 ) \
        newmems = 3 ; \
      else \
        newmems = p ->  size <<  1 ; \
      /* make small place for an array */ \
      { QUEUET * const datanew = aligned_alloc ( _Alignof ( QUEUET [ newmems ] ) , sizeof  ( QUEUET [ newmems ] ) ) ;  \
        /* to use old big space and wait for another chance          */ \
        if  ( not datanew ) \
          return  false ;  \
        /* if last element will be inside new array then simple copy */ \
/* ? to use std::copy              */ \
        if  ( p ->  firstindex + p ->  size <= newmems ) \
          memcpy  ( datanew , p ->  data  , sizeof  ( QUEUET  ) * newmems ) ; \
        else  { \
          size_t  const spacetotheend = p ->  memorysize  - ( p ->  firstindex ) ;  \
          /* copy short place \
 ? to use std::copy              */ \
          if  ( p ->  size <= spacetotheend ) \
            memcpy  ( datanew , p ->  data + ( p ->  firstindex  ) , sizeof  ( QUEUET  ) * ( p ->  size  ) ) ; \
          else  { \
            /* source = end part , target = beginning \
 ? to use std::copy                */ \
            memcpy  ( datanew , p ->  data + ( p ->  firstindex  ) , sizeof  ( QUEUET  ) * spacetotheend ) ; \
            /* source = beginning , target = continue \
 ? to use std::copy                */ \
            memcpy  ( datanew  + spacetotheend  , p ->  data  , sizeof  ( QUEUET  ) * ( p ->  size  - spacetotheend ) ) ; \
          } \
          p ->  firstindex  = 0 ; \
        } \
        free ( p  ->  data  ) ; \
        p ->  data = datanew  ; \
      }  \
      p ->  memorysize  = newmems ; \
    } \
    return  false ; \
  } \
  if  ( p ->  size ==  1 ) { \
    /* it was last */ \
    ( * result  ) = ( p ->  data  [ p ->  firstindex  ] ) ; \
    p ->  size  = 0 ; \
    /* firstindex:=0 : for comfort memcpy after several Push,Push,.. */ \
    p ->  firstindex  = 0 ; \
    /* ! don't remove memory , next will be Push */ \
    if  ( p ->  memorysize > 2 ) { \
      free ( p  ->  data  ) ;  \
      p ->  data  = NULL ; \
      p ->  memorysize  = 0 ; \
      return  false ; \
    } \
    return  false ; \
  } \
  return  true  ; \
}
