// Lisp2D interpreter - Statistics
// ©2016-23 by Alex Glebe

# ifndef LISP2D_STATISTICS_H
# define LISP2D_STATISTICS_H
# include "define.h"

# ifdef  LISP2D_BIMSTAT

# include <stdbool.h> // bool
# include <stddef.h> // size_t
# include <pthread.h> // pthread_mutex_t
# include <setjmp.h>  // jmp_buf
# include <stdlib.h> // free
# ifdef LISP2D_DEBUG
# include <wchar.h> // fputws
# include <stdio.h> // stderr
# endif
# include "statisticsp.h" // private

# define  StatisticsSectionConstructor  Lisp2DStatisticsSectionConstructor
void  StatisticsSectionConstructor  ( struct  Lisp2DStatisticsSection * ) ;

// to free stack place pass ldouble by pointer
// LDouble always transfers by POINTER

# define  StatisticsSectionPush2  Lisp2D_StatisticsSectionPush2
void  StatisticsSectionPush2  ( struct  Lisp2DStatisticsSection * , long  double  x , long  double squarex ) ;

# define  StatisticsSectionPointerSumX  Lisp2DStatisticsSectionPointerSumX
inline  long  double  StatisticsSectionPointerSumX  ( struct  Lisp2DStatisticsSection const * ) ;
# define  StatisticsSectionPointerSumX2 Lisp2DStatisticsSectionPointerSumX2
inline  long  double  StatisticsSectionPointerSumX2 ( struct  Lisp2DStatisticsSection const * ) ;
# define  StatisticsSectionCount  Lisp2DStatisticsSectionCount
inline  size_t  StatisticsSectionCount  ( struct  Lisp2DStatisticsSection const * ) ;
# define  StatisticsSectionCalcAverage  Lisp2DStatisticsSectionCalcAverage
inline  long  double  StatisticsSectionCalcAverage  ( struct  Lisp2DStatisticsSection const * ) ;
# define  StatisticsSectionGlobalPointerS Lisp2DStatisticsSectionGlobalPointerS
inline  struct  Lisp2DStatisticsSection const * StatisticsSectionGlobalPointerS  (
  struct  Lisp2DStatisticsSectionGlobal const * ) ;
# define  StatisticsSectionGlobalPointerAverage Lisp2DStatisticsSectionGlobalPointerAverage
inline  long  double  StatisticsSectionGlobalPointerAverage  ( struct  Lisp2DStatisticsSectionGlobal const * ) ;
# define  StatisticsSectionGlobalPointerDeviation Lisp2DStatisticsSectionGlobalPointerDeviation
inline long  double  StatisticsSectionGlobalPointerDeviation (
  struct  Lisp2DStatisticsSectionGlobal const * ) ;
# define  StatisticsSectionGlobalCount  Lisp2DStatisticsSectionGlobalCount
inline  size_t  StatisticsSectionGlobalCount  ( struct  Lisp2DStatisticsSectionGlobal const * ) ;
# define  StatisticsSectionGlobalPointerSumX  Lisp2DStatisticsSectionGlobalPointerSumX
inline  long  double  StatisticsSectionGlobalPointerSumX ( struct  Lisp2DStatisticsSectionGlobal const * ) ;
# define  StatisticsSectionGlobalCLess  Lisp2DStatisticsSectionGlobalCLess
inline  bool  StatisticsSectionGlobalCLess  ( struct  Lisp2DStatisticsSectionGlobal const * ,
  struct  Lisp2DStatisticsSectionGlobal const * ) ;
# define  StatisticsDestructor  Lisp2DStatisticsDestructor
void StatisticsDestructor ( struct  Lisp2DStatistics * ) ;
# define  StatisticsPointerGlobalStat Lisp2DStatisticsPointerGlobalStat
inline struct  Lisp2DStatisticsSectionGlobal const * StatisticsPointerGlobalStat  ( struct  Lisp2DStatistics * ) ;
# define  StatisticsSize  Lisp2DStatisticsSize
inline  size_t  StatisticsSize  ( struct  Lisp2DStatistics const * ) ;
# define  StatisticsIsBADElt  Lisp2DStatisticsIsBADElt
bool  StatisticsIsBADElt  ( struct  Lisp2DStatistics  const * , size_t , struct Lisp2DStatisticsSection * * ) ;
# define  StatisticsCount Lisp2DStatisticsCount
inline  size_t  StatisticsCount ( struct  Lisp2DStatistics const * ) ;
# define  StatisticsPointerAverage  Lisp2DStatisticsPointerAverage
inline  long  double  StatisticsPointerAverage ( struct Lisp2DStatistics  const * ) ;
# define  StatisticsPointerDeviation  Lisp2DStatisticsPointerDeviation
inline  long  double  StatisticsPointerDeviation ( struct Lisp2DStatistics  const * ) ;
# define  StatisticsCLess Lisp2DStatisticsCLess
inline  bool  StatisticsCLess ( struct Lisp2DStatistics  const * , struct Lisp2DStatistics const * ) ;
# define  StatisticsPointerSumX Lisp2DStatisticsPointerSumX
inline  long  double  StatisticsPointerSumX  ( struct Lisp2DStatistics  const * ) ;
# define  StatisticsCMutex  Lisp2DStatisticsCMutex
inline Lisp2D_atomic_mutex const * StatisticsCMutex ( struct Lisp2DStatistics  const * ) ;

// Dispersy = (1/(n-1))*(Sum(x^2)-Aver(x)*Summ(x))  
# define  StatisticsSectionCalcDeviation  Lisp2DStatisticsSectionCalcDeviation
long  double  StatisticsSectionCalcDeviation ( struct  Lisp2DStatisticsSection const * , long  double  ave ) ;
  
# define  StatisticsSectionGlobalConstructor  Lisp2DStatisticsSectionGlobalConstructor
void  StatisticsSectionGlobalConstructor  ( struct  Lisp2DStatisticsSectionGlobal * ) ;

# define  StatisticsConstructor Lisp2DStatisticsConstructor
void  StatisticsConstructor  ( struct  Lisp2DStatistics * ) ;
  
# define  StatisticsPush  Lisp2D_StatisticsPush
void  StatisticsPush  ( struct  Lisp2DStatistics * , long  double ) ;

// ! to run Exception      
# define  StatisticsIsBADInit Lisp2DStatisticsIsBADInit
bool  StatisticsIsBADInit ( struct  Lisp2DStatistics * , size_t s ) ;

  extern  struct Lisp2DStatistics  GCstat  ;
  extern  struct Lisp2DStatistics  GCstatSymbols ;
  extern  struct Lisp2DStatistics  statCreateLinks2  ;
extern  struct  Lisp2DStatistics  statULongSquare ;
  extern  struct Lisp2DStatistics  statComplexOperatorPlusEq ;
  extern  struct Lisp2DStatistics  statComplexOperatorMulEq ;
  extern  struct Lisp2DStatistics  statComplexOperatorDivEq ;
  extern  struct Lisp2DStatistics  statComplexDelete ;
  extern  struct Lisp2DStatistics  statULongDivWithOstBig  ;
  extern  struct Lisp2DStatistics  statULongDivWithOstBigFast  ;
  extern  struct Lisp2DStatistics  statULongDivWithOstBig2 ;
  
inline  long  double  StatisticsSectionPointerSumX  ( struct  Lisp2DStatisticsSection const * const s ) {
  return  s ->  sumx ;
}

inline  long  double  StatisticsSectionPointerSumX2 ( struct  Lisp2DStatisticsSection const * const s ) {
  return  s ->  sumx2 ;
}

inline  size_t  StatisticsSectionCount  ( struct  Lisp2DStatisticsSection const * const me ) {
  return  me -> count ;
}

inline  long  double  StatisticsSectionCalcAverage  ( struct  Lisp2DStatisticsSection const * const s ) {
  return ( s -> sumx ) / ( s ->  count ) ;
}

inline  struct  Lisp2DStatisticsSection const * StatisticsSectionGlobalPointerS  (
  struct  Lisp2DStatisticsSectionGlobal const * const s ) {
  return  & ( s ->  s ) ;
}

inline  long  double StatisticsSectionGlobalPointerAverage  ( struct  Lisp2DStatisticsSectionGlobal const * const s ) {
  return  s -> average ;
}

inline long  double  StatisticsSectionGlobalPointerDeviation (
  struct  Lisp2DStatisticsSectionGlobal const * const s ) {
  return  s -> deviation ;
}

inline  size_t  StatisticsSectionGlobalCount  ( struct  Lisp2DStatisticsSectionGlobal const * const me ) {
  return  me -> s . count ;
}

inline  long  double  StatisticsSectionGlobalPointerSumX ( struct  Lisp2DStatisticsSectionGlobal const * const s ) {
  return s -> s . sumx ;
}

inline  bool  StatisticsSectionGlobalCLess  ( struct  Lisp2DStatisticsSectionGlobal const * const s ,
  struct  Lisp2DStatisticsSectionGlobal const * const x ) {
  return  ( ( s  -> s . sumx )  ) < ( x  -> s . sumx ) ;
}

inline struct  Lisp2DStatisticsSectionGlobal const * StatisticsPointerGlobalStat  ( struct  Lisp2DStatistics * const s ) {
  return  & ( s -> globalStat ) ;
}

inline  size_t  StatisticsSize  ( struct  Lisp2DStatistics const * const s ) {
  return  ( s ->  sizeset ) ;
}

inline  size_t  StatisticsCount ( struct  Lisp2DStatistics const * const s ) {
  return  Lisp2DStatisticsSectionGlobalCount  ( & ( s -> globalStat ) ) ;
}

inline  long  double  StatisticsPointerAverage ( struct Lisp2DStatistics  const * const s ) {
  return  StatisticsSectionGlobalPointerAverage ( & ( s -> globalStat ) ) ;
}

inline  long  double  StatisticsPointerDeviation ( struct Lisp2DStatistics  const * const s ) {
  return  StatisticsSectionGlobalPointerDeviation ( & ( s -> globalStat ) ) ;
}

inline  bool  StatisticsCLess ( struct Lisp2DStatistics  const * st  , struct Lisp2DStatistics const * const x ) {
  return  Lisp2DStatisticsSectionGlobalCLess ( & ( st ->  globalStat ) , & ( x -> globalStat ) ) ;
}

inline  long  double  StatisticsPointerSumX  ( struct Lisp2DStatistics  const * const st  ) {
  return  StatisticsSectionGlobalPointerSumX ( & ( st -> globalStat ) ) ;
}

inline Lisp2D_atomic_mutex const * StatisticsCMutex ( struct Lisp2DStatistics  const * const s ) {
  return & s -> amutex ;
}

# endif // LISP2D_BIMSTAT
# endif // LISP2D_STATISTICS_H
