// Lisp2D - Stream
// ©2011-23 by Alex Glebe

# ifndef  LISP2D_STREAMC_H
# define  LISP2D_STREAMC_H
# include <fcntl.h> // mode_t
# include "define.h"
# include "widestring/inline.h"

# ifdef  LISP2D_CONSTRSTAT
# define  StreamConstrstat  Lisp2DStreamConstrstat
extern  _Atomic(size_t) StreamConstrstat  ;
# endif

# define  StreamTypeName(x)  Lisp2D_StreamTypeName_ ##  x
# define  eStreamType  eLisp2DStreamType
enum  eStreamType  {
  StreamTypeName(No) ,
  StreamTypeName(FD) 
} ;    

typedef enum  eStreamType  Lisp2DStreamType  ;
typedef Lisp2DStreamType  StreamType  ;

typedef struct sLisp2DPrivateStream Lisp2DPrivateStream ;
typedef Lisp2DPrivateStream PrivateStream ;

struct sLisp2DPrivateStream {
  bool  innerstream ;
  StreamType  stretype  ;
  int intFD ;
  int intflagsFD  ;
  mode_t  modeFD  ;
  bool  flagFDEOF ;
} ;

# ifdef  LISP2D_CONSTRSTAT
# define  StreamAddConstrstat Lisp2DStreamAddConstrstat
# define  StreamConstrstatValue Lisp2DStreamConstrstatValue
# define  LISP2DSTREAMADDCONSTRSTAT ++  StreamConstrstat  ;
# define  LISP2DSTREAMCONSTRSTATVALUE StreamConstrstat
void  StreamAddConstrstat ( void  ) ;
size_t  StreamConstrstatValue ( void  ) ;
# endif

# define  PrivateStreamInit Lisp2DPrivateStreamInit
void  PrivateStreamInit ( PrivateStream * ) ;

# define  PrivateStreamInitFD Lisp2DPrivateStreamInitFD
void  PrivateStreamInitFD ( PrivateStream * , bool inner , int fd  , int flags ) ;

# define  PrivateStreamWritableFD Lisp2DPrivateStreamWritableFD
inline  bool  PrivateStreamWritableFD ( PrivateStream const * const ps ) {
  int const f  = ( ( ps  ->  intflagsFD )  bitand O_ACCMODE  ) ;
  return (  f == O_WRONLY )  or ( f == O_RDWR ) ;
}

typedef struct Lisp2DsStream  Lisp2DStream  ;
typedef Lisp2DStream  Stream ;

struct Lisp2DsStream {
  PrivateStream pro ;
} ;

# define  StreamSetEofFD  Lisp2DStreamSetEofFD
inline  void  StreamSetEofFD  ( Stream * const s ) {
  s -> pro . flagFDEOF = true  ;
}

# define  StreamUnSetEofFD  Lisp2DStreamUnSetEofFD
inline  void  StreamUnSetEofFD  ( Stream * const s ) {
  s -> pro .flagFDEOF = false  ;
}

# define  StreamSetFDMode Lisp2DStreamSetFDMode
inline  void  StreamSetFDMode ( Stream * const s , mode_t  const x ) {
  s -> pro  .modeFD = x ;
}

# define  StreamSetFDFlags  Lisp2DStreamSetFDFlags
inline  void  StreamSetFDFlags ( Stream * const s , int const x ) {
  s -> pro .intflagsFD = x ;
}

# define  StreamPointerToConstFD  Lisp2DStreamPointerToConstFD
inline  int const * StreamPointerToConstFD  ( Stream  const * const me  ) {
  return  & ( me -> pro  .intFD ) ;
}

# define  StreamFDValue Lisp2DStreamFDValue
inline  int StreamFDValue ( Stream const * const s ) {
  return s -> pro .intFD ;
}

# define  StreamStreType  Lisp2DStreamStreType
inline  StreamType  StreamStreType  ( Stream const * const s ) {
  return s -> pro .stretype  ;
}

# define  StreamInitFD  Lisp2DStreamInitFD
inline  void  StreamInitFD  ( Stream * const s , bool const inner ,
  int const fd  , int const flags ) {
  PrivateStreamInitFD ( & s -> pro , inner , fd  , flags ) ; 
}

int  StreamisBADClear  (  Stream * ) ;  

# define  StreamSetFDFromInt  Lisp2DStreamSetFDFromInt
inline    void  StreamSetFDFromInt ( Stream * const st  , int const x ) {
  StreamisBADClear ( st ) ;
  st -> pro . intFD = x ;
  st ->pro . stretype = StreamTypeName ( FD  ) ;
}

# define  StreamInit  Lisp2DStreamInit
inline  void  StreamInit(Stream * const s)  {
  PrivateStreamInit ( & s -> pro ) ; 
}

# define  StreamWritableFD  Lisp2DStreamWritableFD
bool  StreamWritableFD ( Stream const * ) ;

# define  StreamClose Lisp2DStreamClose
void StreamClose  ( Stream * , int * res ) ;

# define  StreamDestr Lisp2DStreamDestr
inline  void  StreamDestr ( Stream * const s ) {
  StreamisBADClear ( s ) ;
}  
  
# define  StreamReadable  Lisp2DStreamReadable
bool  StreamReadable  ( Stream const * ) ;

# define  StreamisBADReadWChar  Lisp2DStreamisBADReadWChar
bool  StreamisBADReadWChar  ( Stream * , wchar_t * resultp  ) ;
    
# define  StreamWritable  Lisp2DStreamWritable
bool  StreamWriteable ( Stream const * ) ;    

# define  StreamGood  Lisp2DStreamGood
bool  StreamGood  ( Stream const * ) ;

# define  StreamEof Lisp2DStreamEof
bool  StreamEof ( Stream const * );  
  
# define  StreamPushWCharArray  Lisp2DStreamPushWCharArray
Stream const *  StreamPushWCharArray  ( Stream  const *  me , PArray_wchar_c wstr ) ;

# define  StreamPushWCharArrayv  Lisp2DStreamPushWCharArrayv
Stream const *  StreamPushWCharArrayv  ( Stream  const *  me , PArray_wchar_cv wstr ) ;

struct  Lisp2D_s_StackClusterPrivate ;

# define  StreamisBADOpenWOutLockGC Lisp2DStreamisBADOpenWOutLockGC
bool  StreamisBADOpenWOutLockGC ( Stream * , String ( WideChar  ) const * , struct  Lisp2D_s_StackClusterPrivate * ) ;

# include "symbolc.h"

# define    isBADPushCStringToStreamFD  Lisp2DisBADPushCStringToStreamFD
PLOSymbol * isBADPushCStringToStreamFD  ( int  const *  pfd , String  ( Char  ) const *  pcstr  ) ;

# endif // LISP2D_STREAMC_H
